(*
%Copyright   (c) Polak IT
%Author      23.05.2005 14:52:10 POLAK\GAV
%Purpose      
%Type        report
%Specific    AP
%Modified    14.04.2014 12:27:19 POLAK\GAV
%Environment
*)

 MODULE BCPrint [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Stock, L_MSO, L_TRS, L_Str, L_General, P_Math, Memo, L_XDoc, L_BarCode, L_Print, L_Pmnt,
        P_Firm, Str, L_DatTim, L_StdFrm, L_CUC, L_AR, L_MathR;

 PRIVATE TYPE
  tRegime = SCAL[4];
  tMode   = SCAL[4];

 PRIVATE CONST
  r_CurCUC    = 0;
  r_LblCUC    = 1;
  r_CurTckBag = 2;
  r_LblTckBag = 3;

  m_MainBC         = 0;
  m_BadXDocBC      = 1;
  m_CancelVerifing = 2;
  m_GasStan        = 3;

 PRIVATE PROCEDURE Print(CmdPJ: PRINTJOB; X, Y: NUMERIC[0]; BarCode: STRING[]; ShiftX, ShiftY: NUMERIC[0]; Descr: STRING[]);
  VAR
   i          : NUMERIC[0];
   SingleStr  : STRING[];
   Pt14,
   Pt24BarCode: TEXTSTYLE;
 BEGIN
  Pt14       :=L_Print.ts_Create(14, "Times New Roman");
  Pt24BarCode:=L_Print.ts_Create(24, "IntP36DlTt");
  Pt24BarCode.Font.Charset:=FONT.Charset_ANSI;
  L_Print.extTypeS(CmdPJ, X, Y, L_BarCode.toFont_IntP36DlTt(BarCode), Pt24BarCode, L_Print.cp_ISO_8859_1_Latin_I);
  INC(X, ShiftX);
  INC(Y, ShiftY);
  i:=0;
  LOOP
   SingleStr:=L_Str.PoolItem(Descr, i);
   IF !SingleStr THEN
    EXIT;
   END;
   L_Print.TypeS(CmdPJ, X, Y, SingleStr, Pt14);
   INC(Y, 7);
   INC(i);
  END; (* LOOP *)
 END {Print};

 PRIVATE PROCEDURE pj_Beg(Name: STRING[]): PRINTJOB;
  VAR
   CmdPJ: PRINTJOB;
 BEGIN
  CmdPJ:=L_Print.CreatePrintJob(L_Print.pt_Default, Name, L_Print.pf_A4, L_Print.o_Portrait);
  CmdPJ.StartPage();
  RETURN CmdPJ;
 END {pj_Beg};

 PRIVATE PROCEDURE pj_Print(CmdPJ: PRINTJOB; BarCode: STRING[]; ShiftX, ShiftY: NUMERIC[0]; Descr: STRING[]);
 BEGIN
  Print(CmdPJ, 30, 40, BarCode, ShiftX, ShiftY, Descr);
 END {pj__Print};

 PRIVATE PROCEDURE pj_End(CmdPJ: PRINTJOB);
 BEGIN
  CmdPJ.EndPage();
  CmdPJ.Close();
 END {pj_End};

 PRIVATE PROCEDURE BarCode_Print(Name: STRING[]; BarCode: STRING[]; ShiftX, ShiftY: NUMERIC[0]; Descr: STRING[]);
  VAR
   CmdPJ: PRINTJOB;
 BEGIN
  CmdPJ:=pj_Beg(Name);
  pj_Print(CmdPJ, BarCode, ShiftX, ShiftY, Descr);
  pj_End(CmdPJ);
 END {BarCode_Print};

 PUBLIC PROCEDURE bcp_Contract();
 BEGIN
  L_Pmnt.AccessContract();
  L_General.Assert(Contract.Leaf = 1, "   蠡 ");
  CONFORM(Contract, Firm);
  BarCode_Print(" ",
                L_XDoc.BarCode(L_XDoc.bck_IdIn20, L_XDoc.xdt_Contract, L_XDoc.EmptyXDocKind, Contract.Contract, 0),
                00,
                20,
                "  " + Contract.SrcNbr + " (" + Contract.Contract + ")  " + L_DatTim.DateStr(Contract.SrcDate) + "|" +
                P_Firm.FP_PersonFullName(Firm.Firm, Memo.ud_cd()) + " (" + Firm.Firm + ")");
 END {bcp_Contract};

 PRIVATE PROCEDURE MathIsDevice(_Math: LIKE(Math.Math); tkType: L_TRS.tTckKindType): SCAL[2];
 BEGIN
  RETURN ~!_Math &
         SELECT(TckKind, Math, _Math) & (TckKind.Type = tkType) |
         ~!P_Math.GetParentMath(_Math, Memo.ud_cd()) &
         SELECT(TckKind, Math, P_Math.GetParentMath(_Math, Memo.ud_cd())) & (TckKind.Type = tkType);
 END {MathIsDevice};

 PRIVATE PROCEDURE ResetLab(fName: STRING[]);
 BEGIN
  TRY
   L_AR.GrantRightsUnconditionally();
   RESETLAB(^fName, 0);
   REPLACE(^fName);
  EXCEPT
  END; (* TRY *)
 END {ResetLab};

 PRIVATE PROCEDURE bcp_Make(Regime: tRegime);
  CONST
   MaxCUCopies = 100;
  VAR
   CUCopies    : LIKE(CUCopy.CUCopy)[MaxCUCopies];
   cucList,
   badCUCList,
   Title,
   prvTemplate,
   curTemplate : STRING[];
   i           : NUMERIC[0];
 BEGIN
  IF    Regime = r_CurCUC THEN
   L_General.Assert(ACCESS(CUCopy), " ⠭ ");
   CHANGESEL(CUCopy, CUCopy, , CUCopy.CUCopy);
  ELSIF Regime = r_LblCUC THEN
   SELECT(CUCopy, CUCopy);
  ELSIF Regime = r_CurTckBag THEN
   L_General.Assert(ACCESS(Storage), " ⠭ ⭠ 㬪");
   L_General.CallUs(Storage.Type = L_Stock.st_TckBag);
   L_General.CallUs(SELECT(CUCopy, Storage, Storage.Storage));
   CHANGESEL(CUCopy, CUCopy, , CUCopy.CUCopy);
  ELSIF Regime = r_LblTckBag THEN
   SELECT(Storage, TpStrg, , L_Stock.st_TckBag);
  ELSE
   L_General.CallUs(0);
  END;
  SETEMPTY(CUCopies, cucList, badCUCList, prvTemplate);
  i:=0;
  IF Regime = r_LblTckBag THEN
   Title      :="㬪 ᪠";
   curTemplate:="BC_TBG_01";
   WHILE OCCURE(Storage) DO
    PROMPT("ࠡ⪠ " + Storage.Storage);
    IF WITHLAB(Storage, 0) THEN
     IF i < MaxCUCopies THEN
      ResetLab("Storage");
      L_General.CallUs(SELECT(CUCopy, Storage, Storage.Storage));
      CUCopies[i]:=CUCopy.CUCopy;
      cucList    :=cucList + CUCopy.CUCopy + " " + Storage.Storage + " " + Storage.Name + L_Str.CRLF;
      INC(i);
     ELSE
      L_StdFrm.Inform(" ⠭ " + L_Str.IntStr(MaxCUCopies) + " ." +
                      "||  ⠢  ᫥  .");
      LAST(Storage);
     END;
    END; (* IF WITHLAB(Storage, 0) ... *)
    NEXT(Storage);
   END; (* WHILE OCCURE(Storage) ... *)
  ELSE
   WHILE OCCURE(CUCopy) DO
    PROMPT("ࠡ⪠ " + CUCopy.CUCopy);
    IF (Regime # r_LblCUC) | WITHLAB(CUCopy, 0) THEN
     IF i < MaxCUCopies THEN
      IF Regime = r_LblCUC THEN
       ResetLab("CUCopy");
      END;
      SETEMPTY(curTemplate);
      IF    ~!CUCopy.Storage THEN
       CONFORM(CUCopy, Storage);
       IF Storage.Type = L_Stock.st_TckBag THEN
        Title      :="㬪 ᪠";
        curTemplate:="BC_TBG_01";
       END;
      ELSIF MathIsDevice(CUCopy.Math, L_TRS.tkt_KKMActivities) THEN
       Title      :="஫쭮-ᮢ 設";
       curTemplate:="BC_KKM_01";
      ELSIF MathIsDevice(CUCopy.Math, L_TRS.tkt_VLDActivities) THEN
       Title      :="";
       curTemplate:="BC_VLD_01";
      ELSIF MathIsDevice(CUCopy.Math, L_TRS.tkt_BankCardTerminal) THEN
       Title      :="ନ ਥ ᪨ ";
       curTemplate:="BC_BCT_01";
      END;
      IF !curTemplate THEN
       CONFORM(CUCopy, Math);
       badCUCList :=badCUCList + CUCopy.CUCopy + " " + Str.Align(CUCopy.Serial, 24, LEFT) + " " + Math.Name + L_Str.CRLF;
      ELSE
       IF !prvTemplate THEN
        prvTemplate:=curTemplate
       ELSIF prvTemplate # curTemplate THEN
        L_General.Error("࠭     ⨯: " + CUCopy.CUCopy + "  " + CUCopies[i-1]);
       END;
       CUCopies[i]:=CUCopy.CUCopy;
       cucList    :=cucList + CUCopy.CUCopy + " " + CUCopy.Serial + L_Str.CRLF;
       INC(i);
      END; (*IF !curTemplate*)
     ELSE
      L_StdFrm.Inform(" ⠭ " + L_Str.IntStr(MaxCUCopies) + " ." +
                      "||  ⠢  ᫥  .");
      LAST(CUCopy);
     END; (* IF i < MaxCUCopies *)
    END; (* IF (Regime # r_LblCUC) | WITHLAB(CUCopy, 0) ... *)
    NEXT(CUCopy);
   END; (* WHILE OCCURE(CUCopy) ... *)
  END;
  IF ~!badCUCList THEN
   STDFORM(" ᫥ 祭 ஢  । 蠡   :"+
           "||%72,16NI" + L_Str.Repeat("|", 16) +
           "|%< {}  ⠫ ஢ >DB   %< {}⬥ >X",
           , INFORM, -1, badCUCList);
  END;
  L_General.Assert(~!cucList, "   ");
  IF STDFORM("|%56NI" +
             "|" +
             "|                                                 " +
             "|%56,16NI" +
             L_Str.Repeat("|", 16) +
             "|%< {} >DB   %< {}⬥ >X",
             "  ஢",
             ,
             -1,
             Title,
             cucList) THEN
   L_Stock.Stick_MakeDoc(curTemplate, CUCopies, L_MSO.PrpMode_Print, 1);
  END;
 END {bcp_Make};

 PUBLIC PROCEDURE bcp_CurCUC();
 BEGIN
  bcp_Make(r_CurCUC);
 END {bcp_CurCUC};

 PUBLIC PROCEDURE bcp_LblCUC();
 BEGIN
  bcp_Make(r_LblCUC);
 END {bcp_LblCUC};

 PUBLIC PROCEDURE bcp_CurTckBag();
 BEGIN
  bcp_Make(r_CurTckBag);
 END {bcp_CurTckBag};

 PUBLIC PROCEDURE bcp_LblTckBag();
 BEGIN
  bcp_Make(r_LblTckBag);
 END {bcp_LblTckBag};

 PUBLIC PROCEDURE bcp_CurOutCBlc();
 BEGIN
  L_General.Assert(ACCESS(OutCBlc), " ⠭ ஭ ॣ");
  CONFORM(OutCBlc, Math);
  CONFORM(OutCBlc, Firm);
  BarCode_Print(" ஭ ॣ",
                L_XDoc.BarCode(L_XDoc.bck_IdIn20, L_XDoc.xdt_CUCopy, L_XDoc.EmptyXDocKind, OutCBlc.CUCopy, 0),
                00,
                10,
                Math.Name + " (" + Math.Math + ")|" +
                "᪮  " + OutCBlc.Serial + " (" + OutCBlc.OutCBlc + ")|" +
                P_Firm.FP_PersonFullName(Firm.Firm, Memo.ud_cd()) + " (" + Firm.Firm + ")");
 END {bcp_CurOutCBlc};

 PRIVATE PROCEDURE AcceptableBCCommand(Mode: tMode; BCCommand: L_XDoc.tXDoc): SCAL[2];
 BEGIN
  IF   Mode = m_MainBC THEN
   RETURN (BCCommand = L_XDoc.cmd_CancelVerifing) |
          (BCCommand = L_XDoc.cmd_BadXDocBC);
  ELSIF  Mode = m_BadXDocBC THEN
   RETURN (BCCommand = L_XDoc.cmd_BadXDocBC);
  ELSIF  Mode = m_CancelVerifing THEN
   RETURN (BCCommand = L_XDoc.cmd_CancelVerifing);
  ELSIF Mode = m_GasStan THEN
   RETURN (BCCommand = L_XDoc.cmd_BadXDocBC)       |
          (BCCommand = L_XDoc.cmd_BuildGasStanTOV) |
          (BCCommand = L_XDoc.cmd_BuildGasStanMSP) |
          (BCCommand = L_XDoc.cmd_BuildGasStanTCh) |
          (BCCommand = L_XDoc.cmd_BegShift)        |
          (BCCommand = L_XDoc.cmd_EndShift);
  ELSE
   L_General.CallUs(0);
  END;
 END {AcceptableBCCommand};

 PRIVATE PROCEDURE bcp_AllCommand(Mode: tMode);
  VAR
   Y            : NUMERIC[0];
   CmdPJ        : PRINTJOB;
   BCCommand    : L_XDoc.tXDoc;
   BCCommandName: STRING[];
 BEGIN
  Y        :=20;
  CmdPJ    :=pj_Beg("ࠢ騥 ");
  BCCommand:=L_XDoc.cmd_CancelVerifing;
  LOOP
   BCCommandName:=L_Str.PoolItem(L_XDoc.cmd_Pool, BCCommand);
   IF !BCCommandName THEN
    EXIT;
   END;
   PROMPT("ࠢ騩  " + BCCommandName);
   IF AcceptableBCCommand(Mode, BCCommand) THEN
    Print(CmdPJ,
          30, Y,
          L_XDoc.BarCode(L_XDoc.bck_IdIn20, L_XDoc.xdt_Command, L_XDoc.EmptyXDocKind, BCCommand, 0),
          80, 0,
          BCCommandName);
    L_Print.TypeS(CmdPJ, 20, Y + 15, L_Str.Repeat("-", 150), L_Print.ts_Create(10, "Times New Roman"));
    INC(Y, 35);
   END;
   INC(BCCommand);
  END; (* LOOP *)
  pj_End(CmdPJ);
 END {bcp_AllCommand};

 PUBLIC PROCEDURE cbcp_MainBC();
 BEGIN
  bcp_AllCommand(m_MainBC);
 END {cbcp_MainBC};

 PUBLIC PROCEDURE cbcp_BadXDocBC();
 BEGIN
  bcp_AllCommand(m_BadXDocBC);
 END {cbcp_BadXDocBC};

 PUBLIC PROCEDURE cbcp_CancelVerifing();
 BEGIN
  bcp_AllCommand(m_CancelVerifing);
 END {cbcp_CancelVerifing};

 PUBLIC PROCEDURE cbcp_GasStan();
 BEGIN
  bcp_AllCommand(m_GasStan);
 END {cbcp_GasStan};

 PUBLIC PROCEDURE sp_CurMath();
  VAR
   trgQnt   : NUMERIC[0];
   PaperForm: STRING[];
   PJ       : PRINTJOB;
 BEGIN
  L_General.Assert(ACCESS(Math), " ⠭ ");
  PJ:=L_CUC.ps_CreateJob(PaperForm);
  trgQnt:=1;
  IF Math.MathKind # L_MathR.mathk_KeyAs THEN
   STDFORM(L_StdFrm.sf_Choice("|⢮  ஢ %##%", "%L<0>I"),
           "   ", INFORM, , trgQnt);
  END;
  L_CUC.PrintSticksByMath(PJ, PaperForm, Math.Math, trgQnt);
  PJ.Close();
 END {sp_CurMath};

 PUBLIC PROCEDURE sp_CurCUC();
  VAR
   PJ       : PRINTJOB;
   PaperForm: STRING[];
 BEGIN
  L_General.Assert(ACCESS(CUCopy), " ⠭ ");
  PJ:=L_CUC.ps_CreateJob(PaperForm);
  L_CUC.PrintStick(PJ, PaperForm, CUCopy.Math, CUCopy.CUCopy);
  PJ.Close();
 END {sp_CurCUC};

 PUBLIC PROCEDURE sp_LblCUC();
  VAR
   PJ       : PRINTJOB;
   PaperForm: STRING[];
 BEGIN
  PJ:=L_CUC.ps_CreateJob(PaperForm);
  SELECT(CUCopy, Math);
  WHILE OCCURE(CUCopy) DO
   NARROWSEL(CUCopy, Math);
   IF ~!CUCopy.Math THEN
    SRCINSEL(CUCopy, GT, , SYSDATE());
    WHILE OCCURE(CUCopy) DO
     PROMPT(CUCopy.Math + " " + CUCopy.CUCopy);
     IF WITHLAB(CUCopy, 0) THEN
      SAVE(CUCopy);
      L_CUC.PrintStick(PJ, PaperForm, CUCopy.Math, CUCopy.CUCopy);
      ResetLab("CUCopy");
      RESTORE(CUCopy);
     END;
     NEXT(CUCopy);
    END; (* WHILE OCCURE(File) *)
   END; (* IF ~!CUCopy.Math *)
   NEXTINEXTENDEDSEL(CUCopy, Math);
  END; (* WHILE OCCURE(CUCopy) *)
  PJ.Close();
 END {sp_LblCUC};

END.
