(*
%Copyright   (c) Polak IT
%Author      04.03.2005 11:25:04 POLAK\LDA
%Purpose
%Type        control,report
%Specific    AP
%Modified    08.09.2014 18:52:41 POLAK\Gina
%Environment
*)

 MODULE CarAD [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Events, L_General, L_Car, P_Route, P_Math, L_DatTim, L_CarAD,
        L_Acnt, L_MDoc, Str, L_StdFrm, L_XDoc, L_Pmnt, L_Str, Memo, L_MathR,
        L_bp, L_Basis, Math, L_AFM, L_Way, FIO;

 PRIVATE CONST
  mdt_First = 0;
  mdt_Last  = 1;
  mdt_Usual = 2;

 PRIVATE PROCEDURE CarAD_OnBeforeInsert();
 BEGIN
  CarAD.Date  :=Memo.ud_cd();
  CarAD.Status:=L_MDoc.st_Prepared;
  REPLACE(CarAD);
 END {CarAD_OnBeforeInsert};

 PRIVATE PROCEDURE CarAD_OnBeforeEdit();
 BEGIN
  L_Pmnt.CheckXDocIsInMDoc(L_XDoc.xdt_CarAD, PROCREF(L_Pmnt.E_IsAcptMDoc));
 END {CarAD_OnBeforeEdit};

 PRIVATE PROCEDURE CheckContract();
 BEGIN
  IF ~!CarAD.Contract THEN
   CONFORM(CarAD, Contract);
   L_General.Assert(Contract.Leaf, "ᯮ짮 蠡 ஢ ⨬");
   L_General.Assert(L_DatTim.IsActive_Inc(CarAD.Date, Contract.Begin, Contract.Till),
                    " " + CarAD.Contract + "     ४ ᤥ" +
                    L_DatTim.DateStr(CarAD.Date));
  END;
 END {CheckContract};

 PRIVATE PROCEDURE CarAD_Date_OnSuccessfulChange();
 BEGIN
  CheckContract();
 END {CarAD_Date_OnSuccessfulChange};

 PRIVATE PROCEDURE CarAD_Contract_OnSuccessfulChange();
 BEGIN
  CheckContract();
 END {CarAD_Contract_OnSuccessfulChange};

 PRIVATE PROCEDURE CheckCarWorkPeriod();
 BEGIN
  L_General.Assert((CarADL.BegDate >= CarAD.BegDate) & (CarADL.EndDate<=CarAD.EndDate), "४ ਮ ࠡ  " + CarADL.BoxNbr);
  IF CarADL.Period # L_DatTim.MonthsInIntvl(CarADL.BegDate, CarADL.EndDate) THEN
   L_StdFrm.Question("ப ४ (" + L_Str.IntStr(CarADL.Period) + " .)  ᮢ|" +
                     " 㪠 ਮ ࠡ ");
  END;
 END {CheckCarWorkPeriod};

 PRIVATE PROCEDURE CarAD_OnAfterEdIns();
 BEGIN
  IF ~!CarAD.BegDate & ~!CarAD.EndDate THEN
   L_General.Assert(CarAD.BegDate <= CarAD.EndDate, "४ ਮ ४ ᤥ");
  END;
  L_General.Assert(CarAD.Date <= CarAD.BegDate, " ४ ᤥ    | 砫 ਮ ⢨ ४ ᤥ");

  L_General.Assert(SELECT(CarADL, CarAD, CarAD.CarAD), " ᤥ  ᮤঠ    ࠡ");
  WHILE OCCURE(CarADL) DO
   PROMPT(" " + CarADL.BoxNbr);
   CheckCarWorkPeriod();
   NEXT(CarADL);
  END; (* WHILE OCCURE(CarADL) ... *)
 END {CarAD_OnAfterEdIns};

 PRIVATE PROCEDURE CarAD_OnAfterEdit();
 BEGIN
  CarAD_OnAfterEdIns();
 END {CarAD_OnAfterEdit};

 PRIVATE PROCEDURE CarAD_OnAfterInsert();
 BEGIN
  CarAD_OnAfterEdIns();
 END {CarAD_OnAfterInsert};

 PRIVATE PROCEDURE GetPriceIdx(RouteGroup: P_Math.tRAT_RouteGroup;
                               Value     : STRING[]): NUMERIC[0];
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=0;
  WHILE i <= HIGH(RouteGroup) DO
   IF Value = RouteGroup[i] THEN
    RETURN i;
   END;
   INC(i);
  END;
  L_General.Error("  :|" +
                  " "+CarADL.Route+"|" +
                  "㯯 ⮢ " + Value + "|"+
                  " " + CarADL.Math);
 END {GetPriceIdx};

 PRIVATE PROCEDURE as_GetPeriodCoef(_Period: NUMERIC[0]; PerCoefCountType: P_Math.tRAT_PerCoefCountType): NUMERIC[2];
 BEGIN
  IF PerCoefCountType = 0 THEN
   IF    _Period < 3 THEN
    RETURN 1.5;
   ELSIF _Period < 6 THEN
    RETURN 1.2;
   ELSIF _Period = 6 THEN
    RETURN 1.0;
   ELSE
    L_General.Error(" ⮨ ४  ப " + L_Str.Months_asStr(_Period) + "  ঠ");
   END;
  ELSIF PerCoefCountType = 1 THEN
   RETURN 1.0;
  ELSE
   L_General.CallUs(0);
  END;
 END {as_GetPeriodCoef};

 PRIVATE PROCEDURE as_CalcSum(_Period: NUMERIC[0]): NUMERIC[2];
 BEGIN
  RETURN (CarADL.Cost[L_CarAD.psidx_VAT] + CarADL.Cost[L_CarAD.psidx_Netto]) * _Period;
 END {as_CalcSum};

 PRIVATE PROCEDURE CalcCarADLSum(MountCalcRequired, DismountCalcRequired: SCAL[2]);
  VAR
   _Sum,
   _Cost,
   Mount,
   Dismount        : NUMERIC[2];
   Price           : P_Math.tRAT_Price;
   RouteGroup      : P_Math.tRAT_RouteGroup;
   PerCoefCountType: P_Math.tRAT_PerCoefCountType;
 BEGIN
  IF ~!CarADL.BoxNbr & ~!CarADL.Math & ~!CarADL.Route & ~!CarADL.Period THEN
   P_Math.GetAdvertisingServicesTariff(CarADL.Math, CarAD.Date, Mount, Dismount, PerCoefCountType, RouteGroup, Price);
   IF MountCalcRequired THEN
    CarADL.Mount[L_CarAD.psidx_VAT]  :=L_Acnt.GetInclVAT(CarAD.Date, Mount);
    CarADL.Mount[L_CarAD.psidx_Netto]:=Mount - CarADL.Mount[L_CarAD.psidx_VAT];
   ELSE
    SETZERO(CarADL.Mount);
   END;
   IF DismountCalcRequired THEN
    CarADL.Dismount [L_CarAD.psidx_VAT]  :=L_Acnt.GetInclVAT(CarAD.Date, Dismount);
    CarADL.Dismount [L_CarAD.psidx_Netto]:=Dismount - CarADL.Dismount[L_CarAD.psidx_VAT];
   ELSE
    SETZERO(CarADL.Dismount);
   END;

   _Cost:=Price[GetPriceIdx(RouteGroup, P_Route.GetRouteProStrong(CarADL.Route, CarAD.Date, P_Route.P_AdvertisingGroup))] * as_GetPeriodCoef(CarADL.Period, PerCoefCountType);

   CarADL.Cost    [L_CarAD.psidx_VAT]  :=L_Acnt.GetInclVAT(CarAD.Date, _Cost);
   CarADL.Cost    [L_CarAD.psidx_Netto]:=_Cost - CarADL.Cost[L_CarAD.psidx_VAT];

   _Sum:=as_CalcSum(CarADL.Period);
   CarADL.Sum     [L_CarAD.psidx_VAT]   :=L_Acnt.GetInclVAT(CarAD.Date, _Sum);
   CarADL.Sum     [L_CarAD.psidx_Netto] :=_Sum - CarADL.Sum[L_CarAD.psidx_VAT];
  ELSE
   SETEMPTY(CarADL.Mount, CarADL.Dismount, CarADL.Cost, CarADL.Sum);
  END;
  REPLACE(CarADL);
 END {CalcCarADLSum};

 PRIVATE PROCEDURE CarADL_BoxNbr_OnSuccessfulChange();
 BEGIN
  IF ~!CarADL.BoxNbr THEN
   L_General.Assert(L_Car.CarIsInOperation(CarADL.BoxNbr, CarAD.Date),
                    "   ᯫ樨  " + L_DatTim.DateStr(CarAD.Date));
  END;
  CalcCarADLSum(~CarADL.Prlngtn, 1);
 END {CarADL_BoxNbr_OnSuccessfulChange};

 PRIVATE PROCEDURE CarADL_Route_OnSuccessfulChange();
  VAR
   Dummy: STRING[];
 BEGIN
  IF ~!CarADL.Route THEN
   Dummy:=P_Route.GetRouteProStrong(CarADL.Route, CarAD.Date, P_Route.P_AdvertisingGroup);
  END;
  CalcCarADLSum(~CarADL.Prlngtn, 1);
 END {CarADL_Route_OnSuccessfulChange};

 PRIVATE PROCEDURE CarADL_Math_OnSuccessfulChange();
  VAR
   Dummy: STRING[];
 BEGIN
  IF ~!CarADL.Math THEN
   CONFORM(CarADL, Math);
   L_General.Assert(Math.MathType = L_MathR.mtyp_AdvertisingServices,
                    "  ");
   Dummy:=P_Math.GetMathProStrong(CarADL.Math, CarAD.Date, P_Math.P_Tariff);
  END;
  CalcCarADLSum(~CarADL.Prlngtn, 1);
 END {CarADL_Math_OnSuccessfulChange};

 PRIVATE PROCEDURE CarADL_Period_OnSuccessfulChange();
 BEGIN
  CalcCarADLSum(~CarADL.Prlngtn, 1);
 END {CarADL_Math_OnSuccessfulChange};

 PRIVATE PROCEDURE CarADL_OnAfterEdIns();
  VAR
   curCarADL       : LIKE(CarADL.CarADL);
   HasFirstCarADL,
   DismountRequired: SCAL[2];
 BEGIN
  CheckCarWorkPeriod();
  curCarADL     :=CarADL.CarADL;
  IF CarADL.Prlngtn THEN
   HasFirstCarADL:=0;
   SAVE(CarADL);
   SELECT(CarADL, DBoxNbr, , CarADL.BoxNbr);
   LAST(CarADL);
   L_General.Assert(CarADL.CarADL = curCarADL, "஫ ४ ᤥ   ᠬ ᫥|" +
                                               "  ४ ᤥ   " + CarADL.BoxNbr);
   DismountRequired:=1;
   WHILE OCCURE(CarADL) DO
    HasFirstCarADL:=~CarADL.Prlngtn;
    CalcCarADLSum(~CarADL.Prlngtn, DismountRequired);
    DismountRequired:=0;
    PREV(CarADL);
   END; (* WHILE OCCURE(CarADL) ... *)

   RESTORE(CarADL);
   L_General.Assert(HasFirstCarADL, "  室 ४ ᤥ   " + CarADL.BoxNbr + ".|" +
                                    "஫ ");
  END;
  INC(CarAD.Total, L_CarAD.GetCarADLTotal());
  INC(CarAD.VAT,   L_CarAD.GetCarADLTotalVAT());
  REPLACE(CarAD);
 END {CarADL_OnAfterEdIns};

 PRIVATE PROCEDURE CarADL_OnAfterEdit();
 BEGIN
  CarADL_OnAfterEdIns();
 END {CarADL_OnAfterEdit};

 PRIVATE PROCEDURE CarADL_OnAfterInsert();
 BEGIN
  CarADL_OnAfterEdIns();
 END {CarADL_OnAfterInsert};

 PRIVATE PROCEDURE CarADL_OnBeforeInsert();
 BEGIN
  CarADL.BegDate:=CarAD.BegDate;
  CarADL.EndDate:=CarAD.EndDate;
  REPLACE(CarADL);
 END {CarADL_OnBeforeInsert};

 PRIVATE PROCEDURE CarADL_OnBeforeEdDelete();
 BEGIN
  DEC(CarAD.Total, L_CarAD.GetCarADLTotal());
  DEC(CarAD.VAT,   L_CarAD.GetCarADLTotalVAT());
  REPLACE(CarAD);
 END {CarADL_OnBeforeEdDelete};

 PRIVATE PROCEDURE CarADL_OnBeforeEdit();
 BEGIN
  CarADL_OnBeforeEdDelete();
 END {CarADL_OnBeforeEdit};

 PRIVATE PROCEDURE CarADL_OnBeforeDelete();
 BEGIN
  CarADL_OnBeforeEdDelete();
 END {CarADL_OnBeforeDelete};

 PRIVATE PROCEDURE CarAD_OnBeforeDelete();
 BEGIN
  L_General.Error(" 饭");
 END {CarAD_OnBeforeDelete};

 PUBLIC PROCEDURE RemAddDoc();
 BEGIN
  L_CarAD.RemAddDoc();
 END {RemAddDoc};

 PRIVATE PROCEDURE CAD01_MakeTop(F: STRING[]);
 BEGIN
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "ࠣ");
  L_Basis.wb_WrStr(F, "ࠣ");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "C");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "ࠦ");
  L_Basis.wb_WrStr(F, "᭮");
  L_Basis.wb_WrStr(F, "ઠ");
  L_Basis.wb_WrStr(F, "ઠ");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "⠫   ");
  L_Basis.wb_WrStr(F, "⠫");
  L_Basis.wb_WrStr(F, "६ ࠡ");
  L_Basis.wb_TermLine(F);
 END {CAD01_MakeTop};

 PRIVATE PROCEDURE CAD01_MakeLine(F: STRING[]; D: DATE; wrkR, wrk: SCAL[2]; wrkTim: NUMERIC[3]; route: LIKE(Route.Route));
 BEGIN
  L_Basis.wb_WrScl(F, CarAD.CarAD);
  L_Basis.wb_WrStr(F, CarAD.SrcNbr);
  L_Basis.wb_WrStr(F, CarAD.Article);
  L_Basis.wb_WrScl(F, Contract.Contract);
  L_Basis.wb_WrStr(F, Contract.Name);
  L_Basis.wb_WrScl(F, Firm.Firm);
  L_Basis.wb_WrStr(F, Firm.Name);
  L_Basis.wb_WrScl(F, CarADL.CarADL);
  L_Basis.wb_WrStr(F, CarADL.RqstNbr);
  L_Basis.wb_WrDat(F, CarADL.BegDate);
  L_Basis.wb_WrDat(F, CarADL.EndDate);
  L_Basis.wb_WrScl(F, CarADL.Route);
  L_Basis.wb_WrScl(F, Car.BoxNbr);
  L_Basis.wb_WrStr(F, Car.StateNbr);
  L_Basis.wb_WrScl(F, Mark.Mark);
  L_Basis.wb_WrStr(F, Mark.Name);
  IF ~!route THEN
   SEARCH(Route, Route, , route);
   L_Basis.wb_WrStr(F, Route.Route);
   L_Basis.wb_WrStr(F, Route.Name);
  ELSE
   L_Basis.wb_WrStr(F, "");
   L_Basis.wb_WrStr(F, "");
  END;
  L_Basis.wb_WrDat(F, D);
  L_Basis.wb_WrNum(F, wrkR, 0);
  L_Basis.wb_WrNum(F, wrk, 0);
  L_Basis.wb_WrNum(F, wrkTim, 3);
  L_Basis.wb_TermLine(F);
 END {CAD01_MakeLine};

 PRIVATE PROCEDURE CAD01_IsWork(trgWJMode: LIKE(WJMode.WJCode)): SCAL[2];
 BEGIN
  L_General.CallUs(SEARCH(WJMode, WJCode, trgWJMode));
  RETURN (WJMode.RunSel > L_Way.runsel_None) & (WJMode.WJType = L_Way.WJType_Work);
 END {CAD01_IsWork};

 PUBLIC PROCEDURE CAD01();
  CONST
   bpId = "CAD01";
  VAR
   fr,
   ut,
   D,
   curFr,
   curUt : DATE;
   wrk   : SCAL[2];
   wrkTim: NUMERIC[3];
 BEGIN
  RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
  L_bp.Begin(bpId, " ᯮ ४ ᤥ", L_bp.t_Basis, L_General.tc_inIntvl);
  fr:=Memo.ud_fr();
  ut:=Memo.ud_ut();
  L_AFM.MakeMultipleIndex("tmp", "Date  : DATE; " +
                                 "Route : LIKE(Route.Route); " +
                                 "TimTot: NUMERIC[3]",
                          "tmp: Date, Route");
  FIO.Open(F, L_General.NameConnection(bpId), "rep", Create);
  CAD01_MakeTop("F");
  SELECT(CarAD, CarAD);
  WHILE OCCURE(CarAD) DO
   PROMPT(CarAD.CarAD);
   IF L_DatTim.IsActiveInIntvl_Inc(fr, ut, CarAD.BegDate, CarAD.EndDate) THEN
    SELECT(CarADL, CarAD, , CarAD.CarAD);
    WHILE OCCURE(CarADL) DO
     PROMPT(CarAD.CarAD + " " + CarADL.CarADL);
     IF L_DatTim.IsActiveInIntvl_Inc(fr, ut, CarADL.BegDate, CarADL.EndDate) THEN
      CONFORM(CarAD, Contract);
      CONFORM(Contract, Firm);
      CONFORM(CarADL, Car);
      L_Car.AccessMark(Car.BoxNbr, ut);

      SELECT(tmp, tmp);
      REMSEL(tmp);
      curFr:=Math.Max(fr, CarADL.BegDate);
      curUt:=Math.Min(ut, CarADL.EndDate);
      SELECT(RWayS, sBoxNbr, , L_Way.rwsStat_Rated, Car.BoxNbr);
      SRCINSEL(RWayS, GE, , curFr - 5);
      WHILE OCCURE(RWayS) & (RWayS.Date <= curUt) DO
       PROMPT(CarAD.CarAD + " " + CarADL.CarADL + "   " + L_DatTim.DateStr(RWayS.Date));
       CONFORM(RWayS, RWayJ);
       WHILE OCCURE(RWayJ) DO
        D:=L_Way.GetDateOfRWayJ();
        IF L_DatTim.IsActive_Inc(D, curFr, curUt) THEN
         CONFORM(RWayJ, Quality);
         CONFORM(RWayJ, SEJobs);
         IF (Quality.Type # 0) & CAD01_IsWork(SEJobs.WJCode) THEN
          ADD(tmp, tmp, , D, SEJobs.Route,
                  tmp.TimTot, RWayJ.StopT - RWayJ.StartT);
         END;
        END;
        NEXT(RWayJ);
       END; (* WHILE OCCURE(RWayJ) ... *)
       NEXT(RWayS);
      END; (* WHILE OCCURE(RWayS) ... *)

      D:=curFr;
      WHILE D <= curUt DO
       PROMPT(CarAD.CarAD + " " + CarADL.CarADL + "  " + L_DatTim.DateStr(D));
       IF SELECT(tmp, tmp, D) THEN
        WHILE OCCURE(tmp) DO
         CAD01_MakeLine("F", D, tmp.Route=CarADL.Route, 1, tmp.TimTot, tmp.Route);
         NEXT(tmp);
        END; (* WHILE OCCURE(tmp) ... *)
       ELSE
        CAD01_MakeLine("F", D, 0, 0, 0, "");
       END;
       INC(D);
      END;
     END; (* IF L_DatTim.IsActiveInIntvl_Inc(fr, ut, CarADL.BegDate, CarADL.EndDate) *)
     NEXT(CarADL);
    END; (* WHILE OCCURE(CarADL) ... *)
   END; (* IF L_DatTim.IsActiveInIntvl_Inc(fr, ut, CarAD.BegDate, CarAD.EndDate) *)
   NEXT(CarAD);
  END; (* WHILE OCCURE(CarAD) ... *)
  FIO.Close(F);
  L_bp.End();
 END {CAD01};

BEGIN
 L_Events.ExecuteHandler();
END.
