(*
%Copyright   (c) Polak Computer System
%Author      21.10.1999 10:21:52 BOR
%Purpose     ᠭ 
%Type        control
%Specific    AP
%Modified    20.06.2014 14:05:09 POLAK\Bor
%Environment
*)

 MODULE CarLog [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_Events, L_MathR, L_Car, L_StdFrm, L_Str, L_MDoc, L_CUC,
        L_AcsPnt, Str, L_Env, L_Rpr, L_Way, L_WDO, P_Firm, L_APtm;

 PRIVATE CONST
  acspnt_CarLogModification = 0000008;

 PRIVATE PROCEDURE DateCross(B1, E1, B2, E2: DATE): SCAL[2];
 BEGIN
  IF !E2 THEN
   IF !E1 THEN
    RETURN 1;
   ELSE
    RETURN B2 < E1;
   END;
  ELSE
   RETURN L_DatTim.DATEINT(B1, E1, B2, E2)>0;
  END;
 END {DateCross};

 PRIVATE PROCEDURE CheckCLField(_Field, _Name: STRING[]);
  VAR
   _BoxNbr : LIKE(CarLog.BoxNbr);
   _Date   : LIKE(CarLog.Date);
   _DateE  : LIKE(CarLog.DateE);
 BEGIN
  SAVE(CarLog);
  _BoxNbr:=CarLog.BoxNbr;
  _Date  :=CarLog.Date;
  _DateE :=CarLog.DateE;
  SELECT(CarLog, ^_Field, , CarLog.^_Field);
  WHILE OCCURE(CarLog) DO
   IF (CarLog.BoxNbr # _BoxNbr) & DateCross(_Date, _DateE, CarLog.Date, CarLog.DateE) THEN
    L_General.Error(_Name + " " +CarLog.^_Field + "  ਮ  " +
                    L_DatTim.DateStr(_Date) + "  " + L_DatTim.DateStr(_DateE) +
                    "|" +
                    "㦥 ᮯ⠢ ࠭ᯮ⭮ । " + CarLog.BoxNbr);
   END;
   NEXT(CarLog);
  END; (* WHILE OCCURE(CarLog) ... *)
  RESTORE(CarLog);
 END {CheckCLField};

 PRIVATE PROCEDURE IsOwnerOrTTPayer(): SCAL[2];
 BEGIN
  CONFORM(CarLog, VehPasp);
  CONFORM(CarLog, Column);
  IF Column.MyFirm = P_Firm.GetMyFirmOfFirm(VehPasp.Firm) THEN
   RETURN 1;
  ELSIF VehPasp.TTPayer = L_Car.ttp_ThirdParty THEN
   RETURN 0;
  ELSIF VehPasp.TTPayer = L_Car.ttp_Operator THEN
   RETURN 1;
  ELSE
   RETURN 0;
  END;
 END {IsOwnerOrTTPayer};

 PRIVATE PROCEDURE CheckMath(StreamCheck: SCAL[2]);
  VAR
   ChDate: DATE;
 BEGIN
  IF L_Env.KeyAssAcntLev(SYSDATE()) & ~L_MathR.E_NotFullActiveKeyAssSystem(SYSDATE())  THEN
   IF ~!CarLog.Math THEN
    L_General.Assert(L_Env.RealBoxNbr(CarLog.BoxNbr), "   ᮯ⠢   䨪⨢ ");
    CONFORM(CarLog, Math);
    L_General.Assert(Math.MathKind=L_MathR.mathk_KeyAs, "ꥪ .  ⭮  ᭮ ।⢠");

    IF StreamCheck & SELECT(MathKAGL, Math, CarLog.Math) THEN
     ChDate:=L_DatTim.MinDATE(CarLog.DateE, SYSDATE());
     IF L_MathR.PosToKAGSoft(CarLog.Math, ChDate) THEN
      L_General.Assert(KAGLog.IsCar, "ꥪ  "+CarLog.Math+"  "+L_DatTim.DateStr(ChDate)+"   ࠭ᯮ ।⢮"+
                                     L_StdFrm.THR(" 稭")+
                                     "1.  訡 ꥪ                                                     |"+
                                     "2.  ᠭ 㯯 த  "+KAGLog.KAGroup+" 訡筮  㪠 ਧ ' '");
     END;
    END;
    CheckCLField("Math", "ꥪ ");
   ELSIF ~StreamCheck &
         (L_Env.KeyAssAcntLev(CarLog.DateE) = L_MathR.kaal_WithActProcess) THEN
    IF !CarLog.VehPasp THEN
     L_StdFrm.CrucialChoice(" 㪠 ᮯ⠢ ꥪ " + L_StdFrm.HR() +
                            " ᪠ ⮫쪮  㦨 ,|"+
                            "  ⥫騪 ࠭ᯮ⭮ |"+
                            "   ।⨥",
                            L_APtm.g_Acnt);
    ELSIF IsOwnerOrTTPayer() THEN
     L_General.Error("室 㪠 ꥪ ");
    END;
   END;
  END;
 END {CheckMath};

 PRIVATE PROCEDURE CheckVehPasp();
 BEGIN
  IF ~!CarLog.VehPasp THEN
   CheckCLField("VehPasp", "ᯮ ");
  END;
 END {CheckVehPasp};

 PRIVATE PROCEDURE CheckCarLogOrder(prv, cur: LIKE(CarLog.Status));
  VAR
   Ok: SCAL[2];
 BEGIN
  IF    prv = L_Car.carst_Retired THEN
   Ok:=!cur;
  ELSIF cur = L_Car.carst_Retired THEN
   Ok:=~!prv;
  ELSE
   Ok:=1;
  END;
  L_General.Assert(Ok, "⨬ 冷 ᫥ ᥩ  ﭨ");
 END {CheckCarLogOrder};

 PRIVATE PROCEDURE SetCarLogFields();
  VAR
   _prv,
   _cur : LIKE(CarLog.Status);
 BEGIN
  SAVE(CarLog);
  IF PREV(CarLog) THEN
   _prv:=CarLog.Status;
  ELSE
   SETEMPTY(_prv);
  END;
  RESTORE(CarLog);
  _cur:=CarLog.Status;
  CheckCarLogOrder(_prv, _cur);
  SAVE(CarLog);
  IF NEXT(CarLog) THEN
   CheckCarLogOrder(_cur, CarLog.Status);
  END;
  RESTORE(CarLog);
 END {SetCarLogFields};

 PRIVATE PROCEDURE SupportSettingBegDate();
 BEGIN
  IF Car.BegDate # CarLog.Date THEN
   IF ~!Car.BegDate THEN
    L_StdFrm.Choice("    ");
   END;
   Car.BegDate:=CarLog.Date;
   REPLACE(Car);
  END;
 END {SupportSettingBegDate};

 PRIVATE PROCEDURE SupportSettingEndDate();
  VAR
   _EndDate: LIKE(Car.EndDate);
 BEGIN
  IF CarLog.Status = L_Car.carst_Retired THEN
   CheckWritingOffCar(CarLog.BoxNbr);
   _EndDate:=CarLog.Date;
  ELSE
   SETEMPTY(_EndDate);
  END;
  IF Car.EndDate # _EndDate THEN
   IF    !Car.EndDate THEN
    L_StdFrm.Choice("   ");
   ELSIF !_EndDate THEN
    L_StdFrm.Choice("   ");
   ELSE
    L_StdFrm.Choice("   ");
   END;
   Car.EndDate:=_EndDate;
   REPLACE(Car);
  END;
 END {SupportSettingEndDate};

 PRIVATE PROCEDURE SupportBegEndDate();
  VAR
   curDateE: LIKE(CarLog.DateE);
 BEGIN
  SAVE(CarLog);
  ACCESS(Car);
  SETSEL(CarLog, BoxNbr, Car.BoxNbr);
  L_General.Assert(LAST(CarLog), "     ப ᠭ");
  SupportSettingEndDate();
  curDateE:=L_DatTim.till2ED(Car.EndDate);
  REPEAT
   CarLog.DateE:=curDateE;
   REPLACE(CarLog);
   CheckMath(1);
   CheckVehPasp();
   curDateE:=CarLog.Date-1;
  UNTIL ~PREV(CarLog);
  FIRST(CarLog);
  SupportSettingBegDate();
  RESTORE(CarLog);
 END {SupportBegEndDate};

 PRIVATE PROCEDURE CarLog_Date_OnSuccessfulChange();
 BEGIN
  SetCarLogFields();
 END {CarLog_Date_OnSuccessfulChange};

 PRIVATE PROCEDURE CarLog_StateNbr_OnSuccessfulChange();
 BEGIN
  CarLog.StateNbr:=L_Car.E_GetCorrectedStateNbr(CarLog.StateNbr, 1);
  REPLACE(CarLog);
 END {CarLog_StateNbr_OnSuccessfulChange};

 PRIVATE PROCEDURE GetChoiceAboutConvergence(Description, Suffix: STRING[]): SCAL[3];
 BEGIN
  RETURN STDFORM(" |" + Description + "  ࠭ |" +
                 " ᮮ⢥ 㪠" + Suffix + "  ᠭ.| |" +
                 L_StdFrm.DrawButton("ਢ  ᮮ⢥⢨", 48, "DB")+ "|" +
                 L_StdFrm.DrawButton("⠢ ஥ 祭", 48, "B")+ "|" +
                 L_StdFrm.DrawButton("⬥", 48, "X"),
                 "",
                 INFORM,
                 1);
 END {GetChoiceAboutConvergence};

 PRIVATE PROCEDURE CarLog_VehPasp_OnSuccessfulChange();
  VAR
   trgStateNbr: LIKE(CarLog.StateNbr);
 BEGIN
  L_General.Assert(~!CarLog.Date, " 㪠  砫 ⢨ ᠭ ");
  IF ~!CarLog.VehPasp THEN
   CheckVehPasp();
   CONFORM(CarLog, VehPasp);
   L_General.Assert(L_DatTim.IsActive(CarLog.Date, VehPasp.Begin, VehPasp.Till),
                    " " + VehPasp.VehPasp + "    " +
                    L_DatTim.DateStr(CarLog.Date));

   IF    !CarLog.Mark THEN
    CarLog.Mark:=VehPasp.Mark;
   ELSIF (CarLog.Mark # VehPasp.Mark) & (GetChoiceAboutConvergence("ઠ", "") = 1) THEN
    CarLog.Mark:=VehPasp.Mark;
   END;

   trgStateNbr:=VehPasp.StateNbr;
   IF ~!trgStateNbr THEN
    IF    !CarLog.StateNbr THEN
     CarLog.StateNbr:=trgStateNbr;
    ELSIF (CarLog.StateNbr # trgStateNbr) & (GetChoiceAboutConvergence("樮 ", "") = 1) THEN
     CarLog.StateNbr:=trgStateNbr;
    END;
   ELSE
    L_StdFrm.RedInform("   㪠 ॣ樮  ");
   END;
   REPLACE(CarLog);
  END;
 END {CarLog_VehPasp_OnSuccessfulChange};

 PRIVATE PROCEDURE SetWindowFields();
  VAR
   Fields  : FORM_FIELDS;
   FlagEdit: SCAL[2];
 BEGIN
  Fields  :=RTI.Get().CurZone.Fields;
  FlagEdit:=CarLog.Status # L_Car.carst_Retired;

  Fields("CarLog.Math").Modifiable    :=FlagEdit & (L_Env.KeyAssAcntLev(SYSDATE()) # L_MathR.kaal_NoKeyAssets);
  Fields("CarLog.Mark").Modifiable    :=FlagEdit;
  Fields("CarLog.StateNbr").Modifiable:=FlagEdit;
  Fields("CarLog.Column").Modifiable  :=FlagEdit;
  Fields("CarLog.InBeg").Modifiable   :=FlagEdit;
  Fields("CarLog.Purpose").Modifiable :=FlagEdit;
  Fields("CarLog.VehPasp").Modifiable :=FlagEdit;
 END {SetWindowFields};

 PRIVATE PROCEDURE CarLog_Status_OnSuccessfulChange();
 BEGIN
  L_General.Assert(~!CarLog.Status, "⨬ ⠢ ");
  SetCarLogFields();
  SetWindowFields();
 END {CarLog_Status_OnSuccessfulChange};

 PRIVATE PROCEDURE CheckWritingOffCar(_BoxNbr: LIKE(Car.BoxNbr));
  VAR
   Msg: STRING[];
 BEGIN
  SAVE(RWayS);
  SETSEL(RWayS, BoxNbr, _BoxNbr);
  IF LAST(RWayS) THEN
   Msg:="⨥ ࠭ᯮ⭮ ।⢠" +
        "|ࠦ  " + _BoxNbr +
        "|⥢  " + RWayS.RWayS + "| |";
   L_General.Assert(RWayS.Date < CarLog.Date,
                    Msg + "⨬  ᫥  ");
   L_General.Assert(RWayS.Status = L_Way.rwsStat_Rated,
                    Msg + "    '஢'");
   L_General.Assert(RWayS.RemEntr = 0,
                    Msg + "⨬ 㫥 ⮪  ");
  END;
  RESTORE(RWayS);

  SELECT(CarEvn, BoxNbr, , _BoxNbr);
  WHILE OCCURE(CarEvn) DO
   IF CarLog.Date < CarEvn.StopD THEN
    CONFORM(CarEvn, RepEvent);
    L_General.Error("    " + L_DatTim.DateStr(CarLog.Date) + "  ६⭮ ய⨥|" +
                    "'" + RepEvent.Name + "' (ਮ  " + L_DatTim.DateStr(CarEvn.StartD) + "  " + L_DatTim.DateStr(CarEvn.StopD) + ")");
   END;
   NEXT(CarEvn);
  END; (* WHILE OCCURE(CarEvn) ... *)

  SELECT(RprS, CUCopy, , Car.CUCopy);
  WHILE OCCURE(RprS) DO
   PROMPT(RprS.RprS);
   IF RprS.Status # L_Rpr.ms_Cancel THEN
    IF !RprS.EDate | (RprS.EDate > CarLog.Date) THEN
     L_General.Error("    " + L_DatTim.DateStr(CarLog.Date) + "    " + RprS.RprS);
    END;
   END;
   NEXT(RprS);
  END; (* WHILE OCCURE(RprS) ... *)

  SELECT(MDocLCU, BoxNbr, , Car.BoxNbr, L_MDoc.st_Passed);
  Msg:="⨥ ࠭ᯮ⭮ ।⢠" +
       "|ࠦ  " + _BoxNbr + "| |";
  WHILE OCCURE(MDocLCU) DO
   NARROWSEL(MDocLCU, CUCopy);
   LAST(MDocLCU);
   L_General.Assert(MDocLCU.Date <= CarLog.Date,
                    Msg + "  ॣ⮢ ᫥ " + L_DatTim.DateStr(CarLog.Date));
   L_General.Assert(~L_CUC.IsCUAtCar(MDocLCU.OpKind),
                    Msg + " ⠭ ॣ  " + L_DatTim.DateStr(CarLog.Date));
   NEXTINEXTENDEDSEL(MDocLCU, CUCopy);
  END; (* WHILE OCCURE(MDocLCU) ... *)
 END {CheckWritingOffCar};

 PRIVATE PROCEDURE CarLog_OnBeforeInsert();
 BEGIN
  L_AcsPnt.PassAccessPoint(acspnt_CarLogModification);
 END {CarLog_OnBeforeInsert};

 PRIVATE PROCEDURE CarLog_OnBeforeEdit();
 BEGIN
  L_AcsPnt.PassAccessPoint(acspnt_CarLogModification);
 END {CarLog_OnBeforeEdit};

 PRIVATE PROCEDURE CheckRWayS(_From, _UpTo: DATE);
  VAR
   Msg       : STRING[];
   chkDone,
   chkMyPlace: SCAL[2];
   newMyPlace: P_Firm.tMyPlace;
 BEGIN
  IF _UpTo < _From THEN (*⨥ *)
   SETEMPTY(_UpTo);
  END;
  TABLE.Get("RWayS").AccessDataMode:=adm_SOFT;
  SETZERO(chkDone, chkMyPlace);
  CONFORM(CarLog, Column);
  SETSEL(RWayS, BoxNbr, CarLog.BoxNbr);
  SRCINSEL(RWayS, GE, , _From);
  WHILE OCCURE(RWayS) & (RWayS.Date<=_UpTo) DO
   PROMPT("஢ઠ  "+RWayS.RWayS);
   IF RWayS.Done & ~chkDone THEN
    Msg:=" ࠡ⠭ ";
    IF !_UpTo THEN
     Msg:=Msg+"  "+L_DatTim.DateStr(_From);
    ELSE
     Msg:=Msg+"| ਮ  "+L_DatTim.DateStr(_From)+"  "+L_DatTim.DateStr(_UpTo);
    END;
    Msg:=Msg+"|४஢   ।⥫쭮 ⪠| ਢ  訡";
    L_General.Assert_wReferToAdmin(0, Msg);
    chkDone:=1;
   END; (* IF RWayS.Done ... *)
   IF ~chkMyPlace THEN
    TRY
     newMyPlace:=L_Way.GetMyPlace(RWayS.Date, RWayS.BoxNbr, RWayS.TSN, RWayS.Exit);
     IF newMyPlace # RWayS.MyPlace THEN
      RWayS.MyPlace:=newMyPlace;
      REPLACE(RWayS);
     END;
    EXCEPT
    ON Exc: Exc_Raise DO
     L_General.Assert_wReferToAdmin(0, Exc.MsgOriginalText);
     chkMyPlace:=1;
    END; (* TRY *)
   END; (* IF ~chkMyPlace *)
   IF (RWayS.MyFirm # Column.MyFirm) & (RWayS.MyFirm # P_Firm.mf_Fict) THEN
    L_General.Error(" , ⭥ᥭ  㣮 ");
   END;
   NEXT(RWayS);
  END; (* WHILE OCCURE(RWayS) ... *)
 END {CheckRWayS};

 PRIVATE PROCEDURE CarLog_OnBeforeDelete();
  VAR
   prv,
   nxt: LIKE(CarLog.Status);
 BEGIN
  L_AcsPnt.PassAccessPoint(acspnt_CarLogModification);
  SETEMPTY(prv, nxt);
  CHANGESEL(CarLog, BoxNbr, , CarLog.BoxNbr);
  SAVE(CarLog);
  IF PREV(CarLog) THEN
   prv:=CarLog.Status;
  END;
  RESTORE(CarLog);
  SAVE(CarLog);
  IF NEXT(CarLog) THEN
   nxt:=CarLog.Status;
  END;
  RESTORE(CarLog);
  CheckCarLogOrder(prv, nxt);
  CheckRWayS(CarLog.Date, CarLog.DateE);
 END {CarLog_OnBeforeDelete};

 PRIVATE PROCEDURE CarLog_OnAfterEdIns();
 BEGIN
  CONFORM(CarLog, Column);
  L_General.Assert(L_DatTim.IsActive_Inc(CarLog.Date, Column.Begin, Column.Till),
                   " " + Column.Column +  "    " + L_DatTim.DateStr(CarLog.Date));

  CONFORM(CarLog, Mark);
  L_General.Assert(L_DatTim.IsActive_Inc(CarLog.Date, Mark.Begin, Mark.Till),
                   "⨬ ਬ ન " + Mark.Mark +  "  " +
                   L_DatTim.DateStr(CarLog.Date));
  CheckMath(0);
  IF ~L_APtm.IsOG() THEN
   IF !CarLog.Order &
      !CarLog.OrdDate &
      !CarLog.OrdCsAtr THEN
    L_StdFrm.Question("  ਡ ਪ");
   END;
  END;
  IF (CarLog.Status = L_Car.carst_OutOperat) & (CarLog.InBeg # L_Car.inBeg_DoesNot) THEN
   L_StdFrm.Choice("浪  ᯫ㥬 ?");
  END;
  L_Car.CheckCarLogToVehPaspCompliance();
  SupportBegEndDate();
 END {CarLog_OnAfterEdIns};

 PRIVATE PROCEDURE CarLog_OnAfterInsert();
 BEGIN
  CarLog_OnAfterEdIns();
  CheckRWayS(CarLog.Date, CarLog.DateE);
 END {CarLog_OnAfterInsert};

 PRIVATE PROCEDURE CarLog_OnAfterEdit();
 BEGIN
  CarLog_OnAfterEdIns();
  IF (CarLog.Date    # GE_PVALUE(CarLog.Date))    |
     (CarLog.DateE   # GE_PVALUE(CarLog.DateE))   |
     (CarLog.Math    # GE_PVALUE(CarLog.Math))    |
     (CarLog.Mark    # GE_PVALUE(CarLog.Mark))    |
     (CarLog.Purpose # GE_PVALUE(CarLog.Purpose)) |
     (CarLog.Column  # GE_PVALUE(CarLog.Column))  |
     (CarLog.Status  # GE_PVALUE(CarLog.Status)) THEN
   CheckRWayS(L_DatTim.MinDATE(CarLog.Date,  GE_PVALUE(CarLog.Date)),
              L_DatTim.MaxDATE(CarLog.DateE, GE_PVALUE(CarLog.DateE)));
  END;
 END {CarLog_OnAfterInsert};

 PRIVATE PROCEDURE CarLog_OnAfterDelete();
 BEGIN
  SupportBegEndDate();
 END {CarLog_OnAfterDelete};

 PRIVATE PROCEDURE CarLog_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAnyModification, L_WDO.e_OnAfterDelete, L_WDO.e_OnReposition]);
  L_WDO.iField       (0, "CarLog.Math").OwnerWindowOriginalName:="Math01";
  L_WDO.iField       (0, "CarLog.Math").OwnerZoneNumber:=1;
  L_WDO.iScsChange   (0, ["CarLog.StateNbr", "CarLog.Status", "CarLog.VehPasp", "CarLog.Date"]);
  L_WDO.iUnmodifiable(0, ["CarLog.BoxNbr"]);
  L_WDO.iMayBeEmpty  (0, ["CarLog.OrdCsAtr", "CarLog.OrdDate", "CarLog.Order", "CarLog.VehPasp",
                          "CarLog.Math", "CarLog.TKind"]);
  L_WDO.iField       (0, "CarLog.Purpose").Format:=L_Car.purp_Pool_Full;
  L_WDO.iField       (0, "CarLog.InBeg").Format:=L_Car.inBeg_Pool;
  L_WDO.iField       (0, "CarLog.Status").Format:=L_Car.carst_Pool;
 END {CarLog_OnInitializeWindow};

 PRIVATE PROCEDURE CarLog_0_OnReposition();
 BEGIN
  SetWindowFields();
 END {CarLog_0_OnReposition};

BEGIN
 L_Events.ExecuteHandler();
END.
