(*
%Copyright   (c) Polak IT
%Author      17.11.2009 08:40:03 LDA
%Purpose
%Type        control,report,temporary,convertor,operation,library,to paste
%Specific    AP,AP_PC,AP_PT,SYS,TOOL,LOC
%Modified    13.11.2013 10:51:26 POLAK\GAV
%Environment
*)

 MODULE Expr [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Events, L_General, L_Man, L_Str, L_AFM, L_OwnData;

 PRIVATE PROCEDURE Expr_1_OnZoneOpen();
 BEGIN
  RTI.Get().CurZone.Fields("Expr.ExprType").Format:=L_Man.et_Pool;
  RTI.Get().CurZone.Fields("Expr.Source").Format:=L_Man.expsrc_Pool;
 END {Expr_1_OnZoneOpen};

 PRIVATE PROCEDURE GetDefaultExprLevelPool(_ExprType: LIKE(Expr.ExprType)): STRING[];
  VAR
   Pool: STRING[];
 BEGIN
  IF    L_Man.ExprIsAnyLanguage(_ExprType) THEN
   IF    L_Man.ExprIsNativeLanguage(_ExprType) THEN
    Pool:=" ᢮|";
   ELSE
    Pool:=L_Man.GetManLanguageLevelPool();
   END;
  ELSIF L_Man.ExprIsSkill(_ExprType) THEN
   Pool:=" ᮢ襭⢥|| 砫쭮 ஢";
  ELSIF L_Man.ExprIsStimulation(_ExprType) THEN
   Pool:="⭠ ࠬ|୮|祥";
  ELSIF L_Man.ExprIsTribute(_ExprType) THEN
   Pool:="थ||祥";
  ELSIF L_Man.ExprIsScienceAchievement(_ExprType) THEN
   Pool:="᪠|᪠|⥭|㡫";
  ELSE
   L_General.Error("GetDefaultExprLevelPool: ⨬ ExprType " + _ExprType);
  END;
  RETURN Pool;
 END {GetDefaultExprLevelPool};

 PRIVATE PROCEDURE GetExprLevelPool(_ExprType: LIKE(Expr.ExprType)): STRING[];
  VAR
   Pool: STRING[];
 BEGIN
  APP.Get().NewPrompt("⮢ 㫠 祭");
  Pool:=GetDefaultExprLevelPool(_ExprType);
  SAVE(Expr);
  SELECT(Expr, ExprTLev, , _ExprType);
  WHILE OCCURE(Expr) DO
   NARROWSEL(Expr, Level);
   PROMPT(Expr.Level);
   IF !L_Str.PositionInPool(Pool, Expr.Level) THEN
    Pool:=L_Str.AddStrDlm(Pool, "|") + Expr.Level;
   END;
   NEXTINEXTENDEDSEL(Expr, Level);
  END; (* WHILE OCCURE(Expr) ... *)
  RESTORE(Expr);
  RETURN Pool;
 END {GetExprLevelPool};

 PRIVATE PROCEDURE GetDefaultExprNamePool(_ExprType: LIKE(Expr.ExprType)): STRING[];
  VAR
   Pool: STRING[];
 BEGIN
  SETEMPTY(Pool);
  IF L_Man.ExprIsAnyLanguage(_ExprType) THEN
   Pool:="᪨|᪨|檨|࠭᪨|ᯠ᪨|" +
         "⠫᪨|᪨|堧᪨|ࡠ᪨|" +
         "᪨|誨᪨|᪨|᪨|" +
         "㧨᪨|⠭᪨|न-᪨|" +
         "᪨|ࣨ᪨||᪨|⮢᪨|" +
         "᪨|म᪨|⨭᪨|᪨|" +
         "᪨|બ᪨|᪨|ࠨ᪨|" +
         "⮭᪨|᪨|모 த ";
  ELSIF L_Man.ExprIsSkill(_ExprType) THEN
   Pool:=L_Man.GetSkillNamePool();
  END;
  RETURN Pool;
 END {GetDefaultExprNamePool};

 PRIVATE PROCEDURE GetExprNamePool(_ExprType: LIKE(Expr.ExprType)): STRING[];
  VAR
   Pool: STRING[];
 BEGIN
  APP.Get().NewPrompt("⮢ 㫠 祭");
  Pool:=GetDefaultExprNamePool(_ExprType);
  IF (_ExprType = L_Man.et_NativeLanguage)  |
     (_ExprType = L_Man.et_OtherLanguage)   |
     (_ExprType = L_Man.et_ForeignLanguage) |
     (_ExprType = L_Man.et_Skill) THEN
   SAVE(Expr);
   SELECT(Expr, ExprType, , _ExprType);
   WHILE OCCURE(Expr) DO
    NARROWSEL(Expr, Name);
    PROMPT("⮢ " + Expr.Name);
    IF !L_Str.PositionInPool(Pool, Expr.Name) THEN
     Pool:=L_Str.AddStrDlm(Pool, "|") + Expr.Name;
    END;
    NEXTINEXTENDEDSEL(Expr, Name);
   END; (* WHILE OCCURE(Expr) ... *)
   RESTORE(Expr);
  END;
  RETURN Pool;
 END {GetExprNamePool};

 PRIVATE PROCEDURE SetExprSourceTemplate();
  VAR
   Field: FORMFIELD;
 BEGIN
  Field:=RTI.Get().CurZone.Fields("Expr.Source");
  IF L_Man.ExprIsStimulation(Expr.ExprType) | L_Man.ExprIsTribute(Expr.ExprType) THEN
   Field.TemplateIsStrict:=1;
   Field.Template:="७|譨";
   Field.Modifiable:=1;
  ELSE
   Expr.Source:=L_Man.expsrc_Skill;
   REPLACE(Expr);
   Field.Modifiable:=0;
  END;
 END {SetExprSourceTemplate};

 PRIVATE PROCEDURE SetExprTemplates(forceEmpty: SCAL[2]);
  VAR
   Pool  : STRING[];
   Fields: FORMFIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  IF ~forceEmpty THEN
   Pool:=GetExprLevelPool(Expr.ExprType);
   IF Expr.ExprType = L_Man.et_ForeignLanguage THEN
    Fields("Expr.Level").TemplateIsStrict:=1;
    Fields("Expr.Level").Template:=Pool;
   ELSE
    Fields("Expr.Level").Template:=Pool;
   END;
   Fields("Expr.Name").Template:=GetExprNamePool(Expr.ExprType);
   SetExprSourceTemplate();
  ELSE
   Fields("Expr.Level").Template:="";
   Fields("Expr.Name").Template:="";
   Fields("Expr.Source").Template:="";
  END;
 END {SetExprTemplates};

 PRIVATE PROCEDURE Expr_OnBeforeEdit();
 BEGIN
  SetExprTemplates(0);
 END {Expr_OnBeforeEdit};

 PRIVATE PROCEDURE Expr_OnBeforeInsert();
 BEGIN
  SetExprTemplates(1);
 END {Expr_OnBeforeInsert};

 PUBLIC PROCEDURE ExprE_0_OnZoneOpen();
 BEGIN
  RTI.Get().CurZone.Fields("Expr.ExprType").Format:=L_Man.et_Pool;
  RTI.Get().CurZone.Fields("Expr.Source").Format:=L_Man.expsrc_Pool;
 END {ExprE_0_OnZoneOpen};

 PRIVATE PROCEDURE Expr_ExprType_OnSuccessfulChange();
 BEGIN
  L_General.Assert(~SELECT(ManExpr, Expr, Expr.Expr),
                   "⨦ ᯮ  ⨦, 몠 ,|" +
                   " ⨯ 饭");
  SETEMPTY(Expr.Level, Expr.Name, Expr.Source);
  REPLACE(Expr);

  SetExprTemplates(0);
 END {Expr_ExprType_OnSuccessfulChange};

 PUBLIC PROCEDURE Expr_Source_OnSuccessfulChange();
 BEGIN
  L_General.Assert(~SELECT(ManExpr, Expr, Expr.Expr),
                   "⨦ ᯮ  ⨦, 몠 ,|" +
                   " 筨 饭");
 END {Expr_Source_OnSuccessfulChange};

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
Expr.Expr + " " + Expr.Level + " " + Expr.Name, C, ""
@
############################################################################################# ########## #####
=#####========================================================================================================
|                                    ᮪ ,  ⨦,                                     |
|############################################################################################################|
|------------------------------------------------------------------------------------------------------------|
|᪮                     |      |,       | 뤠                                 |
|-----------------------------------|----------|------------------|------------------------------------------|
}
*)

(*
$P{ Line:
Man.Man, L, ""
L_Str.ShortName(Man.Name), L, ""
ManExpr.Date, L, "D.N.y"
L_Man.GetManExprDocNum(), L, ""
L_Man.GetManExprDocGiveS(), L, ""
@
|##### #############################|##########|##################|##########################################|
}
*)

(*
$P{ Bot:
@
=(c) Polak====================================================================================================
}
*)

 PUBLIC PROCEDURE ManEx();
  CONST
   RepName = "ManEx";
 BEGIN
  ACCESS(Expr);
  L_AFM.MakeIndex("_ManE", "Man: LIKE(Man.Man)");
  SELECT(ManExpr, Expr, , Expr.Expr);
  WHILE OCCURE(ManExpr) DO
   PROMPT("ઠ " + ManExpr.Man);
   IF ~SEARCH(_ManE, _ManE, ManExpr.Man) THEN
    _ManE.Man:=ManExpr.Man;
    INSERT(_ManE);
   END;
   NEXT(ManExpr);
  END; (* WHILE OCCURE(ManExpr) ... *)

  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, Top, RepName);
  SELECT(_ManE, _ManE);
  WHILE OCCURE(_ManE) DO
   PROMPT(" " + _ManE.Man);
   SELECT(ManExpr, Man, , _ManE.Man);
   CONFORM(ManExpr, Man);
   WHILE OCCURE(ManExpr) DO
    IF ManExpr.Expr = Expr.Expr THEN
     MAKEPART(R, Line);
    END;
    NEXT(ManExpr);
   END; (* WHILE OCCURE(ManExpr) ... *)
   NEXT(_ManE);
  END; (* WHILE OCCURE(_ManE) ... *)
  MAKEPART(R, Bot);
  CLOSEREP(R);
  L_General.SurveyLastReport();
 END {ManEx};

BEGIN
 L_Events.ExecuteHandler();
END.
