(*
%Copyright   (c) Polak IT
%Author      09.04.2008 12:35:47 POLAK\kns
%Purpose
%Type        library
%Specific    SYS
%Modified    05.08.2014 16:19:50 POLAK\San
%Environment
*)

 MODULE L_ACS [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tCardType = SCAL[9];

 IMPORT L_Perco, L_Perco_S20, L_Parsec, Str, L_Str, P_Man, L_General, L_DatTim, L_Env, DBD, L_AFM, L_OwnData,
        P_OwnData, E_Utils, Math, L_SES, L_StdFrm, L_EMail, L_APEMl, FIO;

 PUBLIC CONST
  EventTypPool   = "AP 室|AP 室|室|室|⪠|OTP 室|室 ()|室 ()";

  ct_Wiegand26 = 0;
  ct_Pool      = "Wiegand26";

 PRIVATE VAR
  SrvType       : NUMERIC[0];
  pFill_ACSEvent: PROC;

 PRIVATE CONST
  RepName              = "ACSEr";
  sesfLastLoadUnloaded = "LastLoadUnloaded";
  sitType              = L_SES.sit_Global;

 PUBLIC PROCEDURE Wiegand26CardFamily(CardCode: NUMERIC[0]): NUMERIC[0];
 BEGIN
  RETURN CardCode DIV 65536;
 END {Wiegand26CardFamily};

 PUBLIC PROCEDURE Wiegand26CardNumber(CardCode: NUMERIC[0]): NUMERIC[0];
 BEGIN
  RETURN CardCode MOD 65536;
 END {Wiegand26CardNumber};

 PUBLIC PROCEDURE CardCodeAsStr(CardCode: NUMERIC[0]): STRING[];
  VAR
   l,
   Family: NUMERIC[0];
 BEGIN
  Family:=Wiegand26CardFamily(CardCode);
  l:=Math.Max(3, Str.Length(L_Str.IntStr(Family)));
  RETURN L_Str.FixedIntStr(Family, l) + " " + L_Str.FixedIntStr(Wiegand26CardNumber(CardCode), 5);
 END {CardCodeAsStr};

 PUBLIC PROCEDURE E_GetACSSitePool(): STRING[];
  VAR
   Local : PROC;
 BEGIN
  Local:=PROCREF(E_Utils.GetACSSitePool);
  IF ~!Local THEN
   RETURN Local();
  ELSE
   RETURN "";
  END;
 END {E_GetACSSitePool};

 PUBLIC PROCEDURE OutpostPool(): STRING[];
  VAR
   Amnt      : NUMERIC[0];
   Result    : STRING[];
   trgOutpost: LIKE(ACSEvent.Outpost);
 BEGIN
  P_OwnData.GetACSSystemOutposts(SYSDATE());
  SETEMPTY(Result);
  trgOutpost:=0;
  SELECT(Outpost, Outpost);
  WHILE OCCURE(Outpost) DO
   NARROWSEL(Outpost, Outpost);
   Amnt:=Outpost.Outpost - trgOutpost - 1;
   IF Amnt > 0 THEN
    Result:=Result + L_Str.Repeat(" |", Amnt);
   END;

   trgOutpost:=Outpost.Outpost;
   Result :=Result + Outpost.Name + "|";

   NEXTINEXTENDEDSEL(Outpost, Outpost);
  END; (* WHILE OCCURE(Outpost) ... *)

  RETURN Result;
 END {OutpostPool};

 PUBLIC PROCEDURE GetStartLoadUnloaded(D: DATE): DATE;
  VAR
   Result: DATE;
 BEGIN
  Result:=P_OwnData.GetACSAccountDate(D);
  Result:=Math.Max(Result, L_SES.GetFieldValue_asDATE(sitType, sesfLastLoadUnloaded, Result));
  RETURN Result;
 END {GetStartLoadUnloaded};

 PUBLIC PROCEDURE Correct_sesfLastLoadUnloaded(From, UpTo: DATE; Fill: SCAL[2]);
  VAR
   sesDate: DATE;
 BEGIN
  sesDate:=GetStartLoadUnloaded(UpTo);
  IF ~!sesDate THEN
   IF    Fill & (sesDate < UpTo) & (sesDate >= From) THEN
    L_SES.PutValueAtCurTrn(sitType, sesfLastLoadUnloaded, FLDTOSTR(Math.Min(UpTo, SYSDATE())));
   ELSIF ~Fill & (sesDate > From) THEN
    L_SES.PutValueAtCurTrn(sitType, sesfLastLoadUnloaded, FLDTOSTR(From));
   END;
  END;
 END {Correct_sesfLastLoadUnloaded};

 PRIVATE PROCEDURE CreateErrFile();
 BEGIN
  IF ~DBD.FileExist(ACSErr) THEN
   CreateFILE(ACSErr, "", "訡 ਥ   ", "");
   CreateFIELD(ACSErr, ID,    SCAL[10E3],           "0", "0", "", "稪", "");
   CreateFIELD(ACSErr, Man,   LIKE(Man.Man),          "", "", "", "뫪  ᮭ ", "");
   CreateFIELD(ACSErr, Card,  LIKE(ACSEvent.CardCode),"", "", "", "  ", "");
   CreateFIELD(ACSErr, PMan,  LIKE(Man.Man),          "", "", "", "뫪  ᮭ ", "");
   CreateFIELD(ACSErr, ErrT,  SCAL[3],                "", "", "", " 訡", "");
   CreateFIELD(ACSErr, Text,  STRING[50],             "", "", "", "ᠭ 訡", "");
   CreateFIELD(ACSErr, DTS,   LIKE(ACSEvent.DTS),     "", "", "", ", ६ ᮡ", "");

   CreateINDEX(ACSErr, ID,   AUTOINCR, "", "", ID);
   CreateINDEX(ACSErr, Err,  UNIQUE,   "", "", ErrT, Man, Card, PMan);
   SetVersionOfFILE(ACSErr, 1);
  END;
  L_AFM.AF_CreateTemp("ACSErr", "ACSErr");
 END {CreateErrFile};

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
@
##################################################################                                  ########## #####
=#####==============================================================================================================
|                                           訡 ਥ                                              |
|------------------------------------------------------------------------------------------------------------------|
|/|   |    | |ᠭ 訡                                          |, ६        |
|----|-----|--------------|----------|---------------------------------------------------------|-------------------|
}
*)

(*
$P{ Line:
ACSErr.ID,   L, ""
ACSErr.Man,  L, ""
ACSErr.PMan, L, ""
CardCodeAsStr(ACSErr.Card), L, ""
ACSErr.Text, L, ""
PackTIMESTAMP(30\12\1899, 00:00) + ACSErr.DTS  L, "D.N.y H:M:S"
@
|####|#####|##############|##########|#########################################################|###################|
}
*)

(*
$P{ Bot:
@
=(c) Polak==========================================================================================================
}
*)

 PUBLIC PROCEDURE AddErr(_Man, _PMan: LIKE(Man.Man); _Card: LIKE(ACSEvent.CardCode);
                         _ErrT: SCAL[3]; _Text: STRING[]; _DTS: LIKE(ACSEvent.DTS));
 BEGIN
  IF ~SEARCH(ACSErr, Err, _ErrT, _Man, _Card, _PMan) THEN
   SETEMPTY(ACSErr.ID);
   ACSErr.Man :=_Man;
   ACSErr.Card:=_Card;
   ACSErr.PMan:=_PMan;
   ACSErr.ErrT:=_ErrT;
   ACSErr.Text:=_Text;
   ACSErr.DTS :=_DTS;
   INSERT(ACSErr);
  END;
 END {AddErr};

 PRIVATE PROCEDURE GetCardCodeOwner(D: DATE): LIKE(Man.Man);
 BEGIN
  RETURN P_Man.GetSpeedSearchResults(D, P_Man.P_Pass, L_Str.IntStr(ACSEvent.CardCode))
 END {GetCardCodeOwner};

 PUBLIC PROCEDURE AddACSEvent(sqlDTS    : LIKE(ACSEvent.DTS);
                              sqlCard   : LIKE(ACSEvent.CardCode);
                              sqlOutpost: NUMERIC[0];
                              sqlMan    : STRING[];
                              sqlEvent  : LIKE(ACSEvent.Event));
  VAR
   trgMan,
   curMan,
   prvMan    : LIKE(Man.Man);
   CardType  : tCardType;
   CardCode  : NUMERIC[0];
   trgOutpost: LIKE(ACSEvent.Outpost);
 BEGIN
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   trgMan:=sqlMan;
  EXCEPT
  ELSE
   SETEMPTY(trgMan);
  END;
  IF !trgMan | (trgMan = 0) THEN
   trgMan:=L_Env.fictMan;
  END;
  PROMPT("㧪:  " + FLDTOSTR(trgMan, "", 5) + ", " + FLDTOSTR(sqlCard) + ", " + L_DatTim.DTStampStr(sqlDTS));
  IF    !sqlOutpost THEN
   trgOutpost:=0;
  ELSIF SEARCH(Outpost, o_Enter, sqlOutpost) | SEARCH(Outpost, o_Exit,  sqlOutpost) THEN
   trgOutpost:=Outpost.Outpost;
  ELSE
   L_General.Error("   :" + L_Str.IntStr(sqlOutpost));
  END;
  IF ~SEARCH(ACSEvent, DTS, sqlDTS, trgOutpost, sqlCard, sqlEvent) THEN
   SETEMPTY(ACSEvent.ACSEvent);
   ACSEvent.Man     :=trgMan;
   ACSEvent.CardCode:=sqlCard;
   ACSEvent.Outpost :=trgOutpost;
   ACSEvent.Event   :=sqlEvent;
   ACSEvent.DTS     :=sqlDTS;
   UnPkVARIANTDATE(sqlDTS, ACSEvent.Date, ACSEvent.Time, ACSEvent.Second);

   curMan:=GetCardCodeOwner(ACSEvent.Date);
   prvMan:=GetCardCodeOwner(ACSEvent.Date-1);
   IF    SEARCH(Man, Man, curMan) | SEARCH(Man, Man, prvMan) THEN
    IF (ACSEvent.Man # curMan) & (ACSEvent.Man # prvMan) THEN
     AddErr(curMan, ACSEvent.Man, ACSEvent.CardCode, 0, " ᮢ    ⮏     ", ACSEvent.DTS);
     ACSEvent.Man:=curMan;
    END;
   ELSIF L_Env.RealMan(ACSEvent.Man) THEN
    IF ~SEARCH(Man, Man, ACSEvent.Man) THEN
     ACSEvent.Man :=L_Env.fictMan;
    ELSE
     P_Man.GetPassAttributes(Man.Man, ACSEvent.Date, CardType, CardCode);
     IF ~!CardCode THEN
      AddErr(Man.Man, ACSEvent.Man, ACSEvent.CardCode, 1, "   ⮏ 뤠 㣠 :" + Str.Condense(FLDTOSTR(CardCode)),
             ACSEvent.DTS);
     ELSE
      AddErr(Man.Man, ACSEvent.Man, ACSEvent.CardCode, 2, "   ⮏  㪠  ", ACSEvent.DTS);
     END;
    END;
   END;
   INSERT(ACSEvent);
  END;
 END {AddACSEvent};

 PRIVATE PROCEDURE ACSInitSrvType(From, UpTo: DATE; ErrEmpty: SCAL[2]);
 BEGIN
  SrvType:=P_OwnData.GetACSSystemServerType(From);
  L_General.Assert(~ErrEmpty | (SrvType # P_OwnData.c_acsp_SrvType_Empty), "  ⠭");
  L_General.Assert(OwnDPro.EndDate >= UpTo,
                   "ਮ   ப ⢨ ࠬ஢ ᮯ殮  ");
 END {ACSInitSrvType};

 PUBLIC PROCEDURE CM_EmptySKUD(CardCode: NUMERIC[0]); (*誠   権  ⠬   ⨯ " ⠭"*)
 BEGIN
  L_StdFrm.Inform("  ⠭. ଠ ࠦ ⮫쪮  ⮏");
 END {CM_EmptySKUD};

 PRIVATE PROCEDURE ACSGetProcByName(PName: STRING[]): PROC;
  VAR
   Module: STRING[];
   Result: PROC;
 BEGIN
  SETEMPTY(Module);
  IF    SrvType = P_OwnData.c_acsp_SrvType_Perco THEN
   Module:="L_Perco.";
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Parsec THEN
   Module:="L_Parsec.";
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Perco_S20 THEN
   Module:="L_Perco_S20.";
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Empty THEN
   RETURN PROCREF(CM_EmptySKUD);
  ELSE
   L_General.Error(" ⨯ : " + FLDTOSTR(SrvType));
  END;
  Result:=PROCREF(^(Module + PName));
  L_General.CallUs(~!Result);
  RETURN Result;
 END {ACSGetProcByName};

 PRIVATE PROCEDURE E_ReportContactTo(): L_EMail.tContact;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Utils.ACS_ReportContactTo);
  IF ~!Local THEN
   RETURN Local();
  ELSE
   RETURN L_APEMl.CurrentSenderContact();
  END;
 END {E_ReportContactTo};

 PRIVATE PROCEDURE E_ReportContactFrom(): L_EMail.tContact;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Utils.ACS_ReportContactFrom);
  IF ~!Local THEN
   RETURN Local();
  ELSE
   RETURN L_APEMl.CurrentSenderContact();
  END;
 END {E_ReportContactFrom};

 PRIVATE PROCEDURE facse_MakeErrRep();
 BEGIN
  MAKEPART(R, Top, RepName);
  SELECT(ACSErr, ID);
  WHILE OCCURE(ACSErr) DO
   MAKEPART(R, Line);
   NEXT(ACSErr);
  END; (* WHILE OCCURE(ACSErr) ... *)
  MAKEPART(R, Bot);
 END {facse_MakeErrRep};

 PUBLIC PROCEDURE Fill_ACSEvent(From, UpTo : DATE; _Man: LIKE(Man.Man));
  CONST
   RepWidth = 112;
  VAR
   CardType: tCardType;
   CardCode: NUMERIC[0];
   _From,
   _UpTo   : DATE;
   ErrRepFN: STRING[];
   doEMail : SCAL[2];
 BEGIN
  ACSInitSrvType(From, UpTo, 1);
  pFill_ACSEvent:=ACSGetProcByName("Fill_ACSEvent");
  CreateErrFile();

  SETEMPTY(CardType, CardCode);
  (*᭠砫 饬  *)
  pFill_ACSEvent(From, UpTo, _Man, CardCode);
  (*⮬  ய᪠, ᫨ 롮ઠ  *)
  IF ~!_Man THEN
   SELECT(ManPro, Man, ,_Man, P_Man.P_Pass);
   SRCINSEL(ManPro, LE, ,UpTo);
   WHILE OCCURE(ManPro) & (L_DatTim.IsActiveInIntvl(ManPro.Date, ManPro.DocExpD, From, UpTo)) DO
    _From:=Math.Max(From, ManPro.Date);
    _UpTo:=Math.Min(UpTo, ManPro.EndDate);
    SAVE(ManPro);
    P_Man.GetPassAttributes(Man.Man, _From, CardType, CardCode);
    pFill_ACSEvent(_From, _UpTo, _Man, CardCode);
    RESTORE(ManPro);
    PREV(ManPro);
   END; (* WHILE OCCURE(ManPro) ... *)
  END;
  IF !_Man THEN
   Correct_sesfLastLoadUnloaded(From, UpTo, 1);
  END;
  doEMail:=L_General.BatchExecution_IsRunning();
  ErrRepFN:=FIO.FirstPath(L_General.NameConnection(RepName), "REP");
  OPENREP(R, ErrRepFN, Create);
  facse_MakeErrRep();
  CLOSEREP(R);
  IF doEMail & SELECT(ACSErr, ID) THEN
   ErrRepFN:=FIO.MakePath(RepName, "REP", FIO.AsbSpecialFolders("ASBTemp"));
   FIO.Open(R, ErrRepFN, , Create);
   FIO.SetCP(R, FIO.CP_WindowsRussian);
   facse_MakeErrRep();
   FIO.Close(R);
   TRY
    L_APEMl.EasySend(E_ReportContactTo(), E_ReportContactFrom(), "㧪 室   ⮏",
                     ErrRepFN, " 訡 ਥ   . . ⮪.");
   EXCEPT
    ON Exc:Exc_Root DO
     RAISE(, " 㤠 ࠢ  EMail 訡 㧪 室." + "||" + Exc.MsgOriginalText);
   END;
  END;
 END {Fill_ACSEvent};

 (*㧪  㦥*)
 PUBLIC PROCEDURE LoadUnloaded();
  VAR
   From,
   UpTo: DATE;
 BEGIN
  UpTo:=SYSDATE();
  From:=GetStartLoadUnloaded(UpTo);
  L_General.Assert(~!From, "⢮ 9028:  ।  砫  室");
  Fill_ACSEvent(From, UpTo, "");
 END {LoadUnloaded};

 PRIVATE PROCEDURE CardOperation(Operation: STRING[]; CardCode: NUMERIC[0]);
  VAR
   P: PROC;
 BEGIN
  ACSInitSrvType(SYSDATE(), SYSDATE(), 0);
  P:=ACSGetProcByName(Operation);
  P(CardCode);
 END {CardOperation};

 PUBLIC PROCEDURE CM_Withdraw(CardCode: NUMERIC[0]); (*⨥ *)
 BEGIN
  CardOperation("CM_Withdraw", CardCode);
 END {CM_Withdraw};

 PUBLIC PROCEDURE CM_PutStaff(CardCode: NUMERIC[0]); (*뤠/ *)
 BEGIN
  CardOperation("CM_PutStaff", CardCode);
 END {CM_PutStaff};

 PUBLIC PROCEDURE CM_ToStopList(CardCode: NUMERIC[0]);
 BEGIN
  CardOperation("CM_ToStopList", CardCode);
 END {CM_ToStopList};

 PUBLIC PROCEDURE CM_ThrowFromStopList(CardCode: NUMERIC[0]);
 BEGIN
  CardOperation("CM_ThrowFromStopList", CardCode);
 END {CM_ThrowFromStopList};

 PUBLIC PROCEDURE CM_Renovate(CardCode: NUMERIC[0]);
 BEGIN
  CardOperation("CM_Renovate", CardCode);
 END {CM_Renovate};

 PUBLIC PROCEDURE PutAllMan();
 BEGIN
  ACSInitSrvType(SYSDATE(), SYSDATE(), 0);
  IF    SrvType = P_OwnData.c_acsp_SrvType_Perco THEN
   L_Perco.PutAllMan();
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Parsec THEN
   L_General.Error("  Parsec ᮢ 㧪 ");
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Perco_S20 THEN
   L_Perco_S20.PutAllMan();
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Empty THEN
   L_General.Error("  ⠭");
  ELSE
   L_General.Error(" ⨯ : " + FLDTOSTR(SrvType));
  END
 END {PutAllMan};

 PUBLIC PROCEDURE perco_ForgetGotCardCode();
 BEGIN
  SLEEP(0.4)
 END {perco_ForgetGotCardCode};

 PUBLIC PROCEDURE perco_GetControllersIdList();
 BEGIN
  ACSInitSrvType(SYSDATE(), SYSDATE(), 0);
  IF    SrvType = P_OwnData.c_acsp_SrvType_Perco THEN
   L_General.Error("楤 祭  祪 室  ஥");
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Parsec THEN
   L_General.Error("楤 祭  祪 室  ஥");
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Perco_S20 THEN
   L_Perco_S20.GetControllersIdList();
  ELSIF SrvType = P_OwnData.c_acsp_SrvType_Empty THEN
   L_General.Error("  ⠭");
  ELSE
   L_General.Error(" ⨯ : " + FLDTOSTR(SrvType));
  END
 END {perco_GetControllersIdList};

END.
