(*
%Copyright   (c) Polak Computer System
%Author      MOG
%Purpose     প  
%Type        library
%Specific    AP
%Modified    03.03.2014 16:54:38 LDA
%Environment
*)

 MODULE L_Fuel [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tCarOilCoef  = NUMERIC[3][MAXOF(MarkFuel.InMix)+1];
  tfal         = SCAL[5];
  tGasStanKind = SCAL[3];
  Arr          = LIKE(Staff.TSN)[20];
  tMarkFuelReleaseType = SCAL[3];

 PUBLIC CONST
  im_SimplyFuel      = 0;
  im_MainFuel        = 1;
  im_MotorOil        = 2;
  im_TransmissionOil = 3;
  im_HydroOil        = 4;
  im_PlasticGreasing = 5;
  im_Tosol           = 6;
  im_Other           = 7;
  im_Heater          = 8;
  im_AddMainFuel     = 9;

  im_Pool = "||୮|࠭ᬨᨮ|ࠢ᪮|᪮|'᮫'|祥|⮯⥫|'";

  fal_RWOP           = 0;  (* tfal *)
  fal_Dispatcher     = 1;
  fal_RefuelOperator = 2;
  fal_RfOpOffline    = 3;
  fal_None           = 4;

  gsk_None          = 0;
  gsk_ElectroServis = 1;
  gsk_NeftoServer   = 2;

  mfrt_TO      = 0;
  mfrt_Add     = 1;
  mfrt_Unknown = 2;

 PRIVATE CONST
  Fuel_LongPool  = "쭮 ⮯| -76| -80| -92| -95";
  Fuel_ShortPool = "|76|80|92|95";

 IMPORT L_General, L_DatTim, L_Car, L_Str, L_EC, L_XDoc, L_MDoc, L_Prop, P_Math, E_Way, L_Env, L_Way, P_Car,
        E_Stock, Str, L_MathR, P_Mark;

 PUBLIC PROCEDURE GetFuelName(trgMath: LIKE(Math.Math)): STRING[];
  VAR
   idx           : NUMERIC[0];
   Result,
   ShortPoolItem,
   trgMathN      : STRING[];
 BEGIN
  L_General.CallUs(SEARCH(Math, Math, trgMath));
  trgMathN:=Str.Cap(Math.Name);
  idx:=0;
  SETEMPTY(Result);
  WHILE ~!L_Str.PoolItem(Fuel_ShortPool, idx) & !Result DO
   ShortPoolItem:=L_Str.PoolItem(Fuel_ShortPool, idx);
   IF Str.Pos(trgMathN, ShortPoolItem) # Str.Length(trgMathN) THEN
    Result:=L_Str.PoolItem(Fuel_LongPool, idx);
   END;
   INC(idx);
  END;
  IF !Result THEN
   Result:=Math.Name;
  END;
  RETURN Result;
 END {GetFuelName};

 PUBLIC PROCEDURE AccessFlRec();
 BEGIN
  L_General.Assert(ACCESS(FlRec), " ⠭ ࠢ筠 ");
 END {AccessFlRec};

 PUBLIC PROCEDURE _FuelFact(): LIKE(RWayF.RemExit);
 BEGIN
  RETURN RWayF.RemExit + RWayF.FuelTake - RWayF.FuelRet - RWayF.RemEntr;
 END {_FuelFact};

 PUBLIC PROCEDURE IsAnyFuel(trgInMix: LIKE(MarkFuel.InMix)): SCAL[2];
 BEGIN
  RETURN (trgInMix = im_MainFuel) | (trgInMix = im_AddMainFuel) | (trgInMix = im_SimplyFuel);
 END {IsAnyFuel};

 PUBLIC PROCEDURE IsAnyOil(trgInMix: LIKE(MarkFuel.InMix)): SCAL[2];
 BEGIN
  RETURN (trgInMix = im_MotorOil) |
         (trgInMix = im_TransmissionOil) |
         (trgInMix = im_HydroOil) |
         (trgInMix = im_PlasticGreasing);
 END {IsAnyOil};

 PUBLIC PROCEDURE GetInMixAsStr(trgInMix: LIKE(MarkFuel.InMix)): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(im_Pool, trgInMix);
 END {GetInMixAsStr};

 PUBLIC PROCEDURE AcsMarkFuel(D: DATE; trgMark: LIKE(Mark.Mark); trgMath: LIKE(Math.Math)): SCAL[2];
 BEGIN
  SELECT(MarkFuel, Mark, , trgMark, trgMath);
  WHILE OCCURE(MarkFuel) DO
   IF L_DatTim.IsActive(D, MarkFuel.Begin, MarkFuel.Till) THEN
    RETURN 1;
   END;
   NEXT(MarkFuel);
  END; (* WHILE OCCURE(MarkFuel) ... *)
  RETURN 0;
 END {AcsMarkFuel};

 PUBLIC PROCEDURE IsFuelInMixOnDate(OnDate: DATE; trgMath: LIKE(Math.Math)): SCAL[2];
 BEGIN
  L_Car.AccessMark(Car.BoxNbr, OnDate);
  L_General.Assert(AcsMarkFuel(OnDate, Mark.Mark, trgMath),
                   " ଠ樨  ⥯த " + trgMath + "   " + Mark.Mark + "  " + L_DatTim.DateStr(OnDate));
  RETURN IsAnyFuel(MarkFuel.InMix);
 END {IsFuelInMixOnDate};

 PUBLIC PROCEDURE GetMainFuelSoft(D: DATE; trgMark: LIKE(Mark.Mark)): LIKE(Math.Math);
  VAR
   Result: LIKE(Math.Math);
 BEGIN
  SELECT(MarkFuel, Mark, , trgMark);
  WHILE OCCURE(MarkFuel) DO
   IF (MarkFuel.InMix = im_MainFuel) & L_DatTim.IsActive(D, MarkFuel.Begin, MarkFuel.Till) THEN
    RETURN MarkFuel.Math;
   END;
   NEXT(MarkFuel);
  END; (* WHILE OCCURE(MarkFuel) ... *)
  RETURN EMPTYOF(Math.Math);
 END {GetMainFuelSoft};

 PUBLIC PROCEDURE GetMainFuel(D: DATE; trgMark: LIKE(Mark.Mark)): LIKE(Math.Math);
  VAR
   Result: LIKE(MarkFuel.Math);
 BEGIN
  Result:=GetMainFuelSoft(D, trgMark);
  L_General.Assert(~!Result, " 㪠  ⮯  ન " + trgMark + "  " + L_DatTim.DateStr(D));
  RETURN Result;
 END {GetMainFuel};

 PRIVATE PROCEDURE GetCarAgeOilCoef(D: DATE; y: NUMERIC[0]): NUMERIC[1];
 BEGIN
  IF D >= 01\12\2003 THEN
   IF    y < 5 THEN RETURN 1.0;
   ELSE             RETURN 1.2;
   END;
  ELSE
   IF    y < 3 THEN RETURN 0.5;
   ELSIF y < 8 THEN RETURN 1.0;
   ELSE             RETURN 1.2;
   END;
  END;
 END {GetCarAgeOilCoef};

 PUBLIC PROCEDURE GetCarOilCoef(D: DATE; VAR OilCoef: tCarOilCoef);
  VAR
   i: NUMERIC[0];
   AgeCoef: NUMERIC[1];
 BEGIN
  L_General.Assert(OCCURE(Car), " ⠭ ⥪饥 ");
  AgeCoef:=GetCarAgeOilCoef(D, L_DatTim.CompleteYearsInIntvl(L_Car.GetCarProdDate(Car.BoxNbr, D), D - 1));
  P_Car.CarOilCor(Car.BoxNbr, D, OilCoef);
  i:=0;
  WHILE i<RANGEOF(OilCoef) DO
   OilCoef[i]:=OilCoef[i]*AgeCoef;
   INC(i);
  END;
 END {GetCarOilCoef};

 PUBLIC PROCEDURE IsInMix(D: DATE; trgMark: LIKE(MarkFuel.Mark); trgMath: LIKE(MarkFuel.Math); trgInMix: LIKE(MarkFuel.InMix)): SCAL[2];
 BEGIN
  RETURN AcsMarkFuel(D, trgMark, trgMath) & (MarkFuel.InMix = trgInMix);
 END {IsInMix};

 PRIVATE PROCEDURE IndexInArrTSN(ArrTSN: Arr; _TSN: LIKE(Staff.TSN)): NUMERIC[0];
  VAR
   k: NUMERIC[0];
 BEGIN
  SETZERO(k);
  WHILE ~!ArrTSN[k] DO
   IF ArrTSN[k] = _TSN THEN
    RETURN k;
   END;
   INC(k);
  END; (* WHILE ~!ArrTSN[k] ...*)
  RETURN k;
 END {IndexInArrTSN};

 PUBLIC PROCEDURE GetMarkFuelReleaseType(D: DATE; Fact: NUMERIC[4]): tMarkFuelReleaseType;
  VAR
   Volume : NUMERIC[1];
 BEGIN
  IF    MarkFuel.InMix = im_Tosol THEN
   Volume:=Mark.TosolVol;
  ELSIF MarkFuel.InMix = im_MotorOil THEN
   Volume:=Mark.OilVolum[P_Mark.ovi_Motor];
  ELSIF MarkFuel.InMix = im_TransmissionOil THEN
   Volume:=Mark.OilVolum[P_Mark.ovi_Transmission];
  ELSIF MarkFuel.InMix = im_HydroOil THEN
   Volume:=Mark.OilVolum[P_Mark.ovi_Hydro];
  ELSIF MarkFuel.InMix = im_PlasticGreasing THEN
   Volume:=Mark.OilVolum[P_Mark.ovi_PlasticGreasing];
  ELSE
   L_General.CallUs(0);
  END;
  IF ~!Volume THEN
   IF Fact < Volume THEN
    RETURN mfrt_Add;
   ELSE
    RETURN mfrt_TO;
   END;
  ELSE
   RETURN mfrt_Unknown;
  END;
 END {GetMarkFuelReleaseType};

 PUBLIC PROCEDURE intGetFactByOilMath(   _BoxNbr: LIKE(RWayS.BoxNbr);
                                         _Math  : LIKE(MarkFuel.Math);
                                         _From,
                                         _UpTo  : DATE;
                                         mfrt   : tMarkFuelReleaseType;
                                     VAR ArrTSN : Arr): NUMERIC[4][20];
  VAR
   _Fact   : NUMERIC[4][20];
   i       : NUMERIC[0];
   byPeriod: SCAL[2];
 BEGIN
  IF _From = _UpTo THEN
   byPeriod:=0;
  ELSE
   byPeriod:=1;
  END;
  SETEMPTY(ArrTSN);
  SETZERO(_Fact, i);
  IF E_OutputFuelThroughMDoc(_From) THEN
   SAVE(MDoc);
   SAVE(MDocList);
   SETSEL(MDoc, BoxNbr, _BoxNbr);
   SRCINSEL(MDoc, LE, , _UpTo);
   WHILE OCCURE(MDoc) & (MDoc.Date >= _From) DO
    IF (MDoc.Status = L_MDoc.st_Passed) & (MDoc.MDocKind # L_MDoc.md_MSP) THEN
     SELECT(MDocList, MDLMath, , MDoc.MDocKind, MDoc.MDoc, _Math);
     WHILE OCCURE(MDocList) DO
      IF !mfrt | (GetMarkFuelReleaseType(MDoc.Date, MDocList.Qnt) = mfrt) THEN
       IF ~byPeriod THEN
        i        :=IndexInArrTSN(ArrTSN, MDoc.TSN);
        ArrTSN[i]:=MDoc.TSN;
       END;
       INC(_Fact[i], MDocList.Qnt);
      END;
      NEXT(MDocList);
     END; (* WHILE OCCURE(MDocList) ... *)
    END;
    PREV(MDoc);
   END; (* WHILE OCCURE(MDoc) ... *)
   RESTORE(MDoc);
   RESTORE(MDocList);
  ELSE (*  ३  MDoc,   - 㤠*)
   SAVE(RWayS);
   SAVE(RWayF);
   SETSEL(RWayS, BoxNbr, _BoxNbr);
   SRCINSEL(RWayS, LE, , _UpTo);
   WHILE OCCURE(RWayS) & (RWayS.Date >= _From) DO
    IF ~byPeriod THEN
     i        :=IndexInArrTSN(ArrTSN, RWayS.TSN);
     ArrTSN[i]:=RWayS.TSN;
    END;
    IF SEARCH(RWayF, RWayS, RWayS.RWayS, _Math) THEN
     IF !mfrt | (GetMarkFuelReleaseType(RWayS.Date, _FuelFact()) = mfrt) THEN
      INC(_Fact[i], _FuelFact());
     END;
    END;
    PREV(RWayS);
   END; (* WHILE OCCURE(RWayS) ... *)
   RESTORE(RWayF);
   RESTORE(RWayS);
  END;
  RETURN _Fact;
 END {intGetFactByOilMath};

 PUBLIC PROCEDURE __GetFactByOilMath(    _BoxNbr: LIKE(RWayS.BoxNbr);
                                         _Math  : LIKE(MarkFuel.Math);
                                         _From,
                                         _UpTo  : DATE;
                                     VAR ArrTSN : Arr): NUMERIC[4][20];
 BEGIN
  RETURN intGetFactByOilMath(_BoxNbr, _Math, _From, _UpTo, EMPTYOF(tMarkFuelReleaseType), ArrTSN);
 END {__GetFactByOilMath};

 PUBLIC PROCEDURE GetSRTFactByOilMath(_BoxNbr: LIKE(RWayS.BoxNbr);
                                      _Math  : LIKE(MarkFuel.Math);
                                      _From,
                                      _UpTo  : DATE;
                                      mfrt   : tMarkFuelReleaseType): NUMERIC[4];
  VAR
   ArrTSN: Arr;
   _Fact : NUMERIC[4][20];
   i     : NUMERIC[0];
   _Value: NUMERIC[4];
 BEGIN
  SETEMPTY(ArrTSN);
  _Fact:=intGetFactByOilMath(_BoxNbr, _Math, _From, _UpTo, mfrt, ArrTSN);
  SETZERO(_Value, i);
  WHILE i <= HIGH(_Fact) DO
   INC(_Value, _Fact[i]);
   INC(i);
  END; (* WHILE i <= HIGH(_Fact) ... *)
  RETURN _Value;
 END {GetSRTFactByOilMath};

 PUBLIC PROCEDURE GetFactByOilMath(_BoxNbr: LIKE(RWayS.BoxNbr);
                                   _Math  : LIKE(MarkFuel.Math);
                                   _From,
                                   _UpTo  : DATE): NUMERIC[4];
 BEGIN
  RETURN GetSRTFactByOilMath(_BoxNbr, _Math, _From, _UpTo, EMPTYOF(tMarkFuelReleaseType));
 END {GetFactByOilMath};

 PUBLIC PROCEDURE FlAcntLev(D: DATE): tfal;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(L_Env.FlAcntLev);
  IF ~!Local THEN
   RETURN Local(D);
  ELSE
   RETURN fal_RWOP;
  END;
 END {FlAcntLev};

 PUBLIC PROCEDURE TanksIsStock(D: DATE): SCAL[2];
 BEGIN
  RETURN (FlAcntLev(D) = fal_RefuelOperator) |
         (FlAcntLev(D) = fal_RfOpOffline);
 END {TanksIsStock};

 PUBLIC PROCEDURE TanksIsStockInMonth(P: DATE): SCAL[2];
 BEGIN
  RETURN TanksIsStock(L_DatTim.MonthEnd(P));
 END {TanksIsStockInMonth};

 PUBLIC PROCEDURE E_DoJDocByRWayS(P: DATE): SCAL[2];
  VAR
   Local : PROC;
   Result: SCAL[2];
 BEGIN
  Result:=0;
  IF TanksIsStockInMonth(P) THEN
   Local:=PROCREF(E_Stock.DoJDocByRWayS);
   IF ~!Local THEN
    Result:=Local(P);
   END;
  END;
  RETURN Result;
 END {E_DoJDocByRWayS};

 PUBLIC PROCEDURE E_OutputFuelThroughMDoc(D: DATE): SCAL[2];
 BEGIN
  IF TanksIsStock(D) THEN
   RETURN L_EC.ExecE_Func_Scal2(D, PROCREF(E_Way.OutputFuelThroughMDoc), 1);
  ELSE
   RETURN L_EC.ExecE_Func_Scal2(D, PROCREF(E_Way.OutputFuelThroughMDoc), 0);
  END;
 END {E_OutputFuelThroughMDoc};

 PUBLIC PROCEDURE GetMathIdx(    Maths: LIKE(Math.Math)[];
                                 Math : LIKE(Math.Math);
                             VAR Idx  : NUMERIC[0]): SCAL[2];
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=0;
  WHILE i <= HIGH(Maths) DO
   IF Maths[i] = Math THEN
    Idx:=i;
    RETURN 1;
   END;
   INC(i);
  END;
  RETURN 0;
 END {GetMathIdx};

 PUBLIC PROCEDURE GetOrAddMathIdx(VAR Maths : LIKE(Math.Math)[];
                                      Math  : LIKE(Math.Math);
                                  VAR Idx   : NUMERIC[0];
                                  VAR Amount: NUMERIC[0]);
 BEGIN
  IF ~GetMathIdx(Maths, Math, Idx) THEN
   Idx:=Amount;
   INC(Amount);
   Maths[Idx]:=Math;
  END;
 END {GetOrAddMathIdx};

 PRIVATE PROCEDURE DecodeFlRecId(_FlRecId  : STRING[];
                                 VAR _FlRec: LIKE(FlRec.FlRec));
  VAR
   _XDocType: L_XDoc.tXDocType;
   _XDocKind: L_XDoc.tXDocKind;
 BEGIN
  L_XDoc.DecodeXDocId(_FlRecId, _XDocType, _XDocKind, _FlRec);
  L_General.CallUs(_XDocType = L_XDoc.xdt_FlRec);
 END {DecodeFlRecId};

 PUBLIC PROCEDURE PriceOfOver(trgMath: LIKE(Math.Math); D: DATE): NUMERIC[2];
 BEGIN
  RETURN P_Math.GetPriceOfOver(trgMath, L_Way.GetRWaySMyFirm(), D);
 END {PriceOfOver};

 PUBLIC PROCEDURE E_TankIsFullOnReturn(D: DATE): SCAL[2];
 BEGIN
  RETURN L_EC.ExecE_Func_Scal2(D, PROCREF(E_Stock.TankIsFullOnReturn), 0);
 END {E_TankIsFullOnReturn};

 PUBLIC PROCEDURE E_LoadFullTank(D: DATE): SCAL[2];
 BEGIN
  RETURN E_TankIsFullOnReturn(D) | L_EC.ExecE_Func_Scal2(D, PROCREF(E_Stock.LoadFullTank), 0);
 END {E_LoadFullTank};

 PUBLIC PROCEDURE CarMustHaveNormalChain(_BoxNbr: LIKE(Car.BoxNbr); D: DATE): SCAL[2];
 BEGIN
  RETURN ~((FlAcntLev(D) = fal_RWOP) | ~L_Way.CarBegRWSLev(_BoxNbr, D));
 END {CarMustHaveNormalChain};

 PUBLIC PROCEDURE NeftoStanExist(): SCAL[2];
 BEGIN
  RETURN ~!L_EC.ExecE_Func_String(SYSDATE(), PROCREF(E_Stock.NeftoStan_Path), "");
 END {NeftoStanExist};

 PUBLIC PROCEDURE CheckMathIsMarkFuel(    D        : DATE;
                                          trgMath  : LIKE(Math.Math);
                                          trgBoxNbr: LIKE(Mark.Mark);
                                      VAR Msg      : STRING[]);
 BEGIN
  Msg:="";
  L_Car.AccessMark(trgBoxNbr, D);
  L_General.CallUs(SEARCH(Math, Math, trgMath));
  IF (Math.MathType = L_MathR.mtyp_Fuels) |
     (Math.MathType = L_MathR.mtyp_Oils) THEN
   IF ~AcsMarkFuel(D, Mark.Mark, trgMath) THEN
    CONFORM(Math, MathType);
    Msg:="  " + Math.Math + " (  " + MathType.MathType + " " + MathType.Name +
         ")   " + trgBoxNbr + " ⨬." +
         "|(. 祭 ਬ塞 ⥯த⮢  ᠭ ન  " + Mark.Mark + ")";
   END;
  END;
 END {CheckMathIsMarkFuel};

END.
