(*
%Copyright   (c) Polak IT
%Author      16.09.2009 18:33:50 POLAK\Gina
%Purpose
%Type        library
%Specific    AP
%Modified    11.08.2014 11:28:36 POLAK\San
%Environment
*)

 MODULE L_RWayS [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tTimeCoeff = NUMERIC[3];

 PUBLIC CONST
  Qlt_Good   = 000;
  TripAmount = 100;

 PUBLIC TYPE
  tTripAmount = SCAL[TripAmount];

 PRIVATE CONST
  RWJPNbr    = 100;

 IMPORT L_General, L_Way, L_EC, E_Way, L_Sheet, L_DatTim, L_two, L_Env, L_XDoc, Math, L_Str;

 PRIVATE VAR
  crv_RWayS              : LIKE(RWayS.RWayS);
  crv_GetTimesFromWayLMen: SCAL[2];

  RWJPStore              : LIKE(RWayJP.Value)[RWJPNbr*TripAmount];

 PRIVATE PROCEDURE SetCurrentRWayS();
 BEGIN
  IF crv_RWayS # RWayS.RWayS THEN
   crv_RWayS:=RWayS.RWayS;
   SETEMPTY(crv_GetTimesFromWayLMen);
  END;
 END {SetCurrentRWayS};

 PUBLIC PROCEDURE IsRWayJForCurWayLMen(): SCAL[2];
 BEGIN
  RETURN L_Way._TimeInt(L_Way.FullBegTime(RWayJ.Day, RWayJ.StartT),
                        L_Way.FullEndTime(RWayJ.Day, RWayJ.StartT, RWayJ.StopT),
                        L_Way.FullBegTime(WayLMen.Day, WayLMen.StartT),
                        L_Way.FullEndTime(WayLMen.Day, WayLMen.StartT, WayLMen.StopT)) > 0;
 END {IsRWayJForCurWayLMen};

 PUBLIC PROCEDURE IsSEJobsForCurWayLMen(): SCAL[2];
 BEGIN
  RETURN L_Way._TimeInt(L_Way.FullBegTime(SEJobs.Day, SEJobs.StartT),
                        L_Way.FullEndTime(SEJobs.Day, SEJobs.StartT, SEJobs.StopT),
                        L_Way.FullBegTime(WayLMen.Day, WayLMen.StartT),
                        L_Way.FullEndTime(WayLMen.Day, WayLMen.StartT, WayLMen.StopT)) > 0;
 END {IsSEJobsForCurWayLMen};

 PUBLIC PROCEDURE IsRWayJWithWayLMen_Who(_Who: L_Way.tWLMWho): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  Result:=0;
  IF ~L_Way.E_IsGuardIdleExit(RWayS.Date, RWayS.Exit) THEN
   SAVE(WayLMen);
   SELECT(WayLMen, RWayS, , RWayS.RWayS, _Who);
   WHILE OCCURE(WayLMen) & ~Result DO
    Result:=IsRWayJForCurWayLMen();
    NEXT(WayLMen);
   END;
   RESTORE(WayLMen);
  END;
  RETURN Result;
 END {IsRWayJWithWayLMen_Who};

 PUBLIC PROCEDURE WayLMenWithOwnWJMode(trgWho: L_Way.tRWSWho): SCAL[2];
 BEGIN
  RETURN (trgWho # L_Way.rwswho_Driver) & (trgWho # L_Way.rwswho_Chopper) & ~!WayLMen.asWJMode;
 END {WayLMenWithOwnWJMode};

 PUBLIC PROCEDURE GetasWJMode(trgWho: L_Way.tRWSWho): LIKE(WJMode.WJCode);
 BEGIN
  IF WayLMenWithOwnWJMode(trgWho) THEN
   RETURN WayLMen.asWJMode;
  ELSE
   RETURN SEJobs.WJCode;
  END;
 END {GetasWJMode};

 PUBLIC PROCEDURE E_GetTimesFromWayLMen(): SCAL[2];
  VAR
   default: SCAL[2];
   Proc   : PROC;
 BEGIN
  SetCurrentRWayS();
  IF !crv_GetTimesFromWayLMen THEN
   default:=L_Way.E_IsGuardIdleExit(RWayS.Date, RWayS.Exit);
   Proc:=PROCREF(E_Way.GetTimesFromWayLMen);
   IF ~!Proc THEN
    default:=Proc(default);
   END;
   crv_GetTimesFromWayLMen:=default;
  ELSE
   default:=crv_GetTimesFromWayLMen;
  END;
  RETURN default;
 END {E_GetTimesFromWayLMen};
 
 PUBLIC PROCEDURE E_RWAYS_CoeffForCurTimes(D: DATE; CurWho: L_Way.tRWSWho): tTimeCoeff;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Way.RWAYS_CoeffForCurTimes);
  IF ~!Local THEN
   RETURN Local(D, CurWho);
  ELSE
   RETURN 1;
  END;
 END {E_RWAYS_CoeffForCurTimes};

 PUBLIC PROCEDURE E_RWAYS_CoeffForAddTimes(D: DATE; CurWho: L_Way.tRWSWho): tTimeCoeff;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Way.RWAYS_CoeffForAddTimes);
  IF ~!Local THEN
   RETURN Local(D, CurWho);
  ELSE
   RETURN 0;
  END;
 END {E_RWAYS_CoeffForAddTimes};

 PUBLIC PROCEDURE PositionOnRWayS(_RWayS: LIKE(RWayS.RWayS));
 BEGIN
  L_General.CallUs(SEARCH(RWayS, RWayS, _RWayS));
 END {PositionOnRWayS};

 PRIVATE PROCEDURE GetWorkTimeFromRWayS(_Who: L_Way.tRWSWho; byWLM: SCAL[2]): NUMERIC[3];
  VAR
   _Time,
   _TripTime: NUMERIC[3];
 BEGIN
  IF byWLM & E_GetTimesFromWayLMen() THEN
   _Time:=WayLMen.StopT-WayLMen.StartT;
  ELSE
   IF RWayS.Status = L_Way.rwsStat_Rated THEN
    _Time:=0;
    SELECT(RWayJ, RWayS, , RWayS.RWayS);
    WHILE OCCURE(RWayJ) DO
     CONFORM(RWayJ, Quality);
     _TripTime:=0;
     IF Quality.Type # L_Way.QualType_Annul THEN
      IF ~byWLM | IsRWayJForCurWayLMen() THEN
       _TripTime:=RWayJ.StopT-RWayJ.StartT;
      END;
      IF _TripTime # 0 THEN
       CONFORM(RWayJ, SEJobs);
       L_Way.PositionOnRouteLog(SEJobs.Route, RWayS.Date);
       L_Way.PosToWJMethod(GetasWJMode(_Who), _Who, RouteLog.JRegime, RWayJ.Quality, RWayS.Date);
       IF WJMethod.TimeSel THEN
        INC(_Time, _TripTime*E_RWAYS_CoeffForCurTimes(RWayS.Date, _Who));
       END;
      END;
     END; (* IF Quality.Type ... *)
     NEXT(RWayJ);
    END; (* WHILE OCCURE(RWayJ) ... *)
   ELSE
    _Time:=0;
    SELECT(SEJobs, Exit, , RWayS.Exit);
    WHILE OCCURE(SEJobs) DO
     _TripTime:=0;
     IF ~byWLM | IsSEJobsForCurWayLMen() THEN
      _TripTime:=SEJobs.StopT-SEJobs.StartT;
     END;
     IF _TripTime # 0 THEN
      L_Way.PositionOnRouteLog(SEJobs.Route, RWayS.Date);
      L_Way.PosToWJMethod(GetasWJMode(_Who), _Who, RouteLog.JRegime, Qlt_Good, RWayS.Date);
      IF WJMethod.TimeSel THEN
       INC(_Time, _TripTime*E_RWAYS_CoeffForCurTimes(RWayS.Date, _Who));
      END;
     END;
     NEXT(SEJobs);
    END; (* WHILE OCCURE(SEJobs) ... *)
   END (* IF RWayS.Status... *)
  END; (* IF byWLM... *)
  RETURN _Time;
 END {GetWorkTimeFromRWayS};

 PUBLIC PROCEDURE GetWorkTimeInDate(D: DATE; trgTSN: LIKE(Staff.TSN)): NUMERIC[3];
  VAR
   Time,
   WorkTime     : FLOAT;
   Index        : STRING[];
   GetFromSheet,
   GetFromRWayS,
   GetFromDtwo  : SCAL[2];
   CurWho       : L_Way.tRWSWho;
 BEGIN
  IF !trgTSN | ~L_Env.RealTSN(trgTSN) THEN
   RETURN 0;
  END;
  SAVE(RWayS);
  SAVE(WayLMen);
  SAVE(Dtwo);
  SAVE(Sheet);
  WorkTime:=0;
  GetFromSheet:=0; GetFromRWayS:=0;
  IF SELECT(RWayS, TSN, trgTSN, D) THEN
   GetFromRWayS:=1;
   GetFromSheet:=RWayS.Done;
   WHILE OCCURE(RWayS) DO
    GetFromSheet:=GetFromSheet & RWayS.Done;
    IF RWayS.ToSheet > L_Way.toSheet_Dummy THEN
     INC(WorkTime, GetWorkTimeFromRWayS(L_Way.rwswho_Driver, 0));
    END;
    NEXT(RWayS);
   END; (* WHILE OCCURE(RWayS) ... *)
  END;
  IF SELECT(WayLMen, TSN, trgTSN, D) THEN
   GetFromRWayS:=1;
   CONFORM(WayLMen, RWayS);
   GetFromSheet:=RWayS.Done;
   WHILE OCCURE(WayLMen) DO
    CONFORM(WayLMen, RWayS);
    GetFromSheet:=GetFromSheet & RWayS.Done;
    IF WayLMen.ToSheet > L_Way.toSheet_Dummy THEN
     INC(WorkTime, GetWorkTimeFromRWayS(WayLMen.Who + 1, 1));
    END;
    NEXT(WayLMen);
   END; (* WHILE OCCURE(WayLMen) ... *)
  END;

  IF GetFromSheet THEN
   WorkTime:=0; (*쭨, ஢? ⭮ *)
   IF SEARCH(Sheet, TSN, trgTSN, L_DatTim.Per(D), L_DatTim.DAY(D)) THEN
    CONFORM(Sheet, JMode);
    IF JMode.Type = L_Sheet.jmType_TimDone THEN
     WorkTime:=Sheet.Tim;
    END;
   END;
  ELSIF ~GetFromRWayS & (D >= SYSDATE()) THEN
   GetFromDtwo:=0;
   IF SELECT(Dtwo, DTSN, D, trgTSN) THEN
    CurWho:=L_Way.rwswho_Driver;
    GetFromDtwo:=1;
   ELSIF SELECT(Dtwo, DGuardTSN, D, trgTSN) THEN
    CurWho:=L_Way.rwswho_Guard;
    GetFromDtwo:=1;
   END;
   Time:=0;
   WHILE OCCURE(Dtwo) & GetFromDtwo DO
    IF ~!Dtwo.EndTime THEN
     INC(Time, Dtwo.EndTime - Dtwo.BegTime);
     IF L_two.StandOnExit(Dtwo.ExitClss, Dtwo.Change, Dtwo.Date, 0) & (StExit.Strict = L_Way.ExitStrict_Reg) THEN
      DEC(Time, StExit.RestT);
     END;
    ELSE
     GetFromDtwo:=0;
     Time:=L_two.m_GetDayTime(CurWho, Dtwo.Date, Dtwo.Change, Dtwo.ExitClss);
    END;
    NEXT(Dtwo);
   END; (* WHILE OCCURE(Dtwo) ... *)
   IF GetFromDtwo THEN
    INC(Time, L_Way.E_GetStandartPZVTotalTime(D, CurWho));
   END;
   INC(WorkTime, Time);
  END;
  RESTORE(RWayS);
  RESTORE(WayLMen);
  RESTORE(Dtwo);
  RESTORE(Sheet);

  RETURN WorkTime;
 END {GetWorkTimeInDate};

 PUBLIC PROCEDURE GetLastMdfTime_RatedRWayS(lastDate: DATE): TIMESTAMP;
  VAR
   trgStamp,
   Result       : TIMESTAMP;
 BEGIN
  TRY
   Result:=MINOF(Result);
   SE_ACCESSDATAMODE(AccessDataMode_SOFT);
   SELECT(XDocJrn, UserIdx, , L_XDoc.xdt_RWayS);
   WHILE OCCURE(XDocJrn) DO
    NARROWSEL(XDocJrn, AcsObj);
    LAST(XDocJrn);
    LOOP
     IF ~OCCURE(XDocJrn) THEN
      EXIT;
     END;
     SEARCH(RWayS, RWayS, , XDocJrn.XDoc);
     IF (RWayS.Date <= lastDate) & (RWayS.Status = L_Way.rwsStat_Rated) THEN
      trgStamp:=PackTIMESTAMP(XDocJrn.Date, XDocJrn.BegTime, XDocJrn.BegSec);
      IF trgStamp > Result THEN
       Result:=trgStamp;
      END;
      EXIT;
     ELSE
      PREV(XDocJrn);
     END;
    END; (* LOOP *)
    NEXTINEXTENDEDSEL(XDocJrn, AcsObj);
   END; (* WHILE OCCURE(XDocJrn) ... *)
   L_General.RaiseLockReleaseException();
  EXCEPT
  ON Exc: Exc_Root DO
   L_General.CheckIsLockReleaseException(Exc);
  END;
  RETURN Result;
 END {GetLastMdfTime_RatedRWayS};

 PUBLIC PROCEDURE GetDaemonBoundaries(VAR begD, endD: DATE);
 BEGIN
  begD:=L_DatTim.MonthBeg(SYSDATE());
  IF L_DatTim.DAY(SYSDATE()) < 15 THEN
   begD:=L_DatTim.IncMonth(begD, -1);
  END;
  IF SYSTIME() > 17:00 THEN
   endD:=SYSDATE()-1;
  ELSE
   endD:=SYSDATE()-2;
  END;
 END {GetDaemonBoundaries};

 PUBLIC PROCEDURE IsWith(B, E: NUMERIC[3]; trgWho: L_Way.tWLMWho): SCAL[2];
  VAR
   TstBitSet: SCAL[2][50];
   i,
   MaxNbr   : LIKE(WayLMen.Number);
   Result,
   CheckDifferentNumber: SCAL[2];
 BEGIN
  IF ~E_GetTimesFromWayLMen() THEN
   Result:=0;
   CheckDifferentNumber:=trgWho = L_Way.wlmwho_Guard;
   SAVE(WayLMen);
   SETZERO(MaxNbr);
   SETEMPTY(TstBitSet);
   SETSEL(WayLMen, RWayS, RWayS.RWayS, trgWho);
   FOR_EACH WayLMen DO
    IF L_Way._TimeInt(B, E, L_Way.FullBegTime(WayLMen.Day, WayLMen.StartT),
                            L_Way.FullEndTime(WayLMen.Day, WayLMen.StartT, WayLMen.StopT)) > 0 THEN
     Result:=1;
     IF CheckDifferentNumber THEN
      L_General.Assert(!TstBitSet[WayLMen.Number], " ६ ࠡ饣 ᮭ   ࠧ :"+
                                                   "|ᮭ "+L_Str.PoolItem(L_Way.wlmwho_Pool_Short, trgWho)+"  "+WayLMen.Number);
      MaxNbr:=Math.Max(MaxNbr, WayLMen.Number);
      TstBitSet[WayLMen.Number]:=1;
     END;
    END;
   END; (* FOR_EACH WayLMen *)
   IF CheckDifferentNumber THEN
    i:=1;
    WHILE i<=MaxNbr DO
     L_General.Assert(~!TstBitSet[i], "᫥⥫쭠 㬥 .ᮭ:|ᮭ "+L_Str.PoolItem(L_Way.wlmwho_Pool_Short, trgWho));
     INC(i);
    END;
   END;
   RESTORE(WayLMen);
   RETURN Result;
  ELSE
   RETURN 0;
  END;
 END {IsWith};

 PUBLIC PROCEDURE CheckPeriodForRWaySProcessing(D: DATE);
 BEGIN
  IF D < 01\08\2008 THEN
   L_General.Error("樨 '⪠, ࠡ⪠, ' 饭  ⥢ ⮢  08.2008");
  END;
 END {CheckPeriodForRWaySProcessing};

 PUBLIC PROCEDURE CheckSheet(curToSheet: LIKE(WayLMen.ToSheet));
 BEGIN
  IF (curToSheet > L_Way.toSheet_Dummy) &
     ~L_Sheet.REPSheetCondition(RWayS.Date) & (Sheet.Source # L_Sheet.src_RWayS) THEN
   CONFORM(Sheet, JMode);
   L_StdFrm.Choice(" ⠡ ࠡ⭨ " + Staff.TSN + "  " + L_DatTim.DateStr(RWayS.Date) +
                   ":|㦥 㪠  ࠡ " + Sheet.JMode + ' "' + JMode.Name + '"' +
                   '| 筨 "' + L_Str.PoolItem(L_Sheet.src_Pool, Sheet.Source) + '"');
  END;
 END {CheckSheet};

 PUBLIC PROCEDURE SetRWJP(curTripNbr: tTripAmount; curValue: LIKE(RWayJP.Value));
 BEGIN
  RWJPStore[curTripNbr*RWJPNbr + RWayJP.RWJPro]:=curValue;
 END {SetRWJP};

 PUBLIC PROCEDURE ScanRWJP(curTripNbr: tTripAmount);
 BEGIN
  SETSEL(RWayJP, RWayS, RWayJ.RWayS, RWayJ.Day, RWayJ.StartT);
  FOR_EACH RWayJP DO
   IF ~!RWayJP.Value THEN
    SetRWJP(curTripNbr, RWayJP.Value);
   ELSE
    REMOVE(RWayJP);
   END;
  END; (* FOR_EACH RWayJP ... *)
 END {ScanRWJP};

 PUBLIC PROCEDURE GetRWJP(curTripNbr: tTripAmount; curRWJPro: LIKE(RWJPro.RWJPro)): LIKE(RWayJP.Value);
 BEGIN
  IF ~!RWJPStore[curTripNbr*RWJPNbr + curRWJPro] THEN
   RETURN RWJPStore[curTripNbr*RWJPNbr + curRWJPro];
  ELSE
   RETURN 0;
  END;
 END {GetRWJP};

 PUBLIC PROCEDURE InitRWJPStore();
 BEGIN
  SETEMPTY(RWJPStore);
 END {InitRWJPStore};

BEGIN
 SETEMPTY(crv_RWayS);
END.
