(*
%Copyright   (c) Polak IT
%Author      27.05.2005 16:53:45 POLAK\LDA
%Purpose     ⥪   ⨯   
%Type        library
%Specific    AP
%Modified    08.07.2014 18:55:12 POLAK\GAV
%Environment
*)

 MODULE L_RWSP [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Way, L_Print, L_DatTim, P_Firm, Str, P_Man, P_OwnData, L_Str, P_Car, L_Prop,
        L_BegRWS, FIO, L_General, L_Car, E_Way, L_Fuel, L_StdFrm, L_EC, L_APWI,
        L_ctc, L_APtm, L_Env, L_kn, L_DBD;

 PUBLIC TYPE
  tRWaySType = SCAL[9];

 PUBLIC CONST
  rwspt_Bus              = 0;
  rwspt_PassengerCar     = 1;
  rwspt_Truck            = 2;
  rwspt_SelfPropelledCar = 3;
  rwspt_UserDefined      = 4;

 PUBLIC VAR
  RWSPrt: PRINTJOB;

 PUBLIC PROCEDURE intTypeS(X, Y: NUMERIC[1]; S: STRING[]; P: TEXTSTYLE; CP: NUMERIC[0]);
 BEGIN
  L_Print.extTypeS(RWSPrt, X, Y, S, P, CP);
 END {intTypeS};

 PUBLIC PROCEDURE TypeS(X, Y: NUMERIC[1]; S: STRING[]; P: TEXTSTYLE);
 BEGIN
  intTypeS(X, Y, S, P, L_Print.cp_OEM_Russian);
 END {TypeS};

 PRIVATE PROCEDURE TextLength(S: STRING[]; P: TEXTSTYLE): FLOAT;
  VAR
   A, B: FLOAT;
 BEGIN
  RWSPrt.TextExtent(A, B, , , S, P, L_Print.cp_OEM_Russian);
  RETURN A+B;
 END {TextLength};

 PRIVATE PROCEDURE intTypeTruncS(X, Y, W: NUMERIC[1]; Text: STRING[]; P: TEXTSTYLE): STRING[];
  VAR
   S: STRING[];
 BEGIN
  S:=Text;
  IF W > 0 THEN
   LOOP
    IF TextLength(S, P) <= W THEN
     EXIT;
    END;
    S:=L_Str.Left(S, Str.Length(S)-1);
   END;
  END;
  TypeS(X, Y, S, P);
  RETURN Str.Slice(Text, Str.Length(S), Str.Length(Text));
 END {intTypeTruncS};

 PRIVATE PROCEDURE TypeTruncS(X, Y, W: NUMERIC[1]; S: STRING[]; P: TEXTSTYLE);
  VAR
   Dummy: STRING[];
 BEGIN
  Dummy:=intTypeTruncS(X, Y, W, S, P);
 END {TypeTruncS};

 PUBLIC PROCEDURE _NumStr(N: NUMERIC[5]; P: STRING[]; L: NUMERIC[0]): STRING[];
 BEGIN
  IF !N THEN
   RETURN " ";
  ELSE
   RETURN FLDTOSTR(N, P, L);
  END;
 END {_NumStr};

 PRIVATE PROCEDURE _ScalStr(N: SCAL[1E9]; P: STRING[]; L: NUMERIC[0]): STRING[];
 BEGIN
  IF !N THEN
   RETURN " ";
  ELSE
   RETURN FLDTOSTR(N, P, L);
  END;
 END {_ScalStr};

 PUBLIC PROCEDURE PrintRWaySBackground(FileName: STRING[]);
 BEGIN
  L_Print.PrintBlankBackground(RWSPrt, FIO.BuildPath(FileName, "BMP"));
 END {PrintRWaySBackground};

 PUBLIC PROCEDURE intPrintPhoto(X, Y, Width, Height: FLOAT);
 BEGIN
  P_Man.PrintPhoto(RWSPrt, X, Y, Width, Height, RWayS.Date);
 END {intPrintPhoto};

 PUBLIC PROCEDURE PrintStdProportionPhoto(X, Y, Height: FLOAT);
 BEGIN
  intPrintPhoto(X, Y, Height * 35 / 45, Height);
 END {PrintStdProportionPhoto};

 PUBLIC PROCEDURE PrintPhoto(X, Y: FLOAT);
 BEGIN
  PrintStdProportionPhoto(X, Y, 30.0);
 END {PrintPhoto};

 PUBLIC PROCEDURE PrintStamp(X, Y: FLOAT);
  CONST
   StampFileName = "ofStamp_RWS.gif";
 BEGIN
  L_Print.PrintStamp(RWSPrt, X, Y, 65, 20, StampFileName);
 END {PrintStamp};

 PUBLIC PROCEDURE GetDatesTimes(                  VAR _BTime: TIME;
                                VAR _EDate: DATE; VAR _ETime: TIME);
 BEGIN
  L_General.Assert(SELECT(SEJobs, Exit, RWayS.Exit) & LAST(SEJobs),
                   "室   ३: " + RWayS.Exit);
  _EDate:=RWayS.Date + (SEJobs.Day-1) + (SEJobs.StopT < SEJobs.StartT);
  _BTime:=StExit.BegTime;
  _ETime:=StExit.StopTim;
  IF L_BegRWS.RWayTxtIsValid() THEN
   IF ~!RWayTxt.FExTime THEN
    _BTime:=RWayTxt.FExTime;
   END;
   IF ~!RWayTxt.LEnTime THEN
    _EDate:=RWayS.Date + (RWayTxt.LEnTime < RWayTxt.FExTime);
    _ETime:=RWayTxt.LEnTime;
   END;
  END;
 END {GetDatesTimes};

 PUBLIC PROCEDURE GetLongTimes(VAR BLTime, ELTime: L_General.tLongTime);
  VAR
   BTime,
   ETime: TIME;
   EDate: DATE;
 BEGIN
  GetDatesTimes(BTime, EDate, ETime);
  BLTime:=(StExit.BegDay-1)+(BTime-00:00)/24;
  ELTime:=(StExit.BegDay>1)*(StExit.BegDay-1+(BTime>ETime))+(ETime-00:00)/24;
 END {GetLongTimes};

 PUBLIC PROCEDURE AddStr(Text, S: STRING[]): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  IF ~!S THEN
   IF ~!Text THEN
    Result:=Text + L_Str.CRLF + S;
   ELSE
    Result:=S;
   END;
  ELSE
   Result:=Text;
  END;
  RETURN Result;
 END {AddStr};

 PUBLIC PROCEDURE DefaultConforms();
 BEGIN
  CONFORM(RWayS,  Staff);
  CONFORM(Staff,  Man);
  CONFORM(RWayS,  Car);
  L_Car.StandOnCarLog(RWayS.BoxNbr, RWayS.Date);
  CONFORM(CarLog, Mark);
  CONFORM(CarLog, Column);
  IF P_Firm.E_wMultipleMyFirm(RWayS.Date) THEN
   P_Firm.AccessOwnFirm(Column.MyFirm);
  ELSE
   SEARCH(Firm, Firm, , P_Firm.OWN_FIRM);
  END;
  CONFORM(RWayS,  Calendar);
  CONFORM(RWayS,  StExit);
  CONFORM(StExit, ExitClss);
  CONFORM(RWayS,  RWayTxt);
 END {DefaultConforms};

 PRIVATE PROCEDURE GetFirmName(): STRING[];
 BEGIN
  RETURN P_Firm.FP_ContractorBriefName(Firm.Firm, RWayS.Date);
 END {GetFirmName};

 PRIVATE PROCEDURE GetFirmAddr(): STRING[];
 BEGIN
  RETURN L_ctc.GetAdr_Reg(RWayS.Date, Firm.Firm);
 END {GetFirmAddr};

 PRIVATE PROCEDURE GetPhone_DispOrCommon(): STRING[];
 BEGIN
  IF L_kn.StandSoft(RWayS.Date, L_ctc.erkn_DispatcherOffice) THEN
   RETURN L_ctc.GetCtcVal(1, FirmCtc.Contact, RWayS.Date, L_ctc.cvc_Work, L_ctc.cvt_PhonePST);
  ELSE
   RETURN P_Firm.FP_Phone(Firm.Firm, RWayS.Date);
  END;
 END {GetPhone_DispOrCommon};

 PRIVATE PROCEDURE GetFirmAttr(): STRING[];
 BEGIN
  RETURN GetFirmName() + " " + GetFirmAddr();
 END {GetFirmAttr};

 PRIVATE PROCEDURE AccessMainFuel(D: DATE; trgMark: LIKE(Mark.Mark)): SCAL[2];
  VAR
   mfMath: LIKE(Math.Math);
 BEGIN
  mfMath:=L_Fuel.GetMainFuelSoft(D, trgMark);
  IF ~!mfMath THEN
   L_General.CallUs(SEARCH(Math, Math, mfMath));
   RETURN 1;
  END;
  RETURN 0;
 END {AccessMainFuel};

 PUBLIC PROCEDURE E_CLITermWarning(D: DATE): STRING[];
  VAR
   CLIEndDays: NUMERIC[0];
   _DocExpD  : LIKE(CarPro.DocExpD);
 BEGIN
  IF L_EC.ExecE_Func_Scal2(D, PROCREF(E_Way.BegRWS_NeedCheckPresenceCLIP), 0) THEN
   IF P_Car.HasActualCarPro(RWayS.BoxNbr, D, P_Car.P_CLI_Obligatory) THEN
    REPEAT
     _DocExpD:=CarPro.DocExpD;
    UNTIL ~P_Car.HasActualCarPro(RWayS.BoxNbr, _DocExpD, P_Car.P_CLI_Obligatory) | !_DocExpD;
    IF ~!_DocExpD THEN
     CLIEndDays:=_DocExpD - D + 1;
     IF CLIEndDays < 14 THEN
      RETURN ": " +
              Str.EnumObject(CLIEndDays, "⠫|⠫|⠫") +
              " " + L_Str.IntStr(CLIEndDays) + " " +
              Str.EnumObject(CLIEndDays, "||");
     END;
    END;
   ELSIF ~OCCURE(CarPro) THEN
    RETURN ":  ";
   END;
  END;
  RETURN "";
 END {E_CLITermWarning};

 PRIVATE PROCEDURE E_PrintFOver(FOver: LIKE(RWayS.FuelNorm); wText: SCAL[2]): STRING[];
  CONST
   FOverStr = ["", "० "];
  VAR
   Local : PROC;
   Result: STRING[];
 BEGIN
  IF FOver > 0 THEN
   Local:=PROCREF(E_Way.PrintFOver);
   IF ~!Local THEN
    Result:=Local(RWayS.Date, FOver);
   ELSE
    Result:=FOverStr[wText] + _NumStr(FOver, "1.", 6);
   END;
  ELSE
   Result:="";
  END;
  RETURN Str.Condense(Result);
 END {E_PrintFOver};

 PUBLIC PROCEDURE TypeRWSBarCode(X, Y: NUMERIC[1]);
  VAR
   Pt18BarCode : TEXTSTYLE;
 BEGIN
  Pt18BarCode             :=L_Print.ts_Create(18, "IntP36DlTt");
  Pt18BarCode.Font.Charset:=FONT.Charset_ANSI;
  intTypeS(X, Y, L_Way.GetBarCode_bck_IdIn20(), Pt18BarCode, L_Print.cp_ISO_8859_1_Latin_I);
 END {TypeRWSBarCode};

 PUBLIC PROCEDURE GetManDocExpDate(_Property: LIKE(Property.Property)): DATE;
  VAR
   Result: DATE;
 BEGIN
  Result:=RWayS.Date;
  WHILE ~!Result & P_Man.HasActualManPro(Man.Man, Result, _Property) DO
   Result:=ManPro.DocExpD;
  END;
  RETURN Result;
 END {GetManDocExpDate};

 PUBLIC PROCEDURE GetManDocNum(_Property: LIKE(Property.Property)): STRING[];
 BEGIN
  IF P_Man.HasActualManPro(Man.Man, RWayS.Date, _Property) THEN
   RETURN ManPro.DocNum;
  ELSE
   RETURN "";
  END;
 END {GetManDocNum};

 PUBLIC PROCEDURE TypeMedicalSertificationWarningOrBirthdayCongratulation(X, Y: FLOAT; pt: TEXTSTYLE);
  VAR
   _SP_MedicalSertification: DATE;
 BEGIN
  _SP_MedicalSertification:=GetManDocExpDate(P_Man.P_MedicalSertification);
  IF    _SP_MedicalSertification < RWayS.Date+40 THEN
   TypeS(X, Y, "ன ᬮ  "+L_DatTim.DateStr(_SP_MedicalSertification)+"!", pt);
  ELSIF L_DatTim.IsBirthDay(Man.Birth, RWayS.Date) THEN
   TypeS(X, Y, "  ஦!",                                                    pt);
  END;
 END {TypeMedicalSertificationWarningOrBirthdayCongratulation};

 PUBLIC PROCEDURE WeatherConditionsStr(): STRING[];
  VAR
   Text: STRING[];
 BEGIN
  Text:=P_OwnData.GetOwnDProSoft(RWayS.Date, P_OwnData.P_WeatherConditions);
  IF ~!Text THEN
   Text:=",  ஦. " +  Text;
  END;
  RETURN Text;
 END {WeatherConditionsStr};

 PUBLIC PROCEDURE StdSpecialMarks(): STRING[];
  VAR
   Text: STRING[];
 BEGIN
  Text:=WeatherConditionsStr();
  Text:=AddStr(Text, P_OwnData.MessageToDriver(RWayS.Date));
  Text:=AddStr(Text, E_CLITermWarning(RWayS.Date));
  RETURN Text;
 END {StdSpecialMarks};

 PRIVATE PROCEDURE rs_Form_3();
  VAR
   FNorm,
   FOver    : LIKE(RWayS.FuelNorm);
   DocSer,
   DocNum,
   Text     : STRING[];
   TextArray: STRING[][10];
   AddrArray: STRING[][2];
   i        : NUMERIC[0];
   Pt08,
   Pt09,
   Pt10,
   Pt12,
   Pt15     : TEXTSTYLE;
   BLTime,
   ELTime   : L_General.tLongTime;
 BEGIN
  DefaultConforms();
  GetLongTimes(BLTime, ELTime);

  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Vertical);
  Pt08:=L_Print.ts_Create(08, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt08, 10);
  Pt10:=L_Print.ts_Resize(Pt08, 10);
  Pt12:=L_Print.ts_Resize(Pt08, 12);
  Pt15:=L_Print.ts_Resize(Pt08, 15);
 
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f3_f");

  PrintPhoto(081.5, 010);

  TypeS(154, 029,                           RWayS.RWayS, Pt12);
  TypeS(172, 029,             "(" + RWayS.CopyCnt + ")", Pt10);
  TypeS(110, 034, FLDTOSTR(RWayS.Date, "D "+L_DatTim.MonthStr(RWayS.Date, 1)+" y", 0), Pt10);
  TypeS(148, 034,           L_DatTim.TimeStr(SYSTIME()), Pt10);

  PrintStamp(10, 10);

  Str.Format(AddrArray, GetFirmAttr(), 62, LEFT);

  TypeS(043, 053,                          AddrArray[0], Pt09);
  TypeS(043, 060,                          AddrArray[1], Pt09);

  TypeRWSBarCode(015, 042);

  TypeS(045, 065.5,                             Mark.Name, Pt10);
  TypeS(176, 065.5,                             Mark.Mark, Pt09);
  TypeS(066, 070,                       CarLog.StateNbr, Pt10);
  TypeS(176, 070,                          RWayS.BoxNbr, Pt09);
  TypeS(030, 074,                              Man.Name, Pt10);
  TypeS(175, 074,                             RWayS.TSN, Pt09);

  TypeS(041, 081, GetManDocNum(P_Man.P_DriveSertification), Pt10);
  TypeS(161, 081, P_Man.GetDriveSertification_Category(Man.Man, RWayS.Date), Pt10);

  IF P_Car.HasActualCarPro(RWayS.BoxNbr, RWayS.Date, P_Car.P_CarLicenCard) THEN
   L_Prop.GetDocSerAndNum(CarPro.DocNum, DocSer, DocNum);
   TypeS(045, 092,                       CarPro.Value, Pt10);
   TypeS(090, 092,                             DocNum, Pt10);
   TypeS(135, 092,                             DocSer, Pt10);
  END;
  TypeS(176, 102.5,        _NumStr(RWayS.SMBef, "0", 6), Pt09);
  TypeS(039, 106.5,                         StExit.Name, Pt10);
  TypeS(080, 115.5,                          RWayS.Exit, Pt09);

  TypeS(015, 138,             L_BegRWS.GetCWOrderList(), Pt08);

  TypeS(159, 116,             L_BegRWS.GetMechanicName(RWayS.Date), Pt09);

  TypeS(159, 133,             L_Str.ShortName(Man.Name), Pt09);
  TypeS(072, 215,             L_Str.ShortName(Man.Name), Pt09);

  TypeS(073, 158, L_Str.ShortName(P_Man.GetFullNameOfCurrentUser(SYSDATE())), Pt10);

  TypeS(087, 149,          L_DatTim.LongTimeStr(BLTime), Pt09);
  IF AccessMainFuel(RWayS.Date, Mark.Mark) THEN
   TypeS(143, 148,            L_Str.Left(Math.Name, 11), Pt09);
   TypeS(177, 148,                            Math.Math, Pt09);
  END;
  TypeS(087, 172,          L_DatTim.LongTimeStr(ELTime), Pt09);

  IF P_Car.HasActualCarPro(RWayS.BoxNbr, RWayS.Date, P_Car.P_FuelCard) THEN
   TypeS(133, 180.5, "⮯.  " + CarPro.DocNum, Pt08);
  END;

  TypeS(177, 184.5,      _NumStr(RWayS.RemExit, "0", 3), Pt09);
  L_Way.PrepareFuelInf(FNorm, FOver);
  TypeS(177, 192.5,                _NumStr(FNorm, "0", 3), Pt09);
  TypeS(177, 204.5,                E_PrintFOver(FOver, 0), Pt09);

  TypeMedicalSertificationWarningOrBirthdayCongratulation(016, 234, Pt09);
  IF L_BegRWS.RWayTxtIsValid() THEN
   TypeS(080, 234, RWayTxt.Message,   Pt08);
  END;
  Str.Format(TextArray, StdSpecialMarks(), 110, LEFT);
  i:=0;
  WHILE (i <= HIGH(TextArray)) & ~!TextArray[i] DO
   TypeS(016, 239 + i*5, TextArray[i], Pt08);
   INC(i);
  END; (* WHILE ~!TextArray[i]...*)
  RWSPrt.EndPage();
 END {rs_Form_3};

 PUBLIC PROCEDURE us_AnyForm(FileName: STRING[]);
 BEGIN
  RWSPrt.StartPage();
  PrintRWaySBackground(FileName);
  RWSPrt.EndPage();
 END {us_AnyForm};

 PRIVATE PROCEDURE print_Form_3();
 BEGIN
  rs_Form_3();
  us_AnyForm("rws_f3_b");
 END {print_Form_3};

 PUBLIC PROCEDURE f3Spec_FillCoupons(FillCustomer: SCAL[2]);
  VAR
   Pt07,
   Pt09    : TEXTSTYLE;
   s: STRING[];
   StrArr,
   nStrArr : STRING[][3];
 BEGIN
  Pt07:=L_Print.ts_Create(07, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt07, 09);
  Str.Format(nStrArr, P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date), 30, LEFT);

  TypeS(250, 017.5, RWayS.RWayS, Pt09);
  TypeS(235, 025.5, FLDTOSTR(RWayS.Date, "D", 0), Pt09);
  TypeS(245, 025.5, L_DatTim.MonthStr(RWayS.Date, 1), Pt09);
  TypeS(266, 025.5, FLDTOSTR(RWayS.Date, "y", 0), Pt09);
  TypeS(244, 030, nStrArr[0], Pt07);
  TypeS(244, 032, nStrArr[1], Pt07);
  TypeS(252, 035.5, Mark.Name, Pt07);
  TypeS(268, 039, CarLog.StateNbr, Pt07);

  IF SELECT(CWOrder, RWayS, RWayS.RWayS) THEN
   IF FillCustomer & SEARCH(Contract, Contract, CWOrder.Contract) THEN
    SAVE(Firm);
    CONFORM(Contract, Firm);
    s:=P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date);
    Str.Format(StrArr, s, 33, LEFT);
    TypeS(240, 043.5, StrArr[0], Pt07);
    s:=Str.Condense(Str.Slice(s, Str.Length(Str.Condense(StrArr[0])), Str.Length(s)));
    Str.Format(StrArr, s + " " + GetFirmAddr() + " " + P_Firm.FP_Phone(Firm.Firm, RWayS.Date), 50, LEFT);
    TypeS(225, 047.5, StrArr[0], Pt07);
    TypeS(225, 051.5, StrArr[1], Pt07);
    TypeS(225, 056,   StrArr[2], Pt07);
    RESTORE(Firm);
   END;
  ELSIF L_APtm.IsOG() THEN
   TypeS(240, 043.5, ' ""', Pt07);
   TypeS(225, 047.5, '. , . ᪠, . 26', Pt07);
  END;

  TypeS(256, 121.5, RWayS.RWayS, Pt09);
  TypeS(236, 131, FLDTOSTR(RWayS.Date, "D", 0), Pt09);
  TypeS(245, 131, L_DatTim.MonthStr(RWayS.Date, 1), Pt09);
  TypeS(267, 131, FLDTOSTR(RWayS.Date, "y", 0), Pt09);
  TypeS(244, 135, nStrArr[0], Pt07);
  TypeS(244, 137.5, nStrArr[1], Pt07);
  TypeS(252, 142, Mark.Name, Pt07);
  TypeS(272, 150, CarLog.StateNbr, Pt07);
  IF OCCURE(CWOrder) & NEXT(CWOrder) THEN
   IF FillCustomer & SEARCH(Contract, Contract, CWOrder.Contract) THEN
    SAVE(Firm);
    CONFORM(Contract, Firm);
    s:=P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date);
    Str.Format(StrArr, s, 33, LEFT);
    TypeS(240, 154, StrArr[0], Pt07);
    s:=Str.Condense(Str.Slice(s, Str.Length(Str.Condense(StrArr[0])), Str.Length(s)));
    Str.Format(StrArr, s + " " + GetFirmAddr() + " " + P_Firm.FP_Phone(Firm.Firm, RWayS.Date), 50, LEFT);
    TypeS(225, 158, StrArr[0], Pt07);
    TypeS(225, 162, StrArr[1], Pt07);
    RESTORE(Firm);
   END;
  ELSIF L_APtm.IsOG() THEN
   TypeS(240, 154, ' ""', Pt07);
   TypeS(225, 158, '. , . ᪠, . 26', Pt07);
  END;
 END {f3Spec_FillCoupons};

 PRIVATE PROCEDURE rs_Form_3Spec(AddSpcMark: STRING[]);
  VAR
   AddrArray: STRING[][2];
   Pt06,
   Pt08,
   Pt09,
   Pt10,
   Pt12,
   Pt15     : TEXTSTYLE;
   BLTime,
   ELTime   : L_General.tLongTime;
   DocSer,
   DocNum   : STRING[];
   TextArray: STRING[][7];
   i        : NUMERIC[0];
 BEGIN
  DefaultConforms();
  GetLongTimes(BLTime, ELTime);

  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  Pt08:=L_Print.ts_Create(08, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt08, 10);
  Pt10:=L_Print.ts_Resize(Pt08, 10);
  Pt12:=L_Print.ts_Resize(Pt08, 12);
  Pt15:=L_Print.ts_Resize(Pt08, 15);
  Pt06:=L_Print.ts_Resize(Pt08, 06);
 
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f3Spec_f");

  PrintStdProportionPhoto(083, 004, 28);

  TypeS(157, 025,                           RWayS.RWayS + " (" + RWayS.CopyCnt + ")", Pt10);
  TypeS(108, 030, FLDTOSTR(RWayS.Date, "D "+L_DatTim.MonthStr(RWayS.Date, 1)+" y", 0), Pt12);
  TypeS(148, 030,             L_DatTim.TimeStr(SYSTIME()), Pt09);

  PrintStamp(10, 10);
  Str.Format(AddrArray, GetFirmAttr(), 60, LEFT);

  TypeS(033, 035,                          AddrArray[0], Pt09);
  TypeS(009, 043,                          AddrArray[1], Pt09);

  TypeS(195, 035.5, P_Firm.GetFirmProSoft(Firm.Firm, RWayS.Date, P_Firm.P_OKPO), Pt10);
  TypeS(202, 043,                         CarLog.Column, Pt08);
  TypeS(043, 051,                             Mark.Name, Pt10);
  TypeS(200, 051,                             Mark.Mark, Pt08);
  TypeS(064, 055,                       CarLog.StateNbr, Pt10);
  TypeS(200, 055,                          RWayS.BoxNbr, Pt08);
  TypeS(028, 059,                              Man.Name, Pt10);
  TypeS(199, 059,                             RWayS.TSN, Pt08);

  TypeS(040, 064.5, GetManDocNum(P_Man.P_TractorDriverSertification), Pt10);
  TypeS(150, 064.5, P_Man.GetManProSoft(Man.Man, RWayS.Date, P_Man.P_TractorDriverSertification), Pt10);

  IF P_Car.HasActualCarPro(RWayS.BoxNbr, RWayS.Date, P_Car.P_CarLicenCard) THEN
   L_Prop.GetDocSerAndNum(CarPro.DocNum, DocSer, DocNum);
   TypeS(043, 075,                         CarPro.Value, Pt10);
   TypeS(115, 075,                               DocSer, Pt10);
   TypeS(155, 075,                               DocNum, Pt10);
  END;

  SELECT(WayLCar, RWayS, , RWayS.RWayS);
  IF OCCURE(WayLCar) THEN
   L_Car.StandOnCarLog(WayLCar.BoxNbr, WayLCar.RWayDate);
   CONFORM(CarLog, Mark);
   TypeS(025, 079.5,      CarLog.Mark + " " + Mark.Name, Pt09);
   TypeS(112, 079.5,                    CarLog.StateNbr, Pt09);
   TypeS(198, 079.5,                     WayLCar.BoxNbr, Pt09);
  END;

  TypeRWSBarCode(011, 088);
  TypeS(120, 088, AddSpcMark, Pt08);

  TypeS(035, 113.5,          L_DatTim.LongTimeStr(BLTime), Pt08);
  TypeS(059, 113.5,          _NumStr(RWayS.SMBef, "0", 6), Pt08);
  IF AccessMainFuel(RWayS.Date, Mark.Mark) THEN
   TypeS(093, 113.5,             L_Str.Left(Math.Name, 7), Pt08);
   TypeS(108, 113.5,                            Math.Math, Pt08);
  END;
  TypeS(137, 113.5,        _NumStr(RWayS.RemExit, "0", 3), Pt08);
  TypeS(035, 119,            L_DatTim.LongTimeStr(ELTime), Pt08);

  TypeS(185, 166,    L_BegRWS.GetMechanicName(RWayS.Date), Pt09);
  TypeS(055, 176, L_Str.ShortName(P_Man.GetFullNameOfCurrentUser(SYSDATE())), Pt10);
  TypeS(185, 176,               L_Str.ShortName(Man.Name), Pt09);
  TypeS(185, 192,               L_Str.ShortName(Man.Name), Pt09);

  Str.Format(TextArray, StdSpecialMarks(), 28, LEFT);
  i:=0;
  WHILE (i <= HIGH(TextArray)) & ~!TextArray[i] DO
   TypeS(174, 134 + i*4, TextArray[i], Pt08);
   INC(i);
  END; (* WHILE ~!TextArray[i]...*)

  f3Spec_FillCoupons(1);
  RWSPrt.EndPage();
 END {rs_Form_3Spec};

 PUBLIC PROCEDURE print_AnyForm_3Spec(AddSpcMark: STRING[]);
 BEGIN
  CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  rs_Form_3Spec(AddSpcMark);
  us_AnyForm("rws_f3Spec_b");
  RWSPrt.Close();
 END {print_AnyForm_3Spec};

 PUBLIC PROCEDURE printStdForm_3Spec();
 BEGIN
  print_AnyForm_3Spec("");
 END {printStdForm_3Spec};

 PUBLIC PROCEDURE print_Form_3Spec();
 BEGIN
  rs_Form_3Spec("");
  us_AnyForm("rws_f3Spec_b");
 END {print_Form_3Spec};

 PRIVATE PROCEDURE rs_Form_4P();
  VAR
   Pt08,
   Pt09,
   Pt10,
   Pt12,
   Pt15     : TEXTSTYLE;
   _BTime,
   _ETime   : TIME;
   _EDate   : DATE;
   i        : NUMERIC[0];
   DocSer,
   DocNum   : STRING[];
   TextArray: STRING[][6];
 BEGIN
  DefaultConforms();
  GetDatesTimes(_BTime, _EDate, _ETime);

  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  Pt08:=L_Print.ts_Create(08, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt08, 10);
  Pt10:=L_Print.ts_Resize(Pt08, 10);
  Pt12:=L_Print.ts_Resize(Pt08, 12);
  Pt15:=L_Print.ts_Resize(Pt08, 15);

  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f4p_f");
  TypeS(161, 020,                           RWayS.RWayS, Pt12);
  TypeS(180, 020,             "(" + RWayS.CopyCnt + ")", Pt10);
  TypeS(095, 029.5, FLDTOSTR(RWayS.Date, "D "+L_DatTim.MonthStr(RWayS.Date, 1)+" y", 0), Pt12);
  TypeS(145, 029.5,         L_DatTim.TimeStr(SYSTIME()), Pt09);

  PrintStamp(10, 10);
  TypeS(033, 033.5,                       GetFirmAttr(), Pt10);

  TypeRWSBarCode(010, 045);

  TypeS(140, 048.5,                       CarLog.Column, Pt08);
  TypeS(030, 056,                             Mark.Name, Pt10);
  TypeS(139, 056,                             Mark.Mark, Pt08);
  TypeS(045, 060,                       CarLog.StateNbr, Pt09);
  TypeS(139, 060,                          RWayS.BoxNbr, Pt08);

  TypeS(023, 065,                              Man.Name, Pt09);
  TypeS(138, 065,                             RWayS.TSN, Pt08);
  TypeS(030, 071, GetManDocNum(P_Man.P_DriveSertification), Pt09);
  TypeMedicalSertificationWarningOrBirthdayCongratulation(252.5, 117, Pt08);
  IF P_Car.HasActualCarPro(RWayS.BoxNbr, RWayS.Date, P_Car.P_CarLicenCard) THEN
   L_Prop.GetDocSerAndNum(CarPro.DocNum, DocSer, DocNum);
   TypeS(035, 080.5,                       CarPro.Value, Pt12);
   TypeS(080, 080.5,                             DocSer, Pt12);
   TypeS(102, 080.5,                             DocNum, Pt12);
  END;

  i:=0;
  SELECT(WayLCar, RWayS, , RWayS.RWayS);
  WHILE OCCURE(WayLCar) & (i < 2) DO
   L_Car.StandOnCarLog(WayLCar.BoxNbr, WayLCar.RWayDate);
   TypeS(022, 86 + i * 6,                   CarLog.Mark, Pt09);
   TypeS(073, 86 + i * 6,               CarLog.StateNbr, Pt09);
   TypeS(137, 86 + i * 6,                WayLCar.BoxNbr, Pt09);
   INC(i);
   NEXT(WayLCar);
  END; (* WHILE OCCURE(WayLCar) ... *)

  TypeS(010, 120.5,                          RWayS.Exit, Pt08);
  TypeS(010, 124,                           StExit.Name, Pt08);
  TypeS(045, 133, L_Str.ShortName(P_Man.GetFullNameOfCurrentUser(SYSDATE())), Pt09);

  TypeS(190, 060,  FLDTOSTR(RWayS.Date, "D       N", 0), Pt08);
  TypeS(211, 060,      FLDTOSTR(_BTime, "H       M", 0), Pt08);
  TypeS(240, 060,          _NumStr(RWayS.SMBef, "0", 6), Pt08);
  TypeS(190, 064.5, FLDTOSTR(_EDate,     "D       N", 0), Pt08);
  TypeS(211, 064.5,     FLDTOSTR(_ETime, "H       M", 0), Pt08);

  IF AccessMainFuel(RWayS.Date, Mark.Mark) THEN
   TypeS(164, 095.5,           L_Str.Left(Math.Name, 7), Pt08);
   TypeS(179, 095.5,                          Math.Math, Pt08);
  END;
  TypeS(207, 095.5,      _NumStr(RWayS.RemExit, "0", 3), Pt08);

  TypeS(207, 120.5,L_BegRWS.GetMechanicName(RWayS.Date), Pt09);
  TypeS(207, 126.4,           L_Str.ShortName(Man.Name), Pt09);
  TypeS(190, 139.5,           L_Str.ShortName(Man.Name), Pt09);

  Str.Format(TextArray, StdSpecialMarks(), 40, LEFT);
  i:=0;
  WHILE (i <= HIGH(TextArray)) & ~!TextArray[i] DO
   TypeS(231, 120.5 + i*3.2, TextArray[i], Pt08);
   INC(i);
  END; (* WHILE ~!TextArray[i]...*)

  RWSPrt.EndPage();
 END {rs_Form_4P};

 PUBLIC PROCEDURE printStdForm_4P();
 BEGIN
  CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  rs_Form_4P();
  us_AnyForm("rws_f4p_b");
  RWSPrt.Close();
 END {printStdForm_4P};

 PRIVATE PROCEDURE rs_Form_4S();
  VAR
   Pt08,
   Pt09,
   Pt10,
   Pt12,
   Pt15     : TEXTSTYLE;
   _BTime,
   _ETime   : TIME;
   _EDate   : DATE;
   i        : NUMERIC[0];
   Text,
   DocSer,
   DocNum   : STRING[];
   TextArray: STRING[][5];
 BEGIN
  DefaultConforms();
  GetDatesTimes(_BTime, _EDate, _ETime);

  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  Pt08:=L_Print.ts_Create(08, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt08, 10);
  Pt10:=L_Print.ts_Resize(Pt08, 10);
  Pt12:=L_Print.ts_Resize(Pt08, 12);
  Pt15:=L_Print.ts_Resize(Pt08, 15);

  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f4s_f");
  PrintStdProportionPhoto(192, 006, 28);

  TypeS(158, 020,                           RWayS.RWayS, Pt12);
  TypeS(177, 020,             "(" + RWayS.CopyCnt + ")", Pt10);
  TypeS(100, 030, FLDTOSTR(RWayS.Date, "D "+L_DatTim.MonthStr(RWayS.Date, 1)+" y", 0), Pt12);
  TypeS(145, 030,           L_DatTim.TimeStr(SYSTIME()), Pt09);

  PrintStamp(10, 10);
  TypeS(033, 035,                         GetFirmAttr(), Pt10);

  TypeRWSBarCode(010, 045);

  TypeS(144, 053,                         CarLog.Column, Pt08);
  TypeS(034, 058,                             Mark.Name, Pt10);
  TypeS(142, 058,                             Mark.Mark, Pt08);
  TypeS(050, 063,                       CarLog.StateNbr, Pt09);
  TypeS(142, 063,                          RWayS.BoxNbr, Pt08);

  TypeS(023, 068,                              Man.Name, Pt09);
  TypeS(141, 068,                             RWayS.TSN, Pt08);
  TypeS(034, 074.5, GetManDocNum(P_Man.P_DriveSertification), Pt09);

  IF P_Car.HasActualCarPro(RWayS.BoxNbr, RWayS.Date, P_Car.P_CarLicenCard) THEN
   L_Prop.GetDocSerAndNum(CarPro.DocNum, DocSer, DocNum);
   TypeS(035, 084,                         CarPro.Value, Pt12);
   TypeS(082, 084,                               DocSer, Pt12);
   TypeS(107, 084,                               DocNum, Pt12);
  END;

  i:=0;
  SELECT(WayLCar, RWayS, , RWayS.RWayS);
  WHILE OCCURE(WayLCar) DO
   L_Car.StandOnCarLog(WayLCar.BoxNbr, WayLCar.RWayDate);
   TypeS(022, 90 + i * 5,                   CarLog.Mark, Pt09);
   TypeS(075, 90 + i * 5,               CarLog.StateNbr, Pt09);
   TypeS(142, 90 + i * 5,                WayLCar.BoxNbr, Pt09);
   INC(i);
   NEXT(WayLCar);
  END; (* WHILE OCCURE(WayLCar) ... *)

  TypeS(045, 171, L_Str.ShortName(P_Man.GetFullNameOfCurrentUser(SYSDATE())), Pt09);

  TypeS(181, 062.5,  FLDTOSTR(RWayS.Date, "D         N", 0), Pt08);
  TypeS(203, 062.5,       FLDTOSTR(_BTime, "H        M", 0), Pt08);
  TypeS(236, 062.5,            _NumStr(RWayS.SMBef, "0", 6), Pt08);
  TypeS(181, 067,    FLDTOSTR(_EDate,     "D         N", 0), Pt08);
  TypeS(203, 067,         FLDTOSTR(_ETime, "H        M", 0), Pt08);

  IF AccessMainFuel(RWayS.Date, Mark.Mark) THEN
   TypeS(156.5, 099.5,         L_Str.Left(Math.Name, 7), Pt08);
   TypeS(170.5, 099.5,                        Math.Math, Pt08);
  END;
  TypeS(200, 099.5,      _NumStr(RWayS.RemExit, "0", 3), Pt08);

  TypeS(155, 165.5,  L_BegRWS.GetMechanicName(RWayS.Date), Pt09);

  TypeS(155, 171,             L_Str.ShortName(Man.Name), Pt09);
  TypeS(140, 193,             L_Str.ShortName(Man.Name), Pt09);

  TypeS(247, 171, "室 " + RWayS.Exit, Pt09);

  Text:="";
  IF P_Car.HasActualCarPro(RWayS.BoxNbr, RWayS.Date, P_Car.P_FuelCard) THEN
   Text:="  " + CarPro.DocNum;
  END;
  Str.Format(TextArray, AddStr(Text, StdSpecialMarks()), 50, LEFT);
  i:=0;
  WHILE (i <= HIGH(TextArray)) & ~!TextArray[i] DO
   TypeS(206, 177.5 + i*7, TextArray[i], Pt09);
   INC(i);
  END; (* WHILE ~!TextArray[i]...*)
  RWSPrt.EndPage();
 END {rs_Form_4S};

 PRIVATE PROCEDURE print_Form_4S();
 BEGIN
  rs_Form_4S();
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f4s_b");
  TypeMedicalSertificationWarningOrBirthdayCongratulation(39, 105.5, L_Print.ts_Create(10, "Tahoma"));
  RWSPrt.EndPage();
 END {print_Form_4S};

 PUBLIC PROCEDURE int_rs_Form_6(fName: STRING[]; AddrArray: STRING[][2]; AditionalType: PROC; wBarCode: SCAL[2]);
  VAR
   y,
   IncPlan   : NUMERIC[2];
   FNorm,
   FOver     : LIKE(RWayS.FuelNorm);
   Rest,
   DocSer,
   DocNum    : STRING[];
   sdCnt,
   guCnt,
   day,
   month,
   year      : NUMERIC[0];
   Pt06,
   Pt07,
   Pt08,
   Pt10,
   Pt12,
   Pt16,
   Pt10b     : TEXTSTYLE;
   BLTime,
   ELTime    : L_General.tLongTime;
   VldPsgPlan: L_APWI.tVldPsgAmnt;
 BEGIN
  Pt06:=L_Print.ts_Create(06, "Times New Roman");
  Pt06.Font.Weight:=FONT.Weight_Bold;
  Pt07:=L_Print.ts_Resize(Pt06, 07);
  Pt08:=L_Print.ts_Resize(Pt06, 08);
  Pt10 :=L_Print.ts_Resize(Pt06, 10);
  Pt10b:=L_Print.ts_Resize(Pt06, 10);
  Pt12:=L_Print.ts_Resize(Pt06, 12);
  Pt16:=L_Print.ts_Resize(Pt06, 16);
 
  DefaultConforms();

  GetLongTimes(BLTime, ELTime);

  IF (L_General.GetAPSiteNumber() # 50) THEN
   L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  END;
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6_f");
  PrintStdProportionPhoto(260, 004, 28);

  IF wBarCode THEN
   TypeRWSBarCode(017, 017);
  END;

 
  IncPlan:=L_Way.GetExitPlan();
 
  L_Way.PrepareFuelInf(FNorm, FOver);
 
  UnPkDATE(RWayS.Date, day, month, year);

  TypeS(071, 022,                     _NumStr(day, "0", 2),   Pt12);
  TypeS(083, 022,         L_DatTim.MonthStr(RWayS.Date, 1),   Pt12);
  TypeS(104, 022,                    _NumStr(year, "0", 4),   Pt12);

  TypeS(066, 034, P_Firm.GetFirmProSoft(Firm.Firm, RWayS.Date, P_Firm.P_OKPO), Pt10);

  fName:=intTypeTruncS(036, 026,   50, fName, Pt08);
  IF TextLength(fName, Pt08) > 25 THEN
   fName:=intTypeTruncS(019, 029.5, 30, fName, Pt08);
   Rest :=intTypeTruncS(019, 033,   25, fName, Pt08);
  ELSE
   TypeS(019, 033, fName, Pt08);
  END;
  TypeS(020, 038.5,                             AddrArray[0], Pt08);
  TypeS(020, 044,                               AddrArray[1], Pt08);

  TypeS(156, 017,   RWayS.RWayS + " (" + RWayS.CopyCnt + ")", Pt16);

  TypeS(100, 055.5,                L_Str.ShortName(Man.Name), Pt10b);
  TypeS(140, 055.5,                                RWayS.TSN, Pt10b);
  TypeS(152.5, 055.5, GetManDocNum(P_Man.P_DriveSertification), Pt07);

  TypeS(039, 078,                  L_Str.ShortName(Man.Name), Pt10);

  TypeS(061, 132,                  L_Str.ShortName(Man.Name), Pt10);

  SETZERO(sdCnt, guCnt);
  SELECT(WayLMen, RWayS, , RWayS.RWayS);
  WHILE OCCURE(WayLMen) DO
   CONFORM(WayLMen, Man);
   IF    (WayLMen.Who = L_Way.wlmwho_DrPupil) | (WayLMen.Who = L_Way.wlmwho_SDriver) THEN
    IF sdCnt = 0 THEN
     TypeS(100, 059,             L_Str.ShortName(Man.Name), Pt10);
     TypeS(140, 059,                           WayLMen.TSN, Pt10);
     TypeS(154, 059,                           GetManDocNum(P_Man.P_DriveSertification), Pt07);
    END;
    INC(sdCnt);
   ELSIF (WayLMen.Who = L_Way.wlmwho_Guard) | (WayLMen.Who = L_Way.wlmwho_GuPupil) THEN
    IF guCnt <= 1 THEN
     TypeS(102, 063 + 3.5 * guCnt, L_Str.ShortName(Man.Name), Pt10);
     TypeS(140, 063 + 3.5 * guCnt,               WayLMen.TSN, Pt10);
    END;
    INC(guCnt);
   END;
   NEXT(WayLMen);
  END; (* WHILE OCCURE(WayLMen) ... *)

  TypeS(048, 059,   L_BegRWS.GetMechanicName(RWayS.Date), Pt10);

  TypeS(107, 025.5,                            Mark.Name, Pt10);
  TypeS(170, 025.5,                            Mark.Mark, Pt10);
  TypeS(126, 030,                        CarLog.StateNbr, Pt10b);
  TypeS(170, 030,                           RWayS.BoxNbr, Pt10b);

  TypeTruncS(130, 071.5, 30, StExit.Name, Pt08);
  TypeS(163, 071.5,                           RWayS.Exit, Pt10);


  IF P_Car.HasActualCarPro(Car.BoxNbr, RWayS.Date, P_Car.P_CarLicenCard) THEN
   L_Prop.GetDocSerAndNum(CarPro.DocNum, DocSer, DocNum);
   TypeS(114, 081.5,                        CarPro.Value, Pt10);
   TypeS(143, 081.5,                              DocSer, Pt10);
   TypeS(163, 081.5,                              DocNum, Pt10);
  END;

  TypeS(110, 087.5,            L_BegRWS.GetCWOrderList(), Pt08);

  TypeS(239, 115.5,            _NumStr(IncPlan, "2-", 8), Pt08);
  IF L_Way.E_VldPsgPlanLev(RWayS.Date) THEN
   VldPsgPlan:=L_APWI.GetExitVldPsgPlan();
   IF VldPsgPlan # 0 THEN
    TypeS(253, 115.5,       _NumStr(VldPsgPlan, "2.", 8), Pt08);
   END;
  END;

  TypeS(240, 188.5,           _NumStr(StExit.TripAm, "0", 7), Pt08);

  TypeS(145, 127.5, L_Str.ShortName(P_Man.GetFullNameOfCurrentUser(SYSDATE())), Pt10);
  IF (RWayS.Change=0) | (RWayS.Change=1) THEN
   TypeS(110, 119.5,    L_DatTim.LongTimeStr(BLTime), Pt10);
   TypeS(130, 119.5,    L_DatTim.LongTimeStr(ELTime), Pt10);
  ELSE
   TypeS(110, 123.5,    L_DatTim.LongTimeStr(BLTime), Pt10);
   TypeS(130, 123.5,    L_DatTim.LongTimeStr(ELTime), Pt10);
  END;

  TypeS(048, 155,           _NumStr(RWayS.SMBef, "0", 6), Pt08);

  TypeS(244, 043.5,       _NumStr(RWayS.RemExit, "0", 4), Pt10);
  TypeS(245, 077,                 _NumStr(FNorm, "0", 3), Pt10);
  TypeS(243, 096.5,             E_PrintFOver(FOver, 0), Pt10);
  IF ~!AditionalType THEN
   AditionalType();
  END;
  RWSPrt.EndPage();
 END {int_rs_Form_6};

 PUBLIC PROCEDURE rs_Form_6(AditionalType: PROC; wBarCode: SCAL[2]);
  VAR
   AddrArray: STRING[][2];
 BEGIN
  DefaultConforms();
  Str.Format(AddrArray, GetFirmAddr() + " " + GetPhone_DispOrCommon(), 50, LEFT);
  int_rs_Form_6(P_Firm.FP_FirmShortName(Firm.Firm, RWayS.Date), AddrArray, AditionalType, wBarCode);
 END {rs_Form_6};

 PRIVATE PROCEDURE RunStartT(wSDTim: SCAL[2]): TIME;
 BEGIN
  IF wSDTim & SELECT(SEJMStop, Trip, SEJobs.Trip) THEN
   RETURN SEJobs.StartT + SEJMStop.DTim / 60;
  ELSE
   RETURN SEJobs.StartT;
  END;
 END {RunStartT};

 PRIVATE PROCEDURE RunStopT(): TIME;
  VAR
   Result: TIME;
 BEGIN
  SETEMPTY(Result);
  IF (RoutMod.Run > 0) & SELECT(RtMStop, Route, RoutMod.Route, RoutMod.Modif) THEN
   IF RoutMod.Dir = SEJobs.Dir THEN
    LAST(RtMStop);
   END;
   TRY
    Result:=L_Way.GetStopCtrlTime_ofCurSEJobs(RWayS.Date);
   EXCEPT
   ON Exc: Exc_Root DO
    L_StdFrm.RedInform(Exc.MsgOriginalText);
   END;
  END;
  IF !Result | (Result = SEJobs.StartT) THEN
   Result:=SEJobs.StopT;
  END;
  RETURN Result;
 END {RunStopT};

 PRIVATE PROCEDURE NextSEJobsIsBackward(): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  SAVE(SEJobs);
  Result:=NEXT(SEJobs) & (SEJobs.Dir = L_Way.dir_Backward);
  RESTORE(SEJobs);
  RETURN Result;
 END {NextSEJobsIsBackward};

 PUBLIC PROCEDURE ext_us_Form_6(Fill, AllSEJ, wRMName, wSDTim: SCAL[2]);
  CONST
   MaxTrips   = 22;
   FirstLineY = 41;
  VAR
   Pt07,
   Pt09   : TEXTSTYLE;
   i,
   TripCnt: NUMERIC[0];
   y,
   shift  : FLOAT;
   _Run   : LIKE(RoutMod.Run);
 BEGIN
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6_b");
  IF Fill THEN
   CONFORM(RWayS, StExit);
   IF StExit.Strict = L_Way.ExitStrict_Reg THEN
    Pt09:=L_Print.ts_Create(09, "Tahoma");
    Pt07:=L_Print.ts_Resize(Pt09, 07);
    y   :=FirstLineY;
    SETZERO(i, shift, TripCnt);
    SELECT(SEJobs, Exit, , StExit.Exit);
    WHILE OCCURE(SEJobs) DO
     IF AllSEJ | (SEJobs.WJCode = "00") | (SEJobs.WJCode = "10") THEN
      INC(TripCnt);
     END;
     NEXT(SEJobs);
    END; (* WHILE OCCURE(SEJobs) ... *)

    SELECT(SEJobs, Exit, , StExit.Exit);
    LOOP
     IF ~OCCURE(SEJobs) THEN
      EXIT;
     END;
     CONFORM(SEJobs, WJMode);
     IF AllSEJ | (SEJobs.WJCode = "00") | (SEJobs.WJCode = "10") THEN
      IF i = MaxTrips THEN
       shift:=138;
       y    :=FirstLineY;
      END;
      IF i >= 2* MaxTrips THEN
       L_StdFrm.RedInform("⢮ ३ᮢ  室 ॢ蠥 - ப  .|"+
                          " ३   ⠭");
       EXIT;
      END;
      CONFORM(SEJobs, RoutMod);
      IF wRMName THEN
       TypeS(013.5 + shift, y-1.5, Str.Slice(SEJobs.Route + " " + RoutMod.Name,  0, 14), Pt07);
       TypeS(013.5 + shift, y+1.5, Str.Slice(SEJobs.Route + " " + RoutMod.Name, 14, 14), Pt07);
      ELSE
       TypeS(019   + shift, y, SEJobs.Route, Pt09);
      END;
      _Run:=RoutMod.Run;
      TypeS(033.5 + shift, y, L_DatTim.TimeStr(RunStartT(wSDTim)), Pt09);
      IF (TripCnt > 2* MaxTrips) & (SEJobs.Dir = L_Way.dir_Forward) THEN
       IF NextSEJobsIsBackward() THEN
        NEXT(SEJobs);
        CONFORM(SEJobs, RoutMod);
        INC(_Run, RoutMod.Run);
       END;
      END;
      TypeS(076   + shift, y, L_DatTim.TimeStr(RunStopT()),  Pt09);

      IF _Run # 0 THEN
       IF SEJobs.WJCode = "00" THEN
        TypeS(115.5 + shift, y, FLDTOSTR(_Run, "1.", 6, "R"),   Pt09);
       ELSE
        TypeS(103   + shift, y, FLDTOSTR(_Run, "1.", 6, "R"),   Pt09);
       END;
      END;
      INC(i);
      INC(y, 7.5);
     END;
     NEXT(SEJobs);
    END;
   END; (* IF StExit.Strict = L_Way.ExitStrict_Reg ... *)
  END; (* IF Fill ... *)
  RWSPrt.EndPage();
 END {ext_us_Form_6};

 PUBLIC PROCEDURE us_Form_6(Fill: SCAL[2]);
 BEGIN
  ext_us_Form_6(Fill, 0, 0, 1);
 END {us_Form_6};

 PUBLIC PROCEDURE print_Form_6_wAditionalType(FillBackSide, wBarCode: SCAL[2]; AditionalType: PROC);
 BEGIN
  rs_Form_6(AditionalType, wBarCode);
  us_Form_6(FillBackSide);
 END {print_Form_6_wAditionalType};

 PUBLIC PROCEDURE print_Form_6(FillBackSide, wBarCode: SCAL[2]);
 BEGIN
  print_Form_6_wAditionalType(FillBackSide, wBarCode, L_General.EMPTY_PROC());
 END {print_Form_6};

 PUBLIC PROCEDURE printStdForm_6();
 BEGIN
  CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  IF    (L_General.GetAPSiteNumber() = 21) |
        (L_General.GetAPSiteNumber() = 24) THEN
   print_Form_6(0, 1);
  ELSIF (L_General.GetAPSiteNumber() = 50) THEN
   rs_Form_6(L_General.EMPTY_PROC(), 1);
  ELSE
   print_Form_6(1, 1);
  END;
  RWSPrt.Close();
 END {printStdForm_6};

 PUBLIC PROCEDURE ext_us_Form_6_v2(PrintAllSEJ, wCommentRunSelNone: SCAL[2]);
  CONST
   MaxTrips   = 24;
   FirstLineY = 25;
  VAR
   Pt08,
   Pt09,
   Pt10 : TEXTSTYLE;
   i    : NUMERIC[0];
   y,
   shift   : FLOAT;
   curRoute: LIKE(Route.Route);
 BEGIN
  Pt09:=L_Print.ts_Create(09, "Tahoma");
  Pt10:=L_Print.ts_Resize(Pt09, 10);
  Pt08:=L_Print.ts_Resize(Pt09, 08);
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6_b");
  SETZERO(i, shift);
  y:=FirstLineY;
  CONFORM(RWayS, StExit);
  IF StExit.Strict = L_Way.ExitStrict_Reg THEN
   SELECT(SEJobs, Exit, , StExit.Exit);
   CONFORM(StExit, ExitClss);
   LOOP
    IF ~OCCURE(SEJobs) THEN
     EXIT;
    END;
    CONFORM(SEJobs, WJMode);
    CONFORM(SEJobs, RoutMod);
    IF ((i > 0) | (SEJobs.WJCode = "00") | (SEJobs.WJCode = "10")) & (SEJobs.Dir = L_Way.dir_Forward) THEN
     IF i = MaxTrips THEN
      shift:=108;
      y:=FirstLineY;
     END;
     IF i >= 2* MaxTrips THEN
      L_StdFrm.RedInform("⢮ ३ᮢ  室 ॢ蠥 - ப  .|"+
                         " ३   ⠭");
      EXIT;
     END;
     IF i = 0 THEN
      TypeS(009   + shift, y, SEJobs.Route,                          Pt09);
      TypeS(018   + shift, y, ExitClss.Sign,                         Pt09);
      TypeS(033.5 + shift, y, L_Str.PoolItem("1|1|2", RWayS.Change), Pt09);
      curRoute:=SEJobs.Route;
     ELSIF PrintAllSEJ THEN
      IF (SEJobs.WJCode # "00") & (SEJobs.WJCode # "10") THEN
       TypeS(009 + shift, y, Str.Slice(RoutMod.Name, 0, 11),         Pt08);
      ELSIF SEJobs.Route # curRoute THEN
       TypeS(009 + shift, y, SEJobs.Route,                           Pt09);
       curRoute:=SEJobs.Route;
      END;
     END;
     TypeS(061   + shift, y, L_DatTim.TimeStr(SEJobs.StartT),        Pt10);
     IF wCommentRunSelNone & (WJMode.RunSel = L_Way.runsel_None) THEN
      TypeS(079   + shift, y, L_Str.Left(WJMode.Name, 20),           Pt09);
     END;
     INC(i);
     INC(y, 7.5);
    END;
    NEXT(SEJobs);
   END;
  END; (* IF StExit.Strict = ... *)
  RWSPrt.EndPage();
 END {ext_us_Form_6_v2};

 PUBLIC PROCEDURE us_Form_6_v2();
 BEGIN
  ext_us_Form_6_v2(0, 0);
 END {us_Form_6_v2};

 PUBLIC PROCEDURE us_Form_6_v3();
  CONST
   MaxTrips   = 26;
   FirstLineY = 24;
  VAR
   Pt12,
   Pt09,
   Pt10 : TEXTSTYLE;
   i    : NUMERIC[0];
   y,
   shift: FLOAT;
 BEGIN
  Pt09:=L_Print.ts_Create(09, "Tahoma");
  Pt10:=L_Print.ts_Resize(Pt09, 10);
  Pt12:=L_Print.ts_Resize(Pt09, 12);
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6_b");
  y:=FirstLineY;
  SETZERO(shift, i);
  CONFORM(RWayS, StExit);
  IF (StExit.Strict = L_Way.ExitStrict_Reg) THEN
   CONFORM(StExit, ExitClss);
   SELECT(SEJobs, Exit, , StExit.Exit);
   LOOP
    IF ~OCCURE(SEJobs) THEN
     EXIT;
    END;
    CONFORM(SEJobs, WJMode);
    CONFORM(SEJobs, RoutMod);
    IF (SEJobs.WJCode = "00") | (SEJobs.WJCode = "10") THEN
     IF i = MaxTrips THEN
      shift:=112.5;
      y    :=FirstLineY;
     END;
     IF i >= 2 * MaxTrips THEN
      L_StdFrm.RedInform("⢮ ३ᮢ  室 ॢ蠥 - ப  .|"+
                         " ३   ⠭");
      EXIT;
     END;
     TypeS(011   + shift, y, SEJobs.Route,                          Pt09);
     TypeS(019.5 + shift, y, ExitClss.Sign,                         Pt09);
     TypeS(033.5 + shift, y, L_Str.PoolItem("1|1|2", RWayS.Change), Pt09);
     TypeS(040   + shift, y, L_DatTim.TimeStr(SEJobs.StartT),       Pt10);
     TypeS(059   + shift, y, L_DatTim.TimeStr(RunStopT()),          Pt10);
     INC(i);
     INC(y, 7.15);
    END;
    NEXT(SEJobs);
   END;
  END;
  IF StExit.TripAm > 0 THEN
   TypeS(236, 058, _NumStr(StExit.TripAm, "0", 7), Pt12);
  END;
  RWSPrt.EndPage();
 END {us_Form_6_v3};

 PRIVATE PROCEDURE rs_Form_ESM2();
  VAR
   Pt10,
   Pt12   : TEXTSTYLE;
 BEGIN
  DefaultConforms();

  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  Pt10:=L_Print.ts_Create(10, "Tahoma");
  Pt12:=L_Print.ts_Resize(Pt10, 12);
 
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_esm2_f");
  TypeS(123, 028,                           RWayS.RWayS, Pt12);
  TypeS(142, 028,             "(" + RWayS.CopyCnt + ")", Pt10);
  TypeS(256, 040,          L_DatTim.DateStr(RWayS.Date), Pt12);

  PrintStamp(10, 10);
  TypeS(039, 046,                         GetFirmName(), Pt10);
  TypeS(029, 064,                             Mark.Name, Pt10);
  TypeS(165, 064,                       CarLog.StateNbr, Pt10);
  TypeS(033, 072,                              Man.Name, Pt10);

  TypeS(212, 084,                           Column.Name, Pt10);
  TypeS(250, 084,                          RWayS.BoxNbr, Pt10);
  TypeS(235, 084,                             Mark.Mark, Pt10);
  TypeS(268, 084,                             RWayS.TSN, Pt10);

  TypeRWSBarCode(160, 015);
  RWSPrt.EndPage();
 END {rs_Form_ESM2};

 PUBLIC PROCEDURE printStdForm_ESM2();
 BEGIN
  CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  rs_Form_ESM2();
  us_AnyForm("rws_esm2_b");
  RWSPrt.Close();
 END {printStdForm_4P};

 PUBLIC PROCEDURE FullDateStr(D: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(D, "D "+ L_DatTim.MonthStr(D, 1)+" y", 0);
 END {FullDateStr};

 PUBLIC PROCEDURE f6Spec_FillCoupon(Left: NUMERIC[1]; FillCustomer: SCAL[2]);
  VAR
   Pt07,
   Pt09  : TEXTSTYLE;
   StrArr: STRING[][2];
 BEGIN
  Pt07:=L_Print.ts_Create(07, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt07, 09);
  Str.Format(StrArr, P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date), 30, LEFT);
  TypeTruncS(Left + 25, 034, 42, StrArr[0], Pt07);
  TypeTruncS(Left + 25, 036, 42, StrArr[1], Pt07);
  Str.Format(StrArr, GetFirmAddr() + " " + P_Firm.FP_Phone(Firm.Firm, RWayS.Date), 50, LEFT);
  TypeS(Left +  8, 040, StrArr[0], Pt07);
  TypeS(Left +  8, 042, StrArr[1], Pt07);
  TypeS(Left + 20, 050.5, Mark.Name, Pt07);
  TypeS(Left + 53, 060, CarLog.StateNbr, Pt07);
  IF SELECT(CWOrder, RWayS, RWayS.RWayS) THEN
   TypeS(Left + 10, 082, CWOrder.RespPers, Pt09);
   IF FillCustomer & SEARCH(Contract, Contract, CWOrder.Contract) THEN
    SAVE(Firm);
    CONFORM(Contract, Firm);
    Str.Format(StrArr, P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date), 33, LEFT);
    TypeS(Left + 22, 066, StrArr[0], Pt07);
    TypeS(Left + 22, 068, StrArr[1], Pt07);
    Str.Format(StrArr, GetFirmAddr() + " " + P_Firm.FP_Phone(Firm.Firm, RWayS.Date), 50, LEFT);
    TypeS(Left +  8, 073, StrArr[0], Pt07);
    TypeS(Left +  8, 075, StrArr[1], Pt07);
    RESTORE(Firm);
   END;
  ELSIF L_APtm.IsOG() THEN
   TypeS(Left + 24, 067, ' ""', Pt07);
   TypeS(Left + 10, 075, '. , . ᪠, . 26', Pt07);
  END;
 END {f6Spec_FillCoupon};

 PRIVATE PROCEDURE Any_rs_Form_6Spec(FillCustomer: SCAL[2]; AddSpcMark: STRING[]);
  VAR
   i       : NUMERIC[0];
   FNorm,
   FOver   : LIKE(RWayS.FuelNorm);
   Pt07,
   Pt08,
   Pt09,
   Pt10,
   Pt10b   : TEXTSTYLE;
   BLTime,
   ELTime  : L_General.tLongTime;
   Text,
   asExitN,
   DocSer,
   DocNum   : STRING[];
   TextArray: STRING[][4];
   AddrArray: STRING[][10];
   IsFirstRWS: SCAL[2];
 BEGIN
  DefaultConforms();
  GetLongTimes(BLTime, ELTime);
  Pt10:=L_Print.ts_Create(10, "Tahoma");
  Pt09:=L_Print.ts_Resize(Pt10, 09);
  Pt08:=L_Print.ts_Resize(Pt10, 08);
  Pt07:=L_Print.ts_Resize(Pt10, 07);
  Pt10b:=L_Print.ts_Resize(Pt10, 10);
  Pt10b.Font.Weight:=FONT.Weight_Bold;
  TypeRWSBarCode(164, 077);

  PrintPhoto(081, 006);
 
  TypeS(165, 024, RWayS.RWayS + " (" + RWayS.CopyCnt + ")", Pt10b);
  TypeS(113, 034, FullDateStr(RWayS.Date),  Pt10);
  IF (SYSDATE() = RWayS.Date) THEN    (*  ⭮,  ६ 뢮 *)
   TypeS(156, 034, L_DatTim.TimeStr(SYSTIME()),   Pt10);
  END;
  TypeS(198, 035, P_Firm.GetFirmProSoft(Firm.Firm, RWayS.Date, P_Firm.P_OKPO), Pt09);

  TypeTruncS(028, 040, 140, P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date), Pt10);
  TypeS(028, 044.5, GetFirmAddr() + " " + P_Firm.FP_Phone(Firm.Firm, RWayS.Date), Pt09);

  TypeS(270, 021, RWayS.RWayS + " (" + RWayS.CopyCnt + ")", Pt09);
  TypeS(240, 028.5, FullDateStr(RWayS.Date),  Pt09);

  TypeS(025, 063, Man.Name, Pt10);
  TypeS(201, 063, RWayS.TSN, Pt09);
  TypeS(037, 071, GetManDocNum(P_Man.P_DriveSertification), Pt10);
  TypeS(078, 071, P_Man.GetDriveSertification_Category(Man.Man, RWayS.Date), Pt10);

  TypeS(185, 176, L_Str.ShortName(Man.Name), Pt08);
  TypeS(185, 193, L_Str.ShortName(Man.Name), Pt08);

  TypeS(185, 168, L_BegRWS.GetMechanicName(RWayS.Date), Pt08);

  TypeS(033, 054, Mark.Name, Pt10);
  TypeS(202, 054, Mark.Mark, Pt09);
  TypeS(060, 058.5, CarLog.StateNbr, Pt10);
  TypeS(205, 044.5, CarLog.Column, Pt09);
  TypeS(202, 058.5, RWayS.BoxNbr, Pt09);

  IF P_Car.HasActualCarPro(Car.BoxNbr, RWayS.Date, P_Car.P_CarLicenCard) THEN
   L_Prop.GetDocSerAndNum(CarPro.DocNum, DocSer, DocNum);
   TypeS(050, 082, CarPro.Value, Pt10);
   TypeS(095, 082,       DocSer, Pt10);
   TypeS(120, 082,       DocNum, Pt10);
  END;

  SAVE(RWayS);
  CHANGESEL(RWayS, BoxNbr, , RWayS.BoxNbr);
  IsFirstRWS:=~PREV(RWayS);
  RESTORE(RWayS);
  IF ~(L_APtm.IsOG() & (StExit.ExitClss = 999001)) THEN
   TypeS(050, 186, L_Str.ShortName(P_Man.GetFullNameOfCurrentUser(SYSDATE())), Pt08);
   TypeS(030, 108, L_DatTim.LongTimeStr(BLTime), Pt08);
   TypeS(030, 115, L_DatTim.LongTimeStr(ELTime), Pt08);
  END;
  IF ~(IsFirstRWS | L_APtm.IsOG() & (StExit.ExitClss = 999001)) THEN
   TypeS(070, 108, _NumStr(RWayS.SMBef, "0", 6), Pt08);
   TypeS(048, 142, _NumStr(RWayS.RemExit, "0", 4), Pt10);
  END;

  IF AccessMainFuel(RWayS.Date, Mark.Mark) THEN
   TypeS(018, 142, Math.Math, Pt08);
   TypeTruncS(007, 142, 10, Math.Name, Pt08);
  END;
  L_Way.PrepareFuelInf(FNorm, FOver);
  TypeS(045, 164, E_PrintFOver(FOver, 1), Pt10);

  Text:=AddSpcMark;
  SETEMPTY(AddrArray);
  IF SELECT(CWOrder, RWayS, RWayS.RWayS) THEN
   TypeS(162, 108, L_DatTim.TimeStr(CWOrder.p_BTime), Pt10);
   TypeS(176, 108, L_DatTim.TimeStr(CWOrder.p_ETime), Pt10);
   Text:=AddStr(Text, CWOrder.RouteDsc);
   IF FillCustomer & SEARCH(Contract, Contract, CWOrder.Contract) THEN
    SAVE(Firm);
    CONFORM(Contract, Firm);
    Str.Format(AddrArray, P_Firm.FP_PersonFullName(Firm.Firm, RWayS.Date) + L_Str.CRLF +
                          GetFirmAddr() + " " + P_Firm.FP_Phone(Firm.Firm, RWayS.Date), 25, LEFT);
    RESTORE(Firm);
   END;
  END;

  IF FillCustomer THEN
   IF !AddrArray[0] THEN
    asExitN:=StExit.Name;
    IF L_APtm.IsOG() THEN
     IF StExit.ExitClss = 999001 THEN
      CONFORM(Car, am);
      asExitN:=am.Town+"-"+asExitN;
     END;
     asExitN:=StExit.Exit + " " + asExitN;
    END;
    Str.Format(AddrArray, asExitN, 29, LEFT);
   END;
   i:=0;
   WHILE (i <= HIGH(AddrArray)) & ~!AddrArray[i] DO
    TypeS(114, 108 + 3*i, AddrArray[i], Pt08);
    INC(i)
   END;
  END;

  Text:=AddStr(Text, StdSpecialMarks());
  Str.Format(TextArray, Text, 76, LEFT);
  i:=0;
  WHILE (i <= HIGH(TextArray)) & ~!TextArray[i] DO
   TypeS(114, 147 + i*3, TextArray[i], Pt07);
   INC(i);
  END; (* WHILE ~!TextArray[i]...*)

  TypeS(105, 204, "⠭   ⮏", Pt07);

  f6Spec_FillCoupon(220, FillCustomer);
 END {Any_rs_Form_6Spec};

 PUBLIC PROCEDURE rs_AnyForm_6Spec(AddSpcMark: STRING[]);
 BEGIN
  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6spec_f");
  PrintStamp(08, 10);
  Any_rs_Form_6Spec(1, AddSpcMark);
  RWSPrt.EndPage();
 END {rs_AnyForm_6Spec};

 PUBLIC PROCEDURE rs_Form_6Spec();
 BEGIN
  rs_AnyForm_6Spec("");
 END {rs_Form_6Spec};

 PUBLIC PROCEDURE us_Form_6Spec(AditionalType: PROC);
  VAR
   Pt07,
   Pt08,
   Pt10  : TEXTSTYLE;
 BEGIN
  Pt10:=L_Print.ts_Create(10, "Tahoma");
  Pt08:=L_Print.ts_Resize(Pt10, 08);
  Pt07:=L_Print.ts_Resize(Pt10, 07);

  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6spec_b");
  TypeS(048, 021, RWayS.RWayS + " (" + RWayS.CopyCnt + ")", Pt08);
  TypeS(018.5, 028, FullDateStr(RWayS.Date),  Pt10);

  f6Spec_FillCoupon(0, 1);
  IF ~!AditionalType THEN
   AditionalType();
  END;
  RWSPrt.EndPage();
 END {us_Form_6Spec};

 PUBLIC PROCEDURE printStdForm_6Spec();
 BEGIN
  CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  rs_Form_6Spec();
  us_Form_6Spec(EMPTYOF(PROC));
  RWSPrt.Close();
 END {printStdForm_6Spec};

 PRIVATE PROCEDURE rs_Form_6Spec_GC();
 BEGIN
  L_Print.SetDuplexMode(RWSPrt, L_Print.dm_Horizontal);
  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6specGC_f");
  PrintStamp(10, 10);
  Any_rs_Form_6Spec(0, "");
  RWSPrt.EndPage();
 END {rs_Form_6Spec_GC};

 PRIVATE PROCEDURE us_Form_6Spec_GC();
  VAR
   Pt08,
   Pt10  : TEXTSTYLE;
 BEGIN
  Pt10:=L_Print.ts_Create(10, "Tahoma");
  Pt08:=L_Print.ts_Resize(Pt10, 08);

  RWSPrt.StartPage();
  PrintRWaySBackground("rws_f6specGC_b");
  TypeS(050, 021, RWayS.RWayS + " (" + RWayS.CopyCnt + ")", Pt08);
  TypeS(020, 030, FullDateStr(RWayS.Date),  Pt10);
  RWSPrt.EndPage();
 END {us_Form_6Spec_GC};

 PUBLIC PROCEDURE printStdForm_6Spec_GC(); (*Goverment Court*)
 BEGIN
  CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  rs_Form_6Spec_GC();
  us_Form_6Spec_GC();
  RWSPrt.Close();
 END {printStdForm_6Spec_GC};

 PUBLIC PROCEDURE GetRWaySType(): tRWaySType;
 BEGIN
  L_Car.StandOnCarLog(RWayS.BoxNbr, RWayS.Date);
  CONFORM(CarLog, Mark);
  L_General.Assert(~!Mark.VehType, " થ " + Mark.Mark + "  । ⨯ ");
  CONFORM(Mark, VehType);
  L_General.Assert(~!VehType.IRC_VehT, " ⨯  " + VehType.VehType + " " + VehType.Name + "|" +
                   " ' 䨪樨 '   ");
  IF    (VehType.IRC_VehT = L_Car.ircvt_Bus_wUL_UpTo5Year)   |
        (VehType.IRC_VehT = L_Car.ircvt_Bus_wUL_Over5Year)   THEN
   RETURN rwspt_Bus;
  ELSIF (VehType.IRC_VehT = L_Car.ircvt_PassengerCar)        THEN
   RETURN rwspt_PassengerCar;
  ELSIF (VehType.IRC_VehT = L_Car.ircvt_Truck_wUL_UpTo5Year) |
        (VehType.IRC_VehT = L_Car.ircvt_Truck_wUL_Over5Year) THEN
   RETURN rwspt_Truck;
  ELSIF (VehType.IRC_VehT = L_Car.ircvt_SelfPropelledCar)    |
        (VehType.IRC_VehT = L_Car.ircvt_SnowMobile)          |
        (VehType.IRC_VehT = L_Car.ircvt_Motorcycle)          THEN
   RETURN rwspt_SelfPropelledCar;
  ELSE
   RETURN EMPTYOF(tRWaySType);
  END;
 END {GetRWaySType};

 PRIVATE PROCEDURE CreatePrintJob(PrinterType: NUMERIC[0]; Orientation: L_Print.tOrientation);
 BEGIN
  RWSPrt:=L_Print.CreatePrintJob(PrinterType, "⥢  "+RWayS.RWayS, L_Print.pf_A4, Orientation);
 END {CreatePrintJob};

 PUBLIC PROCEDURE CreateNotTypicalPrintJob(Orientation: L_Print.tOrientation; WithBlank: SCAL[2]);
 BEGIN
  IF WithBlank THEN
   CreatePrintJob(L_Print.pt_BlankAndDoc, Orientation);
  ELSE
   CreatePrintJob(L_Print.pt_Doc2Blank,   Orientation);
  END;
 END {CreateNotTypicalPrintJob};

 PUBLIC PROCEDURE PrintTypicalForm(RWaySType: tRWaySType);
 BEGIN
  IF    RWaySType = rwspt_PassengerCar THEN
   CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Portrait);
  ELSE
   CreatePrintJob(L_Print.pt_BlankAndDoc, L_Print.o_Landscape);
  END;
  IF    RWaySType = rwspt_Bus THEN
   print_Form_6(1, 1);
  ELSIF RWaySType = rwspt_PassengerCar THEN
   print_Form_3();
  ELSIF RWaySType = rwspt_SelfPropelledCar THEN
   print_Form_3Spec();
  ELSIF RWaySType = rwspt_Truck THEN
   print_Form_4S();
  ELSIF !RWaySType THEN
   L_General.Error(" ।  ⥢ ");
  ELSE
   L_General.Error("   " + RWaySType + "  ন");
  END;
  RWSPrt.Close();
 END {PrintTypicalForm};

END.
