(*
%Copyright   (c) Polak Computer System
%Author      09.12.2000 15:48:25 SAM
%Purpose     ஫  ᥩ 䠩 Mark
%Type        control
%Specific    AP
%Modified    22.07.2014 15:07:46 POLAK\Gina
%Environment
*)

 MODULE Mark [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_Events, L_Fuel, L_Way, P_Mark, Memo, L_DatTim, L_WDO, L_StdFrm, Str, L_APtm, L_Prop,
        L_SES, L_Str, L_AFM, L_Group, E_Way, L_r2r, L_DBD, L_SysLog, L_AR, L_Rpr;

 PRIVATE PROCEDURE GrantRightsForTempReplication();
 BEGIN
  L_AR.GrantRightsUnconditionally();
 END {GrantRightsForTempReplication};

 PRIVATE PROCEDURE GroupT_Replicate();
 BEGIN
  GrantRightsForTempReplication();
  GroupT.Name:=MarkGT.Name;
  REPLACE(GroupT);
 END {GroupT_Replicate};

 PRIVATE PROCEDURE Group_Replicate();
 BEGIN
  GrantRightsForTempReplication();
  Group.Name :=MarkG.Name;
  Group.Begin:=MarkG.Begin;
  Group.Till :=MarkG.Till;
  REPLACE(Group);
  L_General.Assert(Group.Begin <= Group.Till, "' '  ' '?");
 END {Group_Replicate};

 PRIVATE PROCEDURE GroupE_Replicate();
 BEGIN
  GrantRightsForTempReplication();
  GroupE.CodeOwn:=Mark.Mark;
  GroupE.NameOwn:=Mark.Name;
  GroupE.Begin  :=Mark.Begin;
  GroupE.Till   :=Mark.Till;
  REPLACE(GroupE);
 END {GroupE_Replicate};

 PRIVATE PROCEDURE InsertMarkGroupE();
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   CONFORM(MarkGM, MarkG);
   CONFORM(MarkGM, Mark);
   GrantRightsForTempReplication();
   DEFAULT(GroupE);
   GroupE.Group  :=MarkG.Group;
   SETEMPTY(GroupE.GroupE);
   GroupE.Owner  :=TABLE.Get("Mark").Id;
   GroupE.riOwner:=Mark.RECORDID;
   INSERT(GroupE);
   GroupE_Replicate();
  EXCEPT
  END; (* TRY *)
 END {InsertMarkGroupE};

 PRIVATE PROCEDURE DeleteMarkGroupE();
  VAR
   Cnt: NUMERIC[0];
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   CONFORM(MarkGM, MarkG);
   CONFORM(MarkGM, Mark);
   Cnt:=0;
   WHILE SELECT(GroupE, GriO, MarkG.Group, Mark.RECORDID) DO
    GrantRightsForTempReplication();
    REMOVE(GroupE);
    INC(Cnt);
   END;
   L_General.CallUs(Cnt = 1);
  EXCEPT
  END; (* TRY *)
 END {DeleteMarkGroupE};

 PRIVATE PROCEDURE AcceptableMarkGT(_MarkGT: LIKE(MarkGT.MarkGT)): SCAL[2];
 BEGIN
  IF    Mark.use_Self THEN
   RETURN 1;
  ELSIF Mark.use_OutC THEN
   RETURN _MarkGT = L_Way.mgt_RprJobBaseTime;
  END;
 END {AcceptableMarkGT};

 PRIVATE PROCEDURE CheckMarkGM();
  VAR
   _MarkG : LIKE(MarkG.MarkG);
   Message: STRING[];
 BEGIN
  SETEMPTY(Message);
  SELECT(MarkGT, MarkGT);
  WHILE OCCURE(MarkGT) DO
   IF AcceptableMarkGT(MarkGT.MarkGT) THEN
    SETEMPTY(_MarkG);
    SELECT(MarkGM, Mark, , Mark.Mark);
    WHILE OCCURE(MarkGM) DO
     CONFORM(MarkGM, MarkG);
     IF MarkG.MarkGT = MarkGT.MarkGT THEN
      IF !_MarkG THEN
       _MarkG:= MarkGM.MarkG;
      ELSE
       IF (MarkG.MarkGT # L_Way.mgt_MathCompatibility) &
          (MarkG.MarkGT # L_Way.mgt_RprJobBaseTime) &
          (MarkG.MarkGT # L_Way.mgt_REPlan) THEN
        L_General.Error("ઠ "+ Mark.Mark + "   祭 |⮫쪮   㯯  " + MarkGT.MarkGT);
       END;
      END;
     END;
     NEXT(MarkGM);
    END; (* WHILE OCCURE(MarkGM) ... *)
    IF !_MarkG THEN
     Message:=Message + MarkGT.MarkGT + " " + Str.Align(MarkGT.Name, Str.MaxLength(MarkGT.Name), LEFT) + "|";
    END;
   END;
   NEXT(MarkGT);
  END; (* WHILE OCCURE(MarkGT) ... *)
  IF ~!Message THEN
   L_StdFrm.Choice("ઠ "+ Mark.Mark + "  祭    㯯 :||" + Message);
  END;
 END {CheckMarkGM};

 PRIVATE PROCEDURE CheckMarkFuelInMix(trgInMix: LIKE(MarkFuel.InMix); Mandatory: SCAL[2]; Msg: STRING[]);
  VAR
   BD: DATE;
 BEGIN
  L_General.Assert(Mandatory = SELECT(MrkFl, MrkFl, trgInMix), Msg);
  BD:=Mark.Begin;
  WHILE OCCURE(MrkFl) DO
   IF    BD < MrkFl.Begin THEN
    L_General.Error("  㪠 ⥯த  ⨯ '"+L_Str.PoolItem(L_Fuel.im_Pool, trgInMix)+"'| ਮ  "+L_DatTim.DateStr(BD)+"  "+L_DatTim.DateStr(MrkFl.Begin));
   ELSIF BD > MrkFl.Begin THEN
    L_General.Error("ᥪ ਮ ਬ ⥯த⮢  ⨯ '"+L_Str.PoolItem(L_Fuel.im_Pool, trgInMix)+"'| ਮ  "+L_DatTim.DateStr(MrkFl.Begin)+"  "+L_DatTim.DateStr(BD));
   END;
   BD:=MrkFl.Till;
   NEXT(MrkFl);
  END; (* WHILE OCCURE(MrkFl) ... *)
 END {CheckMarkFuelInMix};

 PRIVATE PROCEDURE CheckMarkFuel();
 BEGIN
  L_AFM.MakeIndex("MrkFl", "InMix: LIKE(MarkFuel.InMix);"+
                           "Begin: LIKE(MarkFuel.Begin);"+
                           "Till : LIKE(MarkFuel.Till);"+
                           "Math : LIKE(MarkFuel.Math)");
 
  IF    SELECT(MarkFuel, Mark, Mark.Mark) THEN
   WHILE OCCURE(MarkFuel) DO
    ADD(MrkFl, MrkFl, , MarkFuel.InMix, MarkFuel.Begin, MarkFuel.Till, MarkFuel.Math);
    NEXT(MarkFuel);
   END; (* WHILE OCCURE(MarkFuel) ... *)

   IF (L_Way.E_BegRWSLev(SYSDATE()) # L_Way.brwsl_Active) &
      (SELECT(MrkFl, MrkFl, L_Fuel.im_SimplyFuel) | SELECT(MrkFl, MrkFl, L_Fuel.im_AddMainFuel)) THEN
    L_StdFrm.Choice(" ४ ६ ᯮ짮|"+
                    "᪮쪮  ⮯");
   END;

   CheckMarkFuelInMix(L_Fuel.im_MainFuel, 1, " 㪠  ⮯  ન !");
   CheckMarkFuelInMix(L_Fuel.im_Heater, Mark.IsolHeat, "ᮮ⢥⢨ ਧ '⮯⥫'| ⮯  ⮯⥫");
  ELSIF ~Mark.Electric THEN
   L_StdFrm.Choice("  ଠ  ⮯!");
  END;
 END {CheckMarkFuel};

 PRIVATE PROCEDURE PrepareBySrcMark(srcMark: LIKE(Mark.Mark));
 BEGIN
  L_DBD.CopyFromSrcRecord("Mark", srcMark);
  REPLACE(Mark);
 END {PrepareBySrcMark};

 PRIVATE PROCEDURE CopyRelatedFile(srcMark: LIKE(Mark.Mark));
  VAR
   srcRI: LIKE(Mark.RECORDID);
 BEGIN
  SAVE(Mark);
  SEARCH(Mark, Mark, , srcMark);
  srcRI:=Mark.RECORDID;
  RESTORE(Mark);
  L_Prop.CopyProRecords("Mark", srcRI, Mark.RECORDID);

  SELECT(MarkFuel, Mark, , srcMark);
  WHILE OCCURE(MarkFuel) DO
   NARROWSEL(MarkFuel, Math);
   PROMPT("஢ ᢥ  ⥯த: " + MarkFuel.Math);
   IF ~SRCINSEL(MarkFuel, LE, Mark.Begin) THEN
    FIRST(MarkFuel);
   END;
   WHILE OCCURE(MarkFuel) DO
    IF Mark.Begin < MarkFuel.Till THEN
     SAVE(MarkFuel);
     MarkFuel.Mark :=Mark.Mark;
     MarkFuel.Begin:=L_DatTim.MaxDATE(MarkFuel.Begin, Mark.Begin);
     INSERT(MarkFuel);
     RESTORE(MarkFuel);
    END;
    NEXT(MarkFuel);
   END; (* WHILE OCCURE(MarkFuel) ... *)
   NEXTINEXTENDEDSEL(MarkFuel, Math);
  END; (* WHILE OCCURE(MarkFuel) ... *)

  SELECT(MarkGM, MMG, , srcMark);
  WHILE OCCURE(MarkGM) DO
   NARROWSEL(MarkGM, MarkG);
   PROMPT("஢ 宦  㯯 ப: " + MarkGM.MarkG);
   WHILE OCCURE(MarkGM) DO
    SAVE(MarkGM);
    MarkGM.Mark:=Mark.Mark;
    INSERT(MarkGM);
    InsertMarkGroupE();
    RESTORE(MarkGM);
    NEXT(MarkGM);
   END; (* WHILE OCCURE(MarkGM) ... *)
   NEXTINEXTENDEDSEL(MarkGM, MarkG);
  END; (* WHILE OCCURE(MarkGM) ... *)

  SELECT(MarkREN, Mark, , srcMark);
  WHILE OCCURE(MarkREN) DO
   NARROWSEL(MarkREN, RepEvent);
   PROMPT("஢ ଠ⨢  ய: " + MarkREN.RepEvent);
   IF ~SRCINSEL(MarkREN, LE, Mark.Begin) THEN
    FIRST(MarkREN);
   END;
   WHILE OCCURE(MarkREN) DO
    IF Mark.Begin <= MarkREN.DateE THEN
     SAVE(MarkREN);
     MarkREN.Mark:=Mark.Mark;
     MarkREN.Date:=L_DatTim.MaxDATE(MarkREN.Date, Mark.Begin);
     INSERT(MarkREN);
     RESTORE(MarkREN);
    END;
    NEXT(MarkREN);
   END; (* WHILE OCCURE(MarkREN) ... *)
   NEXTINEXTENDEDSEL(MarkREN, RepEvent);
  END; (* WHILE OCCURE(MarkREN) ... *)

  SELECT(MRFlNorm, Mark, , srcMark);
  WHILE OCCURE(MRFlNorm) DO
   NARROWSEL(MRFlNorm, Route);
   PROMPT("஢  ⮯: " + MRFlNorm.Route);
   IF ~SRCINSEL(MRFlNorm, LE, Mark.Begin) THEN
    FIRST(MRFlNorm);
   END;
   WHILE OCCURE(MRFlNorm) DO
    IF Mark.Begin < MRFlNorm.Till THEN
     SAVE(MRFlNorm);
     MRFlNorm.Mark :=Mark.Mark;
     MRFlNorm.Begin:=L_DatTim.MaxDATE(MRFlNorm.Begin, Mark.Begin);
     INSERT(MRFlNorm);
     RESTORE(MRFlNorm);
    END;
    NEXT(MRFlNorm);
   END; (* WHILE OCCURE(MRFlNorm) ... *)
   NEXTINEXTENDEDSEL(MRFlNorm, Route);
  END; (* WHILE OCCURE(MRFlNorm) ... *)
 END {CopyRelatedFile};

 PRIVATE PROCEDURE DeleteRelatedFile();
 BEGIN
  WHILE SELECT(MarkPro, Mark, Mark.Mark) DO
   L_Prop.RemoveCurrentPro("MarkPro");
  END; (* WHILE OCCURE(MarkPro) ... *)

  SELECT(MarkGM, MMG, , Mark.Mark);
  WHILE OCCURE(MarkGM) DO
   DeleteMarkGroupE();
   NEXT(MarkGM);
  END; (* WHILE OCCURE(MarkGM) ... *)

  SETSEL(MarkFuel, Mark, Mark.Mark); REMSEL(MarkFuel);
  SETSEL(MarkGM,   MMG,  Mark.Mark); REMSEL(MarkGM);
  SETSEL(MarkREN,  Mark, Mark.Mark); REMSEL(MarkREN);
  SETSEL(MRFlNorm, Mark, Mark.Mark); REMSEL(MRFlNorm);
  SETSEL(MRFlNorm, Mark, Mark.Mark); REMSEL(MRFlNorm);
 END {DeleteRelatedFile};

 PRIVATE PROCEDURE Mark_Begin_OnSuccessfulChange();
  VAR
   srcMark: LIKE(Mark.Mark);
 BEGIN
  srcMark:=L_WDO.GetInsertionSource();
  IF ~!srcMark THEN
   TRY
    GrantRightsForTempReplication();
    L_General.Assert(~!Mark.Mark, " ન 㪠뢠   ।");
    IF ~!GE_PVALUE(Mark.Begin) THEN
     L_StdFrm.Choice("  (ᢥ  ⥯த,  ⮯  ..)|" +
                     " 㤠  ⠢ , 㠫  " + L_DatTim.DateStr(Mark.Begin));
     DeleteRelatedFile();
    END;
    CopyRelatedFile(srcMark);
   EXCEPT
   END; (* TRY *)
  ELSE
   L_Prop.AnyProBeginChange("Mark", GE_PVALUE());
  END;
 END {Mark_Begin_OnSuccessfulChange};

 PRIVATE PROCEDURE Mark_OnBeforeInsert();
  VAR
   srcMark     : LIKE(Mark.Mark);
   trg_use_Self: LIKE(Mark.use_Self);
   trg_use_OutC: LIKE(Mark.use_OutC);
 BEGIN
  srcMark:=L_WDO.GetInsertionSource();
  IF ~!srcMark THEN
   PrepareBySrcMark(srcMark);
  ELSE
   Mark.use_Self:=RTI.Get().CurWindow.OriginalName = "MarkE";
   Mark.use_OutC:=~Mark.use_Self;
   REPLACE(Mark);

   P_Mark.GetDefaultViewMarkScopeMode(trg_use_Self, trg_use_OutC);
   IF Mark.use_OutC & ~trg_use_OutC THEN
    L_General.Error("ଫ ન ஭   ⮫쪮 |"+
                    "⮡ࠦ ப ஭   ᯨ᪥ ப" +
                    L_StdFrm.THR("") +
                    "믮 ன ० ⮡ࠦ ᯨ᪠ ப| 쭮 ");
   END;
  END;
 END {Mark_OnBeforeInsert};

 PRIVATE PROCEDURE ReplicateToCUCopy();
 BEGIN
  IF Mark.Name # GE_PVALUE(Mark.Name) THEN
   TRY [RESTORE_CURSORS(RC_ON)]
    L_AR.GrantRightsUnconditionally();
    SELECT(Car, Mark, , Mark.Mark);
    WHILE OCCURE(Car) DO
     PROMPT(" " + Car.BoxNbr);
     L_Rpr.SetCUCopySerialByAnyCar(Car.CUCopy, Car.StateNbr, Mark.Mark);
     NEXT(Car);
    END; (* WHILE OCCURE(Car) ... *)
 
    SELECT(OutCar, Mark, , Mark.Mark);
    WHILE OCCURE(OutCar) DO
     PROMPT(" " + OutCar.OutCar);
     L_Rpr.SetCUCopySerialByAnyCar(OutCar.CUCopy, OutCar.StateNbr, Mark.Mark);
     NEXT(OutCar);
    END; (* WHILE OCCURE(OutCar) ... *)
   EXCEPT
   END; (* TRY *)
  END;
 END {ReplicateToCUCopy};

 PRIVATE PROCEDURE CheckCategory(OnInsert: SCAL[2]);
 BEGIN
  IF OnInsert | (Mark.Category # GE_PVALUE(Mark.Category)) | (Mark.Electric # GE_PVALUE(Mark.Electric)) THEN
   IF    (Mark.Category = "Tm") | (Mark.Category = "Tb") THEN
    L_General.Assert(Mark.Electric, " ન ⥣ਨ " + Mark.Category + '   㪠 "࠭ᯮ"');
   ELSIF Mark.Electric THEN
    L_StdFrm.CrucialChoice(" ન ⥣ਨ " + Mark.Category + ' 㪠 "࠭ᯮ"', L_APtm.g_Tech);
   END;
  END;
 END {CheckCategory};

 PRIVATE PROCEDURE Mark_OnAfterEdIns();
 BEGIN
  CheckMarkGM();
  IF Mark.use_Self THEN
   CheckMarkFuel();
  END;
  L_DBD.CheckValidity("Mark");
 END {Mark_OnAfterEdIns};

 PRIVATE PROCEDURE Mark_OnAfterEdit();
 BEGIN
  Mark_OnAfterEdIns();
  CheckCategory(0);
  SELECT(GroupE, OwriOwGr, , TABLE.Get("Mark").Id, Mark.RECORDID);
  WHILE OCCURE(GroupE) DO
   GroupE_Replicate();
   NEXT(GroupE);
  END; (* WHILE OCCURE(GroupE) ... *)
  ReplicateToCUCopy();
 END {Mark_OnAfterEdit};

 PRIVATE PROCEDURE Mark_OnAfterInsert();
 BEGIN
  Mark_OnAfterEdIns();
  CheckCategory(1);
 END {Mark_OnAfterInsert};

 PRIVATE PROCEDURE Mark_OnBeforeDelete();
 BEGIN
  TRY [INTEGRITY(INTEGRITY_ON)]
   GrantRightsForTempReplication();
   SETSEL(GroupE, OwriOwGr, TABLE.Get("Mark").Id, Mark.RECORDID);
   REMSEL(GroupE);
   SETSEL(MRFlNorm, Mark, Mark.Mark);
   REMSEL(MRFlNorm);
   SETSEL(MarkREN, Mark, Mark.Mark);
   REMSEL(MarkREN);
  EXCEPT
  END; (* TRY *)
 END {Mark_OnBeforeDelete};

 PRIVATE PROCEDURE MarkGM_OnAfterEdit();
  VAR
   Dummy    : STRING[];
   prvGroup,
   newGroup : LIKE(Group.Group);
   Cnt      : NUMERIC[0];
 BEGIN
  IF ~!MarkGM.MarkG THEN
   Dummy:=P_Mark.GetMarkGName(MarkGM.MarkGT, MarkGM.MarkG);
  END;

  IF MarkGM.MarkG # GE_PVALUE(MarkGM.MarkG) THEN
   CONFORM(MarkGM, Mark);
   SEARCH(MarkG, MarkG, , GE_PVALUE(MarkGM.MarkG));
   prvGroup:=MarkG.Group;
   SEARCH(MarkG, MarkG, , MarkGM.MarkG);
   newGroup:=MarkG.Group;
   Cnt:=0;
   WHILE SELECT(GroupE, GriO, prvGroup, Mark.RECORDID) DO
    GrantRightsForTempReplication();
    GroupE.Group:=newGroup;
    SETEMPTY(GroupE.GroupE);
    REPLACE(GroupE);
    INC(Cnt);
   END;
   L_General.CallUs(Cnt = 1);
  END;
 END {MarkGM_OnAfterEdit};

 PRIVATE PROCEDURE MarkGM_OnAfterInsert();
 BEGIN
  InsertMarkGroupE();
 END {MarkGM_OnAfterInsert};

 PRIVATE PROCEDURE MarkGM_OnBeforeDelete();
 BEGIN
  DeleteMarkGroupE();
 END {MarkGM_OnBeforeDelete};

 PRIVATE PROCEDURE SetMarkScopeFilter();
  VAR
   Title,
   Filter   : STRING[];
   trg_use_Self: LIKE(Mark.use_Self);
   trg_use_OutC: LIKE(Mark.use_OutC);
 BEGIN
  Title:="ન";
   SETEMPTY(Filter);
  P_Mark.GetDefaultViewMarkScopeMode(trg_use_Self, trg_use_OutC);
  IF    trg_use_Self  & ~trg_use_OutC THEN
   Title:=Title + " (ᮡ⢥ )";
   Filter:="(Mark.use_Self = 1)";
  ELSIF ~trg_use_Self & trg_use_OutC THEN
   Title:=Title + " (஭ )";
   Filter:="(Mark.use_OutC = 1)";
  END;
  RTI.Get().CurWindow.Zones(1).PermanentFilter:=Filter;
  RTI.Get().CurWindow.Title                   :=Title;
 END {SetMarkScopeFilter};

 PRIVATE PROCEDURE Mark_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents(1, [L_WDO.e_OnSetSubwindow]);
  SetMarkScopeFilter();
 END {Mark_OnInitializeWindow};

 PRIVATE PROCEDURE MarkE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnBeforeInsert, L_WDO.e_OnAfterMdf]);
  L_WDO.iField       (0, "Mark.Category").Template:=L_Way.mc_Pool;
  L_WDO.iField       (0, "Mark.Category").TemplateIsStrict:=1;
  L_WDO.iScsChange   (0, "Mark.Begin");
  L_WDO.iMayBeEmpty  (0, ["Mark.TosolVol", "Mark.EngVol", "Mark.MaxWeigh", "Mark.OilVolum[3]", "Mark.OilVolum[2]", "Mark.OilVolum[1]", "Mark.OilVolum[0]", "Mark.Comment", "Mark.Till", "Mark.EngPower", "Mark.VehType", "Mark.NetWeigh", "Mark.EnginTyp", "Mark.Category"]);
  L_WDO.iEvents      (1, L_WDO.e_OnAfterMdf);
  L_WDO.iField       (1, "MarkGM.MarkGT").OnBeforeAsk:=1;
  L_WDO.iField       (1, "MarkGM.MarkG").OnBeforeAsk:=1;
  L_WDO.iFieldAskMode(1, "MarkGM.MarkGT", FORM_FIELD.AskMode_Manual, "MarkG", 1);
  L_WDO.iFieldAskMode(1, "MarkGM.MarkG",  FORM_FIELD.AskMode_Manual, "MarkG", 1);
  L_WDO.iField       (1, "MarkGM.MarkGT").First:=1;
  L_WDO.iUnmodifiable(1, "MarkGM.Nbr");
  L_WDO.iProgs       (1, "Mark");
  L_WDO.iEvents      (2, [L_WDO.e_OnAfterEdit, L_WDO.e_OnAfterInsert]);
  L_WDO.iField       (2, "MarkFuel.InMix").Format:=L_Fuel.im_Pool;
  L_WDO.iMayBeEmpty  (2, "MarkFuel.Till");
  L_WDO.iProgs       (2, "Mark");
 END {MarkE_OnInitializeWindow};

 PRIVATE PROCEDURE MarkE_SC_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnBeforeInsert, L_WDO.e_OnAfterMdf]);
  L_WDO.iField       (0, "Mark.Comment").LineBreakEnable:=1;
  L_WDO.iMayBeEmpty  (0, ["Mark.EngPower", "Mark.EngVol", "Mark.EnginTyp", "Mark.Comment", "Mark.Till"]);
  L_WDO.iScsChange   (0, "Mark.Begin");
  L_WDO.iField       (1, "MarkGM.MarkGT").OnBeforeAsk:=1;
  L_WDO.iField       (1, "MarkGM.MarkG").OnBeforeAsk:=1;
  L_WDO.iFieldAskMode(1, "MarkGM.MarkGT", FORM_FIELD.AskMode_Manual, "MarkG", 1);
  L_WDO.iFieldAskMode(1, "MarkGM.MarkG",  FORM_FIELD.AskMode_Manual, "MarkG", 1);
  L_WDO.iField       (1, "MarkGM.MarkGT").First:=1;
  L_WDO.iScsChange   (1, ["MarkGM.MarkG"]);
  L_WDO.iUnmodifiable(1, ["MarkGM.Nbr"]);
  L_WDO.iProgs       (1, "Mark");
 END {MarkE_SC_OnInitializeWindow};

 PRIVATE PROCEDURE MarkGM_MarkGT_OnBeforeAsk();
 BEGIN
  SETEMPTY(L_Way.v_MarkGT);
 END {MarkGM_MarkGT_OnBeforeAsk};

 PRIVATE PROCEDURE MarkGM_MarkG_OnBeforeAsk();
 BEGIN
  IF OCCURE(MarkGM) THEN
   L_General.Assert(~!MarkGM.MarkGT, " ࠭  㯯 ப");
   L_Way.v_MarkGT:=MarkGM.MarkGT;
  ELSE
   SETEMPTY(L_Way.v_MarkGT);
  END;
 END {MarkGM_MarkG_OnBeforeAsk};

 PRIVATE PROCEDURE MarkGM_MarkGT_OnAskPhase1();
 BEGIN
 END {MarkGM_MarkGT_OnAskPhase1};

 PRIVATE PROCEDURE MarkGM_Any_OnAskPhase2();
  VAR
   trgMarkGT: LIKE(MarkGT.MarkGT);
   trgMarkG : LIKE(MarkG.MarkG);
 BEGIN
  IF    RTI.Get().TrgZone.MasterTable.Name = "MarkGT" THEN
   trgMarkGT:=MarkGT.MarkGT;
   SETEMPTY(trgMarkG);
  ELSIF RTI.Get().TrgZone.MasterTable.Name = "MarkG" THEN
   SAVE(MarkG);
   GE_WZD_CURSOR(RTI.Get().TrgZone.Window.Name, RTI.Get().TrgZone.Number, (RTI.Get().TrgZone.MasterTable.Name));
   trgMarkG :=MarkG.MarkG;
   trgMarkGT:=MarkG.MarkGT;
   RESTORE(MarkG);
  ELSIF RTI.Get().TrgZone.MasterTable.Name = "MarkGM" THEN
   SAVE(MarkGM);
   GE_WZD_CURSOR(RTI.Get().TrgZone.Window.Name, RTI.Get().TrgZone.Number, (RTI.Get().TrgZone.MasterTable.Name));
   trgMarkG :=MarkGM.MarkG;
   trgMarkGT:=MarkGM.MarkGT;
   RESTORE(MarkGM);
  ELSE
   L_General.CallUs(0);
  END;
  MarkGM.MarkGT:=trgMarkGT;
  MarkGM.MarkG :=trgMarkG;
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   REPLACE(MarkGM);
  EXCEPT
  ON Exc: Exc_Root DO
   IF L_General.IsUniquenessViolationException(Exc) THEN
    L_General.Error("ઠ " + MarkGM.Mark + " 㦥 祭  㯯 ப " + trgMarkG);
   ELSE
    RAISE();
   END;
  END; (* TRY *)
 END {MarkGM_Any_OnAskPhase2};

 PRIVATE PROCEDURE MarkGM_MarkGT_OnAskPhase2();
 BEGIN
  MarkGM_Any_OnAskPhase2();
 END {MarkGM_MarkGT_OnAskPhase2};

 PRIVATE PROCEDURE MarkGM_MarkG_OnAskPhase1();
 BEGIN
  LIKEMENU(Ask, ". 2");
 END {MarkGM_MarkG_OnAskPhase1};

 PRIVATE PROCEDURE MarkGM_MarkG_OnAskPhase2();
 BEGIN
  MarkGM_Any_OnAskPhase2();
 END {MarkGM_MarkG_OnAskPhase2};

 PRIVATE PROCEDURE MarkGM_MarkGT_OnAskForViewPhase1();
 BEGIN
 END {MarkGM_MarkGT_OnAskForViewPhase1};

 PRIVATE PROCEDURE MarkGM_MarkGT_OnAskForViewPhase2();
 BEGIN
  SETSEL(MarkGM, MMGT, Mark.Mark);
  IF SRCINSEL(MarkGM, GE, MarkGT.MarkGT) THEN
   SE_WZD_POSITION(GE_CURWIN(), GE_CURZONE(), MarkGM);
  END;
 END {MarkGM_MarkGT_OnAskForViewPhase2};

 PRIVATE PROCEDURE MarkGM_MarkG_OnAskForViewPhase1();
 BEGIN
  LIKEMENU(Ask, ". 2");
 END {MarkGM_MarkG_OnAskForViewPhase1};

 PRIVATE PROCEDURE MarkGM_MarkG_OnAskForViewPhase2();
 BEGIN
  GE_WZD_CURSOR(RTI.Get().TrgZone.Window.Name, RTI.Get().TrgZone.Number, (RTI.Get().TrgZone.MasterTable.Name));
  SETSEL(MarkGM, MMG, Mark.Mark);
  IF SRCINSEL(MarkGM, GE, MarkG.MarkG) THEN
   SE_WZD_POSITION(GE_CURWIN(), GE_CURZONE(), MarkGM);
  END;
 END {MarkGM_MarkG_OnAskForViewPhase2};

 PRIVATE PROCEDURE MarkGM_MarkG_OnSuccessfulChange();
  VAR
   Dummy: STRING[];
 BEGIN
  L_General.Assert(~!MarkGM.MarkGT, " ࠭  㯯 ப");
  IF ~!MarkGM.MarkG THEN
   Dummy:=P_Mark.GetMarkGName(MarkGM.MarkGT, MarkGM.MarkG);
   L_General.Assert(L_DatTim.IsActiveInIntvl(Mark.Begin, Mark.Till, MarkG.Begin, MarkG.Till),
                    "㯯 ப    ਮ ⢨ ન");
  END;
 END {MarkGM_MarkG_OnSuccessfulChange};

 PUBLIC PROCEDURE MakeCopy();
 BEGIN
  ACCESS(Mark);
  L_WDO.LaunchInsertBySource(Mark.Mark, "");
 END {MakeCopy};

 PRIVATE PROCEDURE MarkG_OnBeforeInsert();
 BEGIN
  L_General.Assert(GE_CURWIN() # "MPG_WP", "ଫ ந  ᠭ 㯯 ப");
 END {MarkG_OnBeforeInsert};

 PRIVATE PROCEDURE ConvertMGTToERKNNotion(): LIKE(ERKN.Notion);
  VAR
   Result: LIKE(ERKN.Notion);
 BEGIN
  IF    MarkGT.MarkGT = L_Way.mgt_WagePay THEN
   Result:=L_Group.erkn_Mark_WagePay;
  ELSIF MarkGT.MarkGT = L_Way.mgt_TEP THEN
   Result:=L_Group.erkn_Mark_TEP;
  ELSIF MarkGT.MarkGT = L_Way.mgt_Math THEN
   Result:=L_Group.erkn_Mark_Math;
  ELSIF MarkGT.MarkGT = L_Way.mgt_Loss THEN
   Result:=L_Group.erkn_Mark_Loss;
  ELSIF MarkGT.MarkGT = L_Way.mgt_Fuel THEN
   Result:=L_Group.erkn_Mark_Fuel;
  ELSIF MarkGT.MarkGT = L_Way.mgt_OrderIncome THEN
   Result:=L_Group.erkn_Mark_OrderIncome;
  ELSIF MarkGT.MarkGT = L_Way.mgt_REPlan THEN
   Result:=L_Group.erkn_Mark_REPlan;
  ELSIF MarkGT.MarkGT = L_Way.mgt_Revise_wRDO THEN
   Result:=L_Group.erkn_Mark_Revise_wRDO;
  ELSIF MarkGT.MarkGT = L_Way.mgt_RprJobBaseTime THEN
   Result:=L_Group.erkn_Mark_RprJobBaseTime;
  ELSIF MarkGT.MarkGT = L_Way.mgt_MathCompatibility THEN
   Result:=L_Group.erkn_Mark_MathCompatibility;
  ELSIF MarkGT.MarkGT = L_Way.mgt_AccessDriving THEN
   Result:=L_Group.erkn_Mark_AccessDriving;
  ELSIF MarkGT.MarkGT = L_Way.mgt_KIPandKTG THEN
   Result:=L_Group.erkn_Mark_KIPandKTG;
  ELSIF MarkGT.MarkGT = L_Way.mgt_CostAccounting THEN
   Result:=L_Group.erkn_Mark_CostAccounting;
  ELSE
   SETEMPTY(Result);
  END;
  RETURN Result;
 END {ConvertMGTToERKNNotion};

 PRIVATE PROCEDURE StandOnGroupTByMarkGT();
  VAR
   curNotion: L_APtm.tNotion;
   curGroupT: LIKE(GroupT.GroupT);
   Local    : PROC;
 BEGIN
  curNotion:=ConvertMGTToERKNNotion();
  IF ~!curNotion THEN
   L_APtm.AccessERKN("GroupT", curNotion);
   IF SELECT(GroupT, Owner, TABLE.Get("Mark").Id, ERKN.ERKN) THEN
    SAVE(GroupT);
    L_General.CallUs(~NEXT(GroupT));
    RESTORE(GroupT);
   ELSE
    L_Group.InsertGroupT(MarkGT.Name, TABLE.Get("Mark").Id, ERKN.ERKN);
   END;
  ELSE
   Local:=PROCREF(E_Way.MarkGT2GroupT);
   L_General.Assert(~!Local, "   㯯 ப");
   curGroupT:=Local();
   L_General.Assert(~!curGroupT, "   㯯 ப");
   L_General.CallUs(SEARCH(GroupT, GroupT, curGroupT));
  END;
 END {StandOnGroupTByMarkGT};

 PRIVATE PROCEDURE MarkG_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().OnOpenWindow :=1;
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iEvents      (2, [L_WDO.e_OnAfterMdf]);
  L_WDO.iZone        (2).Narrowing:="MarkG.MarkGT=MarkGT.MarkGT";
  L_WDO.iMayBeEmpty  (2, ["MarkG.Till"]);
  L_WDO.iProgs       (2, "Mark");
  L_WDO.iZone        (3).ViewOnly:=1;
  L_WDO.iZone        (3).Narrowing:="MarkGM.MarkG=MarkG.MarkG";
 END {MarkG_OnInitializeWindow};

 PRIVATE PROCEDURE MarkG_OnOpenWindow();
  VAR
   Value: STRING[];
 BEGIN
  SETEMPTY(Value);
  IF ~!RTI.Get().CurWindow.AskSourceField & ~!L_Way.v_MarkGT THEN
   Value:="MarkGT.MarkGT=L_Way.v_MarkGT";
  END;
  RTI.Get().CurWindow.Zones(1).PermanentFilter:=Value;
 END {MarkG_OnOpenWindow};

 PRIVATE PROCEDURE MarkGT_OnAfterEdit();
 BEGIN
  StandOnGroupTByMarkGT();
  GroupT_Replicate();
 END {MarkGT_OnAfterEdit};

 PRIVATE PROCEDURE MarkGT_OnAfterInsert();
 BEGIN
  StandOnGroupTByMarkGT();
 END {MarkGT_OnAfterInsert};

 PRIVATE PROCEDURE MarkGT_OnBeforeDelete();
 BEGIN
  StandOnGroupTByMarkGT();
  TRY [INTEGRITY(INTEGRITY_ON)]
   GrantRightsForTempReplication();
   REMOVE(GroupT);
  EXCEPT
  END; (* TRY *)
 END {MarkGT_OnBeforeDelete};

 PRIVATE PROCEDURE MarkG_OnAfterEdit();
 BEGIN
  L_Group.StandOnGroup(MarkG.Group);
  Group_Replicate();
 END {MarkG_OnAfterEdit};

 PRIVATE PROCEDURE MarkG_OnAfterInsert();
 BEGIN
  CONFORM(MarkG, MarkGT);
  StandOnGroupTByMarkGT();
  GrantRightsForTempReplication();
  DEFAULT(Group);
  Group.Owner  :=GroupT.Owner;
  Group.GroupT :=GroupT.GroupT;
  INSERT(Group);
  MarkG.Group:=Group.Group;
  REPLACE(MarkG);
  Group_Replicate();
 END {MarkG_OnAfterInsert};

 PRIVATE PROCEDURE MarkG_OnBeforeDelete();
 BEGIN
  L_r2r.Owner_OnBeforeDelete("MarkG");
  L_Group.StandOnGroup(MarkG.Group);
  TRY [INTEGRITY(INTEGRITY_ON)]
   GrantRightsForTempReplication();
   REMOVE(Group);
  EXCEPT
  END; (* TRY *)
 END {MarkG_OnBeforeDelete};

 PRIVATE PROCEDURE Mark_1_OnSetSubwindow();
  VAR
   trg_use_Self : LIKE(Mark.use_Self);
   trg_use_OutC : LIKE(Mark.use_OutC);
   srcMark      : LIKE(Mark.Mark);
   SubwindowName: STRING[];
 BEGIN
  srcMark:=L_WDO.GetInsertionSource();
  IF (RTI.Get().CurProcess.Kind = FORM_PROCESS.Kind_Insert) & !srcMark THEN
   P_Mark.GetDefaultViewMarkScopeMode(trg_use_Self, trg_use_OutC);
   IF L_General.IsMemberOf("RprS_4") | (~trg_use_Self & trg_use_OutC) THEN
    SubwindowName:="MarkE_SC";
   ELSE
    SubwindowName:="MarkE";
   END;
  ELSE
   L_General.Assert(ACCESS(Mark), " ⠭ ઠ");
   IF    Mark.use_Self THEN
    SubwindowName:="MarkE";
   ELSIF Mark.use_OutC THEN
    SubwindowName:="MarkE_SC";
   ELSE
    L_General.CallUs(0);
   END;
  END;
  RTI.Get().CurZone.HasVariousSubwindows :=1;
  RTI.Get().CurZone.SubwindowOriginalName:=SubwindowName;
 END {Mark_1_OnSetSubwindow};

 PUBLIC PROCEDURE ChangeMarkScope();
 BEGIN
  L_General.Assert(ACCESS(Mark), " ⠭ ઠ");
  LOOP
   IF STDFORM(L_StdFrm.sf_Choice("|ઠ " + Mark.Mark + " " +Mark.Name +
                                 "|" +
                                 "|%##########################################%" +
                                 "|" +
                                 "|%##########################################%",
                                 "%<뢠  ᯨ᪥ ᮡ⢥ ப>E" +
                                 "%<뢠  ᯨ᪥ ப ஭ >E"),
              "    ",
              ,
              1,
              Mark.use_Self, Mark.use_OutC) THEN
    IF Mark.use_Self | Mark.use_OutC THEN
     REPLACE(Mark);
     EXIT;
    ELSE
     L_StdFrm.RedInform(" 㪠   ન");
    END;
   END;
  END; (* LOOP *)
 END {ChangeMarkScope};

 PUBLIC PROCEDURE SetViewMarkScopeMode();
  VAR
   _use_Self: LIKE(Mark.use_Self);
   _use_OutC: LIKE(Mark.use_OutC);
 BEGIN
  P_Mark.GetDefaultViewMarkScopeMode(_use_Self, _use_OutC);
  LOOP
   IF STDFORM(L_StdFrm.sf_Choice("|%###################################%" +
                                 "|" +
                                 "|%###################################%",
                                 "%<뢠 ન ᮡ⢥ >E" +
                                 "%<뢠 ન ஭ >E"),
              " ன ⮡ࠦ ᯨ᪠ ப ",
              INFORM,
              1,
              _use_Self, _use_OutC) THEN
    IF _use_Self | _use_OutC THEN
     L_SES.PutValueAtCurTrn(L_SES.sit_User, "ViewMarkScopeMode", _use_Self + ";" + _use_OutC);
     EXIT;
    ELSE
     L_StdFrm.RedInform(" ⮡ࠦ  ஥");
    END;
   END;
  END; (* LOOP *)
  SetMarkScopeFilter();
 END {SetViewMarkScopeMode};

 PRIVATE PROCEDURE MPG_WP_OnAfterEdIns();
  VAR
   D: DATE;
 BEGIN
  L_General.Assert(MPG_WP.Begin < MPG_WP.Till, " ਮ ࠭ 砫?");
  CONFORM(MPG_WP, WagePay);
  L_General.Assert(L_DatTim.InPatIntvl(WagePay.Begin, WagePay.Till, MPG_WP.Begin, MPG_WP.Till),
                   "䭠 ⪠ "+WagePay.WagePay+"    ਮ| "+L_DatTim.DateStr(MPG_WP.Begin)+"  "+L_DatTim.DateStr(MPG_WP.Till));
  CHANGESEL(MPG_WP, MarkG, , MPG_WP.MarkG, MPG_WP.Who, MPG_WP.JRgPayT, MPG_WP.RWPay);
  SAVE(MPG_WP);
  D:=MPG_WP.Till;
  IF NEXT(MPG_WP) THEN
   L_General.Assert(MPG_WP.Begin >= D, "ਮ ⢨ ᥪ  ᫥饩 ᨥ");
   IF (MPG_WP.Begin > D) THEN
    L_StdFrm.Inform(" ᠭ ᮯ⠢  " + L_DatTim.DateStr(D) + "  " + L_DatTim.DateStr(MPG_WP.Begin));
   END;
  END;
  RESTORE(MPG_WP);

  D:=MPG_WP.Begin;
  IF PREV(MPG_WP) THEN
   L_General.Assert(MPG_WP.Till <= D, "ਮ ⢨ ᥪ  ।饩 ᨥ");
   IF (MPG_WP.Till < D) THEN
    L_StdFrm.Inform(" ᠭ ᮯ⠢  " + L_DatTim.DateStr(MPG_WP.Till) + "  " + L_DatTim.DateStr(D));
   END;
  END;
 END {MPG_WP_OnAfterEdIns};

 PRIVATE PROCEDURE MPG_WP_OnAfterEdit();
 BEGIN
  MPG_WP_OnAfterEdIns();
 END {MPG_WP_OnAfterEdit};

 PRIVATE PROCEDURE MPG_WP_OnAfterInsert();
 BEGIN
  MPG_WP_OnAfterEdIns();
 END {MPG_WP_OnAfterInsert};

 PRIVATE PROCEDURE CheckMarkFuelPeriodCrossing(ED, BD: DATE);
 BEGIN
  L_General.Assert(ED<=BD, "ᥪ ਮ ਬ ⥯த "+MarkFuel.Math+"| "+L_DatTim.DateStr(BD)+"  "+L_DatTim.DateStr(ED));
 END {CheckMarkFuelPeriodCrossing};

 PRIVATE PROCEDURE MarkFuel_OnAfterEdIns();
  VAR
   curBegin,
   curTill : DATE;
 BEGIN
  L_General.Assert(MarkFuel.Till > MarkFuel.Begin, " 砭 ਮ ਬ ()     砫 ()");
  curBegin:=MarkFuel.Begin;
  curTill :=MarkFuel.Till;
  CHANGESEL(MarkFuel, Mark, , MarkFuel.Mark, MarkFuel.Math);
  SAVE(MarkFuel);
  IF PREV(MarkFuel) THEN
   CheckMarkFuelPeriodCrossing(MarkFuel.Till, curBegin);
  END;
  RESTORE(MarkFuel);
  SAVE(MarkFuel);
  IF NEXT(MarkFuel) THEN
   CheckMarkFuelPeriodCrossing(curTill, MarkFuel.Begin);
  END;
  RESTORE(MarkFuel);
 END {MarkFuel_OnAfterEdIns};

 PRIVATE PROCEDURE MarkFuel_OnAfterEdit();
 BEGIN
  MarkFuel_OnAfterEdIns();
 END {MarkFuel_OnAfterEdit};

 PRIVATE PROCEDURE MarkFuel_OnAfterInsert();
 BEGIN
  MarkFuel_OnAfterEdIns();
 END {MarkFuel_OnAfterInsert};

BEGIN
 L_Events.ExecuteHandler();
END.
