(*
%Copyright   (c) Polak Computer System
%Author      19.07.2000 12:42:59 SAM
%Purpose     ⥪  ࠡ  ᢮⢠ Mark
%Type        library
%Specific    AP
%Modified    08.08.2014 11:40:12 POLAK\GAV
%Environment
*)

 MODULE P_Mark [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC CONST
  P_FMarkHeat              = 1201;
  P_FMarkWarm              = 1202;
  P_PasVolume              = 1203;
  P_WashTime               = 1205;
  P_OrderBonus             = 1206;
  P_FMarkPerUnitWork       = 1212;
  P_FuelNorms              = 1213;
  P_MarkFuelCor            = 5015;

  ovi_Motor           = 0;
  ovi_Transmission    = 1;
  ovi_Hydro           = 2;
  ovi_PlasticGreasing = 3;

  qsa_PasVol_Lim  = 0;
  qsa_PasVol_Sit  = 1;

  mfn_Amount = 12;

 IMPORT L_Prop, L_General, L_DatTim, L_Way, L_Str, Str, L_WDO, L_SES, L_qs, L_r2r, L_ACL;

 PRIVATE CONST
  mfc_Amount    = 12;
  PassVolAmount = 4;
  WorkAmount    = 3;

 PRIVATE TYPE
  tFMPerUnitWork = NUMERIC[6, 2][WorkAmount];
  tRoutesMFCor   = LIKE(Route.Route)[mfc_Amount];
  tMFCors        = NUMERIC[6, 3]    [mfc_Amount];
  tMFuelCorr     = NUMERIC[6, 3][3];
  tPassVol       = NUMERIC[4, 0][PassVolAmount];

 PUBLIC PROCEDURE GetSubWindowName(_Property: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    (_Property = P_FMarkHeat) | (_Property = P_FMarkWarm) THEN
   RETURN "P_1201";
  ELSIF _Property = P_PasVolume THEN
   RETURN "P_1203";
  ELSIF _Property = P_FMarkPerUnitWork THEN
   RETURN "P_1212";
  ELSIF _Property = P_MarkFuelCor THEN
   RETURN "P_5015";
  ELSIF _Property = P_FuelNorms THEN
   RETURN "P_1213";
  ELSE
   RETURN "";
  END;
 END {GetSubWindowName};

 PRIVATE PROCEDURE Decode_PasVolume(    Value  : STRING[];
                                    VAR PassVol: tPassVol);
  VAR
   Idx: SCAL[PassVolAmount];
 BEGIN
  FOR_EACH Idx DO
   PassVol[Idx]:=STRTOFLD(L_Prop.getRES(MarkPro.Value, Idx), NUMERIC);
  END;
 END {Decode_PasVolume};

 PRIVATE PROCEDURE formSet_PasVolume();
  VAR
   Idx    : SCAL[PassVolAmount];
   Fields : FORM_FIELDS;
   PassVol: tPassVol;
 BEGIN
  Decode_PasVolume(MarkPro.Value, PassVol);
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH Idx DO
   Fields(L_WDO.Fld_wIdx("PassVol", Idx)).Value:=PassVol[Idx];
  END;
 END {formSet_PasVolume};

 PRIVATE PROCEDURE Encode_PasVolume(VAR Value  : STRING[];
                                        PassVol: tPassVol);
  VAR
   Idx: SCAL[PassVolAmount];
 BEGIN
  FOR_EACH Idx DO
   L_Prop.putRES(Value, FLDTOSTR(PassVol[Idx], "0", 0));
  END;
 END {Encode_PasVolume};

 PRIVATE PROCEDURE formGet_PasVolume();
  VAR
   Idx    : SCAL[PassVolAmount];
   Fields : FORM_FIELDS;
   PassVol: tPassVol;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH Idx DO
   PassVol[Idx]:=Fields(L_WDO.Fld_wIdx("PassVol", Idx)).Value;
  END;
  Encode_PasVolume(MarkPro.Value, PassVol);
 END {formGet_PasVolume};

 PUBLIC PROCEDURE Check(_Property: LIKE(Property.Property));
 BEGIN
  IF    (_Property = P_WashTime) |
        (_Property = P_OrderBonus) THEN
   L_Prop.CheckNumericValue(MarkPro.Value, "");
  END;
 END {Check};

 PRIVATE PROCEDURE OnBeforeModification();
 BEGIN
  IF MarkPro.Property = P_PasVolume THEN
   L_qs.OnBeforeDelete("MarkPro");
  END;
 END {OnBeforeModification};

 PUBLIC PROCEDURE OnBeforeEdit();
 BEGIN
  OnBeforeModification();
 END {OnBeforeEdit};

 PUBLIC PROCEDURE OnBeforeDelete();
 BEGIN
  OnBeforeModification();
 END {OnBeforeDelete};

 PUBLIC PROCEDURE qsa_Conv2Value(trgAttr: LIKE(qs.Attr); VAR (*sic!*) trgSource: VARIANT): LIKE(qs.Value);
 BEGIN
  IF (trgAttr = qsa_PasVol_Lim) | (trgAttr = qsa_PasVol_Sit) THEN
   RETURN L_Str.FixedIntStr(trgSource, 4);
  ELSE
   L_General.CallUs(0);
  END;
 END {qsa_Conv2Value};

 PRIVATE PROCEDURE OnAfterEdIns();
  VAR
   PassVol: tPassVol;
 BEGIN
  IF MarkPro.Property = P_PasVolume THEN
   Decode_PasVolume(MarkPro.Value, PassVol);
   L_qs.byPro_Add("MarkPro", qsa_PasVol_Lim, qsa_Conv2Value(qsa_PasVol_Lim, PassVol[1]));
   L_qs.byPro_Add("MarkPro", qsa_PasVol_Sit, qsa_Conv2Value(qsa_PasVol_Sit, PassVol[2]));
  END;
 END {OnAfterEdIns};

 PUBLIC PROCEDURE OnAfterInsert();
 BEGIN
  OnAfterEdIns();
 END {OnAfterInsert};

 PUBLIC PROCEDURE OnAfterEdit();
 BEGIN
  OnAfterEdIns();
 END {OnAfterEdit};

 PUBLIC PROCEDURE HasPro(_Mark: LIKE(Mark.Mark); D: DATE; _Property: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN SELECT(MarkPro, Mark, _Mark, _Property) & SRCINSEL(MarkPro, LE, D);
 END {HasPro};

 PUBLIC PROCEDURE HasActualMarkPro(_Mark: LIKE(Mark.Mark); D: DATE; _Property: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPro(_Mark, D, _Property) & (!MarkPro.DocExpD | (D < MarkPro.DocExpD));
 END {HasActualMarkPro};

 PRIVATE PROCEDURE GetProValue(_Mark                : LIKE(Mark.Mark);
                               D                    : DATE;
                               _Property            : LIKE(Property.Property);
                               OnDateSearchCondition: L_Prop.tOnDateSearchCondition;
                               PresenceCondition    : L_Prop.tPresenceCondition): STRING[];
 BEGIN
  IF HasActualMarkPro(_Mark, D, _Property) THEN
   RETURN MarkPro.Value;
  ELSE
   IF PresenceCondition = L_Prop.pc_Strong THEN
    SEARCH(Mark, Mark, , _Mark);
    L_Prop.ShowUndefinedProValueError(TABLE.Get("Mark").Id, Mark.RECORDID, _Property, D);
   END;
   RETURN "";
  END;
 END {GetProValue};

 PUBLIC PROCEDURE GetProValueSoft(_Mark                : LIKE(Mark.Mark);
                                  D                    : DATE;
                                  _Property            : LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetProValue(_Mark, D, _Property, L_Prop.dsc_Normal, L_Prop.pc_Soft);
 END {GetProValueSoft};

 PUBLIC PROCEDURE GetProValueStrong(_Mark    : LIKE(Mark.Mark);
                                    D        : DATE;
                                    _Property: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetProValue(_Mark, D, _Property, L_Prop.dsc_Normal, L_Prop.pc_Strong);
 END {GetProValueStrong};

(*!*) PUBLIC PROCEDURE GetMarkProStrong(_Mark: LIKE(Mark.Mark); D: DATE; _Property: LIKE(Property.Property)): STRING[];
 BEGIN
  IF ~HasActualMarkPro(_Mark, D, _Property) THEN
   SEARCH(Mark, Mark, , _Mark);
   L_Prop.ShowUndefinedProValueError(TABLE.Get("Mark").Id, Mark.RECORDID, _Property, D);
  END;
  RETURN MarkPro.Value;
 END {GetMarkProStrong};

(*!*) PUBLIC PROCEDURE GetMarkProSoft(_Mark: LIKE(Mark.Mark); D: DATE; _Property: LIKE(Property.Property)): STRING[];
  VAR
   _Value: STRING[];
 BEGIN
  IF HasActualMarkPro(_Mark, D, _Property) THEN
   _Value:=MarkPro.Value;
  ELSE
   SETEMPTY(_Value);
  END;
  RETURN _Value;
 END {GetMarkProSoft};

 PRIVATE PROCEDURE Decode_AnyMarkFuelNorm(curValue: STRING[];
                                          VAR Temperatures: NUMERIC[0][];
                                          VAR Norms       : NUMERIC[2][]);
  VAR
   i        : SCAL[mfn_Amount];
   SingleVal: STRING[];
 BEGIN
  SETEMPTY(Temperatures, Norms);
  FOR_EACH i DO
   SingleVal      :=L_Prop.getAIS(curValue, i);
   IF !SingleVal THEN
    EXIT;
   END;
   Temperatures[i]:=STRTOFLD(L_Prop.getRES(SingleVal, 0), NUMERIC);
   Norms[i]       :=STRTOFLD(L_Prop.getRES(SingleVal, 1), NUMERIC);
  END; (* FOR_EACH *)
 END {Decode_AnyMarkFuelNorm};

 PRIVATE PROCEDURE formSet_AnyFuelMarkNorm();
  VAR
   i           : NUMERIC[0];
   Fields      : FORM_FIELDS;
   Temperatures: NUMERIC[0][mfn_Amount];
   Norms       : NUMERIC[2][mfn_Amount];
 BEGIN
  Decode_AnyMarkFuelNorm(MarkPro.Value, Temperatures, Norms);
  Fields:=RTI.Get().CurZone.Fields;
  i:=0;
  WHILE i <= HIGH(Temperatures) DO
   Fields(L_WDO.Fld_wIdx("Temperature", i)).Value:=Temperatures[i];
   Fields(L_WDO.Fld_wIdx("Norm", i)).Value       :=Norms[i];
   INC(i);
  END; (* WHILE i <= HIGH(Temperatures)...*)
 END {formSet_AnyFuelMarkNorm};

 PRIVATE PROCEDURE formSet_FMarkHeat();
 BEGIN
  formSet_AnyFuelMarkNorm();
 END {formSet_FMarkHeat};

 PRIVATE PROCEDURE formSet_FMarkWarm();
 BEGIN
  formSet_AnyFuelMarkNorm();
 END {formSet_FMarkWarm};

 PRIVATE PROCEDURE formGet_AnyFuelMarkNorm();
  VAR
   i             : NUMERIC[0];
   Fields        : FORM_FIELDS;
   SingleItem    : STRING[];
   Temperatures  : NUMERIC[0][mfn_Amount];
   Norms         : NUMERIC[2][mfn_Amount];
   curTemperature: NUMERIC[0];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  SETEMPTY(curTemperature);
  SETZERO(i);
  WHILE i <= HIGH(Temperatures) DO
   Temperatures[i]:=Fields(L_WDO.Fld_wIdx("Temperature", i)).Value;
   Norms[i]       :=Fields(L_WDO.Fld_wIdx("Norm", i)).Value;
   L_General.Assert((!Temperatures[i] = !Norms[i]), "ப " + L_Str.IntStr(i+1) + ": ⥬  ଠ  ६");

   IF ~!Temperatures[i] THEN
    IF ~!curTemperature THEN
     L_General.Assert(curTemperature > Temperatures[i], "ப " + L_Str.IntStr(i+1) + ": ⥬   㪠  浪 뢠");
    END;
    curTemperature:=Temperatures[i];

    SETEMPTY(SingleItem);
    L_Prop.putREV(SingleItem, Temperatures[i]);
    L_Prop.putREV(SingleItem, Norms[i]);
    L_Prop.putAIS(MarkPro.Value, SingleItem);
   END;
   INC(i);
  END; (* WHILE i <= HIGH(Temperatures)...*)
 END {formGet_AnyFuelMarkNorm};

 PRIVATE PROCEDURE formGet_FMarkHeat();
 BEGIN
  formGet_AnyFuelMarkNorm();
 END {formGet_FMarkHeat};

 PRIVATE PROCEDURE formGet_FMarkWarm();
 BEGIN
  formGet_AnyFuelMarkNorm();
 END {formGet_FMarkWarm};

 PRIVATE PROCEDURE ArrayItemMove_AnyFuelMarkNorm(Dir: NUMERIC[0]);
 BEGIN
  L_Prop.ArrayItemMove(Dir, ["Temperature", "Norm"]);
 END {ArrayItemMove};

 PUBLIC PROCEDURE OnPropertyArrayItemMoveUp(trgProperty: LIKE(Property.Property));
 BEGIN
  IF (trgProperty = P_FMarkHeat) | (trgProperty = P_FMarkWarm) THEN
   ArrayItemMove_AnyFuelMarkNorm(-1);
  END;
 END {OnPropertyArrayItemMoveUp};

 PUBLIC PROCEDURE OnPropertyArrayItemMoveDown(trgProperty: LIKE(Property.Property));
 BEGIN
  IF (trgProperty = P_FMarkHeat) | (trgProperty = P_FMarkWarm) THEN
   ArrayItemMove_AnyFuelMarkNorm(+1);
  END;
 END {OnPropertyArrayItemMoveDown};

 PRIVATE PROCEDURE Decode_FMarkPerUnitWork(    Value    : STRING[];
                                           VAR trgRWJPro: LIKE(RWJPro.RWJPro)[WorkAmount];
                                           VAR FNorm    : tFMPerUnitWork);
  VAR
   Idx   : SCAL[WorkAmount];
   Record: STRING[];
 BEGIN
  SETEMPTY(trgRWJPro, FNorm);
  FOR_EACH Idx DO
   Record        :=L_Prop.getAIS(Value, Idx);
   IF !Record THEN
    EXIT;
   END;
   trgRWJPro[Idx]:=STRTOFLD(L_r2r.DecodeReferenceStamp(L_Prop.getRES(Record, 0)), NUMERIC);
   FNorm[Idx]    :=STRTOFLD(L_Prop.getRES(Record, 1), NUMERIC);
  END;
 END {Decode_FMarkPerUnitWork};

 PRIVATE PROCEDURE formSet_FMarkPerUnitWork();
  VAR
   Idx      : SCAL[WorkAmount];
   Fields   : FORM_FIELDS;
   trgRWJPro: LIKE(RWJPro.RWJPro)[WorkAmount];
   FNorm    : tFMPerUnitWork;
 BEGIN
  Decode_FMarkPerUnitWork(MarkPro.Value, trgRWJPro, FNorm);
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH Idx DO
   L_WDO.extCreateFormLink("RWJPro", EMPTYOF(STRING[]), Idx);
   Fields(L_WDO.Fld_wIdx("RWJPro", Idx)).Value:=trgRWJPro[Idx];
   Fields(L_WDO.Fld_wIdx("FNorm",  Idx)).Value:=FNorm[Idx];
  END;
 END {formSet_FMarkPerUnitWork};

 PRIVATE PROCEDURE Encode_FMarkPerUnitWork(VAR Value    : STRING[];
                                               trgRWJPro: LIKE(RWJPro.RWJPro)[WorkAmount];
                                               FNorm    : tFMPerUnitWork);
  VAR
   Idx   : SCAL[WorkAmount];
   Record: STRING[];
 BEGIN
  FOR_EACH Idx DO
   IF ~!trgRWJPro[Idx] THEN
    SETEMPTY(Record);
    L_Prop.putRES(Record, L_r2r.Put("MarkPro", "RWJPro", trgRWJPro[Idx]));
    L_Prop.putREV(Record, FNorm[Idx]);
    L_Prop.putAIS(Value, Record);
   END;
  END;
 END {Encode_FMarkPerUnitWork};

 PRIVATE PROCEDURE formGet_FMarkPerUnitWork();
  VAR
   Idx      : SCAL[WorkAmount];
   Fields   : FORM_FIELDS;
   trgRWJPro: LIKE(RWJPro.RWJPro)[WorkAmount];
   FNorm    : tFMPerUnitWork;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH Idx DO
   trgRWJPro[Idx]:=Fields(L_WDO.Fld_wIdx("RWJPro", Idx)).Value;
   FNorm[Idx]    :=Fields(L_WDO.Fld_wIdx("FNorm",  Idx)).Value;
   L_General.Assert(~!trgRWJPro[Idx] = ~!FNorm[Idx], "⥫  祭 ᫥  ६");
  END;
  Encode_FMarkPerUnitWork(MarkPro.Value, trgRWJPro, FNorm);
 END {formGet_FMarkPerUnitWork};

 PRIVATE PROCEDURE Decode_MarkFuelCor(    curValue : STRING[];
                                      VAR MFuelCorr: tMFuelCorr;
                                      VAR Routes   : tRoutesMFCor;
                                      VAR MFCors   : tMFCors);
  VAR
   i  : NUMERIC[0];
   Idx: SCAL[mfc_Amount];
   s  : STRING[];
 BEGIN
  i:=0;
  WHILE i <= HIGH(MFuelCorr) DO
   MFuelCorr[i]:=STRTOFLD(L_Prop.getRES(curValue, i), NUMERIC);
   INC(i);
  END;
  s:=L_Prop.getRES(curValue, i);
  SETEMPTY(Routes, MFCors);
  FOR_EACH Idx DO
   Routes[Idx]:=L_r2r.DecodeReferenceStamp(L_Prop.getAIS(s, Idx*2));
   IF !Routes[Idx] THEN
    EXIT;
   END;
   MFCors[Idx]:=L_Str.Str2Num(L_Prop.getAIS(s, Idx*2+1), "3.");
  END;
 END {Decode_MarkFuelCor};

 PRIVATE PROCEDURE formSet_MarkFuelCor();
  VAR
   i     : NUMERIC[0];
   Idx   : SCAL[mfc_Amount];
   Fields: FORM_FIELDS;
   Routes: tRoutesMFCor;
   MFCors: tMFCors;
   MFuelCorr: tMFuelCorr;
 BEGIN
  Decode_MarkFuelCor(MarkPro.Value, MFuelCorr, Routes, MFCors);
  Fields:=RTI.Get().CurZone.Fields;
  i:=0;
  WHILE i<=HIGH(MFuelCorr) DO
   Fields(L_WDO.Fld_wIdx("MFuelCorr", i)).Value:=MFuelCorr[i];
   INC(i);
  END;
  FOR_EACH Idx DO
   L_WDO.extCreateFormLink("Route", "", Idx);
   Fields(L_WDO.Fld_wIdx("Route", Idx)).Value:=Routes[Idx];
   Fields(L_WDO.Fld_wIdx("FCor", Idx)).Value :=MFCors[Idx];
  END;
 END {formSet_MarkFuelCor};

 PRIVATE PROCEDURE Decode_FuelNorms(    Value: STRING[];
                                    VAR AlFTim,
                                        AlFRun,
                                        ConTim,
                                        ConRun,
                                        ConRepr: L_Way.tFuelNorms);
 BEGIN
  AlFTim :=STRTOFLD(L_Prop.getRES(Value, 0), NUMERIC);
  AlFRun :=STRTOFLD(L_Prop.getRES(Value, 1), NUMERIC);
  ConTim :=STRTOFLD(L_Prop.getRES(Value, 2), NUMERIC);
  ConRun :=STRTOFLD(L_Prop.getRES(Value, 3), NUMERIC);
  ConRepr:=STRTOFLD(L_Prop.getRES(Value, 4), NUMERIC);
 END {Decode_FuelNorms};

 PRIVATE PROCEDURE formSet_FuelNorms();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Decode_FuelNorms(MarkPro.Value,
                   Fields("AlFTim").Value,
                   Fields("AlFRun").Value,
                   Fields("ConTim").Value,
                   Fields("ConRun").Value,
                   Fields("ConRepr").Value);
 END {formSet_FuelNorms};

 PRIVATE PROCEDURE Encode_FuelNorms(VAR Value  : STRING[];
                                        AlFTim,
                                        AlFRun,
                                        ConTim,
                                        ConRun,
                                        ConRepr: L_Way.tFuelNorms);
 BEGIN
  L_Prop.putREV(Value, AlFTim);
  L_Prop.putREV(Value, AlFRun);
  L_Prop.putREV(Value, ConTim);
  L_Prop.putREV(Value, ConRun);
  L_Prop.putREV(Value, ConRepr);
 END {Encode_FuelNorms};

 PRIVATE PROCEDURE formGet_FuelNorms();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  L_General.Assert(~!Fields("AlFTim" ).Value &
                   ~!Fields("AlFRun" ).Value &
                   ~!Fields("ConTim" ).Value &
                   ~!Fields("ConRun" ).Value &
                   ~!Fields("ConRepr").Value, "室 㪠뢠  ");
  Encode_FuelNorms(MarkPro.Value, Fields("AlFTim" ).Value,
                                  Fields("AlFRun" ).Value,
                                  Fields("ConTim" ).Value,
                                  Fields("ConRun" ).Value,
                                  Fields("ConRepr").Value);
 END {formGet_FuelNorms};

 PUBLIC PROCEDURE SetDataInForm(_Property: LIKE(Property.Property));
 BEGIN
  IF    _Property = P_FMarkHeat THEN
   formSet_FMarkHeat();
  ELSIF _Property = P_FMarkWarm THEN
   formSet_FMarkWarm();
  ELSIF _Property = P_PasVolume THEN
   formSet_PasVolume();
  ELSIF _Property = P_FMarkPerUnitWork THEN
   formSet_FMarkPerUnitWork();
  ELSIF _Property = P_MarkFuelCor THEN
   formSet_MarkFuelCor();
  ELSIF _Property = P_FuelNorms THEN
   formSet_FuelNorms();
  END;
 END {SetDataInForm};

 PRIVATE PROCEDURE formGet_MarkFuelCor();
  VAR
   S,
   trgValue: STRING[];
   Idx     : SCAL[mfc_Amount];
   Fields  : FORM_FIELDS;
   curRoute: LIKE(Route.Route);
   curFCor : NUMERIC[3];
   i        : NUMERIC[0];
   MFuelCorr: tMFuelCorr;
 BEGIN
  SETEMPTY(trgValue, S);
  Fields:=RTI.Get().CurZone.Fields;
  i:=0;
  WHILE i <= HIGH(MFuelCorr) DO
   curFCor:=Fields(L_WDO.Fld_wIdx("MFuelCorr", i)).Value;
   L_General.Assert(~!curFCor,  "室 㪠뢠  ࠢ");
   L_Prop.putREV(trgValue, curFCor);
   INC(i);
  END;
  FOR_EACH Idx DO
   curRoute:=Fields(L_WDO.Fld_wIdx("Route", Idx)).Value;
   curFCor :=Fields(L_WDO.Fld_wIdx("FCor", Idx)).Value;
   L_General.Assert(!curRoute = !curFCor, "  樥 ᫥  ६");
   IF ~!curRoute THEN
    L_General.Assert(L_Way.E_GettingRouteFuelNormManner(MarkPro.Date) # 0, "樥  ⠬  ਬ  "+L_DatTim.DateStr(MarkPro.Date)+",|..  ।  ⠬");
    L_Prop.putAIS(S, L_r2r.Put("MarkPro", "Route", curRoute));
    L_Prop.putAIS(S, FLDTOSTR(curFCor));
   END;
  END;
  L_Prop.putREV(trgValue, S);
  MarkPro.Value:=trgValue;
 END {formGet_MarkFuelCor};

 PUBLIC PROCEDURE GetDataFromForm(_Property: LIKE(Property.Property));
 BEGIN
  SETEMPTY(MarkPro.Value);
  IF    _Property = P_FMarkHeat THEN
   formGet_FMarkHeat();
  ELSIF _Property = P_FMarkWarm THEN
   formGet_FMarkWarm();
  ELSIF _Property = P_PasVolume THEN
   formGet_PasVolume();
  ELSIF _Property = P_FMarkPerUnitWork THEN
   formGet_FMarkPerUnitWork();
  ELSIF _Property = P_MarkFuelCor THEN
   formGet_MarkFuelCor();
  ELSIF _Property = P_FuelNorms THEN
   formGet_FuelNorms();
  ELSE
   L_General.CallUs(0);
  END;
  REPLACE(MarkPro);
 END {GetDataFromForm};

 PUBLIC PROCEDURE GetP_PasVol(trgMark: LIKE(Mark.Mark); D: DATE): NUMERIC[0];
  VAR
   PassVol: tPassVol;
 BEGIN
  Decode_PasVolume(GetProValueStrong(trgMark, D, P_PasVolume), PassVol);
  RETURN PassVol[0];
 END {GetP_PasVol};

 PUBLIC PROCEDURE GetP_PVLim(trgMark: LIKE(Mark.Mark); D: DATE): NUMERIC[0];
  VAR
   PassVol: tPassVol;
 BEGIN
  Decode_PasVolume(GetProValueStrong(trgMark, D, P_PasVolume), PassVol);
  RETURN PassVol[1];
 END {GetP_PVlim};

 PUBLIC PROCEDURE GetP_PVSitting(trgMark: LIKE(Mark.Mark); D: DATE): NUMERIC[0];
  VAR
   PassVol: tPassVol;
 BEGIN
  Decode_PasVolume(GetProValueStrong(trgMark, D, P_PasVolume), PassVol);
  RETURN PassVol[2];
 END {GetP_PVSitting};

 PUBLIC PROCEDURE GetP_PVInvalid(trgMark: LIKE(Mark.Mark); D: DATE): NUMERIC[0];
  VAR
   PassVol: tPassVol;
 BEGIN
  Decode_PasVolume(GetProValueStrong(trgMark, D, P_PasVolume), PassVol);
  RETURN PassVol[3];
 END {GetP_PVInvalid};

 PUBLIC PROCEDURE GetP_AlFTim(trgMark: LIKE(Mark.Mark); D: DATE): L_Way.tFuelNorms;
  VAR
   AlFTim,
   AlFRun,
   ConTim,
   ConRun,
   ConRepr: L_Way.tFuelNorms;
 BEGIN
  Decode_FuelNorms(GetProValueStrong(trgMark, D, P_FuelNorms),
                   AlFTim,
                   AlFRun,
                   ConTim,
                   ConRun,
                   ConRepr);
  RETURN AlFTim;
 END {GetP_AlFTim};

 PUBLIC PROCEDURE GetP_AlFRun(trgMark: LIKE(Mark.Mark); D: DATE): L_Way.tFuelNorms;
  VAR
   AlFTim,
   AlFRun,
   ConTim,
   ConRun,
   ConRepr: L_Way.tFuelNorms;
 BEGIN
  Decode_FuelNorms(GetProValueStrong(trgMark, D, P_FuelNorms),
                   AlFTim,
                   AlFRun,
                   ConTim,
                   ConRun,
                   ConRepr);
  RETURN AlFRun;
 END {GetP_AlFRun};

 PUBLIC PROCEDURE GetP_FConTim(trgMark: LIKE(Mark.Mark); D: DATE): L_Way.tFuelNorms;
  VAR
   AlFTim,
   AlFRun,
   ConTim,
   ConRun,
   ConRepr: L_Way.tFuelNorms;
 BEGIN
  Decode_FuelNorms(GetProValueStrong(trgMark, D, P_FuelNorms),
                   AlFTim,
                   AlFRun,
                   ConTim,
                   ConRun,
                   ConRepr);
  RETURN ConTim;
 END {GetP_FConTim};

 PUBLIC PROCEDURE GetP_FConRun(trgMark: LIKE(Mark.Mark); D: DATE): L_Way.tFuelNorms;
  VAR
   AlFTim,
   AlFRun,
   ConTim,
   ConRun,
   ConRepr: L_Way.tFuelNorms;
 BEGIN
  Decode_FuelNorms(GetProValueStrong(trgMark, D, P_FuelNorms),
                   AlFTim,
                   AlFRun,
                   ConTim,
                   ConRun,
                   ConRepr);
  RETURN ConRun;
 END {GetP_FConRun};

 PUBLIC PROCEDURE GetP_FConRepr(trgMark: LIKE(Mark.Mark); D: DATE): L_Way.tFuelNorms;
  VAR
   AlFTim,
   AlFRun,
   ConTim,
   ConRun,
   ConRepr: L_Way.tFuelNorms;
 BEGIN
  Decode_FuelNorms(GetProValueStrong(trgMark, D, P_FuelNorms),
                   AlFTim,
                   AlFRun,
                   ConTim,
                   ConRun,
                   ConRepr);
  RETURN ConRepr;
 END {GetP_FConRepr};

 PUBLIC PROCEDURE GetMarkGName(trgMarkGT: LIKE(MarkGT.MarkGT);
                               trgMarkG : LIKE(MarkG.MarkG)): STRING[];
  VAR
   trgMarkGTName: STRING[];
 BEGIN
  IF SEARCH(MarkG, MGTG, trgMarkGT, trgMarkG) THEN
   RETURN MarkG.Name;
  ELSE
   SETEMPTY(trgMarkGTName);
   IF SEARCH(MarkGT, MarkGT, trgMarkGT) THEN
    trgMarkGTName:="'" + MarkGT.Name + "'";
   END;
   L_StdFrm.WrongDB("  㯯 ப " + trgMarkGT + " " + trgMarkGTName + "|" +
                    " 㯯 ப   " + trgMarkG);
  END;
 END {GetMarkGName};

 PUBLIC PROCEDURE GetP_FMarkPUWork(trgMark: LIKE(Mark.Mark); D: DATE; trgRun: LIKE(CarRun.Run)): LIKE(RWayJ.FNorm);
  VAR
   Idx      : SCAL[WorkAmount];
   Value    : STRING[];
   trgFNorm : tFMPerUnitWork;
   trgRWJPro: LIKE(RWJPro.RWJPro)[WorkAmount];
   Result   : NUMERIC[4];
 BEGIN
  Result:=0;
  Value    :=GetProValueSoft(trgMark, D, P_FMarkPerUnitWork);
  IF ~!Value THEN
   Decode_FMarkPerUnitWork(Value, trgRWJPro, trgFNorm);
   FOR_EACH Idx DO
    IF ~!trgRWJPro[Idx] THEN
     IF trgRWJPro[Idx] = L_Way.rwjp_CargoTonnage THEN
      INC(Result, L_Way.E_GetCargoTonnage() * trgFNorm[Idx] / 100 * trgRun);
     ELSE
      INC(Result, L_Way.GetRWJPByCode(trgRWJPro[Idx]) * trgFNorm[Idx]);
     END;
    ELSE
     EXIT;
    END;
   END;
  END;
  IF Result > MAXOF(RWayJ.FNorm) THEN
   L_StdFrm.WrongDB("९ 祭   ⥫쭮  ⮯  ન " + trgMark  + ".||" +
                    "⠭ ଠ ⮯ " + FLDTOSTR(Result, "4."));
  END;
  RETURN Result;
 END {GetP_FMarkPUWork};

 PUBLIC PROCEDURE GetDefaultViewMarkScopeMode(VAR trg_use_Self: LIKE(Mark.use_Self);
                                              VAR trg_use_OutC: LIKE(Mark.use_OutC));
  VAR
   ViewMarkScopeMode: STRING[];
 BEGIN
  trg_use_Self:=1;
  trg_use_OutC:=1;
  IF L_SES.GetValue(L_SES.sit_User, "ViewMarkScopeMode", ViewMarkScopeMode) THEN
   trg_use_Self:=Str.Item(ViewMarkScopeMode, ";", 0);
   trg_use_OutC:=Str.Item(ViewMarkScopeMode, ";", 1);
  END;
 END {GetDefaultViewMarkScopeMode};

 PUBLIC PROCEDURE GetVal2Show(): STRING[];
  VAR
   Routes      : tRoutesMFCor;
   MFCors      : tMFCors;
   Idx         : SCAL[mfc_Amount];
   Record,
   Result      : STRING[];
   i           : NUMERIC[0];
   MFuelCorr   : tMFuelCorr;
   Temperatures: NUMERIC[0][mfn_Amount];
   Norms       : NUMERIC[2][mfn_Amount];
   trgRWJPro   : LIKE(RWJPro.RWJPro)[WorkAmount];
   FNorm       : tFMPerUnitWork;
   j           : SCAL[WorkAmount];
 BEGIN
  SETEMPTY(Result);
  IF    (MarkPro.Property = P_FMarkHeat) | (MarkPro.Property = P_FMarkWarm) THEN
   Decode_AnyMarkFuelNorm(MarkPro.Value, Temperatures, Norms);
   i:=0;
   WHILE i <= HIGH(Temperatures) DO
    IF ~!Temperatures[i] THEN
     L_Prop.putRES(Result, FLDTOSTR(Temperatures[i], "0", 0) + " " + FLDTOSTR(Norms[i], "2.", 0))
    END;
    INC(i);
   END; (* WHILE i <= HIGH(Temperatures)...*)
  ELSIF MarkPro.Property = P_MarkFuelCor THEN
   Decode_MarkFuelCor(MarkPro.Value, MFuelCorr, Routes, MFCors);
   i:=0;
   WHILE i <= HIGH(MFuelCorr) DO
    L_Prop.putREV(Result, MFuelCorr[i]);
    INC(i);
   END;
   FOR_EACH Idx DO
    IF ~!Routes[Idx] THEN
     INC(Result, " " + Routes[Idx]);
     INC(Result, "," + FLDTOSTR(MFCors[Idx], "3.", 0) + ";");
    END;
   END;
  ELSIF MarkPro.Property = P_FMarkPerUnitWork THEN
   Decode_FMarkPerUnitWork(MarkPro.Value, trgRWJPro, FNorm);
   FOR_EACH j DO
    IF ~!trgRWJPro[j] THEN
     SETEMPTY(Record);
     L_Prop.putREV(Record, trgRWJPro[j]);
     L_Prop.putREV(Record, FNorm[j]);
     L_Prop.putAIS(Result, Record);
    END;
   END;
  ELSE
   Result:=MarkPro.Value;
  END;
  RETURN Result;
 END {GetVal2Show};

 PRIVATE PROCEDURE GetAnyMarkFuelCor(trgMark        : LIKE(Mark.Mark);
                                     trgDate        : DATE;
                                     MarkFuelCorKind: NUMERIC[0]): NUMERIC[3];
  VAR
   Result     : NUMERIC[3];
   MFuelCorr  : tMFuelCorr;
   dummyRoutes: tRoutesMFCor;
   dummyMFCors: tMFCors;
 BEGIN
  Decode_MarkFuelCor(GetProValueSoft(trgMark, trgDate, P_MarkFuelCor), MFuelCorr, dummyRoutes, dummyMFCors);
  Result:=MFuelCorr[MarkFuelCorKind];
  IF !Result THEN
   Result:=1;
  END;
  RETURN Result;
 END {GetAnyMarkFuelCor};

 PUBLIC PROCEDURE Get_MarkFuelCor_Move(trgMark: LIKE(Mark.Mark);
                                       trgDate: DATE): NUMERIC[3];
 BEGIN
  RETURN GetAnyMarkFuelCor(trgMark, trgDate, 0);
 END {Get_MarkFuelCor_Move};

 PUBLIC PROCEDURE Get_MarkFuelCor_Heat(trgMark: LIKE(Mark.Mark);
                                       trgDate: DATE): NUMERIC[3];
 BEGIN
  RETURN GetAnyMarkFuelCor(trgMark, trgDate, 1);
 END {Get_MarkFuelCor_Heat};

 PUBLIC PROCEDURE Get_MarkFuelCor_Warm(trgMark: LIKE(Mark.Mark);
                                       trgDate: DATE): NUMERIC[3];
 BEGIN
  RETURN GetAnyMarkFuelCor(trgMark, trgDate, 2);
 END {Get_MarkFuelCor_Warm};

 PUBLIC PROCEDURE Get_MarkFuelCor_ByRoute(trgMark : LIKE(Mark.Mark);
                                          trgDate : DATE;
                                          trgRoute: LIKE(Route.Route)): NUMERIC[3];
  VAR
   i        : NUMERIC[0];
   Result   : NUMERIC[3];
   Routes   : tRoutesMFCor;
   MFCors   : tMFCors;
   MFuelCorr: tMFuelCorr;
 BEGIN
  Decode_MarkFuelCor(GetProValueSoft(trgMark, trgDate, P_MarkFuelCor), MFuelCorr, Routes, MFCors);
  SETEMPTY(Result);
  i:=0;
  WHILE (i <= HIGH(Routes)) & !Result DO
   IF Routes[i] = trgRoute THEN
    Result:=MFCors[i];
   END;
   INC(i);
  END; (* WHILE (i <= HIHG(Routes)) &...*)
  IF !Result THEN
   Result:=1;
  END;
  RETURN Result;
 END {Get_MarkFuelCor_ByRoute};

 PUBLIC PROCEDURE Get_AnyMarkFuelNorm_ByTemp(trgMark    : LIKE(Mark.Mark);
                                             trgDate    : DATE;
                                             trgProperty: LIKE(Property.Property);
                                             trgTemp    : NUMERIC[2]): NUMERIC[2];
  VAR
   i           : NUMERIC[0];
   Temperatures: NUMERIC[0][mfn_Amount];
   Norms       : NUMERIC[2][mfn_Amount];
 BEGIN
  Decode_AnyMarkFuelNorm(GetProValueStrong(trgMark, trgDate, trgProperty), Temperatures, Norms);
  i:=HIGH(Temperatures);
  WHILE i >= 0 DO
   IF ~!Temperatures[i] & (trgTemp <= Temperatures[i]) THEN
    RETURN Norms[i];
   END;
   DEC(i);
  END; (* WHILE i >= 0...*)
  RETURN 0;
 END {Get_AnyMarkFuelNorm_ByTemp};

END.
