(*
%Copyright   (c) Polak IT
%Author      22.06.2010 18:30:10 LDA
%Purpose
%Type        control
%Specific    AP
%Modified    25.07.2014 11:45:35 POLAK\LDA
%Environment
*)

 MODULE StaffSMS [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_Events, L_Str, L_DatTim, L_WDO, L_StdFrm, L_EMail, P_OwnData, L_ctc, L_EC, L_Man,
        L_APtm, P_Man, P_OwnData, L_BegRWS, Str, Memo, L_OwnData, L_two, L_Way, L_AFM, L_Env, E_Utils,
        L_Staff, L_AcsPnt, L_Pool;

 PRIVATE CONST
  src_System   = 0;
  src_Operator = 1;

  src_Pool       = "⥬|";
  src_Pool_Short = "|";

  st_Prepared = 0;
  st_Sent     = 1;

  st_Pool       = "⮢|ࠢ";
  st_Pool_Short = "|";

  CheckPeriodLength = 12;

 PRIVATE VAR
  DEBUG: SCAL[2];

 PRIVATE PROCEDURE GetCheckPeriodDTS(): TIMESTAMP;
 BEGIN
  RETURN SYSTIMESTAMP() - CheckPeriodLength / 24;
 END {GetCheckPeriodDTS};

 PRIVATE PROCEDURE E_GetSendParams(VAR SMTPServer : STRING[];
                                   VAR EMailDomain: STRING[];
                                   VAR FrContact  : L_EMail.tContact;
                                   VAR Login      : STRING[];
                                   VAR Password   : STRING[]);
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Utils.StaffSMS_GetSendParams);
  L_General.Assert(~!Local, "㭪樮쭮 'SMS-饭'  ஥");
  Local(SMTPServer, EMailDomain, FrContact, Login, Password);
 END {E_GetSendParams};

 PRIVATE PROCEDURE CheckSMSSendAllowed();
  VAR
   Dummy: STRING[];
 BEGIN
  E_GetSendParams(Dummy, Dummy, Dummy, Dummy, Dummy);
 END {CheckSMSSendAllowed};

 PRIVATE PROCEDURE CheckStaffSMSModificationAllowed(IsBeforeDelete: SCAL[2]);
 BEGIN
  L_General.Assert(StaffSMS.Status # st_Sent,
                   "SMS  ﭨ '" + L_Str.PoolItem(st_Pool, StaffSMS.Status) + "',  ");
  IF ~IsBeforeDelete THEN
   L_General.Assert(StaffSMS.Source = src_Operator,
                    "SMS  筨 '" + L_Str.PoolItem(src_Pool, StaffSMS.Source) + "',  ");
  END;
 END {CheckStaffSMSModificationAllowed};

 PRIVATE PROCEDURE SetStaffSMSFieldModifiable();
 BEGIN
  RTI.Get().CurZone.Fields("StaffSMS.TSN").Modifiable:=L_WDO.getParentWindowName() # "StaffSMS";
 END {SetStaffSMSFieldModifiable};

 PRIVATE PROCEDURE CalcStaffSMSTextLength();
 BEGIN
  RTI.Get().CurZone.Fields("TextLength").Value:=Str.Length(StaffSMS.Text);
 END {CalcStaffSMSTextLength};

 PRIVATE PROCEDURE StaffSMS_Text_OnSuccessfulChange();
 BEGIN
  CalcStaffSMSTextLength();
 END {StaffSMS_Text_OnSuccessfulChange};

 PRIVATE PROCEDURE StaffSMS_OnBeforeEdit();
 BEGIN
  CheckStaffSMSModificationAllowed(0);
  SetStaffSMSFieldModifiable();
  CalcStaffSMSTextLength();
 END {StaffSMS_OnBeforeEdit};

 PRIVATE PROCEDURE GetStaffClrPhone(_TSN: LIKE(Staff.TSN); D: DATE): STRING[];
  VAR
   ClrPhone: STRING[];
 BEGIN
  SETEMPTY(ClrPhone);
  SEARCH(Staff, TSN, , _TSN);
  ClrPhone:=P_Man.GetAnyClrPhone(Staff.Man, D);
  L_General.Assert(~!ClrPhone, " ࠡ⭨ " + Staff.TSN + "  㪠  쭮 ⥫䮭  " + L_DatTim.DateStr(D));
  ClrPhone:=L_ctc.ObtainClearPhoneNumber(ClrPhone);
  L_General.Assert(Str.Length(ClrPhone) = 11, " ࠡ⭨ " + Staff.TSN + " ४  쭮 ⥫䮭 " + ClrPhone);
  RETURN ClrPhone;
 END {GetStaffClrPhone};

 PRIVATE PROCEDURE SetStaffClrPhone();
 BEGIN
  StaffSMS.ClrPhone:=GetStaffClrPhone(StaffSMS.TSN, SYSDATE());
  REPLACE(StaffSMS);
 END {SetStaffClrPhone};

 PRIVATE PROCEDURE StaffSMS_OnBeforeInsert();
  VAR
   D          : DATE;
   SentAmount,
   ReadyAmount: NUMERIC[0];
 BEGIN
  CheckSMSSendAllowed();
  SetStaffSMSFieldModifiable();
  D:=SYSDATE();
  SETZERO(SentAmount, ReadyAmount);
  SAVE(StaffSMS);
  SETSEL(StaffSMS, TSN, StaffSMS.TSN);
  SRCINSEL(StaffSMS, GE, , GetCheckPeriodDTS());
  WHILE OCCURE(StaffSMS) & ~!StaffSMS.DTS & (StaffSMS.DTS > GetCheckPeriodDTS()) DO
   IF StaffSMS.Status = st_Sent THEN
    IF StaffSMS.Source = src_Operator THEN
     INC(SentAmount);
    END;
   ELSE
    INC(ReadyAmount);
   END;
   NEXT(StaffSMS);
  END; (* WHILE OCCURE(StaffSMS) ... *)
  RESTORE(StaffSMS);

  L_General.Assert(ReadyAmount = 0, " ࠡ⭨  ࠢ SMS. ଫ  SMS ");

  IF SentAmount > 0 THEN
   L_StdFrm.Question("⭨ " + StaffSMS.TSN + "  ᫥ " + L_Str.IntStr(CheckPeriodLength) + " . 㦥 ࠢ " + L_Str.IntStr(SentAmount) + " SMS.|" +
                     "   SMS?");
  END;

  StaffSMS.DTS     :=SYSTIMESTAMP();
  StaffSMS.AcsObj  :=L_General.curUSER;
  StaffSMS.Source  :=src_Operator;
  StaffSMS.Status  :=st_Prepared;
  REPLACE(StaffSMS);

  IF ~!StaffSMS.TSN THEN
   SetStaffClrPhone();
  END;
  CalcStaffSMSTextLength();
 END {StaffSMS_OnBeforeInsert};

 PRIVATE PROCEDURE StaffSMS_OnBeforeDelete();
 BEGIN
  CheckStaffSMSModificationAllowed(1);
 END {StaffSMS_OnBeforeDelete};

 PRIVATE PROCEDURE CheckAcceptableTimeMassSend();
  CONST
   atms_End = 17:00;
 BEGIN
  L_General.Assert_wReferToAdmin((SYSTIME() >= L_Way.DtwoReadyTime) & (SYSTIME() <= atms_End),
                                 "ᮢ 뫪 SMS ࠧ襭||" +
                                 " ਮ  " + L_DatTim.TimeStr(L_Way.DtwoReadyTime) + "  " + L_DatTim.TimeStr(atms_End));
 END {CheckAcceptableTimeMassSend};

 PRIVATE PROCEDURE twoms_Date(): DATE;
 BEGIN
  IF (SYSTIME() >= 00:00) & (SYSTIME() <= 01:00) THEN
   RETURN SYSDATE();
  ELSE
   RETURN SYSDATE()+1;
  END;
 END {twoms_Date};

 PRIVATE PROCEDURE CheckAcceptableTimeSingleSend();
  CONST
   ftss_Beg = 01:00;
   ftss_End = 05:00;
 BEGIN
  L_General.Assert((SYSTIME() < ftss_Beg) | (SYSTIME() > ftss_End),
                   "ࠢ SMS 饭||" +
                   " ਮ  " + L_DatTim.TimeStr(ftss_Beg) + "  " + L_DatTim.TimeStr(ftss_End));
 END {CheckAcceptableTimeSingleSend};

 PRIVATE PROCEDURE SetSendSMSMode();
  VAR
   Text: STRING[];
 BEGIN
  IF ~VAR_IMPL.Get(DEBUG).Initialized THEN
   IF ~L_General.IsOurEngineeringCopy() & ~L_General.IsMemberOf("AP_Support") THEN
    DEBUG:=0;
   ELSE
    IF L_General.IsOurEngineeringCopy() THEN
     Text:="ࠢ SMS  ୮  ";
    ELSE
     Text:="ࠢ SMS 짮⥫ AP_Support  ࠡ祩  ";
    END;

    DEBUG:=1;
    STDFORM(Text + "|" +
            "|%,2<ࠢ SMS ⥫|쪮 ⬥⪠  ࠢ>O|||"+
            L_StdFrm.DrawButton("⬥",  13, "X") + "  " +
            L_StdFrm.DrawButton("믮", 13, "DB"),
            "  ࠢ SMS ",
            ,
            1,
            DEBUG);
    IF ~DEBUG THEN
     L_StdFrm.Choice(Text + "||" +
                     "࠭ ० 뫪 SMS   ⥫䮭");
    END;
   END;
  END;
 END {SetSendSMSMode};

 PRIVATE PROCEDURE SendSingleStaffSMS(InStream: SCAL[2]; VAR smtpClient: SMTP_CLIENT);
  VAR
   Login,
   Password,
   SMTPServer,
   EMailDomain     : STRING[];
   SchannelProtocol: NUMERIC[0];
   FrContact       : L_EMail.tContact;
 BEGIN
  E_GetSendParams(SMTPServer, EMailDomain, FrContact, Login, Password);
  CheckAcceptableTimeSingleSend();
  TRY [TERM_BY_USER(TERM_BY_USER_IGNORE)]
   DEBUGBREAK();
   IF ~DEBUG THEN
    IF !smtpClient | ~smtpClient.Connected THEN
     IF L_General.GetAPSiteNumber() = 82 THEN
      SchannelProtocol:=SMTP_CLIENT.SchannelProtocol_None;
     ELSE
      SchannelProtocol:=SMTP_CLIENT.SchannelProtocol_Auto;
     END;
     smtpClient:=L_EMail.sc_Connect(SMTPServer, SchannelProtocol, Login, Password, 1);
    END;

    L_EMail.sc_Send(smtpClient, L_EMail.AssembleContact("To",   StaffSMS.ClrPhone + "@" + EMailDomain),
                                FrContact,
                                L_EMail.EmptyContact(),
                                L_EMail.EmptyContact(),
                                L_EMail.EmptyContact(),
                                L_EMail.EmptyContact(),
                                "",
                                "",
                                StaffSMS.Text);
   END; (* IF ~DEBUG ...*)

   StaffSMS.SendDTS:=SYSTIMESTAMP();
   StaffSMS.Status :=st_Sent;
   REPLACE(StaffSMS);
   IF ~InStream & ~DEBUG THEN
    L_EMail.sc_Quit(smtpClient);
   END;
  EXCEPT
  ON Exc: Exc_Root DO
   L_General.Error(Exc.MsgOriginalText);
  END; (* TRY *)
 END {SendSingleStaffSMS};

 PRIVATE PROCEDURE CheckStaffSMSLength(trgText: STRING[]);
  CONST
   MaxSMSLength = Str.MaxLength(StaffSMS.Text);
 BEGIN
  L_General.Assert(Str.Length(trgText) <= MaxSMSLength,
                   " SMS (" + L_Str.IntStr(Str.Length(trgText)) + ") ॢ蠥 ᨬ쭮 ⨬ (" + L_Str.IntStr(MaxSMSLength) + ")");
 END {CheckStaffSMSLength};

 PUBLIC PROCEDURE SendStaffSMS();
  VAR
   dummySmtpClient: SMTP_CLIENT;
 BEGIN
  L_General.Assert(ACCESS(StaffSMS), " ࠭ ⥪饥 SMS");
  L_General.Assert(StaffSMS.Status = st_Prepared,
                   "ࠢ ࠧ襭  SMS  ﭨ '" + L_Str.PoolItem(st_Pool, st_Prepared) + "'");
  CheckStaffSMSLength(StaffSMS.Text);
  IF L_StdFrm.Alternative("ࠢ SMS?") THEN
   SETEMPTY(dummySmtpClient);
   SetSendSMSMode();
   SendSingleStaffSMS(0, dummySmtpClient);
  END;
 END {SendStaffSMS};

 PRIVATE PROCEDURE StaffSMS_OnAfterInsert();
 BEGIN
  SendStaffSMS();
 END {StaffSMS_OnAfterInsert};

 PRIVATE PROCEDURE StaffSMS_OnAfterEdit();
 BEGIN
  SendStaffSMS();
 END {StaffSMS_OnAfterEdit};

 PRIVATE PROCEDURE StaffSMSE_0_OnReposition();
 BEGIN
  CalcStaffSMSTextLength();
 END {StaffSMSE_0_OnReposition};

 PRIVATE PROCEDURE StaffSMSE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAnyModification, L_WDO.e_OnReposition]);
  L_WDO.iScsChange   (0, ["StaffSMS.Text"]);
  L_WDO.iUnmodifiable(0, ["TextLength", "StaffSMS.Source", "StaffSMS.TSN", "StaffSMS.SendDTS", "StaffSMS.Status", "StaffSMS.AcsObj", "StaffSMS.DTS"]);
  L_WDO.iField       (0, "StaffSMS.Source").Format:=src_Pool;
  L_WDO.iField       (0, "StaffSMS.Status").Format:=st_Pool;
 END {StaffSMSE_OnInitializeWindow};

 PRIVATE PROCEDURE AnyStaffSMSList_OnSetSubwindow();
  VAR
   SubwindowName: STRING[];
 BEGIN
  SETEMPTY(SubwindowName);
  IF RTI.Get().CurProcess.Kind = FORM_PROCESS.Kind_ShowWin THEN
   SubwindowName:="StaffSMSE";
  END;
  RTI.Get().CurZone.HasVariousSubwindows :=1;
  RTI.Get().CurZone.SubwindowOriginalName:=SubwindowName;
 END {AnyStaffSMSList_OnSetSubwindow};

 PRIVATE PROCEDURE StaffSMS_A_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="StaffSMS";
  L_WDO.iEvents      (1, [L_WDO.e_OnSetSubwindow]);
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iZone        (1).Narrowing:="StaffSMS.Status=0";
  L_WDO.iField       (1, "StaffSMS.Source").Format:=src_Pool_Short;
 END {StaffSMS_A_OnInitializeWindow};

 PRIVATE PROCEDURE StaffSMS_A_1_OnSetSubwindow();
 BEGIN
  AnyStaffSMSList_OnSetSubwindow();
 END {StaffSMS_A_1_OnSetSubwindow};

 PRIVATE PROCEDURE SMS_All_OnInitializeWindow();
 BEGIN
  L_General.AdminOnly();
  L_WDO.iEvents      (1, [L_WDO.e_OnSetSubwindow]);
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iField       (1, "StaffSMS.Source").Format:=src_Pool_Short;
  L_WDO.iField       (1, "StaffSMS.Status").Format:=st_Pool_Short;
 END {SMS_All_OnInitializeWindow};

 PRIVATE PROCEDURE SMS_All_1_OnSetSubwindow();
 BEGIN
  AnyStaffSMSList_OnSetSubwindow();
 END {SMS_All_1_OnSetSubwindow};

 PRIVATE PROCEDURE StaffSMS_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="StaffSMS";
  L_WDO.iSubwindow   (1, "StaffSMSE");
  L_WDO.iField       (1, "StaffSMS.Source").Format:=src_Pool_Short;
  L_WDO.iField       (1, "StaffSMS.Status").Format:=st_Pool_Short;
 END {StaffSMS_OnInitializeWindow};

 PRIVATE PROCEDURE IsFirstMassSendSMS(D: DATE): SCAL[2];
 BEGIN
  RETURN Str.Match(StaffSMS.Text, L_DatTim.DateStr(D) + " ??:?? 室 *") | (StaffSMS.Text = L_DatTim.DateStr(D) + " ");
 END {IsFirstMassSendSMS};

 PRIVATE PROCEDURE IsChangeTWOMassSendSMS(D: DATE): SCAL[2];
 BEGIN
  RETURN Str.Match(StaffSMS.Text, "  ! " + L_DatTim.DateStr(D) + " *");
 END {IsChangeTWOMassSendSMS};

 PRIVATE PROCEDURE IsTWOMassSendSMS(D: DATE): SCAL[2];
 BEGIN
  RETURN IsFirstMassSendSMS(D) | IsChangeTWOMassSendSMS(D);
 END {IsTWOMassSendSMS};

 PRIVATE PROCEDURE GetPreparedSMSAmount(trgDTS: TIMESTAMP): NUMERIC[0];
  VAR
   Result: NUMERIC[0];
 BEGIN
  SETZERO(Result);
  SELECT(StaffSMS, Status, , st_Prepared, trgDTS);
  WHILE OCCURE(StaffSMS) DO
   INC(Result);
   NEXT(StaffSMS);
  END; (* WHILE OCCURE(StaffSMS) ... *)
  RETURN Result;
 END {GetPreparedSMSAmount};

 PRIVATE PROCEDURE CancelSend(Text: STRING[]): SCAL[2];
  VAR
   T, React: NUMERIC[0];
 BEGIN
  T    :=10;
  React:=STDFORM(Text + "||" +
                 "%<    ⬥    >X     %<   த %T >FDB",
                 " 訡  ࠢ SMS ",
                 STOP,
                 ,
                 T);
  RETURN React = 0;
 END {CancelSend};

 PRIVATE PROCEDURE intSendReadyStaffSMS(trgDTS        : TIMESTAMP;
                                        prpErrAmount  : NUMERIC[0]);
  VAR
   i,
   totAmount,
   sndAmount,
   sndErrAmount: NUMERIC[0];
   smtpClient  : SMTP_CLIENT;
   forceCancel : SCAL[2];
 BEGIN
  SetSendSMSMode();
  SETEMPTY(smtpClient);
  SETZERO(sndAmount, sndErrAmount, forceCancel);
  totAmount:=GetPreparedSMSAmount(trgDTS);
  i        :=1;
  WHILE SELECT(StaffSMS, Status, st_Prepared, trgDTS) & ~forceCancel DO
   PROMPT("ࠢ " + StaffSMS.TSN + " (" + L_Str.IntStr(i) + "  " + L_Str.IntStr(totAmount) + ")");
   TRY [TRANSACTION(TRANSACTION_SLAVE), MSG_SUPPRESSION(MSG_SUPPRESSION_ALL)]
    SendSingleStaffSMS(1, smtpClient);
    INC(sndAmount);
   EXCEPT
   ON Exc: Exc_Root DO
    INC(sndErrAmount);
    IF CancelSend(Exc.MsgOriginalText) THEN
     forceCancel:=1;
    END;
   END; (* TRY *)
   INC(i);
   NEXT(StaffSMS);
  END; (* WHILE OCCURE(StaffSMS) ... *)

  IF ~!smtpClient THEN
   L_EMail.sc_Quit(smtpClient);
  END;
  IF    (prpErrAmount + sndErrAmount) > 0 THEN
   L_StdFrm.TimedINFORM_AsSTOP("SMS  ࠢ       : " + Str.Align(L_Str.IntStr(totAmount),    5, LEFT) + "|" +
                               "訡  ⮢: " + Str.Align(L_Str.IntStr(prpErrAmount), 5, LEFT) + "|" +
                               "|" +
                               "ࠢ SMS       : " + Str.Align(L_Str.IntStr(sndAmount),    5, LEFT) + "|" +
                               "訡  ࠢ  : " + Str.Align(L_Str.IntStr(sndErrAmount), 5, LEFT), 10);
  ELSIF sndAmount = 0 THEN
   L_StdFrm.RedInform("  ⮢  ࠢ SMS");
  ELSE
   L_StdFrm.TimedINFORM("ࠢ SMS: " + L_Str.IntStr(sndAmount), 10);
  END;
 END {intSendReadyStaffSMS};

 PRIVATE PROCEDURE ChoosePreparedSMSPacketDTS(Title: STRING[]): TIMESTAMP;
  VAR
   Choice,
   Amount       : NUMERIC[0];
   trgDTS       : TIMESTAMP;
   DTSList,
   SMSPacketList: STRING[];
 BEGIN
  SETEMPTY(DTSList, SMSPacketList);
  SELECT(StaffSMS, Status, , st_Prepared);
  WHILE OCCURE(StaffSMS) DO
   NARROWSEL(StaffSMS, DTS);
   SETZERO(Amount);
   trgDTS:=StaffSMS.DTS;
   WHILE OCCURE(StaffSMS) DO
    INC(Amount);
    NEXT(StaffSMS);
   END; (* WHILE OCCURE(StaffSMS) ... *)
   DTSList      :=DTSList + FLDTOSTR(trgDTS) + "|";
   SMSPacketList:=SMSPacketList + "ନ஢ " + L_DatTim.TimestampStr(trgDTS) + " (" + L_Str.IntStr(Amount) + " .)" + "|";
   NEXTINEXTENDEDSEL(StaffSMS, DTS);
  END; (* WHILE OCCURE(StaffSMS) ... *)
  L_General.Assert(~!DTSList, "  ⮢ SMS");
  Choice:=L_StdFrm.ShowListAsMenu(SMSPacketList, "롮  SMS  " + Title);
  RETURN STRTOFLD(L_Str.PoolItem(DTSList, Choice), TIMESTAMP);
 END {ChoosePreparedSMSPacketDTS};

 PUBLIC PROCEDURE DeletePreparedSMSPacket();
  VAR
   trgDTS: TIMESTAMP;
   Amount: NUMERIC[0];
 BEGIN
  trgDTS:=ChoosePreparedSMSPacketDTS("㤠");
  SETZERO(Amount);
  SELECT(StaffSMS, DTS, , trgDTS);
  WHILE OCCURE(StaffSMS) DO
   PROMPT(" ࠢ SMS: " + StaffSMS.StaffSMS);
   IF StaffSMS.Status = st_Prepared THEN
    REMOVE(StaffSMS);
    INC(Amount);
   ELSE
    NEXT(StaffSMS);
   END;
  END; (* WHILE OCCURE(StaffSMS) ... *)
  L_StdFrm.Inform(" ⮢ SMS: " + L_Str.IntStr(Amount));
 END {DeletePreparedSMSPacket};

 PUBLIC PROCEDURE SendPreparedSMSPacket();
  VAR
   trgDTS: TIMESTAMP;
 BEGIN
  trgDTS:=ChoosePreparedSMSPacketDTS("ࠢ");
  intSendReadyStaffSMS(trgDTS, 0);
 END {SendPreparedSMSPacket};

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
?, C, ""
?, C, P_Man.E_msk_GetPool()
@
#############################################################                                 ########## #####
=#####========================================================================================================
|############################################################################################################|
|############################################################################################################|
}
*)

(*
$P{ Top_Unready:
@
|------------------------------------------------------------------------------------------------------------|
|                            ⭨,    ࠢ SMS-饭                           |
|------------------------------------------------------------------------------------------------------------|
|⭨                            |稭                                                                |
|------------------------------------|-----------------------------------------------------------------------|
}
*)

(*
$P{ Line_Unready:
?, L, ""
?, L, ""
@
|####################################|#######################################################################|
}
*)

(*
$P{ Top_Ready:
@
|------------------------------------------------------------------------------------------------------------|
|                              ⭨, 襤訥 ஢ ⥫䮭 ஢                              |
|------------------------------------------------------------------------------------------------------------|
|⭨                                                                                                    |
|------------------------------------------------------------------------------------------------------------|
}
*)

(*
$P{ Line_Ready:
Staff.TSN, L, ""
Man.Name, L, ""
@
|##### ######################################################################################################|
}
*)

(*
$P{ Bot:
?, L, "⮢ SMS| 訡"
?, R, "0"
?, R, "0"
@
|------------------------------------------------------------------------------------------------------------|
|############################ #####                                                                          |
| 訡                   #####                                                                          |
=(c) Polak====================================================================================================
}
*)

 PRIVATE PROCEDURE InsertSingleStaffSMS(trgTSN     : LIKE(Staff.TSN);
                                        trgDTS     : TIMESTAMP;
                                        trgClrPhone: LIKE(StaffSMS.ClrPhone);
                                        trgText    : STRING[];
                                        trgSource  : LIKE(StaffSMS.Source));
 BEGIN
  SETEMPTY(StaffSMS.StaffSMS, StaffSMS.SendDTS);
  StaffSMS.TSN     :=trgTSN;
  StaffSMS.ClrPhone:=trgClrPhone;
  StaffSMS.DTS     :=trgDTS;
  StaffSMS.AcsObj  :=L_General.curUSER;
  StaffSMS.Source  :=trgSource;
  StaffSMS.Status  :=st_Prepared;
  StaffSMS.Text    :=trgText;
  INSERT(StaffSMS);
 END {InsertSingleStaffSMS};

 PRIVATE PROCEDURE twoms_InsertStaffSMS(trgDate       : DATE;
                                        trgDTS        : TIMESTAMP;
                                        trgTSN        : LIKE(Staff.TSN);
                                        trgClrPhone   : LIKE(StaffSMS.ClrPhone);
                                        trgText       : STRING[];
                                        CheckClrPhones: SCAL[2]): NUMERIC[0];
  VAR
   newText      : STRING[];
   Result,
   StandOnPrvSMS: SCAL[2];
 BEGIN
  Result:=1;
  IF ~CheckClrPhones THEN
   StandOnPrvSMS:=0;
   SETSEL(StaffSMS, TSN, trgTSN);
   IF LAST(StaffSMS) THEN
    WHILE OCCURE(StaffSMS) & ~StandOnPrvSMS & (StaffSMS.DTS >= GetCheckPeriodDTS()) DO
     IF (StaffSMS.Source = src_System) & (StaffSMS.Status = st_Sent) & IsTWOMassSendSMS(trgDate) THEN
      StandOnPrvSMS:=1;
     ELSE
      PREV(StaffSMS);
     END;
    END; (* WHILE OCCURE(StaffSMS) &...*)
   END;
 
   IF StandOnPrvSMS THEN
    newText:="  ! " + trgText;
 
    IF IsChangeTWOMassSendSMS(trgDate) THEN
     Result:=StaffSMS.Text # newText;
    ELSE
     Result:=StaffSMS.Text # trgText;
    END;
   ELSE
    newText:=trgText;
    Result :=1;
   END;
 
   IF  Result THEN
    InsertSingleStaffSMS(trgTSN, trgDTS, trgClrPhone, newText, src_System);
   END;
  END; (* IF ~CheckClrPhones...*)
  RETURN Result;
 END {twoms_InsertStaffSMS};

 PRIVATE PROCEDURE sms_InsertStaffSMS(trgDate       : DATE;
                                      trgDTS        : TIMESTAMP;
                                      trgTSN        : LIKE(Staff.TSN);
                                      trgClrPhone   : LIKE(StaffSMS.ClrPhone);
                                      trgText       : STRING[];
                                      CheckClrPhones: SCAL[2]): NUMERIC[0];
 BEGIN
  IF ~CheckClrPhones THEN
   InsertSingleStaffSMS(trgTSN, trgDTS, trgClrPhone, trgText, src_Operator);
  END;
  RETURN 1;
 END {sms_InsertStaffSMS};

 PRIVATE PROCEDURE InsertTmpSMS(trgTSN   : LIKE(Staff.TSN);
                                trgDate  : DATE;
                                trgBegDTS: TIMESTAMP;
                                trgText  : STRING[]);
  VAR
   trg_prpSMS: SCAL[1E7];
 BEGIN
  IF SELECT(prpSMS, prpSMS) & LAST(prpSMS) THEN
   trg_prpSMS:=prpSMS.prpSMS + 1;
  ELSE
   trg_prpSMS:=1;
  END;

  DEFAULT(prpSMS);
  prpSMS.prpSMS:=trg_prpSMS;
  prpSMS.TSN   :=trgTSN;
  prpSMS.Ready :=1;
  INSERT(prpSMS);
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   IF ~!trgBegDTS THEN
    prpSMS.BegDTS:=trgBegDTS;
   END;
   prpSMS.ClrPhone:=GetStaffClrPhone(trgTSN, trgDate);
   prpSMS.Text    :=trgText;
   REPLACE(prpSMS);
  EXCEPT
  ON Exc: Exc_Root DO
   prpSMS.Ready:=0;
   prpSMS.Text :=Exc.Description;
   REPLACE(prpSMS);
  END; (* TRY *)
 END {InsertTmpSMS};

 PRIVATE PROCEDURE IsAcceptableStaff(trgDate: DATE): SCAL[2];
 BEGIN
  IF L_Staff.rInState(trgDate) THEN
   CONFORM(StaffLog, Officer);
   RETURN L_Env.IsRouteDriver() | L_Env.IsGuard();
  ELSE
   RETURN 0;
  END;
 END {IsAcceptableStaff};

 PRIVATE PROCEDURE twoms_InsertTmpSMS(trgDate        : DATE;
                                      trgTime        : TIME;
                                      trgTSN         : LIKE(Staff.TSN);
                                      InformAboutRest: SCAL[2]);
  VAR
   trgText  : STRING[];
   trgBegDTS: TIMESTAMP;
 BEGIN
  IF L_Env.RealTSN(trgTSN) THEN
   SEARCH(Staff, TSN, , trgTSN);
   IF P_Man.IsSMSMassSendReceiver(Staff.Man, trgDate, P_Man.msk_DrGuNextDayRegimeNotification, 0) THEN
    IF IsAcceptableStaff(trgDate) THEN
     IF ~InformAboutRest THEN
      trgBegDTS:=L_DatTim.GetTIMESTAMP(trgDate, trgTime);
      trgText  :=L_DatTim.DateTimeStr(trgDate, trgTime) + " 室 " + ExitClss.Sign;
     ELSE
      trgText  :=L_DatTim.DateStr(trgDate) + " ";
      SETEMPTY(trgBegDTS);
     END;
     InsertTmpSMS(trgTSN, trgDate, trgBegDTS, trgText);
    END; (* IF L_Env.IsRouteDriver() | L_Env.IsGuard()...*)
   END; (* IF P_Man.IsSMSMassSendReceiver...*)
  END; (* IF L_Env.RealTSN(trgTSN)...*)
 END {twoms_InsertTmpSMS};

 PRIVATE PROCEDURE MassSendAlreadyDone(D: DATE): SCAL[2];
 BEGIN
  SETSEL(StaffSMS, DTS);
  SRCINSEL(StaffSMS, GE, , GetCheckPeriodDTS());
  WHILE OCCURE(StaffSMS) DO
   IF (StaffSMS.Status = st_Sent) & IsFirstMassSendSMS(D) THEN
    RETURN 1;
   END;
   NEXT(StaffSMS);
  END; (* WHILE OCCURE(StaffSMS) ... *)
  RETURN 0;
 END {MassSendAlreadyDone};

 PRIVATE PROCEDURE ClearPreparedStatusStaffSMS(D: DATE);
 BEGIN
  SELECT(StaffSMS, Status, , st_Prepared);
  WHILE OCCURE(StaffSMS) DO
   PROMPT(StaffSMS.TSN + ": ⪠");
   IF (StaffSMS.Source = src_System) & IsTWOMassSendSMS(D) THEN
    REMOVE(StaffSMS);
   ELSE
    NEXT(StaffSMS);
   END;
  END; (* WHILE OCCURE(StaffSMS) ... *)
 END {ClearPreparedStatusStaffSMS};

 PRIVATE PROCEDURE GetRepName(ms_Kind: P_Man.tMassSendKind; CheckClrPhones: SCAL[2]): STRING[];
  VAR
   RepName: STRING[];
 BEGIN
  IF ms_Kind = P_Man.msk_DrGuNextDayRegimeNotification THEN
   RepName:="MMSnd";
  ELSE
   RepName:="SMSnd";
  END;
  RETURN RepName + CheckClrPhones + "_" + ms_Kind + "_" + L_DatTim.SysDateTimeStamp();
 END {GetRepName};

 PRIVATE PROCEDURE Send(ms_Kind: P_Man.tMassSendKind; D: DATE; trgDTS: TIMESTAMP; p: PROC; ClearPreparedStaffSMS, CheckClrPhones: SCAL[2]);
  VAR
   Descr      : STRING[][10];
   trgText,
   RepName    : STRING[];
   trgClrPhone: LIKE(StaffSMS.ClrPhone);
   i, j,
   Amount,
   ErrorAmount: NUMERIC[0];
 BEGIN
  APP.Get().NewPrompt(L_Str.PoolItem("뫪 SMS|஢ઠ ନ஢ SMS", CheckClrPhones));
  RepName:=GetRepName(ms_Kind, CheckClrPhones);
  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, Top, RepName, "⮪ " + L_Str.PoolItem("뫪|஢ન ନ஢", CheckClrPhones) + " SMS-饭  " + L_DatTim.DateStr(D),
                            ms_Kind);

  TRY [TRANSACTION(TRANSACTION_SLAVE)]
   IF ClearPreparedStaffSMS & ~CheckClrPhones THEN
    ClearPreparedStatusStaffSMS(D);
   END;
   i:=1;
   SETZERO(Amount, ErrorAmount);
   SELECT(prpSMS, Ready);
   WHILE OCCURE(prpSMS) DO
    NARROWSEL(prpSMS, Ready);
    IF OCCURE(prpSMS) THEN
     IF prpSMS.Ready THEN
      IF CheckClrPhones THEN
       MAKEPART(R, Top_Ready);
      END;
     ELSE
      MAKEPART(R, Top_Unready);
     END;
    END;
    WHILE OCCURE(prpSMS) DO
     NARROWSEL(prpSMS, TSN);
     SEARCH(Staff, TSN, , prpSMS.TSN);
     PROMPT("⮢. " + prpSMS.TSN + " (" + L_Str.IntStr(i) + "  " + L_Str.IntStr(AMOUNT(prpSMS)) + ")");
     SEARCH(Staff, TSN, , prpSMS.TSN);
     CONFORM(Staff, Man);
     trgClrPhone:=prpSMS.ClrPhone;
     SETEMPTY(trgText);
     WHILE OCCURE(prpSMS) DO
      IF prpSMS.Ready THEN
       trgText:=L_Str.AddStrDlm(trgText, " ") + prpSMS.Text;
       IF CheckClrPhones THEN
        MAKEPART(R, Line_Ready);
       END;
      ELSE
       Str.Format(Descr, prpSMS.Text, 71, LEFT);
       MAKEPART(R, Line_Unready, Staff.TSN + " " + Man.Name, Descr[0]);
       j:=1;
       WHILE ~!Descr[j] DO
        MAKEPART(R, Line_Unready, "", Descr[j]);
        INC(j);
       END; (* WHILE *)
       INC(ErrorAmount);
      END;
      INC(i);
      NEXT(prpSMS);
     END; (* WHILE OCCURE(prpSMS) ... *)
 
     IF ~!trgClrPhone THEN
      INC(Amount, p(D, trgDTS, Staff.TSN, trgClrPhone, trgText, CheckClrPhones));
     END;
     NEXTINEXTENDEDSEL(prpSMS, TSN);
    END; (* WHILE OCCURE(prpSMS) ... *)
    NEXTINEXTENDEDSEL(prpSMS, Ready);
   END; (* WHILE OCCURE(prpSMS) ... *)
  EXCEPT
  END; (* TRY *)

  MAKEPART(R, Bot, CheckClrPhones, Amount, ErrorAmount);
  CLOSEREP(R);
  IF ~CheckClrPhones THEN
   IF Amount > 0 THEN
    intSendReadyStaffSMS(trgDTS, ErrorAmount);
   ELSE
    L_StdFrm.Inform("   ࠢ");
   END;
  END;
  IF    ErrorAmount > 0 THEN
   L_General.SurveyLastReport();
  ELSIF CheckClrPhones THEN
   IF Amount > 0 THEN
    L_StdFrm.Inform("஢७ " + L_Str.IntStr(Amount) + " 祫.|" +
                    " ६ ஢ન ஢ ⥫䮭     ஡");
   ELSE
    L_StdFrm.RedInform("    室饣 ࠡ⭨  ஢ન ஢ ⥫䮭");
   END;
  END;
 END {Send};

 PRIVATE PROCEDURE CreateTmpSMS();
 BEGIN
  L_AFM.MakeMultipleIndex("prpSMS", "prpSMS  : SCAL[1E7];" +
                                    "Ready   : SCAL[2];" +
                                    "TSN     : LIKE(Staff.TSN);" +
                                    "BegDTS  : TIMESTAMP;" +
                                    "ClrPhone: STRING[11];" +
                                    "Text    : STRING[256]",
                          "prpSMS:prpSMS;Ready:Ready,TSN,BegDTS;TSN: TSN,BegDTS,Ready");
 END {CreateTmpSMS};

 PRIVATE PROCEDURE IsStaffInGivenRWayS(D: DATE; trgTSN: LIKE(Staff.TSN)): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  Result:=0;
  IF SELECT(RWayS, sTSN, L_Way.rwsStat_Given, trgTSN, D) THEN
   Result:=1;
  ELSE
   SELECT(WayLMen, TSN, , trgTSN, D);
   WHILE OCCURE(WayLMen) & ~Result DO
    CONFORM(WayLMen, RWayS);
    Result:=RWayS.Status = L_Way.rwsStat_Given;
    NEXT(WayLMen);
   END; (* WHILE OCCURE(WayLMen) ... *)
  END;
  RETURN Result;
 END {IsStaffInGivenRWayS};

 PUBLIC PROCEDURE TWOMassSend(CheckClrPhones: SCAL[2]);
  VAR
   D      : DATE;
   Proceed: SCAL[2];
 BEGIN
  L_AcsPnt.PassAccessPoint(0000064);

  CheckSMSSendAllowed();
  D:=twoms_Date();

  IF ~CheckClrPhones THEN
   IF MassSendAlreadyDone(D) THEN
    L_StdFrm.Question("믮 뫪 SMS-饭    拉  " + L_DatTim.DateStr(D) + "|" +
                      "( 鸞  " + L_DatTim.CompleteSystimeStr() + ")");
   ELSE
    L_StdFrm.Question("믮 뫪 SMS-饭  ० ࠡ    " + L_DatTim.DateStr(D));
    CheckAcceptableTimeMassSend();
   END;
   CheckAcceptableTimeSingleSend();
  END;

  CreateTmpSMS();

  SELECT(RWayS, sDate, , L_Way.rwsStat_Ordered, D);
  WHILE OCCURE(RWayS) DO
   PROMPT(RWayS.TSN + ":  ");
   CONFORM(RWayS, StExit);
   CONFORM(StExit, ExitClss);
   twoms_InsertTmpSMS(D, L_BegRWS.GetRWaySExitTime(0), RWayS.TSN, 0);
   SELECT(WayLMen, RWayS, , RWayS.RWayS);
   WHILE OCCURE(WayLMen) DO
    twoms_InsertTmpSMS(D + (WayLMen.Day - 1), WayLMen.StartT, WayLMen.TSN, 0);
    NEXT(WayLMen);
   END; (* WHILE OCCURE(WayLMen) ... *)
   NEXT(RWayS);
  END; (* WHILE OCCURE(RWayS) ... *)

  SELECT(Mtwo, Per, , L_DatTim.Per(D));
  WHILE OCCURE(Mtwo) DO
   PROMPT(Mtwo.TSN + ":  鸞");
   CONFORM(Mtwo, Staff);
   IF ~SELECT(prpSMS, TSN, Mtwo.TSN) & IsAcceptableStaff(D) & ~IsStaffInGivenRWayS(D, Mtwo.TSN) THEN
    IF L_BegRWS.IsWorkDayType(Mtwo.What[L_two.Date2Idx(D)]) THEN
     L_two.AccessDtwoByMtwo(L_two.Date2Idx(D));
     Proceed:=~!Dtwo.RWayS;
    ELSE
     Proceed:=1;
    END;
    IF Proceed & L_two.IsNormalSheet(Mtwo.TSN, D) THEN
     twoms_InsertTmpSMS(D, L_General.EMPTY_TIME(), Mtwo.TSN, 1);
    END;
   END; (* IF ~SELECT(prpSMS, TSN, trgTSN)...*)
   NEXT(Mtwo);
  END; (* WHILE OCCURE(Mtwo) ... *)

  Send(P_Man.msk_DrGuNextDayRegimeNotification, D, SYSTIMESTAMP(), PROCREF(twoms_InsertStaffSMS), 1, CheckClrPhones);
 END {TWOMassSend};

 PRIVATE PROCEDURE StfSMSPrp_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnBeforeEdit, L_WDO.e_OnAfterEdit]);
  L_WDO.iScsChange   (0, ["CheckClrPhones", "Text", "Kind"]);
  L_WDO.iUnmodifiable(0, ["TextLength", "Date"]);
  L_WDO.iField       (0, "CheckClrPhones").Format:="-|";
  L_WDO.iProgs       (0, "StaffSMS");
 END {StfSMSPrp_OnInitializeWindow};

 PRIVATE PROCEDURE StfSMSPrp_OnBeforeEdit();
  VAR
   Fields: FORMFIELDS;
 BEGIN
  L_AcsPnt.PassAccessPoint(0000055);
  CheckSMSSendAllowed();
  Fields:=RTI.Get().CurZone.Fields;
  Fields("Kind").Format          :=P_Man.E_msk_GetPool();
  Fields("Date").Value           :=SYSDATE();
  Fields("Text").Modifiable      :=1;
  Fields("CheckClrPhones").Value :=0;
  Fields("Text").Template        :=L_Pool.Get(0001004);
  Fields("Text").TemplateIsStrict:=0;

  SETEMPTY(Fields("Kind").Value, Fields("Text").Value);
  SETZERO(Fields("TextLength").Value);
 END {StfSMSPrp_OnBeforeEdit};

 PRIVATE PROCEDURE IsAlreadyDone(trgTSN: LIKE(Staff.TSN); trgDate: DATE; trgText: STRING[]): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  Result:=0;
  SETSEL(StaffSMS, TSN, trgTSN);
  SRCINSEL(StaffSMS, GE, , GetCheckPeriodDTS());
  WHILE OCCURE(StaffSMS) DO
   IF StaffSMS.Text = trgText THEN
    Result:=1;
   END;
   NEXT(StaffSMS);
  END; (* WHILE OCCURE(StaffSMS) ... *)
  RETURN Result;
 END {IsAlreadyDone};

 PRIVATE PROCEDURE ConditionalInsertSMS(trgDate: DATE; trgText: STRING[]; CheckClrPhone: SCAL[2]);
  VAR
   DTS: TIMESTAMP;
 BEGIN
  IF CheckClrPhone | ~IsAlreadyDone(Staff.TSN, trgDate, trgText) THEN
   SETEMPTY(DTS);
   InsertTmpSMS(Staff.TSN, trgDate, DTS, trgText);
  END;
 END {ConditionalInsertSMS};

 PRIVATE PROCEDURE E_ms_AcceptableStaff(ms_Kind: P_Man.tMassSendKind; trgDate: DATE): SCAL[2];
  VAR
   Local  : PROC;
   default: SCAL[2];
 BEGIN
  IF    ms_Kind = P_Man.msk_All THEN
   default:=1;
  ELSIF ms_Kind = P_Man.msk_FemaleOnly THEN
   default:=Man.Sex = L_Man.sex_Female;
  ELSIF ms_Kind = P_Man.msk_MaleOnly THEN
   default:=Man.Sex = L_Man.sex_Male;
  ELSE
   default:=1;
  END;
  default:=default &
           L_Staff.IsStandOnAnyMainContract(Man.Man, trgDate+100) &
           P_Man.IsSMSMassSendReceiver(Man.Man, trgDate, ms_Kind, ms_Kind >= P_Man.msk_UserDefined) &
           (Staff.EndDate > (trgDate - 10));
  Local:=PROCREF(E_Utils.StaffSMS_ms_IsAcceptableStaff);
  IF ~!Local THEN
   RETURN Local(ms_Kind, trgDate, default);
  ELSE
   RETURN default;
  END;
 END {E_ms_AcceptableStaff};

 PUBLIC PROCEDURE MassSend(ms_Kind: P_Man.tMassSendKind; trgText: STRING[]; CheckClrPhone: SCAL[2]);
  VAR
   trgDate: DATE;
 BEGIN
  RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
  CreateTmpSMS();
  trgDate:=SYSDATE();
  SELECT(Man, Man);
  WHILE OCCURE(Man) DO
   PROMPT("⮢: " + Man.Man);
   IF L_Env.RealMan(Man.Man) & E_ms_AcceptableStaff(ms_Kind, trgDate) THEN
    ConditionalInsertSMS(trgDate, trgText, CheckClrPhone);
   END;
   NEXT(Man);
  END; (* WHILE OCCURE(Man) ... *)
  Send(ms_Kind, trgDate, SYSTIMESTAMP(), PROCREF(sms_InsertStaffSMS), 0, CheckClrPhone);
 END {MassSend};

 PRIVATE PROCEDURE StfSMSPrp_Kind_OnSuccessfulChange();
 BEGIN
  L_General.Assert(RTI.Get().CurZone.ActiveField.Value # P_Man.msk_DrGuNextDayRegimeNotification,
                   "  뫪 ᯮ ⮫쪮  ⮬᪮ 饭");
 END {StfSMSPrp_Kind_OnSuccessfulChange};

 PRIVATE PROCEDURE StfSMSPrp_CheckClrPhones_OnSuccessfulChange();
 BEGIN
  RTI.Get().CurZone.Fields("Text").Modifiable:=~RTI.Get().CurZone.ActiveField.Value;
 END {StfSMSPrp_CheckClrPhones_OnSuccessfulChange};

 PRIVATE PROCEDURE StfSMSPrp_Text_OnSuccessfulChange();
 BEGIN
  RTI.Get().CurZone.Fields("TextLength").Value:=Str.Length(RTI.Get().CurZone.ActiveField.Value);
 END {StfSMSPrp_Text_OnSuccessfulChange};

 PRIVATE PROCEDURE StfSMSPrp_OnAfterEdit();
  VAR
   Msg   : STRING[];
   Fields: FORMFIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  CheckStaffSMSLength(Fields("Text").Value);
  Msg:="믮 " + L_Str.PoolItem("ᮢ 뫪|஢ ⥫䮭 ஢", Fields("CheckClrPhones").Value) + "  ਠ:||'" + L_Str.PoolItem(Fields("Kind").Format, Fields("Kind").Value) + "'";
  IF ~Fields("CheckClrPhones").Value THEN
   Msg:=Msg + "||" +
              " 뫪:|" +
              "'" + Fields("Text").Value + "'";
  END;

  L_StdFrm.Question(Msg);
  LIKEMENU(CALL, 'StaffSMS.MassSend(' + Fields("Kind").Value + ', ' +
                                    '"' + L_Str.ExclQM(Fields("Text").Value) + '", ' +
                                    Fields("CheckClrPhones").Value + ')');
 END {StfSMSPrp_OnAfterEdit};

BEGIN
 L_Events.ExecuteHandler();
END.
