(*
%Copyright   (c) Polak IT
%Author      26.03.2003 16:10:51 POLAK\SAN
%Purpose     殮  "࠭᭠"
%Type        operation
%Specific    LOC
%Modified    13.05.2013 12:45:09 POLAK\GAV
%Environment
*)

 MODULE xRfTn;

 IMPORT L_General, L_Str, DBF, L_DatTim, L_AFM, Str, P_Firm, L_Way,
        Math, L_Events, L_StdFrm, E_Way, Memo, L_OwnData, L_XDoc, P_Route, L_ExtDB,
        L_iSwap, L_XML, L_Env, E_Prop, P_OwnData, L_Car, FIO, L_Prop, DBD, L_FIO;

 CONST
  st_Imported = 0;
  st_Initial  = 1;
  st_Done     = 2;

  mFictExit  = 0;
  mOverTrips = 1;
  mLosses    = 2;

  TimeDiffAllowed = 10/60;

 VAR
  streamMode: SCAL[2];
  RepName,
  ErrRepName: STRING[];

 PRIVATE PROCEDURE path_Root(): STRING[];
 BEGIN
  IF (DBD.BaseName() = "KL") | (DBD.BaseName() = "VD") THEN
   RETURN "REP\TRANSNAVI";
  ELSE
   RETURN "TXT";
  END;
 END {path_Root};

 PRIVATE PROCEDURE path_IN(): STRING[];
 BEGIN
  IF (DBD.BaseName() = "KL") | (DBD.BaseName() = "VD") THEN
   RETURN FIO.BuildPath("IN",  , path_Root());
  ELSE
   RETURN path_Root();
  END;
 END {path_IN};

 PRIVATE PROCEDURE path_OUT(): STRING[];
 BEGIN
  IF (DBD.BaseName() = "KL") | (DBD.BaseName() = "VD") THEN
   RETURN FIO.BuildPath("OUT",  , path_Root());
  ELSE
   RETURN path_Root();
  END;
 END {path_OUT};

(*
$P{ Top:
P_Firm.FP_FirmName(P_Firm.OWN_FIRM, SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
ErrRepName, L, ""
?, C, ""
@
############################################################################################# ########## #####
=#####========================================================================================================
|############################################################################################################|
|------------------------------------------------------------------------------------------------------------|
}
*)

(*
$P{ xRfTnIDDescription:
@
|  |⠡  | |||਩                                                                           |
|-----|-----|----|--|-|--------------------------------------------------------------------------------------|
}
*)

(*
$P{ Line:
?, L, ""
@
|############################################################################################################|
}
*)

(*
$P{ Bot:
@
=(c) Polak====================================================================================================
}
*)

 PRIVATE PROCEDURE locError(Text: STRING[]);
 BEGIN
  IF streamMode THEN
   MAKEPART(Err, Line, Text);
  ELSE
   L_General.Error(Text);
  END;
 END {locError};

 PRIVATE PROCEDURE locAssert(cond: SCAL[2]; Text: STRING[]);
 BEGIN
  IF ~cond THEN
   locError(Text);
  END;
 END {locAssert};

 PRIVATE PROCEDURE GetDbfFileName(Prf: STRING[]; D: DATE): STRING[];
 BEGIN
  RETURN Prf + FLDTOSTR(D, "DNY");
 END {GetDbfFileName};

 PRIVATE PROCEDURE SplitTime(asTime: STRING[]; VAR Day: LIKE(RWayJ.Day); VAR Time: TIME);
  VAR
   H, M: NUMERIC[0];
 BEGIN
  H:=STRTOFLD(Str.Item(asTime, ":", 0), NUMERIC);
  M:=STRTOFLD(Str.Item(asTime, ":", 1), NUMERIC);
  Day:=H DIV 24 + 1;
  Time:=00:00 + H MOD 24 + M / 60;
 END {SplitTime};

 PRIVATE PROCEDURE E_convRouteIn(_Route: STRING[4]): STRING[4];
 BEGIN
  IF SELECT(RoutePro, Property, P_Route.P_ASDC, _Route) &
     SRCINSEL(RoutePro, LE, Memo.ud_cd()) & (Memo.ud_cd() < RoutePro.DocExpD) THEN
   RETURN RoutePro.Route;
  ELSE
   WHILE ~SEARCH(Route, Route, _Route) & (Str.Length(_Route) < 4) DO
    _Route:="0"+_Route;
   END;
   RETURN _Route;
  END;
 END {E_convRouteIn};

 PRIVATE PROCEDURE E_PosToRoute(_Exit: LIKE(StExit.Exit)): SCAL[2];
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_PosToRoute);
  IF ~!local THEN
   RETURN local(_Exit);
  ELSE
   SELECT(SEJobs, Exit, , _Exit);
   CONFORM(SEJobs, Route);
   RETURN 1;
  END;
 END {E_PosToRoute};

 PRIVATE PROCEDURE E_convRouteOut(trgRoute: LIKE(Route.Route)): STRING[];
  VAR
   dummy : SCAL[2];
   Result: STRING[];
 BEGIN
  SETEMPTY(Result);
  IF P_Route.HasActualRoutePro(trgRoute, Memo.ud_cd(), P_Route.P_ASDC) THEN
   P_Route.ASDC_DecodeValue(RoutePro.Value, dummy, Result);
  END;
  IF !Result THEN
   Result:=trgRoute;
  END;
  RETURN Result;
 END {E_convRouteOut};

 PRIVATE PROCEDURE E_convBoxNbrOut(BoxNbr: LIKE(Car.BoxNbr)): SCAL[1E5];
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_convBoxNbrOut);
  IF ~!local THEN
   RETURN local(BoxNbr);
  ELSE
   RETURN BoxNbr;
  END;
 END {E_convBoxNbrOut};

 PRIVATE PROCEDURE E_convChangeOut(Change: LIKE(StExit.Change)): STRING[];
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_convChangeOut);
  IF ~!local THEN
   RETURN local(Change);
  ELSE
   IF Change = 2 THEN
    RETURN "2";
   ELSE
    RETURN "1";
   END;
  END;
 END {E_convChangeOut};

 PRIVATE PROCEDURE E_convGrNbrOut(_Exit: LIKE(StExit.Exit)): NUMERIC[0];
  VAR
   local: PROC;
   _GrNbr: NUMERIC[0];
 BEGIN
  local:=PROCREF(E_Way.xRfTn_convGrNbrOut);
  IF ~!local THEN
   RETURN local(_Exit);
  ELSE
   _GrNbr:=_Exit MOD 1E6 DIV 1E3;
   IF (_GrNbr DIV 100) = 4 THEN
    DEC(_GrNbr, 100); (* 400- 䨪    300- *)
   END;
   RETURN _GrNbr;
  END;
 END {E_convGrNbrOut};

 PRIVATE PROCEDURE E_convBoxNbrIn(boxnbr: LIKE(xRfTnH.boxnbr)): LIKE(RWayS.BoxNbr);
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_convBoxNbrIn);
  IF ~!local THEN
   RETURN local(boxnbr);
  ELSE
   RETURN boxnbr;
  END;
 END {E_convBoxNbrIn};

 PRIVATE PROCEDURE Insert_xRfTnH(_date  : LIKE(xRfTnH.date);
                         _boxnbr: LIKE(xRfTnH.boxnbr);
                         _tsn   : LIKE(xRfTnH.tsn);
                         _route : LIKE(xRfTnH.route);
                         _grnbr : LIKE(xRfTnH.grnbr);
                         _change: LIKE(xRfTnH.change));
 BEGIN
  IF ~SEARCH(xRfTnH, date, _date, _boxnbr, _tsn, _route, _grnbr, _change) THEN
   locAssert(SEARCH(Staff, TSN, _tsn), " ⠡ " + _tsn);
   DEFAULT(xRfTnH);
   xRfTnH.date  :=_date;
   xRfTnH.boxnbr:=_boxnbr;
   xRfTnH.tsn   :=_tsn;
   xRfTnH.route :=_route;
   xRfTnH.grnbr :=_grnbr;
   xRfTnH.change:=_change;
   xRfTnH.Status:=st_Imported;
   INSERT(xRfTnH);
  END;
 END {Insert_xRfTnH};

 PRIVATE PROCEDURE E_SpecialCaseConversion(VAR _route : LIKE(xRfTnH.route);
                                   VAR _grnbr : LIKE(xRfTnH.grnbr);
                                   VAR _change: LIKE(xRfTnH.change));
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_SpecialCaseConversion);
  IF ~!local THEN
   local(_route, _grnbr, _change);
  END;
 END {E_SpecialCaseConversion};

 PRIVATE PROCEDURE E_dbfFieldName(FieldName: STRING[]): STRING[];
  VAR
   local       : PROC;
   FieldRename,
   FieldRenames: STRING[];
   Idx         : NUMERIC[0];
 BEGIN
  local:=PROCREF(E_Way.xRfTn_FieldRenames);
  IF ~!local THEN
   FieldRenames:=local();
   Idx:=0;
   LOOP
    FieldRename:=L_Str.PoolItem(FieldRenames, Idx);
    IF !FieldRename THEN
     EXIT;
    ELSIF Str.Item(FieldRename, ",", 0) = FieldName THEN
     RETURN Str.Item(FieldRename, ",", 1);
    END;
    INC(Idx);
   END;
  END;
  RETURN FieldName;
 END {E_dbfFieldName};

 PRIVATE PROCEDURE E_DataFilePrefixes(): STRING[];
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_DataFilePrefixes);
  IF ~!local THEN
   RETURN local();
  ELSE
   RETURN "Rf|Rz";
  END;
 END {E_DataFilePrefixes};

 PRIVATE PROCEDURE Import();
  VAR
   D       : DATE;
   i       : NUMERIC[0];
   _date   : LIKE(xRfTnH.date);
   _tsn    : LIKE(xRfTnH.tsn);
   _boxnbr : LIKE(xRfTnH.boxnbr);
   _route  : LIKE(xRfTnH.route);
   grTmp   : SCAL[1E3];
   _grnbr  : LIKE(xRfTnH.grnbr);
   _change : LIKE(xRfTnH.change);
   impFPfx : STRING[2];
   impFN   : STRING[12];
   DummyDay: NUMERIC[0];
 BEGIN
  L_General.Assert(DBD.FileExist(xRfTnH), "  '࠭᭠樨'  祭");
  D:=Memo.ud_cd();
  ErrRepName:="BRTIm";
  streamMode:=1;
  STDMSG("   " + L_DatTim.DateStr(D), CHOICE);
  IF SELECT(xRfTnH, date, D) THEN
   STDMSG("    " + L_DatTim.DateStr(D) + ". ?",
          CHOICE);
   INTEGRITY(NoAsk);
   WHILE OCCURE(xRfTnH) DO
    L_General.Assert(xRfTnH.Status # st_Done,
                     "  - 㦥  ࠡ⠭ ");
    PROMPT(xRfTnH.boxnbr);
    REMOVE(xRfTnH);
   END; (* WHILE OCCURE(xRfTnH) ... *)
   INTEGRITY(Off);
  END;
  OPENREP(Err, L_General.NameConnection(ErrRepName), Create);
  MAKEPART(Err, Top, "஡   " + L_DatTim.DateStr(D));

  impFPfx:=L_Str.PoolItem(E_DataFilePrefixes(), 0);
  impFN:=GetDbfFileName(impFPfx, D)+".dbf";
  DBF.Open(Rf, FIO.BuildPath(impFN, , path_IN()), Input);
  i:=0;
  WHILE DBF.Seek(Rf, i) DO
   INC(i);
   PROMPT(": ࠡ⪠ ப " + L_Str.IntStr(i) +
          "  " + L_Str.IntStr(DBF.Amount(Rf)));
   IF ~DBF.Deleted(Rf) THEN
    _date  :=DBF.GetField(Rf, E_dbfFieldName("DATE"));
    _tsn   :=DBF.GetField(Rf, E_dbfFieldName("NSID_TABEL"));
    _boxnbr:=DBF.GetField(Rf, E_dbfFieldName("NSIT_GARAG"));
    _route :=DBF.GetField(Rf, E_dbfFieldName("MARSH"));
    grTmp  :=DBF.GetField(Rf, E_dbfFieldName("GRAPH"));
    _grnbr :=grTmp MOD 50;
    _change:=DBF.GetField(Rf, E_dbfFieldName("SMENA"));

    E_SpecialCaseConversion(_route, _grnbr, _change);
    Insert_xRfTnH(_date, _boxnbr, _tsn, _route, _grnbr, _change);

    xRfTn.xRfTnH:=xRfTnH.xRfTnH;
    xRfTn.workC :=0;(*DBF.GetField(Rf, E_dbfFieldName("WORKCODE"));*)
    xRfTn.qualC :=DBF.GetField(Rf, E_dbfFieldName("MNGM_ID"));
    xRfTn.Run   :=DBF.GetField(Rf, E_dbfFieldName("LINLENFACT"));
    SplitTime(DBF.GetField(Rf, E_dbfFieldName("RBEGINPLAN")), xRfTn.PlDay, xRfTn.PlBeg);
    SplitTime(DBF.GetField(Rf, E_dbfFieldName("RENDPLAN")),   DummyDay,    xRfTn.PlEnd);
    IF !DBF.GetField(Rf, E_dbfFieldName("RBEGINFACT")) |
       (DBF.GetField(Rf, E_dbfFieldName("RBEGINFACT")) = "03:00")  THEN
     xRfTn.FcDay:=xRfTn.PlDay;
     xRfTn.FcBeg:=xRfTn.PlBeg;
     xRfTn.FBS  :=1;
    ELSE
     SplitTime(DBF.GetField(Rf, E_dbfFieldName("RBEGINFACT")), xRfTn.FcDay, xRfTn.FcBeg);
     xRfTn.FBS  :=0;
    END;
    IF !DBF.GetField(Rf, E_dbfFieldName("RENDFACT")) |
       (DBF.GetField(Rf, E_dbfFieldName("RENDFACT")) = "03:00") THEN
     xRfTn.FcEnd:=xRfTn.PlEnd;
     xRfTn.FES  :=1;
    ELSE
     SplitTime(DBF.GetField(Rf, E_dbfFieldName("RENDFACT")), DummyDay, xRfTn.FcEnd);
     xRfTn.FES  :=0;
    END;
    xRfTn.Status  :=st_Imported;
    xRfTn.impFName:=impFN;
    xRfTn.PosInFil:=i;
    INSERT(xRfTn);
   END;
  END;
  DBF.Close(Rf);
(*
  impFPfx:=L_Str.PoolItem(E_DataFilePrefixes(), 1);
  IF ~!impFPfx THEN
   impFN:=GetDbfFileName(impFPfx, D)+".dbf";
   DBF.Open(Rz, FIO.BuildPath(impFN, , path_IN()), Input);
   i:=0;
   WHILE DBF.Seek(Rz, i) DO
    INC(i);
    PROMPT(": ࠡ⪠ ப " + L_Str.IntStr(i) +
           "  " + L_Str.IntStr(DBF.Amount(Rz)));
    IF ~DBF.Deleted(Rz) THEN
     _date  :=DBF.GetField(Rz, E_dbfFieldName("NDATA"));
     _tsn   :=DBF.GetField(Rz, E_dbfFieldName("TABNUM"));
     _boxnbr:=DBF.GetField(Rz, E_dbfFieldName("GARAGNUMB"));
     _route :=DBF.GetField(Rz, E_dbfFieldName("MARSH"));
     grTmp  :=DBF.GetField(Rz, E_dbfFieldName("GRAPH"));
     _grnbr :=grTmp MOD 50;
     _change:=DBF.GetField(Rz, E_dbfFieldName("SMENA"));
 
     Insert_xRfTnH(_date, _boxnbr, _tsn, _route, _grnbr, _change);
 
     xRfTn.xRfTnH  :=xRfTnH.xRfTnH;
     xRfTn.workC   :=4; (* १ *)
     xRfTn.qualC   :=0;
     xRfTn.Run     :=0;
     xRfTn.PlDay   :=convDay (DBF.GetField(Rz, E_dbfFieldName("BEGINTIME")));
     xRfTn.PlBeg   :=convTime(DBF.GetField(Rz, E_dbfFieldName("BEGINTIME")));
     xRfTn.PlEnd   :=convTime(DBF.GetField(Rz, E_dbfFieldName("ENDTIME")));
     xRfTn.FcDay   :=convDay (DBF.GetField(Rz, E_dbfFieldName("BEGINTIME")));
     xRfTn.FcBeg   :=convTime(DBF.GetField(Rz, E_dbfFieldName("BEGINTIME")));
     xRfTn.FcEnd   :=convTime(DBF.GetField(Rz, E_dbfFieldName("ENDTIME")));
     xRfTn.FBS     :=0;
     xRfTn.FES     :=0;
     xRfTn.Status  :=st_Imported;
     xRfTn.impFName:=impFN;
     xRfTn.PosInFil:=i;
     INSERT(xRfTn);
    END;
   END;
   DBF.Close(Rz);
  END;
*)

  MAKEPART(Err, Bot);
  Initialize();
 END {Import};

 PRIVATE PROCEDURE MakeIdxOnExit(D: DATE);
  VAR
   _Nbr   : LIKE(xRfTnH.grnbr);
   _Change: LIKE(xRfTnH.change);
 BEGIN
  SEARCH(Calendar, Date, , D);
  SELECT(StExit, Exit);
  WHILE OCCURE(StExit) DO
   PROMPT(" 室 " + StExit.Exit);
   IF (StExit.Strict = L_Way.ExitStrict_Reg) &
      (L_Way._CheckExitOnDate(StExit.Exit, D) = L_Way.exvld_Active) &
      E_PosToRoute(StExit.Exit) THEN
    _Nbr   :=Str.Slice(StExit.Exit, 4, 2);
    _Change:=E_convChangeOut(StExit.Change);
    IF SELECT(_Rou1, _Rou1, Route.Route, _Nbr, _Change) THEN
     L_StdFrm.Choice(" 室   䨪:|" + StExit.Exit + "  " + _Rou1._Exit);
    ELSE
     _Rou1._Route :=Route.Route;
     _Rou1._Nbr   :=_Nbr;
     _Rou1._Change:=_Change;
     _Rou1._Exit  :=StExit.Exit;
     INSERT(_Rou1);
    END;
   END;
   NEXT(StExit);
  END; (* WHILE OCCURE(StExit)... *)
 END {MakeIdxOnExit};

 PRIVATE PROCEDURE _AccessExit(_Exit: LIKE(StExit.Exit));
 BEGIN
  L_General.Assert(SEARCH(StExit, Exit, _Exit), "  室 "+_Exit);
 END {_AccessExit};

 PRIVATE PROCEDURE E_RouteCodeForExit(): STRING[3];
 BEGIN
  IF ~!Route.dSign THEN
   RETURN Route.dSign;
  ELSE
   RETURN L_Str.Right("00" + Route.Route, 3);
  END;
 END {E_RouteCodeForExit};

 PRIVATE PROCEDURE StandOnAddExit(_Route: LIKE(Route.Route); Mode: SCAL[3]): SCAL[2];
  VAR
   local   : PROC;
   _Exit   : LIKE(StExit.Exit);
   OkStExit: SCAL[2];
 BEGIN
  WHILE ~SEARCH(Route, Route, _Route) & (Str.Length(_Route) < 4) DO
   _Route:="0"+_Route;
  END;
  IF ~SEARCH(Route, Route, _Route) THEN
   locError("  " + _Route);
   RETURN 0;
  END;

  local:=PROCREF(E_Way.xRfTn_AddExitCode);
  IF !local & (Mode = mLosses) THEN
   (*⠭⭠ 㬥 室    ।ᬠਢ ᨩ*)
   _Exit:="000009" + E_RouteCodeForExit();
   IF SEARCH(StExit, Exit, _Exit) &
      L_DatTim.IsActive(Memo.ud_cd(), StExit.FromD, StExit.UpDate) THEN
    RETURN 1;
   ELSE
    locError(" ⥫쭮 室   " + _Route);
    RETURN 0;
   END;
  ELSE
   IF ~!local THEN
    _Exit:=local(_Route, Mode);
   ELSIF Mode = mFictExit THEN
    _Exit:=E_RouteCodeForExit() + "001000";
   ELSIF Mode = mOverTrips THEN
    _Exit:=E_RouteCodeForExit() + "002000";
   END;
 
   SELECT(StExit, Exit);
   IF SRCINSEL(StExit, GE, _Exit) THEN
    LOOP
     IF ~OCCURE(StExit) |
        (Str.Slice(StExit.Exit, 0, 7) # Str.Slice(_Exit, 0, 7)) THEN
      locAssert(0, " ⥫쭮 室   " + _Route);
      RETURN 0;
     END;
     IF L_DatTim.IsActive(Memo.ud_cd(), StExit.FromD, StExit.UpDate) THEN
      EXIT;
     END;
     NEXT(StExit);
    END; (* LOOP *)
    RETURN 1;
   ELSE
    RETURN 0;
   END;
  END;
 END {StandOnAddExit};

 PRIVATE PROCEDURE E_StandOnDebilCodedExit(_Route : LIKE(Route.Route);
                                   Nbr    : LIKE(xRfTnH.grnbr);
                                   _Change: LIKE(xRfTnH.change)): SCAL[2];
  VAR
   local: PROC;
 BEGIN
  local:=PROCREF(E_Way.xRfTn_StandOnDebilCodedExit);
  IF ~!local THEN
   RETURN local(_Route, Nbr, _Change);
  ELSE
   RETURN 0;
  END;
 END {E_StandOnDebilCodedExit};

 PRIVATE PROCEDURE AccessExit(_Route : LIKE(Route.Route);
                      Nbr    : LIKE(xRfTnH.grnbr);
                      _Change: LIKE(xRfTnH.change)): SCAL[2];
  VAR
   Result: SCAL[2];
   local : PROC;
 BEGIN
  IF ~SELECT(_Rou1, _Rou1, _Route) THEN
   locError("  " + _Route);
   Result:=0;
  ELSIF Nbr >= 30 THEN
   Result:=StandOnAddExit(_Route, mOverTrips);
  ELSE
   IF SELECT(_Rou1, _Rou1, _Route, Nbr, _Change) |
      ((_Change = 2) &
       SELECT(_Rou1, _Rou1, _Route, Nbr) &
       (Str.Slice(_Rou1._Exit, 3, 1) = "8")) |
      E_StandOnDebilCodedExit(_Route, Nbr, _Change) THEN
    Result:=1;
    _AccessExit(_Rou1._Exit);
   ELSE
    local:=PROCREF(E_Way.xRfTn_UseAddExitIfNoRegular);
    Result:=~!local & local() & StandOnAddExit(_Route, mOverTrips);
    locAssert(Result, " ⠪ ⢠ 室 " + _Route +
                                                 " : " + Nbr +
                                                 " : " + _Change);
   END;
  END;
  RETURN Result;
 END {AccessExit};

 PRIVATE PROCEDURE Initialize();
  VAR
   D: DATE;
 BEGIN
  D:=Memo.ud_cd();
  IF SELECT(xRfTnH, date, D) THEN
   MAKEPART(Err, Top, "஡ 樠樨   " + L_DatTim.DateStr(D));
   L_AFM.MakeIndex("_Rou1", "_Route : LIKE(Route.Route);"+
                               "_Nbr   : LIKE(xRfTnH.grnbr);"+
                               "_Change: LIKE(xRfTnH.change);"+
                               "_Exit  : LIKE(StExit.Exit)");
   MakeIdxOnExit(D);
   WHILE OCCURE(xRfTnH) DO
    PROMPT("樠 " + xRfTnH.boxnbr);
    IF xRfTnH.Status = st_Imported THEN
     IF AccessExit(E_convRouteIn(xRfTnH.route), xRfTnH.grnbr, xRfTnH.change) THEN
      xRfTnH.Exit  :=StExit.Exit;
      xRfTnH.Status:=st_Initial;
      REPLACE(xRfTnH);
     END;
    END;
    NEXT(xRfTnH);
   END; (* WHILE OCCURE(xRfTnH)... *)
   MAKEPART(Err, Bot);
  END;
 END {Initialize};

 PRIVATE PROCEDURE Deinitialize();
  VAR
   D: DATE;
 BEGIN
  D:=Memo.ud_cd();
  STDMSG("樠 ࠡ⠭ ᥩ  " + L_DatTim.DateStr(D), CHOICE);
  SELECT(xRfTnH, date, , D);
  WHILE OCCURE(xRfTnH) DO
   PROMPT("樠 " + xRfTnH.boxnbr);
   IF xRfTnH.Status = st_Initial THEN
    xRfTnH.Status:=st_Imported;
    REPLACE(xRfTnH);
   END;
   NEXT(xRfTnH);
  END; (* WHILE OCCURE(xRfTnH)... *)
 END {Deinitialize};

 PRIVATE PROCEDURE Delta(r1, r2: FLOAT): NUMERIC[1];
 BEGIN
  RETURN Math.Round(Math.Abs(r1 - r2), 0.1);
 END {Delta};

 PRIVATE PROCEDURE GetTrip(wjcode: LIKE(SEJobs.WJCode)): SCAL[3];
  CONST
   SearchPrecision = 5/60;
  VAR
   Day     : LIKE(SEJobs.Day);
   StartT  : LIKE(SEJobs.StartT);
   aeMode  : SCAL[3];
   RunDelta: NUMERIC[1];
 BEGIN
  IF E_Way.xRfTn_IsLossWJCode(wjcode) THEN
   aeMode:=mLosses;
  ELSIF ~!xRfTnH.Exit THEN
   Day   :=xRfTn.PlDay;
   StartT:=xRfTn.PlBeg + SearchPrecision;
   IF StartT <= (00:00 + SearchPrecision) THEN
    INC(Day);
   END;
   SELECT(SEJobs, Exit, , xRfTnH.Exit);
   SRCINSEL(SEJobs, LE, , Day, StartT);
   WHILE OCCURE(SEJobs) &
         (L_Way.FullBegTime(SEJobs.Day, SEJobs.StartT) >=
          (L_Way.FullBegTime(xRfTn.PlDay, xRfTn.PlBeg) - SearchPrecision)) DO
    IF (SEJobs.WJCode = wjcode) THEN
     RETURN 2;
    END;
    PREV(SEJobs);
   END;
   aeMode:=mFictExit;
  ELSE
   aeMode:=mOverTrips;
  END;
  IF StandOnAddExit(E_convRouteIn(xRfTnH.route), aeMode) THEN
   RunDelta:=xRfTn.Run;
   SELECT(SEJobs, Exit, , StExit.Exit);
   WHILE OCCURE(SEJobs) DO
    IF (SEJobs.Route = E_convRouteIn(xRfTnH.route)) & (SEJobs.WJCode = wjcode) &
       SEARCH(RoutMod, Route, SEJobs.Route, SEJobs.Modif) &
       (Delta(xRfTn.Run, RoutMod.Run) < RunDelta) THEN
     RunDelta:=Delta(xRfTn.Run, RoutMod.Run);
    END;
    NEXT(SEJobs);
   END; (* WHILE OCCURE(SEJobs) ... *)
   SELECT(SEJobs, Exit, , StExit.Exit);
   WHILE OCCURE(SEJobs) DO
    IF (SEJobs.Route = E_convRouteIn(xRfTnH.route)) & (SEJobs.WJCode = wjcode) &
       SEARCH(RoutMod, Route, SEJobs.Route, SEJobs.Modif) &
       (Delta(xRfTn.Run, RoutMod.Run) = RunDelta) THEN
     RETURN 1;
    END;
    NEXT(SEJobs);
   END; (* WHILE OCCURE(SEJobs) ... *)
  END;
  RETURN 0;
 END {GetTrip};

 PRIVATE PROCEDURE InitRWayS();
  VAR
   HasOtherStatusRWayS: SCAL[2];
   _RWayS             : LIKE(RWayS.RWayS);
   MsgStr             : STRING[];
 BEGIN
  HasOtherStatusRWayS:=0;
  SETEMPTY(_RWayS);

  SELECT(RWayS, TSN, , xRfTnH.tsn, xRfTnH.date);
  WHILE OCCURE(RWayS) DO
   IF OCCURE(RWayS) & (RWayS.BoxNbr = E_convBoxNbrIn(xRfTnH.boxnbr)) THEN
    _locAssert(RWayS.Status # L_Way.rwsStat_Given,
               " " + RWayS.RWayS + ' 室  ﭨ "뤠"');
    IF (RWayS.Status = L_Way.rwsStat_Ordered) |
       (RWayS.Status = L_Way.rwsStat_Taken) THEN
     IF !_RWayS | (RWayS.Exit = xRfTnH.Exit) THEN
      _locAssert(~L_Way.RWaySSomeTimeWas(L_XDoc.action_Edit),
                 " " + RWayS.RWayS + " ।஢");
      _RWayS:=RWayS.RWayS;
     END;
    ELSE
     HasOtherStatusRWayS:=1;
    END;
   END;
   NEXT(RWayS);
  END;

  IF HasOtherStatusRWayS THEN
   _locAssert(~!_RWayS, " 㦥 ⠪஢");
   STDMSG("E   " + L_DatTim.DateStr(xRfTnH.date) + ", ⥫ " + xRfTnH.tsn +
          ", ࠦ  " + xRfTnH.boxnbr +
          '| ﭨ "஢".' +
          "||  ஢   " + _RWayS, CHOICE);
  ELSIF !_RWayS THEN
   L_Car.StandOnCarLog(E_convBoxNbrIn(xRfTnH.boxnbr), xRfTnH.date);
   _locAssert(0 (* (CarLog.InBeg = L_Car.inBeg_DoesNot) *), "  拉    " + CarLog.BoxNbr);
  END;

  IF ~!_RWayS THEN
   SEARCH(RWayS, RWayS, , _RWayS);
   IF RWayS.Status = L_Way.rwsStat_Ordered THEN
    RWayS.Status:=L_Way.rwsStat_Taken;
    REPLACE(RWayS);
   ELSE
    L_General.CallUs(RWayS.Status = L_Way.rwsStat_Taken);
   END;
  ELSE
   DEFAULT(RWayS);
   RWayS.Date  :=xRfTnH.date;
   RWayS.TSN   :=xRfTnH.tsn;
   RWayS.BoxNbr:=E_convBoxNbrIn(xRfTnH.boxnbr);
   RWayS.Status:=L_Way.rwsStat_Taken;
   INSERT(RWayS);
   L_Way.CurRWayS_XDocJrn_Mark(L_XDoc.action_AutoBuild);
  END;
 END {InitRWayS};

 PRIVATE PROCEDURE IsLossRWayJ(): SCAL[2];
 BEGIN
  CONFORM(RWayJ, Quality);
  RETURN Quality.Type = L_Way.QualType_Annul;
 END {IsLossRWayJ};

 PRIVATE PROCEDURE FinishRWayS();
  VAR
   _Time   : TIME;
   _Day    : LIKE(RWayJ.Day);
   _Trip   : LIKE(RWayJ.Trip);
   _Quality: LIKE(RWayJ.Quality);
   _Fact   : LIKE(RWayJ.Fact);
 BEGIN
  CONFORM(RWayS, StExit);
  RWayS.Change  :=StExit.Change;
  RWayS.Gap     :=StExit.Gap;
  RWayS.DspConn :=StExit.DspConn;
  RWayS.Type    :=StExit.Type;

  _locAssert(SELECT(RWayJ, RWayS, RWayS.RWayS), "  ଠ쭮 ३");
  RWayS.StartTim:=RWayJ.StartT;
  LAST(RWayJ);

  WHILE OCCURE(RWayJ) & (!RWayJ.StopT | IsLossRWayJ()) DO
   PREV(RWayJ);
  END;
  IF OCCURE(RWayJ) THEN
   _Day :=RWayJ.Day + (RWayJ.StopT < RWayJ.StartT);
   _Time:=RWayJ.StopT;
   WHILE OCCURE(RWayJ) DO
    IF Math.Abs(L_Way.FullEndTime(RWayJ.Day, RWayJ.StartT, RWayJ.StopT) -
                L_Way.FullBegTime(_Day, _Time)) < TimeDiffAllowed THEN
     RWayJ.StopT:=_Time;
     REPLACE(RWayJ);
    END;
    _Day :=RWayJ.Day;
    _Time:=RWayJ.StartT;
    WHILE PREV(RWayJ) & (!RWayJ.StopT | IsLossRWayJ()) DO
    END;
   END; (* WHILE OCCURE(RWayJ) ... *)
  END;

  FIRST(RWayJ);
  WHILE OCCURE(RWayJ) DO
   IF ~!RWayJ.StopT & ~IsLossRWayJ()  THEN
    CALL(S_Tr_Fc.Compress());
   END;
   _Trip   :=RWayJ.Trip;
   _Quality:=RWayJ.Quality;
   _Day    :=RWayJ.Day;
   IF OCCURE(RWayJ) THEN
    NEXT(RWayJ);
   END;
   WHILE OCCURE(RWayJ) & (RWayJ.Quality = _Quality) &
         (RWayJ.Trip = _Trip) & (RWayJ.Day = _Day) & ~!RWayJ.StopT DO
    _Time:=RWayJ.StopT;
    _Fact:=RWayJ.Fact;
    PREV(RWayJ);
    INC(RWayJ.Fact, _Fact);
    RWayJ.StopT:=_Time;
    REPLACE(RWayJ);
    NEXT(RWayJ);
    REMOVE(RWayJ);
   END;
  END; (* WHILE OCCURE(RWayJ) ... *)
  REPLACE(RWayS);
  L_Way.CurRWayS_XDocJrn_Mark(L_XDoc.action_AutoEdit);
 END {FinishRWayS};

 PRIVATE PROCEDURE xRfTnID(): STRING[];
 BEGIN
  RETURN xRfTnH.boxnbr + "|" + xRfTnH.tsn + "|" +
         Str.Align(xRfTnH.route, 4, RIGHT) + "|" +
         Str.Align(xRfTnH.grnbr, 2, RIGHT) + "|" + xRfTnH.change + "|";
 END {xRfTnID};

 PRIVATE PROCEDURE _locError(Text: STRING[]);
 BEGIN
  IF streamMode THEN
   MAKEPART(Err, Line, xRfTnID() + Text);
   RAISE(1);
  ELSE
   L_General.Error(Text);
  END;
 END {_locError};

 PRIVATE PROCEDURE _locAssert(cond: SCAL[2]; Text: STRING[]);
 BEGIN
  IF ~cond THEN
   _locError(Text);
  END;
 END {_locAssert};

 PRIVATE PROCEDURE _locInform(Text: STRING[]);
 BEGIN
  IF streamMode THEN
   MAKEPART(Err, Line, xRfTnID() + Text);
  ELSE
   L_StdFrm.ShowINFORM_AsSTOP(Text);
  END;
 END {_locInform};

 PRIVATE PROCEDURE GetAcceptableTime(VAR day: LIKE(RWayJ.Day); VAR beg: TIME);
 BEGIN
  WHILE SEARCH(RWayJ, RWayS, RWayS.RWayS, day, beg) DO
   INC(beg, 1/60);
   IF beg < RWayJ.StartT THEN
    INC(day);
   END;
  END;
 END {GetAcceptableTime};

 PRIVATE PROCEDURE MakeRWayS();
  VAR
   fcDay,
   day    : LIKE(RWayJ.Day);
   fcBeg,
   beg    : TIME;
   Result : SCAL[3];
   wjcode : LIKE(WJMode.WJCode);
   quality: LIKE(Quality.Quality);
   _FBT,
   _FET   : NUMERIC[3];
 BEGIN
  TRY
   IF ~!xRfTnH.rways THEN
    _locError(" ᮧ  " + xRfTnH.rways);
   END;
   _locAssert(SEARCH(Staff, TSN, xRfTnH.tsn),
              " ⠡ " + xRfTnH.tsn);
   SELECT(xRfTnH, date, , xRfTnH.date, xRfTnH.boxnbr, xRfTnH.tsn);
   InitRWayS();
 
   WHILE OCCURE(xRfTnH) DO
    CONFORM(xRfTnH, xRfTn);
    WHILE OCCURE(xRfTn) DO
     IF ~((xRfTn.qualC = 0) & xRfTn.FBS & xRfTn.FES) THEN
      _locAssert(E_Way.xRfTn_SetQualityAndWJCode(wjcode, quality),
                 "३ " +xRfTn.FcDay + " " + L_DatTim.TimeStr(xRfTn.FcBeg) +
                 " ᮮ⢥⢨ ⢠  ࠡ");

      IF    xRfTn.FcEnd = xRfTn.FcBeg THEN
       _locInform("३ " +xRfTn.FcDay + " " +
                  L_DatTim.TimeStr(xRfTn.FcBeg) +
                  " 㫥 த⥫쭮 ३");
      ELSIF (xRfTn.FcEnd - xRfTn.FcBeg) > 22 THEN
       _locInform("३ " +xRfTn.FcDay + " " +
                  L_DatTim.TimeStr(xRfTn.FcBeg) +
                  " ६ 砭  ६ 砫");
      ELSIF ~!wjcode THEN
       fcDay:=xRfTn.FcDay;
       fcBeg:=xRfTn.FcBeg;
       IF SEARCH(RWayJ, RWayS, RWayS.RWayS, fcDay, fcBeg) THEN
        CONFORM(RWayJ, SEJobs);
        IF !RWayJ.StopT | E_Way.xRfTn_IsLossWJCode(SEJobs.WJCode) THEN
         SAVE(RWayJ);
         day:=RWayJ.Day;
         beg:=RWayJ.StartT;
         GetAcceptableTime(day, beg);
         RESTORE(RWayJ);
         RWayJ.Day   :=day;
         RWayJ.StartT:=beg;
         REPLACE(RWayJ);
        ELSE
         IF ~E_Way.xRfTn_IsLossWJCode(wjcode) THEN
          _locInform("३ "+xRfTn.FcDay+" "+L_DatTim.TimeStr(xRfTn.FcBeg)+
                     "  ३   ६???");
         END;
         GetAcceptableTime(fcDay, fcBeg);
        END;
       END;
       _locAssert(GetTrip(wjcode) > 0,
                  "३ " +xRfTn.FcDay + " " + L_DatTim.TimeStr(xRfTn.FcBeg) +
                  "  室饣 ३  ࠢ筨");
       CONFORM(SEJobs, RoutMod);
       RWayJ.Trip    :=SEJobs.Trip;
       RWayJ.Run     :=RoutMod.Run;
       RWayJ.RWayS   :=RWayS.RWayS;
       RWayJ.Day     :=fcDay;
       RWayJ.StartT  :=fcBeg;
       RWayJ.StopT   :=xRfTn.FcEnd;
       RWayJ.Quality :=quality;
       RWayJ.PlIncome:=0;
       RWayJ.FNorm   :=0;
       RWayJ.ByTS    :=0;
       IF E_Way.xRfTn_IsLossWJCode(wjcode) THEN
        RWayJ.Fact   :=0;
        IF ~!xRfTnH.Exit THEN
         SELECT(SEJobs, Exit, , xRfTnH.Exit);
         _FBT:=L_Way.FullBegTime(RWayJ.Day, RWayJ.StartT) - TimeDiffAllowed;
         _FET:=L_Way.FullEndTime(RWayJ.Day, RWayJ.StartT, RWayJ.StopT) +
               TimeDiffAllowed;
         WHILE OCCURE(SEJobs) &
               (L_Way.FullEndTime(SEJobs.Day,
                                  SEJobs.StartT,
                                  SEJobs.StopT) <= _FET) DO
          IF L_Way.FullBegTime(SEJobs.Day, SEJobs.StartT) >= _FBT THEN
           CONFORM(SEJobs, WJMode);
           INC(RWayJ.Fact, WJMode.IncSel & (WJMode.RunSel=L_Way.runsel_Work));
           _FBT:=L_Way.FullEndTime(SEJobs.Day, SEJobs.StartT, SEJobs.StopT);
          END;
          NEXT(SEJobs);
         END; (* WHILE OCCURE(SEJobs) ... *)
        END;
       ELSE
        RWayJ.Fact   :=1;
       END;
       INSERT(RWayJ);
       xRfTn.Status :=st_Done;
       REPLACE(xRfTn);
      END;
     END;
     NEXT(xRfTn);
    END; (* WHILE OCCURE(xRfTn) ... *)
    xRfTnH.rways :=RWayS.RWayS;
    xRfTnH.Status:=st_Done;
    REPLACE(xRfTnH);
    IF !RWayS.Exit THEN
     RWayS.Exit:=xRfTnH.Exit;
    END;
    NEXT(xRfTnH);
   END; (* WHILE OCCURE(xRfTnH) ... *)
   IF !RWayS.Exit THEN
    FIRST(xRfTnH);
    WHILE OCCURE(xRfTnH) DO
     _locError(" । 室");
     NEXT(xRfTnH);
    END; (* WHILE OCCURE(xRfTnH) ... *)
    RAISE(1);
   END;
   FIRST(xRfTnH);
   FinishRWayS();
  EXCEPT
   ON Exc: Exc_Raise DO
    IF Exc.ErrorCode # 1 THEN
     RAISE();
    END;
  END;
 END {MakeRWayS};

 PRIVATE PROCEDURE HasModificaton(): SCAL[2];
 BEGIN
  IF SELECT(XDocJrn, XDocJrn, L_XDoc.xdt_RWayS, L_XDoc.EmptyXDocKind, RWayS.RWayS) THEN
   LAST(XDocJrn);
   RETURN XDocJrn.Action # L_XDoc.action_AutoEdit; (*AutoEdit!!!*)
  END;
  RETURN 0;
 END {HasModificaton};

 PRIVATE PROCEDURE SetPreviousStatus();
 BEGIN
  SELECT(XDocJrn, XDocJrn, , L_XDoc.xdt_RWayS, L_XDoc.EmptyXDocKind, RWayS.RWayS);
  LAST(XDocJrn);
  L_General.CallUs(XDocJrn.Action = L_XDoc.action_AutoEdit);
  REMOVE(XDocJrn);
  LAST(XDocJrn);
  WHILE OCCURE(XDocJrn) &
        (XDocJrn.Action # L_XDoc.action_begRcp) &
        (XDocJrn.Action # L_XDoc.action_begRcpCncl) &
        (XDocJrn.Action # L_XDoc.action_begInsert) DO
   PREV(XDocJrn);
  END; (* WHILE OCCURE(XDocJrn) ... *)
  L_General.CallUs(OCCURE(XDocJrn));
  IF XDocJrn.Action = L_XDoc.action_begRcp THEN
   RWayS.Status:=L_Way.rwsStat_Taken;
  ELSE
   RWayS.Status:=L_Way.rwsStat_Ordered;
  END;
  REPLACE(RWayS);
 END {SetPreviousStatus};

 PRIVATE PROCEDURE UnmakeRWayS();
  VAR
   HasOtherRWayJ: SCAL[2];
 BEGIN
  TRY
   _locAssert(~!xRfTnH.rways, xRfTnID() + "   ᮧ");
   _locAssert(SEARCH(RWayS, RWayS, xRfTnH.rways), xRfTnID() + "   "+xRfTnH.rways);
   _locAssert(~HasModificaton(), xRfTnID() + "⥢  㦥 ஢.");
   SELECT(RWayJ, RWayS, , RWayS.RWayS);
   HasOtherRWayJ:=0;
   WHILE OCCURE(RWayJ) DO
    IF ~!RWayJ.StopT THEN
     REMOVE(RWayJ);
    ELSE
     HasOtherRWayJ:=1;
     NEXT(RWayJ);
    END;
   END; (* WHILE OCCURE(RWayJ) ... *)

   SetPreviousStatus();
 
   SELECT(xRfTnH, date, , xRfTnH.date, xRfTnH.boxnbr, xRfTnH.tsn);
   WHILE OCCURE(xRfTnH) DO
    SETEMPTY(xRfTnH.rways);
    IF ~!xRfTnH.Exit THEN
     xRfTnH.Status:=st_Initial;
    ELSE
     xRfTnH.Status:=st_Imported;
    END;
    REPLACE(xRfTnH);
    SELECT(xRfTn, xRfTnH, , xRfTnH.xRfTnH);
    WHILE OCCURE(xRfTn) DO
     xRfTn.Status:=st_Imported;
     REPLACE(xRfTn);
     NEXT(xRfTn);
    END; (* WHILE OCCURE(xRfTn) ... *)
    NEXT(xRfTnH);
   END; (* WHILE OCCURE(xRfTnH) ... *)
  EXCEPT
   ON Exc: Exc_Raise DO
    IF Exc.ErrorCode # 1 THEN
     RAISE();
    END;
  END;
 END {UnmakeRWayS};

 PRIVATE PROCEDURE Make();
  VAR
   D: DATE;
 BEGIN
  L_General.Assert(DBD.FileExist(xRfTnH), "  '࠭᭠樨'  祭");
  D:=Memo.ud_cd();
  ErrRepName:="BRTMk";
  streamMode:=1;
  STDMSG("ନ஢   "+L_DatTim.DateStr(D), CHOICE);
  IF SELECT(xRfTnH, date, D) THEN
   OPENREP(Err, L_General.NameConnection(ErrRepName), Create);
   MAKEPART(Err, Top, "஡   " + L_DatTim.DateStr(D));
   MAKEPART(Err, xRfTnIDDescription);
   WHILE OCCURE(xRfTnH) DO
    PROMPT("ࠡ⪠ " + xRfTnH.boxnbr);
    IF xRfTnH.Status # st_Done THEN
     SAVE(xRfTnH);
     MakeRWayS();
     RESTORE(xRfTnH);
     CHANGESEL(xRfTnH, date, , xRfTnH.date, xRfTnH.boxnbr, xRfTnH.tsn);
     LAST(xRfTnH);
     CHANGESEL(xRfTnH, date, , D);
    END;
    NEXT(xRfTnH);
   END; (* WHILE OCCURE(xRfTnH)... *)
   MAKEPART(Err, Bot);
   CLOSEREP(Err);
  END;
 END {Make};

 PRIVATE PROCEDURE Unmake();
  VAR
   D: DATE;
 BEGIN
  L_General.Assert(DBD.FileExist(xRfTnH), "  '࠭᭠樨'  祭");
  D:=Memo.ud_cd();
  ErrRepName:="BRTUM";
  streamMode:=1;
  STDMSG(" ନ஢   "+L_DatTim.DateStr(D), CHOICE);
  IF SELECT(xRfTnH, date, D) THEN
   OPENREP(Err, L_General.NameConnection(ErrRepName), Create);
   MAKEPART(Err, Top, "஡ ⬥   " + L_DatTim.DateStr(D));
   MAKEPART(Err, xRfTnIDDescription);
   WHILE OCCURE(xRfTnH) DO
    PROMPT("⬥  " + xRfTnH.boxnbr);
    IF xRfTnH.Status = st_Done THEN
     SAVE(xRfTnH);
     UnmakeRWayS();
     RESTORE(xRfTnH);
    END;
    NEXT(xRfTnH);
   END; (* WHILE OCCURE(xRfTnH)... *)
   MAKEPART(Err, Bot);
   CLOSEREP(Err);
  END;
 END {Unmake};

 PRIVATE PROCEDURE ExportOrder();
  VAR
   D         : DATE;
   dbfName,
   dbfBlankFN: STRING[];
   RecNo,
   TakenCnt  : NUMERIC[0];
 BEGIN
  dbfBlankFN:=FIO.BuildPath("n_trans", "dbf", path_Root());
  D:=Memo.ud_cd();
  SE_ACCESSDATAMODE(AccessDataMode_SOFT);
  dbfName:=FIO.BuildPath(GetDbfFileName("N_", D), "dbf", path_OUT());
  STDMSG("ନ஢ 䠩 ᯮ 鸞  " + L_DatTim.DateStr(D), CHOICE);
  L_FIO.FileCopy(dbfBlankFN, dbfName);
  DBF.Open(F, dbfName, Output);
  RecNo:=0;
  TakenCnt:=0;
  SELECT(RWayS, Date, , D);
  WHILE OCCURE(RWayS) DO
   PROMPT(RWayS.RWayS);
   CONFORM(RWayS, StExit);
   IF E_PosToRoute(StExit.Exit) & P_Route.HasActualRoutePro(Route.Route, D, P_Route.P_ASDC) THEN
    IF RWayS.Status # L_Way.rwsStat_Ordered THEN
     IF TakenCnt = 0 THEN
      L_StdFrm.OrdinaryCHOICE("  " + L_DatTim.DateStr(D) + " 㦥 뤠." +
                              "| 믮 ᢮६ ᯮ 鸞?");
     END;
     INC(TakenCnt);
    END;
    DBF.SetAmount(F, RecNo + 1);
    DBF.Seek(F, RecNo);
    DBF.SetField(F, "N_MARCH",  E_convRouteOut(Route.Route));
    DBF.SetField(F, "N_V",      L_Str.IntStr(E_convGrNbrOut(StExit.Exit)));
    DBF.SetField(F, "N_SMENA",  E_convChangeOut(RWayS.Change));
    DBF.SetField(F, "NOMASH",   E_convBoxNbrOut(RWayS.BoxNbr));
    DBF.SetField(F, "TABNOM",   RWayS.TSN);
    DBF.SetField(F, "ZAKREP",  "0");
    IF SELECT(WayLMen, RWayS, RWayS.RWayS, L_Way.wlmwho_Guard) THEN
     DBF.SetField(F, "TABCOND", WayLMen.TSN);
    END;
    IF L_ExtDB.DBF_FieldExist(dbfBlankFN, "NOMASH2") THEN
     IF SELECT(WayLCar, RWayS, RWayS.RWayS) THEN
      DBF.SetField(F, "NOMASH2", E_convBoxNbrOut(WayLCar.BoxNbr));
      L_General.Assert(~NEXT(WayLCar), "    ⥬?");
     ELSE
      DBF.SetField(F, "NOMASH2", "0");
     END;
    END;
    DBF.SetDeleted(F, 0);
    DBF.Update(F);
    INC(RecNo);
   END;
   NEXT(RWayS);
  END; (* WHILE OCCURE(RWayS) ... *)
  DBF.Close(F);

(*
  P_OwnData.InsertProperty(D,
                           E_Prop.P_XRfTn_Dbf_IsSent,
                           L_DatTim.CompleteSystimeStr(),
                           1,
                           "",
                           "",
                           L_General.EMPTY_DATE(),
                           D+1);
*)

  STDMSG("⮢ 䠩 " + dbfName, INFORM);
 END {ExportOrder};

 PRIVATE PROCEDURE xRfTnH_Exit_OnSuccessfulChange();
 BEGIN
  L_General.Assert(xRfTnH.Status # st_Done, " 㦥 ࠡ⠭");
  xRfTnH.Status:=~!xRfTnH.Exit;
  REPLACE(xRfTnH);
 END {xRfTnH_Exit_OnSuccessfulChange};

(*
$P{ rdr_Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
RepName, L, ""
Memo.ud_cd(), L, "D.N.y"
@
############################################################# ########## #####
=#####========================================================================
|   ஡  室묨 묨  ⥢묨 ⠬  ########## |
|----------------------------------------------------------------------------|
|     |஡  ||  |  ||||஡  .|宦.|
|-------|------------|---------|-----|-----|----|--|-|--------------|--------|
}
*)

(*
$P{ rdr_xRfTnHLine:
xRfTnH.xRfTnH, L, ""
xRfTnH.tsn, L, ""
xRfTnH.boxnbr, L, ""
xRfTnH.route, L, ""
xRfTnH.grnbr, L, ""
xRfTnH.change, L, ""
?, R, "2."
@
|       |            |#######  |#####|#####|####|##|#|##############|        |
}
*)

(*
$P{ rdr_SingleLine:
?, L, ""
?, R, "2."
xRfTnH.xRfTnH, L, ""
xRfTnH.tsn, L, ""
xRfTnH.boxnbr, L, ""
xRfTnH.route, L, ""
xRfTnH.grnbr, L, ""
xRfTnH.change, L, ""
?, R, "2."
?, R, "2."
@
|#######|############|#######  |#####|#####|####|##|#|##############|########|
}
*)

(*
$P{ rdr_TotalLine:
xRfTnH.rways, L, ""
?, R, "2."
?, R, "2."
?, R, "2."
@
|#######|############|       ᥣ  室 |##############|########|
}
*)

(*
$P{ rdr_Bot:
?, R, "2."
?, R, "2."
?, R, "2."
@
|-------|------------|-------------------------------|--------------|--------|
|⮣  |############|                               |##############|########|
=(c) Polak====================================================================
}
*)


 PRIVATE PROCEDURE GetRunFromRWayS(_RWayS: LIKE(RWayS.RWayS)): NUMERIC[2];
  VAR
   TotalRun: NUMERIC[2];
 BEGIN
  TotalRun:=0;
  SELECT(RWayJ, RWayS, , _RWayS);
  WHILE OCCURE(RWayJ) DO
   CONFORM(RWayJ, SEJobs);
   CONFORM(SEJobs, WJMode);
   CONFORM(RWayJ, Quality);
   IF WJMode.RunSel*Quality.Type > 0 THEN
    INC(TotalRun, RWayJ.Run * RWayJ.Fact);
   END;
   NEXT(RWayJ);
  END; (* WHILE OCCURE(RWayJ) ... *)
  RETURN TotalRun;
 END {GetRunFromRWayS};

 PRIVATE PROCEDURE GetRunFromxRfTnH(_xRfTnH: LIKE(xRfTnH.xRfTnH)): NUMERIC[2];
  VAR
   TotalRun: NUMERIC[2];
 BEGIN
  TotalRun:=0;
  SELECT(xRfTn, xRfTnH, , _xRfTnH);
  WHILE OCCURE(xRfTn) DO
   INC(TotalRun, xRfTn.Run);
   NEXT(xRfTn);
  END; (* WHILE OCCURE(xRfTn) ... *)
  RETURN TotalRun;
 END {GetRunFromxRfTnH};

 PRIVATE PROCEDURE MakeRunDifRep();
  VAR
   Amount       : NUMERIC[0];
   RWayS_Run,
   xRfTnH_Run,
   RWayS_TotRun,
   xRfTnH_TotRun: NUMERIC[2];
 BEGIN
  SE_ACCESSDATAMODE(AccessDataMode_SOFT);
  SETZERO(RWayS_TotRun, xRfTnH_TotRun);
  L_AFM.MakeIndex("tmp", "RWayS : LIKE(xRfTnH.rways);"+
                         "xRfTnH: LIKE(xRfTnH.xRfTnH)");
  SELECT(xRfTnH, date, , Memo.ud_cd());
  WHILE OCCURE(xRfTnH) DO
   PROMPT(" " + xRfTnH.xRfTnH);
   ADD(tmp, tmp, , xRfTnH.rways, xRfTnH.xRfTnH);
   NEXT(xRfTnH);
  END; (* WHILE OCCURE(xRfTnH) ... *)

  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, rdr_Top);
  SELECT(tmp, tmp);
  WHILE OCCURE(tmp) DO
   IF ~!tmp.RWayS THEN
    CHANGESEL(tmp, tmp, , tmp.RWayS);
    PROMPT(" " + tmp.RWayS);
    RWayS_Run:=GetRunFromRWayS(tmp.RWayS);
    INC(RWayS_TotRun, RWayS_Run);
   ELSE
    CHANGESEL(tmp, tmp, , tmp.RWayS, tmp.xRfTnH);
    PROMPT(" " + tmp.xRfTnH);
    RWayS_Run:=0;
   END;
   SETZERO(Amount, xRfTnH_Run);
   WHILE OCCURE(tmp) DO
    INC(Amount);
    INC(xRfTnH_Run, GetRunFromxRfTnH(tmp.xRfTnH));
    NEXT(tmp);
   END; (* WHILE OCCURE(tmp) ... *)
   INC(xRfTnH_TotRun, xRfTnH_Run);
   IF RWayS_Run # xRfTnH_Run THEN
    FIRST(tmp);
    SEARCH(xRfTnH, xRfTnH, , tmp.xRfTnH);
    IF Amount = 1 THEN
     IF !tmp.RWayS THEN
      MAKEPART(R, rdr_SingleLine,  " ",          , xRfTnH_Run);
     ELSE
      MAKEPART(R, rdr_SingleLine, tmp.RWayS, RWayS_Run, xRfTnH_Run,
                                  RWayS_Run - xRfTnH_Run);
     END;
    ELSE
     MAKEPART(R, rdr_TotalLine,  RWayS_Run, xRfTnH_Run, RWayS_Run - xRfTnH_Run);
     WHILE OCCURE(tmp) DO
      SEARCH(xRfTnH, xRfTnH, , tmp.xRfTnH);
      MAKEPART(R, rdr_xRfTnHLine, GetRunFromxRfTnH(tmp.xRfTnH));
      NEXT(tmp);
     END; (* WHILE OCCURE(tmp) ... *)
    END;
   END;
   LAST(tmp);
   CHANGESEL(tmp, tmp);
   NEXT(tmp);
  END; (* WHILE OCCURE(tmp) ... *)
  MAKEPART(R, rdr_Bot, RWayS_TotRun, xRfTnH_TotRun, RWayS_TotRun - xRfTnH_TotRun);
  CLOSEREP(R);
  L_AFM.AF_PurgeTemp("tmp");
 END {MakeRunDifRep};

 PRIVATE PROCEDURE iSwap_DirectConnect(): OBJECT;
  VAR
   Result: OBJECT;
 BEGIN
  TRY
   Result:=CreateOBJECT("ADODB.Connection");
   Result.ConnectionString:="Provider=SQLOLEDB.1;" +
                            "Password=15;" +
                            "Persist Security Info=True;" +
                            "User ID=atp15;" +
                            "Initial Catalog=TRANSNAVI;"+
                            "Data Source=SQLSERVER";
   Result.ConnectionTimeout:=15;
   Result.Open();
   RETURN Result;
  EXCEPT
  ON Exc: Exc_Object DO
   SETEMPTY(Result);
  END;
  RETURN Result;
 END {iSwap_DirectConnect};

 PRIVATE PROCEDURE iSwap_Execute_StoredProc(Name  : STRING[];
                                    PrmVal: STRING[][];
                                    PrmAmn: NUMERIC[0]): NUMERIC[0];
  VAR
   cm  : OBJECT;
   blin: OBJECT;
   i   : NUMERIC[0];
   Res : NUMERIC[0];
 BEGIN
  L_iSwap.Agent_Prompt("祭");
  cm:=CreateOBJECT("ADODB.Command");
  blin:=iSwap_DirectConnect();
  IF ~!blin THEN
   L_iSwap.Agent_Prompt("믮 楤");
   cm.ActiveConnection:=blin;
   cm.CommandText:=Name;
   cm.CommandType:=4(*adCmdStoredProc*);
   cm.Parameters.Refresh();
   i:=1;
   WHILE i <= PrmAmn DO
    cm.Parameters(i).Value:=^(PrmVal[i-1]);
    INC(i);
   END;
   cm.Execute();
   RETURN cm.Parameters(i).Value;
  ELSE
   RETURN L_General.EMPTY_NUMBER();
  END;
 END {iSwap_Execute_StoredProc};

 PRIVATE PROCEDURE iSwap_Send_ChangeBegRWS(Dct     : L_XML.tDictionary;
                                   _SrvDTS :FLOAT;
                                   VAR sr: L_iSwap.tSendingResult;
                                   VAR nr: STRING[];
                                   VAR dt: FLOAT);
  CONST
   PrmAmn = 6;
  VAR
   singleRWayS: STRING[];
   PrmVal     : STRING[][PrmAmn];
   espResult  : NUMERIC[0];
 BEGIN
  singleRWayS:=L_XML.ICN("鸞", L_XML.AssembleIdexedIN("⥢", 0));
  PrmVal[0]:=L_XML.dct_getStr(Dct, L_XML.ICN(singleRWayS, L_XML.AssembleIdexedIN("",      0)));
  PrmVal[1]:=L_XML.dct_getStr(Dct, L_XML.ICN(singleRWayS, L_XML.AssembleIdexedIN("",   0)));
  PrmVal[2]:=L_XML.dct_getStr(Dct, L_XML.ICN(singleRWayS, L_XML.AssembleIdexedIN("䨪",    0)));
  PrmVal[3]:=L_XML.dct_getStr(Dct, L_XML.ICN(singleRWayS, L_XML.AssembleIdexedIN("",     0)));
  PrmVal[4]:=L_XML.dct_getStr(Dct, L_XML.ICN(singleRWayS, L_XML.AssembleIdexedIN("ࠦ",  0)));
  PrmVal[5]:=L_XML.dct_getStr(Dct, L_XML.ICN(singleRWayS, L_XML.AssembleIdexedIN("", 0)));
  espResult:=iSwap_Execute_StoredProc("dspsp_ParkChangeNariad", PrmVal, PrmAmn);
  IF    !espResult    (* 믮 *) THEN
   IF (L_DatTim.GetSysVARIANTDATE() > (_SrvDTS + 15/(60*24))) THEN
    sr:=L_iSwap.sr_Impossible;
   ELSE
    sr:=L_iSwap.sr_Wait;
    dt:=10/(60*24);
   END;
  ELSIF espResult = -4(*  鸞  楫 *) THEN
   sr:=L_iSwap.sr_Postpone;
   dt:=30/(60*24);
  ELSIF espResult < 0 THEN
   sr:=L_iSwap.sr_Impossible;
  ELSE
   sr:=L_iSwap.sr_Sent;
  END;
  nr:=" = '" + FLDTOSTR(espResult, "0", 0) + "'";
 END {iSwap_Send_ChangeBegRWS};

 PRIVATE PROCEDURE iSwap_Value_AsStr(S: STRING[]): STRING[];
 BEGIN
  RETURN '"' + L_Str.Replace(S, '"', "'") + '"';
 END {iSwap_Value_AsStr};

 PRIVATE PROCEDURE DbfIsSent(D: DATE): SCAL[2];
  VAR
   S: STRING[];
 BEGIN
  S:=P_OwnData.GetOwnDProSoft(D, E_Prop.P_XRfTn_Dbf_IsSent);
  RETURN ~!S;
 END {DbfIsSent};

 PRIVATE PROCEDURE iSwap_BuildChangeBegRWS();
  VAR
   nRWayS: L_XML.tNode;
   vd    : FLOAT;
 BEGIN
  IF DbfIsSent(RWayS.Date) THEN
   (*  ᫮  ।,    ⠪- ᫮ 㦥 । *)
   SAVE(StExit);
   SAVE(Route);
   IF E_PosToRoute(RWayS.Exit) & P_Route.HasActualRoutePro(Route.Route, RWayS.Date, P_Route.P_ASDC) THEN
    CONFORM(RWayS, StExit);
    nRWayS:=L_XML.bd_PutNode(L_XML.bd_Create("鸞"), "⥢", "");
    PackVARIANTDATE(vd, RWayS.Date);
    L_XML.bd_PutLeafNode(nRWayS, "",      FLDTOSTR(vd, "6.", 0));
    L_XML.bd_PutLeafNode(nRWayS, "",   iSwap_Value_AsStr(E_convRouteOut(Route.Route)));
    L_XML.bd_PutLeafNode(nRWayS, "䨪",    FLDTOSTR(E_convGrNbrOut(StExit.Exit), "0", 0));
    L_XML.bd_PutLeafNode(nRWayS, "",     iSwap_Value_AsStr(E_convChangeOut(StExit.Change)));
    L_XML.bd_PutLeafNode(nRWayS, "ࠦ",  E_convBoxNbrOut(RWayS.BoxNbr));
    L_XML.bd_PutLeafNode(nRWayS, "", RWayS.TSN);
 (*
    L_XML.bd_Save(L_XML.bd_GetDocument(nRWayS), "S:\DEVS\XML\ISWAP.XML");
 *)
    L_iSwap.Send(L_iSwap.sp_TN, L_iSwap.sp_Own, L_iSwap.sk_ChangeBegRWS,
                 L_XDoc.curXDocId(L_XDoc.xdt_RWayS),
                 L_XML.GetAsStr(L_XML.bd_GetDocument(nRWayS)));
   END;
   RESTORE(Route);
   RESTORE(StExit);
  END;
 END {iSwap_BuildChangeBegRWS};

 PRIVATE PROCEDURE iSwap_Agent();
 BEGIN
  L_iSwap.Agent_Run(L_iSwap.sp_TN,
                    L_iSwap.sp_Own,
                    L_iSwap.sk_ChangeBegRWS,
                    PROCREF(iSwap_Send_ChangeBegRWS));
 END {iSwap_Agent};

 PUBLIC PROCEDURE PosToRWayS();
 BEGIN
  L_General.Assert(ACCESS(xRfTnH) & ~!xRfTnH.rways, "  ࠦ  ");
  L_XDoc.PositionInVideoSystem(L_XDoc.pm_StandOnly,
                               L_XDoc.XDocId(L_XDoc.xdt_RWayS, L_XDoc.EmptyXDocKind, xRfTnH.rways), "");
 END {PosToRWayS};

BEGIN
 IF    L_Events.IsInternalEvents(MODE) THEN
  ^(L_Events.GetInternalEventHandlerName(MODE))();
 ELSIF MODE = 100 THEN
  RepName:="xrt01";
  MakeRunDifRep();
 ELSIF MODE = 121 THEN
  ErrRepName:="BRTIn";
  streamMode:=1;
  OPENREP(Err, L_General.NameConnection(ErrRepName), Create);
  Initialize();
 ELSIF MODE = 122 THEN
  streamMode:=0;
  L_General.Assert(ACCESS(xRfTnH), " ⠭  믮 ");
  MakeRWayS();
 ELSIF MODE = 123 THEN
  streamMode:=0;
  L_General.Assert(ACCESS(xRfTnH), " ⠭  믮 ");
  UnmakeRWayS();
 ELSIF MODE = 134 THEN
  iSwap_Agent();
 ELSE
  L_General.BadMode(MODE);
 END;
END.
