(*
%Copyright   (c) Polak IT
%Author      23.03.2007 18:24:25 POLAK\GAV
%Purpose       
%Type        operation
%Specific    AP_PC
%Modified    19.02.2014 16:11:26 POLAK\GAV
%Environment
*)

 MODULE i_SocPsg3 [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT FIO, L_AFM, Str, L_Str, L_General, L_Way, L_DatTim, L_OwnData,
        E_Utils, Memo, L_StdFrm, L_DFS, L_TRS, L_PRoll, L_bp, L_DBD;

 PRIVATE CONST
  Delta = 10/60;
  dlm   = CHR(9);

 PRIVATE VAR
  RepName: STRING[];

 PRIVATE PROCEDURE E_GetImpFileNames(D: DATE): STRING[];
  VAR
   default: STRING[];
   local  : PROC;
 BEGIN
  default:="TXT\KASKAD\" + FLDTOSTR(D, "yND") + "_00000.txt";
  local:=PROCREF(E_Utils.isp_GetImpFileNames);
  IF ~!local THEN
   RETURN local(D, default);
  ELSE
   RETURN default;
  END;
 END {E_GetImpFileNames};

 PRIVATE PROCEDURE GetDate(s: STRING[]): DATE;
 BEGIN
  TRY
   L_General.DisableDebuggerExceptionBreak();
   RETURN STRTOFLD(s, DATE, "y-N-D");
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode = L_General.ec_rt_FailedSTRTOFLD THEN
    TRY
     L_General.DisableDebuggerExceptionBreak();
     RETURN STRTOFLD(s, DATE);
    EXCEPT
    ON Exc: Exc_RunTime DO
     IF Exc.ErrorCode = L_General.ec_rt_FailedSTRTOFLD THEN
      RETURN STRTOFLD(s, DATE, "D/N/Y");
     ELSE
      RAISE();
     END;
    END; (* TRY *)
   ELSE
    RAISE();
   END;
  END; (* TRY *)
 END {GetDate};

 PRIVATE PROCEDURE E_PosToRoute(D: DATE; route: STRING[]): SCAL[2];
  VAR
   local : PROC;
   Result: SCAL[2];
 BEGIN
  Result:=SEARCH(Route, Route, route) |
          SEARCH(Route, Route, "0" + L_Str.Left(route, 3)) |
          (L_Str.TestStrAlowChar(route, "09") & SELECT(Route, dSign, STRTOFLD(route, SCAL)));
  local:=PROCREF(E_Utils.isp_PosToRoute);
  IF ~!local THEN
   RETURN local(D, route, Result);
  ELSE
   RETURN Result;
  END;
 END {E_PosToRoute};

 PRIVATE PROCEDURE FillDT(part: STRING[]; VAR D: DATE; VAR T: TIME; VAR S: SCAL[60]);
 BEGIN
  D:=GetDate(Str.Item(part, " ", 0));
  part:=L_Str.AnyItem(part, " ", 1);
  S:=STRTOFLD(Str.Item(part, ":.", 2), SCAL);
  part:=Str.Item(part, ":", 0) + ":" + Str.Item(part, ":", 1);
  IF Str.Length(part) = 4 THEN
   part:="0" + part;
  END;
  T:=STRTOFLD(part, TIME);
 END {FillDT};

 PRIVATE PROCEDURE Remove_ElTck();
 BEGIN
  L_TRS.Unbind_ElTck();
  REMOVE(ElTck);
 END {Remove_ElTck};

 PRIVATE PROCEDURE PosToExistent(_VldId   : LIKE(ElTck.VldId);
                                 _ElTckId : LIKE(ElTck.ElTckId);
                                 _ElTckNum: LIKE(ElTck.ElTckNum);
                                 _DateIn  : LIKE(ElTck.DateIn);
                                 _TimeIn  : LIKE(ElTck.TimeIn);
                                 _SecIn   : LIKE(ElTck.SecIn)): SCAL[2];
  VAR
   Cnt: NUMERIC[0];
 BEGIN
  Cnt:=0;
  SELECT(ElTck, VldId, , _VldId, _DateIn, _TimeIn, _SecIn);
  WHILE OCCURE(ElTck) DO
   IF (_ElTckId = ElTck.ElTckId) & (_ElTckNum = ElTck.ElTckNum) THEN
    RETURN 1;
   END;
   INC(Cnt);
   NEXT(ElTck);
  END; (* WHILE OCCURE(ElTck) ... *)
  IF Cnt = 1 THEN
   MAKEPART(R, Line1, " ६ ᪠஢: " + _VldId + ", " +
                      L_DatTim.DateStr(_DateIn) + ", " +
                      L_DatTim.TimeStr(_TimeIn) + ":" + L_Str.FixedIntStr(_SecIn, 2));
  END;
  RETURN 0;
 END {PosToExistent};

 PRIVATE PROCEDURE MakeErrorLine(S, Text: STRING[]);
 BEGIN
  MAKEPART(R, Line1, Str.Replace(S, dlm, " "));
  MAKEPART(R, Line1, "   " + Text);
 END {MakeErrorLine};

 PRIVATE PROCEDURE GetStopFromFile(S: STRING[]; Idx: NUMERIC[0]; Descr: STRING[]): LIKE(ElTck.StopNIn);
  VAR
   Result   : LIKE(ElTck.StopNIn);
   tmpStop  : SCAL[1E3];
   StopAsStr: STRING[];
 BEGIN
  StopAsStr:=L_Str.AnyItem(S, dlm, Idx);
  tmpStop  :=STRTOFLD(StopAsStr, SCAL);
  IF tmpStop > MAXOF(Result) THEN
   SETEMPTY(Result);
   MakeErrorLine(S, Descr + " (" + StopAsStr + ") ॢ蠥 ᨬ쭮 ⨬.  ⠭  ஢.");
  ELSE
   Result:=tmpStop;
  END;
  RETURN Result;
 END {GetStopFromFile};

 PRIVATE PROCEDURE GetChangeFromFile(S, part: STRING[]): LIKE(ElTck.Change);
  VAR
   Result   : LIKE(ElTck.Change);
   tmpChange: SCAL[1E2];
 BEGIN
  tmpChange:=STRTOFLD(Str.Slice(part, 6, 1), SCAL);
  IF tmpChange > MAXOF(Result) THEN
   SETEMPTY(Result);
   MakeErrorLine(S, " ᬥ " + " (" + tmpChange + ") ॢ蠥 ᨬ쭮 ⨬.  ᬥ  ஢.");
  ELSE
   Result:=tmpChange;
  END;
  RETURN Result;
 END {GetChangeFromFile};

 PRIVATE PROCEDURE Insert_ElTck(D: DATE; S: STRING[]; Append: SCAL[2]);
  VAR
   part     : STRING[];
   _TSN     : LIKE(ElTck.TSN);
   _Route   : LIKE(ElTck.Route);
   _Schedule: LIKE(ElTck.Schedule);
   _Change  : LIKE(ElTck.Change);
   _VldId   : LIKE(ElTck.VldId);
   _ElTckId : LIKE(ElTck.ElTckId);
   _ElTckNum: LIKE(ElTck.ElTckNum);
   _DateIn  : LIKE(ElTck.DateIn);
   _TimeIn  : LIKE(ElTck.TimeIn);
   _SecIn   : LIKE(ElTck.SecIn);
   _StopNIn : LIKE(ElTck.StopNIn);
   _DateOut : LIKE(ElTck.DateOut);
   _TimeOut : LIKE(ElTck.TimeOut);
   _SecOut  : LIKE(ElTck.SecOut);
   _StopNOut: LIKE(ElTck.StopNOut);
   _ElTckT  : LIKE(ElTck.ElTckT);
   _Price   : LIKE(ElTck.Price);
 BEGIN
  _ElTckT  :=STRTOFLD(L_Str.AnyItem(S, dlm, 9), SCAL);
  IF (_ElTckT < 13) | (_ElTckT > 21) THEN (* "孮᪠ " . 0066482, 0070829*)
   FillDT(L_Str.AnyItem(S, dlm, 5), _DateIn,  _TimeIn,  _SecIn);
   FillDT(L_Str.AnyItem(S, dlm, 6), _DateOut, _TimeOut, _SecOut);
   _TSN     :=STRTOFLD(L_Str.AnyItem(S, dlm, 0), SCAL);
   L_General.Assert(SEARCH(Staff, TSN, _TSN), " ⠡ " + _TSN);
 
   part:=L_Str.AnyItem(S, dlm, 2);
   _Schedule:=STRTOFLD(Str.Slice(part, 4, 2), NUMERIC);
   _Change  :=GetChangeFromFile(S, part);
   part:=Str.Condense(Str.Slice(part, 0, 4));
   L_General.Assert(E_PosToRoute(D, part), "  " + part);
   _Route   :=Route.Route;
   _Price   :=STRTOFLD(L_Str.AnyItem(S, dlm, 10), NUMERIC);
   _VldId   :=L_Str.AnyItem(S, dlm, 1);
   _ElTckId :=L_Str.AnyItem(S, dlm, 3);
   _ElTckNum:=STRTOFLD(L_Str.AnyItem(S, dlm, 4), SCAL);
   _StopNIn :=GetStopFromFile(S, 7, " ⠭ 室");
   _StopNOut:=GetStopFromFile(S, 8, " ⠭ 室");
 
   IF PosToExistent(_VldId, _ElTckId, _ElTckNum, _DateIn, _TimeIn, _SecIn) THEN
    L_General.Assert(Append, "  ᪠஢     ᥪ㭤");
    IF (ElTck.TSN      = _TSN     ) & (ElTck.Route   = _Route  ) & (ElTck.Schedule = _Schedule) &
       (ElTck.Change   = _Change  ) & (ElTck.VldId   = _VldId  ) & (ElTck.ElTckId  = _ElTckId ) &
       (ElTck.ElTckNum = _ElTckNum) & (ElTck.DateIn  = _DateIn ) & (ElTck.TimeIn   = _TimeIn  ) &
       (ElTck.SecIn    = _SecIn   ) & (ElTck.StopNIn = _StopNIn) & (ElTck.DateOut  = _DateOut ) &
       (ElTck.TimeOut  = _TimeOut ) & (ElTck.SecOut  = _SecOut ) & (ElTck.StopNOut = _StopNOut) &
       (ElTck.ElTckT   = _ElTckT  ) & (ElTck.Price   = _Price  ) THEN
     L_General.Assert(L_DBD.lr_Label("ElTck"), " 㡫 ப");
     L_DBD.lr_Forget("ElTck");
     RETURN;
    ELSE
     L_General.Assert(L_DBD.lr_Label("ElTck"), "  ᪠஢     ᥪ㭤");
     MAKEPART(R, Line1, " : " + _VldId + ", " + _ElTckNum + ", " +
                                              L_DatTim.DateStr(_DateIn) + ", " +
                                              L_DatTim.TimeStr(_TimeIn) + ":" + L_Str.FixedIntStr(_SecIn, 2));
     L_DBD.lr_Forget("ElTck");
     Remove_ElTck();
    END;
   ELSE
    IF Append THEN
     MAKEPART(R, Line1, " : " + _VldId + ", " +  _ElTckNum + ", " +
                                           L_DatTim.DateStr(_DateIn) + ", " +
                                           L_DatTim.TimeStr(_TimeIn) + ":" + L_Str.FixedIntStr(_SecIn, 2));
    END;
   END;
   ElTck.TSN     :=_TSN;
   ElTck.Route   :=_Route;
   ElTck.badRoute:=0;
   ElTck.Schedule:=_Schedule;
   ElTck.Change  :=_Change;
   ElTck.VldId   :=_VldId;
   ElTck.ElTckId :=_ElTckId;
   ElTck.ElTckNum:=_ElTckNum;
   ElTck.DateIn  :=_DateIn;
   ElTck.TimeIn  :=_TimeIn;
   ElTck.SecIn   :=_SecIn;
   ElTck.StopNIn :=_StopNIn;
   ElTck.DateOut :=_DateOut;
   ElTck.TimeOut :=_TimeOut;
   ElTck.SecOut  :=_SecOut;
   ElTck.StopNOut:=_StopNOut;
   ElTck.ElTckT  :=_ElTckT;
   ElTck.Price   :=_Price;
   ElTck.Date    :=D;
   SETEMPTY(ElTck.TckRcS);
   IF (ElTck.DateIn - (ElTck.TimeIn < 03:00)) # D THEN
    L_General.Error("   " + L_DatTim.DateStr(D));
   END;
   INSERT(ElTck);
  END; (* IF (_ElTckT < 13) | (_ElTckT > 21) *)
 END {Insert_ElTck};

 PRIVATE PROCEDURE Cards_Make(Append: SCAL[2]; D: DATE);
  VAR
   S,
   ImpFileNames,
   ImpFileName,
   FullImpFileName: STRING[];
   i              : NUMERIC[0];
 BEGIN
  ImpFileNames:=E_GetImpFileNames(D);
  i:=0;
  ImpFileName:=L_Str.PoolItem(ImpFileNames, i);
  WHILE ~!ImpFileName DO
   L_General.Assert(FIO.Exists(ImpFileName, , FullImpFileName),
                    "   뢠 " + ImpFileName);

   FIO.Open(F, ImpFileName, , Input);
   WHILE FIO.RdStr(F, S) DO
    PROMPT(S);
    TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
     Insert_ElTck(D, S, Append);
    EXCEPT
    ON Exc: Exc_Root DO
     MakeErrorLine(S, Exc.Description)
    END;
   END; (* TRY *)
   FIO.Close(F);
   INC(i);
   ImpFileName:=L_Str.PoolItem(ImpFileNames, i);
  END;
 END {Cards_Make};

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
@
############################################################################################# ########## #####
=#####========================================================================================================
}
*)

(*
$P{ Line1:
?, L, ""
@
|############################################################################################################|
}
*)

(*
$P{ LineC:
?, C, ""
@
|############################################################################################################|
}
*)

(*
$P{ SubTop:
@
|------------------------------------------------------------------------------------------------------------|
}
*)

(*
$P{ fSubTop:
@
|------------------------------------------------------------------------------------------------------------|
|. |                              ||    |   |..|   ᪢|. |  |    ᥣ||
|-----|---------------------------------|----|-----|-----|---------|---------|---------|---------|---------|-|
}
*)


(*
$P{ LineStart:
Staff.TSN, L, ""
Man.Name, L, ""
?, L, ""
?, L, "H:M"
?, L, "H:M"
@
|#####|#################################|####|#####|#####}
*)

(*
$P{ LinePart:
?, R, ""
@
|#########}
*)

(*
$P{ LineStop:
?, R, ""
?, L, "!||?"
@
|#########|#|
}
*)

(*
$P{ Bot:
@
=(c) Polak====================================================================================================
}
*)

 PRIVATE PROCEDURE ftrjr_MakeLine(Exc: Exc_Root; VAR bdTRSCnt: NUMERIC[0]; Title: STRING[]);
 BEGIN
  IF bdTRSCnt = 0 THEN
   MAKEPART(R, LineC, Title);
   MAKEPART(R, SubTop);
  END;
  MAKEPART(R, Line1, TckRcS.TckRcS + "|" + Exc.Description);
  INC(bdTRSCnt);
 END {ftrjr_MakeLine};

 PRIVATE PROCEDURE FillTckRcJR(wCheckRoute: SCAL[2]; From, UpTo: DATE);
  VAR
   D         : DATE;
   Beg,
   End       : TIME;
   vpk,
   TCount,
   bdTRSCnt  : NUMERIC[0];
   Count     : NUMERIC[0][MAXOF(L_Way.tVldPsgKind)+1];
   prv       : SCAL[2];
   prvTSN    : LIKE(Staff.TSN);
   prvRoute  : LIKE(ElTck.Route);
   Accessible: SCAL[2];
 BEGIN
  L_AFM.MakeMultipleIndex("udTRS", "TckRcS: LIKE(TckRcS.TckRcS); Blocked: SCAL[2]", "udTRS: TckRcS");
  D:=From;
  WHILE D <= UpTo DO
   SELECT(udTRS, udTRS);
   REMSEL(udTRS);
   bdTRSCnt:=0;
   SELECT(ElTck, Date, , D);
   WHILE OCCURE(ElTck) DO
    CHANGESEL(ElTck, Date, , ElTck.Date, ElTck.TSN);
    PROMPT(L_DatTim.DateStr(ElTck.Date) + ", " + ElTck.TSN);
    WHILE OCCURE(ElTck) DO
     IF !ElTck.TckRcS & SELECT(TckRcS, TSN, ElTck.TSN, ElTck.Date) THEN
      WHILE OCCURE(TckRcS) & (L_TRS.StandOnJobByElTck(wCheckRoute) # L_TRS.sojbet_Found) DO
       NEXT(TckRcS);
      END; (* WHILE OCCURE(TckRcS) ... *)
      IF OCCURE(TckRcS) THEN
       TRY [TRANSACTION(TRANSACTION_SLAVE)]
        IF (TckRcS.Done | L_TRS.IsPartialDoneTRS()) & ~ADD(udTRS, udTRS, TckRcS.TckRcS) THEN
         udTRS.Blocked:=0;
         REPLACE(udTRS);
         SAVE(TckRcJ);
         CALL(TckRcS.SingleMakeUnDone());
         RESTORE(TckRcJ);
        END;
        PROMPT(L_DatTim.DateStr(ElTck.Date) + ", " + ElTck.TSN + ", " + L_DatTim.TimeStr(ElTck.TimeIn));
        IF ~TckRcS.Done & ~(SEARCH(udTRS, udTRS, TckRcS.TckRcS) & udTRS.Blocked) THEN
         ADD(TckRcJR, TckRcS, , TckRcJ.TckRcS, TckRcJ.Day, TckRcJ.StartT, L_TRS.TckKindOfVldCardType(ElTck.ElTckT),
                                TckRcJR.Sum, ElTck.Price,
                                TckRcJR.Psg, 1);
         ElTck.TckRcS:=TckRcS.TckRcS;
         IF ~L_TRS.E_IsGoodRouteForElTck(TckRcS.Date) THEN
          L_General.CallUs(~wCheckRoute);
          ElTck.badRoute:=1;
         END;
         L_TRS.ReplaceElTckAfterBind();
        END;
       EXCEPT
       ON Exc: Exc_Root DO
        ADD(udTRS, udTRS, , TckRcS.TckRcS);
        udTRS.Blocked:=Exc.Class = "Exc_AccessDeny";
        REPLACE(udTRS);
        ftrjr_MakeLine(Exc, bdTRSCnt, " 㤠 ⪠ ᫥騥 ");
       END; (* TRY *)
      END;
     END;
     NEXT(ElTck);
    END; (* WHILE OCCURE(ElTck) ... *)
    LAST(ElTck);
    CHANGESEL(ElTck, Date, , ElTck.Date);
    NEXT(ElTck);
   END; (* WHILE OCCURE(ElTck) ... *)
 
   IF bdTRSCnt > 0 THEN
    MAKEPART(R, SubTop);
   END;
 
   bdTRSCnt:=0;
   SELECT(udTRS, udTRS);
   WHILE OCCURE(udTRS) DO
    IF ~udTRS.Blocked THEN
     SEARCH(TckRcS, TckRcS, , udTRS.TckRcS);
     IF ~TckRcS.Done THEN
      TRY [TRANSACTION(TRANSACTION_SLAVE)]
       CALL(TckRcS.SingleMakeDone());
      EXCEPT
      ON Exc: Exc_Root DO
       ftrjr_MakeLine(Exc, bdTRSCnt, " 㤠 ࠡ ᫥騥 ");
      END; (* TRY *)
     END;
    END;
    NEXT(udTRS);
   END; (* WHILE OCCURE(udTRS) ... *)
   IF bdTRSCnt > 0 THEN
    MAKEPART(R, SubTop);
   END;

   IF D > From THEN
    MAKEPART(R, SubTop);
   END;
   MAKEPART(R, LineC, "᮪ ࠧᥭ   ᪠஢   " + L_DatTim.DateStr(D));
   MAKEPART(R, LineC, "஫ ⮢ " + L_Str.PoolItem("|祭", wCheckRoute));
   MAKEPART(R, fSubTop);
   prv:=1;
   SETEMPTY(prvTSN, prvRoute);
   SETZERO(TCount);
   SELECT(ElTck, Date, , D);
   LOOP
    IF ~OCCURE(ElTck) |
       ~prv       & (~!ElTck.TckRcS # prv) |
       ~!prvTSN   & (ElTck.TSN   # prvTSN) |
       ~!prvRoute & (ElTck.Route # prvRoute) THEN
     IF TCount # 0 THEN
      MAKEPART(R, LineStart, prvRoute, Beg, End);
      vpk:=0;
      WHILE vpk <= MAXOF(L_Way.tVldPsgKind) DO
       MAKEPART(R, LinePart, Count[vpk]);
       INC(vpk);
      END;
      MAKEPART(R, LineStop, TCount, prv);
      SETZERO(Count, TCount);
     END;
    END;
    IF ~OCCURE(ElTck) THEN
     EXIT;
    END;
    IF !prvTSN | (ElTck.TSN # prvTSN) THEN
     PROMPT(L_DatTim.DateStr(ElTck.Date) + ", " + ElTck.TSN);
    END;
    IF !ElTck.TckRcS &
       ((~!ElTck.TckRcS # prv) | !prvTSN | (ElTck.TSN # prvTSN) | !prvRoute | (ElTck.Route # prvRoute)) THEN
     CONFORM(ElTck, Staff);
     CONFORM(Staff, Man);
     Beg:=ElTck.TimeIn;
     SETZERO(Count, TCount);
    END;
    prvTSN  :=ElTck.TSN;
    prvRoute:=ElTck.Route;
    prv     :=~!ElTck.TckRcS;
    IF    !ElTck.TckRcS THEN
     End:=ElTck.TimeIn;
     INC(Count[L_TRS.VldPsgKindOfVldCardType(ElTck.ElTckT)]);
     INC(TCount);
    END;
    NEXT(ElTck);
   END; (* LOOP *)
   INC(D);
  END;
  L_AFM.AF_PurgeTemp("udTRS");
 END {FillTckRcJR};

 PRIVATE PROCEDURE RenovateElTck(D: DATE);
  VAR
   oc,
   Append: SCAL[2];
 BEGIN
  IF SELECT(ElTck, Date, D) THEN
   Append:=1;
   WHILE OCCURE(ElTck) DO
    PROMPT("⠭ ⪨  : " + ElTck.TSN + ", " + L_DatTim.TimeStr(ElTck.TimeIn));
    L_DBD.lr_Remember("ElTck");
    NEXT(ElTck);
   END; (* WHILE OCCURE(ElTck) *)
  ELSE
   Append:=0;
  END;
  Cards_Make(Append, D);
  IF Append THEN
   oc:=L_DBD.lr_PosToFirst("ElTck");
   WHILE oc DO
    PROMPT("஢ઠ 㤠 ᥩ: " + ElTck.TSN + ", " + L_DatTim.TimeStr(ElTck.TimeIn));
    MAKEPART(R, Line1, " 㤠: " + ElTck.VldId + ", " +
                                            L_DatTim.DateStr(ElTck.DateIn) + ", " +
                                            L_DatTim.TimeStr(ElTck.TimeIn) + ":" + L_Str.FixedIntStr(ElTck.SecIn, 2));
    Remove_ElTck();
    oc:=L_DBD.lr_PosToNext("ElTck");
   END; (* WHILE oc *)
  END;
 END {RenovateElTck};

 PRIVATE PROCEDURE SetAccessDataMode();
 BEGIN
  TABLE.Get("Staff").AccessDataMode:=adm_FINISHED;
  TABLE.Get("Man"  ).AccessDataMode:=adm_FINISHED;
  TABLE.Get("Route").AccessDataMode:=adm_FINISHED;
 END {SetAccessDataMode};

 PRIVATE PROCEDURE DateForCleanElTck(OnDate: DATE): DATE;
 BEGIN
   RETURN L_DatTim.IncMonth(L_DatTim.MonthBeg(OnDate), -6);
 END {DateForCleanElTck};

 PRIVATE PROCEDURE JustWarning(): SCAL[2];
 BEGIN
  IF AMOUNT(ElTck) >= 9000000 THEN
   RETURN 0;
  ELSIF SYSDATE() - L_DatTim.YearBeg(SYSDATE()) < 15 THEN
   RETURN ElTck.Date >= DateForCleanElTck(L_DatTim.YearBeg(SYSDATE()) - 1);
  ELSE
   RETURN ElTck.Date >= DateForCleanElTck(SYSDATE() - 7);
  END;
 END {JustWarning};

 PRIVATE PROCEDURE CleanOldElTck(D: DATE);
 BEGIN
  L_StdFrm.Question("।⥫쭮  㤠   ஭   " + L_DatTim.DateStr(D) +
                    "( 室 - ࠭祭  ࠬ 䠩  )." +
                    "||䨪   " + L_DatTim.DateStr(D) + " ⠭ .");
  WHILE SELECT(ElTck, Date) & (ElTck.Date < D) DO
   PROMPT(L_DatTim.DateStr(ElTck.Date));
   TRY [TRANSACTION(TRANSACTION_SLAVE), MSG_SUPPRESSION(MSG_SUPPRESSION_NONE)]
    CHANGESEL(ElTck, Date, , ElTck.Date);
    REMSEL(ElTck);
   EXCEPT
   END; (* TRY *)
  END; (* WHILE OCCURE(ElTck) ... *)
 END {CleanOldElTck};

 PUBLIC PROCEDURE Import();
  CONST
   bpId = "iScPs";
  VAR
   OnDate: DATE;
   msg   : STRING[];
 BEGIN
  L_bp.extBegin(bpId, "   ᠦ", L_bp.t_Prc_wRp, L_General.tc_onDate, L_bp.pt_StrictlyInTime);
  OnDate:=Memo.ud_cd();
  SetAccessDataMode();
  L_PRoll.Salary_CheckNoTouch(OnDate);
  L_General.Assert(~L_TRS.E_FillRWJP(OnDate),
                   "  " + L_DatTim.DateStr(OnDate) + " ᫥ 믮 '' ᯮᮡ");

  SELECT(ElTck, Date);
  IF OCCURE(ElTck) & (ElTck.Date < DateForCleanElTck(SYSDATE())) THEN
   CleanOldElTck(DateForCleanElTck(SYSDATE()));
  END;
  L_General.CallUs(AMOUNT(ElTck) < 9000000);
  L_General.Assert(OnDate >= DateForCleanElTck(SYSDATE()),
                   "  ஭   " + L_DatTim.DateStr(OnDate) + " 㦥 㤠");

  RepName:=bpId;
  OPENREP(R, L_General.NameConnection(RepName + FLDTOSTR(OnDate, "_DNy")), Create);
  MAKEPART(R, Top, RepName);
  MAKEPART(R, LineC, "⮪      " + L_DatTim.DateStr(OnDate));
  MAKEPART(R, SubTop);
  RenovateElTck(OnDate);
  MAKEPART(R, Bot);
  CLOSEREP(R);
  L_bp.End();
 END {Import};

 PUBLIC PROCEDURE ImportForYesterday();
  VAR
   sd: DATE;
 BEGIN
  sd:=Memo.ud_cd();
  Memo.ud_SetCurDate(SYSDATE()-1);
  Import();
  Memo.ud_SetCurDate(sd);
 END {ImportForYesterday};

 PRIVATE PROCEDURE Make_ForTheDate(trgRepName: STRING[]; wCheckRoute: SCAL[2]);
 BEGIN
  SetAccessDataMode();
  RepName:=trgRepName;
  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, Top, RepName);
  FillTckRcJR(wCheckRoute, Memo.ud_cd(), Memo.ud_cd());
  MAKEPART(R, Bot);
  CLOSEREP(R);
  L_General.SurveyLastReport();
 END {Make_ForTheDate};

 PUBLIC PROCEDURE dScPs();
 BEGIN
  Make_ForTheDate("dScPs", 1);
 END {dScPs};

 PUBLIC PROCEDURE fScPs();
 BEGIN
  Make_ForTheDate("fScPs", 0);
 END {fScPs};

 PRIVATE PROCEDURE Make_ForThePeriod(trgRepName: STRING[]; wCheckRoute: SCAL[2]);
 BEGIN
  SetAccessDataMode();
  RepName:=trgRepName;
  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, Top, RepName);
  MAKEPART(R, LineC, "ᥭ   ᪠஢   ਮ  " + L_DatTim.DateStr(Memo.ud_fr()) +
                     "  " + L_DatTim.DateStr(Memo.ud_ut()));
  MAKEPART(R, SubTop);
  FillTckRcJR(wCheckRoute, Memo.ud_fr(), Memo.ud_ut());
  MAKEPART(R, Bot);
  CLOSEREP(R);
 END {Make_ForThePeriod};

 PUBLIC PROCEDURE tScPs();
  CONST
   bpId = "tScPs";
 BEGIN
  L_bp.Begin(bpId, "ᥭ   ᪠஢   ", L_bp.t_Prc_wRp, L_General.tc_inIntvl);
  Make_ForThePeriod(bpId, 1);
  L_bp.End();
 END {tScPs};

 PUBLIC PROCEDURE pScPs();
  CONST
   bpId = "pScPs";
 BEGIN
  L_bp.Begin(bpId, "ᥭ   ᪠஢   ", L_bp.t_Prc_wRp, L_General.tc_inIntvl);
  Make_ForThePeriod(bpId, 0);
  L_bp.End();
 END {pScPs};

END.
