(*
%Copyright   (c) Polak IT
%Author      01.10.2008 12:59:49 POLAK\GAV
%Purpose
%Type        control,report,temporary,convertor,operation,library,to paste
%Specific    AP,AP_PC,AP_PT,SYS,TOOL,LOC
%Modified    15.01.2013 09:45:47 POLAK\MLB
%Environment
*)

 IMPORT L_Excel, FIO, L_Str, L_PayP, L_PDoc, L_Str, L_Staff,
        P_Firm, L_General, L_DatTim, Str, Memo;

 PROCEDURE GetFileName(): STRING[];
 BEGIN
  RETURN FIO.MakePath("  ᫥ " + PDoc.PDoc, "xls", "REP\TO_BANK");
 END {GetFileName};

 PROCEDURE GetItemForIndex(): STRING[];
 BEGIN
  RETURN "TimeKeep.Cash";
 END {GetItemForIndex};

 PROCEDURE SpecConformForIndexBy();
 BEGIN
  L_Staff.PosToStaffLog(Staff.TSN, PDoc.Date, 1);
  CONFORM(StaffLog, Officer);
  CONFORM(Officer, TimeKeep);
 END {SpecConformForIndexBy};

 PROCEDURE MayCreateInAutoMode(): SCAL[2];
 BEGIN
  RETURN ~L_PDoc.IsSingleTSN_PDoc();
 END {MayCreateInAutoMode};

 VAR
  Total  : NUMERIC[2];
  objXL,
  objWB,
  objWS  : OBJECT;
  curCash: LIKE(Cash.Cash);
  SysDTStr: STRING[];

 PROCEDURE PrologForPDoc();
  VAR
   _FileName: STRING[];
 BEGIN
  _FileName:=GetFileName();
  IF FIO.Exists(_FileName) THEN
   STDMSG("  䠩 " + _FileName, CHOICE);
   FIO.Erase(_FileName);
  END;

  L_General.Assert(L_Excel.ExcelInstalled(), " ⠭ MS Excel");
  objXL:=CreateOBJECT("Excel.Application");
  objXL.ScreenUpdating:=L_Excel.xlFalse;
  objWB:=L_Excel.NewWorkbook(objXL, 1);
  objWS:=L_Excel.NewWorksheet(objWB, 0, "BANKM");
  L_Excel.ws_Init(objWS);

  L_Excel.ws_SetFontSize(objWS, 10);
  L_Excel.ws_SkipCols(objWS, 1);
  L_Excel.ws_WrStr(objWS, "  ᫥ ࠡ⭮  㤭");
  L_Excel.ws_TermLine(objWS);
  L_Excel.ws_SkipCols(objWS, 1);
  L_PDoc.AccessPDocOwnFirmAcnt();
  IF ~!FirmAcnt.Name THEN
   L_Excel.ws_WrStr(objWS, FirmAcnt.Name);
  ELSE
   L_Excel.ws_WrStr(objWS, P_Firm.FP_ContractorBriefName(L_PDoc.GetPDocOwnFirm(), PDoc.Date));
  END;
  L_Excel.ws_TermLine(objWS);
  L_Excel.ws_SkipCols(objWS, 1);
  L_Excel.ws_WrStr(objWS, " " + Str.Cap(L_DatTim.MonthStr(Memo.gl_mb(), 0), 0, 1) + " " +
                          FLDTOSTR(Memo.gl_mb(), "y") + " .");
  L_Excel.ws_TermLine(objWS);
  L_Excel.ws_AddColumn(objWS, L_General.ft_STRING, 0, "", "");
  L_Excel.ws_AddColumn(objWS, L_General.ft_STRING, 0, "", "");
  L_Excel.ws_AddColumn(objWS, L_General.ft_NUMERIC, 2, "", "");
  L_Excel.ws_TermLine(objWS);
  SETEMPTY(curCash);
  SysDTStr:=L_DatTim.FullDateStr(SYSDATE()) + ". ६:" + L_DatTim.FullSystimeStr();
 END {PrologForPDoc};

 PROCEDURE EpilogForCash();
 BEGIN
  L_Excel.ws_PaintGrid(objWS, 2, 0);
  L_Excel.ws_WrStr(objWS, "⮣  ࠧ:");
  L_Excel.ws_SkipCols(objWS, 1);
  L_Excel.ws_WrNum(objWS, Total);
  L_Excel.ws_TermLine(objWS);
 END {EpilogForCash};

 PROCEDURE EpilogForPDoc();
 BEGIN
  EpilogForCash();
  L_Excel.ws_WrStr(objWS, ":");
  L_Excel.ws_SkipCols(objWS, 1);
  L_Excel.ws_WrNum(objWS, PDoc.Total);
  L_Excel.ws_TermLine(objWS);
  L_Excel.ws_TermLine(objWS);
  L_Excel.ws_WrStr(objWS, P_Firm.FP_LeadPostName1(L_PDoc.GetPDocOwnFirm(), PDoc.Date));
  L_Excel.ws_WrStr(objWS, "____________ (" + L_Str.ShortName(P_Firm.FP_LeadName(L_PDoc.GetPDocOwnFirm(), PDoc.Date)) + ")");
  L_Excel.ws_TermLine(objWS);
  L_Excel.ws_TermLine(objWS);

  L_Excel.ws_WrStr(objWS, P_Firm.FP_AcntPostName(L_PDoc.GetPDocOwnFirm(), PDoc.Date));
  L_Excel.ws_WrStr(objWS, "____________ (" + L_Str.ShortName(P_Firm.FP_AcntName(L_PDoc.GetPDocOwnFirm(), PDoc.Date)) + ")");
  L_Excel.ws_TermLine(objWS);

  objWB.SaveAs(FIO.BuildPath(GetFileName(), , FIO.AsbSpecialFolders("ClientProjectRoot")));
  L_Excel.CloseExcel(objXL, objWB);

  STDMSG("ନ஢ 䠩 " + GetFileName(), INFORM);
 END {EpilogForPDoc};

 PROCEDURE CreateDocument();
  VAR
   Sum : NUMERIC[2] [L_PayP.MaxSumAmnt];
   _Sum: LIKE(PDocList.Sum);
 BEGIN
  SEARCH(PDocList, PDoc, , PDoc.PDocKind, PDoc.PDoc, Deduct.TSN);
  PROMPT(" ⥫쭮 㬥: "+PDocList.TSN);
  CONFORM(PDocList, Staff);
  SpecConformForIndexBy();
  IF TimeKeep.Cash # curCash THEN
   IF ~!curCash THEN
    EpilogForCash();
   END;
   curCash:=TimeKeep.Cash;
   CONFORM(TimeKeep, Cash);
   Total:=0;

   L_Excel.ws_TermLine(objWS);
   L_Excel.ws_WrStr(objWS, "ࠧ: " + Cash.Name);
   L_Excel.ws_TermLine(objWS);
   L_Excel.ws_WrStr(objWS, " 祭: " + SysDTStr);
   L_Excel.ws_TermLine(objWS);
   L_Excel.ws_StartHeader(objWS);
   L_Excel.ws_AddHeaderItem(objWS, " ", 20, 0);
   L_Excel.ws_AddHeaderItem(objWS, ", , ⢮ 㤭", 45, 0);
   L_Excel.ws_AddHeaderItem(objWS, "㬬  ᫥", 19, 0);
   L_Excel.ws_StopHeader(objWS);
  END;

  L_Excel.ws_WrStr(objWS, Deduct.PCode);
  L_Excel.ws_WrStr(objWS, Deduct.Person);
  IF L_PDoc.IsSalPDoc() THEN
   _Sum:=PDocList.Sum;
  ELSE
   L_PayP.GetDedAcntDebts(Sum[0], Sum[1]);
   _Sum:=Sum[0]+Sum[1];
  END;
  INC(Total, _Sum);
  L_Excel.ws_WrNum(objWS, _Sum);
  L_Excel.ws_TermLine(objWS);
 END {CreateDocument};

END.