(*
%Copyright   (c) Polak Computer System  
%Author      SAN                        
%Purpose      ਢ  ﭨ "⠭ ..."
%Type        operation
%Specific    SYS
%Modified    20.08.2012 18:03:20 POLAK\Gina
%Environment
*)

 MODULE CalcDed [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_PRoll, L_APtm, L_DatTim, L_Staff, L_General, L_Env, L_Tax, L_STx, Memo, L_AP, L_AcsPnt, L_AR,
        L_Events, L_Str;

 TYPE
  tCountUpManner = SCAL[5];
  tForceType     = SCAL[4];

 CONST
  cup_Single         = 0;
  cup_Stream         = 1;
  cup_wKTM           = 2;
  cup_OtherContracts = 3;
  cup_PrvMonths      = 4;

  f_None = 0;
  f_Ave  = 1;
  f_Sht  = 2;
  f_Ded  = 3;

 VAR
  countCPW: NUMERIC[0];

 PUBLIC PROCEDURE ktm_Begin();
 BEGIN
  IF CALL(SheetP.ktm_DoIt()) THEN
   IF ~L_General.KioskExecution_IsRunning() THEN
    OPENREP(ktm_R, L_General.NameConnection("ktmSh"), Create);
   END;
   CALL(SheetP.ktm_MakePartTop());
  END;
 END {ktm_Begin};

 PUBLIC PROCEDURE ktm_End();
 BEGIN
  IF CALL(SheetP.ktm_DoIt()) THEN
   CALL(SheetP.ktm_MakePartBot());
   IF ~L_General.KioskExecution_IsRunning() THEN
    CLOSEREP(ktm_R);
   END;
  END;
 END {ktm_End};

 PRIVATE PROCEDURE ProceedBySht(FillSheetOnly: SCAL[2]);
  VAR
   First: SCAL[2];
 BEGIN
  SETZERO(PCalc.HBal, PCalc.CorBal, PCalc.TimDone);
  REPLACE(PCalc);
  HISTCONF(Memo.gl_me(), Staff, StaffLog);
  L_General.CallUs(OCCURE(StaffLog));
  First:=1;
  REPEAT
   IF L_Staff.ItsNotLastDischargeStaffLog() THEN
    SAVE(StaffLog);
    IF FillSheetOnly THEN
     CALL(SheetP.FillSheetByRegMain());
    ELSE
     CALL(SheetP.Proceed());
    END;
    RESTORE(StaffLog);
    First:=0;
   END;
  UNTIL ~PREV(StaffLog) | (~First & (StaffLog.DateE < Memo.gl_mb()));
  L_General.CallUs(~First);
 END {ProceedBySht};

 PRIVATE PROCEDURE CPWarning();
 BEGIN
  L_APtm.CheckIfBeforeSystemStart(Memo.gl_mb(), L_APtm.system_Salary);
  IF countCPW = 0 THEN
   L_PRoll.Salary_CheckClcPer();
  END;
  INC(countCPW);
 END {CPWarning};

 PRIVATE PROCEDURE CheckOtherContract(P: DATE): SCAL[2];
  VAR
   curTSN: LIKE(Staff.TSN);
   Result: SCAL[2];
 BEGIN
  SAVE(Staff);
  Result:=1;
  curTSN:=Staff.TSN;
  SELECT(Staff, MyFirm, , Staff.MyFirm, Staff.Man);
  WHILE OCCURE(Staff) & Result DO
   Result:=(Staff.TSN = curTSN) | ~L_Staff.IsBegan(P) | L_PRoll.IsDedD_AP(P);
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
  RESTORE(Staff);
  RETURN Result;
 END {CheckOtherContract};

 PRIVATE PROCEDURE ClearMoneyForSheet();
  VAR
   SourcesToClear: SCAL[2][MAXOF(Money.Source)+1];
 BEGIN
  SETZERO(SourcesToClear);
  SourcesToClear[L_PRoll.src_Sheet]   :=1;
  SourcesToClear[L_PRoll.src_SheetDec]:=1;
  SourcesToClear[L_PRoll.src_OrderP]  :=1;
  SourcesToClear[L_PRoll.src_OrderS]  :=1;
  L_PRoll.MultipleClearMoney(SourcesToClear);
  IF L_PRoll.Salary_Clc_CntIsEqual() THEN
   SELECT(Money, TSN, , Staff.TSN);
   SRCINSEL(Money, GE, , L_DatTim.IncMonth(Memo.gl_mb(), +1), Memo.gl_mb());
   WHILE OCCURE(Money) DO
    IF (Money.WFPeriod = Memo.gl_mb()) & (Money.Source = L_PRoll.src_FtSht) THEN
    L_PRoll.Remove_Money(0);
    END;
    NEXT(Money);
   END;
  END; (* IF Clc_CntIsE ... *)
  L_AP.ClearMonthly(Staff.TSN, Memo.gl_mb(), L_AP.mthlSrc_Sheet);
 END {ClearMoneyForSheet};

 PRIVATE PROCEDURE ClearMoneyForDedF();
  VAR
   SourcesToClear: SCAL[2][MAXOF(Money.Source)+1];
 BEGIN
  SETZERO(SourcesToClear);
  SourcesToClear[L_PRoll.src_System]:=1;
  SourcesToClear[L_PRoll.src_FtSys]:=1;
  SourcesToClear[L_PRoll.src_XCalc]:=1;
  L_PRoll.MultipleClearMoney(SourcesToClear);
 END {ClearMoneyForDedF};

 PRIVATE PROCEDURE AccessStaff();
 BEGIN
  L_General.Assert(ACCESS(Staff), " ⠭ ⥪騩 ࠡ⭨");
 END {AccessStaff};

 PUBLIC PROCEDURE intCountUp(cup: tCountUpManner; force: tForceType);
  VAR
   Prompt: PROMPTLINE;
   curTSN: LIKE(Staff.TSN);
 BEGIN
  L_General.CallUs(VAR_IMPL.Get(L_PRoll.curCountUp).Initialized);

  TRY
   AccessStaff();
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode = L_General.ec_rt_LockedByMainASL THEN
    IF cup = cup_PrvMonths THEN
     L_General.Error(" ।⥫쭮 믮   ।騥 ");
    ELSE
     L_General.CallUs(0);
    END;
   ELSE
    RAISE();
   END;
  END; (* TRY *)

  Prompt:=PROMPTLINE.Create(L_Str.PoolItem("|  ਮ", cup = cup_PrvMonths)
                            + ": " + Staff.TSN + ", " + L_DatTim.PeriodStr(Memo.gl_mb()));

  IF (cup = cup_Single) | (cup = cup_Stream) | (cup = cup_wKTM) THEN
   IF ~L_General.KioskExecution_IsRunning() THEN
    L_AcsPnt.PassAccessPoint(0000007);
   END;
   ProceedPrvMonth();
  ELSE
   L_General.CallUs((cup = cup_PrvMonths) | (cup = cup_OtherContracts));
  END;

  IF (cup = cup_PrvMonths) | (cup = cup_OtherContracts) | (cup = cup_Stream) THEN
   countCPW:=1;
  ELSE
   countCPW:=0;
  END;

  SAVE(Staff);
  IF cup # cup_OtherContracts THEN
   curTSN:=Staff.TSN;
   L_Staff.StandOnMainContractByStaff(Memo.gl_me());
   IF Staff.TSN # curTSN THEN
    INC(Prompt.Title, " (''  " + Staff.TSN + ")");
   END;
  END;

  IF Memo.gl_mb() < L_Env.aptm_Start THEN
   L_PRoll.FixPCalc(Memo.gl_mb());
   L_PRoll.ForcedSetAllDone();
   IF cup # cup_OtherContracts THEN
    ProceedOtherContract(force);
   END;
  ELSE
   CONFORM(Staff, Man);
 
   IF ~L_PRoll.IsAveD() | (force = f_Ave) THEN
    CPWarning();
    CALL(AvePay.AvePay_OnReproceedAll());
    L_General.CallUs(L_PRoll.IsAveD());
   END;
  
   IF ~L_PRoll.IsShtD() | (force = f_Sht) | (cup = cup_wKTM) THEN
    CPWarning();
    ClearMoneyForSheet();
    ktm_Begin();
    ProceedBySht(1);
    ProceedBySht(0);
    ktm_End();
    L_PRoll.ShtDone(1);
   END; (* IF ~L_PRoll.IsShtD() ... *)
  
   IF (~L_PRoll.IsDedD() | (force = f_Ded) | (cup = cup_OtherContracts)) | ~CheckOtherContract(Memo.gl_mb()) THEN
    IF L_PRoll.Salary_Clc_CntIsEqual() THEN
     IF cup # cup_OtherContracts THEN
      L_Tax.Tax_ClearFormedTaxAdv();
      L_STx.STx_ClearForSTax(Memo.gl_mb());
      L_PRoll.ClearMoneyFromXCalc();
      ProceedOtherContract(force);
     END;
     CPWarning();
     ClearMoneyForDedF();
     L_Staff.PosToStaffLog(Staff.TSN, Memo.gl_me(), 1);
     CALL(DedF.DedF());
    ELSE
     IF cup # cup_OtherContracts THEN
      ProceedOtherContract(force);
     END;
     CALL(DedF.DedF_BeforeClcPer());
    END;
   END; (* IF ... *)
   L_General.CallUs(L_PRoll.IsDedD());
   IF ~L_Env.RealTSN(Staff.TSN) & ((PCalc.Incr # 0) | (PCalc.Decr # 0)) THEN
    L_General.Error(" 䨪⨢ ࠡ⭨    ᥩ  ⭮ ");
   END;
  END; (* IF Memo.gl_mb() < L_Env.aptm_Start *)
  RESTORE(Staff);
 END {intCountUp};

 PRIVATE PROCEDURE ProceedOtherContract(force: tForceType);
  VAR
   curTSN: LIKE(Staff.TSN);
 BEGIN
  IF L_PRoll.curCountUp = L_PRoll.cu_snglFull_wKTM THEN
   L_PRoll.curCountUp:=L_PRoll.cu_snglFull;
  END;
  SAVE(Staff);
  curTSN:=Staff.TSN;
  SELECT(Staff, MyFirm, , Staff.MyFirm, Staff.Man);
  WHILE OCCURE(Staff) DO
   IF Staff.TSN # curTSN THEN
    IF  L_Staff.IsBegan(Memo.gl_mb()) THEN
     CALL(CalcDed.intCountUp(cup_OtherContracts, force));
    END;
   END;
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
  RESTORE(Staff);
 END {ProceedOtherContract};

 PRIVATE PROCEDURE ProceedPrvMonth();
  VAR
   cur_gl_cd,
   P        : DATE;
 BEGIN
  cur_gl_cd:=Memo.gl_cd();
  P:=L_DatTim.IncMonth(L_DatTim.Per(cur_gl_cd), -1);
  WHILE (P >= L_Env.aptm_Start) &
        ((L_Staff.IsBegan(P) & ~L_PRoll.IsDedD_AP(P)) | ~CheckOtherContract(P)) DO
   P:=L_DatTim.IncMonth(P, -1);
  END;
  TRY
   P:=L_DatTim.IncMonth(P, +1);
   WHILE P < L_DatTim.Per(cur_gl_cd) DO
    Memo.gl_SetCurDate(P);
    TRY [TRANSACTION(TRANSACTION_SLAVE), MSGSUPPRESSIONMODE(MSGSUPPRESS_NONE)]
     CALL(CalcDed.intCountUp(cup_PrvMonths, f_None));
    EXCEPT
    END;
    P:=L_DatTim.IncMonth(P, +1);
   END;
  EXCEPT
  FINALLY
   Memo.gl_SetCurDate(cur_gl_cd);
  END;
 END {ProceedPrvMonth};

 PRIVATE PROCEDURE intCountUp_Single(f: tForceType);
 BEGIN
  L_PRoll.curCountUp:=L_PRoll.cu_snglFull;
  intCountUp(cup_Single, f);
 END {intCountUp_Single};

 PUBLIC PROCEDURE CountUp();
 BEGIN
  intCountUp_Single(f_None);
 END {CountUp};

 PUBLIC PROCEDURE CountUp_forcedAve();
 BEGIN
  intCountUp_Single(f_Ave);
 END {CountUp_forcedAve};

 PUBLIC PROCEDURE CountUp_forcedSht();
 BEGIN
  intCountUp_Single(f_Sht);
 END {CountUp_forcedSht};

 PUBLIC PROCEDURE CountUp_forcedDed();
 BEGIN
  intCountUp_Single(f_Ded);
 END {CountUp_forcedDed};

 PUBLIC PROCEDURE CountUp_Stream();
 BEGIN
  intCountUp(cup_Stream, f_None);
 END {CountUp_Stream};

 PUBLIC PROCEDURE CountUp_wKTM();
 BEGIN
  L_PRoll.curCountUp:=L_PRoll.cu_snglFull_wKTM;
  intCountUp(cup_wKTM, f_Sht);
  IF ~L_General.KioskExecution_IsRunning() THEN
   LIKEMENU(Survey, "");
  END;
 END {CountUp_wKTM};

 PUBLIC PROCEDURE FillSheet();
 BEGIN
  AccessStaff();
  L_PRoll.curCountUp:=L_PRoll.cu_snglFull;
  L_AR.GrantRightsUnconditionally();
  L_PRoll.FixPCalc(Memo.gl_mb());
  ProceedBySht(1);
 END {FillSheet};

 PUBLIC PROCEDURE FillSheetOnDate();
 BEGIN
  AccessStaff();
  L_PRoll.curCountUp:=L_PRoll.cu_strmUpToDate;
  L_AR.GrantRightsUnconditionally();
  L_PRoll.FixPCalc(Memo.gl_mb());
  ProceedBySht(1);
 END {FillSheetOnDate};

 PUBLIC PROCEDURE ClearMoney_Sht_Ded();
 BEGIN
  AccessStaff();
  L_General.Assert(L_PRoll.Salary_Clc_CntIsEqual(), "⪠  ࠡ⭨  ⮫쪮  ⮬ ");
  ClearMoneyForSheet();
  ClearMoneyForDedF();
  L_PRoll.ShtDone(0);
 END {ClearMoney_Sht_Ded};

END.
