(*
%Copyright   (c) Polak Computer System
%Author      06.03.2001 13:18:11 SAN
%Purpose     ⥪  ࠡ  -  "஦"
%Type        library
%Specific    SYS
%Modified    12.03.2014 12:03:11 POLAK\Gina
%Environment
*)

 MODULE CB_BVz1;

 IMPORT L_General, L_DatTim, P_Firm, FIO, L_PDoc, L_Str, E_PDoc, Str,
        L_ExtAb, L_CB, L_EC;

 PUBLIC CONST
  Nick  = "CB_BVz1";
  BIC   = "044525181";

 PRIVATE CONST
  Payer     = 0;
  Recipient = 1;
  Format    = "PayDocRuXE";

 PRIVATE PROCEDURE ClientBankName(_CB: NUMERIC[0]): STRING[];
 BEGIN
  RETURN " '஦'";
 END {ClientBankName};

 PRIVATE PROCEDURE PutField(Value: STRING[]; maxLength: NUMERIC[0]);
 BEGIN
  IF ~!Value THEN
   FIO.WrStr(CDoc, Value);
  END;
  FIO.WrLn(CDoc);
 END {PutField};

 PRIVATE PROCEDURE OpenPacket(_CB: NUMERIC[0]): STRING[];
 BEGIN
  RETURN "";
 END {OpenPacket};

 PRIVATE PROCEDURE ClosePacket(_CB: NUMERIC[0]): STRING[];
 BEGIN
  RETURN "";
 END {ClosePacket};

 PRIVATE PROCEDURE FillCode(): STRING[];
  VAR
   _Code: STRING[];
 BEGIN
  _Code:=P_Firm.GetOurCode(Firm.Firm, PDoc.Date);
  IF ~!_Code THEN
   _Code:=": " + _Code;
  END;
  RETURN _Code;
 END {FillCode};

 PRIVATE PROCEDURE GetYearCode(): STRING[1];
  VAR
   y: NUMERIC[0];
 BEGIN
  y:=L_DatTim.YEAR(PDoc.Date);
  IF y < 2000 THEN
   L_General.Assert(y>=1990, "誮 쪨 ");
   RETURN CHR(ORD("0")+y-1990);
  ELSE
   L_General.Assert(y<=2000+ORD("Z"), "誮 让 ");
   RETURN CHR(ORD("A")+y-2000);
  END;
 END {GetYearCode};

 PRIVATE PROCEDURE ExpFirmData(Who: SCAL[2]);
  VAR
   KPP,
   Name,
   Bank,
   BIC,
   INN     : STRING[];
   Acnt    : STRING[][2];
 BEGIN
  IF Who = Payer THEN
   P_Firm.AccessOwnFirm(PDoc.MyFirm);
   P_Firm.CheckFirmAcnt(PDoc.Date, Firm.Firm, PDoc.OwnAcnt);
  ELSE
   CONFORM(PDoc, Firm);
   P_Firm.CheckFirmAcnt(PDoc.Date, Firm.Firm, PDoc.FirmAcnt);
  END;
  P_Firm.GetFirmData(PDoc.Date, Name, Bank, Acnt, BIC, INN, 1);
  IF Who = Payer THEN
   PutField(Acnt[0],  20); (* ⥫*)
   PutField(   Name, 160); (* ⥫*)
   SETEMPTY(KPP);
   IF L_PDoc.PDocMustHaveKPP() THEN
    KPP:=L_PDoc.GetPDocOwnFirmKPP();
   END;
   PutField(KPP, 09);      (* ⥫*)
  ELSE
   PutField(    BIC,  09); (* *)
   PutField(Acnt[0],  20); (* *)
   PutField(   Name, 160); (* *)
   PutField(    INN,  12); (* *)
  END;
 END {ExpFirmData};

 PRIVATE PROCEDURE Spec_DocDate(_DocDate: STRING[]; Idx: SCAL[5]): STRING[];
 BEGIN
  IF _DocDate = "0" THEN
   RETURN "0";
  ELSE
   RETURN L_Str.AnyItem(_DocDate, ".", Idx);
  END;
 END {Spec_DocDate};

 PRIVATE PROCEDURE E_UseNewPayDocRuXEFormat(D: DATE): SCAL[2];
 BEGIN
  RETURN L_EC.ExecE_Func_Scal2(D, PROCREF(E_PDoc.UseNewPayDocRuXEFormat), 1);
 END {E_UseNewPayDocRuXEFormat};

 PRIVATE PROCEDURE OpenFile(_OutFile: STRING[]; FirstPass: SCAL[2]);
 BEGIN
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   FIO.Open(CDoc, _OutFile, "", Create);
  EXCEPT
  ON Exc: Exc_Root DO
   IF (Exc.ErrorCode = 29) & FirstPass THEN
    FIO.Close(CDoc);
    OpenFile(_OutFile, 0);
   ELSE
    RAISE();
   END;
  END; (* TRY *)
 END {OpenFile};

 PRIVATE PROCEDURE UploadPDoc(_CB: NUMERIC[0]): STRING[];
  VAR
   _OutFile,
   OutFile: STRING[];
   curCode,
   _KPP,
   _Former,
   _KBK,
   _OKATO,
   _Reason,
   _Period,
   _DocNbr,
   _DocDate,
   _Type,
   PDocPmntKind_Pool: STRING[];
 BEGIN
  L_General.Assert(ACCESS(PDoc), " ⠭ ⥪騩 ⥦ 㬥");

  OutFile:=FLDTOSTR(PDoc.PDocKind, "", 2) +
           GetYearCode() +
           Str.Slice(PDoc.PDoc, Str.Length(PDoc.PDoc)-5, 5) + ".PDE";
  _OutFile:=E_PDoc.ClientBankUploadPath(_CB) + "\" + OutFile;

  OpenFile(_OutFile, 1);

  L_PDoc.CB_PutStamp(PDoc.PDocKind, PDoc.PDoc,
                     OutFile + ";" +
                     FLDTOSTR(SYSDATE(), "yND") + L_DatTim.LongSystimeStr() + ";" +
                     LOGINNAME());

  PutField("#1|"+Format,                                13); (* *)
  PutField(FLDTOSTR(PDoc.PrpDate, "D.N.y"),             10); (* *)
  PutField(PDoc.SrcNbr,                                 05); (* *)

  PutField("810", 03); (* *)

  ExpFirmData(Payer);
  ExpFirmData(Recipient);

  PutField(FLDTOSTR(PDoc.Total, "2.", 0),               15);  (*㬬*)
  PutField(L_PDoc.GetPDocComment() + " " + FillCode(), 240); (* ⥦*)
  PutField(L_PDoc.VatStr(),                             39);  (**)

  PutField(PDoc.Queue,    01);                    (*।*)
  PutField(FLDTOSTR(PDoc.Date, "D.N.y", 10), 10); (*ப*)
  PutField("01",          02);                    (* 樨*)

  IF E_UseNewPayDocRuXEFormat(PDoc.Date) THEN
   PDocPmntKind_Pool:="⮩|䮬|஭| ";
  ELSE
   PDocPmntKind_Pool:="⮩|䮬| | ";
  END;
  PutField(L_Str.PoolItem(PDocPmntKind_Pool, PDoc.PmntKind), 10); (*ᯮᮡ ࠢ*)

  L_ExtAb.SliceExtAbout(PDoc.ExtAbout, _Former, _KBK, _OKATO, _Reason,
                                       _Period, _DocNbr, _DocDate, _Type, curCode);
  IF L_PDoc.PDocMustHaveKPP() THEN
   _KPP:=P_Firm.FP_KPP(PDoc.Firm, PDoc.Date);
  ELSE
   SETEMPTY(_KPP);
  END;
  PutField(_KPP,    09);                          (* *)
  PutField(_Former, 02);                          (*(ନ⥫)*)
  PutField(_KBK,    19);                          (**)
  PutField(_OKATO,  11);                          (**)
  PutField(_Reason, 02);                          (*᭮ ⥦*)
  PutField(L_Str.AnyItem(_Period, ".", 0), 02);   (* ਮ:   ᫮ *)
  PutField(L_Str.AnyItem(_Period, ".", 1), 02);   (* ਮ: ਮ   *)
  PutField(L_Str.AnyItem(_Period, ".", 2), 04);   (* ਮ: ਮ   *)
  PutField(_DocNbr, 13);                          (* 㬥1*)
  PutField(Spec_DocDate(_DocDate, 0), 02);        (* -: ᫮   "0"*)
  PutField(Spec_DocDate(_DocDate, 1), 02);        (* 㬥: *)
  PutField(Spec_DocDate(_DocDate, 2), 04);        (* 㬥: *)
  PutField(_Type,   02);                          (* ⥦*)

  PutField(";end", 04);
  FIO.Close(CDoc);

  RETURN "";
 END {UploadPDoc};

 PRIVATE PROCEDURE CancelUpload(_CB: NUMERIC[0]): STRING[];
 BEGIN
  L_CB.CheckCancelUploadAllowed(L_CB.cua_Unknown);
  RETURN L_CB.CancelUpload_SinglePDoc(_CB);
 END {CancelUpload};

 PRIVATE PROCEDURE DownloadPDoc(_CB: NUMERIC[0]): STRING[];
 BEGIN
  RETURN L_CB.f1c_DownloadPDoc(_CB);
 END {DownloadPDoc};

 PRIVATE PROCEDURE CancelDownloadPDoc(_CB: NUMERIC[0]): STRING[];
 BEGIN
  RETURN L_CB.f1c_CancelDownloadPDoc(_CB);
 END {CancelDownloadPDoc};

 PRIVATE PROCEDURE ViewSrcFile(_CB: NUMERIC[0]): STRING[];
 BEGIN
  RETURN L_CB.f1c_ViewDownloadSrcFile(_CB);
 END {ViewSrcFile};

END.
