(*
%Copyright   (c) Polak Computer System
%Author      19.06.2003 10:15:31 POLAK\San
%Purpose     ஫ 䨪樨 ⥩ 室  室
%Type        control
%Specific    SYS
%Modified    30.07.2014 19:24:31 POLAK\San
%Environment
*)

 MODULE CostItem [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_Events, L_XObj, L_DBD, L_Acnt, L_WDO, L_StdFrm, L_Str, L_DatTim, L_CI, L_APtm,
        P_Firm, Str, DBD, Math, L_SysLog, L_r2r, L_OwnData, P_MCList;

 PRIVATE CONST
  ci_Pool = "㯯 |       ";

 PRIVATE PROCEDURE SetCostItemFieldModifiable(trgLeaf, IsInsert: SCAL[2]);
  VAR
   Fields: FORMFIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Fields("CostItem.iuPrefix").Modifiable:=trgLeaf = 0;
  Fields("CostItem.MyFirm").Modifiable  :=P_Firm.E_wMultipleMyFirm(SYSDATE()) & (IsInsert | (trgLeaf = 0));
 END {SetCostItemFieldModifiable};

 PRIVATE PROCEDURE HasNameDelimiter(_Name: STRING[]): SCAL[2];
  VAR
   LastChar: STRING[1];
 BEGIN
  IF ~!_Name THEN
   LastChar:=L_Str.Char(_Name, Str.Length(_Name)-1);
   RETURN (LastChar = ":") |
          (LastChar = "/") |
          (LastChar = "\");
  ELSE
   RETURN 1;
  END;
 END {HasNameDelimiter};

 PRIVATE PROCEDURE NeedSmallFirstCharIn_iuPrefix(_iuPrefix: STRING[]): SCAL[2];
  VAR
   FirstChar,
   SecondChar: STRING[1];
 BEGIN
  FirstChar :=L_Str.Char(_iuPrefix, 1);
  SecondChar:=L_Str.Char(_iuPrefix, 2);
  RETURN (L_Str.TestStrAlowChar(FirstChar, "") | L_Str.TestStrAlowChar(FirstChar, "AZ")) & (Str.Cap(SecondChar) # SecondChar);
 END {NeedSmallFirstCharIn_iuPrefix};

 PRIVATE PROCEDURE GetRealName(_Name, _iuPrefix: STRING[]): STRING[];
 BEGIN
  IF ~HasNameDelimiter(_Name) & ~!_iuPrefix THEN
   _Name:=_Name + " ";
  END;

  IF NeedSmallFirstCharIn_iuPrefix(_iuPrefix) THEN
   _iuPrefix:=Str.Small(L_Str.Char(_iuPrefix, 0)) + L_Str.RightFrom(_iuPrefix, 1);
  END;

  RETURN _Name + _iuPrefix;
 END {GetRealName};

 PRIVATE PROCEDURE GetParentAttr(    _Parent  : LIKE(CostItem.Parent);
                                 VAR _iuPrefix: STRING[];
                                 VAR _ACSet   : LIKE(ACSet.ACSet);
                                 VAR _MyFirm  : P_Firm.tMyFirm);
 BEGIN
  IF ~!_Parent THEN
   SAVE(CostItem);
   SEARCH(CostItem, CostItem, , _Parent);
   _iuPrefix:=CostItem.iuPrefix;
   _ACSet   :=CostItem.ACSet;
   _MyFirm  :=CostItem.MyFirm;
   RESTORE(CostItem);
  ELSE
   SETEMPTY(_iuPrefix, _ACSet, _MyFirm);
  END;
 END {GetParentAttr};

 PRIVATE PROCEDURE CostItem_OnBeforeInsert();
  VAR
   Fields                : FORMFIELDS;
   MyFirmFieldFlagFirst  : SCAL[2];
   parent_iuPrefix       : STRING[];
   parentACSet           : LIKE(ACSet.ACSet);
   parentMyFirm          : P_Firm.tMyFirm;
 BEGIN
  GetParentAttr(CostItem.Parent, parent_iuPrefix, parentACSet, parentMyFirm);
  IF L_WDO.InHierarchyInsMode() THEN
   CostItem.Leaf  :=0;
  ELSE
   CostItem.Leaf  :=1;
  END;

  IF ~P_Firm.E_wMultipleMyFirm(SYSDATE()) THEN
   CostItem.MyFirm  :=P_Firm.DefineMyFirm();
  ELSE
   IF (parentMyFirm # P_Firm.mf_Fict) | (CostItem.Leaf = 0) THEN
    CostItem.MyFirm:=parentMyFirm;
   END;
  END;
  CostItem.Name    :=parent_iuPrefix;
  IF CostItem.Leaf = 0 THEN
   CostItem.iuPrefix:=parent_iuPrefix;
  END;
  CostItem.ACSet   :=parentACSet;
  REPLACE(CostItem);
  SetCostItemFieldModifiable(CostItem.Leaf, 1);

  MyFirmFieldFlagFirst:=(CostItem.Leaf = 1) & (!CostItem.MyFirm | (CostItem.MyFirm = P_Firm.mf_Fict));
  Fields:=RTI.Get().CurZone.Fields;
  Fields("CostItem.MyFirm").First:=MyFirmFieldFlagFirst;
  Fields("CostItem.iiName").First:=~MyFirmFieldFlagFirst;
 END {CostItem_OnBeforeInsert};

 PRIVATE PROCEDURE CostItem_OnBeforeEdit();
 BEGIN
  SetCostItemFieldModifiable(CostItem.Leaf, 0);
  RTI.Get().CurZone.Fields("CostItem.iiName").First:=1;
 END {CostItem_OnBeforeEdit};

 PRIVATE PROCEDURE CheckCostItemIntervals(_Begin,
                                          _Till                    : DATE;
                                          _CostItem                : LIKE(CostItem.CostItem);
                                          _Leaf                    : LIKE(CostItem.Leaf);
                                          CheckIntvlInCostItemIntvl: SCAL[2]);
  VAR
   Result: SCAL[2];
 BEGIN
  IF CheckIntvlInCostItemIntvl THEN
   Result:=L_DatTim.InPatIntvl(CostItem.Begin, CostItem.Till, _Begin, _Till);
  ELSE
   Result:=L_DatTim.InPatIntvl(_Begin, _Till, CostItem.Begin, CostItem.Till);
  END;
  L_General.Assert(Result,
                  "ਮ ਬ  ᮣᮢ:||" +
                  L_Str.PoolItem(ci_Pool, _Leaf)         + " (" + _CostItem         + ")  " + L_DatTim.DateStr(_Begin)         + "  " + L_DatTim.DateStr(_Till) + "|" +
                  L_Str.PoolItem(ci_Pool, CostItem.Leaf) + " (" + CostItem.CostItem + ")  " + L_DatTim.DateStr(CostItem.Begin) + "  " + L_DatTim.DateStr(CostItem.Till));
 END {CheckCostItemIntervals};

 PRIVATE PROCEDURE CheckCostItem();
  VAR
   _Begin,
   _Till    : DATE;
   _CostItem: LIKE(CostItem.CostItem);
   _Leaf    : LIKE(CostItem.Leaf);
 BEGIN
  SAVE(CostItem);
  _CostItem:=CostItem.CostItem;
  _Begin   :=CostItem.Begin;
  _Till    :=CostItem.Till;
  _Leaf    :=CostItem.Leaf;
  IF ~_Leaf THEN
   SELECT(CostItem, Parent, , _CostItem);
   WHILE OCCURE(CostItem) DO
    CheckCostItemIntervals(_Begin, _Till, _CostItem, _Leaf, 0);
    NEXT(CostItem);
   END; (* WHILE OCCURE(CostItem) ... *)
  END;
  RESTORE(CostItem);
 END {CheckCostItem};

 PRIVATE PROCEDURE CostItem_OnAfterEdIns();
 BEGIN
  L_DBD.CheckOwnersValidity("CostItem");
  IF CostItem.Leaf = 1 THEN
   L_General.Assert(~!CostItem.Parent, " 㪠 㯯 ⥩ 室  室");
   L_General.Assert(CostItem.MyFirm # P_Firm.mf_Fict, " 室  室   ⭮   0000");
   L_General.Assert(CostItem.ACSet # L_Acnt.fictACSet, "  室  室    㪠 䨪⨢   ࠧ१");
  END;
  CheckCostItem();
 END {CostItem_OnAfterEdIns};

 PRIVATE PROCEDURE CostItem_OnAfterInsert();
 BEGIN
  IF CostItem.Leaf = 1 THEN
   L_XObj.AfterOriginInsert(L_XObj.o_CostItem, CostItem.MyFirm);
  END;
  CostItem_OnAfterEdIns();
 END {CostItem_OnAfterInsert};

 PRIVATE PROCEDURE CostItem_OnAfterEdit();
 BEGIN
  CostItem_OnAfterEdIns();
 END {CostItem_OnAfterEdit};

 PRIVATE PROCEDURE CostItem_OnBeforeDelete_Explicit(wRemove: SCAL[2]);
  VAR
   curCostItem: LIKE(CostItem.CostItem);
   curIntegrity: SCAL[2];
 BEGIN
  IF CostItem.Leaf = 1 THEN
   L_XObj.BeforeOriginDelete(L_XObj.o_CostItem);
  ELSE
   SAVE(CostItem);
   curCostItem:=CostItem.CostItem;
   WHILE SELECT(CostItem, Parent, curCostItem) DO
    CostItem_OnBeforeDelete_Explicit(1);
   END;
   RESTORE(CostItem);
  END;
  IF wRemove THEN
   curIntegrity:=GE_INTEGRITY();
   SE_INTEGRITY(1);
   REMOVE(CostItem);
   SE_INTEGRITY(curIntegrity);
  END;
 END {CostItem_OnBeforeDelete_Explicit};

 PRIVATE PROCEDURE CostItem_OnBeforeDelete();
 BEGIN
  CostItem_OnBeforeDelete_Explicit(0);
  L_DBD.CheckUsers(GE_CURFILE());
  L_r2r.Owner_OnBeforeDelete("CostItem");
 END {CostItem_OnBeforeDelete};

 PRIVATE PROCEDURE TestIfChangeCostItemACSet(oldACSet, newACSet: LIKE(ACSet.ACSet));
 BEGIN
  IF ~!oldACSet & (oldACSet # newACSet) THEN
   L_General.Assert_wReferToAdmin(L_DBD.TestUsers("CostItem", 1), "   ࠧ१  ᯮ饩  室  室");
  END;

  IF CostItem.Leaf = 1 THEN
   L_XObj.CheckXCalcIsZero(CostItem.XObject);
  END;
 END {TestIfChangeCostItemACSet};

 PRIVATE PROCEDURE GetCostItemParentName(): STRING[];
  VAR
   ci_ParentName: STRING[];
 BEGIN
  SETEMPTY(ci_ParentName);
  IF ACCESS(CostItem) & ~!CostItem.Parent THEN
   SAVE(CostItem);
   SEARCH(CostItem, CostItem, , CostItem.Parent);
   ci_ParentName:=CostItem.Name;
   RESTORE(CostItem);
  END;
  RETURN ci_ParentName;
 END {GetCostItemParentName};

 PRIVATE PROCEDURE ListCostItemIsSameMyFirm(VAR _MyFirm: P_Firm.tMyFirm): SCAL[2];
 BEGIN
  SAVE(CostItem);
  IF SELECT(CostItem, Parent, CostItem.CostItem) THEN
   WHILE OCCURE(CostItem) DO
    IF (_MyFirm = P_Firm.mf_Fict) THEN
     _MyFirm:=CostItem.MyFirm;
    END;
    IF CostItem.MyFirm # P_Firm.mf_Fict THEN
     IF CostItem.MyFirm # _MyFirm THEN
      RESTORE(CostItem);
      RETURN 0;
     END;
    ELSE
     IF ~ListCostItemIsSameMyFirm(_MyFirm) THEN
      RESTORE(CostItem);
      RETURN 0;
     END;
    END;
    NEXT(CostItem);
   END; (* WHILE OCCURE(CostItem)...*)
  END;
  RESTORE(CostItem);
  RETURN 1;
 END {ListCostItemIsSameMyFirm};

 PRIVATE PROCEDURE AllowedChangeMyFirm(parentMyFirm: P_Firm.tMyFirm);
 BEGIN
  IF CostItem.Leaf = 1 THEN
   IF ~!CostItem.MyFirm & (CostItem.MyFirm # P_Firm.mf_Fict) THEN
    L_General.CallUs(~!parentMyFirm);
    IF (parentMyFirm # P_Firm.mf_Fict) & (CostItem.MyFirm # parentMyFirm) THEN
     L_General.Error("㯯  " + CostItem.Parent + " ᮯ⠢  㣮 ' ࣠樥' (" + parentMyFirm + ")");
    END;
   END;
  ELSE
   IF ~!parentMyFirm    & (parentMyFirm # P_Firm.mf_Fict) &
      ~!CostItem.MyFirm & (CostItem.MyFirm # P_Firm.mf_Fict) &
      (CostItem.MyFirm # parentMyFirm) THEN
    L_General.Error(" 㯯  㪠 㣠 ");
   END;

   IF ~!CostItem.MyFirm & (CostItem.MyFirm # P_Firm.mf_Fict) THEN
    parentMyFirm:=CostItem.MyFirm;
    SAVE(CostItem);
    IF SELECT(CostItem, Parent, CostItem.CostItem) THEN
     WHILE OCCURE(CostItem) DO
      L_General.Assert(parentMyFirm = CostItem.MyFirm, " 㯯    室  室  ࠧ묨 ");
      NEXT(CostItem);
     END; (* WHILE OCCURE(CostItem)...*)
    END;
    RESTORE(CostItem);
   END;
  END;
 END {AllowedChangeMyFirm};

 PRIVATE PROCEDURE AllowedChangeACSet(oldACSet: LIKE(CostItem.ACSet));
 BEGIN
  IF ~!CostItem.ACSet THEN
   CONFORM(CostItem, ACSet);
   L_General.Assert(ACSet.Kind = L_Acnt.acsk_CostItem, "  室  室   㪠|" +
                                                       "  ࠧ१ ⮫쪮  '" + L_Str.PoolItem(L_Acnt.acsk_Pool, L_Acnt.acsk_CostItem) + "'");
  END;
  TestIfChangeCostItemACSet(oldACSet, CostItem.ACSet);
 END {AllowedChangeACSet};

 PRIVATE PROCEDURE AllowedRenew(Value, _iuPrefix: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Match(Value, _iuPrefix + " " + "*")
 END {AllowedRenew};

 PRIVATE PROCEDURE RenewSingleCostItem(new_iuPrefix, prv_iuPrefix: STRING[]);
 BEGIN
  IF (CostItem.Leaf = 0) & AllowedRenew(CostItem.iuPrefix, prv_iuPrefix) THEN
   CostItem.iuPrefix:=Str.Replace(CostItem.iuPrefix, prv_iuPrefix, new_iuPrefix);
   REPLACE(CostItem);
  END;

  IF AllowedRenew(CostItem.Name, prv_iuPrefix) THEN
   CostItem.Name:=Str.Replace(CostItem.Name, prv_iuPrefix, new_iuPrefix);
   REPLACE(CostItem);
  END;
 END {RenewSingleCostItem};

 PRIVATE PROCEDURE RenewAllCostItem(_CostItem: LIKE(CostItem.CostItem); new_iuPrefix, prv_iuPrefix: STRING[]);
 BEGIN
  SAVE(CostItem);
  SELECT(CostItem, Parent, , _CostItem);
  WHILE OCCURE(CostItem) DO
   RenewSingleCostItem(new_iuPrefix, prv_iuPrefix);
   IF CostItem.Leaf = 0 THEN
    RenewAllCostItem(CostItem.CostItem, new_iuPrefix, prv_iuPrefix);
   END;
   NEXT(CostItem);
  END; (* WHILE OCCURE(CostItem) ... *)
  RESTORE(CostItem);
 END {RenewAllCostItem};

 PRIVATE PROCEDURE ChangeNameAnd_uiPrefix(prv_iuPrefix, parent_iuPrefix: STRING[]);
  VAR
   new_iuPrefix: STRING[];
 BEGIN
(*
  prv_iuPrefix:=GetRealName(prv_iuPrefix,    prv_iiName);
*)
  new_iuPrefix:=GetRealName(parent_iuPrefix, CostItem.iiName);

  IF !CostItem.Name | L_Str.IsEqStrI(CostItem.Name, parent_iuPrefix) | L_Str.IsEqStrI(CostItem.Name, prv_iuPrefix) THEN
   CostItem.Name    :=new_iuPrefix;
  END;
  IF (CostItem.Leaf = 0) & (!CostItem.iuPrefix | L_Str.IsEqStrI(CostItem.iuPrefix, parent_iuPrefix) | L_Str.IsEqStrI(CostItem.iuPrefix, prv_iuPrefix)) THEN
   CostItem.iuPrefix:=new_iuPrefix;
  END;
  REPLACE(CostItem);

  RenewAllCostItem(CostItem.CostItem, CostItem.iuPrefix, prv_iuPrefix);
 END {ChangeNameAnd_uiPrefix};

 PRIVATE PROCEDURE CostItem_Parent_OnSuccessfulChange();
  VAR
   parent_iuPrefix: STRING[];
   parentACSet    : LIKE(ACSet.ACSet);
   parentMyFirm   : P_Firm.tMyFirm;
 BEGIN
  IF ~!CostItem.Parent THEN
   SAVE(CostItem);
   SEARCH(CostItem, CostItem, , CostItem.Parent);
   L_General.Assert(CostItem.Leaf=0, "室 㪠뢠 㯯 ⥩ 室  室");
   RESTORE(CostItem);
   L_General.Assert(CostItem.Parent # CostItem.CostItem, " ᯮ짮 ⥪ ");
  ELSE
   L_General.Assert(CostItem.Leaf = 0, " 㪠 㯯 ⥩ 室  室");
  END;

  GetParentAttr(CostItem.Parent, parent_iuPrefix, parentACSet, parentMyFirm);

  AllowedChangeMyFirm(parentMyFirm);
  IF !CostItem.MyFirm | (CostItem.MyFirm = P_Firm.mf_Fict) THEN
   CostItem.MyFirm:=parentMyFirm;
   REPLACE(CostItem);
  END;

  IF !CostItem.Parent THEN
   IF L_StdFrm.Alternative("  '', '  娨'  '䨪'?") THEN
    SETEMPTY(CostItem.iuPrefix, CostItem.Name, CostItem.iiName);
    REPLACE(CostItem);
   END;
  ELSE
   ChangeNameAnd_uiPrefix(CostItem.iuPrefix, parent_iuPrefix);

   IF parentACSet # L_Acnt.fictACSet THEN
    IF ~!CostItem.ACSet & (CostItem.ACSet # parentACSet) THEN
     TOOLTIP("  ࠧ१   㣮, 祬  '㯯 '");
    END;
   END;
  END;
  RTI.Get().CurZone.Fields("v_ParentName").Value:=GetCostItemParentName();
 END {CostItem_Parent_OnSuccessfulChange};

 PRIVATE PROCEDURE CostItem_MyFirm_OnSuccessfulChange();
  VAR
   parent_iuPrefix: STRING[];
   parentACSet    : LIKE(ACSet.ACSet);
   parentMyFirm   : P_Firm.tMyFirm;
 BEGIN
  IF CostItem.Leaf = 1 THEN
   L_General.Assert(CostItem.MyFirm # P_Firm.mf_Fict, " 室  室   ⭮   0000");
   L_General.Assert(~!CostItem.Parent, " 㪠 㯯 ");
  END;
  GetParentAttr(CostItem.Parent, parent_iuPrefix, parentACSet, parentMyFirm);
  AllowedChangeMyFirm(parentMyFirm);
 END {CostItem_MyFirm_OnSuccessfulChange};

 PRIVATE PROCEDURE CostItem_iiName_OnSuccessfulChange_Explicit(PValue: LIKE(CostItem.iiName));
  VAR
   parent_iuPrefix: STRING[];
   parentACSet    : LIKE(ACSet.ACSet);
   parentMyFirm   : P_Firm.tMyFirm;
 BEGIN
  GetParentAttr(CostItem.Parent, parent_iuPrefix, parentACSet, parentMyFirm);
  ChangeNameAnd_uiPrefix(GetRealName(parent_iuPrefix, PValue), parent_iuPrefix);
 END {CostItem_iiName_OnSuccessfulChange_Explicit};

 PRIVATE PROCEDURE CostItem_iiName_OnSuccessfulChange();
 BEGIN
  CostItem_iiName_OnSuccessfulChange_Explicit(GE_PVALUE());
 END {CostItem_iiName_OnSuccessfulChange};

 PRIVATE PROCEDURE CostItem_ACSet_OnSuccessfulChange();
 BEGIN
  IF CostItem.Leaf = 1 THEN
   L_General.Assert(CostItem.ACSet # L_Acnt.fictACSet, "     㪠 䨪⨢   ࠧ१");
  END;
  AllowedChangeACSet(GE_PVALUE());
 END {CostItem_ACSet_OnSuccessfulChange};

 PRIVATE PROCEDURE CostItem_iuPrefix_OnSuccessfulChange_Explicit(PValue: STRING[]);
 BEGIN
  RenewAllCostItem(CostItem.CostItem, CostItem.iuPrefix, PValue);
 END {CostItem_iuPrefix_OnSuccessfulChange_Explicit};

 PRIVATE PROCEDURE CostItem_iuPrefix_OnSuccessfulChange();
 BEGIN
  CostItem_iuPrefix_OnSuccessfulChange_Explicit(GE_PVALUE());
 END {CostItem_iuPrefix_OnSuccessfulChange};

 PRIVATE PROCEDURE CostItem_ACSet_OnAskPhase1();
 BEGIN
  IF SELECT(ACSet, Kind, L_Acnt.acsk_CostItem) THEN
   SE_WZD_CURSOR(RTI.Get().TrgWindow.Name, 1, ACSet);
   IF ~!CostItem.ACSet THEN
    SEARCH(ACSet, KindACS, , L_Acnt.acsk_CostItem, CostItem.ACSet);
    SE_WZD_POSITION(RTI.Get().TrgWindow.Name, RTI.Get().TrgZone.Number, ACSet);
   END;
  ELSE
   L_StdFrm.Inform("祣 室饣!");
  END;
 END {CostItem_ACSet_OnAskPhase1};

 PRIVATE PROCEDURE CostItem_ACSet_OnAskPhase2();
 BEGIN
  IF ACCESS(ACSet) THEN
   CostItem.ACSet:=ACSet.ACSet;
   REPLACE(CostItem);
  END;
 END {CostItem_ACSet_OnAskPhase2};

 PRIVATE PROCEDURE GetCostItemHyerarchyItemName(): STRING[];
 BEGIN
  ACCESS(CostItem);
  RETURN L_Str.PoolItem("㯯 ⥩ 室  室| 室  室", CostItem.Leaf);
 END {GetCostItemHyerarchyItemName};

 PRIVATE PROCEDURE CstItmE_0_OnReposition();
 BEGIN
  IF ~L_WDO.IsLoadableAskClone() THEN
   RTI.Get().CurZone.Fields("v_ParentName").Value:=GetCostItemParentName();
   RTI.Get().CurWindow.Title                     :=GetCostItemHyerarchyItemName();
  END;
 END {CstItmE_0_OnReposition};

 PRIVATE PROCEDURE CostItem_1_OnReposition();
 BEGIN
  IF ~L_WDO.IsLoadableAskClone() THEN
   IF ACCESS(CostItem) THEN
    IF CostItem.Leaf = 1 THEN
     RTI.Get().CurProcess.ReplaceContexts("View_CostItem_1_RCD_CostItem");
    ELSE
     RTI.Get().CurProcess.ReplaceContexts("View_CostItem_1_RCD_Group");
    END;
   ELSE
    RTI.Get().CurProcess.ReplaceContexts("");
   END;
  END;
 END {CostItem_1_OnReposition};

 PUBLIC PROCEDURE CostItem_1_OnChangeHierarchicView();
  VAR
   ShowCostItemName : SCAL[2];
 BEGIN
  ShowCostItemName:=RTI.Get().CurZone.BanHierarchicView | (~!RTI.Get().CurZone.ActiveField & (RTI.Get().CurZone.ActiveField.Name # "CostItemName"));
  L_WDO.SetFieldExpression_wTitle(1,
                                  "CostItemName",
                                  L_Str.PoolItem("CostItem.iiName|CostItem.Name",        ShowCostItemName),
                                  L_Str.PoolItem("  娨|", ShowCostItemName));
 END {CostItem_1_OnChangeHierarchicView};

 PRIVATE PROCEDURE CostItem_OnInitializeWindow();
 BEGIN
  L_WDO.iField       (0, "CostItemPanelName").LineBreakEnable:=1;
  L_WDO.iEvents      (1, [L_WDO.e_OnReposition, L_WDO.e_OnChangeIndex]);
  L_WDO.iSubwindow   (1, "CstItmE");
  L_WDO.iZone        (1).HierarchicLink:=LINK.Get("CostItem", "Parent");
  L_WDO.iField       (1, "CostItemName").Hierarchic:=1;
  IF L_WDO.IsLoadableAskClone() & (RTI.Get().CurWindow.AskSourceField.Name = "CostItem.Parent") THEN
   L_WDO.iZone       (1).PermanentFilter:="CostItem.Leaf = 0";
  END;
 END {CostItem_OnInitializeWindow};

 PRIVATE PROCEDURE CostItem_1_OnChangeIndex();
 BEGIN
  CostItem_1_OnChangeHierarchicView();
 END {CostItem_1_OnChangeIndex};

 PRIVATE PROCEDURE CstItmE_0_OnZoneOpen();
  VAR
   Fields: FORMFIELDS;
 BEGIN
  RTI.Get().CurWindow.Zones(0).Fields("CIKind.Type").Format :=L_CI.cikt_Pool_Short;
  RTI.Get().CurWindow.Zones(1).Fields("ACSetL.ACPrp").Format:=L_Acnt.acp_Pool;
 END {CstItmE_0_OnZoneOpen};

 PUBLIC PROCEDURE FillNameAndPrefixFromGroupCI();
  VAR
   parent_iuPrefix: STRING[];
   parentACSet    : LIKE(ACSet.ACSet);
   parentMyFirm   : P_Firm.tMyFirm;
 BEGIN
  L_General.Assert(~!CostItem.Parent, " 㯯  室  孥 ஢ 娨");
  L_StdFrm.Question("   ࠢ  㬮砭");
  GetParentAttr(CostItem.Parent, parent_iuPrefix, parentACSet, parentMyFirm);
  CostItem.Name    :=GetRealName(parent_iuPrefix, CostItem.iiName);
  REPLACE(CostItem);
 END {FillNameAndPrefixFromGroupCI};

 PRIVATE PROCEDURE CIKind_1_OnZoneOpen();
 BEGIN
  RTI.Get().CurZone.Fields("CIKind.Type").Format :=L_CI.cikt_Pool;
 END {CIKind_1_OnZoneOpen};

 PRIVATE PROCEDURE CIKindE_0_OnZoneOpen();
 BEGIN
  RTI.Get().CurZone.Fields("CIKind.Type").Format :=L_CI.cikt_Pool;
 END {CIKindE_0_OnZoneOpen};

 PRIVATE PROCEDURE CIKind_ERKN_OnBeforeAskForView();
 BEGIN
  L_APtm.CurrentEntityId:=DBD.FileNameToId(GE_CURFILE());
 END {CIKind_ERKN_OnBeforeAskForView};

 PRIVATE PROCEDURE CIKind_ERKN_OnBeforeAskForEdit();
 BEGIN
  L_APtm.CurrentEntityId:=DBD.FileNameToId(GE_CURFILE());
 END {CIKind_ERKN_OnBeforeAskForEdit};

 PRIVATE PROCEDURE CIKind_OnAfterEdIns();
 BEGIN
  IF CIKind.Type # L_CI.cikt_Element THEN
   L_General.Assert(!CIKind.ERKN, " ⨯ '"+L_Str.PoolItem(L_CI.cikt_Pool, CIKind.Type)+"'  '祭'   ");
  END;
 END {CIKind_OnAfterEdIns};

 PRIVATE PROCEDURE CIKind_OnAfterEdit();
 BEGIN
  CIKind_OnAfterEdIns();
 END {CIKind_OnAfterEdit};

 PRIVATE PROCEDURE CIKind_OnAfterInsert();
 BEGIN
  CIKind_OnAfterEdIns();
 END {CIKind_OnAfterInsert};

 PRIVATE PROCEDURE mcic_MakeSingleCopy(prnCostItem: LIKE(CostItem.Parent); _MyFirm: P_Firm.tMyFirm): LIKE(CostItem.CostItem);
  VAR
   _Begin: LIKE(CostItem.Begin);
   _Till : LIKE(CostItem.Till);
   newp_iuPrefix: LIKE(CostItem.iuPrefix);
   Result: LIKE(CostItem.CostItem);
 BEGIN
  _Begin       :=CostItem.Begin;
  _Till        :=CostItem.Till;
  newp_iuPrefix:=CostItem.iuPrefix;
  IF ~!prnCostItem THEN
   SAVE(CostItem);
   SEARCH(CostItem, CostItem, , prnCostItem);
   newp_iuPrefix:=CostItem.iuPrefix;
   _Begin       :=Math.Max(_Begin, CostItem.Begin);
   _Till        :=Math.Min(_Till,  CostItem.Till);
   L_General.CallUs(_Begin <= _Till);
   RESTORE(CostItem);
  END;
  SAVE(CostItem);
  SETEMPTY(CostItem.CostItem, CostItem.XObject, CostItem.RECORDID);
  CostItem.Parent:=prnCostItem;
  IF ~!_MyFirm & (CostItem.MyFirm # P_Firm.mf_Fict) THEN
   CostItem.MyFirm:=_MyFirm;
  END;
  CostItem.Begin:=_Begin;
  CostItem.Till :=_Till;
  CostItem.Name:=GetRealName(newp_iuPrefix, CostItem.iiName);
  IF ~CostItem.Leaf THEN
   CostItem.iuPrefix:=CostItem.Name;
  END;
  INSERT(CostItem);
  L_APtm.SysLogHandler("CostItem", L_SysLog.knd_AutoBuild);
  Result:=CostItem.CostItem;
  CostItem_OnAfterInsert();
  RESTORE(CostItem);
  RETURN Result;
 END {mcic_MakeSingleCopy};

 PRIVATE PROCEDURE mcic_SingleLevel(srcParent, dstCostItem: LIKE(CostItem.CostItem); _MyFirm: P_Firm.tMyFirm);
 BEGIN
  SAVE(CostItem);
  SELECT(CostItem, Parent, , srcParent);
  WHILE OCCURE(CostItem) DO
   mcic_SingleLevel(CostItem.CostItem,
                    mcic_MakeSingleCopy(dstCostItem, _MyFirm), _MyFirm);
   NEXT(CostItem);
  END;
  RESTORE(CostItem);
 END {mcic_SingleLevel};

 PRIVATE PROCEDURE intMakeCostItemCopy(SrcCostItem: LIKE(CostItem.CostItem);
                                       DstParent  : LIKE(CostItem.Parent));
  VAR
   newCostItem: LIKE(CostItem.CostItem);
   parent_iuPrefix: STRING[];
   parent_ACSet   : LIKE(ACSet.ACSet);
   _MyFirm,
   parent_MyFirm  : P_Firm.tMyFirm;
 BEGIN
  L_General.CallUs(SEARCH(CostItem, CostItem, SrcCostItem));
  GetParentAttr(DstParent, parent_iuPrefix, parent_ACSet, parent_MyFirm);
  IF !parent_MyFirm | (parent_MyFirm = P_Firm.mf_Fict) THEN
   _MyFirm:=P_Firm.mf_Fict;
   IF (CostItem.MyFirm # P_Firm.mf_Fict) | ListCostItemIsSameMyFirm(_MyFirm) THEN
    _MyFirm:=P_Firm.DefineMyFirm();
   ELSE
    SETEMPTY(_MyFirm);
   END;
  ELSE
   _MyFirm:=parent_MyFirm;
  END;
  newCostItem:=mcic_MakeSingleCopy(DstParent, _MyFirm);
  mcic_SingleLevel(CostItem.CostItem, newCostItem, _MyFirm);
  SEARCH(CostItem, CostItem, , newCostItem);
 END {intMakeCostItemCopy};

 PUBLIC PROCEDURE MakeCostItemCopy();
  CONST
   xPrf = "!!!";
  VAR
   _iiName,
   _iuPrefix: STRING[];
 BEGIN
  L_General.Assert(ACCESS(CostItem), " ⠭ ⥪   ");
  _iiName:=CostItem.iiName;
  _iuPrefix:=CostItem.iuPrefix;
  intMakeCostItemCopy(CostItem.CostItem, CostItem.Parent);
  CostItem.iiName:=xPrf + CostItem.iiName;
  REPLACE(CostItem);
  CostItem_iiName_OnSuccessfulChange_Explicit(_iiName);
  IF (CostItem.Leaf = 0) & (CostItem.iuPrefix = _iuPrefix) THEN
   CostItem.iuPrefix:=xPrf + CostItem.iuPrefix;
   REPLACE(CostItem);
   CostItem_iuPrefix_OnSuccessfulChange_Explicit(_iuPrefix);
  END;
  SE_WZD_POSITION(GE_CURWIN(), GE_CURZONE(), CostItem);
 END {MakeCostItemCopy};

 PRIVATE PROCEDURE HasCIKind(_Parent: LIKE(CostItem.Parent); _CIKind: LIKE(CostItem.CIKind)): SCAL[2];
 BEGIN
  SELECT(CostItem, Parent, , _Parent);
  WHILE OCCURE(CostItem) DO
   IF CostItem.CIKind = _CIKind THEN
    RETURN 1;
   END;
   NEXT(CostItem);
  END; (* WHILE OCCURE(CostItem) ... *)
  RETURN 0;
 END {HasCIKind};

 PRIVATE PROCEDURE MakeCopyElementsList(SrcParent, DstParent: LIKE(CostItem.Parent));
  VAR
   _CostItem: LIKE(CostItem.CostItem);
   _Begin : LIKE(CostItem.Begin);
   _Till  : LIKE(CostItem.Till);
 BEGIN
  SAVE(CostItem);
  _Begin:=CostItem.Begin;
  _Till :=CostItem.Till;
  SELECT(CostItem, Parent, , SrcParent);
  WHILE OCCURE(CostItem) DO
   IF L_DatTim.IsActiveInIntvl(_Begin, _Till, CostItem.Begin, CostItem.Till) &
      ~!CostItem.CIKind THEN
    CONFORM(CostItem, CIKind);
    IF CIKind.Type = L_CI.cikt_Element THEN
     SAVE(CostItem);
     _CostItem:=CostItem.CostItem;
     IF ~SELECT(CostItem, Parent, DstParent, CostItem.Leaf, CostItem.iiName) &
        ~HasCIKind(DstParent, CIKind.CIKind) THEN
      intMakeCostItemCopy(_CostItem, DstParent);
     END;
     RESTORE(CostItem);
    END;
   END;
   NEXT(CostItem);
  END; (* WHILE OCCURE(CostItem) ... *)
  RESTORE(CostItem);
 END {MakeCopyElementsList};

 PUBLIC PROCEDURE MakeCarCostItemCopy();
  VAR
   CurCostItem: LIKE(CostItem.CostItem);
 BEGIN
  ACCESS(CostItem);
  CurCostItem:=CostItem.CostItem;
  L_StdFrm.Choice("  㯯 "+CurCostItem+"|  㯯   ⨯ ''?");
  CONFORM(CostItem, CIKind);
  L_General.Assert(OCCURE(CIKind) & (CIKind.Type = L_CI.cikt_Car), "    ⨯ ''");

  SELECT(CostItem, CIKind, , CIKind.CIKind);
  WHILE OCCURE(CostItem) DO
   IF CostItem.CostItem # CurCostItem THEN
    PROMPT(CostItem.CostItem);
    MakeCopyElementsList(CurCostItem, CostItem.CostItem);
   END;
   NEXT(CostItem);
  END; (* WHILE OCCURE(CostItem) ... *)
 END {MakeCarCostItemCopy};

(*
$P{ u_Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, <D.N.y>
SYSTIME(), L, <H:M>
?, L, <>
CostItem.CostItem, L, <>
'"'+CostItem.Name+'"', L, <>
@
############################################################################################# ########## #####
=#####========================================================================================================
| ᯮ짮  ####### #################################################################################|
}
*)

(*
$P{ u_SDSTop:
@
|------------------------------------------------------------------------------------------------------------|
|                                        ᯮᮡ ।                                        |
|------------------------------------------------------------------------------------------------------------|
|                                                          |ᮡ                      |      |     |
|---------------------------------------------------------------|----------------------------|-------|-------|
}
*)

(*
$P{ u_SDSLine:
SalDSet.SalDSet, L, <>
SalDSet.Name, L, <>
DstMode.DstMode, L, <>
DstMode.Name, L, <>
SalDSetL.Begin, L, <N.y>
?, L, <N.y>
@
|#### ##########################################################|## #########################|#######|#######|
}
*)

(*
$P{ u_MCPTop:
@
|------------------------------------------------------------------------------------------------------------|
|                                          ᫥/㤥ঠ                                          |
|------------------------------------------------------------------------------------------------------------|
| /                                                        |ᮡ                      |      |     |
|---------------------------------------------------------------|----------------------------|-------|-------|
}
*)

(*
$P{ u_MCPLine:
MCList.MCode, L, <>
MCList.Name, L, <>
DstMode.DstMode, L, <>
DstMode.Name, L, <>
MCPro.Date, L, <N.y>
?, L, <N.y>
@
|### ###########################################################|## #########################|#######|#######|
}
*)

(*
$P{ u_RtLTop:
@
|------------------------------------------------------------------------------------------------------------|
|                                                                                                    |
|------------------------------------------------------------------------------------------------------------|
|                                                                                     |      |     |
|--------------------------------------------------------------------------------------------|-------|-------|
}
*)

(*
$P{ u_RtLLine:
Route.Route, L, <>
Route.Name, L, <>
RouteLog.Date, L, <N.y>
?, L, <N.y>
@
|#### #######################################################################################|#######|#######|
}
*)

(*
$P{ u_Bot:
@
=(c) Polak====================================================================================================
}
*)

 PUBLIC PROCEDURE Usage();
  CONST
   RepName = "CIUsg";
  VAR
   curDstMode : LIKE(DstMode.DstMode);
   dmyCostItem: LIKE(CostItem.CostItem);
   dmyAcCut   : LIKE(AcCut.AcCut);
   trgTill    : DATE;
 BEGIN
  ACCESS(CostItem);
  RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, u_Top, RepName);
  IF SELECT(SalDSetL, CostItem, CostItem.CostItem) THEN
   MAKEPART(R, u_SDSTop);
   WHILE OCCURE(SalDSetL) DO
    PROMPT(SalDSetL.SalDSet);
    CONFORM(SalDSetL, SalDSet);
    CONFORM(SalDSetL, DstMode);
    trgTill:=Math.Min(SalDSetL.Till, SalDSet.Till);
    MAKEPART(R, u_SDSLine, trgTill);
    NEXT(SalDSetL);
   END; (* WHILE OCCURE(SalDSetL) ... *)
  END;
  IF SELECT(r2r, byOwn, TABLE.Get("CostItem").Id, CostItem.RECORDID, TABLE.Get("MCPro").Id) THEN
   MAKEPART(R, u_MCPTop);
   WHILE OCCURE(r2r) DO
    PROMPT(r2r.RECORDID);
    SEARCH(MCPro, RECORDID, , r2r.riUsr);
    CONFORM(MCPro, MCList);
    P_MCList.StandOnDstMode(MCList.MCode, MCPro.Date);
    IF ~!MCPro.EndDate THEN
     trgTill:=MCPro.EndDate+1;
    ELSE
     trgTill:=MCList.Till;
    END;
    MAKEPART(R, u_MCPLine, trgTill);
    NEXT(r2r);
   END; (* WHILE OCCURE(r2r) ... *)
  END;
  IF SELECT(RouteLog, CostItem, CostItem.CostItem) THEN
   MAKEPART(R, u_RtLTop);
   WHILE OCCURE(RouteLog) DO
    PROMPT(RouteLog.Route);
    CONFORM(RouteLog, Route);
    trgTill:=Math.Min(RouteLog.Till, Route.Till);
    MAKEPART(R, u_RtLLine, trgTill);
    NEXT(RouteLog);
   END; (* WHILE OCCURE(RouteLog) ... *)
  END;
  MAKEPART(R, u_Bot);
  CLOSEREP(R);
  L_General.SurveyLastReport();
 END {Usage};

BEGIN
 L_Events.ExecuteHandler();
END.
