(*
%Copyright   (c) Polak Computer System
%Author      30.11.1993 18:07:06 MOG
%Purpose       ப DedAcnt
%Type        control
%Specific    SYS
%Modified    23.07.2014 18:21:30 POLAK\Bor
%Environment
*)

 MODULE DedAcnt [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT Memo, L_General, L_PRoll, L_DatTim, L_Events;

 PRIVATE PROCEDURE BalDedAcnt(): NUMERIC[2];
  VAR
   _Saldo: LIKE(DedAcnt.Saldo);
 BEGIN
  LOOP _Saldo := DedAcnt.Saldo + DedAcnt.NetDebt - DedAcnt.Cred;
   IF NEXT(DedAcnt) THEN
    DedAcnt.Saldo:=_Saldo; REPLACE(DedAcnt); 
   ELSE
    EXIT;
   END;
  END;
  RETURN _Saldo;
 END {BalDedAcnt};

 PRIVATE PROCEDURE CheckCurMonth();
 BEGIN
  L_PRoll.Salary_CheckNoTouch(DedAcnt.Period);
  L_PRoll.DedDone_AP(DedAcnt.Period, 0);
 END {CheckCurMonth};

 PRIVATE PROCEDURE DedAcnt_OnBeforeEdit();
 BEGIN
  CONFORM(Deduct, Staff);
  CheckCurMonth();
  CONFORM(Deduct, DedType);
  RTI.Get().CurZone.Fields("DedAcnt.Cred").Modifiable:=DedType.Limited;
 END {DedAcnt_OnBeforeEdit};

 PRIVATE PROCEDURE CheckLimited();
  VAR
   HasLimit: SCAL[2];
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   CONFORM(DedAcnt, DedType);
   IF DedType.Limited THEN
    HasLimit:=0;
    CHANGESEL(DedAcnt, TSN, , DedAcnt.TSN);
    WHILE OCCURE(DedAcnt) & ~HasLimit DO
     IF DedAcnt.Cred # 0 THEN
      HasLimit:=1;
     END;
     PREV(DedAcnt);
    END; (* WHILE OCCURE(DedAcnt) ... *)
    L_General.Assert(HasLimit, " ⨯ 㤥ঠ ' ࠭祭'|  㪠 㬬 ࠭祭   '⠥'");
   END;
  EXCEPT
  END; (* TRY *)
 END {CheckLimited};

 PRIVATE PROCEDURE DedAcnt_OnAfterEdIns();
 BEGIN
  CheckLimited();
 END {DedAcnt_OnAfterEdIns};

 PRIVATE PROCEDURE DedAcnt_OnAfterEdit();
  VAR
   _Saldo: LIKE(DedAcnt.Saldo);
 BEGIN
  DedAcnt_OnAfterEdIns();
  SAVE   (DedAcnt);
  _Saldo:=BalDedAcnt();
  RESTORE(DedAcnt);
 END {DedAcnt_OnAfterEdit};

 PRIVATE PROCEDURE DedAcnt_OnBeforeInsert();
  VAR
   _Period: LIKE(DedAcnt.Period);
 BEGIN
  L_General.Assert(~!Deduct.DedNbr & ~!Deduct.DedType, "室 ।  㤥ঠ  ⨯ 㤥ঠ");
  CONFORM(Deduct, DedType);
  L_General.Assert(Deduct.DedType # 0,
                   " 㤥ঠ   "+Deduct.DedType+" "+DedType.Name+
                   "|  ⢮ ப ⮢");
  IF ~DedType.Limited THEN
   DedAcnt.Cred:=0;
   REPLACE(DedAcnt);
  END;
  RTI.Get().CurZone.Fields("DedAcnt.Cred").Modifiable:=DedType.Limited;
  SAVE   (DedAcnt);
  LAST(DedAcnt);
  IF PREV(DedAcnt) THEN
   _Period:=L_DatTim.IncMonth(DedAcnt.Period, +1);
  ELSE
   _Period:=Memo.ud_mb();
  END;
  RESTORE(DedAcnt);
  DedAcnt.Period:=_Period;
  REPLACE(DedAcnt);
 END {DedAcnt_OnBeforeInsert};

 PRIVATE PROCEDURE DedAcnt_OnAfterInsert();
  VAR
   _Saldo: LIKE(DedAcnt.Saldo);
 BEGIN
  DedAcnt_OnAfterEdIns();
  CONFORM(Deduct, Staff);
  CheckCurMonth();
  SAVE   (DedAcnt);
  IF ~PREV(DedAcnt) THEN
   FIRST(DedAcnt);
  END;
  _Saldo:=BalDedAcnt();
  RESTORE(DedAcnt);
 END {DedAcnt_OnAfterInsert};

 PRIVATE PROCEDURE DedAcnt_OnBeforeDelete();
 BEGIN
  CONFORM(Deduct, Staff);
  CheckCurMonth();
  IF L_DatTim.IsActive_Inc(DedAcnt.Period, Deduct.BegDate, Deduct.EndDate) THEN
   L_General.AdminOnly();
  ELSIF (DedAcnt.Saldo # 0) | (DedAcnt.TotDebt # 0) | (DedAcnt.NetDebt # 0) | (DedAcnt.Cred # 0) THEN
   L_General.AdminOnly();
  END;
 END {DedAcnt_OnBeforeDelete};

BEGIN
 L_Events.ExecuteHandler();
END.
