(*
%Copyright   (c) Polak IT
%Author      28.11.2003 18:28:05 POLAK\ATS
%Purpose
%Type        control
%Specific    LOC
%Modified    22.09.2014 16:02:53 POLAK\LDA
%Environment
*)

 MODULE Dspt [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Events, L_General, L_Dspt, L_XDoc, L_Str, Memo, L_MDoc,
        Str, L_Pmnt, Math, L_Acnt, L_DatTim, P_DsptM, L_Prop, L_ODoc, P_Firm,
        L_lsStore, L_APtm, L_StdFrm, L_SysLog, L_ctc, L_WDO, L_kn;

 PRIVATE CONST
  v_Doc            = "v_Doc";
  v_DocKind        = "v_DocKind";
  v_DocType        = "v_DocType";
  v_Descrip        = "v_Descrip";
  v_FullXDocName   = "v_FullXDocName";
  v_dp_SendDate    = "v_dp_SendDate";
  v_dp_SendTime    = "v_dp_SendTime";
  v_dp_CourierTSN  = "v_dp_CourierTSN";

 PRIVATE PROCEDURE SetDsptFieldsModifiable();
  VAR
   Fields: FORM_FIELDS;
   fld   : FORM_FIELD;
   HasDsptMode: SCAL[2];
 BEGIN
  IF (RTI.Get().CurWindow.OriginalName = "DsptOE") |
     (RTI.Get().CurWindow.OriginalName = "DsptIE") THEN
   CONFORM(Dspt, DsptMode);
   Fields:=RTI.Get().CurWindow.Zones(0).Fields;
   IF RTI.Get().CurWindow.OriginalName = "DsptOE" THEN
    fld:=Fields("Dspt.DsptPck");
    IF    !Dspt.DsptMode THEN
     fld.Modifiable    :=0;
    ELSIF DsptMode.PackProp = L_Dspt.pck_Not_Allowed THEN
     fld.Modifiable    :=0;
    ELSIF DsptMode.PackProp = L_Dspt.pck_Allowed THEN
     fld.Modifiable    :=1;
     fld.MustBeNonempty:=0;
    ELSIF DsptMode.PackProp = L_Dspt.pck_Must THEN
     fld.Modifiable    :=1;
     fld.MustBeNonempty:=1;
    END;
   END;

   fld:=Fields("Dspt.CSFirm");
   fld.Modifiable    :=~!Dspt.DsptMode & (DsptMode.PostProp # L_Dspt.dapp_NotPost);
   fld.MustBeNonempty:=fld.Modifiable;

   Fields("Dspt.Status").Modifiable:=(Dspt.Status = L_Dspt.st_Prepared) | (Dspt.Status = L_Dspt.st_Canceled);
  END;
 END {SetDsptFieldsModifiable};

 PRIVATE PROCEDURE SetDsptStatus();
 BEGIN
  SELECT(DsptTime, Dspt, , Dspt.Type, Dspt.Dspt);
  IF SRCINSEL(DsptTime, LT, L_Dspt.rp_DelayNotice) THEN
   IF    DsptTime.RefPoint = L_Dspt.rp_Send THEN
    Dspt.Status:=L_Dspt.st_Sent;
   ELSIF DsptTime.RefPoint < L_Dspt.rp_Delivery THEN
    Dspt.Status:=L_Dspt.st_CS;
   ELSE
    Dspt.Status:=L_Dspt.st_Delivered;
   END;
  ELSE
   Dspt.Status:=L_Dspt.st_Prepared;
  END;
  REPLACE(Dspt);
  SetDsptFieldsModifiable();
 END {SetDsptStatus};

 PRIVATE PROCEDURE Dspt_Status_OnSuccessfulChange();
 BEGIN
  L_General.Assert((Dspt.Status = L_Dspt.st_Canceled) | (Dspt.Status = L_Dspt.st_Prepared),
                   "ﭨ '" + L_Str.PoolItem(L_Dspt.st_Pool, Dspt.Status) +
                   "' ⠭ ⮬᪨ (. ஢)");
 END {Dspt_Status_OnSuccessfulChange};

 PRIVATE PROCEDURE IsAfter(d1: DATE; t1: TIME; d2: DATE; t2: TIME): SCAL[2];
 BEGIN
  RETURN (d1 > d2) | (d1 = d2) & ~!t1 & ~!t2 & (t1 > t2);
 END {IsAfter};

 PRIVATE PROCEDURE InsertDsptTimeByDspt(trgRefPoint: LIKE(DsptTime.RefPoint); trgDate: DATE; trgTime: TIME);
 BEGIN
  SELECT(DsptTime, Dspt, , Dspt.Type, Dspt.Dspt);
  WHILE OCCURE(DsptTime) DO
   IF    IsAfter(DsptTime.Date, DsptTime.Time, trgDate, trgTime) THEN
    IF (trgRefPoint > DsptTime.RefPoint) THEN
     L_General.Error("塞 ஢ 蠥 楯|" +
                     '"' + L_Str.PoolItem(L_Dspt.dt_RefPointPool, trgRefPoint) + '" ࠭ ' +
                     '"' + L_Dspt.GetDsptTimeRefPoint_asStr() + '"');
    END;
   ELSIF IsAfter(trgDate, trgTime, DsptTime.Date, DsptTime.Time) THEN
    IF (trgRefPoint < DsptTime.RefPoint) THEN
     L_General.Error("塞 ஢ 蠥 楯|" +
                     '"' + L_Str.PoolItem(L_Dspt.dt_RefPointPool, trgRefPoint) + '"  ' +
                     '"' + L_Dspt.GetDsptTimeRefPoint_asStr() + '"');
    END;
   END;
   NEXT(DsptTime);
  END; (* WHILE OCCURE(DsptTime) ... *)
  DsptTime.Dspt    :=Dspt.Dspt;
  DsptTime.Type    :=Dspt.Type;
  DsptTime.Date    :=trgDate;
  DsptTime.Time    :=trgTime;
  DsptTime.RefPoint:=trgRefPoint;
  INSERT(DsptTime);
  SetDsptStatus();
  L_APtm.SysLogHandler("DsptTime", L_SysLog.knd_AutoBuild);
 END {InsertDsptTimeByDspt};

 PRIVATE PROCEDURE RemoveSingleDsptTime();
 BEGIN
  L_APtm.SysLogHandler("DsptTime", L_SysLog.knd_Delete);
  REMOVE(DsptTime);
  SetDsptStatus();
 END {RemoveSingleDsptTime};

 PRIVATE PROCEDURE RemoveDsptTimeByDspt();
 BEGIN
  SELECT(DsptTime, Dspt, , Dspt.Type, Dspt.Dspt);
  WHILE OCCURE(DsptTime) DO
   RemoveSingleDsptTime();
  END;
 END {RemoveDsptTimeByDspt};

 PRIVATE PROCEDURE IsRequiredRecipient(rcpK: LIKE(Dspt.RcpKind)): SCAL[2];
 BEGIN
  RETURN (rcpK = L_Dspt.rcpk_Authorization) | (rcpK = L_Dspt.rcpk_Personally);
 END {IsRequiredRecipient};

 PRIVATE PROCEDURE CheckRecipientFill(wCheckRcpKind: SCAL[2]; rcpK: LIKE(Dspt.RcpKind); rcp: STRING[]): STRING[];
 BEGIN
  IF Dspt.Type = L_Dspt.t_Outgoing THEN
   IF ~!rcpK THEN
    IF IsRequiredRecipient(rcpK) THEN
     IF !rcp THEN
      RETURN "   祭   㪠 ⥫";
     END;
    ELSE
     IF ~!rcp THEN
      RETURN "   祭 ⥫    㪠";
     END;
    END;
   ELSE
    IF wCheckRcpKind THEN
     RETURN " 㪠  祭";
    END;
   END;
  END;
  RETURN "";
 END {CheckRecipientFill};

 PRIVATE PROCEDURE CheckCorrectFillAdvice(dt_Date: DATE[L_Dspt.rp_Amount];
                                          dt_Time: TIME[L_Dspt.rp_Amount];
                                          TimeReq: P_DsptM.tTimeReq;
                                          rcpK   : LIKE(Dspt.RcpKind);
                                          rcp    : STRING[]): SCAL[2];
  VAR
   i  : NUMERIC[0];
   Msg: STRING[];
 BEGIN
  Msg:=CheckRecipientFill(1, rcpK, rcp);
  IF ~!Msg THEN
   L_StdFrm.RedInform(Msg);
   RETURN 0;
  END;
  i:=0;
  WHILE i < L_Dspt.rp_Amount DO
   IF (!dt_Date[i] # !dt_Time[i]) & (!dt_Time[i] & TimeReq[i]) THEN
    L_StdFrm.RedInform(" '" + L_Str.PoolItem(L_Dspt.dt_RefPointPool, i) + "':|" +
                       "  ६     ६");
    RETURN 0;
   END;
   INC(i);
  END; (* WHILE i < L_Dspt.rp_Amount...*)
  RETURN 1;
 END {CheckCorrectFillAdvice};

 PRIVATE PROCEDURE AllowedRefPointForSave(_RefPoint: LIKE(DsptTime.RefPoint)): SCAL[2];
 BEGIN
  IF Dspt.Type = L_Dspt.t_Incoming THEN
   RETURN 1;
  ELSE
   RETURN (_RefPoint # L_Dspt.rp_Send) & (_RefPoint # L_Dspt.rp_OwnCS_Receive);
  END;
 END {AllowedRefPointForSave};

 PUBLIC PROCEDURE ShowDsptTimeDialog();
  VAR
   dt_Date       : DATE[L_Dspt.rp_Amount];
   dt_Time       : TIME[L_Dspt.rp_Amount];
   Body,
   Vars,
   Title,
   trgRecip,
   trgSrcNbr,
   Controls,
   FieldFlagEdit,
   SingleBodyLine: STRING[];
   trgRcpKind    : LIKE(Dspt.RcpKind);
   DeadLines     : P_DsptM.tDeadLine;
   TimeReq       : P_DsptM.tTimeReq;
   i             : NUMERIC[0];
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  L_General.Assert(Dspt.Status # L_Dspt.st_Canceled, "ࠢ 㫨஢");
  L_General.Assert(~!Dspt.DsptMode, " 㪠 ᯮᮡ ࠢ");
  IF Dspt.Type = L_Dspt.t_Outgoing THEN
   L_General.Assert(L_Dspt.IsDsptSent(Dspt.Type, Dspt.Dspt), "ࠢ " + Dspt.Dspt + "  ࠢ");
   CONFORM(Dspt, DsptMode);
   IF DsptMode.PostProp # L_Dspt.dapp_NotPost THEN
    L_General.Assert(SEARCH(DsptTime, Dspt, Dspt.Type, Dspt.Dspt, L_Dspt.rp_OwnCS_Receive),
                     "ࠢ " + Dspt.Dspt + "  ।  ");
   END;
  END;
  SETEMPTY(dt_Date, dt_Time);
  CONFORM(Dspt, DsptTime);
  WHILE OCCURE(DsptTime) DO
   dt_Date[DsptTime.RefPoint]:=DsptTime.Date;
   dt_Time[DsptTime.RefPoint]:=DsptTime.Time;
   NEXT(DsptTime);
  END; (* WHILE OCCURE(DsptTime) ... *)

  CONFORM(Dspt, DsptMode);
  P_DsptM.GetDeadline(Dspt.DsptMode, Dspt.Date, DeadLines, TimeReq);
  L_General.Assert(~L_Prop.IsEmptyNumArray(DeadLines),
                   " ᯮᮡ ࠢ "+Dspt.DsptMode+"  ஫ ப  " + L_DatTim.DateStr(Dspt.Date));
  SETEMPTY(Body, Controls, Vars, trgRcpKind, trgRecip, trgSrcNbr);

  i:=0;
  WHILE i < L_Dspt.rp_Amount DO
   IF ~!DeadLines[i] THEN
    IF (Dspt.Type = L_Dspt.t_Outgoing) &
       ((i = L_Dspt.rp_Send) | (i = L_Dspt.rp_OwnCS_Receive)) THEN
     FieldFlagEdit:="N";
    ELSE
     SETEMPTY(FieldFlagEdit);
    END;
    Controls      :=Controls + "%L<D.N.y>" + FieldFlagEdit + "YI";
    SingleBodyLine:=Str.Align(L_Str.PoolItem(L_Dspt.dt_RefPointPool, i), 40, LEFT) + "    %##########%";
    Vars          :=L_Str.AddStrDlm(Vars, ",") + L_Str.Var_wIdx("dt_Date", i);
    IF TimeReq[i] THEN
     Controls      :=Controls + "%L<H:M>" + FieldFlagEdit + "YI";
     SingleBodyLine:=SingleBodyLine + " %#####%";
     Vars          :=Vars + "," + L_Str.Var_wIdx("dt_Time", i);
    ELSE
     SingleBodyLine:=SingleBodyLine + "        ";
    END;
    Body:=Body + Str.Align(SingleBodyLine, 76, LEFT) + "|";
    IF i = L_Dspt.rp_Delivery THEN
     IF Dspt.Type = L_Dspt.t_Outgoing THEN
      trgRcpKind:=Dspt.RcpKind;
      trgRecip  :=Dspt.Recip;
      Vars      :=Vars + ", trgRcpKind, trgRecip";
      Controls  :=Controls  + "%L<" + L_Dspt.rcpk_Pool + ">I" + "%L,5<>KYI";
      Body      :=Body + "| 祭                               %###############%               " +
                         "|⥫                                  %##############################%|||||";
     END;
     trgSrcNbr:=Dspt.SrcNbr;
     Controls :=Controls + "%L<>YI";
     Body     :=Body + "|室                               %##############################%||";
     Vars     :=Vars + ", trgSrcNbr";
    END;

    IF ((Dspt.Type = L_Dspt.t_Outgoing) & (i = L_Dspt.rp_AdviceReceive) & ~!DeadLines[i] & !dt_Date[i]) |
       ((Dspt.Type = L_Dspt.t_Incoming) & (i = L_Dspt.rp_Delivery)      & ~!DeadLines[i] & !dt_Date[i]) THEN
     dt_Date[i]:=SYSDATE();
     IF TimeReq[i] THEN
      dt_Time[i]:=SYSTIME();
     END;
    END;
   END; (* IF ~!DeadLines[i]...*)
   INC(i);
  END; (* WHILE i < L_Dspt.rp_Amount...*)

  IF Dspt.Type = L_Dspt.t_Incoming THEN
   Title:="ਥ ࠢ";
  ELSE
   Title:="  祭";
  END;

  LOOP
   IF STDFORM(L_StdFrm.sf_Choice(Body, Controls),
              " "+Title+" ",
              ,
              1,
              ^(Vars)) THEN
 
    IF CheckCorrectFillAdvice(dt_Date, dt_Time, TimeReq, trgRcpKind, trgRecip) THEN
     i:=0;
     WHILE i < L_Dspt.rp_Amount DO
      IF ~!DeadLines[i] & AllowedRefPointForSave(i) THEN
       IF SEARCH(DsptTime, Dspt, Dspt.Type, Dspt.Dspt, i) THEN
        RemoveSingleDsptTime();
       END;
      END;
      INC(i);
     END; (* WHILE i < L_Dspt.rp_Amount...*)
     i:=0;
     WHILE i < L_Dspt.rp_Amount DO
      IF ~!DeadLines[i] & AllowedRefPointForSave(i) THEN
       IF ~!dt_Date[i] THEN
        InsertDsptTimeByDspt(i, dt_Date[i], dt_Time[i]);
       END;
      END;
      INC(i);
     END; (* WHILE i < L_Dspt.rp_Amount...*)

     Dspt.SrcNbr  :=trgSrcNbr;
     IF Dspt.Type = L_Dspt.t_Outgoing THEN
      Dspt.RcpKind:=trgRcpKind;
      Dspt.Recip  :=trgRecip;
     END;
     REPLACE(Dspt);
     EXIT;
    ELSIF trgSrcNbr # Dspt.SrcNbr THEN
     IF L_StdFrm.Alternative("쪮 ⠭ 室  '" + trgSrcNbr+"'?") THEN
      Dspt.SrcNbr:=trgSrcNbr;
      REPLACE(Dspt);
      EXIT;
     END;
    END; (* IF CheckCorrectFill(dt_Date, dt_Time)...*)
   END; (* IF STDFORM ... *)
  END; (* LOOP *)
  IF Dspt.Type = L_Dspt.t_Outgoing THEN
   L_XDoc.XDocJrn_Mark(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Edit);
  END;
 END {ShowDsptTimeDialog};

 PRIVATE PROCEDURE DsptI_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="Dspt";
  L_WDO.iZone        (1).Narrowing:="Dspt.Type=" + L_Str.IntStr(L_Dspt.t_Incoming);
  L_WDO.iSubwindow   (1, "DsptIE");
  L_WDO.iField       (1, "Dspt.Dspt").ReverseSorting:=1;
  L_WDO.iField       (1, "Dspt.Status").Format:=L_Dspt.st_ShortPool;
 END {DsptI_OnInitializeWindow};

 PRIVATE PROCEDURE DsptO_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="Dspt";
  L_WDO.iZone        (1).Narrowing:="Dspt.Type=" + L_Str.IntStr(L_Dspt.t_Outgoing);
  L_WDO.iSubwindow   (1, "DsptOE");
  L_WDO.iField       (1, "Dspt.Dspt").ReverseSorting:=1;
  L_WDO.iField       (1, "Dspt.Status").Format:=L_Dspt.st_ShortPool;
 END {DsptO_OnInitializeWindow};

 PRIVATE PROCEDURE MayIncludeDsptInDsptPck();
  VAR
   trgDspt  : LIKE(Dspt.Dspt);
   trgCSFirm: LIKE(Firm.Firm);
   trgMode  : LIKE(Dspt.DsptMode);
 BEGIN
  SETEMPTY(trgDspt, trgCSFirm, trgMode);
  L_General.CallUs(OCCURE(Dspt));
  L_General.CallUs(OCCURE(DsptPck));
  trgDspt:=Dspt.Dspt;
  SAVE(Dspt);
  IF SELECT(Dspt, DsptPck, DsptPck.DsptPck) THEN
   IF Dspt.Dspt = trgDspt THEN
    NEXT(Dspt);
   END;
   IF OCCURE(Dspt) THEN
    L_General.Assert(~!Dspt.CSFirm,
                     "ࠢ " + Dspt.Dspt + " (" + L_Dspt.GetDsptType_asStr() + "):  㪠  裡");
    trgCSFirm:=Dspt.CSFirm;
    trgMode  :=Dspt.DsptMode;
   END;
  END;
  RESTORE(Dspt);
  L_General.Assert(~!Dspt.CSFirm,
                   "ࠢ " + Dspt.Dspt + " (" + L_Dspt.GetDsptType_asStr() + "):  㪠  裡");
  L_General.Assert(!trgCSFirm | (Dspt.CSFirm = trgCSFirm), "ᮮ⢥⢨  裡");
  L_General.Assert(!trgMode   | (Dspt.DsptMode = trgMode), "ᮮ⢥⢨ ᯮᮡ ࠢ");
 END {MayIncludeDsptInDsptPck};

 PRIVATE PROCEDURE Dspt_DsptPck_OnSuccessfulChange();
 BEGIN
  IF ~!Dspt.DsptPck THEN
   CONFORM(Dspt, DsptPck);
   MayIncludeDsptInDsptPck();
  END;
 END {Dspt_DsptPck_OnSuccessfulChange};

 PRIVATE PROCEDURE OnePaperWeight(): NUMERIC[2];
 BEGIN
  RETURN 5;
 END {OnePaperWeight};

 PRIVATE PROCEDURE GetPaperAmount(): NUMERIC[0];
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=0;
  CONFORM(Dspt, DsptDoc);
  WHILE OCCURE(DsptDoc) DO
   IF ~!DsptDoc.PaperNum THEN
    INC(i, DsptDoc.PaperNum);
   ELSE
    INC(i, 1);
   END;
   NEXT(DsptDoc);
  END; (* WHILE OCCURE(DsptDoc) ... *)
  RETURN i;
 END {GetPaperAmount};

 PRIVATE PROCEDURE CalculateWeight();
  VAR
   Weight: NUMERIC[0];
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  CONFORM(Dspt, EnvlLttr);
  IF OCCURE(EnvlLttr) THEN
   Weight:=EnvlLttr.Weight + GetPaperAmount() * OnePaperWeight();
   INC(Weight, P_DsptM.GetWeight(Dspt.DsptMode, Dspt.Date, P_DsptM.dw_InLetter));
   INC(Weight, P_DsptM.GetWeight(Dspt.DsptMode, Dspt.Date, P_DsptM.dw_OutLetter));
   Dspt.Weight :=Weight * 0.001;
   Dspt.Freight:=P_DsptM.GetSendingRate(Dspt.DsptMode, Dspt.Date, Weight, Dspt.Worth);
   REPLACE(Dspt);
  END;
 END {CalculateWeight};

 PUBLIC PROCEDURE SetEnvlLttr();
  VAR
   PaperAmount: NUMERIC[0];
   Weight,
   trgWeight,
   BestWeight: LIKE(EnvlLttr.Weight);
   BestEnvl  : LIKE(EnvlLttr.EnvlLttr);
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  CONFORM(Dspt, DsptMode);
  IF OCCURE(DsptMode) & (Dspt.Type = L_Dspt.t_Outgoing) &
     (L_Dspt.IsPostalDspt() | L_Dspt.IsPostalWorthDspt()) THEN
   PaperAmount:=GetPaperAmount();
   Weight:=0;
   SETEMPTY(BestEnvl);
   INC(Weight, P_DsptM.GetWeight(Dspt.DsptMode, Dspt.Date, P_DsptM.dw_InLetter));
   BestWeight:=200000;
   SELECT(EnvlLttr, EnvlLttr);
   WHILE OCCURE(EnvlLttr) DO
    IF L_DatTim.IsActive(Dspt.Date, EnvlLttr.Begin, EnvlLttr.Till) THEN
     trgWeight:=Weight + PaperAmount * OnePaperWeight() + EnvlLttr.Weight;
     IF  (trgWeight < BestWeight) & (trgWeight < EnvlLttr.MaxWeigh) THEN
      BestEnvl  :=EnvlLttr.EnvlLttr;
      BestWeight:=trgWeight;
     END;
    END;
    NEXT(EnvlLttr);
   END; (* WHILE OCCURE(EnvlLttr) ... *)
   L_General.Assert(~!BestEnvl, "   ࠢ  諮!");
   Dspt.EnvlLttr:=BestEnvl;
   REPLACE(Dspt);
  END;
  CalculateWeight();
 END {SetEnvlLttr};

 PRIVATE PROCEDURE InsertCourier(_TSN: LIKE(Staff.TSN));
 BEGIN
  IF ~!_TSN THEN
   IF ~SELECT(DsptStf, Dspt, Dspt.Type, Dspt.Dspt, L_Dspt.dsptwho_Courier) THEN
    DEFAULT(DsptStf);
    DsptStf.Type:=Dspt.Type;
    DsptStf.Dspt:=Dspt.Dspt;
    DsptStf.Who :=L_Dspt.dsptwho_Courier;
    INSERT(DsptStf);
   END;
   DsptStf.TSN  :=_TSN;
   REPLACE(DsptStf);
  END;
 END {InsertCourier};

 PRIVATE PROCEDURE Dspt_DsptMode_OnSuccessfulChange();
 BEGIN
  CONFORM(Dspt, DsptMode);
  L_General.Assert(DsptMode.Type = Dspt.Type, " ᯮᮡ ࠢ  ᮮ⢥ ⨯ ࠢ");
  SetDsptFieldsModifiable();

  InsertCourier(P_DsptM.GetDsptMProSoft(Dspt.DsptMode, Dspt.Date, P_DsptM.P_Courier));

  IF L_Dspt.IsOutgoingDspt() & (DsptMode.PackProp = L_Dspt.pck_Not_Allowed) THEN
   SETEMPTY(Dspt.DsptPck);
  END;
  IF DsptMode.PostProp = L_Dspt.dapp_NotPost THEN
   SETEMPTY(Dspt.CSFirm);
  ELSE
   Dspt.CSFirm:=P_DsptM.GetCSFirm(Dspt.DsptMode, Dspt.Date, Dspt.Type);
  END;
  REPLACE(Dspt);
  SetEnvlLttr();
 END {Dspt_DsptMode_OnSuccessfulChange};

 PRIVATE PROCEDURE Dspt_Worth_OnSuccessfulChange();
 BEGIN
  SetEnvlLttr();
 END {Dspt_Worth_OnSuccessfulChange};

 PRIVATE PROCEDURE Dspt_Firm_OnSuccessfulChange();
 BEGIN
  IF P_Firm.StandOnFirmCtcByKindType(0, Dspt.Firm, Dspt.Date, L_ctc.ckt_DftCtcPerson) THEN
   Dspt.FCId:=FirmCtc.Id;
  ELSE
   SETEMPTY(Dspt.FCId);
  END;
  REPLACE(Dspt);
 END {Dspt_Firm_OnSuccessfulChange};

 PRIVATE PROCEDURE GetDefaultEnvlLttr(): LIKE(EnvlLttr.EnvlLttr);
  CONST
   woEnvlLttr = 00;
 BEGIN
  IF ~SEARCH(EnvlLttr, EnvlLttr, woEnvlLttr) THEN
   TRY [TRANSACTION(TRANSACTION_SLAVE)]
    DEFAULT(EnvlLttr);
    EnvlLttr.EnvlLttr:=woEnvlLttr;
    EnvlLttr.Name    :=" ";
    EnvlLttr.Begin   :=L_Env.aptm_Start;
    SETZERO(EnvlLttr.Width, EnvlLttr.Height, EnvlLttr.Weight, EnvlLttr.MaxWeigh);
    INSERT(EnvlLttr);
   EXCEPT
   END; (* TRY *)
  END;
  RETURN woEnvlLttr;
 END {GetDefaultEnvlLttr};

 PRIVATE PROCEDURE Dspt_OnBeforeInsert();
 BEGIN
  Dspt.MyFirm  :=P_Firm.DefineMyFirm();
  Dspt.PrpDate :=Memo.ud_cd();
  Dspt.Date    :=Memo.ud_cd();
  Dspt.EnvlLttr:=GetDefaultEnvlLttr();
  REPLACE(Dspt);
  L_XDoc.XDocJrn_Start(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Insert);
  SetDsptFieldsModifiable();
 END {Dspt_OnBeforeInsert};

 PRIVATE PROCEDURE Dspt_OnBeforeEdit();
 BEGIN
  CONFORM(Dspt, DsptMode);
  IF L_Dspt.IsOutgoingDspt() & L_Dspt.IsDsptSent(Dspt.Type, Dspt.Dspt) THEN
   L_StdFrm.CrucialChoice(" 㦥 ࠢ 室饥 ࠢ", L_APtm.g_Secretary);
  END;
  L_XDoc.XDocJrn_Start(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Edit);
  SetDsptFieldsModifiable();
 END {Dspt_OnBeforeEdit};

 PRIVATE PROCEDURE Dspt_OnAfterEdIns();
  VAR
   DeadLines: P_DsptM.tDeadLine;
   Dummy    : P_DsptM.tTimeReq;
   i        : NUMERIC[0];
   Msg      : STRING[];
 BEGIN
  IF Dspt.Status # L_Dspt.st_Canceled THEN
   CONFORM(Dspt, DsptMode);
   L_General.Assert((DsptMode.PostProp # L_Dspt.dapp_Worth) | ~!Dspt.Worth,
                    "ᮡ ࠢ ।ᬠਢ 㪠 業");
   L_General.Assert(~!Dspt.Firm,
                    "室 㪠 " +
                    L_Str.PoolItem("ࠢ⥫|⥫", Dspt.Type) + " ࠢ");
   L_General.Assert(L_DatTim.IsActive(Dspt.Date, DsptMode.Begin, DsptMode.Till),
                    "ᮡ ࠢ " + DsptMode.DsptMode + "   " + L_DatTim.DateStr(Dspt.Date));
 
   Msg:=CheckRecipientFill(0, Dspt.RcpKind, Dspt.Recip);
   L_General.Assert(!Msg, Msg);
 
   IF Dspt.Type = L_Dspt.t_Incoming THEN
    P_DsptM.GetDeadline(Dspt.DsptMode, Dspt.Date, DeadLines, Dummy);
    L_General.Assert(SEARCH(DsptTime, Dspt, Dspt.Type, Dspt.Dspt, L_Dspt.rp_Delivery),
                     " ᮡ '" + L_Str.PoolItem(L_Dspt.dt_RefPointPool, L_Dspt.rp_Delivery) + "'");
    i:=0;
    WHILE i < L_Dspt.rp_Amount DO
     IF  ~!DeadLines[i] & ~SEARCH(DsptTime, Dspt, Dspt.Type, Dspt.Dspt, i) THEN
      L_StdFrm.CrucialChoice(" ᮡ '" + L_Str.PoolItem(L_Dspt.dt_RefPointPool, i) + "'",
                             L_APtm.g_Secretary);
     END; (* IF ~!DeadLines[i]...*)
     INC(i);
    END; (* WHILE i < L_Dspt.rp_Amount...*)
   END;
  END; (* IF Dspt.Status # L_Dspt.st_Canceled THEN *)
  L_XDoc.XDocJrn_Stop(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt);
 END {Dspt_OnAfterEdIns};

 PRIVATE PROCEDURE Dspt_OnAfterEdit();
 BEGIN
  Dspt_OnAfterEdIns();
 END {Dspt_OnAfterEdit};

 PRIVATE PROCEDURE Dspt_OnAfterInsert();
 BEGIN
  Dspt_OnAfterEdIns();
 END {Dspt_OnAfterInsert};

 PRIVATE PROCEDURE Dspt_OnBeforeDelete();
 BEGIN
  L_General.Error(" ࠢ 饭");
  L_XDoc.XDoc_Clear(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt);
 END {Dspt_OnBeforeDelete};

 PRIVATE PROCEDURE Any_DsptE_OnInitializeWindow();
 BEGIN
  L_WDO.iField       (0, "Dspt.DsptMode").AskMode:=1;
  L_WDO.iField       (0, "Dspt.DsptMode").OwnerWindowOriginalName:="DsptMode";
  L_WDO.iField       (0, "Dspt.DsptMode").OwnerZoneNumber:=1;
  L_WDO.iField       (0, "Dspt.DsptMode").First:=1;
  L_WDO.iField       (0, "Dspt.Status").Format      :=L_Dspt.st_Pool;
  L_WDO.iSubwindow   (1, "DsptDocE");
  L_WDO.iProgs       (1, "Dspt");
  L_WDO.iField       (1, "DsptDoc.DocType").Format  :=L_XDoc.ref_ShortNames;
  L_WDO.iField       (1, "DsptDoc.Instance").Format :=L_Dspt.InstancePool;
  L_WDO.iField       (1, "DsptDoc.Own").Format      :=L_Dspt.OwnDocPool;
  L_WDO.iField       (2, "DsptStf.Who").Format      :=L_Dspt.StfWhoPool;
  L_WDO.iField       (3, "DsptTime.RefPoint").Format:=L_Dspt.dt_RefPointPool;
  L_WDO.iZone        (3).ViewOnly:=1;
 END {Any_DsptE_OnInitializeWindow};

 PRIVATE PROCEDURE DsptIE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAnyModification]);
  L_WDO.iScsChange   (0, ["Dspt.DsptMode", "Dspt.Status", "Dspt.Dspt"]);
  L_WDO.iUnmodifiable(0, ["Dspt.Dspt"]);
  L_WDO.iMayBeEmpty  (0, ["Dspt.Comment", "Dspt.Recip", "Dspt.CSFirm", "Dspt.PrpDate", "Dspt.SrcNbr"]);
  Any_DsptE_OnInitializeWindow();
 END {DsptIE_OnInitializeWindow};

 PRIVATE PROCEDURE DsptOE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAnyModification]);
  L_WDO.iScsChange   (0, ["Dspt.Firm", "Dspt.Status", "Dspt.Date", "Dspt.EnvlLttr", "Dspt.Dspt",
                          "Dspt.Worth", "Dspt.DsptPck", "Dspt.DsptMode"]);
  L_WDO.iUnmodifiable(0, ["Dspt.Weight", "Dspt.Freight", "Dspt.Status", "Dspt.Recip", "Dspt.CopyCnt",
                          "Dspt.Dspt", "Dspt.RcpKind"]);
  L_WDO.iMayBeEmpty  (0, ["Dspt.FCId", "Dspt.PrpDate", "Dspt.SrcNbr", "Dspt.Comment", "Dspt.CSFirm",
                          "Dspt.Worth", "Dspt.DsptPck"]);
  L_WDO.iField       (0, "Dspt.RcpKind").Format:=L_Dspt.rcpk_Pool;

  Any_DsptE_OnInitializeWindow();
 END {DsptOE_OnInitializeWindow};

 PRIVATE PROCEDURE DsptDocE_0_OnZoneOpen();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Fields(v_DocType).Format         :=L_Dspt.DocNeed_Names();
  Fields("DsptDoc.Instance").Format:=L_Dspt.InstancePool;
  Fields("DsptDoc.Own").Format     :=L_Dspt.OwnDocPool;
 END {DsptDocE_0_OnZoneOpen};

 PRIVATE PROCEDURE DsptDoc_SetEditFlag(trgType: L_XDoc.tXDocType; trgKind: L_XDoc.tXDocKind);
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Fields(v_DocKind).Modifiable        :=L_XDoc.XDocTypeHasXDocKind(trgType);
  Fields(v_Doc).Modifiable            :=~!trgKind;
  Fields(v_Doc).MustBeNonempty        :=1;
  Fields("DsptDoc.CopyCnt").Modifiable:=L_Dspt.XDocMustHaveCopyCnt(L_XDoc.XDocId(trgType, 0, 0));
 END {DsptDoc_SetEditFlag};

 PRIVATE PROCEDURE GetTypeAsXDocType(): L_XDoc.tXDocType;
 BEGIN
  RETURN L_Dspt.GetTypeAsXDocType(RTI.Get().CurZone.Fields(v_DocType).Value);
 END {GetTypeAsXDocType};

 PRIVATE PROCEDURE DsptDocE_0_OnReposition();
  VAR
   xdt   : L_XDoc.tXDocType;
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  IF !DsptDoc.XDocId THEN
   SETEMPTY(Fields(v_DocKind).Value,
            Fields(v_Doc).Value,
            Fields(v_Descrip).Value,
            Fields(v_FullXDocName).Value);
  ELSE
   L_XDoc.DecodeXDocId(DsptDoc.XDocId, xdt, Fields(v_DocKind).Value, Fields(v_Doc).Value);
   Fields(v_DocType).Value :=L_Dspt.GetTypeAsDsptType(xdt);
   Fields(v_Descrip).Value :=L_Dspt.GetDescriptionForXDocId(DsptDoc.XDocId);
   L_XDoc.PositionOnXDoc(DsptDoc.XDocId);
   Fields(v_FullXDocName).Value:=L_ODoc.GetXDocName_Full(DsptDoc.XDocId);
  END;
  DsptDoc_SetEditFlag(GetTypeAsXDocType(), Fields(v_DocKind).Value);
 END {DsptDocE_0_OnReposition};

 PRIVATE PROCEDURE DsptDoc_OnBeforeDelete();
 BEGIN
  (*    SysLog_ *)
 END {DsptDoc_OnBeforeDelete};

 PRIVATE PROCEDURE DsptDoc_OnAfterDelete();
 BEGIN
  SetEnvlLttr();
 END {DsptDoc_OnAfterDelete};

 PRIVATE PROCEDURE DsptDoc_OnBeforeEdit();
 BEGIN
  RTI.Get().CurZone.Fields(v_DocKind).OwnerWindowOriginalName:=L_Dspt.Ref_WindowKindName(GetTypeAsXDocType());
  RTI.Get().CurZone.Fields(v_DocKind).OwnerZoneNumber        :=1;
 END {DsptDoc_OnBeforeEdit};

 PRIVATE PROCEDURE DsptDoc_v_DocType_OnSuccessfulChange();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  SETEMPTY(Fields(v_DocKind).Value, Fields(v_Doc).Value, Fields(v_FullXDocName).Value, Fields(v_Descrip).Value);

  SETEMPTY(DsptDoc.DocKind, DsptDoc.Doc, DsptDoc.CopyCnt);
  DsptDoc.DocType:=GetTypeAsXDocType();
  IF ~L_XDoc.XDocTypeHasXDocKind(DsptDoc.DocType) THEN
   DsptDoc.DocKind        :=00;
   Fields(v_DocKind).Value:=DsptDoc.DocKind;
  ELSE
   L_WDO.CreateFormLink_Simple(L_XDoc.Ref_KindName(DsptDoc.DocType), "v_DocKind")
  END;
  REPLACE(DsptDoc);

  Fields(v_DocKind).OwnerWindowOriginalName:=L_XDoc.Ref_KindName(DsptDoc.DocType);
  Fields(v_DocKind).OwnerZoneNumber        :=1;

  DsptDoc_SetEditFlag(GetTypeAsXDocType(), Fields(v_DocKind).Value);
 END {DsptDoc_v_DocType_OnSuccessfulChange};

 PRIVATE PROCEDURE DsptDoc_OnBeforeInsert();
 BEGIN
  IF Dspt.Type=L_Dspt.t_Incoming THEN
   DsptDoc.Own:=0;
  ELSE
   DsptDoc.Own:=1;
  END;
  REPLACE(DsptDoc);
  RTI.Get().CurZone.Fields(v_DocType).Value:=01;
  DsptDoc_v_DocType_OnSuccessfulChange();
 END {DsptDoc_OnBeforeInsert};

 PRIVATE PROCEDURE TestInsertingDsptDoc(trgXDocId: L_XDoc.tXDocId; CopyCnt: L_XDoc.tCopyCnt);
  VAR
   XDocFirm: LIKE(Firm.Firm);
 BEGIN
  L_General.Assert(L_XDoc.StandOnXDoc(trgXDocId), "㬥  ");
  L_General.Assert(L_Dspt.AcceptableXDocIdInDspt(trgXDocId),
                   " ⨯ 㬥  ⨬  ࠢ");
  IF L_Dspt.GetRightCopyCntByXDocId(trgXDocId) # CopyCnt THEN
   L_StdFrm.Choice("     ᮮ⢥  㬠 ⥫");
  END;

  IF !Dspt.Firm THEN
   Dspt.Firm:=L_XDoc.GetFirmByXDocId(trgXDocId);
   REPLACE(Dspt);
   L_General.Assert(~!Dspt.Firm, "㬥  ᮤন ଠ樨  ࠣ");
  ELSE
   XDocFirm:=L_XDoc.GetFirmByXDocId(trgXDocId);
   IF ~!XDocFirm & (XDocFirm # Dspt.Firm) THEN
    L_StdFrm.Choice("'㦠 ஭' ࠢ  ࠣ 㬥 ࠧ");
   END;
  END;
  IF L_XDoc.GetMyFirmByXDocId(trgXDocId) # Dspt.MyFirm THEN
   L_StdFrm.Choice("'' ࠢ  '' 㬥 ࠧ");
  END;
 END {TestInsertingDsptDoc};

 PRIVATE PROCEDURE DsptDoc_OnAfterEdIns();
  VAR
   xdt   : L_XDoc.tXDocType;
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  L_General.Assert(~!Fields(v_Doc).Value, "  㬥");
  xdt:=GetTypeAsXDocType();
  DsptDoc.XDocId :=L_XDoc.XDocId(xdt, Fields(v_DocKind).Value, Fields(v_Doc).Value);
  TestInsertingDsptDoc(DsptDoc.XDocId, DsptDoc.CopyCnt);
  DsptDoc.DocType:=xdt;
  DsptDoc.DocKind:=Fields(v_DocKind).Value;
  DsptDoc.Doc    :=Fields(v_Doc).Value;
  REPLACE(DsptDoc);
  SetEnvlLttr();
 END {DsptDoc_OnAfterEdIns};

 PRIVATE PROCEDURE DsptDoc_OnAfterInsert();
 BEGIN
  DsptDoc_OnAfterEdIns();
 END {DsptDoc_OnAfterInsert};

 PRIVATE PROCEDURE DsptDoc_OnAfterEdit();
 BEGIN
  DsptDoc_OnAfterEdIns();
 END {DsptDoc_OnAfterEdit};

 PRIVATE PROCEDURE DsptDoc_v_Doc_OnBeforeAsk();
 BEGIN
  L_XDoc.PositionInVideoSystem(L_XDoc.pm_WaitUserChoice,
                               L_XDoc.XDocId(GetTypeAsXDocType(), RTI.Get().CurZone.Fields(v_DocKind).Value, RTI.Get().CurZone.Fields(v_Doc).Value),
                               "Dspt.DsptDoc_XDoc_OnAfterAsk()");
 END {DsptDoc_v_Doc_OnBeforeAsk};

 PRIVATE PROCEDURE DsptDoc_v_Doc_OnSuccessfulChange();
  VAR
   Fields  : FORM_FIELDS;
   FileName: STRING[];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  FileName:=L_XDoc.Ref_FileName(GetTypeAsXDocType());
  L_General.Assert(MAXOF(^(FileName + "." + FileName)) > Fields(v_Doc).Value,
                   "誮 讥 祭");
  DsptDoc.Doc    :=Fields(v_Doc).Value;
  DsptDoc.XDocId :=L_XDoc.XDocId(DsptDoc.DocType, DsptDoc.DocKind, DsptDoc.Doc);
  L_General.Assert(L_XDoc.StandOnXDoc(DsptDoc.XDocId), "㬥  ");
  IF L_XDoc.ExtractXDocType(DsptDoc.XDocId) = L_XDoc.xdt_Contract THEN
   L_General.Assert(Contract.Leaf, "ᯮ짮 蠡 ஢ ⨬");
  END;
  DsptDoc.CopyCnt:=L_Dspt.GetRightCopyCntByXDocId(DsptDoc.XDocId);
  REPLACE(DsptDoc);
  L_XDoc.PositionOnXDoc(DsptDoc.XDocId);
  Fields(v_FullXDocName).Value:=L_ODoc.GetXDocName_Full(DsptDoc.XDocId);
  Fields(v_Descrip).Value     :=L_Dspt.GetDescriptionForXDocId(DsptDoc.XDocId);
 END {DsptDoc_v_Doc_OnSuccessfulChange};

 PUBLIC PROCEDURE DsptDoc_XDoc_OnAfterAsk();
  VAR
   trgXDocType: L_XDoc.tXDocType;
   trgXDocKind: L_XDoc.tXDocKind;
   trgXDoc    : L_XDoc.tXDoc;
 BEGIN
  L_XDoc.DecodeXDocId(L_XDoc.ref_XDocId, trgXDocType, trgXDocKind, trgXDoc);
  DsptDoc.Doc   :=trgXDoc;
  DsptDoc.XDocId:=L_XDoc.ref_XDocId;
  REPLACE(DsptDoc);
  RTI.Get().CurZone.Fields(v_Doc).Value:=DsptDoc.Doc;
  DsptDoc_v_Doc_OnSuccessfulChange()
 END {DsptDoc_XDoc_OnAfterAsk};

 PRIVATE PROCEDURE DsptDoc_v_DocKind_OnSuccessfulChange();
  VAR
   Fields  : FORM_FIELDS;
   KindName: STRING[];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  IF ~!Fields(v_DocKind).Value THEN
   KindName:=L_XDoc.Ref_KindName(GetTypeAsXDocType());
   L_General.Assert(SEARCH(^KindName, ^KindName, Fields(v_DocKind).Value),
                    " 㬥⮢  ");
  END;
  SETEMPTY(Fields(v_Doc).Value, Fields(v_Descrip).Value, Fields(v_FullXDocName).Value);
  SETEMPTY(DsptDoc.Doc, DsptDoc.CopyCnt);
  DsptDoc.DocKind:=Fields(v_DocKind).Value;
  REPLACE(DsptDoc);
  DsptDoc_SetEditFlag(GetTypeAsXDocType(), Fields(v_DocKind).Value);
 END {DsptDoc_v_DocKind_OnSuccessfulChange};

 PRIVATE PROCEDURE XDocKindCanBeReturned(trgXDocType: L_XDoc.tXDocType;
                                         trgXDocKind: L_XDoc.tXDocKind): SCAL[2];
 BEGIN
  IF    (trgXDocType = L_XDoc.xdt_Contract) THEN
   RETURN 1;
  ELSIF (trgXDocType = L_XDoc.xdt_MDoc)     THEN
   RETURN (trgXDocKind = L_MDoc.md_ITN) |
          (trgXDocKind = L_MDoc.md_IAS) |
          (trgXDocKind = L_MDoc.md_IAP) |
          (trgXDocKind = L_MDoc.md_IAU) |
          (trgXDocKind = L_MDoc.md_VAU) |
          (trgXDocKind = L_MDoc.md_PIO);
  ELSIF (trgXDocType = L_XDoc.xdt_ODoc)     THEN
   SEARCH(ODocKind, ODocKind, , trgXDocKind);
   RETURN L_kn.Is(SYSDATE(), L_ODoc.erkn_ContractAddIns) |
          L_kn.Is(SYSDATE(), L_ODoc.erkn_FirmMessage);
  ELSE
   RETURN 0;
  END;
 END {XDocKindCanBeReturned};

 PRIVATE PROCEDURE IsSpecificIngoingMDoc(trgXDocType: L_XDoc.tXDocType;
                                         trgXDocKind: L_XDoc.tXDocKind): SCAL[2];
 BEGIN
  RETURN (trgXDocType = L_XDoc.xdt_MDoc) & (trgXDocKind = L_MDoc.md_VAU);
 END {IsSpecificIngoingMDoc};

 PRIVATE PROCEDURE GetOwn(trgXDocType: L_XDoc.tXDocType;
                          trgXDocKind: L_XDoc.tXDocKind;
                          trgXDoc    : L_XDoc.tXDoc): LIKE(DsptDoc.Own);
  VAR
   Result: LIKE(DsptDoc.Own);
 BEGIN
  IF    Dspt.Type = L_Dspt.t_Incoming THEN
   Result:=L_Dspt.d_Own;
  ELSIF Dspt.Type = L_Dspt.t_Outgoing THEN
   Result:=L_Dspt.d_Alien;
  ELSE
   L_General.CallUs(0);
  END;

  IF SELECT(DsptDoc, DsptDoc, Dspt.Type, Dspt.Dspt, trgXDocType, trgXDocKind, trgXDoc, Result) THEN
   IF XDocKindCanBeReturned(trgXDocType, trgXDocKind) THEN
    L_General.Assert(~SELECT(DsptDoc, DsptDoc, Dspt.Type, Dspt.Dspt, trgXDocType, trgXDocKind, trgXDoc, ~Result),
                     "  㬥?");
    Result:=~Result;
   ELSE
    L_General.Error(" 㬥    ");
   END;
  END;
  RETURN Result;
 END {GetOwn};

 PRIVATE PROCEDURE GetInstance(trgOwn     : LIKE(DsptDoc.Own);
                               trgXDocType: L_XDoc.tXDocType;
                               trgXDocKind: L_XDoc.tXDocKind): LIKE(DsptDoc.Instance);
  VAR
   Result: LIKE(DsptDoc.Instance);
 BEGIN
  IF    trgXDocType = L_XDoc.xdt_MDoc THEN
   SEARCH(MDocKind, MDocKind, , trgXDocKind);
   IF    IsSpecificIngoingMDoc(trgXDocType, trgXDocKind) THEN
    IF trgOwn = L_Dspt.d_Own THEN
     Result:=L_Dspt.inst_Copy;
    ELSE
     Result:=L_Dspt.inst_Original;
    END;
   ELSIF L_MDoc.IsPrchMDocKind() THEN
    IF trgOwn = L_Dspt.d_Own THEN
     Result:=L_Dspt.inst_Original;
    ELSE
     Result:=L_Dspt.inst_Copy;
    END;
   ELSE
    IF trgOwn = L_Dspt.d_Own THEN
     Result:=L_Dspt.inst_Copy;
    ELSE
     Result:=L_Dspt.inst_Original;
    END;
   END;
  ELSIF (trgXDocType = L_XDoc.xdt_ODoc) & (trgOwn = L_Dspt.d_Own) THEN
   SEARCH(ODocKind, ODocKind, , trgXDocKind);
   IF L_kn.Is(SYSDATE(), L_ODoc.erkn_FirmMessage) THEN
    Result:=L_Dspt.inst_Copy;
   ELSE
    Result:=L_Dspt.inst_Original;
   END;
  ELSE
   Result:=L_Dspt.inst_Original;
  END;
  RETURN Result;
 END {GetInstance};

 PRIVATE PROCEDURE InsDsptDoc(    trgXDocId : L_XDoc.tXDocId;
                                  trgCopyCnt: L_XDoc.tCopyCnt;
                              VAR trgMessage: STRING[]);
  VAR
   trgXDocType: L_XDoc.tXDocType;
   trgXDocKind: L_XDoc.tXDocKind;
   trgXDoc    : L_XDoc.tXDoc;
   trgOwn     : LIKE(DsptDoc.Own);
   trgInstance: LIKE(DsptDoc.Instance);
 BEGIN
  trgMessage:="";
  GE_WZD_CURSOR(GE_CURWIN(), 0, Dspt);
  L_XDoc.DecodeXDocId(trgXDocId, trgXDocType, trgXDocKind, trgXDoc);
  IF ~L_Dspt.XDocMustHaveCopyCnt(trgXDocId) THEN
   SETEMPTY(trgCopyCnt);
  END;
  TestInsertingDsptDoc(trgXDocId, trgCopyCnt);

  trgOwn     :=GetOwn(trgXDocType, trgXDocKind, trgXDoc);
  trgInstance:=GetInstance(trgOwn, trgXDocType, trgXDocKind);

  DEFAULT(DsptDoc);
  DsptDoc.Dspt    :=Dspt.Dspt;
  DsptDoc.Type    :=Dspt.Type;
  SETEMPTY(DsptDoc.Nbr);
  DsptDoc.XDocId  :=trgXDocId;
  DsptDoc.DocType :=trgXDocType;
  DsptDoc.DocKind :=trgXDocKind;
  DsptDoc.Doc     :=trgXDoc;
  DsptDoc.CopyCnt :=trgCopyCnt;
  DsptDoc.Own     :=trgOwn;
  DsptDoc.Instance:=trgInstance;
  INSERT(DsptDoc);
  SetEnvlLttr();
 END {InsDsptDoc};

 PRIVATE PROCEDURE SendSingleDspt(trgDate: DATE; trgTime: TIME; trgTSN: LIKE(Staff.TSN));
 BEGIN
  L_General.Assert(Dspt.Type # L_Dspt.t_Incoming, "室饥 ࠢ  ࠢ");
  IF L_Dspt.IsDsptSent(Dspt.Type, Dspt.Dspt) THEN
   L_General.Assert_wReferToAdmin(0, "ࠢ " + Dspt.Dspt + " 㦥 ࠢ");
   RemoveSingleDsptTime();
  END;
  InsertDsptTimeByDspt(L_Dspt.rp_Send, trgDate, trgTime);
  InsertCourier(trgTSN);
  L_XDoc.XDocJrn_Mark(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Edit);
 END {SendSingleDspt};

 PUBLIC PROCEDURE SendDspt();
  VAR
   D: DATE;
   T: TIME;
   m: STRING[1];
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  L_General.Assert(Dspt.Status # L_Dspt.st_Canceled, "ࠢ 㫨஢");
  D:=SYSDATE();
  T:=SYSTIME();
  IF L_General.IsAdmin() THEN
   m:="";
  ELSE
   m:="N";
  END;

  STDFORM("|ࠢ " + Dspt.Dspt + " ࠢ %" + m + "I%" + m + "YI" +
          "||%< {C} >X   %< {}믮 >DB", , INFORM, -1, D, T);
  CONFORM(Dspt, DsptMode);
  L_General.Assert(DsptMode.PackProp # L_Dspt.pck_Must,
                   "ࠢ ࠢ ᪠ ⮫쪮  ⠢ ");
  L_General.Assert(L_DatTim.IsActive(D, DsptMode.Begin, DsptMode.Till),
                   "ᮡ ࠢ " + DsptMode.DsptMode + "   " + L_DatTim.DateStr(D));
  SendSingleDspt(D, T, L_General.EMPTY_NUMBER());
  L_StdFrm.Inform("ࠢ ࠢ " + L_DatTim.DateTimeStr(D, T));
 END {SendDspt};

 PUBLIC PROCEDURE CancelSendDspt();
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  L_General.Assert(L_Dspt.IsDsptSent(Dspt.Type, Dspt.Dspt), "ࠢ " + Dspt.Dspt + "  ࠢ");
  L_StdFrm.Choice("⬥ ࠢ ࠢ " + Dspt.Dspt);
  SELECT(DsptTime, Dspt, , Dspt.Type, Dspt.Dspt);
  L_General.Assert(LAST(DsptTime), " ⬥⪨  ࠢ ࠢ");
  L_General.Assert_wReferToAdmin(DsptTime.RefPoint = L_Dspt.rp_Send, " ஢ ᫥ ࠢ");
  RemoveDsptTimeByDspt();
  L_XDoc.XDocJrn_Mark(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Edit);
 END {CancelSendDspt};

 PUBLIC PROCEDURE PosOnDsptByXDoc();
  VAR
   XDocId      : L_XDoc.tXDocId;
   DsptXDocIds,
   List        : STRING[];
 BEGIN
  DsptXDocIds:="";
  List       :="";
  SELECT(DsptDoc, XDocId, , L_XDoc.GetCurrentXDocIdSoft());
  WHILE OCCURE(DsptDoc) DO
    XDocId:=L_XDoc.XDocId(L_XDoc.xdt_Dispatch, DsptDoc.Type, DsptDoc.Dspt);
    IF ~Str.Match(DsptXDocIds, "*" + XDocId + "*") THEN
     CONFORM(DsptDoc, Dspt);
     DsptXDocIds:=DsptXDocIds + "|" + XDocId;
     List       :=List + "|" + Dspt.Dspt + " " +
                               L_Str.PoolItem(L_Dspt.TypePool, Dspt.Type) + " " +
                               L_Str.PoolItem(L_Dspt.st_Pool, Dspt.Status);
    END;
   NEXT(DsptDoc);
  END; (* WHILE OCCURE(DsptDoc) ... *)
  XDocId:=L_Str.PoolItem(DsptXDocIds, L_StdFrm.ShowListAsMenu(List, "ࠢ"));
  IF ~!XDocId THEN
   L_XDoc.PositionInVideoSystem(L_XDoc.pm_StandOnly, XDocId, "");
  END;
 END {PosOnDsptByXDoc};

 PUBLIC PROCEDURE PosOnCurXDoc();
 BEGIN
  L_General.Assert(ACCESS(DsptDoc), " । ⥪騩 㬥");
  L_XDoc.PositionInVideoSystem(L_XDoc.pm_StandOnly, DsptDoc.XDocId, "");
 END {PosOnCurXDoc};

 PRIVATE PROCEDURE GetOwnCSReceiveDateTime(SingleDspt: SCAL[2]);
 BEGIN
  STDFORM("| %<D.N.y>" + L_Str.PoolItem("N|", SingleDspt) + "I  " +
          "६ %<H:M>" + L_Str.PoolItem("N|", SingleDspt) + "I" +
          "|室  %16YI" +
          "||%< {}⬥ >X   %< {}믮 >DB"),
          " ਥ " + L_Str.PoolItem("|ࠢ", SingleDspt) + " 訬 ஬ 裡 ",
          ,
          -1,
          DsptTime.Date, DsptTime.Time, Dspt.SrcNbr);
 END {GetOwnCSReceiveDateTime};

 PRIVATE PROCEDURE SingleDspt_OwnCS_Receive(trgDate: DATE; trgTime: TIME; SingleDspt: SCAL[2]);
  VAR
   trgSrcNbr: LIKE(Dspt.SrcNbr);
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  PROMPT("ਥ ࠢ 訬 ஬ 裡 " + Dspt.Dspt);
  L_General.Assert(L_Dspt.IsDsptSent(Dspt.Type, Dspt.Dspt), "ࠢ " + Dspt.Dspt + "  ࠢ");
  L_General.Assert(~SEARCH(DsptTime, Dspt, Dspt.Type, Dspt.Dspt, L_Dspt.rp_OwnCS_Receive),
                   " ࠢ " + Dspt.Dspt + " 㦥   ਥ 訬 ஬ 裡");
  trgSrcNbr:=Dspt.SrcNbr;
  STDFORM("|ࠢ " + Dspt.Dspt + L_Str.Repeat(" ", 28) +
          "| %40,5NI" + L_Str.Repeat("|", 5) +
          "| %<D.N.y>" + L_Str.PoolItem("N|", SingleDspt) + "I  " +
          "६ %<H:M>" + L_Str.PoolItem("N|", SingleDspt) + "I" + L_Str.Repeat(" ", 16) +
          "|室  %16YI" + L_Str.Repeat(" ", 14) +
          "||%< {}⬥ >X   %< {}믮 >DB",
          " ਥ ࠢ 訬 ஬ 裡 ",
          ,
          -1,
          L_ctc.GetAdr_Pst(SYSDATE(), Dspt.Firm), trgDate, trgTime, trgSrcNbr);

  Dspt.SrcNbr:=trgSrcNbr;
  REPLACE(Dspt);
  InsertDsptTimeByDspt(L_Dspt.rp_OwnCS_Receive, trgDate, trgTime);
  L_XDoc.XDocJrn_Mark(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Edit);
 END {SingleDspt_OwnCS_Receive};

 PUBLIC PROCEDURE Dspt_OwnCS_Receive();
 BEGIN
  SingleDspt_OwnCS_Receive(SYSDATE(), SYSTIME(), 1);
 END {Dspt_OwnCS_Receive};

 PUBLIC PROCEDURE DsptPck_OwnCS_Receive();
  VAR
   trgDate: DATE;
   trgTime: TIME;
 BEGIN
  L_General.Assert(ACCESS(DsptPck), " ⠭ ");
  trgDate:=SYSDATE();
  trgTime:=SYSTIME();
  STDFORM("| %<D.N.y>I  ६ %<H:M>I" +
          "||%< {}⬥ >X   %< {}믮 >DB",
          " ਥ  訬 ஬ 裡 ",
          ,
          -1,
          trgDate, trgTime);
  SELECT(Dspt, DsptPck, , DsptPck.DsptPck);
  WHILE OCCURE(Dspt) DO
   SingleDspt_OwnCS_Receive(trgDate, trgTime, 0);
   NEXT(Dspt);
  END; (* WHILE OCCURE(Dspt) ... *)
  L_StdFrm.Inform(" ਭ 訬 ஬ 裡 " + L_DatTim.DateTimeStr(trgDate, trgTime));
 END {DsptPck_OwnCS_Receive};

 PRIVATE PROCEDURE IncludeDsptInDsptPck(xdId: L_XDoc.tXDocId; CopyCnt: L_XDoc.tCopyCnt; VAR _Message: STRING[]);
 BEGIN
  L_General.Assert(ACCESS(DsptPck), " ⠭ ");
  _Message:="";
  L_General.Assert(L_XDoc.ExtractXDocType(xdId) = L_XDoc.xdt_Dispatch, "  ࠢ");
  L_General.CallUs(L_XDoc.StandOnXDoc(xdId));
  MayIncludeDsptInDsptPck();
  IF ~!Dspt.DsptPck THEN
   L_General.Error("ࠢ 㦥 祭  " +
                   L_Str.PoolItem("㣮|", Dspt.DsptPck = DsptPck.DsptPck) + " ");
  END;
  Dspt.DsptPck:=DsptPck.DsptPck;
  REPLACE(Dspt);
 END {IncludeDsptInDsptPck};

 PRIVATE PROCEDURE Dspt_Date_OnSuccessfulChange();
 BEGIN
  SetEnvlLttr();
 END {Dspt_Date_OnSuccessfulChange};

 PRIVATE PROCEDURE Dspt_EnvlLttr_OnSuccessfulChange();
 BEGIN
  CalculateWeight();
 END {Dspt_EnvlLttr_OnSuccessfulChange};

 PRIVATE PROCEDURE DsptPckE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAfterEdit, L_WDO.e_OnAfterInsert]);
  L_WDO.iProgs       (0, "Dspt");
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iProgs       (1, "Dspt");
  L_WDO.iEvents      (1, [L_WDO.e_OnSetSubwindow]);
  L_WDO.iZone        (1).Narrowing:="Dspt.DsptPck=DsptPck.DsptPck";
  L_WDO.iField       (1, "Dspt.Type").Format:="室饥|室饥";
 END {DsptPckE_OnInitializeWindow};

 PRIVATE PROCEDURE DsptPckE_1_OnSetSubwindow();
  VAR
   curProcessKind: NUMERIC[0];
 BEGIN
  curProcessKind:=RTI.Get().CurProcess.Kind;
  IF    curProcessKind = FORM_PROCESS.Kind_Insert THEN
   L_StdFrm.BlueStop("ࠢ    ⮫쪮  - ࠢ (. 쭮 )");
  ELSIF curProcessKind = FORM_PROCESS.Kind_Edit THEN
   L_General.Error("᪠ ⮫쪮  ࠢ    -|" +
                   " ⨥ ࠢ   (. 쭮 )");
  END;
 END {DsptPckE_1_OnSetSubwindow};

 PUBLIC PROCEDURE DeleteDsptFromDsptPck();
 BEGIN
   ACCESS(Dspt);
   L_StdFrm.Choice(" ࠢ " + Dspt.Dspt + "   " + DsptPck.DsptPck);

   SETEMPTY(Dspt.DsptPck);
   REPLACE(Dspt);
 END {DeleteDsptFromDsptPck};

 PRIVATE PROCEDURE DsptPck_OnBeforeEdit();
  VAR
   Fields: FORM_FIELDS;
   Field : FORM_FIELD;
 BEGIN
  IF RTI.Get().CurWindow.OriginalName = "DsptPckSend" THEN
   L_WDO.CreateFormLink("Staff", "v_dp_Courier");
   Fields:=RTI.Get().CurZone.Fields;

   Field:=Fields(v_dp_SendDate);
   Field.Value     :=SYSDATE();
   Field.Modifiable:=L_General.IsAdmin();
 
   Field:=Fields(v_dp_SendTime);
   Field.Value     :=SYSTIME();
   Field.Modifiable:=L_General.IsAdmin();
 
   Field:=Fields(v_dp_CourierTSN);
   ACCESS(DsptPck);
   SELECT(Dspt, DsptPck, , DsptPck.DsptPck);
   Field.Value:=P_DsptM.GetDsptMProSoft(Dspt.DsptMode, DsptPck.Date, P_DsptM.P_Courier);

   FIXALTERING();
  END;
 END {DsptPck_OnBeforeEdit};

 PRIVATE PROCEDURE DsptPck_OnAfterEdIns();
  VAR
   List,
   mfList: STRING[];
   Cnt   : NUMERIC[0];
 BEGIN
  IF (DsptPck.MyFirm = P_Firm.mf_Fict) & SELECT(Dspt, DsptPck, DsptPck.DsptPck) THEN
   SETEMPTY(List, mfList);
   Cnt:=0;
   WHILE OCCURE(Dspt) DO
    CONFORM(Dspt, MyFirm);
    IF (Cnt = 0) | (Str.Pos(mfList, MyFirm.MyFirm) = Str.Length(mfList)) THEN
     INC(Cnt);
     mfList:=mfList + MyFirm.MyFirm + "|";
     List  :=List + MyFirm.MyFirm + " " + MyFirm.vzSign + " " + MyFirm.Name + "|";
    END;
    NEXT(Dspt);
   END; (* WHILE OCCURE(Dspt) ... *)
   IF Cnt = 1 THEN
    DsptPck.MyFirm:=L_Str.PoolItem(mfList, 0);
   ELSE
    DsptPck.MyFirm:=L_Str.PoolItem(mfList, L_StdFrm.ShowListAsMenu(List, "  "));
   END;
   REPLACE(DsptPck);
  END;
 END {DsptPck_OnAfterEdIns};

 PRIVATE PROCEDURE DsptPck_OnAfterInsert();
 BEGIN
  DsptPck_OnAfterEdIns();
 END {DsptPck_OnAfterInsert};

 PRIVATE PROCEDURE DsptPck_OnAfterEdit();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  DsptPck_OnAfterEdIns();
  IF RTI.Get().CurWindow.OriginalName = "DsptPckSend" THEN
   SELECT(Dspt, DsptPck, , DsptPck.DsptPck);
   WHILE OCCURE(Dspt) DO
    PROMPT("ࠢ ࠢ " + Dspt.Dspt);
    SendSingleDspt(Fields(v_dp_SendDate).Value,
                   Fields(v_dp_SendTime).Value,
                   Fields(v_dp_CourierTSN).Value);
    NEXT(Dspt);
   END; (* WHILE OCCURE(Dspt) ... *)
   L_StdFrm.Inform(" ࠢ " +
                   L_DatTim.DateTimeStr(Fields(v_dp_SendDate).Value, Fields(v_dp_SendTime).Value));
  END;
 END {DsptPck_OnAfterEdit};

 PUBLIC PROCEDURE Dspt_MarkDelayNotice();
  VAR
   trgDate: DATE;
   trgTime: TIME;
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ⠭ ࠢ");
  L_General.Assert(L_Dspt.IsDsptSent(Dspt.Type, Dspt.Dspt), "ࠢ " + Dspt.Dspt + "  ࠢ");
  P_Firm.AccessFirm(Dspt.Firm);
  IF SEARCH(DsptTime, Dspt, Dspt.Type, Dspt.Dspt, L_Dspt.rp_DelayNotice) THEN
   L_General.Error(Firm.Name + "|㦥  㢥  窥  㬥⮢ " +
                   L_DatTim.DateTimeStr(DsptTime.Date, DsptTime.Time));
  END;
  trgDate:=SYSDATE();
  trgTime:=SYSTIME();
  STDFORM("|" + Firm.Name +
          "|㢥  窥  㬥⮢."+
          "|| %I  ६ %I" +
          "||%< {}⬥ >X   %< {}믮 >DB", "  ⬥ ",
          INFORM,
          -1,
          trgDate, trgTime);
  InsertDsptTimeByDspt(L_Dspt.rp_DelayNotice, trgDate, trgTime);
  L_XDoc.XDocJrn_Mark(L_XDoc.xdt_Dispatch, Dspt.Type, Dspt.Dspt, L_XDoc.action_Edit);
 END {Dspt_MarkDelayNotice};

 PUBLIC PROCEDURE InsertDocByBarCode();
 BEGIN
  L_XDoc.DoByBarCode("⠢", PROCREF(InsDsptDoc), 1);
 END {InsertDocByBarCode};

 PUBLIC PROCEDURE InsertPacketDocByBarCode();
 BEGIN
  L_XDoc.DoByBarCode("祭  ", PROCREF(IncludeDsptInDsptPck), 1);
 END {InsertPacketDocByBarCode};

 PRIVATE PROCEDURE DsptMode_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="Dspt";
  L_WDO.iSubwindow   (1, "DsptMdeE");
  L_WDO.iField       (1, "DsptMode.Type").Format:=L_Dspt.TypeShortPool;
 END {DsptMode_OnInitializeWindow};

 PRIVATE PROCEDURE DsptMdeE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAfterEdit, L_WDO.e_OnAfterInsert]);
  L_WDO.iProgs       (0, "Dspt");
  L_WDO.iUnmodifiable(0, ["DsptMode.DsptMode"]);
  L_WDO.iMayBeEmpty  (0, ["DsptMode.Till", "DsptMode.DlvNtf", "DsptMode.URL"]);
  L_WDO.iField       (0, "DsptMode.Name").First:=1;
  L_WDO.iField       (0, "DsptMode.Type").Format:=L_Dspt.TypePool;
 END {DsptMdeE_OnInitializeWindow};

 PRIVATE PROCEDURE GetPosOfBorderSymbol(Symbol, Text: STRING[]): NUMERIC[0];
  VAR
   tmpPos,
   Result: NUMERIC[0];
 BEGIN
  Result:=Str.Pos(DsptMode.URL, Symbol);
  IF Result = Str.Length(DsptMode.URL) THEN
   L_General.Error("४ ଠ  URL:  " + Text + " ᨬ '"+Symbol+"',|" +
                   "ࠬ騩 ६  ");
  END;
  tmpPos:=Str.Pos(DsptMode.URL, Symbol, Result+1);
  IF tmpPos # Str.Length(DsptMode.URL) THEN
   L_General.Error("४ ଠ  URL:    ⢮|" +
                   "⢥ " + Text + " ᨬ '" + Symbol + "', ࠬ騩 ६  ");
  END;
  RETURN Result;
 END {GetPosOfBorderSymbol};

 PRIVATE PROCEDURE GetFullURL(CheckOnly: SCAL[2]): STRING[];
  VAR
   begPos,
   endPos: NUMERIC[0];
   Param,
   Value,
   Result: STRING[];
 BEGIN
  L_General.Assert(~!DsptMode.URL, " ᯮᮡ ࠢ " + DsptMode.DsptMode + " '" + DsptMode.Name + "'|" +
                                   " 㪠 URL  ᫥ ⮢ ࠢ");

  begPos:=GetPosOfBorderSymbol("<", "뢠騩");
  endPos:=GetPosOfBorderSymbol(">", "뢠騩");
  IF endPos < begPos THEN
   L_General.Error("४ ଠ  URL: 뢠騩 ᨬ '<' ६  |" +
                   " ᯮ  뢠饣 ᨬ '>'");
  END;

  Param:=Str.Condense(Str.Slice(DsptMode.URL, begPos+1, endPos-begPos-1));
  L_General.Assert(~!Param, " 㪠 ६  ");
  IF L_Str.IsEqStrI(Param, "DSPT.SRCNBR") THEN
   IF CheckOnly THEN
    Value:="XXX";
   ELSE
    L_General.Assert(~!Dspt.SrcNbr, " ࠢ " + Dspt.Dspt + "  㪠  ⮢ ࠢ|" +
                                    "(室  ࠢ)");
    Value:=Dspt.SrcNbr;
   END;
  ELSE
   L_General.InformUs("⭮ ᠭ ६   '" + Param +"'");
  END;
  Result:=Str.Delete(DsptMode.URL, begPos, endPos);
  Result:=Str.Insert(Result, Value, begPos);
  RETURN Result;
 END {GetFullURL};

 PRIVATE PROCEDURE DsptMode_OnAfterEdIns();
  VAR
   Dummy: STRING[];
 BEGIN
  IF ~!DsptMode.URL THEN
   Dummy:=GetFullURL(1);
  END;
 END {DsptMode_OnAfterEdIns};

 PRIVATE PROCEDURE DsptMode_OnAfterInsert();
 BEGIN
  DsptMode_OnAfterEdIns();
 END {DsptMode_OnAfterInsert};

 PRIVATE PROCEDURE DsptMode_OnAfterEdit();
 BEGIN
  DsptMode_OnAfterEdIns();
 END {DsptMode_OnAfterEdit};

 PRIVATE PROCEDURE Dspt_DsptMode_OnAskPhase1();
  VAR
   OwnerZone: FORM_ZONE;
 BEGIN
  IF  RTI.Get().CurWindow.OriginalName = "DsptIE" THEN
   RTI.Get().TrgZone.PermanentFilter:="(DsptMode.Type=" + L_Str.IntStr(L_Dspt.t_Incoming) + ")";
  ELSIF RTI.Get().CurWindow.OriginalName = "DsptOE" THEN
   RTI.Get().TrgZone.PermanentFilter:="(DsptMode.Type=" + L_Str.IntStr(L_Dspt.t_Outgoing) + ")";
  ELSE
   L_General.CallUs(0);
  END;
  IF ~!Dspt.DsptMode THEN
   CONFORM(Dspt, DsptMode);
   L_WDO.PositionInOwnerZoneOfActiveField("DsptMode");
  END;
 END {Dspt_DsptMode_OnAskPhase1};

 PRIVATE PROCEDURE Dspt_DsptMode_OnAskPhase2();
 BEGIN
  Dspt.DsptMode:=DsptMode.DsptMode;
  REPLACE(Dspt);
 END {Dspt_DsptMode_OnAskPhase2};

 PUBLIC PROCEDURE scs_GetFilters_DsptO_1(VAR Filters: L_WDO.tFilters);
 BEGIN
  L_WDO.PutNewFilter(Filters, " 祭",  "(Dspt.Status#" + L_Str.IntStr(L_Dspt.st_Delivered) + ")", 1);
 END {scs_GetFilters_DsptO_1};

 PUBLIC PROCEDURE scs_GetNarrowing_DsptO_1(VAR NarrowingDescriptions: STRING[]);
 BEGIN
  NarrowingDescriptions:="⮢" +
                         "|ࠢ";
 END {scs_GetNarrowing_DsptO_1};

 PUBLIC PROCEDURE scs_SetNarrowing_DsptO_1(SelectChoice: NUMERIC[0]);
 BEGIN
  IF    SelectChoice = 1 THEN
   SELECT(Dspt, DsptSt, , L_Dspt.t_Outgoing, L_Dspt.st_Prepared);
  ELSIF SelectChoice = 2 THEN
   SELECT(Dspt, DsptSt, , L_Dspt.t_Outgoing, L_Dspt.st_Sent);
  END;
 END {scs_SetNarrowing_DsptO_1};

 PUBLIC PROCEDURE Tracking();
  VAR
   trgURL: STRING[];
 BEGIN
  L_General.Assert(ACCESS(Dspt), " ࠭ ࠢ");
  L_General.Assert(~!Dspt.DsptMode, " ࠢ " + Dspt.Dspt + "  㪠 ᯮᮡ ࠢ");
  CONFORM(Dspt, DsptMode);
  trgURL:=GetFullURL(0);
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   Shell.Execute("open", trgURL);
  EXCEPT
  ON Exc:Exc_Root DO
   L_General.Error("४ URL '" + trgURL + "'");
  END; (* TRY *)
 END {Tracking};

BEGIN
 L_Events.ExecuteHandler();
END.
