(*
%Copyright   (c) Polak Computer System
%Author      06.09.2001 20:19:26 SAN
%Purpose     প Automated Converter Launching
%Type        library
%Specific    SYS
%Modified    10.09.2014 13:14:03 POLAK\San
%Environment
*)

 MODULE L_ACL [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_StdFrm, L_Str, SysLog, Str, FIO, L_Basis, L_FIO;

 PRIVATE TYPE
  tDoneStatus = SCAL[3];

 PRIVATE CONST
  ProgramsPath   = "ASL";
  PrgPrefix      = "ACL_";
  ACLMask        = PrgPrefix + "???????";
  PrgExt         = "ASL";
  PrgMask        = ACLMask + "." + PrgExt;
  sle_RunACLItem = SysLog.UserEvent + 1;

  ed_NotEssential   = "    ஥ 05.11.2002 19:23:05 POLAK\SAN";
  ed_DelaedByAdmin  = "⫮ ஬ 04.11.2001 21:43:06 SAN";
  ed_MarkOnlyByUs   = "쪮 ⬥⪠  SYSLOG 14.11.2001 18:14:23 SAN";
  ed_AdminsConflict = " ,   믮  ⮩ ᨨ." +
                      "|" +
                      "|,   㣮 "+
                      "|室    믮  㣮 ᨨ";

  ds_Never = 0;
  ds_Done  = 1;
  ds_Newer = 2;

 PUBLIC PROCEDURE TestEssentialityForTheProject(Cond: SCAL[2]);
 BEGIN
  IF ~Cond THEN
   L_StdFrm.Inform(" ᭨,  室  믮 ");
   RAISE(1, ed_NotEssential);
  END;
 END {TestEssentialityForTheProject};

 PRIVATE PROCEDURE GetACLName(): STRING[];
  VAR
   Point     : CALL_STACK_CELL;
   Result,
   ModuleName: STRING[];
 BEGIN
  SETEMPTY(Result);
  FOR_EACH Point IN RTI.Get().Stack DO
   ModuleName:=Str.Cap(Point.Proc.Module.Name);
   IF Str.Match(ModuleName, ACLMask) THEN
    L_General.CallUs(!Result);
    Result:=ModuleName;
   END;
  END; (* FOR_EACH Point IN RTI.Get().Stack...*)
  RETURN Result;
 END {GetACLName};

 PUBLIC PROCEDURE Inform(WhatToDo, WhatToPrepare: STRING[]);
  VAR
   Msg,
   OurButton: STRING[];
   Result   : NUMERIC[0];
 BEGIN
  Msg:="|";
  IF ~!WhatToPrepare THEN
   Msg:=Msg+
        WhatToPrepare+
        "| |";
  END;
  Msg:=Msg+WhatToDo;
  IF ~APP.Get().ProductiveCopy THEN
   OurButton:=L_StdFrm.DrawButton( " ᯮ", 16, "B");
  ELSE
   SETEMPTY(OurButton);
  END;
  Result:=STDFORM(Msg +
                  "| |"+
                  L_StdFrm.DrawButton("믮", 16, "D")+
                  "            " +
                  L_StdFrm.DrawButton( "⫮", 16, "X")+
                  " " +
                  OurButton,
                  ": 믮  " + GetACLName(),
                  CHOICE);
  IF    Result = 0 THEN
   L_StdFrm.RedInform(" ⫮ 믮 ஢"+
                      "|"+
                      ", ࠡ 짮⥫ 㤥 㤭");
   RAISE(1, ed_DelaedByAdmin);
  ELSIF (Result = 1) & APP.Get().SupportMode THEN
   L_StdFrm.Choice("믮   ࠡ祩 .  㢥७?");
  ELSIF (Result = 2) & ~APP.Get().ProductiveCopy THEN
   L_StdFrm.RedInform("  䠪 ᯮ ");
   RAISE(1, ed_MarkOnlyByUs);
  END;
 END {Inform};

 PUBLIC PROCEDURE TalkAboutTransparency(): STRING[];
 BEGIN
  RETURN L_StdFrm.HR() + "室 믮  孨᪨ 稭";
 END {TalkAboutTransparency};

 PUBLIC PROCEDURE LongTimeProceed_wRenewal(): STRING[];
 BEGIN
  RETURN L_StdFrm.HR() + "믮     ⥫쭮 ६." +
                         "  砥 室, 믮   ࢠ - " +
                         " ᫥饬 ᪥  筥 ࠡ   ⠭.";
 END {LongTimeProceed_wRenewal};

 PUBLIC PROCEDURE TalkAboutYouMayInterrupt(): STRING[];
 BEGIN
  RETURN L_StdFrm.HR() + "[   뢠]";
 END {TalkAboutYouMayInterrupt};

 PUBLIC PROCEDURE wEMailSend(): STRING[];
 BEGIN
  RETURN "|" + "( ࠢ  ஭ )" + "|";
 END {wEMailSend};

 PRIVATE PROCEDURE SysLogCommentStamp(ACLName: STRING[]; UTCDTS: TIMESTAMP): STRING[];
  VAR
   fd: DATE;
   ft: TIME;
   fs: NUMERIC[0];
   fx: NUMERIC[3];
 BEGIN
  UnPkTIMESTAMP(UTCDTS, fd, ft, fs, fx);
  INC(fs, fx);
  RETURN "஢ " + L_DatTim.DateStr(fd) + " " + FLDTOSTR(ft, "H:M", 5) + FLDTOSTR(fs, "L:00", 3);
 END {SysLogCommentStamp};

 PRIVATE PROCEDURE GetACLDoneStatus(ACLName: STRING[]; UTCDTS: TIMESTAMP): tDoneStatus;
  VAR
   to    : NUMERIC[0];
   Result: tDoneStatus;
 BEGIN
  TRY [MSG_SUPPRESSION(MSG_SUPPRESSION_ALL)]
   Result:=ds_Never;
   SELECT(SYSLOG, SYSLOG1, , L_General.sle_RunACLItem, ACLName);
   WHILE OCCURE(SYSLOG) & (Result # ds_Done) DO
    IF SysLogCommentStamp(ACLName, UTCDTS) = SYSLOG.Comment THEN
     Result:=ds_Done;
    ELSE
     Result:=ds_Newer;
    END;
    NEXT(SYSLOG);
   END;
   RETURN Result;
  EXCEPT
  ON Exc:Exc_AccessDeny DO
   IF Exc.ErrorCode = L_General.ec_ad_IsLockedByOtherConnection THEN
    to:=10;
    STDMSG(ed_AdminsConflict, STOP, , to);
   ELSE
    RAISE();
   END;
  END; (* TRY *)
 END {GetACLDoneStatus};

 PRIVATE PROCEDURE IsDone(Number: SCAL[1E7]): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  TRY
   TABLE.Get("SYSLOG").AccessDataMode:=adm_SOFT;
   Result:=SELECT(SYSLOG, SYSLOG1, L_General.sle_RunACLItem, "ACL_"+Number);
  EXCEPT
  END; (* TRY *)
  RETURN Result;
 END {IsDone};

 PUBLIC PROCEDURE CheckACLIsDone(Number: SCAL[1E7]);
  VAR
   Msg: STRING[];
 BEGIN
  IF ~IsDone(Number) THEN
   Msg:=" 믮   ஬ " + Number;
   IF L_General.IsAdmin() THEN
    L_General.Error(Msg);
   ELSE
    STDMSG("    ᮮ  ᫥饥:"+L_StdFrm.HR()+Msg, STOP_AND_CALL_ADMIN);
   END;
  END;
 END {CheckACLIsDone};

 PRIVATE PROCEDURE AlreadyDone(ACLName: STRING[]; UTCDTS: TIMESTAMP): SCAL[2];
  VAR
   ds: tDoneStatus;
 BEGIN
  ds:=GetACLDoneStatus(ACLName, UTCDTS);
  IF    ds = ds_Never THEN
   RETURN 0;
  ELSIF ds = ds_Done THEN
   RETURN 1;
  ELSIF ds = ds_Newer THEN
   IF ~APP.Get().ProductiveCopy | APP.Get().SupportMode THEN
    RETURN ~L_StdFrm.RedAlternative(" " + ACLName + " 㦥 믮!!!||믮 ୮?");
   ELSE
    TRY
     L_General.extCallUs(" " + ACLName + " 㦥 믮");
    EXCEPT
    ON Exc: Exc_Raise DO
     L_StdFrm.RedInform(Exc.MsgOriginalText);
    ELSE
     RAISE();
    END; (* TRY *)
    RETURN 1;
   END;
  END; (* IF ds ... *)
 END {AlreadyDone};

 PRIVATE PROCEDURE MarkAsDone(ACLName: STRING[]; UTCDTS: TIMESTAMP; InSlaveTransaction: SCAL[2]);
  VAR
   dummyDS    : tDoneStatus;
   Transaction: NUMERIC[0];
 BEGIN
  IF InSlaveTransaction THEN
   Transaction:=TRANSACTION_SLAVE;
  ELSE
   Transaction:=TRANSACTION_CURRENT;
  END;
  TRY [TRANSACTION(Transaction)]
   dummyDS:=GetACLDoneStatus(ACLName, UTCDTS); (* ஫ ६ ᯮ *)
   SysLog.Mark(ACLName, SysLogCommentStamp(ACLName, UTCDTS), L_General.sle_RunACLItem);
  EXCEPT
  ON Exc: Exc_Root DO
   RAISE();
  END; (* TRY *)
 END {MarkAsDone};

 PRIVATE PROCEDURE GetPatternPath(PrgName: STRING[]): STRING[];
 BEGIN
  IF L_General.IsPolakSpecific() THEN
   RETURN "\\polaknt03\Account\Polak\PIT\"+ProgramsPath+"\"+PrgName+"."+PrgExt;
  ELSE
   RETURN "\\polaknt03\Prj_Patterns\"+DATABASE.Get().Name+"\"+ProgramsPath+"\"+PrgName+"."+PrgExt;
  END;
 END {GetPatternPath};

 PRIVATE PROCEDURE HasEqualVersionOnPatterns(ACLName: STRING[];
                                             ACLFile: FOLDER_ENTRY): SCAL[2];
 BEGIN
  IF L_General.IsPolakSpecific() | (L_General.GetAPSiteID() = "JOURNAL") THEN
   RETURN 1;
  ELSE
   RETURN L_FIO.IsEqualFiles(ACLFile, FOLDER_ENTRY.Create(GetPatternPath(ACLName)));
  END;
 END {HasEqualVersionOnPatterns};

 PRIVATE PROCEDURE IsACL(ACLName: STRING[]): SCAL[2];
 BEGIN
  RETURN L_Str.TestStrAlowChar(Str.Slice(ACLName, Str.Length(PrgPrefix), 7), "09");
 END {IsACL};

 PUBLIC PROCEDURE InternalProceed(): NUMERIC[0];
  CONST
   ACLStoragePeriod = 42;
  VAR
   Msg,
   trgACLName   : STRING[];
   ACLFiles     : ENUMERATOR;
   trgUTCDTS    : TIMESTAMP;
   ACLFolder,
   trgACLFile   : FOLDER_ENTRY;
   i,
   ACLAmount,
   DoneACLAmount: NUMERIC[0];
 BEGIN
  IF L_General.IsAdmin() THEN
   APP.Get().NewPrompt("");
   L_AFM.MakeMultipleIndex("ACLCl", "UTCDTS : TIMESTAMP;" +
                                    "ACLName: STRING[32]",
                           "ACLCl:ACLName;UTCDTS:UTCDTS");
   ACLFolder:=FOLDER_ENTRY.CreateRedirector(PrgMask);
   ACLFiles :=CreateENUMERATOR(ACLFolder.FolderContent);
   WHILE ~ACLFiles.AtEnd() DO
    trgACLFile:=ACLFiles.Item();
    trgACLName:=Str.Cap(L_FIO.effp_FileName_woExt(trgACLFile.Path));
    PROMPT("஢ઠ 室 믮 " + trgACLName);
    IF IsACL(trgACLName) THEN
     trgUTCDTS:=trgACLFile.UTCLastModified - 1/(24*60*60) / 2;
     IF    ~AlreadyDone(trgACLName, trgUTCDTS) THEN
      L_General.CallUs(~SEARCH(ACLCl, ACLCl, trgACLName));
      ACLCl.ACLName:=trgACLName;
      ACLCl.UTCDTS :=trgUTCDTS;
      INSERT(ACLCl);
     ELSIF ~APP.Get().ProductiveCopy               &
           !FIO.AsbSpecialFolders("PrototypeRoot") &
           (L_DatTim.GetDateOfTIMESTAMP(trgUTCDTS) <= (SYSDATE() - ACLStoragePeriod)) THEN
      IF FIO.Exists(GetPatternPath(trgACLName)) THEN
       IF HasEqualVersionOnPatterns(trgACLName, trgACLFile) |
          L_StdFrm.RedAlternative("  " + trgACLName + "." +
                                  "|| Pattern 室   㣮 ⮩ 䨪樨." +
                                  "|| ࠡ   " + trgACLName + "?") THEN
        FIO.SetAttr(trgACLFile.Path, , 0);
        FIO.Erase(trgACLFile.Path);
       END;
      END;
     END;
    END; (* IF IsACL(trgACLName)...*)
    ACLFiles.MoveNext();
   END; (* WHILE ~ACLFiles.AtEnd()...*)

   ACLAmount    :=AMOUNT(ACLCl);
   SETZERO(i, DoneACLAmount);
   SELECT(ACLCl, UTCDTS);
   LOOP
    IF ~OCCURE(ACLCl) THEN
     EXIT;
    END;
    PROMPT("믮 " + ACLCl.ACLName);
    TRY [TRANSACTION(TRANSACTION_SLAVE), MSG_SUPPRESSION(MSG_SUPPRESSION_NONE), TERM_BY_USER(TERM_BY_USER_THROW)]
     SETEMPTY(Msg);
     MarkAsDone(ACLCl.ACLName, ACLCl.UTCDTS, 0);
     CALL(^ACLCl.ACLName, 000);
     INC(DoneACLAmount);
    EXCEPT
    ON Exc: Exc_Raise DO
     IF    (Exc.MsgOriginalText = ed_MarkOnlyByUs) | (Exc.MsgOriginalText = ed_NotEssential) THEN
      INC(DoneACLAmount);
      MarkAsDone(ACLCl.ACLName, ACLCl.UTCDTS, 1);
     ELSIF (Exc.MsgOriginalText # ed_DelaedByAdmin) & (Exc.MsgOriginalText # ed_AdminsConflict) THEN
      Msg:=Exc.MsgOriginalText;
     END;
    ON Exc: Exc_Root DO
     Msg:=Exc.MsgOriginalText;
    END; (* TRY *)

    IF ~!Msg THEN
     Msg:=Msg + "|| " + ACLCl.ACLName + "  !|";
     IF    i = (ACLAmount-1) THEN
      L_StdFrm.RedInform(Msg);
     ELSIF ~L_StdFrm.RedAlternative(Msg + "||த 믮 ஢ (⠫ " + L_Str.IntStr(ACLAmount - i-1) + ")?") THEN
      EXIT;
     END;
    END; (* IF ~!Msg... *)
    INC(i);
    NEXT(ACLCl);
   END; (* LOOP *)
   RETURN ACLAmount - DoneACLAmount;
  ELSE
   RETURN 0;
  END; (* IF L_General.IsAdmin() ... *)
 END {InternalProceed};

 PUBLIC PROCEDURE ProceedFromAutoExec(LaunchAtLast: STRING[]);
  VAR
   Dummy: NUMERIC[0];
 BEGIN
  Dummy:=InternalProceed();
  IF ~!LaunchAtLast THEN
   LIKEMENU(^LaunchAtLast);
  END;
 END {ProceedFromAutoExec};

 PRIVATE PROCEDURE MakeTop();
 BEGIN
  L_Basis.wb_WrStr("R", "஥");
  L_Basis.wb_WrStr("R", "");
  L_Basis.wb_WrStr("R", "");
  L_Basis.wb_WrStr("R", "");
  L_Basis.wb_WrStr("R", "६");
  L_Basis.wb_WrStr("R", "");
  L_Basis.wb_TermLine("R");
 END {MakeTop};

 PRIVATE PROCEDURE MakeLine(SingleRepName,
                            RepName: STRING[];
                            RepDate: DATE;
                            RepTime: TIME;
                            RepSize: NUMERIC[0]);
 BEGIN
  L_Basis.wb_WrStr("R", Str.Cap(L_General.GetAPSiteID()));
  L_Basis.wb_WrStr("R", SingleRepName);
  L_Basis.wb_WrStr("R", RepName);
  L_Basis.wb_WrDat("R", RepDate);
  L_Basis.wb_WrTim("R", RepTime);
  L_Basis.wb_WrStr("R", L_Str.IntStr(RepSize));
  L_Basis.wb_TermLine("R");
 END {MakeLine};

 PRIVATE PROCEDURE WriteRep(RepName,
                            SingleRepName: STRING[]);
  VAR
   DTS    : TIMESTAMP;
 BEGIN
  DTS:=L_FIO.GetFileLastModifiedDTS(RepName);
  MakeLine(SingleRepName, RepName, L_DatTim.GetDateOfTIMESTAMP(DTS), L_DatTim.GetTimeOfTIMESTAMP(DTS), L_FIO.GetFileSize(RepName));
 END {WriteRep};

 PRIVATE PROCEDURE FindRep(RepName      : STRING[];
                           VAR RepAmnt  : NUMERIC[0];
                           SingleRepName: STRING[]);
 BEGIN
  PROMPT(RepName);
  IF FIO.Exists(RepName) THEN
   WriteRep(RepName, SingleRepName);
   INC(RepAmnt);
  END;
 END {FindRep};

 PRIVATE PROCEDURE FindRepTxt(RepName,
                              RepPath      : STRING[];
                              VAR RepAmnt  : NUMERIC[0];
                              SingleRepName: STRING[]);
  VAR
   i     : NUMERIC[0];
   enm   : ENUMERATOR;
   f, fso: OBJECT;
 BEGIN
  IF FIO.DirExists(RepPath) THEN
   fso:=CreateOBJECT("Scripting.FileSystemObject");
   f  := fso.GetFolder(RepPath);
   enm:= CreateENUMERATOR(f.Files);
   WHILE ~enm.AtEnd() DO
    IF Str.Match(enm.Item().Name, RepName) THEN
     WriteRep(FIO.MakePath(enm.Item().Name, , RepPath), SingleRepName);
     INC(RepAmnt);
    END;
    enm.MoveNext();
   END(*WHILE*);
  END;
 END {FindRepTxt};

 PRIVATE PROCEDURE IsNotTemplate(S: STRING[]):SCAL[2];
 BEGIN
  RETURN (Str.Length(S)=Str.Pos(S, "*")) & (Str.Length(S)=Str.Pos(S, "?"));
 END {IsNotTemplate};

 PUBLIC PROCEDURE RequestAboutUsingRep(Request,
                                       SearchTemplate: STRING[]): STRING[];
  CONST
   RepName = "RqRes";
  VAR
   SingleRepName: STRING[];
   i, j,
   SearchAmnt   : NUMERIC[0];
   TmpDate,
   SysDate      : DATE;
 BEGIN
  L_General.Assert(~!Request, " 㪠 ");
  L_General.Assert(IsNotTemplate(SearchTemplate), "ᯮ짮 ᪨ ᪠  ⨬");
  FIO.Open(R, L_General.NameConnection(RepName), "REP", Create);
  MakeTop();
  j            :=0;
  SearchAmnt   :=0;
  SingleRepName:=L_Str.AnyItem(SearchTemplate, ";", j);
  WHILE ~!SingleRepName DO
   L_General.Assert((Str.Length(SingleRepName) = 5) |
                    (Str.Length(SingleRepName) = 3),
                    "⠭⭠    " + SingleRepName);
   IF Str.Length(SingleRepName) = 5 THEN
    i:=0;
    WHILE i <= 99 DO
     FindRep(SingleRepName + "_" + L_Str.Repeat("0", 2 - Str.Length(L_Str.IntStr(i))) + L_Str.IntStr(i) + ".rep",
             SearchAmnt, SingleRepName);
     INC(i);
    END;
    FindRepTxt(SingleRepName + "*" + ".txt",
               FIO.MakePath(SingleRepName, , L_Basis.GetBasisPath(L_Basis.CurrentBranch(), SYSDATE())),
               SearchAmnt,
               SingleRepName);
   ELSE
    SysDate:=L_DatTim.Per(SYSDATE());
    TmpDate:=L_DatTim.IncYear(SysDate, -3);
    WHILE TmpDate <= SysDate DO
     FindRep(SingleRepName + "_" + FLDTOSTR(TmpDate, "YN") + ".rep",
             SearchAmnt, SingleRepName);
     TmpDate:=L_DatTim.IncMonth(TmpDate, +1);
    END; (*WHILE TmpDate <= SysDate...*)
   END;
   INC(j);
   SingleRepName:=L_Str.AnyItem(SearchTemplate, ";", j);
  END; (* WHILE ~!SingleRepName...*)
  FIO.Close(R);
  RETURN FIO.FirstPath(L_General.NameConnection(RepName), "rep");
 END {RequestAboutUsingRep};

 PRIVATE PROCEDURE _SubjectForMassInquiry(ACLName, BaseName: STRING[]): STRING[];
 BEGIN
  RETURN "On request " + ACLName + ". Send from " + BaseName + ".";
 END {_SubjectForMassInquiry};

 PUBLIC PROCEDURE SubjectForMassInquiry(ACLName: STRING[]): STRING[];
 BEGIN
  RETURN _SubjectForMassInquiry(ACLName, L_General.GetAPSiteID());
 END {SubjectForMassInquiry};

 PUBLIC PROCEDURE IsReplyOnMassInquiry(Subject: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Match(Subject, _SubjectForMassInquiry("*", "*"));
 END {IsReplyOnMassInquiry};

 PUBLIC PROCEDURE ACLNameFromMassInquiryReply(Subject: STRING[]): STRING[];
 BEGIN
  RETURN Str.Item(Subject, " .", 2);
 END {ACLNameFromMassInquiryReply};

 PUBLIC PROCEDURE BaseNameFromMassInquiryReply(Subject: STRING[]): STRING[];
  VAR
   p: NUMERIC[0];
 BEGIN
  p:=Str.Pos(Subject, ". Send from ");
  Subject:=Str.Slice(Subject, p + 12, Str.Length(Subject));
  Subject:=Str.Slice(Subject, 0, Str.Length(Subject)-1);
  RETURN Str.Replace(Subject, "'", "");
 END {BaseNameFromMassInquiryReply};

END.
