(*
%Copyright   (c) Polak Computer System
%Author      28.05.2000 19:24:36 SAN
%Purpose     ⥪  ࠡ  ATTACHED 䠩 (Attached Files Manipulations)
%Type        library
%Specific    SYS
%Modified    29.05.2014 12:48:58 POLAK\LDA
%Environment
*)

 MODULE L_AFM [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tStage = SCAL[2];

 PUBLIC CONST
  tdm_None      = 0;
  tdm_Full      = 1;
  tdm_CurSelect = 2;

  s_Registering = 0;
  s_Attaching   = 1;

 IMPORT L_General, FIO, Str, L_Basis, L_DatTim, L_Str, L_StdFrm, L_DBD, Math, L_FIO, Memo;

 PRIVATE TYPE
  tDestination = SCAL[2];

 PRIVATE CONST
  dst_Basis   = 0;
  dst_ASBTemp = 1;

  om_Input  = "Input";
  om_Output = "Output";
  om_Create = "Create";

 PUBLIC PROCEDURE AF_CreateTemp(_Name, _File: STRING[]);
 BEGIN
  ATTACH_TABLE(^_Name, Temporary, _File + "_" + APP.Get().MyConnection.Number + ".did");
 END {AF_CreateTemp};

 PUBLIC PROCEDURE CreateTemp(_Name: STRING[]);
 BEGIN
  AF_CreateTemp(_Name, _Name);
 END {CreateTemp};

 PUBLIC PROCEDURE ReattachTemp(_Name, _File: STRING[]);
 BEGIN
  L_General.CallUs(~!DATABASE.Get().Tables(_Name));
  AF_CreateTemp(_Name, _File);
 END {ReattachTemp};

 PUBLIC PROCEDURE AF_PurgeTemp(_Name: STRING[]);
 BEGIN
  DETACH_TABLE(^_Name, Purge);
 END {AF_PurgeTemp};

 PUBLIC PROCEDURE MakeMultipleIndex(trgFile, FieldsList, IndexesList: STRING[]);
  VAR
   i, j, Pos       : NUMERIC[0];
   Char,
   tmpVal,
   trgField,
   trgType,
   trgDefault,
   trgIndexName,
   trgIndexFields  : STRING[];
   ForceCreateIndex: SCAL[2];
 BEGIN
  IF !DATABASE.Get().Tables(trgFile) THEN
   ForceCreateIndex:=!IndexesList;

   CREATE_TABLE(^trgFile, "", "", "");
   SETZERO(i);
   LOOP
    tmpVal:=Str.Item(FieldsList, ";", i);
    IF !tmpVal THEN
     EXIT;
    END;

    trgField:=Str.Condense(Str.Item(tmpVal, ":", 0));
    tmpVal  :=Str.Condense(Str.Item(tmpVal, ":", 1));

    IF ~Str.Match(tmpVal, "*DEFAULT*") THEN
     trgType:=tmpVal;
     SETEMPTY(trgDefault);
    ELSE
     Pos    :=Str.Pos(tmpVal, "DEFAULT");
     trgType:=Str.Slice(tmpVal, 0, Pos);
     j:=Str.Length(trgType);
     LOOP
      Char   :=L_Str.Char(trgType, j);
      trgType:=Str.Delete(trgType, j, 1);
      IF Char = "," THEN
       EXIT;
      END;
      DEC(j);
      L_General.CallUs(j >= 0);
     END; (* LOOP *)

     trgDefault:=Str.Condense(Str.Item(Str.Slice(tmpVal, Pos, Str.Length(tmpVal)), "=", 1));
    END;

    CREATE_FIELD(^trgFile, ^trgField, ^trgType, , trgDefault);

    IF ForceCreateIndex THEN
     IndexesList:=L_Str.AddStrDlm(IndexesList, ",") + trgField;
    END;
    INC(i);
   END; (* LOOP *)

   IF ForceCreateIndex THEN
    IndexesList:=trgFile + ":" + IndexesList;
   END;

   SETZERO(i);
   LOOP
    tmpVal:=Str.Item(IndexesList, ";", i);
    IF !tmpVal THEN
     EXIT;
    END;
    trgIndexName  :=Str.Condense(Str.Item(tmpVal, ":", 0));
    trgIndexFields:=Str.Condense(Str.Item(tmpVal, ":", 1));
    IF i = 0 THEN
     CREATE_INDEX(^trgFile, ^trgIndexName, UNIQUE, , , ^trgIndexFields);
    ELSE
     CREATE_INDEX(^trgFile, ^trgIndexName, PLURAL, , , ^trgIndexFields);
    END;
    INC(i);
   END; (* LOOP *)

   SET_VERSION_OF_TABLE(^trgFile, 0000);
  END; (* IF !DATABASE.Get().Tables( ... *)
  CreateTemp(trgFile);
 END {MakeMultipleIndex};

 PUBLIC PROCEDURE MakeIndex(trgFile, FieldsList: STRING[]);
 BEGIN
  MakeMultipleIndex(trgFile, FieldsList, "");
 END {MakeIndex};

 PUBLIC PROCEDURE extMakeDataBaseStructureCopy(srcFileName,
                                               dstFileName,
                                               NewFieldList,
                                               NewIndexList    : STRING[];
                                               TransferDataMode: SCAL[9]);
  VAR
   i, j            : NUMERIC[0];
   SingleFieldDescr,
   SingleIndexDescr: STRING[];
   srcTable,
   dstTable        : TABLE;
   curField        : FIELD;
 BEGIN
  L_DBD.CloneFileStructure(srcFileName, dstFileName, 1);
  i:=0;
  LOOP
   SingleFieldDescr:=Str.Item(NewFieldList, ";", i);
   IF !SingleFieldDescr THEN
    EXIT;
   END;
   CREATE_FIELD(^dstFileName, ^(Str.Item(SingleFieldDescr, ":", 0)), ^(Str.Item(SingleFieldDescr, ":", 1)));
   INC(i);
  END;

  i:=0;
  LOOP
   SingleIndexDescr:=Str.Item(NewIndexList, ";", i);
   IF !SingleIndexDescr THEN
    EXIT;
   END;
   CREATE_INDEX(^dstFileName,
                ^(Str.Item(SingleIndexDescr, ":", 0)),
                ^(Str.Item(SingleIndexDescr, ":", 1)),
                ,
                ,
                ^(Str.Item(SingleIndexDescr, ":", 2)));
   INC(i);
  END;

  CreateTemp(dstFileName);
  srcTable:=DATABASE.Get().Tables(srcFileName);
  dstTable:=DATABASE.Get().Tables(dstFileName);

  IF TransferDataMode # tdm_None THEN
   IF    TransferDataMode = tdm_Full THEN
    SELECT(^srcFileName, ^(srcTable.Indexes(0).Name));
   ELSIF TransferDataMode = tdm_CurSelect THEN
    L_General.CallUs(OCCURE(^srcFileName));
   ELSE
    L_General.CallUs(0);
   END;
   j:=0;
   WHILE OCCURE(^srcFileName) DO
    PROMPT(" ஢  " + L_Str.IntStr(AMOUNT(^srcFileName)-j) + "  " + L_Str.IntStr(AMOUNT(^srcFileName)));
    DEFAULT(^dstFileName);
    FOR_EACH curField IN srcTable.Fields DO
     ^(dstTable.Fields(curField.Name).ObtainName() + ":=" + curField.ObtainName());
    END; (* FOR_EACH curField IN ...*)
    INSERT(^dstFileName);
    NEXT(^srcFileName);
    INC(j);
   END; (* WHILE OCCURE(^srcFileName) ... *)
  END;
 END {extMakeDataBaseStructureCopy};

 PUBLIC PROCEDURE AF_MakeDataBaseStructureCopy(srcFileName,
                                               dstFileName,
                                               NewFieldList    : STRING[];
                                               TransferDataMode: SCAL[9]);
 BEGIN
  extMakeDataBaseStructureCopy(srcFileName, dstFileName, NewFieldList, "", TransferDataMode);
 END {AF_MakeDataBaseStructureCopy};

 PRIVATE CONST
  rf_MaxNum = 32;

 PRIVATE TYPE
  tUseItArr = SCAL[2][rf_MaxNum];

 PRIVATE VAR
  rf_DBDName  : STRING[]           [rf_MaxNum];
  rf_DiskName : STRING[]           [rf_MaxNum];
  rf_TimeCond : L_General.tTimeCond[rf_MaxNum];
  rf_IntvlB,
  rf_IntvlE   : DATE               [rf_MaxNum];
  rf_UseIt    : tUseItArr;
  rf_Count    : NUMERIC[0];

 PRIVATE PROCEDURE AccessRFIndex(DBDName: STRING[]): NUMERIC[0];
  VAR
   Index: SCAL[rf_MaxNum];
 BEGIN
  FOR_EACH Index DO
   IF rf_DBDName[Index] = DBDName THEN
    RETURN Index;
   END;
  END;
  L_General.CallUs(0);
 END {AccessRFIndex};

 PRIVATE PROCEDURE GetFullPath(Destination: tDestination;
                               Index      : NUMERIC[0];
                               VAR
                                datPath,
                                keyPath   : STRING[]);
  VAR
   _Path: STRING[];
 BEGIN
  L_General.Assert(Str.Length(rf_DiskName[Index]) <= 5, " (᪮) ६ 䠩 .. <= 5");
  IF    Destination = dst_Basis THEN
   _Path:=L_Basis.GetFullPath(0,
                              rf_DiskName[Index],
                              rf_TimeCond[Index],
                              rf_IntvlB[Index],
                              rf_IntvlB[Index],
                              rf_IntvlE[Index],
                              0);
  ELSIF Destination = dst_ASBTemp THEN
   _Path:=FIO.MakePath(rf_DiskName[Index], , FIO.AsbSpecialFolders("ASBTemp"));
  ELSE
   L_General.CallUs(0);
  END;
  datPath:=FIO.MakePath(_Path, "did");
  keyPath:=FIO.MakePath(_Path, "key");
 END {GetFullPath};

 PRIVATE PROCEDURE AttachRF(Index      : NUMERIC[0];
                            Mode       : STRING[];
                            Destination: tDestination;
                            JustMaked  : SCAL[2]): NUMERIC[0];
  VAR
   fullDatPath,
   fullKeyPath : STRING[];
 BEGIN
  GetFullPath(Destination, Index, fullDatPath, fullKeyPath);
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF), MSG_SUPPRESSION(MSG_SUPPRESSION_ALL)]
   ATTACH_TABLE(^(rf_DBDName[Index]), ^Mode, fullDatPath, fullKeyPath);
   RETURN 0;
  EXCEPT
  ON Exc: Exc_Root DO
   IF    (Exc.ErrorCode = 353) | (Exc.ErrorCode = 376) THEN
    L_General.Error("ॡ  ࠭ 䠩 "+ L_FIO.effp_FileName_woExt(fullDatPath) + " ᮧ  㣮 ⠭樨."+
                    "||१ ஥ ६  ᬮ  ᯮ짮");
   ELSIF Exc.ErrorCode = 115 THEN
    L_General.CallUs(0);
   ELSIF JustMaked & (Exc.ErrorCode = 987) THEN
    L_General.Error(" ६ ନ஢ ࠭ 䠩 '"+TABLE.Get(rf_DBDName[Index]).Description + "'" +
                    "|뫨 ᥭ      ࠢ筨" +
                    L_StdFrm.HR()+"室 믮 ନ஢ 䠩 ");
   ELSE
    RETURN Exc.ErrorCode;
   END;
  END; (* TRY *)
 END {AttachRF};

 PRIVATE PROCEDURE GetUserChoice(Index: NUMERIC[0]): SCAL[3];
  VAR
   bTS   : TIMESTAMP;
   Msg   : STRING[];
   Timing: NUMERIC[0];
 BEGIN
  bTS:=GETFILETIME((rf_DBDName[Index]));
  Timing:=60;
  Msg:="|ॡ 祭   ᭮ 䠩" +
       "|'" + DATABASE.Get().Tables(rf_DBDName[Index]).Description + "'" +
       "|"+L_Basis.TimeCondStr(rf_TimeCond[Index],
                               rf_IntvlB[Index], rf_IntvlB[Index], rf_IntvlE[Index]) +
       "|"+
       "| ⮢  : " + L_DatTim.TimestampStr(bTS) +
       "|⥬ ६   : " + L_DatTim.TimestampStr(SYSTIMESTAMP()) +
       "||"+
       "%<      ᯮ짮 騩 䠩       >FB|"+
       "%<       ந 䠩  (%T)       >DB|"+
       "%<                ⬥                >Z" ;
  RETURN STDFORM(Msg, " 業 ਬ ࠭ 䠩 ", QUESTION, , Timing);
 END {GetUserChoice};

 PRIVATE PROCEDURE rf_Register(DBDName,
                               DiskName: STRING[];
                               TimeCond: L_General.tTimeCond;
                               IntvlB,
                               IntvlE  : DATE);
 BEGIN
  rf_DBDName [rf_Count]:=DBDName;
  rf_DiskName[rf_Count]:=DiskName;
  rf_TimeCond[rf_Count]:=TimeCond;
  rf_IntvlB  [rf_Count]:=IntvlB;
  rf_IntvlE  [rf_Count]:=IntvlE;
  SETEMPTY(rf_UseIt[rf_Count]);
  INC(rf_Count);
 END {rf_Register};

 PUBLIC PROCEDURE ssrf_Register(DBDName,
                                DiskName: STRING[]);
 BEGIN
  rf_Register(DBDName, DiskName, L_General.timeCond_OnDate, L_General.EMPTY_DATE(), L_General.EMPTY_DATE());
 END {ssrf_Register};

 PUBLIC PROCEDURE msrf_Register(DBDName,
                                DiskName: STRING[];
                                TimeCond: L_General.tTimeCond;
                                IntvlB,
                                IntvlE  : DATE);
 BEGIN
  rf_Register(DBDName, DiskName, TimeCond, IntvlB, IntvlE);
 END {msrf_Register};

 PRIVATE PROCEDURE rf_Reuse(DBDName    : STRING[];
                            Mode       : STRING[];
                            Silent     : SCAL[2];
                            Destination: tDestination;
                            limDTS     : TIMESTAMP): SCAL[2];
  VAR
   rf_Index : NUMERIC[0];
   usrChoice: NUMERIC[0];
   bDTS     : TIMESTAMP;
 BEGIN
  rf_Index:=AccessRFIndex(DBDName);
  IF    AttachRF(rf_Index, Mode, Destination, 0) # 0 THEN
   RETURN 0;
  ELSIF Silent THEN
   bDTS:=GETFILETIME((rf_DBDName[rf_Index]));
   RETURN bDTS>=limDTS;
  ELSIF ~!rf_UseIt[rf_Index] THEN
   RETURN rf_UseIt[rf_Index];
  ELSE
   usrChoice:=GetUserChoice(rf_Index);
   IF usrChoice = 2 THEN
    DETACH_TABLE(^(rf_DBDName[rf_Index]));
    RETURN 0;
   ELSE
    RETURN 1;
   END;
  END;
 END {rf_Reuse};

 PUBLIC PROCEDURE ssrf_Reuse(DBDName: STRING[]): SCAL[2];
 BEGIN
  RETURN rf_Reuse(DBDName, om_Input, 1, dst_ASBTemp, MINOF(TIMESTAMP));
 END {ssrf_Reuse};

 PUBLIC PROCEDURE ssrf_Reuse_InOutputMode(DBDName: STRING[]): SCAL[2];
 BEGIN
  RETURN rf_Reuse(DBDName, om_Output, 1, dst_ASBTemp, MINOF(TIMESTAMP));
 END {ssrf_Reuse_InOutputMode};

 PUBLIC PROCEDURE msrf_Reuse(DBDName: STRING[]; Silent: SCAL[2]): SCAL[2];
 BEGIN
  RETURN rf_Reuse(DBDName, om_Input, Silent, dst_Basis, MINOF(TIMESTAMP));
 END {msrf_Reuse};

 PUBLIC PROCEDURE ext_msrf_Reuse(DBDName: STRING[]; Silent: SCAL[2]; limDTS: TIMESTAMP): SCAL[2];
 BEGIN
  RETURN rf_Reuse(DBDName, om_Input, Silent, dst_Basis, limDTS);
 END {ext_msrf_Reuse};

 PRIVATE PROCEDURE rf_Create(DBDName    : STRING[];
                             Destination: tDestination);
 BEGIN
  DETACH_TABLE(^DBDName, Purge);
  L_General.CallUs(AttachRF(AccessRFIndex(DBDName), om_Create, Destination, 0) = 0);
 END {rf_Create};

 PUBLIC PROCEDURE ssrf_Create(DBDName: STRING[]);
 BEGIN
  rf_Create(DBDName, dst_ASBTemp);
 END {ssrf_Create};

 PUBLIC PROCEDURE msrf_Create(DBDName: STRING[]);
 BEGIN
  rf_Create(DBDName, dst_Basis);
 END {msrf_Create};

 PRIVATE PROCEDURE rf_Store(DBDName    : STRING[];
                            Mode       : STRING[];
                            Destination: tDestination);
  VAR
   rf_Index: NUMERIC[0];
 BEGIN
  rf_Index:=AccessRFIndex(DBDName);
  DETACH_TABLE(^(rf_DBDName[rf_Index]), Save);
  L_General.CallUs(AttachRF(rf_Index, Mode, Destination, 1) = 0);
 END {rf_Store};

 PUBLIC PROCEDURE ssrf_Store(DBDName: STRING[]);
 BEGIN
  rf_Store(DBDName, om_Input, dst_ASBTemp);
 END {ssrf_Store};

 PUBLIC PROCEDURE ssrf_Store_InOutputMode(DBDName: STRING[]);
 BEGIN
  rf_Store(DBDName, om_Output, dst_ASBTemp);
 END {ssrf_Store_InOutputMode};

 PUBLIC PROCEDURE msrf_Store(DBDName: STRING[]);
 BEGIN
  rf_Store(DBDName, om_Input, dst_Basis);
 END {msrf_Store};

 PUBLIC PROCEDURE msrf_Store_InOutputMode(DBDName: STRING[]);
 BEGIN
  rf_Store(DBDName, om_Output, dst_Basis);
 END {msrf_Store_InOutputMode};

 PUBLIC PROCEDURE ssrf_Delete(_File: STRING[]);
  VAR
   datPath, keyPath: STRING[];
 BEGIN
  GetFullPath(dst_ASBTemp, AccessRFIndex(_File), datPath, keyPath);
  IF FIO.Exists(datPath) THEN
   FIO.SetAttr(datPath, , 0, 0);
   FIO.Erase(datPath);
  END;
  IF FIO.Exists(keyPath) THEN
   FIO.SetAttr(keyPath, , 0, 0);
   FIO.Erase(keyPath);
  END;
 END {ssrf_Delete};

(*!*) PUBLIC PROCEDURE ReAttachReusableFile(DBDName: STRING[]);
 BEGIN
  rf_Store(DBDName, om_Input, dst_Basis);
 END {ReAttachReusableFile};

(*!*) PUBLIC PROCEDURE CreateReusableFile(DBDName: STRING[]);
 BEGIN
  rf_Create(DBDName, dst_Basis);
 END {CreateReusableFile};

(*!*) PUBLIC PROCEDURE CanReuseFile(DBDName: STRING[]; Silent: SCAL[2]): SCAL[2];
 BEGIN
  RETURN rf_Reuse(DBDName, om_Input, Silent, dst_Basis, MINOF(TIMESTAMP));
 END {CanReuseFile};

(*!*) PUBLIC PROCEDURE NewReusableFile(DBDName,
                                       DiskName: STRING[];
                                       TimeCond: L_General.tTimeCond;
                                       IntvlB,
                                       IntvlE  : DATE);
 BEGIN
  rf_Register(DBDName, DiskName, TimeCond, IntvlB, IntvlE);
 END {NewReusableFile};

 PUBLIC PROCEDURE Init();
 BEGIN
  rf_Count:=0;
  SETEMPTY(rf_DBDName, rf_DiskName, rf_TimeCond, rf_IntvlB, rf_IntvlE, rf_UseIt,
           curStage);
 END {Init};

 PRIVATE VAR
  curStage: tStage;

 PUBLIC PROCEDURE inRegistering(): SCAL[2];
 BEGIN
  RETURN !curStage | (curStage = s_Registering);
 END {inRegistering};

 PUBLIC PROCEDURE inAttaching(): SCAL[2];
 BEGIN
  RETURN !curStage | (curStage = s_Attaching);
 END {inAttaching};

 PRIVATE PROCEDURE rf_PrepareAll_Ask(tc: L_General.tTimeCond);
  VAR
   bDate  : DATE;
   bTime  : TIME;
   Msg,
   Vars,
   theOne : STRING[];
   asUseIt: tUseItArr;
   i      : NUMERIC[0];
   R      : NUMERIC[0];
 BEGIN
  Msg:="|"
     + "ॡ 祭  " + L_Basis.TimeCondStr(tc, Memo.ud_cd(), Memo.ud_fr(), Memo.ud_ut())
     + "||"
     + L_StdFrm.THR(" ᯮ짮 ᫥騥 ࠭ 䠩");
  SETEMPTY(Vars);
  i:=0;
  WHILE i < rf_Count DO
   theOne:="%<" + Str.Align("'" + DATABASE.Get().Tables(rf_DBDName[i]).Description + "' ", 54, LEFT);
   IF AttachRF(i, om_Input, dst_Basis, 0) = 0 THEN
    rf_UseIt[i]:=1;
    GETTIMESTAMP(^(rf_DBDName[i]), bDate, bTime);
    INC(theOne, L_DatTim.DateStr(bDate) + " " + L_DatTim.TimeStr(bTime) + ">E");
    DETACH_TABLE(^(rf_DBDName[i]));
   ELSE
    rf_UseIt[i]:=0;
    INC(theOne, Str.Align("㤥 ஥", 16, LEFT) + ">NE");
   END;
   INC(Msg, "|" + theOne);
   IF ~!Vars THEN
    INC(Vars,  ", ");
   END;
   INC(Vars, "asUseIt[" + L_Str.IntStr(i) + "]");
   INC(i);
  END;
  asUseIt:=rf_UseIt;
  IF ~L_General.BatchExecution_IsRunning() THEN
   LOOP
    R:=STDFORM(Msg + "||"
                   + L_StdFrm.DrawButton(" 믮 ", 0, "DB")
                   + L_StdFrm.DrawButton(" ᯮ짮  ", 0, "B")
                   + L_StdFrm.DrawButton("  ᯮ짮 祣 ", 0, "B")
                   + L_StdFrm.DrawButton(" ⮯ ", 0, "Z"),
               " 業 ਬ ࠭ 䠩 ",
               QUESTION, , ^(Vars));
    IF    R = 2 THEN
     i:=0;
     WHILE ~!rf_UseIt[i] DO
      asUseIt[i]:=rf_UseIt[i];
      INC(i);
     END;
    ELSIF R = 3 THEN
     i:=0;
     WHILE ~!rf_UseIt[i] DO
      asUseIt[i]:=0;
      INC(i);
     END;
    ELSE
     i:=0;
     WHILE ~!rf_UseIt[i] DO
      rf_UseIt[i]:=asUseIt[i];
      INC(i);
     END;
     EXIT;
    END;
   END; (* LOOP *)
  END;
 END {rf_PrepareAll_Ask};

 PUBLIC PROCEDURE rf_PrepareAll(P: PROC; tc: L_General.tTimeCond);
 BEGIN
  TRY
   Init();
   curStage:=s_Registering;
   P();
   rf_PrepareAll_Ask(tc);
   curStage:=s_Attaching;
   P();
  EXCEPT
  FINALLY
   Init();
  END;
 END {rf_PrepareAll};

 PUBLIC PROCEDURE rf_PreparePromptStr(DBDName: STRING[]): STRING[];
 BEGIN
  RETURN "⮢ ࠭ 䠩 '"+TABLE.Get(DBDName).Description+"'";
 END {rf_PreparePromptStr};

BEGIN
 Init();
END.
