(*
%Copyright   (c) Polak Computer System
%Author      19.03.2002 02:27:41 MOG
%Purpose     ⥪  ࠡ  -
%Type        library
%Specific    SYS
%Modified    08.07.2014 16:39:23 POLAK\LDA
%Environment
*)

 MODULE L_BarCode [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tBarCodeType    = SCAL[1E3];
  tBCSRegime      = SCAL[4];
  tBCScannerLevel = SCAL[3];

 PUBLIC CONST
  BCS_ProfileName = "AutoPark";

  BCSr_OnLine         = 0;
  BCSr_Disconnect     = 1;
  BCSr_None           = 2;
  BCSr_AutoDisconnect = 3;

  bcsl_None         = 0;
  bcsl_Introduction = 1;
  bcsl_Normal       = 2;

  ed_FinalError   = "25.05.2004 12:11:40 POLAK\San - 訡,    ࠡ  ᪠஬";

 (* OPOS "State" Property Constants*)
  S_CLOSED = 1;
  S_IDLE   = 2;
  S_BUSY   = 3;
  S_ERROR  = 4;
 
 (* OPOS "ResultCode" Property Constants *)
  SUCCESS      = 0;
  E_CLOSED     = 101;
  E_CLAIMED    = 102;
  ENotclaimed  = 103;
  E_NOSERVICE  = 104;
  E_DISABLED   = 105;
  E_ILLEGAL    = 106;
  E_NOHARDWARE = 107;
  E_OFFLINE    = 108;
  E_NOEXIST    = 109;
  E_EXISTS     = 110;
  E_FAILURE    = 111;
  E_TIMEOUT    = 112;
  E_BUSY       = 113;
  E_EXTENDED   = 114;
 
 (* OPOS "OpenResult" Property Constants *)
  OPENERR          = 300;
  OR_ALREADYOPEN   = 301;
  OR_REGBADNAME    = 302;
  OR_REGPROGID     = 303;
  OR_CREATE        = 304;
  OR_BADIF         = 305;
  OR_FAILEDOPEN    = 306;
  OR_BADVERSION    = 307;
  OPENERRSO        = 400;
  ORS_NOPORT       = 401;
  ORS_NOTSUPPORTED = 402;
  ORS_CONFIG       = 403;
  ORS_SPECIFIC     = 450;
 
 (* OPOS "BinaryConversion" Property Constants *)
  BC_NONE    = 0;
  BC_NIBBLE  = 1;
  BC_DECIMAL = 2;
 
 (* "CheckHealth" Method: "Level" Parameter Constants *)
  CH_INTERNAL    = 1;
  CH_EXTERNAL    = 2;
  CH_INTERACTIVE = 3;
 
 (* OPOS "CapPowerReporting", "PowerState", "PowerNotify" Property Constants *)
  PR_NONE     = 0;
  PR_STANDARD = 1;
  PR_ADVANCED = 2;
 
  PN_DISABLED = 0;
  PN_ENABLED  = 1;
 
  PS_UNKNOWN     = 2000;
  PS_ONLINE      = 2001;
  PS_OFF         = 2002;
  PS_OFFLINE     = 2003;
  PS_OFF_OFFLINE = 2004;
 
 (* OPOS.Scanner "ScanDataType" Property Constants *)
  SCAN_SDT_UPCA     = 101;  (* Digits *)
  SCAN_SDT_UPCE     = 102;  (* Digits *)
  SCAN_SDT_JAN8     = 103;  (* = EAN 8 *)
  SCAN_SDT_EAN8     = 103;  (* = JAN 8 (added in 1.2) *)
  SCAN_SDT_JAN13    = 104;  (* = EAN 13 *)
  SCAN_SDT_EAN13    = 104;  (* = JAN 13 (added in 1.2) *)
  SCAN_SDT_TF       = 105;  (* (Discrete 2 of 5) Digits *)
  SCAN_SDT_ITF      = 106;  (* (Interleaved 2 of 5) Digits *)
  SCAN_SDT_Codabar  = 107;  (* Digits, -, $, :, /, ., +; *)
                            (* 4 start/stop characters *)
                            (* (a, b, c, d) *)
  SCAN_SDT_Code39   = 108;  (* Alpha, Digits, Space, -, ., *)
                            (* $, /, +, %; start/stop *)
                            (* Also has Full ASCII feature *)
  SCAN_SDT_Code93   = 109;  (* Same characters as Code 39 *)
  SCAN_SDT_Code128  = 110;  (* 128 data characters *)
  SCAN_SDT_UPCA_S   = 111;  (* UPC-A with supplemental barcode *)
  SCAN_SDT_UPCE_S   = 112;  (* UPC-E with supplemental barcode *)
  SCAN_SDT_UPCD1    = 113;  (* UPC-D1 *)
  SCAN_SDT_UPCD2    = 114;  (* UPC-D2 *)
  SCAN_SDT_UPCD3    = 115;  (* UPC-D3 *)
  SCAN_SDT_UPCD4    = 116;  (* UPC-D4 *)
  SCAN_SDT_UPCD5    = 117;  (* UPC-D5 *)
  SCAN_SDT_EAN8_S   = 118;  (* EAN 8 with supplemental barcode *)
  SCAN_SDT_EAN13_S  = 119;  (* EAN 13 with supplemental barcode *)
  SCAN_SDT_EAN128   = 120;  (* EAN 128 *)
  SCAN_SDT_OCRA     = 121;  (* OCR "A" *)
  SCAN_SDT_OCRB     = 122;  (* OCR "B" *)
 
  (* Two dimensional symbologies *)
  SCAN_SDT_PDF417   = 201;
  SCAN_SDT_MAXICODE = 202;
 
  (* Special cases *)
  SCAN_SDT_OTHER    = 501;  (* Start of Scanner-Specific bar code symbologies *)
  SCAN_SDT_UNKNOWN  = 0;    (* Cannot determine the barcode symbology. *)
  SCAN_STD_KBSCAN   = 999;  (* keyboard input *)

  bcsm_Normal = 0;
  bcsm_Debug  = 1;

 PUBLIC PERMANENT VAR
  bcs_Mode: SCAL[2];

 PRIVATE PERMANENT VAR
  BCSRegime: tBCSRegime;

 IMPORT Str, L_Str, L_StdFrm, L_General, SysLog, Math, L_SpcEq, DBD, L_EC, E_Utils, L_GUI;

 PUBLIC PROCEDURE CheckCheckDigit(s: STRING[]): SCAL[2];
  VAR
   c: STRING[1];
   d,
   j,
   i,
   S: NUMERIC[0];
 BEGIN
  i:=Str.Length(s);
  j:=0;
  IF i=14 THEN (* ITF-14 ? *)
   c:=L_Str.Char(s, j);
   IF (c<"1") | (c>"8") THEN
    RETURN 0;
   END;
   j:=1;
  END;
  d:=3; S:=0;
  WHILE i > j DO
   DEC(i); d:=4-d; c:=L_Str.Char(s, i);
   IF (c>"9") | (c<"0") THEN
    RETURN 0;
   END;
   INC(S, STRTOFLD(c, NUMERIC)*d);
  END;
  RETURN (S MOD 10) = 0;
 END {CheckCheckDigit};

 PUBLIC PROCEDURE IsEAN(s: STRING[]): SCAL[2];
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=Str.Length(s);
  IF (i#8) & (i#12) & (i#13) & (i#14) & (i#17) THEN
   RETURN 0;
  END;
  RETURN CheckCheckDigit(s);
 END {IsEAN};

 PRIVATE PROCEDURE AddCheckDigit(Src: STRING[]): STRING[];
  VAR
   c: STRING[1];
   i, d, S: NUMERIC[0];
 BEGIN
  d:=1; S:=0; i:=Str.Length(Src);
  WHILE i>=1 DO
   DEC(i);
   d:=4-d;
   c:=L_Str.Char(Src, i);
   INC(S, STRTOFLD(c, NUMERIC)*d);
  END;
  RETURN Src + L_Str.IntStr((10 - S MOD 10) MOD 10);
 END {AddCheckDigit};

 PUBLIC PROCEDURE BarCode(s: STRING[]): STRING[];
 BEGIN
  RETURN AddCheckDigit(s);
 END {BarCode};

 PUBLIC PROCEDURE toFont_IntP36DlTt(Src: STRING[]): STRING[];
  CONST
   Step = 2;
   ExeptionLAT   = CHR(196)+CHR(197)+CHR(199)+CHR(201)+
                   CHR(209)+CHR(214)+CHR(220)+CHR(225); (* ANSI "" *)
   ExeptionStart = 92;
  VAR
   i, tmp,
   length: NUMERIC[0];
   Part,
   Dest  : STRING[];
 BEGIN
  L_General.Assert(L_Str.TestStrAlowChar(Src, "09"),
                   "୮ 祭 (⠢)");
  length:=Str.Length(Src);
  L_General.Assert(Math.Mod(length, 2) = 0,
                   " 室 ப   ⭮ ᫮ ᨬ");
  Dest:="";
  i   :=0;
  WHILE i < length DO
   Part:=Str.Slice(Src, i, Step);
   tmp:=STRTOFLD(Part, NUMERIC);
   IF tmp >= ExeptionStart THEN
    Dest:=Dest + L_Str.Char(ExeptionLAT, tmp - ExeptionStart);
   ELSE
    Dest:=Dest + CHR(35 + tmp);
   END;
   INC(i, Step);
  END; (* WHILE i <= length...*)
  RETURN '!' + Dest + '"';
 END {toFont_IntP36DlTt};

 PERMANENT VAR
  oScanner: OBJECT;

 PUBLIC PROCEDURE InitializeMode();
 BEGIN
  IF ~VAR_IMPL.Get(bcs_Mode).Initialized THEN
   bcs_Mode:=bcsm_Normal;
  END;
 END {InitializeMode};

 PRIVATE PROCEDURE InitiallySetBCSRegime();
 BEGIN
  IF L_SpcEq.IsInstalled(L_SpcEq.et_OPOSBCScanner) THEN
   BCSRegime:=BCSr_OnLine;
  ELSE
   BCSRegime:=BCSr_None;
  END;
 END {InitiallySetBCSRegime};

 PUBLIC PROCEDURE GetBCSRegime(): tBCSRegime;
 BEGIN
  InitializeMode();
  IF ~VAR_IMPL.Get(BCSRegime).Initialized | ((BCSRegime # BCSr_None) # L_SpcEq.IsInstalled(L_SpcEq.et_OPOSBCScanner)) THEN
   InitiallySetBCSRegime();
  END;
  RETURN BCSRegime;
 END {GetBCSRegime};

 PUBLIC PROCEDURE SetBCSRegime(_BCSRegime: tBCSRegime);
 BEGIN
  BCSRegime:=_BCSRegime;
  SysLog.Mark("BCS_Reg", L_Str.PoolItem("祭|⪫祭| ⠭|⮮⪫祭", BCSRegime));
 END {SetBCSRegime};

 PUBLIC PROCEDURE DisconnectBarCodeScanner();
 BEGIN
  IF GetBCSRegime() = BCSr_OnLine THEN
   L_StdFrm.Question(" ॡ ६ ⪫祭 ᪠ -,|" +
                     "஥ ᪠ ⮫쪮  砥|  .||"+
                     "᫥ ⪫祭 ᪠ 室 믮 ॡ㥬 .");
   SetBCSRegime(BCSr_Disconnect);
   L_StdFrm.Inform(" ⪫祭");
  ELSE
   L_StdFrm.Question("  ६ ⪫祭 ।騬 맮 ⮩ 樨|" +
                     "த ਢ  祭 ᪠.");
   SetBCSRegime(BCSr_OnLine);
  END;
 END {DisconnectBarCodeScanner};

 PRIVATE PROCEDURE FinalAssert(Cond: SCAL[2]; Message: STRING[]);
 BEGIN
  IF ~Cond THEN
   L_StdFrm.RedInform(Message);
   RAISE(,ed_FinalError);
  END;
 END {FinalAssert};

 PUBLIC PROCEDURE InitScanner(force: SCAL[2]);
 BEGIN
  IF force | (GetBCSRegime() # BCSr_None) THEN
   TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
    IF !oScanner THEN
     L_General.Error("஢ઠ  樠஢,  . ⮣ ⥪   㢨");
    END;
   EXCEPT
   ON Exc: Exc_Root DO
    TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
     oScanner:= CreateOBJECT("OPOS.Scanner");
    EXCEPT
    ON Exc: Exc_Root DO
     FinalAssert(0, " 믮 ⠭ MetrOPOS-ࠩ ᪠");
    END; (* TRY *)
    IF oScanner.Open(BCS_ProfileName) = SUCCESS THEN
     oScanner.AutoDisable     :=-1;
     oScanner.DecodeData      :=-1;
     oScanner.DataEventEnabled:= 0;
    ELSE
     SETEMPTY(oScanner);
     FinalAssert(0, "訡  䨫 ᪠ '" + BCS_ProfileName + "'");
    END;
   END; (* TRY *)
  END;
 END {InitScanner};

 PRIVATE VAR
  ip_BarCode    : STRING[];
  ip_BarCodeType: tBarCodeType;

 PRIVATE PROCEDURE GetBarCode_byOPOS_BCS_IdleProc(): SCAL[2];
 BEGIN
  IF oScanner.DataCount > 0 THEN
   oScanner.DataEventEnabled:=-1;
   WHILE oScanner.DataEventEnabled | oScanner.DeviceEnabled DO
    (*  ࠡ⪨ ᮡ.  ந  稭 AutoDisable *)
    SLEEP(0.05);
   END;
   ip_BarCode    :=oScanner.ScanDataLabel;
   ip_BarCodeType:=oScanner.ScanDataType;
   RETURN 1;
  ELSE
   RETURN 0;
  END;
 END {GetBarCode_byOPOS_BCS_IdleProc};

 PRIVATE PROCEDURE tbc_FormatMessage(VAR Message   : STRING[];
                                     IsError: SCAL[2];
                                     VAR LineAmount: NUMERIC[0];
                                     VAR LineLength: NUMERIC[0]);
  CONST
   MaxMessageLength = 65;
   MaxLineAmount    = 40;
  VAR
   i, j  : NUMERIC[0];
   ArrStr: STRING[][MaxLineAmount];
   Rest  : STRING[];
 BEGIN
  SETZERO(LineAmount, LineLength);
  IF ~!Message THEN
   IF IsError THEN
    Str.Format(ArrStr, Str.Flatten(Message), MaxMessageLength, LEFT, Rest);
   ELSE
    Str.Format(ArrStr, Str.Replace(Message, "|", L_Str.CRLF), MaxMessageLength, LEFT, Rest);
   END;
   j:=0;
   WHILE (j < MaxLineAmount) & ~!ArrStr[j] DO
    INC(j);
   END;

   SETEMPTY(Message);
   i:=0;
   WHILE i < j DO
    LineLength:=Math.Max(LineLength, Str.Length(ArrStr[i]));
    Message   :=L_Str.AddStrDlm(Message, L_Str.CRLF) + ArrStr[i];
    INC(i);
   END;
   LineAmount:=j;
  END;
 END {tbc_FormatMessage};

 PUBLIC PROCEDURE gbc_OnClose(dlg: DIALOG);
 BEGIN
  dlg.Visible:=0;
  dlg.Redraw();
  ip_BarCode    :=EMPTYOF(ip_BarCode);
  ip_BarCodeType:=SCAN_SDT_UNKNOWN;
 END {gbc_OnClose};

 PRIVATE PROCEDURE gbc_OnSuccessfulChange_Edit(Edit: EDIT; Prev: ANYTYPE);
 BEGIN
  Edit.Parent.Parent.Visible:=0;
  Edit.Parent.Parent.Redraw();
  ip_BarCode    :=Edit.Value;
  ip_BarCodeType:=SCAN_STD_KBSCAN;
 END {gbc_OnSuccessfulChange_Edit};

 PRIVATE PROCEDURE gbc_GetControlSize(VAR Text: STRING[]; IsError: SCAL[2]; VAR W, H, H1Str: FLOAT);
  VAR
   LineAmount,
   LineLength: NUMERIC[0];
 BEGIN
  tbc_FormatMessage(Text, IsError, LineAmount, LineLength);
  IF LineLength = 0 THEN
   LineLength:=45;
  END;
  L_GUI.GetDlgTextSize(L_Str.Repeat("", LineLength), W, H1Str);
  H:=H1Str*LineAmount;
 END {gbc_GetControlSize};

 PRIVATE PROCEDURE add_StaticText(dlg     : DIALOG;
                                  Id,
                                  Text    : STRING[];
                                  Left,
                                  Top,
                                  Width,
                                  Height  : NUMERIC[0];
                                  FontSize: FLOAT;
                                  IsError : SCAL[2]);
 BEGIN
  IF ~!Text THEN
   L_GUI.add_StaticText(dlg, Id, Text, Left, Top, Width, Height, FontSize, CONTROL.HorizontalAlignment_Center, L_GUI.br_st_None);
   IF IsError THEN
    L_GUI.SetControlFgColor(dlg.Controls.Item(Id), RGBCOLOR.Create(255, 255, 255));
   END;
   dlg.Controls.Item(Id).VerticalAlignment:=CONTROL.VerticalAlignment_Top;
   dlg.Controls.Item(Id).Visible:=1;
  END;
 END {add_StaticText};

 PRIVATE PROCEDURE gbc_CreateDialog(Title, Hint, Message, asBarCode: STRING[]; IsError, wKeyboard: SCAL[2]): DIALOG;
  CONST
   id_BarCode  = "e_BarCode";
   id_Message  = "st_Message";
   id_Hint     = "st_Hint";

   sH_Shift    = 5;
   sW_Shift    = 5;
   pL_Hint     = sW_Shift;
   pT_Hint     = sH_Shift;
   pL_Message  = sW_Shift;
   pL_BarCode  = sW_Shift;

  VAR
   dlg        : DIALOG;
   HA         : NUMERIC[0];
   fS,
   H1Str,
   pT_Message,
   sH_Message,
   sW_Message,
   sH_Hint,
   sW_Hint,
   sH_BarCode,
   sW_BarCode,
   pT_BarCode,
   dlgWidth,
   dlgHeight  : FLOAT;
 BEGIN
  fS:=EMPTYOF(FLOAT);
  gbc_GetControlSize(Hint, 0, sW_Hint, sH_Hint, H1Str);
  IF ~!Hint THEN
   pT_BarCode:=pT_Hint + sH_Hint + sH_Shift;
  ELSE
   pT_BarCode:=pT_Hint;
  END;
  gbc_GetControlSize(Message, IsError, sW_Message, sH_Message, H1Str);
  IF sW_Hint > sW_Message THEN
   sW_Message:=sW_Hint;
  ELSE
   sW_Hint:=sW_Message;
  END;

  sW_BarCode:=sW_Message;
  sH_BarCode:=H1Str+10;
  pT_Message:=pT_BarCode + sH_BarCode + sH_Shift;

  dlgWidth  :=sW_BarCode + 2*sW_Shift;
  dlgHeight :=pT_Message + (sH_Message + sH_Shift)*(~!Message);

  dlg:=L_GUI.CreateNoSizebleDialog(DIALOG.WindowStyle_FixedSize, dlgWidth, dlgHeight, Title, PROCREF(gbc_OnClose));
  IF IsError THEN
   L_GUI.SetDlgBkColor(dlg, RGBCOLOR.Create(165, 0, 0));
  END;
  add_StaticText(dlg, id_Hint,    Hint,    pL_Hint,    pT_Hint,    sW_Hint,    sH_Hint,    fS, IsError);
  add_StaticText(dlg, id_Message, Message, pL_Message, pT_Message, sW_Message, sH_Message, fS, IsError);

  IF wKeyboard THEN
   HA:=CONTROL.HorizontalAlignment_Left;
  ELSE
   HA:=CONTROL.HorizontalAlignment_Center;
  END;
  L_GUI.add_Edit(dlg, id_BarCode, pL_BarCode, pT_BarCode, sW_BarCode, sH_BarCode, fS, HA, L_GUI.br_st_Solid);
  dlg.Controls.Item(id_BarCode).Value             :=asBarCode;
  dlg.Controls.Item(id_BarCode).OnSuccessfulChange:=PROCREF(gbc_OnSuccessfulChange_Edit);
  dlg.Controls.Item(id_BarCode).Visible           :=1;
  dlg.Controls.Item(id_BarCode).Enable            :=wKeyboard;

  RETURN dlg;
 END {gbc_CreateDialog};

 PUBLIC PROCEDURE GetBarCode(    Title,
                                 Hint,
                                 Message     : STRING[];
                                 IsError,
                                 wKeyboard   : SCAL[2];
                             VAR BarCode     : STRING[];
                             VAR BarCodeType : tBarCodeType);
  CONST
   BCLength = 33;
  VAR
   asBarCode: STRING[];
   idleProc : PROC;
   dlg      : DIALOG;
 BEGIN
  InitScanner(0);
  SLEEP(0.25); (*  Releas&Claim 蠥 ᪠ *)
  IF BCSRegime = BCSr_OnLine THEN
   FinalAssert(oScanner.ClaimDevice(1000) = SUCCESS,
               "訡 쭮 墠 ᪠| |" +
               ",  襬  㦥 饭 ணࠬ,|" +
               " ᯮ ᪠");
   oScanner.DeviceEnabled:=-1;
   FinalAssert(oScanner.ResultCode = SUCCESS, "訡 ⨢樨 ᪠.||" +
                                              ", ᪠ 室  ୮ ०");
   idleProc:=PROCREF(GetBarCode_byOPOS_BCS_IdleProc);
  ELSE
   FinalAssert(wKeyboard,
               "  ਫ ॡ OPOS ᪠.   ");
   SETEMPTY(idleProc);
  END;
  IF wKeyboard THEN
   SETEMPTY(asBarCode);
  ELSE
   asBarCode:="ਫ ᪠";
  END;

  TRY [TERM_BY_USER(TERM_BY_USER_THROW)]
   dlg        :=gbc_CreateDialog(Title, Hint, Message, asBarCode, IsError, wKeyboard);
   dlg.Visible:=1;
   dlg.Redraw();
   LOOP
    IF ~dlg.Visible THEN
     EXIT;
    ELSIF ~!idleProc & idleProc() THEN
     dlg.Visible:=0;
     dlg.Redraw();
     EXIT;
    END;
    dlg.Wait(0.5);
   END; (* WHILE dlg.Visible *)
  EXCEPT
  ON Exc: Exc_TermByUser DO
   ip_BarCode    :=EMPTYOF(ip_BarCode);
   ip_BarCodeType:=SCAN_SDT_UNKNOWN;
  END;

  BarCode    :=ip_BarCode;
  BarCodeType:=ip_BarCodeType;
  IF BCSRegime = BCSr_OnLine THEN
   oScanner.ReleaseDevice();
   WHILE oScanner.Claimed DO
    SLEEP(0.05);
   END;
  END;
 END {GetBarCode};

END.
