(*
%Copyright   (c) Polak Computer System
%Author      15.06.1999 14:48:18 SAN
%Purpose      ᮢ
%Type        library
%Specific    SYS
%Modified    05.06.2014 19:29:33 ?
%Environment
*)

 MODULE L_Basis [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_OwnData, Str, L_Str, FIO, Reg, L_DatTim, L_MSWStp, L_Env, L_FIO,
        Shell, L_OLE, Memo, L_StdFrm, L_Excel, L_AFM, Math, L_PCS;

 PUBLIC CONST
  BasExt = "TXT";
  APIExt = "CSV";
  XLSExt = "XLS";

  ModuleType_FRM = 0;
  ModuleType_ASL = 1;

  us_UserTemplate = 0;
  us_All          = 1;

 PRIVATE CONST
  TPLExt = "XLT";

  XLTSuffixDlm  = "#";
  XLTSuffixMask = XLTSuffixDlm + "??";

  RTR_DefName = "RT_";

 PUBLIC PERMANENT VAR
  UsingStage    : SCAL[3];
  pRTRSuffix,
  pReportName,
  pXLTFullName,
  pReportTitle  : STRING[];
  pTimeCond     : L_General.tTimeCond;
  pCanRemakeBas,
  pInBatch,
  pRunTimeReport: SCAL[2];

 PRIVATE PROCEDURE CBM_Consolidating(): NUMERIC[0];
 BEGIN
  RETURN -1;
 END {CBM_Consolidating};

 PUBLIC PROCEDURE CurrentBranch(): NUMERIC[0];
 BEGIN
  IF L_OwnData.E_BranchesAmount(SYSDATE()) > 0 THEN
   L_General.Assert(~!L_OwnData.CurBrnch, " ⠭ ⥪騩 䨫");
   IF L_OwnData.CurBrnch = L_OwnData.branchName_Consolidating() THEN
    RETURN CBM_Consolidating();
   ELSE
    RETURN L_OwnData.BranchNumber(SYSDATE(), L_OwnData.CurBrnch);
   END;
  ELSE
   RETURN 0;
  END;
 END {CurrentBranch};

 PRIVATE PROCEDURE CurrentBranchIsOwnBranch(D: DATE): SCAL[2];
 BEGIN
  IF L_OwnData.E_BranchesAmount(D) > 0 THEN
   RETURN CurrentBranch() = L_OwnData.OwnBranchNumber(D);
  ELSE
   RETURN 1;
  END;
 END {CurrentBranchIsOwnBranch};

 PUBLIC PROCEDURE ExclQM(S: STRING[]): STRING[];
 BEGIN
  RETURN Str.Replace(S, '"', "'");
 END {ExclQM};

 PUBLIC PROCEDURE wb_WrInQM(F: STRING[]; S: STRING[]);
 BEGIN
  FIO.WrStr(^F, '"' + Str.Replace(S, '"', "'") + '";');
 END {wb_WrInQM};

 PUBLIC PROCEDURE wb_WrNum(F: STRING[]; Num: FLOAT; Prc: NUMERIC[0]);
 BEGIN
  FIO.WrStr(^F, FLDTOSTR(Num, FLDTOSTR(Prc, "0", 0)+".", 0)+";");
 END {wb_WrNum};

 PUBLIC PROCEDURE wb_WrOrg(F: STRING[]; S: STRING[]);
 BEGIN
  FIO.WrStr(^F, S);
 END {wb_WrOrg};

 PUBLIC PROCEDURE wb_WrStr(F: STRING[]; S: STRING[]);
 BEGIN
  FIO.WrStr(^F, '"' + Str.Replace(S, '"', "'") + '";');
 END {wb_WrStr};

 PUBLIC PROCEDURE wb_WrDat(F: STRING[]; D: DATE);
 BEGIN
  IF ~!D THEN
   FIO.WrStr(^F, L_DatTim.DateStr(D)+";");
  ELSE
   FIO.WrStr(^F, ";");
  END;
 END {wb_WrDat};

 PUBLIC PROCEDURE wb_WrTim(F: STRING[]; T: TIME);
 BEGIN
  IF ~!T THEN
   FIO.WrStr(^F, L_DatTim.TimeStr(T)+";");
  ELSE
   FIO.WrStr(^F, ";");
  END;
 END {wb_WrTim};

 PUBLIC PROCEDURE wb_WrLongTim(F: STRING[]; T: TIME; Sec: SCAL[99]);
 BEGIN
  IF ~!T THEN
   FIO.WrStr(^F, L_DatTim.TimeStr(T) + ":" + Sec +";");
  ELSE
   FIO.WrStr(^F, ";");
  END;
 END {wb_WrLongTim};

 PUBLIC PROCEDURE wb_WrScl(F: STRING[]; S: STRING[]);
 BEGIN
  FIO.WrStr(^F, '"' + Str.Replace(S, '"', "'") + '";');
 END {wb_WrScl};

 PUBLIC PROCEDURE wb_TermLine(F: STRING[]);
 BEGIN
  FIO.Seek(^F, FIO.Tell(^F)-1);
  FIO.WrLn(^F);
 END {wb_TermLine};

 PRIVATE PROCEDURE GetDTStamp(fullPath: STRING[]; VAR d: DATE; VAR t: TIME);
  VAR
   DTS: TIMESTAMP;
 BEGIN
  DTS:=L_FIO.GetFileLastModifiedDTS(fullPath);
  d  :=L_DatTim.GetDateOfTIMESTAMP(DTS);
  t  :=L_DatTim.GetTimeOfTIMESTAMP(DTS);
 END {GetDTStamp};

 PRIVATE PROCEDURE GetHeader(xltFullName, Title: STRING[];
                             TimeCond: L_General.tTimeCond): STRING[];
 BEGIN
  RETURN "|ॡ 祭 |" + "'"+L_FIO.effp_FileName_woExt(xltFullName)+" "+Title+"'|"+
          TimeCondStr(TimeCond, Memo.ud_cd(), Memo.ud_fr(), Memo.ud_ut());
 END {GetHeader};

 PRIVATE PROCEDURE GetRTBMask(XLTFullName: STRING[]): STRING[];
 BEGIN
  RETURN L_FIO.effp_FileName_woExt(XLTFullName) + "_" + RTR_DefName;
 END {GetRTBMask};

 PRIVATE PROCEDURE GetRTBName(TimeCond: L_General.tTimeCond): STRING[];
  VAR
   tmpRepName: STRING[32];
   Idx       : NUMERIC[0];
   NameIsGood: SCAL[2];
   c         : STRING[1];
 BEGIN
  IF ~!pRTRSuffix THEN
   tmpRepName:=pRTRSuffix;
   pRTRSuffix:="";
  ELSE
   tmpRepName:="Default";
  END;
  tmpRepName:=APP.Get().MyConnection.Number + "_" + tmpRepName;
  LOOP
   STDFORM(L_StdFrm.sf_Choice(GetHeader(pXLTFullName, pReportTitle, TimeCond) +
                              "||"+
                              "  ନ஢  ᭮ ⢠  |"+
                              "  ࠭ ᮧ  .||" +
                              "᫨ ॡ ⥫쭮 ࠭ /|"+
                              "ᮢ⭮ ᯮ짮 ᪮쪨 |"+
                              "⮢,   㪠 㭨쭮  |"+
                              "%##########%%################################%||",
                              "%LNI%LI"),
           ,
           INFORM,
           -1,
           GetRTBMask(pXLTFullName),
           tmpRepName);
   Idx:=0;
   NameIsGood:=1;
   c:=L_Str.Char(tmpRepName, Idx);
   WHILE ~!c & NameIsGood DO
    NameIsGood:=(c >= "0") & (c <= "9") |
                (c >= "A") & (c <= "Z") |
                (c >= "a") & (c <= "z") |
                (c >= "") & (c <= "") |
                (c >= "") & (c <= "") |
                (c = "_");
    IF ~NameIsGood THEN
     L_StdFrm.RedInform("⨬ ᯮ짮   ᨬ '" + c + "'");
    END;
    INC(Idx);
    c:=L_Str.Char(tmpRepName, Idx);
   END;
   IF NameIsGood THEN
    EXIT;
   END;
  END;
  RETURN RTR_DefName + tmpRepName;
 END {GetRTBName};

 PRIVATE PROCEDURE BasisDate(TimeCond : L_General.tTimeCond;
                             _CurDate,
                             _From,
                             _UpTo    : DATE): DATE;
 BEGIN
  IF    pRunTimeReport THEN
   RETURN SYSDATE();
  ELSIF TimeCond = L_General.timeCond_OnDate THEN
   RETURN _CurDate;
  ELSIF TimeCond = L_General.timeCond_InIntvl THEN
   RETURN _From;
  ELSIF (TimeCond = L_General.timeCond_InMonth) |
        (TimeCond = L_General.timeCond_OnMonthBeg) THEN
   RETURN L_DatTim.MonthBeg(_CurDate);
  ELSIF (TimeCond = L_General.timeCond_OnMonthEnd) THEN
   RETURN L_DatTim.MonthEnd(_CurDate)+1;
  ELSIF  TimeCond = L_General.timeCond_Entire THEN
   RETURN SYSDATE();
  ELSE
   L_General.CallUs(0);
  END;
 END {BasisDate};

 PRIVATE PROCEDURE DefineBasisName(TimeCond : L_General.tTimeCond;
                                   _CurDate,
                                   _From,
                                   _UpTo    : DATE): STRING[];
 BEGIN
  IF    pRunTimeReport THEN
   RETURN GetRTBName(TimeCond);
  ELSIF TimeCond = L_General.timeCond_OnDate THEN
   RETURN FLDTOSTR(_CurDate, "yND", 8);
  ELSIF TimeCond = L_General.timeCond_InIntvl THEN
   IF L_DatTim.Per(_From) = L_DatTim.Per(_UpTo) THEN
    IF (_From = L_DatTim.MonthBeg(_From)) &
       (_UpTo = L_DatTim.MonthEnd(_UpTo)) THEN
     RETURN FLDTOSTR(_From, "yN", 6);
    ELSE
     RETURN FLDTOSTR(_From, "yN", 6)+"_"+
            FLDTOSTR(_From, "D",  2)+"_"+
            FLDTOSTR(_UpTo, "D",  2);
    END;
   ELSE
    IF (_From = L_DatTim.MonthBeg(_From)) &
       (_UpTo = L_DatTim.MonthEnd(_UpTo)) THEN
     RETURN FLDTOSTR(_From, "yN", 6)+"_"+
            FLDTOSTR(_UpTo, "yN", 6);
    ELSE
     RETURN FLDTOSTR(_From, "yND", 8)+"_"+
            FLDTOSTR(_UpTo, "yND", 8);
    END;
   END;
  ELSIF (TimeCond = L_General.timeCond_InMonth) |
        (TimeCond = L_General.timeCond_OnMonthBeg)|
        (TimeCond = L_General.timeCond_OnMonthEnd) THEN
   RETURN FLDTOSTR(_CurDate, "yN", 6);
  ELSIF  TimeCond = L_General.timeCond_Entire THEN
   RETURN FLDTOSTR(SYSDATE(), "yND", 8);
  ELSE
   L_General.CallUs(0);
  END;
 END {DefineBasisName};

 PUBLIC PROCEDURE IsRTBasis(FullBasisName: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Cap(Str.Slice(FullBasisName, 6, 3)) = RTR_DefName;
 END {IsRTBasis};

 PRIVATE PROCEDURE StrToDate(s, format: STRING[]): DATE;
 BEGIN
  TRY [EXCEPTIONHANDLING(EXCEPTIONHANDLING_OFF)]
   RETURN STRTOFLD(s, DATE, format);
  EXCEPT
  END;
 END {StrToDate};

 PUBLIC PROCEDURE SplitBasisName(    BasisName: STRING[];
                                 VAR TimeCond : L_General.tTimeCond;
                                 VAR _CurDate,
                                     _From,
                                     _UpTo    : DATE): SCAL[2];
 BEGIN
  (*    १ ७,  ᨫ 筮 १.
       timeCond_Entire                㤥   timeCond_OnDate
              timeCond_OnMonthBeg                              timeCond_InMonth
              timeCond_OnMonthEnd                              timeCond_InMonth
              timeCond_InIntvl                      timeCond_InMonth *)
  TRY
   IF    !BasisName THEN
    RETURN 0;
   ELSIF !Str.Item(BasisName, "_", 1) THEN
    IF Str.Length(BasisName) = 6 THEN
     TimeCond:=L_General.timeCond_InMonth;
     _CurDate:=StrToDate(BasisName, "yN");
     _From   :=L_DatTim.MonthBeg(_CurDate);
     _UpTo   :=L_DatTim.MonthEnd(_CurDate);
    ELSE
     TimeCond:=L_General.timeCond_OnDate;
     _CurDate:=StrToDate(BasisName, "yND");
     _From   :=_CurDate;
     _UpTo   :=_CurDate;
    END;
   ELSE
    TimeCond:=L_General.timeCond_InIntvl;
    IF !Str.Item(BasisName, "_", 2) THEN
     IF Str.Length(Str.Item(BasisName, "_", 0)) = 6 THEN
      _From  :=L_DatTim.MonthBeg(StrToDate(Str.Item(BasisName, "_", 0), "yN"));
      _UpTo  :=L_DatTim.MonthEnd(StrToDate(Str.Item(BasisName, "_", 1), "yN"));
     ELSE
      _From  :=StrToDate(Str.Item(BasisName, "_", 0), "yND");
      _UpTo  :=StrToDate(Str.Item(BasisName, "_", 1), "yND");
     END;
    ELSE
     _From   :=StrToDate(Str.Item(BasisName, "_", 0)+Str.Item(BasisName, "_", 1), "yND");
     _UpTo   :=StrToDate(Str.Item(BasisName, "_", 0)+Str.Item(BasisName, "_", 2), "yND");
    END;
    _CurDate:=_From;
   END;
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode = L_General.ec_rt_FailedSTRTOFLD THEN
    RETURN 0;
   ELSE
    RAISE();
   END;
  END;
  RETURN 1;
 END {SplitBasisName};

 PRIVATE PROCEDURE Make_API(apiPath : STRING[];
                            TimeCond: L_General.tTimeCond;
                            sDate   : DATE;
                            sTime   : TIME;
                            Title   : STRING[]);
  VAR
   _From,
   _UpTo    : STRING[10];
   _FirmName: STRING[];
 BEGIN
  FIO.Open(api, apiPath,  , Create);
  FIO.WrStr(api, "From;UpTo;sysdate;sysop;customer;Name");
  FIO.WrLn(api);
  IF    TimeCond = L_General.timeCond_OnDate THEN
   _From:=FLDTOSTR(Memo.ud_cd(), "D.N.y", 10);
   _UpTo:=" ";
  ELSIF TimeCond = L_General.timeCond_InIntvl THEN
   _From:=FLDTOSTR(Memo.ud_fr(), "D.N.y", 10);
   _UpTo:=FLDTOSTR(Memo.ud_ut(), "D.N.y", 10);
  ELSIF TimeCond = L_General.timeCond_InMonth THEN
   _From:=FLDTOSTR(Memo.ud_mb(), "D.N.y", 10);
   _UpTo:=FLDTOSTR(Memo.ud_me(), "D.N.y", 10);
  ELSIF TimeCond = L_General.timeCond_OnMonthBeg THEN
   _From:=FLDTOSTR(Memo.ud_mb(), "D.N.y", 10);
   _UpTo:=" ";
  ELSIF TimeCond = L_General.timeCond_OnMonthEnd THEN
   _From:=FLDTOSTR(Memo.ud_me() + 1, "D.N.y", 10);
   _UpTo:=" ";
  ELSIF TimeCond = L_General.timeCond_Entire THEN
   _From:=FLDTOSTR(SYSDATE(), "D.N.y", 10);
   _UpTo:=" ";
  END;
  _FirmName:=L_OwnData.ODFirmName(sDate);
  IF CurrentBranch() = CBM_Consolidating() THEN
   _FirmName:=L_OwnData.branchName_Consolidating() + ": " + _FirmName;
  END;
  FIO.WrStr(api,
                 '"' + _From                   +'";'+
                 '"' + _UpTo                   +'";'+
                 '"' + L_DatTim.DateStr(sDate) +" " +
                       L_DatTim.TimeStr(sTime) +'";'+
                 '"' + ExclQM(L_PCS.GetFullNameOfCurrentUser(sDate)) +'";'+
                 '"' + ExclQM(_FirmName)       +'";'+
                 '"' + ExclQM(Title)           +'"'
           );
  FIO.WrLn(api);
  FIO.SetFileTime(api, , , L_DatTim.GetTIMESTAMP(sDate, sTime));
  FIO.Close(api);
 END {Make_API};

 PRIVATE PROCEDURE MakeDir(Path: STRING[]);
 BEGIN
  IF ~FIO.DirExists(Path) THEN
   FIO.MkDir(Path);
  END;
 END {MakeDir};

 PRIVATE PROCEDURE MakeXLSbyXLT(xltFullName, xlsFullPath: STRING[250]);
  VAR
   Msg: STRING[];
 BEGIN
  L_General.CallUs(FIO.Exists(xltFullName));
  IF FIO.Exists(xlsFullPath) THEN
   LOOP
    SETEMPTY(Msg);
    TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF), MSG_SUPPRESSION(MSG_SUPPRESSION_ALL)]
     FIO.Erase(xlsFullPath);
    EXCEPT
    ON Exc: Exc_Root DO
     Msg:=Exc.MsgOriginalText;
    END; (* TRY *)
    IF ~!Msg THEN
     L_StdFrm.Choice(Msg + "|" +
                     L_StdFrm.THR(" 稭") +
                     "1   㦥   Excel 㣨 짮⥫. ஢  ࢥ  䠩.|" +
                     "2  짮⥫  ࠢ    ⠫ BASIS. ஢ NTFS-ࠧ襭.  ");
    ELSE
     EXIT;
    END;
   END; (* LOOP *)
  END;
  L_FIO.ExactFileCopy(xltFullName, xlsFullPath);
 END {MakeXLSbyXLT};

 PUBLIC PROCEDURE TimeCondStr(TimeCond: L_General.tTimeCond;
                              _CurDate,
                              _From,
                              _UpTo   : DATE): STRING[];
 BEGIN
  IF    TimeCond = L_General.timeCond_OnDate THEN
   RETURN " "+L_DatTim.DateStr(_CurDate);
  ELSIF TimeCond = L_General.timeCond_InIntvl THEN
   RETURN " ਮ  "+L_DatTim.DateStr(_From)+"  "+L_DatTim.DateStr(_UpTo);
  ELSIF TimeCond = L_General.timeCond_InMonth THEN
   RETURN " ਮ  "+L_DatTim.DateStr(L_DatTim.MonthBeg(_CurDate))+"  "+L_DatTim.DateStr(L_DatTim.MonthEnd(_CurDate));
  ELSIF TimeCond = L_General.timeCond_OnMonthBeg THEN
   RETURN " "+L_DatTim.DateStr(L_DatTim.MonthBeg(_CurDate));
  ELSIF TimeCond = L_General.timeCond_OnMonthEnd THEN
   RETURN " "+L_DatTim.DateStr(L_DatTim.MonthEnd(_CurDate) + 1);
  ELSIF TimeCond = L_General.timeCond_Entire THEN
   RETURN " "+L_DatTim.DateStr(SYSDATE());
  END;
 END {TimeCondStr};

 PRIVATE PROCEDURE GetUserChoice(xltFullName,
                                 Title      : STRING[];
                                 TimeCond   : L_General.tTimeCond;
                                 basExists,
                                 xlsExists  : SCAL[2];
                                 basFullPath: STRING[]): SCAL[4];
  VAR
   Header: STRING[];
   Res   : SCAL[4];
   Msg,
   Tit   : STRING[];
   D     : DATE;
   T     : TIME;
 BEGIN
  L_General.CallUs(VAR_IMPL.Get(pCanRemakeBas).Initialized);
  Header:=GetHeader(xltFullName, Title, TimeCond);
  IF    ~basExists THEN
   IF ~CurrentBranchIsOwnBranch(SYSDATE()) THEN
    L_General.Error(Header +
                    "|騩 䨫: " + L_OwnData.CurBrnch +
                    "|⢥騩   ");
   END;
   Res:=3;
  ELSE
   GetDTStamp(basFullPath, D, T);
   Msg:=Header +
        "||  ᮧ "+L_DatTim.DateStr(D)+"  "+L_DatTim.TimeStr(T)+"||";
   IF    xlsExists THEN
    Msg:=Msg + "%< ᯮ짮 騩   Excel   >D|";
    Msg:=Msg + "%< ᯮ짮 騩   Excel 蠡 >B|";
   ELSE
    Msg:=Msg + "%< ᯮ짮 騩   Excel 蠡 >D|";
   END;
   IF pCanRemakeBas THEN
    IF    CurrentBranchIsOwnBranch(SYSDATE()) THEN
     Msg:=Msg + "%<             ந            >B|";
    ELSIF CurrentBranch() = CBM_Consolidating() THEN
     Msg:=Msg + "%<  ନ஢ ᮫஢  >B|";
    END;
   END;
   Msg :=Msg + "%<                   ⬥                  >Z" ;
   IF L_OwnData.E_BranchesAmount(SYSDATE()) > 0 THEN
    Tit:=" " + L_OwnData.CurBrnch + " ";
   ELSE
    SETEMPTY(Tit);
   END;
   Res:=STDFORM(Msg, Tit, INFORM);
   IF ~xlsExists THEN
    INC(Res);
   END;
  END;
  RETURN Res;
 END {GetUserChoice};

 PRIVATE PROCEDURE ExcludeXLTSuffix(xltName: STRING[]): STRING[];
 BEGIN
  IF Str.Match(xltName, "*"+XLTSuffixMask+"*") THEN
   RETURN Str.Delete(xltName, Str.Pos(xltName, XLTSuffixDlm), Str.Length(XLTSuffixMask));
  ELSE
   RETURN xltName;
  END;
 END {ExcludeXLTSuffix};

 PRIVATE PROCEDURE IsCorrectXLTName(xltName, basName: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Cap(ExcludeXLTSuffix(xltName)) = Str.Cap(basName);
 END {IsCorrectXLTName};

 PRIVATE PROCEDURE badOfficeVersion_PurgeXLT(VAR Workbook: OBJECT);
  VAR
   DummyFileName: STRING[];
 BEGIN
  IF ~!Workbook THEN
   Workbook.SaveAs(FIO.FullPath(FIO.MakePath(L_DatTim.SysDateTimeStamp(), , FIO.AsbSpecialFolders("ASBTemp"))));
   DummyFileName:=Workbook.FullName;
   Workbook.Close();
   SETEMPTY(Workbook);
   FIO.Erase(DummyFileName);
  END;
 END {badOfficeVersion_PurgeXLT};

 PRIVATE PROCEDURE IsAllowedDir(xltDir, rqDir: STRING[]): SCAL[2];
  VAR
   Result: SCAL[2];
   rqFolders,
   xltFolders: STRING[][20];
   rqFoldersAmount,
   xltFoldersAmount: NUMERIC[0];
 BEGIN
  L_FIO.SplitFolders(Str.Cap(rqDir),  rqFolders,  rqFoldersAmount);
  L_FIO.SplitFolders(Str.Cap(xltDir), xltFolders, xltFoldersAmount);

  IF rqFoldersAmount <= xltFoldersAmount THEN
   Result:=1;
   WHILE rqFoldersAmount >= 0 DO
    Result:=Result & (xltFolders[rqFoldersAmount] = rqFolders[rqFoldersAmount]);
    DEC(rqFoldersAmount);
   END; (* WHILE rqFoldersAmount <= 0...*)
  ELSE
   Result:=0;
  END;
  RETURN Result;
 END {IsAllowedDir};

 PRIVATE PROCEDURE GetSystemXLTFolder(srcFolder: STRING[]): STRING[];
  VAR
   rootFolder: STRING[];
 BEGIN
  IF APP.Get().ProductiveCopy THEN
   rootFolder:="$_LIB";
  ELSE
   rootFolder:="AUTOPARK.LIB";
  END;
  RETURN FIO.BuildPath(srcFolder, , rootFolder);
 END {GetSystemXLTFolder};

 PRIVATE PROCEDURE MakeXLTList(XLTFolder: FOLDERENTRY; basName: STRING[]);
  VAR
   XLTFiles: ENUMERATOR;
   XLTFile : OBJECT;
   Folder,
   xltName : STRING[];
 BEGIN
  XLTFiles :=CreateENUMERATOR(XLTFolder.FolderContent);
  WHILE ~XLTFiles.AtEnd() DO
   XLTFile:=XLTFiles.Item();
   xltName:=L_FIO.effp_FileName_woExt(XLTFile.Path);
   PROMPT("  " + xltName);
   IF IsCorrectXLTName(xltName, basName) THEN
    XLTLs.FileName:=xltName;
    XLTLs.FileMDT :=L_DatTim.DTStampStr(XLTFile.DateLastModified);
    XLTLs.FullFN  :=XLTFile.Path;
    Folder:=L_FIO.effp_Folder(XLTFile.Path);
    IF    IsAllowedDir(Folder, "XLT") THEN
     XLTLs.Location := 0;
    ELSIF IsAllowedDir(Folder, "XLT\LOCAL") THEN
     XLTLs.Location := 1;
    ELSIF IsAllowedDir(Folder, "XLT\TMP_LIB") THEN
     XLTLs.Location := 2;
    ELSIF IsAllowedDir(Folder, GetSystemXLTFolder("XLT\AP")) |
          IsAllowedDir(Folder, GetSystemXLTFolder("XLT\SYS")) THEN
     XLTLs.Location := 3;
    ELSE
     XLTLs.Location := 4;
    END;
    INSERT(XLTLs);
   END;
   XLTFiles.MoveNext();
  END; (* WHILE ~XLTFiles.AtEnd()... *)
 END {MakeXLTList};

 PRIVATE PROCEDURE ViewAllXLTTemplates(): SCAL[2];
 BEGIN
  RETURN (UsingStage = us_All) | L_General.IsOurEngineeringCopy();
 END {ViewAllXLTTemplates};

 PRIVATE PROCEDURE FillSingleDialogLine(basName: STRING[]; VAR XLTs, XLTFileList: STRING[]; wXLTComment: SCAL[2]; VAR Excel, Workbook: OBJECT; DialogWidth: NUMERIC[0]);
  CONST
   sl_XLTName     = 8;
   sl_XLTDateTime = 10+1+5;
   sl_XLTPath     = 7;

   XLTPath_Pool = " |LOCAL|TMP_LIB|$_LIB|???";
  VAR
   Comment,
   SingleVal,
   XLTLocation  : STRING[];
   doSaveAs     : SCAL[2];
   sl_XLTComment: NUMERIC[0];
 BEGIN
  sl_XLTComment:=DialogWidth-sl_XLTName-1-sl_XLTDateTime-1;
  IF ViewAllXLTTemplates() THEN
   sl_XLTComment:=sl_XLTComment-sl_XLTPath-1;
  END;
  SingleVal    :=Str.Align(XLTLs.FileName, sl_XLTName, LEFT);
  SETEMPTY(Comment);
  IF wXLTComment THEN
   PROMPT("㧪 ᠭ 蠡 " + XLTLs.FileName);

   IF !Excel THEN
    TRY
     Excel:=CreateOBJECT("Excel.Application");
    EXCEPT
    ON Exc: Exc_Root DO
     L_General.Error(" ⠭ ਫ MS Excel");
    END; (* TRY *)
    Excel.Visible           :=L_Excel.xlFalse;
    Excel.EnableEvents      :=L_Excel.xlFalse;
    Excel.ScreenUpdating    :=L_Excel.xlFalse;
   END;
   TRY
    doSaveAs:=0;
    Excel.AutomationSecurity:=3(*msoAutomationSecurityForceDisable*);
   EXCEPT
   ON Exc: Exc_Root DO
    doSaveAs:=1;
   END;
   TRY
    Workbook:=Excel.Workbooks.Open(XLTLs.FullFN, 0(*UpdateLinks*), (*L_Excel.xlTrue(*Readonly*)*), , , , , , , L_Excel.xlTrue(*Editable*));
    Comment:="(" + Workbook.Names("RepComment").RefersToRange.Text + ")";
   EXCEPT
   ON Exc: Exc_Root DO
    SETEMPTY(Comment);
   END;
   IF doSaveAs THEN
    badOfficeVersion_PurgeXLT(Workbook);
   ELSE
    L_Excel.CloseWorkbook(Workbook);
   END;
  END;

  IF    L_Str.IsEqStrI(basName, XLTLs.FileName) THEN
   SingleVal:=SingleVal + " " + Str.Align("᭮ 蠡 " + Comment, sl_XLTComment, LEFT);
  ELSIF ~!Comment THEN
   SingleVal:=SingleVal + " " + Str.Align(Comment, sl_XLTComment, LEFT);
  END;

  SETEMPTY(XLTLocation);

  IF    L_General.IsOurEngineeringCopy() THEN
   XLTLocation:=" " + Str.Align(L_Str.PoolItem(XLTPath_Pool, XLTLs.Location), sl_XLTPath, LEFT);
  ELSIF XLTLs.Location = 4 THEN
   XLTLocation:=" " + "???????";
  ELSE
   XLTLocation:=" " + L_Str.PoolItem("ࠧ|       ", XLTLs.Location = 0);
  END;

  XLTs       :=XLTs + Str.Align(SingleVal, sl_XLTName + 1 + sl_XLTComment, LEFT) + " " +
                      XLTLs.FileMDT                                                    +
                      XLTLocation                                                      + "|";

  XLTFileList:=XLTFileList + XLTLs.FullFN + "|";
 END {FillSingleDialogLine};

 PRIVATE PROCEDURE ChoiceXLT(basName, Title: STRING[]): STRING[];
  VAR
   XLTs,
   XLTFileList   : STRING[];
   Excel,
   Workbook      : OBJECT;
   Choice,
   ChooseXLT,
   DialogWidth,
   DialogLineAmount,
   XLTAmount,
   curWidth,
   curHeight,
   TotalXLTAmount: NUMERIC[0];
   Result        : STRING[];
   oPrompt       : PROMPTLINE;
 BEGIN
  oPrompt:=PROMPTLINE.Create("롮 蠡");
  L_AFM.MakeIndex("XLTLs",
                     "Location: SCAL[9];" +
                     "FileName: STRING[8];" +
                     "FileMDT : STRING[16];" +
                     "FullFN  : STRING[128]");

  MakeXLTList(FOLDERENTRY.CreateRedirector(basName+"."+TPLExt),               basName);
  MakeXLTList(FOLDERENTRY.CreateRedirector(basName+XLTSuffixMask+"."+TPLExt), basName);

  IF pInBatch THEN
   SELECT(XLTLs, XLTLs);
   Result:=XLTLs.FullFN;
  ELSE
   IF L_General.IsOurSite() THEN
    DialogWidth:=87;
   ELSE
    DialogWidth:=74;
   END;
 
   IF ~VAR_IMPL.Get(UsingStage).Initialized THEN
    UsingStage:=us_UserTemplate;
   END;
 
   SETEMPTY(Excel, Workbook);
   SETZERO(Choice);
   LOOP
    SETEMPTY(XLTs, XLTFileList);
    SETZERO(XLTAmount, TotalXLTAmount);
    SELECT(XLTLs, XLTLs);
    WHILE OCCURE(XLTLs) DO
     PROMPT(XLTLs.FileName);
     INC(TotalXLTAmount);
     IF ViewAllXLTTemplates() |
        ((UsingStage = us_UserTemplate) & (XLTLs.Location = 0)) THEN
      FillSingleDialogLine(basName, XLTs, XLTFileList, Choice = 2, Excel, Workbook, DialogWidth);
      INC(XLTAmount);
     END;
     NEXT(XLTLs);
    END; (* WHILE OCCURE(XLTLs) ... *)
    IF ~!Excel THEN
     L_Excel.CloseExcel(Excel, Workbook);
    END;
 
    L_General.Assert(TotalXLTAmount > 0,
                     "  蠡  - " + basName + "."+TPLExt +
                     "|,   뢠  , ⮬,"+
                     "|,   ࠡ稪");
 
    ChooseXLT:=0;
    IF (UsingStage = us_UserTemplate) & (XLTAmount <= 1) THEN
     EXIT;
    END;
 
    GE_APPWIN_SIZE(curWidth, curHeight);
    DialogLineAmount:=Math.Min(curHeight - 8, Math.Max(XLTAmount, 2));
 
    Choice:=STDFORM("|" + ": " + "%" + L_Str.IntStr(Str.Length(Title)) + "NI" + L_Str.Repeat(" ", DialogWidth - 7 - Str.Length(Title) - 1) + "||" +
                    "%L" + L_Str.IntStr(DialogWidth) + "," + L_Str.IntStr(DialogLineAmount) + "<" + XLTs + ">I" + L_Str.Repeat("|", DialogLineAmount) +
                    "|%<  믮  >DB   %< 㧨 ᠭ >B   %<   室   >X",
                    " 롮 蠡  ⮢  ",
                    ,
                    1,
                    Title, ChooseXLT);
    IF Choice = 1 THEN
     EXIT;
    END;
   END; (* LOOP *)
 
   IF XLTAmount = 0 THEN
    SELECT(XLTLs, XLTLs);
    Result:=XLTLs.FullFN;
   ELSE
    Result:=Str.Item(L_Str.PoolItem(XLTFileList, ChooseXLT), " ", 0);
   END;
  END; (* IF pInBatch *)
  RETURN Result;
 END {ChoiceXLT};

 PUBLIC PROCEDURE ChoiceRTBXLT(basName, Title: STRING[]): STRING[];
 BEGIN
  pInBatch      :=0;
  pRunTimeReport:=1;
  RETURN ChoiceXLT(basName, Title);
 END {ChoiceRTBXLT};

 PUBLIC PROCEDURE GetBasisPath(_Branch: NUMERIC[0]; D: DATE): STRING[];
  VAR
   BasisPath,
   Result    : STRING[];
 BEGIN
  BasisPath:=FIO.AsbSpecialFolders("BasisRoot");
  IF ~!BasisPath THEN
   L_General.Assert(FIO.DirExists(BasisPath), "   " + BasisPath);
   Result:=BasisPath;
  ELSIF _Branch = 0 THEN
   Result:=FIO.FullPath("BASIS");
  ELSIF _Branch = CBM_Consolidating() THEN
   Result:=FIO.FullPath("BASIS\BASISC");
  ELSIF L_OwnData.OwnBranchNumber(D) # 0 THEN
   Result:=FIO.FullPath("BASIS");
  ELSE
   Result:=FIO.FullPath("BASIS\BASIS" + FLDTOSTR(_Branch, "0", 0));
  END;
  RETURN Result;
 END {GetBasisPath};

 PRIVATE PROCEDURE GetBasisPath_woFileName(_Branch  : NUMERIC[0];
                                           repName  : STRING[128];
                                           TimeCond : L_General.tTimeCond;
                                           _CurDate,
                                           _From,
                                           _UpTo    : DATE): STRING[];
  VAR
   P: STRING[250];
 BEGIN
  P:=GetBasisPath(_Branch, BasisDate(TimeCond, _CurDate, _From, _UpTo));
  MakeDir(P);
  P:=FIO.MakePath(ExcludeXLTSuffix(repName), , P);
  MakeDir(P);
  RETURN P;
 END {GetBasisPath_woFileName};

 PUBLIC PROCEDURE GetBasisPath_wMask(_Branch  : NUMERIC[0];
                                     repName  : STRING[128];
                                     XLTPath  : STRING[];
                                     TimeCond : L_General.tTimeCond;
                                     _CurDate,
                                     _From,
                                     _UpTo    : DATE): STRING[];
 BEGIN
  RETURN FIO.MakePath(GetRTBMask(XLTPath),
                      ,
                      GetBasisPath_woFileName(_Branch, repName, TimeCond, _CurDate, _From, _UpTo));
 END {GetBasisPath_wMask};

 PRIVATE PROCEDURE _GetFullPath(_Branch  : NUMERIC[0];
                                repName  : STRING[128];
                                TimeCond : L_General.tTimeCond;
                                _CurDate,
                                _From,
                                _UpTo    : DATE): STRING[];
 BEGIN
  RETURN FIO.MakePath(repName + "_" + DefineBasisName(TimeCond, _CurDate, _From, _UpTo),
                      ,
                      GetBasisPath_woFileName(_Branch, repName, TimeCond, _CurDate, _From, _UpTo));
 END {_GetFullPath};

 PUBLIC PROCEDURE GetFullPath(_Branch      : NUMERIC[0];
                              repName      : STRING[128];
                              TimeCond     : L_General.tTimeCond;
                              _CurDate,
                              _From,
                              _UpTo        : DATE;
                              RunTimeReport: SCAL[2]): STRING[];
 BEGIN
  pRunTimeReport:=RunTimeReport;
  RETURN _GetFullPath(_Branch, repName, TimeCond, _CurDate, _From, _UpTo);
 END {GetFullPath};

 PRIVATE PROCEDURE GetFullPathByMemo(_Branch : NUMERIC[0];
                                     repName : STRING[128];
                                     TimeCond: L_General.tTimeCond): STRING[];
 BEGIN
  RETURN _GetFullPath(_Branch, repName, TimeCond, Memo.ud_cd(), Memo.ud_fr(), Memo.ud_ut());
 END {GetFullPathByMemo};

 PRIVATE PROCEDURE Consolidate(repName : STRING[];
                               TimeCond: L_General.tTimeCond;
                               Title   : STRING[]);
  VAR
   i,
   n,
   k          : NUMERIC[0];
   S,
   FileName,
   BranchName : STRING[];
   minDate, d : DATE;
   minTime, t : TIME;
 BEGIN
  FIO.Open(Dst, GetFullPathByMemo(CBM_Consolidating(), repName, TimeCond), BasExt, Create);
  i:=0;
  n:=L_OwnData.E_BranchesAmount(SYSDATE());
  minDate:=L_DatTim.MAXDATE(); minTime:=23:59;
  WHILE i <= n DO
   BranchName:=Str.Item(L_Env.BranchesNamesPool(), "|", i);
   FileName  :=FIO.BuildPath(GetFullPathByMemo(i, repName, TimeCond), BasExt);
   L_General.Assert(FIO.Exists(FileName),
                    "᮫:     䨫 " + BranchName);
   GetDTStamp(FileName, d, t);
   IF    d < minDate THEN
    minDate:=d; minTime:=t;
   ELSIF (d = minDate) & (t < minTime) THEN
    minTime:=t;
   END;
   FIO.Open(Src, FileName, , Input);
   L_General.Assert(FIO.RdStr(Src, S), "   ப  : " + BranchName);
   IF i = 0 THEN
    FIO.WrStr(Dst, "" + ";");
    FIO.WrStr(Dst,              S);
    FIO.WrLn (Dst);
   END;
   k:=0;
   WHILE FIO.RdStr(Src, S) DO INC(k);
    PROMPT("᮫ " + BranchName + ":" + L_Str.IntStr(k));
    FIO.WrStr(Dst, BranchName + ";");
    FIO.WrStr(Dst,                S);
    FIO.WrLn (Dst);
   END;
   FIO.Close(Src);
   INC(i);
  END;
  FIO.SetFileTime(Dst, , , L_DatTim.GetTIMESTAMP(minDate, minTime));
  FIO.Close(Dst);
  Make_API(GetFullPathByMemo(CBM_Consolidating(), repName, TimeCond)+"."+APIExt,
           TimeCond, minDate, minTime, Title);
 END {Consolidate};

 PUBLIC PROCEDURE PlaceInBasis(RepIsDone,
                               BatchMode   : SCAL[2];
                               basName,
                               xltFullName: STRING[128];
                               TimeCond   : L_General.tTimeCond;
                               Title      : STRING[128]);
  VAR
   SrcName     : STRING[80];
   SrcFullPath,
   DstFullPath,
   basFullPath,
   apiFullPath,
   xlsFullPath : STRING[250];
   sDate       : DATE;
   sTime       : TIME;
   usrCh       : SCAL[100];
   basExists   : SCAL[2];
   DoExcel     : SCAL[3];
 BEGIN
  L_General.Assert(~L_General.IsLocalMode(), "ᯮ짮 ᢮ ⠡ MS Excel|" +
                                             " ਠ  ⠫樨  ");
  L_General.CallUs(IsCorrectXLTName(L_FIO.effp_FileName_woExt(xltFullName), basName));
  DstFullPath:=GetFullPathByMemo(CurrentBranch(), L_FIO.effp_FileName_woExt(xltFullName), TimeCond);
  basFullPath:=ExcludeXLTSuffix(DstFullPath)+"."+BasExt;
  apiFullPath:=ExcludeXLTSuffix(DstFullPath)+"."+APIExt;
  xlsFullPath:=DstFullPath                  +"."+XLSExt;
  DoExcel:=1;
  IF RepIsDone THEN
   SrcName:=L_General.NameConnection(basName);
   L_General.Assert(FIO.Exists(SrcName, , SrcFullPath), "  १ "+ SrcName);
   FIO.RenMov(SrcFullPath, , basFullPath, );
   GetDTStamp(basFullPath, sDate, sTime);
   Make_API  (apiFullPath, TimeCond, sDate, sTime, Title);
   IF ~BatchMode THEN
    MakeXLSbyXLT(xltFullName, xlsFullPath);
   END;
  ELSIF ~BatchMode THEN
   basExists:=FIO.Exists(basFullPath) & FIO.Exists(apiFullPath);
   IF (CurrentBranch() = CBM_Consolidating()) & ~basExists THEN
    Consolidate(basName, TimeCond, Title);
    MakeXLSbyXLT(xltFullName, xlsFullPath);
   ELSE
    usrCh:=GetUserChoice(xltFullName,
                         Title,
                         TimeCond,
                         basExists,
                         FIO.Exists(xlsFullPath), basFullPath);
    IF    usrCh = 1 (* existing xls *) THEN
     (* nothing *)
    ELSIF usrCh = 2 (* existing txt *) THEN
     MakeXLSbyXLT(xltFullName, xlsFullPath);
    ELSIF usrCh = 3 (* to do new txt *) THEN
     IF CurrentBranch() = CBM_Consolidating() THEN
      Consolidate(basName, TimeCond, Title);
      MakeXLSbyXLT(xltFullName, xlsFullPath);
     ELSE
      DoExcel:=0;
     END;
    ELSE
     L_General.Error("Wrong user choice - " + usrCh);
    END;
   END;
  END;
  IF BatchMode THEN
   DoExcel:=0;
  END;
  IF DoExcel THEN
   SETEMPTY(pReportName);
   L_MSWStp.AddExcel12TrustedLocations();
   IF L_General.GetOSPlatform() = L_General.osp_WinNT THEN
    TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
     Shell.Execute("open", xlsFullPath);
    EXCEPT
    END; (* TRY *)
    HALT(0);
   ELSE
    LIKEMENU(ExecAsb, "Start " + xlsFullPath);
   END;
  ELSIF RepIsDone THEN
   SETEMPTY(pReportName);
  END;
 END {PlaceInBasis};

 PUBLIC PROCEDURE PlayBasis();
 BEGIN
  L_General.CallUs(VAR_IMPL.Get(pReportName).Initialized & ~!pReportName);
  PlaceInBasis(1, pInBatch, pReportName, pXLTFullName, pTimeCond, pReportTitle);
 END {PlayBasis};

 PRIVATE PROCEDURE Do_AnyRep(moduleName: STRING[];
                             moduleType: SCAL[2];
                             Mode      : SCAL[254];
                             ModuleMode: SCAL[254];
                             reportName: STRING[];
                             TimeCond  : L_General.tTimeCond;
                             Title     : STRING[];
                             wOLE,
                             RunTimeRep: SCAL[2]);
 BEGIN
  L_General.Assert(~L_General.IsLocalMode(), "ᯮ짮 ᢮ ⠡ MS Excel|" +
                                             " ਠ  ⠫樨  ");

  IF RunTimeRep THEN
   IF    Mode < 100 THEN
    pInBatch:=0;
   ELSE
    L_General.Error("    ᯮ짮|" +
                    " ⭮ ᪥");
   END;
  ELSE
   IF    Mode < 100 THEN
    pInBatch:=0;
   ELSIF Mode < 200 THEN
    pInBatch:=1;
   ELSE
    L_General.Error("Error in L_Basis. Wrong using");
   END;
  END;
  IF wOLE THEN
   L_OLE.CheckRegistry();
  END;
  pCanRemakeBas :=1;
  pRunTimeReport:=RunTimeRep;
  pReportName   :=reportName;
  pTimeCond     :=TimeCond;
  pReportTitle  :=Title;
  pXLTFullName  :=ChoiceXLT(pReportName, pReportTitle);
  IF ~pRunTimeReport THEN
   PlaceInBasis(0, pInBatch, pReportName, pXLTFullName, pTimeCond, pReportTitle);
  ELSE
   SETEMPTY(pRTRSuffix);
  END;
  IF moduleType = ModuleType_FRM THEN
   LIKEMENU( Report, moduleName + " " + ModuleMode, RunProg, "BAS_DSP");
  ELSE
   LIKEMENU(RunProg, moduleName + " " + ModuleMode, RunProg, "BAS_DSP");
  END;
 END {Do_AnyRep};

 PUBLIC PROCEDURE ExtPlayMakedRunTimeRep(reportName,
                                         reportSuffix: STRING[];
                                         TimeCond    : L_General.tTimeCond;
                                         Title       : STRING[];
                                         wOLE        : SCAL[2]);
 BEGIN
  pCanRemakeBas :=1;
  pRTRSuffix    :=reportSuffix;
  pInBatch      :=0;
  pRunTimeReport:=1;
  pReportName   :=reportName;
  pTimeCond     :=TimeCond;
  pReportTitle  :=Title;
  pXLTFullName  :=ChoiceXLT(pReportName, pReportTitle);
  IF wOLE THEN
   L_OLE.CheckRegistry();
  END;
  PlayBasis();
 END {ExtPlayMakedRunTimeRep};

 PRIVATE PROCEDURE extTryExisting(ReportName: STRING[];
                                  TimeCond  : L_General.tTimeCond;
                                  Title     : STRING[];
                                  wOLE      : SCAL[2]);
 BEGIN
  L_General.CallUs(VAR_IMPL.Get(pCanRemakeBas).Initialized);
  IF wOLE THEN
   L_OLE.CheckRegistry();
  END;
  pInBatch      :=L_General.BatchExecution_IsRunning();
  pRunTimeReport:=0;
  pReportName   :=ReportName;
  pTimeCond     :=TimeCond;
  pReportTitle  :=Title;
  pXLTFullName  :=ChoiceXLT(pReportName, pReportTitle);
  IF ~pRunTimeReport THEN
   PlaceInBasis(0, pInBatch, pReportName, pXLTFullName, pTimeCond, pReportTitle);
  ELSE
   SETEMPTY(pRTRSuffix);
  END;
 END {extTryExisting};

 PUBLIC PROCEDURE TryExisting(ReportName, Title: STRING[]; TimeCond: L_General.tTimeCond);
 BEGIN
  pCanRemakeBas:=1;
  extTryExisting(ReportName, TimeCond, Title, 0);
 END {TryExisting};

 PUBLIC PROCEDURE TryExisting_woRemakeBasis(ReportName, Title: STRING[]; TimeCond: L_General.tTimeCond);
 BEGIN
  pCanRemakeBas:=0;
  extTryExisting(ReportName, TimeCond, Title, 0);
 END {TryExisting_woRemakeBasis};

 PUBLIC PROCEDURE TryExistingRunTimeRep_woRemakeBasis(XLTFullName,
                                                      XLSFullPath,
                                                      BasFullPath,
                                                      Title        : STRING[];
                                                      TimeCond     : L_General.tTimeCond);
  VAR
   usrCh: SCAL[100];
 BEGIN
  pCanRemakeBas:=0;
  usrCh:=GetUserChoice(XLTFullName,
                       Title,
                       TimeCond,
                       1,
                       FIO.Exists(XLSFullPath), BasFullPath);
  IF    usrCh = 1 (* existing xls *) THEN
   (* nothing *)
  ELSIF usrCh = 2 (* existing txt *) THEN
   MakeXLSbyXLT(XLTFullName, XLSFullPath);
  ELSE
   L_General.Error("Wrong user choice - " + usrCh);
  END;
  Shell.Execute("open", XLSFullPath);
 END {TryExistingRunTimeRep_woRemakeBasis};

 PUBLIC PROCEDURE PlayMakedRunTimeRep(reportName: STRING[];
                                      TimeCond  : L_General.tTimeCond;
                                      Title     : STRING[];
                                      wOLE      : SCAL[2]);
 BEGIN
  ExtPlayMakedRunTimeRep(reportName,
                         "",
                         TimeCond,
                         Title,
                         wOLE);
 END {PlayMakedRunTimeRep};

 PUBLIC PROCEDURE Do_wAnyRepName(moduleName: STRING[];
                                 moduleType: SCAL[2];
                                 Mode      : SCAL[254];
                                 ModuleMode: SCAL[254];
                                 reportName: STRING[];
                                 TimeCond  : L_General.tTimeCond;
                                 Title     : STRING[]);
 BEGIN
  Do_AnyRep(moduleName, moduleType, Mode, ModuleMode, reportName,
            TimeCond, Title, 0, 0);
 END {Do_wAnyRepName};

 PUBLIC PROCEDURE Do_RunTime(moduleName: STRING[];
                             moduleType: SCAL[2];
                             Mode      : SCAL[254];
                             ModuleMode: SCAL[254];
                             reportName: STRING[];
                             TimeCond  : L_General.tTimeCond;
                             Title     : STRING[];
                             wOLE      : SCAL[2]);
 BEGIN
  Do_AnyRep(moduleName, moduleType, Mode, ModuleMode, reportName,
            TimeCond, Title, wOLE,  1);
 END {Do_RunTime};

 PUBLIC PROCEDURE Do_Advanced(moduleName: STRING[];
                              moduleType: SCAL[2];
                              Mode      : SCAL[254];
                              ModuleMode: SCAL[254];
                              reportName: STRING[];
                              TimeCond  : L_General.tTimeCond;
                              Title     : STRING[];
                              wOLE      : SCAL[2]);
 BEGIN
  Do_AnyRep(moduleName, moduleType, Mode, ModuleMode, reportName,
            TimeCond, Title, wOLE, 0);
 END {Do_Advanced};

 PUBLIC PROCEDURE MakeBasis(moduleName: STRING[];
                            moduleType: SCAL[2];
                            Mode      : SCAL[254];
                            ModuleMode: SCAL[254];
                            TimeCond  : L_General.tTimeCond;
                            Title     : STRING[]);
 BEGIN
  Do_wAnyRepName(moduleName, moduleType, Mode,
                 ModuleMode, moduleName, TimeCond, Title);
 END {MakeBasis};

 PUBLIC PROCEDURE MakeLocalBasis(moduleName: STRING[128];
                                 moduleType: SCAL[2];
                                 Mode      : SCAL[254];
                                 ModuleMode: SCAL[254];
                                 TimeCond  : L_General.tTimeCond;
                                 Title     : STRING[128]);
 BEGIN
  MakeBasis(moduleName, moduleType, Mode, ModuleMode, TimeCond, Title);
 END {MakeLocalBasis};

END.
