(*
%Copyright   (c) Polak Computer System
%Author
%Purpose     樨  CARDS
%Type        library to paste
%Specific    SYS
%Modified    24.03.2014 10:19:03 POLAK\LDA
%Environment
*)

 MODULE L_Card [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PRIVATE CONST
  CardsNum    = 24;
  VersionsNum = 32;

 PUBLIC TYPE
  tType  = SCAL[CardsNum];
  tClass = SCAL[2];
  tName  = STRING[9];
  tNick  = STRING[];

 PUBLIC CONST
  RussianPool = " |"                   +
                "⥢ |"       +
                "  ⠡|"    +
                "  ।|"  +
                " 㬥|"  +
                "⥦ 㬥|" +
                "ਪ  ᮭ|"+
                " | | | | | |"         +
                "஢  |"   +
                "஢  |"    +
                " 㤥ঠ|"    +
                "⠢  ";

 PUBLIC CONST
  t_NoM = 00;
  t_RWS = 01;
  t_SHT = 02;
  t_AVP = 03;
  t_MDC = 04;
  t_PDc = 05;
  t_EDO = 06;
  t_WJM = 07;
  t_AVT = 08;
  t_MDT = 09;
  t_RWT = 10;
  t_EDT = 11;
  t_MCM = 12;
  t_MXO = 13;
  t_SHP = 14;
  t_DdF = 15;
  t_TXS = 16;

  c_LOC = 0;
  c_SYS = 1;

 PRIVATE CONST
  NicksPool = "---|RWS|SHT|AVP|MDC|PDoc|EDO|WJM|AVT|MDT|RWT|EDT|MCM|MXO|SHP|DedF|TXS";

 IMPORT Memo, L_General, L_DatTim, L_Str, L_Env, FIO, E_Card, L_APtm;

 PRIVATE PERMANENT VAR
  CardsNotches,
  CardsDates   : DATE[CardsNum * VersionsNum];

 PRIVATE PROCEDURE GetSysType(LocType: tType): tType;
 BEGIN
  IF    (LocType = t_RWS) THEN RETURN t_RWT;
  ELSIF (LocType = t_SHT) THEN RETURN t_SHP;
  ELSIF (LocType = t_AVP) THEN RETURN t_AVT;
  ELSIF (LocType = t_MDC) THEN RETURN t_MDT;
  ELSIF (LocType = t_EDO) THEN RETURN t_EDT;
  ELSE                         RETURN t_NoM;
  END;
 END {GetSysType};

 PUBLIC PROCEDURE CardNick(_Card: tType): tNick;
 BEGIN
  RETURN L_Str.PoolItem(NicksPool, _Card);
 END {CardNick};

 PRIVATE PROCEDURE GetCardFirstIndex(_Card: tType): NUMERIC[0];
 BEGIN
  RETURN _Card * VersionsNum;
 END {GetCardFirstIndex};

 PRIVATE PROCEDURE GetCardLastIndex(_Card: tType): NUMERIC[0];
 BEGIN
  RETURN (_Card + 1) * VersionsNum - 1;
 END {GetCardLastIndex};

 PUBLIC PROCEDURE IsStandartCardType(_Card: tType): SCAL[2];
 BEGIN
  RETURN (_Card = t_RWT) |
         (_Card = t_AVT) |
         (_Card = t_MDT) |
         (_Card = t_EDT) |
         (_Card = t_SHP);
 END {IsStandartCardType};

 PUBLIC PROCEDURE _SetCard(_Card: tType; From, CardDate: STRING[]);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=GetCardFirstIndex(_Card);
  WHILE ~!CardsNotches[i] DO
   INC(i);
  END;
  L_General.Assert(i <= GetCardLastIndex(_Card),
                   "誮  ᨩ " + CardNick(_Card));
  CardsNotches[i]:=STRTOFLD(    From, DATE, "y.N");
  CardsDates  [i]:=STRTOFLD(CardDate, DATE, "y.N");
  L_General.Assert((i = GetCardFirstIndex(_Card)) |
                   (CardsNotches[i - 1] >= CardsNotches[i]),
                   " 冷 ᫥ " + CardNick(_Card));
 END {_SetCard};

 PUBLIC PROCEDURE _ReSetCard(_Card: tType; From, CardDate: STRING[]);
  VAR
   i,
   last : NUMERIC[0];
   _From: DATE;
 BEGIN
  L_General.Assert(IsStandartCardType(_Card),
                   CardNick(_Card) + "   ⠭ ᬥ 㫥");
  _From:=STRTOFLD(From, DATE, "y.N");
  last :=GetCardLastIndex(_Card);
  i    :=GetCardFirstIndex(_Card);
  WHILE ~!CardsNotches[i] & (CardsNotches[i] > _From) DO
   INC(i);
  END;
  IF CardsNotches[i] # _From THEN
   L_General.Assert(i < last, "誮  ᨩ " + CardNick(_Card));
   WHILE i < last DO
    CardsNotches[last]:=CardsNotches[last - 1];
    CardsDates  [last]:=CardsDates  [last - 1];
    DEC(last);
   END;
  END;
  CardsNotches[i]:=STRTOFLD(    From, DATE, "y.N");
  CardsDates  [i]:=STRTOFLD(CardDate, DATE, "y.N");
 END {_ReSetCard};

 PUBLIC PROCEDURE SetCard(_Card: tType; From: STRING[]);
 BEGIN
  _SetCard(_Card, From, From);
 END {SetCard};

 PUBLIC PROCEDURE ReSetCard(_Card: tType; From: STRING[]);
 BEGIN
  _ReSetCard(_Card, From, From);
 END {ReSetCard};

 PRIVATE PROCEDURE _GetCardName(_Card: tType; Period: DATE; Strong: SCAL[2]): tName;
  VAR
   i,
   last    : NUMERIC[0];
   FullName: STRING[];
 BEGIN
  IF _Card = t_WJM THEN
   WHILE Period >= L_Env.aptm_Start DO
    FullName:=CardNick(_Card) + L_DatTim.PerStr(Period);
    IF FIO.Exists(FullName, "ASL") THEN
     RETURN FullName;
    END;
    Period:=L_DatTim.IncMonth(Period, -1);
   END;
   L_General.Error(" ந  ⮤   " +
                   L_DatTim.PeriodStr(Period) + "|  ");
  ELSE
   i   :=GetCardFirstIndex(_Card);
   last:=GetCardLastIndex(_Card);
   WHILE (i <= last) & (CardsNotches[i] > Period) DO
    INC(i);
   END;
   IF (_Card = t_PDc) | (_Card = t_MCM) | (_Card = t_MXO) | (_Card = t_DdF) THEN
    FullName:=CardNick(_Card);
   ELSE
    FullName:=CardNick(_Card) + L_DatTim.PerStr(CardsDates[i]);
   END;
   IF (i > last) THEN
    IF Strong THEN
     L_General.Error(" 㪠 ᬥ  " + CardNick(_Card) +
                     "  " + L_DatTim.PeriodStr(Period));
    ELSE
     FullName:="";
    END;
   END;
  END;
  RETURN FullName;
 END {_GetCardName};

 PUBLIC PROCEDURE GetCardNameSoft(_Card: tType; Period: DATE): tName;
 BEGIN
  RETURN _GetCardName(_Card, Period, 0);
 END {GetCardNameSoft};

 PUBLIC PROCEDURE GetCardName(_Card: tType; Period: DATE): tName;
 BEGIN
  RETURN _GetCardName(_Card, Period, 1);
 END {GetCardName};

 PUBLIC PROCEDURE GetTEDParamsForEditRule(): STRING[];
  VAR
   CardType : tType;
   CardName : tName;
   Params   : STRING[];
 BEGIN
  Params:="";
  CardType:=Rule.CardType;
  CardName:=GetCardNameSoft(CardType, Memo.tmp_mb());
  IF ~!CardName THEN
   Params:=Params + CardName + ".ASL ";
  END;
  CardType:=GetSysType(CardType);
  IF    (CardType = t_RWT) | (CardType = t_SHP) THEN
   Params:=Params + GetCardNameSoft(CardType, Memo.tmp_mb()) + ".ASL ";
  ELSIF CardType # t_NoM THEN
   Params:=GetCardName(CardType, Memo.tmp_mb()) + ".ASL " + Params;
  END;
  L_General.Assert(~!Params, " ᠭ   ᬥ ,|" +
                             " ஬    ࠢ " + Rule.Rule);
  RETURN Params + " /S:PROCEDURE\c" + Rule.Rule + "(";
 END {GetTEDParamsForEditRule};

 PUBLIC PROCEDURE Init();
 BEGIN
  SETEMPTY(CardsNotches, CardsDates);

  SetCard(t_MDT, "2002.01");
  SetCard(t_MDT, "1996.04");

  SetCard(t_AVT, "2010.01");
  SetCard(t_AVT, "2000.05");

  SetCard(t_SHP, "2000.05");

  SetCard(t_PDc, "1998.01");

  SetCard(t_EDT, "1998.01");

  SetCard(t_MCM, "1991.01");

  SetCard(t_MXO, "2001.01");

  SetCard(t_DdF, "2001.01");

  IF L_APtm.IsMGT() THEN
   SetCard(t_RWT, "2005.01");
   SetCard(t_RWT, "2001.10");
   SetCard(t_RWT, "2000.10");
   SetCard(t_RWT, "2000.05");
   SetCard(t_RWT, "1999.11");
   SetCard(t_RWT, "1999.07");
   SetCard(t_RWT, "1999.04");
   SetCard(t_RWT, "1999.02");
   SetCard(t_RWT, "1998.01");
  ELSE
   SetCard(t_RWT, "1993.04");
  END;

  SetCard(t_TXS, "2014.01");
  SetCard(t_TXS, "2013.01");
  SetCard(t_TXS, "2011.01");
  SetCard(t_TXS, "2010.01");
  SetCard(t_TXS, "2009.01");
  SetCard(t_TXS, "2004.04");

  E_Card.E_Init();

 END {Init};

BEGIN
 IF ~L_General.VariableIsInitialized(CardsNotches[0]) THEN
  Init();
 END;
END.
