(*
%Copyright   (c) Polak Computer System
%Author
%Purpose       ⠬  ६
%Type        library
%Specific    sys
%Modified    12.09.2014 19:19:01 POLAK\GAV
*)

 MODULE L_DatTim [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tWeekday = SCAL[8];
  tMonth   = SCAL[12];

 PUBLIC CONST
  wd_Mon = 0;
  wd_Tue = 1;
  wd_Wed = 2;
  wd_Thu = 3;
  wd_Fri = 4;
  wd_Sat = 5;
  wd_Sun = 6;
  wd_Hol = 7;

  WDStr_0 = "쭨|୨|।|⢥|⭨|㡡|ᥭ|ࠧ";
  WDStr_3 = "쭨|୨|।|⢥|⭨|㡡|ᥭ|ࠧ";
  WDStr_Pool_Short = "|||||||";

  wd_DaysAmount = 7;

  AveDaysInMonth = 30.4375;

  _Month_0 = "ﭢ|䥢ࠫ||५|||||ᥭ|||";
  _Month_1 = "ﭢ|䥢ࠫ||५|||||ᥭ|||";

 IMPORT L_General, L_Str, Math, L_Hld;

 PUBLIC PROCEDURE DAY(d: DATE): NUMERIC[0];
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  UnPkDATE(d, D, M, Y); RETURN(D);
 END {DAY};

 PUBLIC PROCEDURE MONTH(d: DATE): tMonth;
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  UnPkDATE(d, D, M, Y); RETURN(M);
 END {MONTH};

 PUBLIC PROCEDURE QUARTAL(D: DATE): NUMERIC[0];
 BEGIN
  RETURN MONTH(D) DIV 3;
 END {QUARTAL};

 PUBLIC PROCEDURE YEAR(d: DATE): NUMERIC[0];
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  UnPkDATE(d, D, M, Y); RETURN(Y);
 END {YEAR};

 PUBLIC PROCEDURE WEEKDAY(d: DATE): tWeekday;
  VAR w: NUMERIC[0];
 BEGIN
  w:=(d-04\01\1993) MOD 7;
  IF w < 0 THEN
   RETURN w+7;
  ELSE
   RETURN w;
  END;
 END {WEEKDAY};

 PUBLIC PROCEDURE DATEINT(B, E, B1, E1: DATE): NUMERIC[0];
 BEGIN
  B:=Math.Max(B, B1);
  IF ~!E1 | ~!E THEN
   E:=Math.Min(E, E1);
  ELSE
   E:=MAXOF(DATE);
  END;
  IF E >= B THEN
   RETURN E-B+1;
  ELSE
   RETURN 0;
  END;
 END {DATEINT};

 PUBLIC PROCEDURE IncMonth(d: DATE; i: NUMERIC[0]): DATE;
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  UnPkDATE(d, D, M, Y);
  RETURN PackDATE(01, Math.Mod(M+i, 12), Y + Math.Div(M+i, 12));
 END {IncMonth};

 PUBLIC PROCEDURE IncCalendarMonth(Date: DATE; i: NUMERIC[0]): DATE;
  VAR
   NextD  : DATE;
   D, M, Y: NUMERIC[0];
 BEGIN
  NextD:=IncMonth(Date, i);
  UnPkDATE(NextD, D, M, Y);
  RETURN PackDATE(Math.Min(DAY(MonthEnd(NextD)), DAY(Date)), M, Y);
 END {IncCalendarMonth};

 PUBLIC PROCEDURE IncYear(d: DATE; i: NUMERIC[0]): DATE;
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  UnPkDATE(d, D, M, Y);
  RETURN PackDATE(01, M, Y + i);
 END {IncYear};

 PUBLIC PROCEDURE MonthBeg(d: DATE): DATE;
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  UnPkDATE(d, D, M, Y); RETURN PackDATE(1, M, Y);
 END {MonthBeg};

 PUBLIC PROCEDURE MonthEnd(d: DATE): DATE;
 BEGIN
  RETURN IncMonth(d, +1)-1;
 END {MonthEnd};

 PUBLIC PROCEDURE YearBeg(d: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, 0, YEAR(d));
 END {YearBeg};

 PUBLIC PROCEDURE YearEnd(D: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, 0, YEAR(D)+1)-1;
 END {YearEnd};

 PUBLIC PROCEDURE DecadeBeg(D: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, 0, (YEAR(D) DIV 10)*10);
 END {DecadeBeg};

 PUBLIC PROCEDURE DecadeEnd(D: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, 0, (YEAR(D) DIV 10 + 1)*10) - 1;
 END {DecadeEnd};

 PUBLIC PROCEDURE CenturyBeg(D: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, 0, (YEAR(D) DIV 100)*100);
 END {CenturyBeg};

 PUBLIC PROCEDURE CenturyEnd(D: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, 0, (YEAR(D) DIV 100 + 1)*100) - 1;
 END {CenturyEnd};

 PUBLIC PROCEDURE Per(d: DATE): DATE;
 BEGIN
  RETURN MonthBeg(d);
 END {Per};

 PUBLIC PROCEDURE NextPer(d: DATE): DATE;
 BEGIN
  RETURN IncMonth(d, +1);
 END {NextPer};

 PUBLIC PROCEDURE PrevPer(d: DATE): DATE;
 BEGIN
  RETURN IncMonth(d, -1);
 END {PrevPer};

 PUBLIC PROCEDURE lastMonthWD(D: DATE; _WD: tWeekday): DATE;
 BEGIN
  D:=MonthEnd(D);
  WHILE WEEKDAY(D) # _WD DO
   DEC(D);
  END;
  RETURN D;
 END {lastMonthWD};

 PUBLIC PROCEDURE DaylightSavingTimeSwitchDate(D: DATE; On: SCAL[2]): DATE;
  CONST
   lastSwitchDate = 27\03\2011;
   veryLastSwitchDate = 26\10\2014;
  VAR
   m: tMonth;
 BEGIN
  IF    D >= veryLastSwitchDate THEN
   IF On THEN RETURN MAXOF(DATE);
   ELSE       RETURN veryLastSwitchDate
   END;
  ELSIF D >= lastSwitchDate THEN
   IF On THEN RETURN lastSwitchDate;
   ELSE       RETURN veryLastSwitchDate;
   END;
  ELSE
   IF On THEN
    m:=02;(*   *)
   ELSE
    m:=09;(**)
   END;
   RETURN lastMonthWD(PackDATE(1, m, YEAR(D)), wd_Sun);
  END;
 END {DaylightSavingTimeSwitchDate};

 PUBLIC PROCEDURE MonthLen(D: DATE): NUMERIC[0];
 BEGIN
  RETURN DAY(MonthEnd(D));
 END {MonthLen};

 PUBLIC PROCEDURE YearLen(D: DATE): NUMERIC[0];
 BEGIN
  RETURN YearEnd(D)-YearBeg(D)+1;
 END {YearLen};

 PUBLIC PROCEDURE IsStarted(D, BegD: DATE): SCAL[2];
 BEGIN
  RETURN BegD <= D;
 END {IsStarted};

 PUBLIC PROCEDURE IsExpired(D: DATE; EndD: DATE): SCAL[2];
 BEGIN
  RETURN D >= EndD;
 END {IsExpired};

 PUBLIC PROCEDURE IsExpired_Inc(D: DATE; EndD: DATE): SCAL[2];
 BEGIN
  RETURN D > EndD;
 END {IsExpired_Inc};

 PUBLIC PROCEDURE IsActive(D, BegD, EndD: DATE): SCAL[2];
 BEGIN
  RETURN (BegD <= D) & (D < EndD);
 END {IsActive};

 PUBLIC PROCEDURE IsActive_Inc(D, BegD, EndD: DATE): SCAL[2];
 BEGIN
  RETURN (BegD <= D) & (D <= EndD);
 END {IsActive_Inc};

 PUBLIC PROCEDURE DTSIsActive_Inc(DTS, BegDTS, EndDTS: TIMESTAMP): SCAL[2];
 BEGIN
  RETURN (BegDTS <= DTS) & (DTS <= EndDTS);
 END {DTSIsActive_Inc};

 PUBLIC PROCEDURE IsActiveInIntvl_Inc(IntvlB, IntvlE, BegD, EndD: DATE): SCAL[2];
 BEGIN
  RETURN DATEINT(IntvlB, IntvlE, BegD, EndD) > 0;
 END {IsActiveInIntvl_Inc};

 PUBLIC PROCEDURE till2ED(_Till: DATE): DATE;
 BEGIN
  IF ~!_Till THEN
   RETURN _Till-1;
  ELSE
   RETURN _Till
  END;
 END {till2ED};

 PUBLIC PROCEDURE IsActiveInIntvl(IntvlB, IntvlE, BegD, EndD: DATE): SCAL[2];
 BEGIN
  RETURN IsActiveInIntvl_Inc(IntvlB, till2ED(IntvlE), BegD, till2ED(EndD));
 END {IsActiveInIntvl};

 (* ५, ᯮ짮 MINOF() *)
 (*!*) PUBLIC PROCEDURE MINDATE(): DATE;
 BEGIN
  RETURN 01\01\1900;
 END {MINDATE};

 (* ५, ᯮ짮 MAXOF() *)
 (*!*) PUBLIC PROCEDURE MAXDATE(): DATE;
 BEGIN
  RETURN 01\12\9999;
 END {MAXDATE};

 (*!*) PUBLIC PROCEDURE MinDATE(d1, d2: DATE): DATE;
 BEGIN
  IF d1 < d2 THEN
   RETURN d1;
  ELSE
   RETURN d2;
  END;
 END {MinDATE};

 (*!*) PUBLIC PROCEDURE MaxDATE(d1, d2: DATE): DATE;
 BEGIN
  IF d1 > d2 THEN
   RETURN d1;
  ELSE
   RETURN d2;
  END;
 END {MaxDATE};

 PUBLIC PROCEDURE InPatIntvl(PatB, PatE, B1, E1: DATE): SCAL[2];
 BEGIN
  RETURN (PatB<=B1) & (PatE >= E1);
 END {InPatIntvl};

 PUBLIC PROCEDURE GetDatesDifferenceIn3Part(BD, ED: DATE; VAR D, M, Y: NUMERIC[0]);
 BEGIN
  INC(ED); (* ??? *)
  SETZERO(D, M, Y);
  IF ED > BD THEN
   IF (DAY(BD) = MonthLen(BD)) & (DAY(ED) = MonthLen(ED)) THEN
    D:=0;
   ELSE
    D:=DAY(ED)-DAY(BD);
    IF D < 0 THEN
     M:=-1; INC(D, MonthLen(BD));
    END;
   END;
   INC(M, MONTH(ED)-MONTH(BD));
   IF M < 0 THEN
    Y:=-1; INC(M, 12);
   END;
   INC(Y, YEAR(ED)-YEAR(BD));
   Y:=Math.Max(Y, 0);
  END;
 END {GetDatesDifferenceIn3Part};

 PUBLIC PROCEDURE DaysToMonths(Days: NUMERIC[0]): NUMERIC[6];
 BEGIN
  RETURN Days / AveDaysInMonth;
 END {DaysToMonths};

 PUBLIC PROCEDURE MonthsInIntvl(IntvlB, IntvlE: DATE): NUMERIC[6];
  VAR D, M, Y: NUMERIC[0];
 BEGIN
  GetDatesDifferenceIn3Part(IntvlB, IntvlE, D, M, Y);
  RETURN (Y*12) + M + DaysToMonths(D);
 END {MonthsInIntvl};

 PUBLIC PROCEDURE CompleteMonthsInIntvl(IntvlB, IntvlE: DATE): NUMERIC[0];
 BEGIN
  RETURN MonthsInIntvl(IntvlB, IntvlE) DIV 1;
 END {CompleteMonthsInIntvl};

 PUBLIC PROCEDURE YearsInIntvl(IntvlB, IntvlE: DATE): NUMERIC[6];
 BEGIN
  RETURN MonthsInIntvl(IntvlB, IntvlE) / 12;
 END {YearsInIntvl};

 PUBLIC PROCEDURE CompleteYearsInIntvl(IntvlB, IntvlE: DATE): NUMERIC[0];
 BEGIN
  RETURN MonthsInIntvl(IntvlB, IntvlE) DIV 12;
 END {CompleteYearsInIntvl};

 PUBLIC PROCEDURE HasBirthdayInPer(BirthDate, From, UpTo: DATE): SCAL[2];
 BEGIN
  RETURN CompleteYearsInIntvl(BirthDate, From - 2) # CompleteYearsInIntvl(BirthDate, UpTo - 1);
 END {HasBirthdayInPer};

 PUBLIC PROCEDURE IsBirthDay(BirthDate, CurrentDate: DATE): SCAL[2];
 BEGIN
  RETURN HasBirthdayInPer(BirthDate, CurrentDate, CurrentDate);
 END {IsBirthDay};

 PUBLIC PROCEDURE NormalizeDMY(VAR DaysOfStage,
                               MonthsOfStage,
                               YearsOfStage   : NUMERIC[0]);
 BEGIN
  INC(MonthsOfStage,   DaysOfStage DIV AveDaysInMonth);
  DaysOfStage:=DaysOfStage MOD AveDaysInMonth;
  INC( YearsOfStage, MonthsOfStage DIV 12);
  MonthsOfStage:=MonthsOfStage MOD 12;
 END {NormalizeDMY};

 PUBLIC PROCEDURE CheckWMIntvl(IntvlB, IntvlE: DATE);
 BEGIN
  L_General.Assert((DAY(IntvlB) = 1) & (DAY(IntvlE+1) = 1),
                   "ਮ   楫 ᫮ 楢");
 END {CheckWMIntvl};

 PUBLIC PROCEDURE IsWorkDay(D: DATE): SCAL[2];
 BEGIN
  RETURN L_Hld.WeekDay_TransfersConsidered(D) <= wd_Fri;
 END {IsWorkDay};

 PUBLIC PROCEDURE WorkDaysInIntvl_Inc(From, UpTo: DATE): NUMERIC[0];
  VAR
   D       : DATE;
   workdays: NUMERIC[0];
 BEGIN
  D:=From;
  workdays:=0;
  WHILE D <= UpTo DO
   IF IsWorkDay(D) THEN
    INC(workdays);
   END;
   INC(D);
  END;
  RETURN workdays;
 END {WorkDaysInIntvl_Inc};

 PUBLIC PROCEDURE IncWorkDay(D: DATE; CntDays: NUMERIC[0]): DATE;
  VAR
   Sign  : NUMERIC[0];
   Result: DATE;
 BEGIN
  IF CntDays < 0 THEN Sign:=-1;
  ELSE                Sign:=+1;
  END;
  Result:=D;
  WHILE CntDays # 0 DO
   INC(Result,  Sign);
   DEC(CntDays, Sign * IsWorkDay(Result));
  END;
  RETURN Result;
 END {IncWorkDay};

(*-ப ८ࠧ-------------------------------------------------*)

 PUBLIC PROCEDURE DateStr(d: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(d, "D.N.y", 10);
 END {DateStr};

 PUBLIC PROCEDURE PeriodStr(p: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(p, "N.y", 7);
 END {PeriodStr};

 PUBLIC PROCEDURE PerStr(P: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(P, "_YN", 5);
 END {PerStr};

 PUBLIC PROCEDURE TimeStr(t: LONGTIME[2]): STRING[];
 BEGIN
  RETURN FLDTOSTR(t, "H:M", 5);
 END {TimeStr};

 PUBLIC PROCEDURE LongTimeStr(t: L_General.tLongTime): STRING[];
 BEGIN
  RETURN FLDTOSTR(t, "RH:M");
 END {LongTimeStr};

 PUBLIC PROCEDURE DateTimeStr(Date: DATE; Time: TIME): STRING[];
 BEGIN
  RETURN DateStr(Date) + " " + TimeStr(Time);
 END {DateTimeStr};

 PUBLIC PROCEDURE MonthAmountStr(d: DATE): STRING[];
 BEGIN
  IF MONTH(d)+1 < 5 THEN
   RETURN FLDTOSTR(d, "N  y ", 0);
  ELSE
   RETURN FLDTOSTR(d, "N 楢 y ", 0);
  END;
 END {MonthAmountStr};

 PUBLIC PROCEDURE MonthStr(D: DATE; Case: SCAL[6]): STRING[];
 BEGIN
  IF    Case = 0 THEN
   RETURN L_Str.PoolItem(_Month_0, MONTH(D));
  ELSIF Case = 1 THEN
   RETURN L_Str.PoolItem(_Month_1, MONTH(D));
  ELSE
   RETURN "";
  END;
 END {MonthStr};

 PUBLIC PROCEDURE FullDateStr(D: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(D, "D "+MonthStr(D, 1)+" y ", 0);
 END {FullDateStr};

 PUBLIC PROCEDURE DateByWordStr(D: DATE): STRING[];
  VAR
   day,
   year  : NUMERIC[0];
   result: STRING[];
 BEGIN
  day :=DAY(D);
  IF    day =  1 THEN result:="ࢮ";
  ELSIF day =  2 THEN result:="஥";
  ELSIF day =  3 THEN result:="";
  ELSIF day =  4 THEN result:="⢥⮥";
  ELSIF day =  5 THEN result:="⮥";
  ELSIF day =  6 THEN result:="⮥";
  ELSIF day =  7 THEN result:="쬮";
  ELSIF day =  8 THEN result:="쬮";
  ELSIF day =  9 THEN result:="⮥";
  ELSIF day = 10 THEN result:="⮥";
  ELSIF day = 11 THEN result:="⮥";
  ELSIF day = 12 THEN result:="⮥";
  ELSIF day = 13 THEN result:="ਭ⮥";
  ELSIF day = 14 THEN result:="ୠ⮥";
  ELSIF day = 15 THEN result:="⭠⮥";
  ELSIF day = 16 THEN result:="⭠⮥";
  ELSIF day = 17 THEN result:="⮥";
  ELSIF day = 18 THEN result:="ᥬ⮥";
  ELSIF day = 19 THEN result:="⭠⮥";
  ELSIF day = 20 THEN result:="⮥";
  ELSIF day = 21 THEN result:=" ࢮ";
  ELSIF day = 22 THEN result:=" ஥";
  ELSIF day = 23 THEN result:=" ";
  ELSIF day = 24 THEN result:=" ⢥⮥";
  ELSIF day = 25 THEN result:=" ⮥";
  ELSIF day = 26 THEN result:=" ⮥";
  ELSIF day = 27 THEN result:=" ᥤ쬮";
  ELSIF day = 28 THEN result:=" 쬮";
  ELSIF day = 29 THEN result:=" ⮥";
  ELSIF day = 30 THEN result:="ਤ⮥ ";
  ELSIF day = 31 THEN result:="ਤ ࢮ";
  END;
  result:=result + " " + MonthStr(D, 1);
  year:=YEAR(D);
  L_General.CallUs((year > 2000) & (year < 2100));
  result:=result + "  ";
  IF    year MOD 10 = 0 THEN
   IF    year DIV 10 = 1 THEN result:=result + " ⮣";
   ELSIF year DIV 10 = 2 THEN result:=result + " ⮣";
   ELSIF year DIV 10 = 3 THEN result:=result + " ਤ⮣";
   ELSIF year DIV 10 = 4 THEN result:=result + " ப";
   ELSIF year DIV 10 = 5 THEN result:=result + " ⨤⮣";
   ELSIF year DIV 10 = 6 THEN result:=result + " ⨤⮣";
   ELSIF year DIV 10 = 7 THEN result:=result + " ᥬ⮣";
   ELSIF year DIV 10 = 8 THEN result:=result + " 쬨⮣";
   ELSIF year DIV 10 = 9 THEN result:=result + " ﭮ⮣";
   END;
  ELSIF year MOD 100 DIV 10 = 1 THEN
   IF    year MOD 10 = 1 THEN result:=result + " ⮣";
   ELSIF year MOD 10 = 2 THEN result:=result + " ⮣";
   ELSIF year MOD 10 = 3 THEN result:=result + " ਭ⮣";
   ELSIF year MOD 10 = 4 THEN result:=result + " ୠ⮣";
   ELSIF year MOD 10 = 5 THEN result:=result + " ⭠⮣";
   ELSIF year MOD 10 = 6 THEN result:=result + " ⭠⮣";
   ELSIF year MOD 10 = 7 THEN result:=result + " ᥬ⮣";
   ELSIF year MOD 10 = 8 THEN result:=result + " ᥬ⮣";
   ELSIF year MOD 10 = 9 THEN result:=result + " ⭠⮣";
   END;
  ELSE
   IF    year MOD 100 DIV 10 = 2 THEN result:=result + " ";
   ELSIF year MOD 100 DIV 10 = 3 THEN result:=result + " ਤ";
   ELSIF year MOD 100 DIV 10 = 4 THEN result:=result + " ப";
   ELSIF year MOD 100 DIV 10 = 5 THEN result:=result + " 줥";
   ELSIF year MOD 100 DIV 10 = 6 THEN result:=result + " 줥";
   ELSIF year MOD 100 DIV 10 = 7 THEN result:=result + " ᥬ줥";
   ELSIF year MOD 100 DIV 10 = 8 THEN result:=result + " ᥬ줥";
   ELSIF year MOD 100 DIV 10 = 9 THEN result:=result + " ﭮ";
   END;
   IF    year MOD 10 = 1 THEN result:=result + " ࢮ";
   ELSIF year MOD 10 = 2 THEN result:=result + " ண";
   ELSIF year MOD 10 = 3 THEN result:=result + " 쥣";
   ELSIF year MOD 10 = 4 THEN result:=result + " ⢥⮣";
   ELSIF year MOD 10 = 5 THEN result:=result + " ⮣";
   ELSIF year MOD 10 = 6 THEN result:=result + " ⮣";
   ELSIF year MOD 10 = 7 THEN result:=result + " ᥤ쬮";
   ELSIF year MOD 10 = 8 THEN result:=result + " 쬮";
   ELSIF year MOD 10 = 9 THEN result:=result + " ⮣";
   END;
  END;
  RETURN result + " ";
 END {DateByWordStr};

 PUBLIC PROCEDURE LongSystimeStr(): STRING[];
  VAR
   s: FLOAT;
 BEGIN
  RETURN FLDTOSTR(SYSTIME(s), "HM")+FLDTOSTR(s DIV 1, "L", 2);
 END {LongSystimeStr};

 PUBLIC PROCEDURE SysDateTimeStamp(): STRING[];
 BEGIN
  RETURN FLDTOSTR(SYSDATE(), "yND") + LongSystimeStr();
 END {SysDateTimeStamp};

 PUBLIC PROCEDURE FullSystimeStr(): STRING[];
  VAR
   s: FLOAT;
 BEGIN
  RETURN FLDTOSTR(SYSTIME(s), "H:M:")+FLDTOSTR(s DIV 1, "L", 2);
 END {FullSystimeStr};

 PUBLIC PROCEDURE CompleteSystimeStr(): STRING[];
 BEGIN
  RETURN FLDTOSTR(SYSDATE(), "D.N.y") + " " + FullSystimeStr();
 END {CompleteSystimeStr};

 PUBLIC PROCEDURE WeekDayStr(w: tWeekday; c: SCAL[6]): STRING[];
 BEGIN
  IF    c = 0 THEN
   RETURN L_Str.PoolItem(WDStr_0, w);
  ELSIF c = 3 THEN
   RETURN L_Str.PoolItem(WDStr_3, w);
  ELSE
   RETURN "";
  END;
 END {WeekDayStr};

 PUBLIC PROCEDURE WeekDayShortStr(w: tWeekday): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(WDStr_Pool_Short, w);
 END {WeekDayShortStr};

 PUBLIC PROCEDURE FullQuartStr(D: DATE; Case: SCAL[8]): STRING[];
 BEGIN
  IF    Case = 3 THEN
   RETURN FLDTOSTR(D, FLDTOSTR(QUARTAL(D)+1, "0", 0)+" ⠫ y ", 0);
  ELSIF Case = 7 THEN
   RETURN FLDTOSTR(D, FLDTOSTR(QUARTAL(D)+1, "0", 0)+" ⠫ y ", 0);
  ELSE
   RETURN "";
  END;
 END {FullQuartStr};

 PUBLIC PROCEDURE DTStampStr(DTS: FLOAT): STRING[];
 BEGIN
  RETURN  DateTimeStr(GetDateOfVARIANTDATE(DTS), GetTimeOfVARIANTDATE(DTS));
 END {DTStampStr};

 PUBLIC PROCEDURE IsLastMonthOfQuartal(D: DATE): SCAL[2];
 BEGIN
  RETURN (MONTH(D) MOD 3 = 2);
 END {IsLastMonthOfQuartal};

 PUBLIC PROCEDURE GetFirstDateOfQuartal(D: DATE): DATE;
 BEGIN
  RETURN PackDATE(1, QUARTAL(D)*3, YEAR(D));
 END {GetFirstDateOfQuartal};

 PUBLIC PROCEDURE GetLastDateOfQuartal(D: DATE): DATE;
 BEGIN
  RETURN IncMonth(GetFirstDateOfQuartal(D), +3)-1;
 END {GetLastDateOfQuartal};

 PUBLIC PROCEDURE GetVARIANTDATE(Date: DATE; Time: LTIME; Sec: FLOAT): FLOAT;
  VAR
   Result: FLOAT;
 BEGIN
  PackVARIANTDATE(Result, Date, Time, Sec);
  RETURN Result;
 END {GetVARIANTDATE};

 PUBLIC PROCEDURE GetVARIANTDATEOfDate(Date: DATE): FLOAT;
 BEGIN
  RETURN GetVARIANTDATE(Date, 00:00, 0);
 END {GetVARIANTDATEOfDate};

 PUBLIC PROCEDURE GetSysVARIANTDATE(): FLOAT;
  VAR
   vtDate: FLOAT;
   SysSec: FLOAT;
 BEGIN
  IF L_General.IsAcceptableProcessorVersion(014076013) THEN
   VARIANTSYSDATE(vtDate);
   RETURN vtDate;
  ELSE
   RETURN GetVARIANTDATE(SYSDATE(), SYSTIME(SysSec), SysSec);
  END;
 END {GetSysVARIANTDATE};

 PUBLIC PROCEDURE GetTIMESTAMP(D: DATE; T: LTIME): TIMESTAMP;
 BEGIN
  RETURN PackTIMESTAMP(D, T);
 END {GetTIMESTAMP};

 PUBLIC PROCEDURE GetTIMESTAMPOfDate(Date: DATE): TIMESTAMP;
 BEGIN
  RETURN GetTIMESTAMP(Date, 00:00);
 END {GetTIMESTAMPOfDate};

 PUBLIC PROCEDURE GetSysTIMESTAMP(): TIMESTAMP;
 BEGIN
  RETURN SYSTIMESTAMP();
 END {GetSysTIMESTAMP};

 PUBLIC PROCEDURE GetVARIANTDATEPeriod_Hour(BIntvl, EIntvl: FLOAT): NUMERIC[3];
 BEGIN
  RETURN (EIntvl - BIntvl) * 24;
 END {GetVARIANTDATEPeriod_Hour};

 PUBLIC PROCEDURE GetDateOfVARIANTDATE(d: FLOAT): DATE;
  VAR
   Result: DATE;
 BEGIN
  UnPkVARIANTDATE(d, Result);
  RETURN Result;
 END {GetDateOfVARIANTDATE};

 PUBLIC PROCEDURE GetTimeOfVARIANTDATE(d: FLOAT): TIME;
  VAR
   Result: TIME;
 BEGIN
  UnPkVARIANTDATE(d, , Result);
  RETURN Result;
 END {GetTimeOfVARIANTDATE};

 PUBLIC PROCEDURE TimestampStr(_DTS: TIMESTAMP): STRING[];
 BEGIN
  RETURN FLDTOSTR(_DTS, "D.N.y H:M");
 END {TimestampStr};

 PUBLIC PROCEDURE GetDateOfTIMESTAMP(d: TIMESTAMP): DATE;
  VAR
   Result: DATE;
 BEGIN
  UnPkTIMESTAMP(d, Result);
  RETURN Result;
 END {GetDateOfTIMESTAMP};

 PUBLIC PROCEDURE GetTimeOfTIMESTAMP(d: TIMESTAMP): TIME;
  VAR
   Result: TIME;
 BEGIN
  UnPkTIMESTAMP(d, , Result);
  RETURN Result;
 END {GetTimeOfTIMESTAMP};

 PRIVATE PROCEDURE inGMT(Zone: NUMERIC[2]; VAR D: DATE; VAR T: TIME);
  VAR
   Result: FLOAT;
 BEGIN
  PackVARIANTDATE(Result, D, T);
  DEC(Result, Zone / 24);
  UnPkVARIANTDATE(Result, D, T);
 END {inGMT};

 PRIVATE PROCEDURE InDST(D: DATE): SCAL[2];
 BEGIN
  RETURN IsActive_Inc(D, DaylightSavingTimeSwitchDate(D, 1), DaylightSavingTimeSwitchDate(D, 0));
 END {InDST};

 PUBLIC PROCEDURE inGMT_Moscow(VAR D: DATE; VAR T: TIME);
 BEGIN
  IF InDST(D) THEN
   inGMT(+4, D, T);
  ELSE
   inGMT(+3, D, T);
  END;
 END {inGMT_Moscow};

 PUBLIC PROCEDURE TS2FL(ts: TIMESTAMP): FLOAT;
 BEGIN
  RETURN ts-PackTIMESTAMP(30\12\1899, 00:00);
 END {TS2FL};

 PUBLIC PROCEDURE FL2TS(fl: FLOAT): TIMESTAMP;
 BEGIN
  IF ~!fl THEN
   RETURN PackTIMESTAMP(30\12\1899, 00:00) + fl;
  ELSE
   RETURN EMPTYOF(TIMESTAMP);
  END;
 END {FL2TS};

END.
