(*
%Copyright   (c) Polak IT
%Author      01.12.2003 13:25:25 POLAK\ATS
%Purpose
%Type        library
%Specific    SYS
%Modified    06.07.2012 16:45:51 POLAK\San
%Environment
*)

 MODULE L_Dspt [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_XDoc, L_Str, Str, L_General, L_DatTim, DBD, L_ODoc, P_Firm, L_StdFrm, L_WDO, L_PDoc, L_lsStore;

 PRIVATE CONST
  DocNeed = [L_XDoc.xdt_PDoc,
             L_XDoc.xdt_MDoc,
             L_XDoc.xdt_Contract,
             L_XDoc.xdt_ODoc,
             L_XDoc.xdt_RWayS,
             L_XDoc.xdt_Packet];
 PRIVATE TYPE
  tDsptDocType = SCAL[HIGH(DocNeed) + 1];

 PUBLIC TYPE
  tDirection = SCAL[3];
  tWeight    = NUMERIC[2];

 PUBLIC CONST
  t_Incoming = 0; (* Dspt.Type *)
  t_Outgoing = 1;

  wdn_Incoming = "DsptI";
  wdn_Outgoing = "DsptO";

  pck_Not_Allowed = 0; (* DsptMode.PackProp *)
  pck_Allowed     = 1;
  pck_Must        = 2;

  d_Own    = 0; (* DsptDoc.Own *)
  d_Alien  = 1;

  inst_Original  = 0; (* DsptDoc.Instance *)
  inst_Copy      = 1;
  inst_Duplicate = 2;

  rp_Send                = 0;
  rp_OwnCS_Receive       = 1;
  rp_OwnCS_Process       = 2;
  rp_AlnCS_Receive       = 3;
  rp_Delivery            = 4;
  rp_AlnCSProcess        = 5;
  rp_OwnCS_AdviceReceive = 6;
  rp_AdviceReceive       = 7;
  rp_DelayNotice         = 8;

  rp_Amount = 9;

  dt_RefPointPool = "ࠢ" +
                    "|祭  ࠢ⥫" +
                    "|ࠢ  ࠢ⥫" +
                    "|祭  " +
                    "|祭" +
                    "|ࠢ   " +
                    "|祭   ࠢ⥫" +
                    "|祭  ࠢ⥫" +
                    "|  窥";

  dsptwho_Courier = 0; (* DsptStf.Who *)
  dsptwho_Sender  = 1;

  dapp_NotPost = 0;  (* DsptMode.PostProp *)
  dapp_Post    = 1;
  dapp_Worth   = 2;

  rcpk_Pool = " ७|筮||ࠧ稢| 㪠";
  rcpk_Authorization = 0;
  rcpk_Personally    = 1;

  TypePool     = "室騥|室騥";
  TypeShortPool = "|";
  InstancePool = "ਣ||㡫";
  StfWhoPool   = "|ࠢ⥫";
  OwnDocPool   = "|㦮";

  st_Prepared  = 0; (*Dspt.Status*)
  st_Sent      = 1;
  st_CS        = 2;
  st_Delivered = 3;
  st_Canceled  = 4;
  st_Pool      = "⮢|ࠢ|। |祭|㫨஢";
  st_ShortPool = "||||";

 PUBLIC PROCEDURE GetDsptType_asStr(): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(TypePool, Dspt.Type);
 END {GetDsptType_asStr};

 PUBLIC PROCEDURE GetDsptDocInstance_asStr(): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(InstancePool, DsptDoc.Instance);
 END {GetDsptDocInstance_asStr};

 PUBLIC PROCEDURE GetDsptStfWho_asStr(): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(StfWhoPool, DsptStf.Who);
 END {GetDsptStfWho_asStr};

 PUBLIC PROCEDURE GetDsptDocOwn_asStr(): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(OwnDocPool, DsptDoc.Own);
 END {GetDsptDocOwn_asStr};

 PUBLIC PROCEDURE GetDsptTimeRefPoint_asStr(): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(dt_RefPointPool, DsptTime.RefPoint);
 END {GetDsptTimeRefPoint_asStr};

 PUBLIC PROCEDURE AcceptableXDocIdInDspt(trgXDocId: L_XDoc.tXDocId): SCAL[2];
  VAR
   Result: SCAL[2];
   xdt   : L_XDoc.tXDocType;
   i     : tDsptDocType;
 BEGIN
  xdt:=L_XDoc.ExtractXDocType(trgXDocId);
  Result:=0;
  FOR_EACH i DO
   IF DocNeed[i] = xdt THEN
    Result:=1;
    EXIT;
   END;
  END;
  RETURN Result;
 END {AcceptableXDocIdInDspt};

 PUBLIC PROCEDURE Ref_KindName(rdt: L_XDoc.tXDocType): STRING[];
 BEGIN
  RETURN  L_XDoc.Ref_KindName(rdt);
 END {Ref_KindName};

 PUBLIC PROCEDURE Ref_WindowKindName(rdt: L_XDoc.tXDocType): STRING[];
 BEGIN
  RETURN Ref_KindName(rdt);
 END {Ref_WindowKindName};

 PUBLIC PROCEDURE Ref_FieldKindName(rdt: L_XDoc.tXDocType): STRING[];
 BEGIN
  RETURN Ref_KindName(rdt) + "." + Ref_KindName(rdt);
 END {Ref_FieldKindName};

 PUBLIC PROCEDURE DocNeed_Names(): STRING[];
  VAR
   Result: STRING[];
   i     : tDsptDocType;
 BEGIN
  Result :="";
  FOR_EACH i DO
   Result:=Result + L_XDoc.Ref_FullName(DocNeed[i]) + "|";
  END;
  RETURN Result;
 END {DocNeed_Names};

 PUBLIC PROCEDURE GetTypeAsXDocType(ddt: tDsptDocType): L_XDoc.tXDocType;
 BEGIN
  RETURN DocNeed[ddt];
 END {GetTypeAsXDocType};

 PUBLIC PROCEDURE GetTypeAsDsptType(xdt: L_XDoc.tXDocType): tDsptDocType;
  VAR
   i,
   Result: tDsptDocType;
 BEGIN
  Result:="";
  FOR_EACH i DO
   IF DocNeed[i] = xdt THEN
    Result:=i;
    EXIT;
   END;
  END;
  RETURN Result;
 END {GetTypeAsDsptType};

 PRIVATE PROCEDURE gd4xdi_Add(VAR Descrip: STRING[]; Prefix, Value: STRING[]);
 BEGIN
  IF ~!Value THEN
   Descrip:=L_Str.AddStrDlm(Descrip, L_Str.CRLF) + Prefix + Value;
  END;
 END {gd4xdi_Add};

 PRIVATE PROCEDURE gd4xdi_AddCntr(VAR Descrip: STRING[]);
 BEGIN
  IF OCCURE(Contract) THEN
   CONFORM(Contract, CntrT);
   gd4xdi_Add(Descrip, ": ", Contract.Contract + " " + Contract.Name + " " +
                                    ": " + Contract.SrcNbr + " " +
                                    " " + L_DatTim.DateStr(Contract.SrcDate) + " " +
                                    " : " + Contract.CntrT + " " + CntrT.Name);
  END;
 END {gd4xdi_AddCntr};

 PRIVATE PROCEDURE gd4xdi_AddFirm(VAR Descrip: STRING[]);
 BEGIN
  IF OCCURE(Firm) THEN
   gd4xdi_Add(Descrip, "ࠣ: ", Firm.Firm + " " + Firm.Name);
  END;
 END {gd4xdi_AddFirm};

 PUBLIC PROCEDURE GetDescriptionForXDocId(xdId: L_XDoc.tXDocId): STRING[];
  VAR
   xdt    : L_XDoc.tXDocType;
   xdk    : L_XDoc.tXDocKind;
   xd     : L_XDoc.tXDoc;
   Descrip: STRING[];
 BEGIN
  TRY
   L_XDoc.DecodeXDocId(xdId, xdt, xdk, xd);
   Descrip:="";
   IF    (xdt = L_XDoc.xdt_MDoc) & ~!xdk & ~!xd THEN
    IF SEARCH(MDoc, MDoc, xdk, xd) THEN
     CONFORM(MDoc, MDocKind);
     gd4xdi_Add(Descrip, ": ", MDocKind.Name);
     CONFORM(MDoc, Contract);
     gd4xdi_AddCntr(Descrip);
     CONFORM(MDoc, Staff);
     IF OCCURE(Staff) THEN
      CONFORM(Staff, Man);
      gd4xdi_Add(Descrip, "⭨: ", Staff.TSN + " " + Man.Name);
     END;
     CONFORM(MDoc, Firm);
     gd4xdi_AddFirm(Descrip);
    END;
   ELSIF (xdt = L_XDoc.xdt_PDoc) & ~!xdk & ~!xd THEN
    IF SEARCH(PDoc, PDoc, xdk, xd) THEN
     CONFORM(PDoc, PDocKind);
     gd4xdi_Add(Descrip, ": ", PDocKind.Name);
     gd4xdi_Add(Descrip, "", L_PDoc.GetPDocCommentPart(0));
     gd4xdi_Add(Descrip, "", L_PDoc.GetPDocCommentPart(1));
     gd4xdi_Add(Descrip, "", L_PDoc.GetPDocCommentPart(2));
    END;
   ELSIF (xdt = L_XDoc.xdt_Contract) & ~!xd THEN
    IF SEARCH(Contract, Contract, xd) THEN
     gd4xdi_AddCntr(Descrip);
     CONFORM(Contract, Firm);
     gd4xdi_AddFirm(Descrip);
    END;
   ELSIF (xdt = L_XDoc.xdt_ODoc) & ~!xdk & ~!xd THEN
    IF SEARCH(ODoc, ODoc, xdk, xd) THEN
     CONFORM(ODoc, ODocKind);
     gd4xdi_Add(Descrip, ": ", ODocKind.Name);
     IF ~Str.Match(L_ODoc.GetXDocName(xdId), "*" + ODoc.Descript + "*") THEN
      gd4xdi_Add(Descrip, "ᠭ: ", ODoc.Descript);
     END;
     CONFORM(ODoc, Firm);
     gd4xdi_AddFirm(Descrip);
    END;
   END;
   L_General.DisableDebuggerExceptionBreak();
   L_General.RaiseLockReleaseException();
  EXCEPT
   ON Exc: Exc_Raise DO
    L_General.CheckIsLockReleaseException(Exc);
  END;
  RETURN Descrip;
 END {GetDescriptionForXDocId};

 PUBLIC PROCEDURE XDocMustHaveCopyCnt(xdId: L_XDoc.tXDocId): SCAL[2];
  VAR
   DocFileName: STRING[];
 BEGIN
  DocFileName:=L_XDoc.Ref_FileName(L_XDoc.ExtractXDocType(xdId));
  RETURN DBD.FieldExist(^DocFileName, CopyCnt);
 END {XDocMustHaveCopyCnt};

 PUBLIC PROCEDURE GetRightCopyCntByXDocId(xdId: L_XDoc.tXDocId): L_XDoc.tCopyCnt;
  VAR
   DocFileName,
   CopyCntFieldName: STRING[];
 BEGIN
  L_XDoc.PositionOnXDoc(xdId);
  DocFileName:=L_XDoc.Ref_FileName(L_XDoc.ExtractXDocType(xdId));
  IF DBD.FieldExist(^DocFileName, CopyCnt) THEN
   RETURN ^(DocFileName+".CopyCnt");
  ELSE
   RETURN "";
  END;
 END {GetRightCopyCntByXDocId};

 PUBLIC PROCEDURE IsPostalDspt(): SCAL[2];
 BEGIN
  CONFORM(Dspt, DsptMode);
  RETURN (DsptMode.PostProp = dapp_Post) |
         (DsptMode.PostProp = dapp_Worth);
 END {IsPostalDspt};

 PUBLIC PROCEDURE IsPostalWorthDspt(): SCAL[2];
 BEGIN
  CONFORM(Dspt, DsptMode);
  RETURN DsptMode.PostProp = dapp_Worth;
 END {IsPostalWorthDspt};

 PUBLIC PROCEDURE IsOutgoingDspt(): SCAL[2];
 BEGIN
  RETURN Dspt.Type = t_Outgoing;
 END {IsOutgoingDspt};

 PUBLIC PROCEDURE IsIncomingDspt(): SCAL[2];
 BEGIN
  RETURN Dspt.Type = t_Incoming;
 END {IsIncomingDspt};

 PRIVATE PROCEDURE DecodeDsptId(DsptId: L_XDoc.tXDocId; VAR type: LIKE(Dspt.Type); VAR dspt: LIKE(Dspt.Dspt));
  VAR
   xdt: L_XDoc.tXDocType;
 BEGIN
  L_XDoc.DecodeXDocId(DsptId, xdt, type, dspt);
  L_General.CallUs(xdt = L_XDoc.xdt_Dispatch);
 END {DecodeDsptId};

 PRIVATE PROCEDURE DecodeDsptPckId(trgDsptPckId: STRING[]; VAR trgDsptPck: LIKE(DsptPck.DsptPck));
  VAR
   xdt: L_XDoc.tXDocType;
   xdk: L_XDoc.tXDocKind;
 BEGIN
  L_XDoc.DecodeXDocId(trgDsptPckId, xdt, xdk, trgDsptPck);
  L_General.CallUs(xdt = L_XDoc.xdt_DsptPck);
 END {DecodeDsptPckId};

 PUBLIC PROCEDURE IsDsptSent(trgType: LIKE(Dspt.Type); trgDspt: LIKE(Dspt.Dspt)): SCAL[2];
 BEGIN
  RETURN SEARCH(DsptTime, Dspt, trgType, trgDspt, rp_Send);
 END {IsDsptSent};

END.
