(*
%Copyright   (c) Polak Computer System
%Author      25.05.2001 10:46:54 BOR
%Purpose
%Type        library
%Specific    SYS
%Modified    10.09.2014 18:18:27 POLAK\Bor
%Environment
*)

 MODULE L_Events [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tMode = SCAL[1E3];

 PUBLIC CONST
  ee_ProcessId             = "ProcessId";

  md_SubWinInsert          = 011;(*???*)
  md_SubWinShowWin         = 013;(*???*)
  md_SingleMakeDone        = 051;
  md_StreamMakeDone        = 061;
  md_SingleMakeUnDone      = 050;
  md_StreamMakeUnDone      = 060;
  md_SinglePrint           = 081;
  md_SingleRePrint         = 082;
  md_StreamPrint           = 083;
  md_StreamRePrint         = 084;
  md_SinglePrePrint        = 085;
  md_SingleSending         = 086;
  md_StreamSending         = 087;
  md_InIMPORT              = 999;

 PRIVATE CONST
  md_BeforeEdit            = 000;
  md_AfterEdit             = 001;
  md_BeforeInsert          = 002;
  md_AfterInsert           = 003;
  md_BeforeDelete          = 004;
  md_AfterDelete           = 005;
  md_ZoneOpen              = 006;
  md_Reposition            = 007;
  md_ChangeIndex           = 008;
  md_RedrawZone            = 009;
  md_SubWinEdit            = 010;
  md_SubWinDelete          = 012;
  md_ConformToWindow       = 015;
  md_InitializeWindow      = 016;
  md_CloseWindow           = 017;
  md_OpenWindow            = 018;
  md_OnGotFocus            = 243;
  md_ExplicitAskPhase1     = 245;
  md_ExplicitAskPhase2     = 246;
  md_BeforeAskForView      = 247;
  md_AskForViewPhase1      = 248;
  md_AskForViewPhase2      = 249;
  md_BeforeAskForEdit      = 250;
  md_AskForEditPhase1      = 251;
  md_AskForEditPhase2      = 252;
  md_FieldSuccessfulChange = 254;

 IMPORT DBD, L_WDO, L_General, L_PCS, L_Str, Str, L_StdFrm;

 (*!*)PUBLIC PROCEDURE IsBeforeEditEvent(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN _MODE = md_BeforeEdit;
 END {IsBeforeEditEvent};

 PUBLIC PROCEDURE IsAfterEditInsertEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE = md_AfterInsert) | (_MODE = md_AfterEdit);
 END {IsAfterEditInsertEvents};

 PUBLIC PROCEDURE IsBeforeEditDeleteEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN IsBeforeEditEvent(_MODE) | (_MODE = md_BeforeDelete);
 END {IsBeforeEditDeleteEvents};

 PRIVATE PROCEDURE IsRecordEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN ((_MODE >= md_BeforeEdit) & (_MODE <= md_AfterDelete));
 END {IsRecordEvent};

 PRIVATE PROCEDURE RecordEventHandlerName(_MODE: tMode): STRING[];
 BEGIN
  IF    _MODE = md_BeforeEdit   THEN RETURN "OnBeforeEdit";
  ELSIF _MODE = md_AfterEdit    THEN RETURN "OnAfterEdit";
  ELSIF _MODE = md_BeforeInsert THEN RETURN "OnBeforeInsert";
  ELSIF _MODE = md_AfterInsert  THEN RETURN "OnAfterInsert";
  ELSIF _MODE = md_BeforeDelete THEN RETURN "OnBeforeDelete";
  ELSIF _MODE = md_AfterDelete  THEN RETURN "OnAfterDelete";
  END;
 END {RecordEventHandlerName};

 (*!*)PUBLIC PROCEDURE IsSubWinInsertEvent(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN _MODE = md_SubWinInsert;
 END {IsSubWinInsertEvent};

 PUBLIC PROCEDURE IsSubWinEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE >= md_SubWinEdit) & (_MODE <= md_SubWinShowWin);
 END {IsSubWinEvent};

 PUBLIC PROCEDURE IsWindowEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE = md_ConformToWindow) | (_MODE = md_InitializeWindow) | (_MODE = md_OpenWindow)| (_MODE = md_CloseWindow);
 END {IsWindowEvents};

 PRIVATE PROCEDURE IsZoneEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE = md_ZoneOpen) | (_MODE = md_ChangeIndex) | (_MODE = md_RedrawZone) | (_MODE = md_Reposition);
 END {IsZoneEvent};

 PUBLIC PROCEDURE IsExplicitAskEvent(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE = md_ExplicitAskPhase1) | (_MODE = md_ExplicitAskPhase2);
 END {IsExplicitAskEvent};

 PRIVATE PROCEDURE ExplicitAskHandlerName(_MODE: tMode): STRING[];
 BEGIN
  IF    _MODE = md_ExplicitAskPhase1 THEN
   RETURN "OnExplicitAskPhase1";
  ELSIF _MODE = md_ExplicitAskPhase2 THEN
   RETURN "OnExplicitAskPhase2";
  END;
 END {ExplicitAskHandlerName};

 (*!*)PUBLIC PROCEDURE IsFieldSuccessfulChangeEvent(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN _MODE = md_FieldSuccessfulChange;
 END {IsFieldSuccessfulChangeEvent};

 PRIVATE PROCEDURE IsFieldEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE = md_BeforeAskForView)      |
         (_MODE = md_AskForViewPhase1)      |
         (_MODE = md_AskForViewPhase2)      |
         (_MODE = md_BeforeAskForEdit)      |
         (_MODE = md_AskForEditPhase1)      |
         (_MODE = md_AskForEditPhase2)      |
         (_MODE = md_FieldSuccessfulChange) |
         (_MODE = md_OnGotFocus);
 END {IsFieldEvents};

 PRIVATE PROCEDURE FieldEventHandlerName(_MODE: tMode; newStyle: SCAL[2]): STRING[];
 BEGIN
  IF    _MODE = md_BeforeAskForView THEN
   IF newStyle THEN                      RETURN "OnBeforeAsk";
   ELSE                                  RETURN "OnBeforeAskForView";
   END;
  ELSIF _MODE = md_AskForViewPhase1      THEN RETURN "OnAskForViewPhase1";
  ELSIF _MODE = md_AskForViewPhase2      THEN RETURN "OnAskForViewPhase2";
  ELSIF _MODE = md_BeforeAskForEdit THEN
   IF newStyle THEN                      RETURN "OnBeforeAsk";
   ELSE                                  RETURN "OnBeforeAskForEdit";
   END;
  ELSIF _MODE = md_AskForEditPhase1      THEN RETURN "OnAskPhase1";
  ELSIF _MODE = md_AskForEditPhase2      THEN RETURN "OnAskPhase2";
  ELSIF _MODE = md_FieldSuccessfulChange THEN RETURN "OnSuccessfulChange";
  ELSIF _MODE = md_OnGotFocus            THEN RETURN "OnGotFocus";
  END;
 END {FieldEventHandlerName};

 PRIVATE PROCEDURE ZoneEventHandlerName(_MODE: tMode): STRING[];
 BEGIN
  IF    _MODE = md_ZoneOpen    THEN RETURN "OnZoneOpen";
  ELSIF _MODE = md_ChangeIndex THEN RETURN "OnChangeIndex";
  ELSIF _MODE = md_RedrawZone  THEN RETURN "OnRedraw";
  ELSIF _MODE = md_Reposition  THEN RETURN "OnReposition";
  END;
 END {ZoneEventHandlerName};

 PRIVATE PROCEDURE WindowEventHandlerName(_MODE: tMode): STRING[];
 BEGIN
  IF    _MODE = md_ConformToWindow  THEN RETURN "OnConformToWindow";
  ELSIF _MODE = md_InitializeWindow THEN RETURN "OnInitializeWindow";
  ELSIF _MODE = md_CloseWindow      THEN RETURN "OnCloseWindow";
  ELSIF _MODE = md_OpenWindow       THEN RETURN "OnOpenWindow";
  END;
 END {WindowEventHandlerName};

 PUBLIC PROCEDURE IsStreamEvents(_MODE: tMode): SCAL[2];
 BEGIN
  RETURN (_MODE = md_StreamMakeDone)   |
         (_MODE = md_StreamMakeUnDone) |
         (_MODE = md_StreamPrint)      |
         (_MODE = md_StreamRePrint)    |
         (_MODE = md_StreamSending);
 END {IsStreamEvents};

 PRIVATE PROCEDURE checkMODE(VAR _MODE: tMode);
 BEGIN
  IF RTI.Get().StartMode # _MODE THEN
   IF L_General.IsOurSite() THEN
    L_General.CallUs(0);
   END;
  END;
 END {checkMODE};

 PUBLIC PROCEDURE IsInternalEvents(_MODE: tMode): SCAL[2];
 BEGIN
checkMODE(_MODE);
  RETURN IsRecordEvents(_MODE) | IsWindowEvents(_MODE) |
         IsZoneEvents(_MODE) | IsFieldEvents(_MODE) |
         IsExplicitAskEvent(_MODE) | (_MODE = md_InIMPORT);
 END {IsInternalEvents};

 PUBLIC PROCEDURE IsDocumentProprietyExaminationEvents(_MODE: tMode; OnlyInteractive: SCAL[2]): SCAL[2];
 BEGIN
  IF OnlyInteractive THEN
   RETURN IsAfterEditInsertEvents(_MODE);
  ELSE
   RETURN IsAfterEditInsertEvents(_MODE) |
          (_MODE = md_SingleMakeDone) | (_MODE = md_StreamMakeDone);
  END;
 END {IsDocumentProprietyExaminationEvents};

 PRIVATE PROCEDURE intGetEventClassName(_MODE: tMode; newStyle: SCAL[2]): STRING[];
  VAR
   curField: FORM_FIELD;
 BEGIN
  IF    IsSubWinEvents(_MODE) THEN
   RETURN L_Str.IntStr(RTI.Get().CurZone.Number) + "_" + "OnSetSubwindow";
  ELSIF IsRecordEvents(_MODE) THEN
   RETURN RecordEventHandlerName(_MODE);
  ELSIF IsWindowEvents(_MODE) THEN
   RETURN WindowEventHandlerName(_MODE);
  ELSIF ~newStyle & (_MODE = md_Reposition) THEN
   RETURN ZoneEventHandlerName(_MODE);
  ELSIF IsZoneEvents(_MODE) THEN
   RETURN L_Str.IntStr(RTI.Get().CurZone.Number) + "_" + ZoneEventHandlerName(_MODE);
  ELSIF IsExplicitAskEvent(_MODE) THEN
   RETURN ExplicitAskHandlerName(_MODE);
  ELSIF IsFieldEvents(_MODE) THEN
   curField:=RTI.Get().CurZone.ActiveField;
   IF ~!curField.TableField THEN
    RETURN curField.TableField.Name + "_" + FieldEventHandlerName(_MODE, newStyle);
   ELSE
    IF newStyle THEN
     RETURN curField.Name + "_" + FieldEventHandlerName(_MODE, newStyle);
    ELSE
     RETURN "_" + FieldEventHandlerName(_MODE, newStyle);
    END;
   END;
  ELSE
   L_General.CallUs(0);
  END;
 END {intGetEventClassName};

 PUBLIC PROCEDURE GetEventClassName(_MODE: tMode): STRING[];
 BEGIN
  RETURN intGetEventClassName(_MODE, 0);
 END {GetEventClassName};

 PUBLIC PROCEDURE Stub_InIMPORT();
 BEGIN
  (*  楤 ࠡ⪨   ,  饣 ⥪ *)
 END {Stub_InIMPORT};

 PRIVATE PROCEDURE GetStandartAlias(_MODE: tMode; newStyle: SCAL[2]): STRING[];
 BEGIN
  IF    _MODE = md_InIMPORT THEN
   RETURN "";
  ELSIF IsSubWinEvents(_MODE) THEN
   RETURN RTI.Get().CurWindow.OriginalName;
  ELSIF ~newStyle & (_MODE = md_Reposition) THEN
   IF ~!RTI.Get().CurZone.MasterTable THEN
    RETURN RTI.Get().CurZone.MasterTable.Name;
   ELSE
    L_General.CallUs(0);
   END;
  ELSIF IsZoneEvents(_MODE) THEN
   RETURN RTI.Get().CurWindow.OriginalName;
  ELSIF _MODE = md_ConformToWindow THEN
   RETURN RTI.Get().ConformTargetWindow.OriginalName;
  ELSIF IsWindowEvents(_MODE) THEN
   RETURN RTI.Get().CurWindow.OriginalName;
  ELSE
   IF ~!RTI.Get().CurZone.MasterTable THEN
    RETURN RTI.Get().CurZone.MasterTable.Name;
   ELSE
    IF newStyle THEN
     RETURN RTI.Get().CurWindow.OriginalName;
    ELSE
     RETURN "";
    END;
   END;
  END;
 END {GetStandartAlias};

 PRIVATE PROCEDURE GetAnyStyleInternalEventHandlerName_wAlias(Alias: STRING[]; _MODE: tMode; newStyle: SCAL[2]): STRING[];
 BEGIN
  IF !Alias THEN
   Alias:=GetStandartAlias(_MODE, newStyle);
  END;
  RETURN Alias + "_" + intGetEventClassName(_MODE, newStyle);
 END {GetAnyStyleInternalEventHandlerName_wAlias};

 PRIVATE PROCEDURE IsProcExist(ProcName: STRING[]): SCAL[2];
 BEGIN
  RETURN ~!RTI.Get().Stack(RTI.Get().Stack.Count-1).Proc.Module.ProcSpecs(ProcName);
 END {IsProcExist};

 PRIVATE PROCEDURE intGetInternalEventHandlerName_wAlias(Alias: STRING[]; _MODE: tMode; wSysLogHandler: SCAL[2]): STRING[];
  VAR
   Msg,
   Result,
   ModuleName,
   newstyleProc,
   oldstyleProc,
   newstyleProcFull,
   oldstyleProcFull: STRING[];
 BEGIN
checkMODE(_MODE);
  IF _MODE = md_InIMPORT THEN
   Result:="L_Events.Stub_InIMPORT";
  ELSE
   IF wSysLogHandler &
     ((_MODE = md_AfterEdit) | (_MODE = md_AfterInsert) | (_MODE = md_BeforeDelete)) THEN
    L_PCS.SysLogHandler(_MODE);
   END;
   newstyleProc    :=GetAnyStyleInternalEventHandlerName_wAlias(Alias, _MODE, 1);
   oldstyleProc    :=GetAnyStyleInternalEventHandlerName_wAlias(Alias, _MODE, 0);
   Result      :=newstyleProc;

   ModuleName      :=RTI.Get().Stack(RTI.Get().Stack.Count-1).Proc.Module.Name;
   newstyleProcFull:=ModuleName+"."+newstyleProc;
   oldstyleProcFull:=ModuleName+"."+oldstyleProc;
   IF IsProcExist(oldstyleProc) THEN
    SETEMPTY(Msg);
    IF    ~IsProcExist(newstyleProc) THEN
     Result:=oldstyleProc;
     Msg   :="맮 楤 '"+oldstyleProcFull+"'  ன 樨.||" +
             " : '" +newstyleProcFull+"'";
    ELSIF newstyleProc # oldstyleProc THEN
     Msg   :="楤 '"+newstyleProcFull+"' ଫ   樨,|" +
             "     '"+oldstyleProcFull+"'||" +
             " 稨 楤   樨|" +
             "   뢠    㤠";
    END; (* IF IsProcExist(newstyleProc)...*)
    IF ~!Msg & ~APP.Get().ProductiveCopy & (Str.Match(Str.Cap(RTI.Get().Stack(RTI.Get().Stack.Count-1).Proc.Module.Path), "*TMP_LIB*")) THEN
     L_StdFrm.CustomQuestion(Msg);
    END;
   END; (* IF IsProcExist(oldstyleProc)...*)
  END; (* IF _MODE = md_InIMPORT...*)
  RETURN Result;
 END {intGetInternalEventHandlerName_wAlias};

 PUBLIC PROCEDURE extGetInternalEventHandlerName(_MODE: tMode): STRING[];
 BEGIN
  RETURN intGetInternalEventHandlerName_wAlias("", _MODE, 0);
 END {extGetInternalEventHandlerName};

 PUBLIC PROCEDURE GetInternalEventHandlerName(_MODE: tMode): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  Result:=intGetInternalEventHandlerName_wAlias("", _MODE, 1);
  IF _MODE # md_InIMPORT THEN DEBUGBREAK(); END; RETURN Result;
 END {GetInternalEventHandlerName};

 PUBLIC PROCEDURE GetHandlerName(_Event: EVENT): STRING[];
  VAR
   _MODE: tMode;
 BEGIN
_MODE:=_Event.Mode;
checkMODE(_MODE);
  IF IsExplicitAskEvent(_MODE) THEN
   RETURN intGetInternalEventHandlerName_wAlias(RTI.Get().Env.Item(ee_ProcessId).Value, _MODE, 1);
  ELSE
   RETURN GetInternalEventHandlerName(_MODE);
  END;
 END {GetHandlerName};

 PRIVATE PROCEDURE intExecuteHandler(Alias: STRING[]);
  VAR
   P           : PROC;
   ProcName,
   ModuleName,
   ProcFullName: STRING[];
   trgStartMode: tMode;
 BEGIN
  trgStartMode:=RTI.Get().StartMode;
  IF ~!trgStartMode (*. 0090382 -  ࢥ ⮬⨧樨 *) THEN
   L_General.CallUs(IsSubWinEvents(trgStartMode) | IsInternalEvents(trgStartMode));
   IF IsExplicitAskEvent(trgStartMode) THEN
    ProcName:=intGetInternalEventHandlerName_wAlias(RTI.Get().Env.Item(ee_ProcessId).Value, trgStartMode, 1);
   ELSE
    ProcName:=intGetInternalEventHandlerName_wAlias(Alias, trgStartMode, 1);
   END;
 
   IF trgStartMode # md_InIMPORT THEN
    ModuleName:=RTI.Get().Stack(RTI.Get().Stack.Count-1).Proc.Module.Name;
    ProcFullName:=ModuleName + "." + ProcName;
    TRY [EXCEPTIONHANDLING(EXCEPTIONHANDLING_OFF)]
     P:=STRTOFLD(ProcFullName, PROC);
    EXCEPT
    ON Exc: Exc_RunTime DO
     IF Exc.ErrorCode = L_General.ec_rt_FailedSTRTOFLD THEN
      IF L_General.IsProgrammer() THEN
       STDMSG("ॡ ࠡ稪 ᮡ: " + ProcFullName, INFORM_RED);
       CALL(CallStub.AppendMacro("EditText, '" + ModuleName + ".asl"
                                               + " /S:" + ProcName
                                               + " /M:" + ProcName
                                               + "'"));
       HALT();
      ELSE
       STDMSG("ॡ ࠡ稪 ᮡ: " + ProcFullName, STOP_AND_CALL_AUTHOR);
      END;
     ELSE
      RAISE();
     END;
    END;
    DEBUGBREAK(); P();
   END; (* IF trgStartMode # md_InIMPORT *)
  END;
 END {intExecuteHandler};

 PUBLIC PROCEDURE ExecuteHandler();
 BEGIN
  intExecuteHandler("");
 END {ExecuteHandler};

 PUBLIC PROCEDURE extExecuteHandler(Alias: STRING[]);
 BEGIN
  intExecuteHandler(Alias);
 END {extExecuteHandler};

END.
