(*
%Copyright   (c) Polak IT
%Author      04.08.2005 18:01:39 POLAK\LDA
%Purpose     ⥪  ࠡ  MS Excel
%Type        library
%Specific    SYS
%Modified    12.09.2014 12:28:27 POLAK\GAV
%Environment
*)

 MODULE L_Excel [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC CONST
  xlFalse      = 0;
  xlTrue       = -1;

  xlRight      = -4152; (* Alignment *)
  xlCenter     = -4108;
  xlLeft       = -4131;

  xlTop        = -4160;

  xlThin       = 2;     (* Borders.Weight *)
  xlMedium     = 3;

  xlLandscape  = 2;     (* PageSetup.Orientation *)

  xlEdgeBottom = 9;     (* Border(index) *)

 IMPORT Memo, L_General, L_Str, L_StdFrm, L_DatTim, L_DBD, L_AFM, L_Str, L_MSWStp, L_PCS,
        Str, FIO, DBD;

 PRIVATE TYPE
  tCond = SCAL[4];

 PRIVATE CONST
  cnd_All     = 0;
  cnd_OnDate  = 1;
  cnd_OnIntvl = 2;
  cnd_FromWDO = 3;

  BlankCellAmount = 254;

 PRIVATE VAR
  BlankWS,
  BlankRow          : OBJECT;
  BlankCells        : OBJECT[BlankCellAmount+1];
  xlDateFormat,
  xlTimeFormat,
  xlDecimalSeparator: STRING[];
  ws_CurIdx         : NUMERIC[0];
  ue_MainFN,
  ue_Filter         : STRING[];

 PRIVATE PERMANENT VAR
  ExcelFound: SCAL[2];

 PUBLIC PROCEDURE CreateExcel(Strong: SCAL[2]): OBJECT;
  VAR
   Result: OBJECT;
 BEGIN
  APP.Get().NewPrompt(" MS Excel Application");
  SETEMPTY(Result);
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   Result:=CreateOBJECT("Excel.Application");
  EXCEPT
  ON Exc: Exc_Root DO
   L_General.Assert(~Strong, " ⠭ MS Excel");
  END; (* TRY *)
  RETURN Result;
 END {CreateExcel};

 PUBLIC PROCEDURE ExcelInstalled(): SCAL[2];
 BEGIN
  IF ~VAR_IMPL.Get(ExcelFound).Initialized THEN
   ExcelFound:=~!CreateExcel(0);
  END;
  RETURN ExcelFound;
 END {ExcelInstalled};

 PUBLIC PROCEDURE ExcelVersion(objXL: OBJECT): NUMERIC[1];
  VAR
   tmpVersion: NUMERIC[1];
 BEGIN
  SETEMPTY(tmpVersion);
  TRY
   tmpVersion:=STRTOFLD(objXL.Application.Version, NUMERIC);
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode # L_General.ec_rt_FailedSTRTOFLD THEN
    RAISE();
   END;
  END; (* TRY *)
  RETURN tmpVersion;
 END {ExcelVersion};

 PUBLIC PROCEDURE NewWorksheet(objWB: OBJECT; Idx: NUMERIC[0]; Name: STRING[]): OBJECT;
  VAR
   objWS: OBJECT;
 BEGIN
  IF Idx >= 1 THEN
   objWS:=objWB.Worksheets.Add(, objWB.Worksheets(Idx));
  ELSE
   objWS:=objWB.Worksheets(1);
  END;
  objWS.Name:=Str.Slice(Name, 0, 30);
  RETURN objWS;
 END {NewWorksheet};

 PUBLIC PROCEDURE NewWorkbook(objXL: OBJECT; SheetAmount: NUMERIC[0]): OBJECT;
  VAR
   objWB          : OBJECT;
   _SheetsInNewWorkbook: NUMERIC[0];
 BEGIN
  _SheetsInNewWorkbook:=objXL.SheetsInNewWorkbook;
  IF ~!SheetAmount THEN
   objXL.SheetsInNewWorkbook:=SheetAmount;
  END;
  objWB:=objXL.Workbooks.Add();
  objXL.SheetsInNewWorkbook:=_SheetsInNewWorkbook;
  RETURN objWB;
 END {NewWorkbook};

 PUBLIC PROCEDURE CloseWorkbook(VAR objWB: OBJECT);
 BEGIN
  IF ~!objWB THEN
   objWB.Saved:=xlTrue;
   objWB.Close(xlFalse);
   SETEMPTY(objWB);
  END;
 END {CloseWorkbook};

 PUBLIC PROCEDURE CloseExcel(VAR objXL, objWB: OBJECT);
 BEGIN
  CloseWorkbook(objWB);
  IF ~!objXL THEN
   objXL.Quit();
   SETEMPTY(objXL);
  END;
 END {CloseExcel};

 PUBLIC PROCEDURE CloseExcelAfterCrash(VAR objXL, objWB: OBJECT);
 BEGIN
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   CloseExcel(objXL, objWB);
  EXCEPT
  ON Exc: Exc_Root DO
  ELSE
  END; (* TRY *)
 END {CloseExcelAfterCrash};

 PRIVATE PROCEDURE SetExcelFormats();
  VAR
   objXL,
   objWB,
   objWS,
   objCell: OBJECT;
 BEGIN
  objXL:=CreateOBJECT("Excel.Application");
  objXL.Visible:=xlFalse;
  objWB:=objXL.Workbooks.Add();
  objWS:=objWB.WorkSheets(1);
  objCell:=objWS.Cells(1, 1);
  objCell.NumberFormat:="";
  objCell.Value:=0.5;
  objCell.EntireColumn.AutoFit();
  xlDecimalSeparator:=L_Str.Char(objCell.Text, 1);
  objCell.Value:=12:30;
  objCell.EntireColumn.AutoFit();
(*
  xlTimeFormat:=objCell.Text;
  xlTimeFormat:=L_Str.Replace(xlTimeFormat,   "12", "H");
  xlTimeFormat:=L_Str.Replace(xlTimeFormat,   "30", "M");
  (*㭤 -   祩   "PM" / ᥪ㭤,   ⮫쪮 ࠧ⥫*)
*)
  xlTimeFormat:="H" + L_Str.Char(objCell.Text, 2) + "M";
  objCell.Value:=CreateVARIANT(Date, 31\12\2001);
  objCell.EntireColumn.AutoFit();
  xlDateFormat:=objCell.Text;
  xlDateFormat:=L_Str.Replace(xlDateFormat,   "31", "D");
  xlDateFormat:=L_Str.Replace(xlDateFormat,   "12", "N");
  xlDateFormat:=L_Str.Replace(xlDateFormat, "2001", "y");
  xlDateFormat:=L_Str.Replace(xlDateFormat,   "01", "Y");
  CloseWorkbook(objWB);
 END {SetExcelFormats};

 PUBLIC PROCEDURE DecimalSeparator(): STRING[];
 BEGIN
  IF ~VAR_IMPL.Get(xlDecimalSeparator).Initialized THEN
   SetExcelFormats();
  END;
  RETURN xlDecimalSeparator;
 END {DecimalSeparator};

 PUBLIC PROCEDURE TimeFormat(): STRING[];
 BEGIN
  IF ~VAR_IMPL.Get(xlTimeFormat).Initialized THEN
   SetExcelFormats();
  END;
  RETURN xlTimeFormat;
 END {TimeFormat};

 PUBLIC PROCEDURE DateFormat(): STRING[];
 BEGIN
  IF ~VAR_IMPL.Get(xlDateFormat).Initialized THEN
   SetExcelFormats();
  END;
  RETURN xlDateFormat;
 END {DateFormat};

 PUBLIC PROCEDURE AddCellComment(cell: OBJECT; Text: STRING[]);
 BEGIN
  cell.AddComment(Text);
  cell.Comment.Visible:=xlFalse;
 END {AddCellComment};

 PUBLIC PROCEDURE AddComment(objWS: OBJECT; row, column: NUMERIC[0]; Text: STRING[]);
 BEGIN
  AddCellComment(objWS.Cells(row, column), Text);
 END {AddComment};

 PRIVATE PROCEDURE AddColumnFromRow(objWS     : OBJECT;
                                    row,
                                    column,
                                    FieldType,
                                    Precision : NUMERIC[0];
                                    Name,
                                    Comment   : STRING[]);
  VAR
   NumberFormat: STRING[];
   headcell    : OBJECT;
 BEGIN
  headcell:=objWS.Cells(row, column);
  IF    (FieldType = L_General.ft_NUMERIC) THEN
   IF Precision > 0 THEN
    NumberFormat:="0" + DecimalSeparator() + L_Str.Repeat("0", Precision);
   ELSE
    NumberFormat:="0";
   END;
  ELSIF (FieldType = L_General.ft_SCAL) THEN
   NumberFormat:=L_Str.Repeat("0", Precision);
  ELSIF (FieldType = L_General.ft_STRING) THEN
   NumberFormat:="@";
  ELSIF (FieldType = L_General.ft_TIME) THEN
   NumberFormat:=":";
  ELSE
   NumberFormat:=headcell.NumberFormat;
  END;
  headcell.EntireColumn.NumberFormat:=NumberFormat;
  IF ~!BlankCells[column] THEN
   BlankCells[column].NumberFormat:=NumberFormat;
  END;
  headcell.NumberFormat:="@";
  headcell.WrapText    :=xlTrue;
  headcell.Value       :=Name;
  IF ~!Comment THEN
   AddCellComment(headcell, Comment);
  END;
 END {AddColumnFromRow};

 PRIVATE PROCEDURE ue_PrepareFileList(    File            : STRING[];
                                          wMainBorrow     : SCAL[2];
                                      VAR ue_prpFileList  : STRING[];
                                          ExcludeFileList : STRING[];
                                          Level           : NUMERIC[0];
                                      VAR ue_FileArr      : SCAL[2][];
                                      VAR ue_FileLev      : NUMERIC[0][];
                                      VAR ue_FileAmount   : NUMERIC[0];
                                          ue_MaxFileAmount: NUMERIC[0]);
  VAR
   i,
   LinkAmount: NUMERIC[0];
   LinkKind  : NUMERIC[0];
   LinkName  : STRING[];
   curWidth,
   curHeight : NUMERIC[0];
 BEGIN
  IF !L_Str.PositionInPool(ue_prpFileList, File) &
     !L_Str.PositionInPool(ExcludeFileList, File) THEN
   ue_prpFileList           :=ue_prpFileList + File + "|";
   ue_FileArr[ue_FileAmount]:=0;
   ue_FileLev[ue_FileAmount]:=Level;
   INC(ue_FileAmount);
   i:=0;
   LinkAmount:=DBD.LinkAmount(^File);
   WHILE (i < LinkAmount) & (ue_FileAmount < ue_MaxFileAmount) DO
    LinkName:=DBD.LinkName(^File, i);
    LinkKind:=DBD.LinkKind(^File, ^LinkName);
    IF (LinkKind = L_General.lk_USER)    |
       (LinkKind = L_General.lk_BORROW)  |
       (LinkKind = L_General.lk_HISTORY) |
       (wMainBorrow & ((LinkKind = L_General.lk_LEND) |
                       (LinkKind = L_General.lk_REVERSE_HISTORY))) THEN
     ue_PrepareFileList(DBD.LinkCooFileName(^File, ^LinkName), 0, ue_prpFileList, ExcludeFileList, Level+1, ue_FileArr, ue_FileLev, ue_FileAmount, ue_MaxFileAmount);
    END;
    INC(i);
   END;
  END;
 END {ue_PrepareFileList};

 PRIVATE PROCEDURE HasGoodLink(ownFile, cooFile: STRING[]; wMainBorrow: SCAL[2]): SCAL[2];
  VAR
   link: STRING[];
   kind: NUMERIC[0];
 BEGIN
  link:=L_DBD.Get2FilesLinkName(ownFile, cooFile);
  IF !link THEN
   RETURN 0;
  ELSE
   kind:=DBD.LinkKind(^ownFile, ^link);
   RETURN (kind = L_General.lk_USER) |
          (kind = L_General.lk_LEND) |
          (kind = L_General.lk_REVERSE_HISTORY) |
          wMainBorrow & (kind = L_General.lk_BORROW) |
          wMainBorrow & (kind = L_General.lk_HISTORY) ;
  END;
 END {HasGoodLink};

 PRIVATE PROCEDURE CanSortByDate(FileName: STRING[]): SCAL[2];
 BEGIN
  RETURN DBD.FieldExist(^FileName, Date) & DBD.IndexExist(^FileName, Date);
 END {CanSortByDate};

 PRIVATE PROCEDURE cf_AcceptableFile(File: STRING[]): SCAL[2];
  VAR
   BF, EF: STRING[];
 BEGIN
  IF (File = GE_CURFILE()) | (AMOUNT(^File) < 10000) | CanSortByDate(File) THEN
   RETURN 1;
  ELSIF DBD.FieldExist(^File, Date) THEN
   RETURN 1;
  ELSE
   L_DBD.CollateValidityFieldNames(File, BF, EF);
   RETURN  ~!BF & ~!EF;
  END;
 END {cf_AcceptableFile};

 PRIVATE PROCEDURE ue_SetupDialog(    CurFile,
                                      CurRecord      : SCAL[2];
                                  VAR ue_FileList    : STRING[];
                                      ExcludeFileList: STRING[];
                                  VAR Cond           : tCond);
  CONST
   FieldWith = 50;
   MaxFiles  = 1000;
  VAR
   ControlsList,
   FileNameList,
   VariableList,
   file,
   ue_prpFileList,
   CondPrefix,
   CondContrValName,
   CondContrBlank,
   CondContrPool,
   CondContrList   : STRING[];
   ue_FileArr      : SCAL[2][MaxFiles];
   ue_FileLev      : NUMERIC[0][MaxFiles];
   i,
   j,
   curWidth,
   curHeight,
   ue_FileAmount,
   ue_MaxFileAmount: NUMERIC[0];
   LinkSelected    : SCAL[2];
 BEGIN
  SETEMPTY(ue_prpFileList, ue_FileArr);
  SETZERO(ue_FileAmount);
  CondContrPool:=" " +
                 "|쪮 騥   " + L_DatTim.DateStr(Memo.ud_cd()) +
                 "|쪮   ਮ  " + L_DatTim.DateStr(Memo.ud_fr()) + "  " + L_DatTim.DateStr(Memo.ud_ut());
  IF CurFile THEN
   L_General.Assert(~!GE_CURFILE(), " ⠭ ⥪騩 䠩");
   ue_MainFN:=GE_CURFILE();
   CondContrPool:=CondContrPool + "|쪮 , ⮡ࠦ  ";
  ELSE
   ue_MainFN:=L_DBD._ChooseFileName("㧪 ࠢ筨  Excel", PROCREF(cf_AcceptableFile), 1);
  END;
  L_General.Assert_wReferToAdmin(!L_Str.PositionInPool(L_PCS.Files_UploadToExcel_wAdminOnlyRights_Pool, ue_MainFN),
                                 "㧪 䠩 '" + TABLE.Get(ue_MainFN).Description + "' 㯭 ⮫쪮 " +
                                 "|᫥⢨ ⥬ ࠭祭 㯠");

  IF AMOUNT(^ue_MainFN) = 0 THEN
   L_StdFrm.BlueStop(" 䠩 '" + TABLE.Get(ue_MainFN).Description + "' (" + ue_MainFN + ")|" +
                     "   , 㧪  Excel ");
  END;

  IF L_General.IsAcceptableProcessorVersion(014261010) THEN
   ue_MaxFileAmount:=999; (*  ue_MaxFileAmount  *)
  ELSE
   GE_APPWIN_SIZE(curWidth, curHeight);
   ue_MaxFileAmount:=curHeight-10;
  END;
  ue_Access_Rows(ue_MainFN);

  IF CurRecord | (~!_Rows.BegDate & ~!_Rows.EndDate) THEN
   Cond      :=cnd_All;
   CondPrefix:="";
  ELSIF DBD.FieldExist(^ue_MainFN, Date) THEN
   Cond      :=cnd_OnIntvl;
   CondPrefix:="N";
  ELSE
   Cond      :=cnd_All;
   CondPrefix:="";
  END;
  CondContrValName:="Cond, ";
  CondContrBlank:="|" +
                  "|᫮ 뢮 ᥩ 䠩                           " +
                  "|%######################################################%";
  CondContrList:="%<" + CondContrPool + ">" + CondPrefix + "I";
  ue_PrepareFileList(ue_MainFN, 1, ue_prpFileList, ExcludeFileList, 0, ue_FileArr, ue_FileLev, ue_FileAmount, ue_MaxFileAmount);

  ue_FileArr[0]:=1;
  SETEMPTY(ControlsList, FileNameList, VariableList);

  i:=1;
  LOOP
   IF !L_Str.PoolItem(ue_prpFileList, i) THEN
    EXIT;
   END;
   ControlsList:=ControlsList + Str.Slice("%<" + L_Str.Repeat(" ", ue_FileLev[i]-1)+DBD.FileDescription(^(L_Str.PoolItem(ue_prpFileList, i))), 0, 53)+ ">" + L_Str.AnyItem("|N", "|", i=0)+ "E";
   FileNameList:=FileNameList + "|%######################################################%";
   VariableList:=L_Str.AddStrDlm(VariableList, ", ") + L_Str.Var_wIdx("ue_FileArr", i);
   INC(i);
  END; (* LOOP *)
  VariableList:=CondContrValName + VariableList;
  IF ~!FileNameList THEN
   FileNameList:="||롥 ⥫ 䠩  㧪              " + FileNameList;
  END;

  LOOP
   STDFORM(L_StdFrm.sf_Choice( "᭮ 䠩:                                          " +
                              "|%######################################################%" +
                              CondContrBlank +
                              FileNameList,
                              "%<"+ DBD.FileDescription(^ue_MainFN) +">NE"+
                              CondContrList +
                              ControlsList),
           " 㧪  Excel" + L_Str.PoolItem(" | ⥪饩 ", CurRecord),
           ,
           ,
           ue_FileArr[0], ^(VariableList));
   SETEMPTY(ue_FileList);
   i:=0;
   WHILE (i <= HIGH(ue_FileArr)) & ~!ue_FileArr[i] DO
    IF ue_FileArr[i] THEN
     file:=L_Str.PoolItem(ue_prpFileList, i);
     j:=0;
     LinkSelected:=i = 0;
     WHILE ~LinkSelected & (j <= HIGH(ue_FileArr)) & ~!ue_FileArr[j] DO
      LinkSelected:=ue_FileArr[j] & HasGoodLink(L_Str.PoolItem(ue_prpFileList, j), file, j = 0);
      INC(j);
     END;
     IF ~LinkSelected THEN
      L_StdFrm.Choice(" " + file + "  㤥 㦥, "+
                      "|⠪     뫠    ⬥祭 䠩");
     END;
     ue_FileList:=ue_FileList + file + "|";
    END;
    INC(i);
   END;
   IF !ue_FileList THEN
    L_StdFrm.RedInform(" ࠭   䠩  㧪  Excel");
   ELSE
    EXIT;
   END;
  END; (* LOOP *)
 END {ue_SetupDialog};

 PUBLIC PROCEDURE ReplaceIllegalChars(Name: STRING[]): STRING[];
  CONST
   IllegalChars = ".<>?[]:|*/\";
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=0;
  WHILE ~!L_Str.Char(IllegalChars, i) DO
   Name:=L_Str.Replace(Name, L_Str.Char(IllegalChars, i), "_");
   INC(i);
  END; (* WHILE ~!L_Str.Char(IllegalChars, i)...*)
  RETURN L_Str.Replace(Name, "'", '"');
 END {ReplaceIllegalChars};

 PRIVATE PROCEDURE CreateWorkSheetName(FileName: STRING[]): STRING[];
  VAR
   Name: STRING[];
 BEGIN
  Name:=Str.Slice(FileName + "_" + DBD.FileDescription(^FileName), 0, 31);
  RETURN ReplaceIllegalChars(Name);
 END {CreateWorkSheetName};

 PRIVATE PROCEDURE ue_MakeWorkSheetHeader(objWS: OBJECT; FileName: STRING[]);
  VAR
   Comment,
   FieldName,
   FieldDescr,
   FullFieldName: STRING[];
   i,
   k,
   Field,
   Precision,
   FieldType    : NUMERIC[0];
 BEGIN
  objWS.Name:=FileName;
  ws_Init(objWS);
  Field :=0;
  i:=0;
  BlankRow.Clear();
  WHILE DBD.FieldAmount(^FileName) > Field DO
   FieldName:=DBD.FieldName(^FileName, Field);
   FieldDescr:=DBD.FieldDescription(^FileName, ^FieldName);
   IF !FieldDescr THEN
    FieldDescr:=" ᠭ ";
   END;
   Comment:=FieldDescr;
   IF ~!DBD.FieldComment(^FileName, ^FieldName) THEN
    Comment:=Comment + CHR(10) + "(" + DBD.FieldComment(^FileName, ^FieldName) +")";
   END;
   Comment:=L_Str.ExcludeCRinCRLF(Comment);
   k:=0;
   WHILE RANGEOF(^(L_DBD.GetFieldName(FileName, FieldName))) > k DO
    FullFieldName:=L_DBD.GetFullFieldName(FileName, FieldName, k);
    FieldType    :=TYPEOF(^FullFieldName);
    IF    (FieldType = L_General.ft_NUMERIC) THEN
     Precision:=PRECISIONOF(^FullFieldName);
    ELSIF (FieldType = L_General.ft_SCAL) THEN
     IF (ExcelVersion(objWS) >= 14.0) &
        (L_PCS.IsLinkTo_lsStore(FileName, FieldName) | ~!L_PCS.gsv_GetCallProc(FileName, FieldName)) THEN
      FieldType:=L_General.ft_STRING;
      Precision:=0;
     ELSE
      Precision:=Str.Length(MAXOF(^FullFieldName));
     END;
    ELSE
     Precision:=0;
    END;
    ws_AddColumn(objWS, FieldType, Precision, Str.Item(FullFieldName, ".", 1), Comment);
    ADD(Flds, Flds, , FileName, i, FieldType, FieldName, FullFieldName, k);
    INC(i);
    IF SELECT(FwUsr, FwUsr, FieldName) THEN
     ws_AddColumn(objWS, TYPESPEC.Id_String, 0, FieldName + "_" + FwUsr.cooName, FieldDescr + ", ");
    END;
    INC(k);
   END; (* WHILE HIGH(^FullFieldName)>k DO *)
   INC(Field);
  END; (* WHILE DBD.FieldAmount(^FileName)>Field DO *)
 END {ue_MakeWorkSheetHeader};

 PRIVATE CONST
  ws_MaxWS = 10000;

 PRIVATE VAR
  ws_WS          : OBJECT[ws_MaxWS];
  ws_Col,
  ws_Row,
  ws_HeaderStart,                        (*ப 砫  ⠡*)
  ws_HeaderDepth,                        (*㡨 (⢮ ஢)*)
  ws_HeaderLength: NUMERIC[0][ws_MaxWS]; (*⢮  ⠡*)

(*!!!!*)  PRIVATE PROCEDURE ws_InitHeaderData();
 BEGIN
  SETZERO(ws_HeaderStart);
  SETZERO(ws_HeaderDepth);
  SETZERO(ws_HeaderLength);
 END {ws_InitHeaderData};

(*!!!!*) PRIVATE PROCEDURE ws_GetI(objWS: OBJECT; Strong: SCAL[2]): NUMERIC[0];
  VAR
   i: NUMERIC[0];
 BEGIN
  IF ~VAR_IMPL.Get(ws_Col).Initialized[0] THEN
   SETEMPTY(ws_WS);
   SETZERO(ws_Col, ws_Row);
   ws_InitHeaderData();
   IF ~L_General.OBJECTIsInitialized(BlankWS) THEN
    SETEMPTY(BlankWS, BlankRow, BlankCells);
   END;
  END;
  i:=0;
  WHILE ~!ws_WS[i] DO
   IF ws_WS[i] = objWS THEN
    RETURN i;
   END;
   INC(i);
  END;
  L_General.CallUs(~Strong);
  RETURN i;
 END {ws_GetI};

(*!!!!*) PUBLIC PROCEDURE ws_GetCurPos(objWS: OBJECT; VAR row, column: NUMERIC[0]);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 1);
  row   :=ws_Row[i];
  column:=ws_Col[i];
 END {ws_GetCurPos};

(*!!!!*) PUBLIC PROCEDURE ws_SetCurPos(objWS: OBJECT; row, column: NUMERIC[0]);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 1);
  ws_Row[i]:=row;
  ws_Col[i]:=column;
 END {ws_SetCurPos};

(*!!!!*) PUBLIC PROCEDURE ws_Wr(objWS: OBJECT; VAR Value: VARIANT);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 1);
  IF ~!Value THEN
   objWS.Cells(ws_Row[i], ws_Col[i]).Value:=Value;
  END;
  INC(ws_Col[i]);
 END {ws_Wr};

(*!!!!*) PUBLIC PROCEDURE ws_Init(objWS: OBJECT);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 0);
  ws_WS[i]:=objWS;
  ws_Col[i]:=1;
  ws_Row[i]:=1;

  ws_HeaderStart[i] := 0;
  ws_HeaderDepth[i] := 1;
  ws_HeaderLength[i] := 0;
 END {ws_Init};

(*!!!!*) PUBLIC PROCEDURE ws_AddColumn(objWS: OBJECT; FieldType, Precision: NUMERIC[0]; Name, Comment: STRING[]);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 1);
  AddColumnFromRow(objWS, ws_Row[i], ws_Col[i], FieldType, Precision, Name, Comment);
  INC(ws_Col[i]);
  IF ws_Col[i] = BlankCellAmount THEN
   L_General.Error(" Excel- ᫨誮 讥 ⢮ ⮫殢 ( " + L_Str.IntStr(HIGH(BlankCells)+1) + ")." +
                   L_StdFrm.HR()+
                   "᫨  ⢨⥫쭮 ॡ 㧨  Excel  䠩,|" +
                   "  ");
  END;
 END {ws_AddColumn};

(*!!!!*) PUBLIC PROCEDURE ws_WrNum(objWS: OBJECT; Num: FLOAT);
 BEGIN
  ws_Wr(objWS, Num);
 END {ws_WrNum};

(*!!!!*) PUBLIC PROCEDURE ws_WrStr(objWS: OBJECT; Str: STRING[]);
 BEGIN
  Str:=L_Str.ExcludeCRinCRLF(Str);
  ws_Wr(objWS, Str);
 END {ws_WrStr};

(*!!!!*) PUBLIC PROCEDURE ws_WrScl(objWS: OBJECT; Scl: STRING[]);
 BEGIN
  ws_Wr(objWS, Scl);
 END {ws_WrScl};

(*!!!!*) PUBLIC PROCEDURE ws_WrDat(objWS: OBJECT; D: DATE);
 BEGIN
  ws_Wr(objWS, D);
 END {ws_WrDat};

(*  0101345
 PUBLIC PROCEDURE ws_WrTim(objWS: OBJECT; T: TIME);
  VAR
   t: STRING[];
 BEGIN
  t:=FLDTOSTR(T, TimeFormat());
  ws_Wr(objWS, t);
 END {ws_WrTim};
*)

(*!!!!*) PUBLIC PROCEDURE ws_SkipCols(objWS: OBJECT; Num: NUMERIC[0]);
  VAR
   i   : NUMERIC[0];
   Cell: OBJECT;
 BEGIN
  i:=ws_GetI(objWS, 1);
  INC(ws_Col[i], Num);
 END {ws_SkipCols};

(*!!!!*) PUBLIC PROCEDURE ws_WrFormula(objWS: OBJECT; _Formula: STRING[]);
  VAR
   i   : NUMERIC[0];
   Cell: OBJECT;
 BEGIN
  i:=ws_GetI(objWS, 1);
  Cell:=objWS.Cells(ws_Row[i], ws_Col[i]);
  Cell.FormulaR1C1:=_Formula;
  INC(ws_Col[i]);
 END {ws_WrFormula};

(*!!!!*) PUBLIC PROCEDURE ws_TermLine(objWS: OBJECT);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 1);
  INC(ws_Row[i]);
  ws_Col[i]:=1;
 END {ws_TermLine};

(*!!!!*) PUBLIC PROCEDURE ws_DeInit(objWS: OBJECT);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 1);
  ws_Row[i]:=0;
  ws_Col[i]:=0;
 END {ws_DeInit};

 PUBLIC PROCEDURE AcceptableFileForUpload(FileName, ExcludeFileList: STRING[]): SCAL[2];
  VAR
   ExcludeFile: STRING[];
   i          : NUMERIC[0];
 BEGIN
  IF ~!ExcludeFileList THEN
   i:=0;
   LOOP
    ExcludeFile:=Str.Item(ExcludeFileList, ";", i);
    IF !ExcludeFile THEN
     EXIT;
    END;
    IF L_Str.IsEqStrI(FileName, ExcludeFile) THEN
     RETURN 0;
    END;
    INC(i);
   END; (* LOOP *)
  END;
  RETURN 1;
 END {AcceptableFileForUpload};

 PUBLIC PROCEDURE ue_Access_Rows(FileName: STRING[]);
 BEGIN
  IF ~SELECT(_Rows, _Rows, FileName) THEN
   _Rows.FileName:=FileName;
   _Rows.Row     :=0;
   L_DBD.CollateValidityFieldNames(FileName, _Rows.BegDate, _Rows.EndDate);
   INSERT(_Rows);
  END;
 END {ue_Access_Rows};

 PRIVATE PROCEDURE CurPositionId(FileName: STRING[]): STRING[];
 BEGIN
  RETURN L_DBD.GetPrimaryIndexValue(FileName);
 END {CurPositionId};

 PUBLIC PROCEDURE ue_MarkRecord(FileName: STRING[]);
 BEGIN
  IF OCCURE(^FileName) THEN
   IF ~L_DBD.lr_Label(FileName) THEN
    L_DBD.lr_Remember(FileName);
    ue_Access_Rows(FileName);
    INC(_Rows.Row);
    REPLACE(_Rows);
   END;
  END;
 END {ue_MarkRecord};

 PRIVATE PROCEDURE psf_Cond_All(FileName: STRING[]): SCAL[2];
 BEGIN
  RETURN 1;
 END {psf_Cond_All};

 PRIVATE PROCEDURE psf_Cond_OnDate(FileName: STRING[]): SCAL[2];
 BEGIN
  RETURN L_DBD.xRecordIsActive(Memo.ud_cd(), FileName, _Rows.BegDate, _Rows.EndDate);
 END {psf_Cond_OnDate};

 PRIVATE PROCEDURE psf_Cond_OnIntvl(FileName: STRING[]): SCAL[2];
 BEGIN
  IF ~!_Rows.BegDate & ~!_Rows.EndDate THEN
   RETURN L_DBD.xRecordIsActiveInIntvl(Memo.ud_fr(), Memo.ud_ut(), FileName, _Rows.BegDate, _Rows.EndDate);
  ELSIF DBD.FieldExist(^FileName, Date) THEN
   RETURN L_DatTim.IsActive_Inc(^FileName.Date, Memo.ud_fr(), Memo.ud_ut());
  ELSE
   RETURN 1;
  END;
 END {psf_Cond_OnIntvl};

 PRIVATE PROCEDURE psf_Cond_FromWDO(FileName: STRING[]): SCAL[2];
 BEGIN
  IF FileName = ue_MainFN THEN
   RETURN !ue_Filter | ^(ue_Filter);
  ELSE
   RETURN 1;
  END;
 END {psf_Cond_FromWDO};

 PUBLIC PROCEDURE ue_PrepareSingleFile(curRecord      : SCAL[2];
                                       Cond           : PROC;
                                       FileName,
                                       ExcludeFileList,
                                       ue_FileList    : STRING[]);
  VAR
   i, a,
   LinkKind: NUMERIC[0];
   Coo,
   LinkName: STRING[];
   NoWhile : SCAL[2];
   oPrompt : PROMPT_LINE;
 BEGIN
  ue_Access_Rows(FileName);
  IF ~L_DBD.lr_Label(FileName) & Cond(FileName) THEN
   ue_MarkRecord(FileName);
   PROMPT(L_Str.IntStr(_Rows.Row));

   i:=0;
   a:=DBD.LinkAmount(^FileName);
   WHILE i < a DO
    LinkName:=DBD.LinkName(^FileName, i);
    LinkKind:=DBD.LinkKind(^FileName, ^LinkName);
    Coo     :=DBD.LinkCooFileName(^FileName, ^LinkName);

    IF ((LinkKind = L_General.lk_USER) |
        (LinkKind = L_General.lk_LEND) |
        (LinkKind = L_General.lk_REVERSE_HISTORY)) & (L_Str.PositionInPool(ue_FileList, Coo) # 0) |
       ((LinkKind = L_General.lk_BORROW) |
        (LinkKind = L_General.lk_HISTORY)) & (L_Str.PositionInPool(ue_FileList, FileName) = 0) THEN
     IF AcceptableFileForUpload(Coo, ExcludeFileList) THEN
      IF ~!L_Str.PositionInPool(ue_FileList, Coo) THEN
       IF LinkKind = L_General.lk_REVERSE_HISTORY THEN
        HISTCONF(Memo.ud_cd(), ^FileName, ^Coo);
        NoWhile:=0;
       ELSE
        TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
         CONFORM(^FileName, ^Coo);
        EXCEPT
        ON Exc: Exc_RunTime DO
         IF Exc.ErrorCode = 62 THEN
          L_DBD.SelectCooFile(FileName, LinkName);
         ELSE
          RAISE();
         END;
        ELSE
         RAISE();
        END;
        NoWhile:=LinkKind # L_General.lk_LEND;
       END;
 
       SAVE(^FileName);
       IF NoWhile THEN
        IF OCCURE(^Coo) THEN
         ue_PrepareSingleFile(curRecord, Cond, Coo, ExcludeFileList, ue_FileList);
        END;
       ELSE
        oPrompt:=PROMPT_LINE.Create("⮢ " + Coo + " (" + L_Str.IntStr(AMOUNT(^Coo)) + " ᥩ)");
        FIRST(^Coo);
        WHILE OCCURE(^Coo) DO
         ue_PrepareSingleFile(curRecord, Cond, Coo, ExcludeFileList, ue_FileList);
         NEXT(^Coo);
        END;
        SETEMPTY(oPrompt);
       END;
       RESTORE(^FileName);
      END; (* IF ~!L_Str.PositionInPool ...*)
     END; (* IF AcceptableFileForUpload()...*)
    END; (* IF ((LinkKind = ...*)
    INC(i);
   END; (* WHILE i < a...*)
  END;
 END {ue_PrepareSingleFile};

 PRIVATE PROCEDURE ue_CondProc(Cond: tCond): PROC;
 BEGIN
  IF Cond = cnd_OnDate THEN
   RETURN PROCREF(psf_Cond_OnDate);
  ELSIF Cond = cnd_OnIntvl THEN
   RETURN PROCREF(psf_Cond_OnIntvl);
  ELSIF Cond = cnd_All THEN
   RETURN PROCREF(psf_Cond_All);
  ELSIF Cond = cnd_FromWDO THEN
   RETURN PROCREF(psf_Cond_FromWDO);
  ELSE
   L_General.CallUs(0);
  END;
 END {ue_CondProc};

 PRIVATE PROCEDURE PrepareExcelFileName(FileName: STRING[]): STRING[];
 BEGIN
  FileName:=      FileName +
            "_" + DBD.FileDescription(^FileName) +
            "_" + FLDTOSTR(SYSDATE(), "yND", 8)  +
            "_" + FLDTOSTR(SYSTIMESTAMP(), "HMS",  6);
  RETURN FIO.BuildPath(ReplaceIllegalChars(FileName), , FIO.AsbSpecialFolders("ASBTemp"));
 END {PrepareExcelFileName};

 PRIVATE PROCEDURE SetRangeVal(R: OBJECT; VAR Arr: VARIANT);
 BEGIN
  R.Value:=Arr;
 END {SetRangeVal};

 PRIVATE PROCEDURE ue_SingleRecord(objWS: OBJECT; row: NUMERIC[0]);
  VAR
   i,
   column: NUMERIC[0];
   Value,
   tmpStr: STRING[];
(*
   BlankStr: STRING[][BlankCellAmount+1];
*)
 BEGIN
(*
  SETEMPTY(BlankStr);
*)
  column:=1;
  PROMPT("㧪 " + _Rows.FileName + " " + L_Str.IntStr(row-1) + "  " + L_Str.IntStr(_Rows.Row));

  SELECT(Flds, Flds, , _Rows.FileName);
  WHILE OCCURE(Flds) DO
   IF ~!Flds.FullName THEN
    IF    Flds.fType = L_General.ft_TIME THEN
     BlankCells[column].Value:=FLDTOSTR(^(Flds.FullName), TimeFormat());
(*
     BlankStr[column]:=FLDTOSTR(^(Flds.FullName), TimeFormat());
*)
    ELSIF Flds.fType = L_General.ft_SCAL THEN
     BlankCells[column].Value:="'"+L_Str.ExcludeCRinCRLF(L_PCS.GetScalValueInterpretation(_Rows.FileName, Flds.Field, Flds.ArrIdx));
(*
     BlankStr[column]:="'"+L_Str.ExcludeCRinCRLF(L_PCS.GetScalValueInterpretation(_Rows.FileName, Flds.Field, Flds.ArrIdx));
*)
    ELSIF Flds.fType = L_General.ft_STRING THEN
     BlankCells[column].Value:="'" + L_Str.ExcludeCRinCRLF(^(Flds.FullName));
(*
     BlankStr[column]:="'" + L_Str.ExcludeCRinCRLF(^(Flds.FullName));
*)
    ELSE
     BlankCells[column].Value:=^(Flds.FullName);
(*
     BlankStr[column]:=FLDTOSTR(^(Flds.FullName));
*)
    END;
   END; (* IF ~!Flds.FullName...*)
   INC(column);
   IF SELECT(FwUsr, FwUsr, Flds.Field) THEN
    SAVE(^_Rows.FileName);
    SAVE(^FwUsr.cooFile);
    L_DBD.SelectCooFile(_Rows.FileName, FwUsr.Link);
    IF OCCURE(^FwUsr.cooFile) THEN
     BlankCells[column].Value:="'" + L_Str.ExcludeCRinCRLF(^(FwUsr.cooFile + "." + FwUsr.cooName));
(*
     BlankStr[column]:="'" + L_Str.ExcludeCRinCRLF(^(FwUsr.cooFile + "." + FwUsr.cooName));
*)
    ELSE
     BlankCells[column].Value:=CreateVARIANT(Empty);
    END;
    INC(column);
    RESTORE(^FwUsr.cooFile);
    RESTORE(^_Rows.FileName);
   END;
   NEXT(Flds);
  END; (* WHILE OCCURE(Flds) ... *)
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   objWS.Rows(row).Value:=BlankRow.Value;
(*
   objWS.Rows(row).Value:=CreateSAFEARRAY(String, BlankStr, 2 (* 㬥 2*), 1, BlankCellAmount+1);
*)
  EXCEPT
  ON Exc: Exc_Object DO
   i:=objWS.UsedRange.Columns.Count;
   WHILE i > 0  DO
    TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
     objWS.Cells(row, i).Value:=BlankCells[i].Value;
    EXCEPT
    ON Exc: Exc_Object DO
     objWS.Cells(row, i).Value:="'"+BlankCells[i].Value;
    END;
    DEC(i);
   END;
  END; (* TRY *)
 END {ue_SingleRecord};

 PRIVATE PROCEDURE ue_SingleFile(objWB: OBJECT);
  VAR
   row     : NUMERIC[0];
   objWS   : OBJECT;
   Link    : LINK;
   Links   : TABLELINKS;
   cooTable: TABLE;
   ctf_Name: STRING[];
   oc      : SCAL[2];
 BEGIN
  Links:=TABLE.Get(_Rows.FileName).Links;
  FOR_EACH Link IN Links DO
   IF (Link.Kind = LINK.Kind_User) | (Link.Kind = LINK.Kind_Borrow) | (Link.Kind = LINK.Kind_History) THEN
    PROMPT(Link.Name);
    cooTable:=Link.CooIndex.Table;
    ctf_Name:=L_PCS.GetNameFieldName(cooTable.Name);
    IF ~!cooTable.Fields(ctf_Name) THEN
     FwUsr.Field  :=Link.Index.Fields.Item(0).Name;
     FwUsr.Link   :=Link.Name;
     FwUsr.cooFile:=cooTable.Name;
     FwUsr.cooName:=ctf_Name;
     INSERT(FwUsr);
    END;
   END;
  END; (* FOR_EACH *)

  objWS:=objWB.Worksheets.Add(, objWB.Worksheets(objWB.Worksheets.Count));
  row:=1;
  ue_MakeWorkSheetHeader(objWS, _Rows.FileName);
  oc:=L_DBD.lr_PosToFirst(_Rows.FileName);
  WHILE oc DO
   INC(row);
   ue_SingleRecord(objWS, row);
   oc:=L_DBD.lr_PosToNext(_Rows.FileName);
  END; (* WHILE oc *)

  SELECT(FwUsr, FwUsr);
  REMSEL(FwUsr);
 END {ue_SingleFile};

 PUBLIC PROCEDURE ue_PostProcessing(objXL, objWB: OBJECT);
  VAR
   i    : NUMERIC[0];
   objWS: OBJECT;
 BEGIN
  i:=objWB.Worksheets.Count;
  WHILE i >= 1 DO
   objWS:=objWB.Worksheets(i);
   L_General.CallUs(SELECT(_Rows, _Rows, objWS.Name));
   objWS.Name:=CreateWorkSheetName(_Rows.FileName);
   objWS.Activate();
   objWS.Cells(2, DBD.IndexFieldAmount(^(_Rows.FileName), ^(DBD.IndexName(^(_Rows.FileName), 0))) + 1).Select();
   objXL.ActiveWindow.FreezePanes:=1;
   objWS.Parent.Names.Add(_Rows.FileName, objWS.Range(objWS.Cells(2, 1),
                                                      objWS.Cells(objWS.UsedRange.Rows.Count, objWS.UsedRange.Columns.Count)));
   objWS.UsedRange.AutoFilter();
   objWS.Rows("1:1").RowHeight:=17; (* ⮡ ⪠   ᫮﫠 䨫஬*)
   objWS.UsedRange.EntireColumn.AutoFit();
   DEC(i);
  END;
 END {ue_PostProcessing};

 PUBLIC PROCEDURE ue_Create_Rows();
 BEGIN
  L_AFM.MakeIndex("_Rows", "FileName: STRING[70]; Row: NUMERIC[0]; BegDate: STRING[8]; EndDate: STRING[8]");
  L_AFM.MakeIndex("FwUsr", "Field: STRING[32]; Link: STRING[32]; cooFile: STRING[32]; cooName: STRING[32]");
  L_AFM.MakeIndex("Flds", "File: STRING[16]; Idx: NUMERIC[0]; fType: NUMERIC[0]; Field: STRING[32]; FullName: STRING[48]; ArrIdx: NUMERIC[0]");
 END {ue_Create_Rows};

 PUBLIC PROCEDURE ue_Purge_Rows();
 BEGIN
  L_AFM.AF_PurgeTemp("_Rows");
  L_AFM.AF_PurgeTemp("FwUsr");
  L_AFM.AF_PurgeTemp("Flds");
 END {ue_Purge_Rows};

 PUBLIC PROCEDURE ue_MarkRecords(curRecord       : SCAL[2];
                                 Cond            : tCond;
                                 ExcludeFileList : STRING[];
                                 VAR ue_FileList : STRING[]);
  VAR
   Dummy     : STRING[];
   SortByDate: SCAL[2];
   CondProc  : PROC;
   CurZone   : FORM_ZONE;
 BEGIN
  CondProc:=ue_CondProc(Cond);
  IF curRecord THEN
   L_General.Assert(CondProc(ue_MainFN), "   㤮⢮ ࠭ ᫮");
   ue_PrepareSingleFile(curRecord, CondProc, ue_MainFN, ExcludeFileList, ue_FileList);
   ue_FileList:=ue_MainFN;
   SELECT(_Rows, _Rows);
   WHILE OCCURE(_Rows) DO
    IF _Rows.FileName # ue_MainFN THEN
     ue_FileList:=ue_FileList+"|"+_Rows.FileName;
    END;
    NEXT(_Rows);
   END; (* WHILE OCCURE(_Rows) ... *)
  ELSE
   IF Cond = cnd_FromWDO THEN
    SortByDate:=0;
    CurZone:=RTI.Get().CurZone;
    GE_WWD_CURSOR(RTI.Get().CurWindow.Name, ^(RTI.Get().CurWindow.MasterCursor.TABLE.Name));
    IF ~!CurZone.ParentZone THEN
     GE_WZD_CURSOR(CurZone.ParentZone.Window.Name,
                   CurZone.ParentZone.Number,
                   ^(CurZone.ParentZone.MasterTable.Name));
    END;
    GE_WZD_CURSOR(RTI.Get().CurWindow.Name, CurZone.Number, ^ue_MainFN);
    ue_Filter:=L_WDO.GetActiveFiltersCondtion(CurZone);
    FIRST(^ue_MainFN);
   ELSIF (Cond = cnd_OnIntvl) & CanSortByDate(ue_MainFN) THEN
    SortByDate:=1;
    SELECT(^ue_MainFN, Date);
    SRCINSEL(^ue_MainFN, GE, , Memo.ud_fr());
   ELSE
    SortByDate:=0;
    SELECT(^ue_MainFN, ^(DBD.IndexName(^ue_MainFN, 0)));
   END;
   APP.Get().NewPrompt("⮢ " + ue_MainFN + " (" + L_Str.IntStr(AMOUNT(^ue_MainFN)) + " ᥩ)");
   Dummy:="OCCURE("+ue_MainFN+") &"+
          "(~SortByDate | ("+ue_MainFN+".Date <= Memo.ud_ut()))"; (* 0063890 06.05.2008 15:57:16 POLAK\Bor - ஡   ^  ᫮ ᫮ *)
   WHILE ^Dummy DO
    ue_PrepareSingleFile(curRecord, CondProc, ue_MainFN, ExcludeFileList, ue_FileList);
    NEXT(^ue_MainFN);
   END; (* WHILE OCCURE(^ue_MainFN)...*)
  END;
 END {ue_MarkRecords};

 PRIVATE PROCEDURE ue_Prepare(    curFile,
                                  curRecord      : SCAL[2];
                                  ExcludeFileList: STRING[];
                              VAR ue_FileList    : STRING[]);
  VAR
   Cond    : tCond;
   FileName: STRING[];
 BEGIN
  IF curRecord THEN
   Cond    :=cnd_All;
   FileName:=GE_CURFILE();
   L_General.Assert(~!FileName, " ⠭ ⥪騩 䠩");
   L_General.Assert(ACCESS(^(FileName)), " ⠭ ⥪ ");
  END;
  ue_SetupDialog(curFile, curRecord, ue_FileList, ExcludeFileList, Cond);
  ue_MarkRecords(curRecord, Cond, ExcludeFileList, ue_FileList);
 END {ue_Prepare};

 PUBLIC PROCEDURE ue_DoUpload(ue_FileList: STRING[]);
  VAR
   objXL,
   objWB             : OBJECT;
   AlreadyShowWarning: SCAL[2];
   FileName          : STRING[];
   i,
   T                 : NUMERIC[0];
 BEGIN
  SETEMPTY(BlankWS, BlankRow, BlankCells);
  SETEMPTY(objXL, objWB);
  TRY
   objXL:=CreateOBJECT("Excel.Application");
   objXL.Visible       :=xlFalse;
   objXL.EnableEvents  :=xlFalse;
   objXL.ScreenUpdating:=xlFalse;
   objXL.Workbooks.Open(FIO.MakePath(L_MSWStp.AutoparkLNKName, "",
                                     FIO.MakePath("XLStart", "", objXL.Path)), , xlTrue);
   objWB  :=NewWorkbook(objXL, 1);
   BlankWS:=objWB.WorkSheets(1);
   i:=1;
   WHILE i <= BlankCellAmount DO
    BlankCells[i]:=BlankWS.Cells(1, i);
    INC(i);
   END;
   BlankRow:=BlankWS.Rows(1);

   i:=0;
   FileName:=L_Str.PoolItem(ue_FileList, i);
   WHILE ~!FileName DO
    IF SELECT(_Rows, _Rows, FileName) THEN
     AlreadyShowWarning:=0;
     IF    _Rows.Row > 65534 THEN
      L_General.Error(" 䠩 " + _Rows.FileName + " 祭  㧪 " +
                      L_Str.IntStr(_Rows.Row) + " ᥩ." +
                      "|MS Excel  ন ⠪ ⢮ ப  ");
     ELSIF (_Rows.Row > 10000) & ~AlreadyShowWarning THEN
      T:=10;
      STDFORM("| 䠩 " + _Rows.FileName + " 祭  㧪 " +
               L_Str.IntStr(_Rows.Row) + " ᥩ." +
              "|㧪  MS Excel  த⥫쭮 ६." +
              "|" +
              "| %<    ⬥    >X     %<   㧨 %T >DB",
              " 㧪 䠩  Excel ", STOP, -1, T);
      AlreadyShowWarning:=1;
     END;
    END;
    INC(i);
    FileName:=L_Str.PoolItem(ue_FileList, i);
   END;

   i:=0;
   FileName:=L_Str.PoolItem(ue_FileList, i);
   WHILE ~!FileName DO
    IF SELECT(_Rows, _Rows, FileName) THEN
     ue_SingleFile(objWB);
    END;
    INC(i);
    FileName:=L_Str.PoolItem(ue_FileList, i);
   END;

   IF objWB.WorkSheets.Count > 1 THEN
    objXL.DisplayAlerts:=xlFalse;
    BlankWS.Delete();
    objXL.DisplayAlerts:=xlTrue;
 
    ue_PostProcessing(objXL, objWB);
   END;

   objWB.SaveAs(PrepareExcelFileName(ue_MainFN));
   objXL.ScreenUpdating:=xlTrue;
   objXL.EnableEvents  :=xlTrue;
   objXL.Visible       :=xlTrue;
  EXCEPT
  ON Exc: Exc_Root DO
   CloseExcelAfterCrash(objXL, objWB);
   RAISE();
  END; (* TRY *)
 END {ue_DoUpload};

 PUBLIC PROCEDURE UploadFileToExcel(curFile, curRecord: SCAL[2]; ExcludeFileList: STRING[]);
  VAR
   ue_FileList: STRING[];
 BEGIN
  ue_Create_Rows();
  ue_Prepare(curFile, curRecord, ExcludeFileList, ue_FileList);
  ue_DoUpload(ue_FileList);
  ue_Purge_Rows();
 END {UploadFileToExcel};

 PRIVATE CONST
  cExcelRepHeaderBorderWidth = 2; (*⮫騭 ࠬ  ⠡*)
  cExcelRepBodyBorderWidth   = 1; (*⮫騭 ࠬ ⠡*)

(*!!!!*) PUBLIC PROCEDURE ws_CurrCell(objWS: OBJECT): OBJECT;
  VAR
   ExcelCurRow,                (*⥪ ப  뢮  ⠡ *)
   ExcelCurColumn: NUMERIC[0]; (*⥪騩 ⮫*)
 BEGIN
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  RETURN objWS.Cells(ExcelCurRow, ExcelCurColumn);
 END {ws_CurrCell};

(*!!!!*) PUBLIC PROCEDURE ws_SetFontSize(objWS: OBJECT; FontSize: NUMERIC[0]);
  VAR
   Cell: OBJECT;
 BEGIN
  Cell:=ws_CurrCell(objWS);
  Cell.Font.Size:=FontSize;
 END {ws_SetFontSize};

(*!!!!*) PRIVATE PROCEDURE ws_DecorCurCell(objWS: OBJECT; Height, Width, BorderWidth: NUMERIC[0]; WrapText: SCAL[2]);
  VAR
   CurCell        : OBJECT;
   ExcelCurRow,
   ExcelCurColumn,
   i              : NUMERIC[0];
 BEGIN
  CurCell:=ws_CurrCell(objWS);
  CurCell.VerticalAlignment  := 2;    (*ࠢ ⥪  祩  ⨪  業*)
  CurCell.HorizontalAlignment:= 3;    (*ࠢ ⥪  祩  ਧ⠫  業*)
  IF WrapText THEN
   CurCell.WrapText:=xlTrue;
  ELSE
   CurCell.WrapText:=xlFalse;
  END;
  IF Width > 0 THEN
   CurCell.ColumnWidth:=Width;
  ELSE
   CurCell.Columns.AutoFit();
  END;
  IF Height > 0 THEN
   CurCell.RowHeight:=Height;
  ELSE
   CurCell.Rows.AutoFit();
  END;
  CurCell.Borders.Weight:=2;
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  i:=ws_GetI(objWS, 0);
  IF ws_HeaderStart[i] = 0 THEN
   ws_HeaderStart[i]:=ExcelCurRow;
  END;
 END {DecorCell};

(*!!!!*) PRIVATE PROCEDURE ws_MergeUnderItem(objWS: OBJECT);
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
   i,
   j             : NUMERIC[0];
   S             : STRING[];
   range         : OBJECT;
 BEGIN
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  i:=ws_GetI(objWS, 0);
  IF (ws_HeaderStart[i] < ExcelCurRow) THEN
   S:=objWS.Cells(ExcelCurRow - 1, ExcelCurColumn - 1).Value;
   IF !S THEN
    DEC(ExcelCurRow);
    DEC(ExcelCurColumn);
    WHILE ws_HeaderStart[i] <= ExcelCurRow DO
     range:=objWS.Range(objWS.Cells(ExcelCurRow, ExcelCurColumn-1), objWS.Cells(ExcelCurRow, ExcelCurColumn));
     range.Merge();
     range.Borders.Weight:=2;
     DEC(ExcelCurRow);
    END;
   END;
  END;
 END {ws_MergeUnderItem};

(*!!!!*) PUBLIC PROCEDURE ws_StartHeader(objWS: OBJECT);
  VAR
   i,
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 0);
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  ws_HeaderStart[i]:=ExcelCurRow;
  ws_HeaderDepth[i]:= 1;
 END {ws_StartHeader};

(*!!!!*) PUBLIC PROCEDURE ws_StopHeader(objWS: OBJECT);
  VAR
   ExcelCurRow,
   ExcelCurColumn,
   i,
   j,
   k              : NUMERIC[0];
   S              : STRING[];
   range          : OBJECT;
 BEGIN
  i:=ws_GetI(objWS, 0);
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  ws_HeaderLength[i]:=ExcelCurColumn - 1;
  k:=ws_HeaderLength[i];
  WHILE k > 0 DO
   j:=ws_HeaderDepth[i];
   WHILE j > 1 DO
    S:=objWS.Cells(ws_HeaderStart[i] + j - 1, k).Value;
    objWS.Cells(ws_HeaderStart[i], k).Borders.Weight:=2;
    IF !S THEN
     range:=objWS.Range(objWS.Cells(ws_HeaderStart[i] + j - 1, k), objWS.Cells(ws_HeaderStart[i] + j - 2, k));
     range.Merge();
     range.Borders.Weight:=2;
     DEC(j);
    ELSE
     j:=1;
    END;
   END; (* WHILE ... *)
   DEC(k);
  END;
  ExcelCurRow   :=ExcelCurRow + ws_HeaderDepth[i];
  ExcelCurColumn:=1;
  ws_SetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
 END {ws_StopHeader};

(*!!!!*) PUBLIC PROCEDURE ws_AddHeaderItem(objWS: OBJECT; Text: STRING[]; Size, Height: NUMERIC[0]);
 BEGIN
  ws_DecorCurCell(objWS, Height, Size, cExcelRepHeaderBorderWidth, 1);
  ws_WrStr(objWS, Text);
  ws_MergeUnderItem(objWS);
 END {wsAddHeaderItem};

(*!!!!*) PUBLIC PROCEDURE ws_AddHeaderSubItem(objWS: OBJECT);
  VAR
   i,
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  INC(ExcelCurRow);
  DEC(ExcelCurColumn);
  i:=ws_GetI(objWS, 0);
  IF ExcelCurRow > ws_HeaderStart[i] + ws_HeaderDepth[i] - 1 THEN
   INC(ws_HeaderDepth[i]);
  END;
  DEC(ws_HeaderLength[i]);
  ws_SetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
 END {ws_AddHeaderSubItem};

(*!!!!*) PUBLIC PROCEDURE ws_HeaderReturnFromSubItem(objWS: OBJECT);
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  ws_GetCurPos(objWS, ExcelCurRow,     ExcelCurColumn);
  ws_SetCurPos(objWS, ExcelCurRow - 1, ExcelCurColumn);
 END {ws_HeaderReturnFromSubItem};

(*!!!!*) PUBLIC PROCEDURE ws_FreezeHeader(objXL, objWB, objWS: OBJECT);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=ws_GetI(objWS, 0);
  objWS.Activate();
  objWS.Range(objWS.Cells(ws_HeaderStart[i] + ws_HeaderDepth[i], 1),
              objWS.Cells(ws_HeaderStart[i] + ws_HeaderDepth[i], 2)).Select();
       (*  㪠뢠 易⥫쭮,  ⮡ Excel ࠢ쭮 ࠢ *)
  objXL.ActiveWindow.FreezePanes:=xlTrue;
 END {ws_FreezeHeader};

 (*६  祩   ⠡  ⥪ 樨  ࠬ*)
(*!!!!*) PUBLIC PROCEDURE ws_PaintGrid(objWS: OBJECT; BorderWidth: NUMERIC[0]; AutoFilter: SCAL[2]);
  VAR
   i,
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  ws_GetCurPos(objWS, ExcelCurRow, ExcelCurColumn);
  i:=ws_GetI(objWS, 0);
  objWS.Range(objWS.Cells(ws_HeaderStart[i] + ws_HeaderDepth[i], 1),
              objWS.Cells(ExcelCurRow-1, ws_HeaderLength[i])).Borders.Weight:=BorderWidth;
  IF AutoFilter THEN
   objWS.Range(objWS.Cells(ws_HeaderStart[i] + ws_HeaderDepth[i] - 1, 1),
               objWS.Cells(ws_HeaderStart[i] + ws_HeaderDepth[i] - 1, ws_HeaderLength[i])).AutoFilter();
  END;
 END {ws_PaintGrid};

(*!!!!*) PUBLIC PROCEDURE ws_GreedRep(objWS: OBJECT; BorderWidth: NUMERIC[0]; AutoFilter: SCAL[2]);
 BEGIN
  ws_PaintGrid(objWS, BorderWidth, AutoFilter);
 END {ws_GreedRep};

 PUBLIC PROCEDURE extBuildRep(MakeRepOnWS: PROC; Path, RepName: STRING[]; ToFile, OverwriteFile: SCAL[2]): STRING[];
  VAR
   objXL,
   objWB : OBJECT;
   FullFileName: STRING[];
 BEGIN
  objXL:=CreateExcel(1);
  objXL.StandardFontSize:=12;
  IF !Path THEN
   Path:=FIO.BuildPath("REP\$$", , FIO.AsbSpecialFolders("ClientProjectRoot"))
  END;
  FullFileName:=FIO.BuildPath(ReplaceIllegalChars(RepName), , Path);

  TRY
   objXL.ScreenUpdating:=xlFalse;
   objWB:=NewWorkbook(objXL, 1);

   IF ~VAR_IMPL.Get(ws_Col).Initialized[0] THEN
    SETEMPTY(ws_WS);
    SETZERO(ws_Col, ws_Row);
    ws_InitHeaderData();
    IF ~L_General.OBJECTIsInitialized(BlankWS) THEN
     SETEMPTY(BlankWS, BlankRow, BlankCells);
    END;
   END;

   MakeRepOnWS(objWB);

   TRY
    IF OverwriteFile THEN
     objXL.DisplayAlerts:=xlFalse;
    END;
    objWB.SaveAs(FullFileName);
    FullFileName:=objWB.FullName;
   EXCEPT
   ELSE
    IF ToFile THEN
     RAISE();
    END;
   END;
   IF ToFile THEN
    CloseExcel(objXL, objWB);
   ELSE
    objXL.Visible       :=xlTrue;
    objXL.ScreenUpdating:=xlTrue;
   END;
  EXCEPT
  ELSE
   CloseExcelAfterCrash(objXL, objWB);
   RAISE();
  END; (* TRY *)
  RETURN FullFileName;
 END {extBuildRep};

 PUBLIC PROCEDURE BuildRep(MakeRepOnWS: PROC; Path, RepName: STRING[]; ToFile, OverwriteFile: SCAL[2]);
  VAR
   Dummy: STRING[];
 BEGIN
  Dummy:=extBuildRep(MakeRepOnWS, Path, RepName, ToFile, OverwriteFile);
 END {BuildRep};

(*!*) PUBLIC PROCEDURE MakeRep(MakeRepOnWS: PROC; RepName: STRING[]);
 BEGIN
  BuildRep(MakeRepOnWS, FIO.MakePath("TXT", , FIO.AsbSpecialFolders("ClientProjectRoot")), RepName, 0, 0);
 END {MakeRep};

(*  䥩 *)

 PRIVATE PROCEDURE wsn_Init(ws_Idx: NUMERIC[0]);
 BEGIN
  ws_CurIdx                 :=ws_Idx;
  ws_Col[ws_CurIdx]         :=1;
  ws_Row[ws_CurIdx]         :=1;

  ws_HeaderStart[ws_CurIdx] :=0;
  ws_HeaderDepth[ws_CurIdx] :=1;
  ws_HeaderLength[ws_CurIdx]:=0;
 END {wsn_Init};

 PUBLIC PROCEDURE wsn_SetCurIdx(ws_Idx: NUMERIC[0]);
 BEGIN
  ws_CurIdx:=ws_Idx;
 END {wsn_SetCurIdx};

 PUBLIC PROCEDURE ws_Create_Int(objWB: OBJECT; ws_Idx: NUMERIC[0]; ws_Name: STRING[]);
 BEGIN
  ws_Name:=Str.Slice(ws_Name, 0, 30);
  IF ws_Idx = 0 THEN
   ws_WS[ws_Idx]:=objWB.Worksheets(1);
  ELSE
   ws_WS[ws_Idx]:=objWB.Worksheets.Add(, objWB.Worksheets(ws_Idx));
  END;
  ws_WS[ws_Idx].Name:=ws_Name;
  wsn_Init(ws_Idx);
 END {ws_Create_Int};

 PUBLIC PROCEDURE ws_Create(objWB: OBJECT; ws_Name: STRING[]);
  VAR
   ws_Idx: NUMERIC[0];
 BEGIN
  ws_Idx:=objWB.WorkSheets.Count;
  IF (ws_Idx # 0) & !ws_WS[0] THEN
   ws_Idx:=0;
  END;
  ws_Create_Int(objWB, ws_Idx, ws_Name);
 END {ws_Create};

 PUBLIC PROCEDURE wsn_AddColumn(FieldType, Precision: NUMERIC[0]; Name, Comment: STRING[]);
 BEGIN
  AddColumnFromRow(ws_WS[ws_CurIdx], ws_Row[ws_CurIdx], ws_Col[ws_CurIdx], FieldType, Precision, Name, Comment);
  INC(ws_Col[ws_CurIdx]);
  IF ws_Col[ws_CurIdx] = BlankCellAmount THEN
   L_General.Error(" Excel- ᫨誮 讥 ⢮ ⮫殢 ( " + L_Str.IntStr(HIGH(BlankCells)+1) + ")." +
                   L_StdFrm.HR()+
                   "᫨  ⢨⥫쭮 ॡ 㧨  Excel  䠩,|" +
                   "  ");
  END;
 END {wsn_AddColumn};

 PUBLIC PROCEDURE wsn_GetCurPos(VAR row, column: NUMERIC[0]);
 BEGIN
  row   :=ws_Row[ws_CurIdx];
  column:=ws_Col[ws_CurIdx];
 END {wsn_GetCurPos};

 PUBLIC PROCEDURE wsn_SetCurPos(row, column: NUMERIC[0]);
 BEGIN
  ws_Row[ws_CurIdx]:=row;
  ws_Col[ws_CurIdx]:=column;
 END {wsn_SetCurPos};

 PUBLIC PROCEDURE wsn_CurCell(): OBJECT;
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  RETURN ws_WS[ws_CurIdx].Cells(ExcelCurRow, ExcelCurColumn);
 END {wsn_CurCell};

 PRIVATE PROCEDURE wsn_Wr(VAR Value: VARIANT);
 BEGIN
  IF ~!Value THEN
   ws_WS[ws_CurIdx].Cells(ws_Row[ws_CurIdx], ws_Col[ws_CurIdx]).Value:=Value;
  END;
  INC(ws_Col[ws_CurIdx]);
 END {wsn_Wr};

 PUBLIC PROCEDURE wsn_WrNum(Value: FLOAT);
 BEGIN
  wsn_Wr(Value);
 END {wsn_WrNum};

 PUBLIC PROCEDURE wsn_WrNumF(Value: FLOAT; trgNumberFormat: STRING[]);
 BEGIN
  IF ~!Value & ~!trgNumberFormat THEN
   ws_WS[ws_CurIdx].Cells(ws_Row[ws_CurIdx], ws_Col[ws_CurIdx]).NumberFormat:=trgNumberFormat;
  END;
  wsn_Wr(Value);
 END {wsn_WrNumF};

 PUBLIC PROCEDURE wsn_WrStr(Value: STRING[]);
 BEGIN
  Value:=L_Str.ExcludeCRinCRLF(Value);
  wsn_Wr(Value);
 END {wsn_WrStr};

 PUBLIC PROCEDURE wsn_WrScl(Value: STRING[]);
 BEGIN
  wsn_Wr(Value);
 END {wsn_WrScl};

 PUBLIC PROCEDURE wsn_WrDat(Value: DATE);
 BEGIN
  wsn_Wr(Value);
 END {wsn_WrDat};

(*  0101345
 PUBLIC PROCEDURE wsn_WrTim(Value: TIME);
 BEGIN
  (* ᯮ짮*)
 END {wsn_WrTim};
*)

 PUBLIC PROCEDURE wsn_SkipCols(Num: NUMERIC[0]);
 BEGIN
  INC(ws_Col[ws_CurIdx], Num);
 END {wsn_SkipCols};

 PUBLIC PROCEDURE wsn_WrFormula(trgFormula: STRING[]);
 BEGIN
  ws_WS[ws_CurIdx].Cells(ws_Row[ws_CurIdx], ws_Col[ws_CurIdx]).FormulaR1C1:=trgFormula;
  INC(ws_Col[ws_CurIdx]);
 END {wsn_WrFormula};

 PUBLIC PROCEDURE wsn_TermLine();
 BEGIN
  ws_Row[ws_CurIdx]:=ws_Row[ws_CurIdx] + 1;
  ws_Col[ws_CurIdx]:=1;
 END {wsn_TermLine};

 PUBLIC PROCEDURE wsn_DeInit();
 BEGIN
  ws_Row[ws_CurIdx]:=0;
  ws_Col[ws_CurIdx]:=0;
 END {wsn_DeInit};

 PUBLIC PROCEDURE wsn_Worksheet(): OBJECT;
 BEGIN
  RETURN ws_WS[ws_CurIdx];
 END {wsn_Worksheet};

 PUBLIC PROCEDURE wsn_FreezePanes(row, column: NUMERIC[0]);
 BEGIN
  ws_WS[ws_CurIdx].Cells(row, column).Select();
  ws_WS[ws_CurIdx].Application.ActiveWindow.FreezePanes:=xlTrue;
 END {wsn_FreezePanes};

 PUBLIC PROCEDURE wsn_PaintGrid(BorderWidth: NUMERIC[0]; AutoFilter: SCAL[2]);
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  ws_WS[ws_CurIdx].Range(ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx] + ws_HeaderDepth[ws_CurIdx], 1),
                         ws_WS[ws_CurIdx].Cells(ExcelCurRow-1, ws_HeaderLength[ws_CurIdx])).Borders.Weight:=BorderWidth;
  IF AutoFilter THEN
   ws_WS[ws_CurIdx].Range(ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx] + ws_HeaderDepth[ws_CurIdx] - 1, 1),
                          ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx] + ws_HeaderDepth[ws_CurIdx] - 1, ws_HeaderLength[ws_CurIdx])).AutoFilter();
  END;
 END {wsn_PaintGrid};

 PUBLIC PROCEDURE wsn_SetCurCellFontSize(trgFontSize: NUMERIC[0]);
 BEGIN
  wsn_CurCell().Font.Size:=trgFontSize;
 END {wsn_SetCurCellFontSize};

 PRIVATE PROCEDURE wsn_DecorCurCell(Height, Width, BorderWidth: NUMERIC[0]; WrapText: SCAL[2]);
  VAR
   CurCell       : OBJECT;
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  CurCell:=wsn_CurCell();
  CurCell.VerticalAlignment  := 2;    (*ࠢ ⥪  祩  ⨪  業*)
  CurCell.HorizontalAlignment:= 3;    (*ࠢ ⥪  祩  ਧ⠫  業*)
  IF WrapText THEN
   CurCell.WrapText:=xlTrue;
  ELSE
   CurCell.WrapText:=xlFalse;
  END;
  IF Width > 0 THEN
   CurCell.ColumnWidth:=Width;
  ELSE
   CurCell.Columns.AutoFit();
  END;
  IF Height > 0 THEN
   CurCell.RowHeight:=Height;
  ELSE
   CurCell.Rows.AutoFit();
  END;
  CurCell.Borders.Weight:=2;
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  IF ws_HeaderStart[ws_CurIdx] = 0 THEN
   ws_HeaderStart[ws_CurIdx]:=ExcelCurRow;
  END;
 END {wsn_DecorCurCell};

 PRIVATE PROCEDURE wsn_MergeUnderItem();
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
   j             : NUMERIC[0];
   S             : STRING[];
   range         : OBJECT;
 BEGIN
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  IF (ws_HeaderStart[ws_CurIdx] < ExcelCurRow) THEN
   S:=ws_WS[ws_CurIdx].Cells(ExcelCurRow - 1, ExcelCurColumn - 1).Value;
   IF !S THEN
    DEC(ExcelCurRow);
    DEC(ExcelCurColumn);
    WHILE ws_HeaderStart[ws_CurIdx] <= ExcelCurRow DO
     range:=ws_WS[ws_CurIdx].Range(ws_WS[ws_CurIdx].Cells(ExcelCurRow, ExcelCurColumn-1),
                                   ws_WS[ws_CurIdx].Cells(ExcelCurRow, ExcelCurColumn));
     range.Merge();
     range.Borders.Weight:=2;
     DEC(ExcelCurRow);
    END;
   END;
  END;
 END {wsn_MergeUnderItem};

 PUBLIC PROCEDURE wsn_StartHeader();
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  ws_HeaderStart[ws_CurIdx]:=ExcelCurRow;
  ws_HeaderDepth[ws_CurIdx]:= 1;
 END {wsn_StartHeader};

 PUBLIC PROCEDURE wsn_StopHeader();
  VAR
   ExcelCurRow,
   ExcelCurColumn,
   j,
   k              : NUMERIC[0];
   S              : STRING[];
   range          : OBJECT;
 BEGIN
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  ws_HeaderLength[ws_CurIdx]:=ExcelCurColumn - 1;
  k:=ws_HeaderLength[ws_CurIdx];
  WHILE k > 0 DO
   j:=ws_HeaderDepth[ws_CurIdx];
   WHILE j > 1 DO
    S:=ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx] + j - 1, k).Value;
    ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx], k).Borders.Weight:=2;
    IF !S THEN
     range:=ws_WS[ws_CurIdx].Range(ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx] + j - 1, k),
                                   ws_WS[ws_CurIdx].Cells(ws_HeaderStart[ws_CurIdx] + j - 2, k));
     range.Merge();
     range.Borders.Weight:=2;
     DEC(j);
    ELSE
     j:=1;
    END;
   END; (* WHILE ... *)
   DEC(k);
  END;
  ExcelCurRow   :=ExcelCurRow + ws_HeaderDepth[ws_CurIdx];
  ExcelCurColumn:=1;
  wsn_SetCurPos(ExcelCurRow, ExcelCurColumn);
 END {ws_StopHeader};

 PUBLIC PROCEDURE wsn_AddHeaderItem(Text: STRING[]; Size, Height: NUMERIC[0]);
 BEGIN
  wsn_DecorCurCell(Height, Size, cExcelRepHeaderBorderWidth, 1);
  wsn_WrStr(Text);
  wsn_MergeUnderItem();
 END {wsn_AddHeaderItem};

 PUBLIC PROCEDURE wsn_AddHeaderSubItem();
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  wsn_GetCurPos(ExcelCurRow, ExcelCurColumn);
  INC(ExcelCurRow);
  DEC(ExcelCurColumn);
  IF ExcelCurRow > ws_HeaderStart[ws_CurIdx] + ws_HeaderDepth[ws_CurIdx] - 1 THEN
   INC(ws_HeaderDepth[ws_CurIdx]);
  END;
  DEC(ws_HeaderLength[ws_CurIdx]);
  wsn_SetCurPos(ExcelCurRow, ExcelCurColumn);
 END {wsn_AddHeaderSubItem};

 PUBLIC PROCEDURE wsn_HeaderReturnFromSubItem();
  VAR
   ExcelCurRow,
   ExcelCurColumn: NUMERIC[0];
 BEGIN
  wsn_GetCurPos(ExcelCurRow,     ExcelCurColumn);
  wsn_SetCurPos(ExcelCurRow - 1, ExcelCurColumn);
 END {wsn_HeaderReturnFromSubItem};

 PUBLIC PROCEDURE wsn_MergeCells(begRow, begCol        : NUMERIC[0];
                                 endRow, endCol        : NUMERIC[0];
                                 trgHorizontalAlignment: NUMERIC[0];
                                 trgVerticalAlignment  : NUMERIC[0]);
  VAR
   range: OBJECT;
 BEGIN
  range:=ws_WS[ws_CurIdx].Range(ws_WS[ws_CurIdx].Cells(begRow, begCol), ws_WS[ws_CurIdx].Cells(endRow, endCol));
  range.Merge();

  range.Borders.Weight     :=2;
  range.HorizontalAlignment:=trgHorizontalAlignment;
  range.VerticalAlignment  :=trgVerticalAlignment;
 END {wsn_MergeCells};

END.
