(*
%Copyright   (c) Polak IT
%Author      05.02.2009 13:49:13 POLAK\LDA
%Purpose     ⥪ ࠡ  䠩
%Type        library
%Specific    SYS
%Modified    27.03.2014 17:57:08 POLAK\LDA
%Environment
*)

 MODULE L_FIO [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT FIO, L_MSWStp, L_General, L_Str, Str, Math;

 PUBLIC PROCEDURE effp_FileName(FullFileName: STRING[]): STRING[];
  VAR
   Dummy,
   FileName: STRING[];
 BEGIN
  FIO.SplitPath(FullFileName, FileName, , Dummy);
  RETURN FileName;
 END {effp_FileName};

 PUBLIC PROCEDURE effp_FileName_woExt(FullFileName: STRING[]): STRING[];
  VAR
   Dummy,
   FileName_woExt: STRING[];
 BEGIN
  FIO.SplitPath(FullFileName, FileName_woExt, Dummy, Dummy);
  RETURN FileName_woExt;
 END {effp_FileName_woExt};

 PUBLIC PROCEDURE effp_FileExt(FullFileName: STRING[]): STRING[];
  VAR
   Dummy,
   Ext: STRING[];
 BEGIN
  FIO.SplitPath(FullFileName, Dummy, Ext);
  RETURN Ext;
 END {effp_FileExt};

 PUBLIC PROCEDURE effp_Folder(FullFileName: STRING[]): STRING[];
  VAR
   Dummy,
   Folder: STRING[];
 BEGIN
  FIO.SplitPath(FullFileName, Dummy, , Folder);
  RETURN Folder;
 END {effp_Folder};

 PRIVATE PROCEDURE GetFile(FileName: STRING[]): FOLDER_ENTRY;
  VAR
   FullFileName: STRING[];
 BEGIN
  L_General.Assert(FIO.Exists(FileName, , FullFileName), "  䠩|" + FileName);
  RETURN FOLDER_ENTRY.Create(FullFileName);
 END {GetFile};

 PUBLIC PROCEDURE GetFileSize(FileName: STRING[]): NUMERIC[0];
 BEGIN
  RETURN GetFile(FileName).Size;
 END {GetFileSize};

 PUBLIC PROCEDURE GetDirSize(DirName: STRING[]): NUMERIC[0];
  VAR
   File,
   Folder       : FOLDER_ENTRY;
   FolderContent: ENUMERATOR;
   Result       : NUMERIC[0];
 BEGIN
  L_General.Assert(FIO.DirExists(DirName), "  |" + DirName);
  Result       :=0;
  Folder       :=FOLDER_ENTRY.Create(DirName);
  FolderContent:=CreateENUMERATOR(Folder.FolderContent);
  WHILE ~FolderContent.AtEnd() DO
   File:=FolderContent.Item();
   PROMPT("᫥ ꥬ  '" + DirName + "': " + effp_FileName(File.Path));
   IF File.Attributes.Directory THEN
    INC(Result, GetDirSize(File.Path));
   ELSE
    INC(Result, File.Size);
   END;
   FolderContent.MoveNext();
  END;
  RETURN Result;
 END {GetDirSize};

 PUBLIC PROCEDURE SetFileLastModifiedDTS(FullFileName: STRING[]; LastModifiedDTS: TIMESTAMP);
  CONST
   MaxTryAmount = 10;
  VAR
   TryAmount: NUMERIC[0];
 BEGIN
  TryAmount:=0;
  LOOP
   TRY [MSG_SUPPRESSION(MSG_SUPPRESSION_ALL), EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
    FIO.Open(tmpDst, FullFileName, "", Output);
    FIO.SetFileTime(tmpDst, , , LastModifiedDTS);
    FIO.Close(tmpDst);
    EXIT;
   EXCEPT
   ON Exc:Exc_Root DO
    INC(TryAmount);
    SLEEP(1.0);
    IF TryAmount > MaxTryAmount THEN
     RAISE();
    END;
   END; (* TRY *)
  END; (* LOOP *)
 END {SetFileLastModifiedDTS};

 PUBLIC PROCEDURE GetFileLastModifiedDTS(FileName: STRING[]): TIMESTAMP;
 BEGIN
  RETURN GetFile(FileName).TimeLastModified;
 END {GetFileLastModifiedDTS};

 PUBLIC PROCEDURE GetUTCFileLastModifiedDTS(FileName: STRING[]): TIMESTAMP;
 BEGIN
  RETURN GetFile(FileName).UTCLastModified;
 END {GetUTCFileLastModifiedDTS};

 PRIVATE PROCEDURE _FileCopy(SrcFile,
                             DstFile    : STRING[];
                             ExactlyCopy: SCAL[2]);
  VAR
   lastWriteDTS: TIMESTAMP;
 BEGIN
  lastWriteDTS:=GetFileLastModifiedDTS(SrcFile);
  FIO.Open(Src, SrcFile, "", Input);
  TRY
   FIO.Open(Dst, DstFile, "", Create);
   FIO.Copy(Src, Dst);
   FIO.Close(Dst);
   FIO.Close(Src);
  EXCEPT
  ON Exc: Exc_Root DO
   FIO.Close(Src);
   RAISE();
  END; (* TRY *)

  IF ExactlyCopy THEN
   SetFileLastModifiedDTS(DstFile, lastWriteDTS);
  END;
 END {_FileCopy};

 PUBLIC PROCEDURE FileCopy(SrcFile, DstFile: STRING[]);
 BEGIN
  _FileCopy(SrcFile, DstFile, 0);
 END {FileCopy};

 PUBLIC PROCEDURE ExactFileCopy(SrcFile, DstFile: STRING[]);
 BEGIN
  _FileCopy(SrcFile, DstFile, 1);
 END {ExactFileCopy};

 PUBLIC PROCEDURE tf_GetFullPath(FileName, FileExt: STRING[]): STRING[];
  VAR
   tmpPath: STRING[];
 BEGIN
  tmpPath:=FIO.AsbSpecialFolders("ASBTemp");
  IF ~FIO.DirExists(tmpPath) THEN
   L_General.Error(" 室 ᯮ ணࠬ 室 ᮧ ६ 䠩," +
                   "|   ࠭ ६ 䠩  :" +
                   "|" + tmpPath +
                   "||஡ ॣ㧨 .");
  END;
  RETURN FIO.BuildPath(FileName, FileExt, tmpPath);
 END {tf_GetFullPath};

 PUBLIC PROCEDURE rf_GetPath(ByPrototypeRoot: SCAL[2]): STRING[];
  VAR s: STRING[];
 BEGIN
  SETEMPTY(s);
  IF ByPrototypeRoot THEN
   s:=FIO.AsbSpecialFolders("PrototypeRoot");
  END;
  IF ~!s THEN
   RETURN FIO.BuildPath("TXT\RES", , s);
  ELSE
   RETURN "TXT\RES";
  END;
 END {rf_GetPath};

 PUBLIC PROCEDURE rf_GetFullFileName(FileName: STRING[]): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  Result:=FIO.BuildPath(FileName, , FIO.UNCPath(rf_GetPath(0)));
  IF ~FIO.Exists(Result) THEN
   Result:=FIO.BuildPath(FileName, , FIO.UNCPath(rf_GetPath(1)));
  END;
  RETURN Result;
 END {rf_GetFullFileName};

 PUBLIC PROCEDURE IsEqualFiles(File1, File2: OBJECT): SCAL[2];
  VAR
   t: FLOAT;
 BEGIN
  t:=Math.Abs(File1.DateLastModified-File2.DateLastModified);
  RETURN L_Str.IsEqStrI(effp_FileName(File1.Path), effp_FileName(File2.Path)) &
         (t<=(2.05/(60*60*24))); (*⨬ 2 ᥪ㭤 ࠧ (+0.05ᥪ  譮)*)
 END {IsEqualFiles};

 PUBLIC PROCEDURE ext_IsEqualFiles(File1, File2: STRING[]): SCAL[2];
  VAR
   oFile1, oFile2: FOLDER_ENTRY;
 BEGIN
  IF FIO.Exists(File1) & FIO.Exists(File2) THEN
   oFile1:=FOLDER_ENTRY.Create(File1);
   oFile2:=FOLDER_ENTRY.Create(File2);
   RETURN IsEqualFiles(oFile1, oFile2) & (oFile1.Size = oFile2.Size);
  ELSE
   RETURN 0;
  END;
 END {ext_IsEqualFiles};

 PUBLIC PROCEDURE rf_GetCHMFullName(CHMFileName: STRING[]): STRING[];
  VAR
   srcCHMFullPath,
   dstCHMFullPath: STRING[];
 BEGIN
  srcCHMFullPath:=rf_GetFullFileName(FIO.BuildPath(CHMFileName, "chm"));
  IF ~L_General.IsWindows7() & ISTERMINALMODE() THEN
   dstCHMFullPath:=srcCHMFullPath;
  ELSE
   dstCHMFullPath:=tf_GetFullPath(CHMFileName, "chm");
   IF ~ext_IsEqualFiles(srcCHMFullPath, dstCHMFullPath) THEN
    ExactFileCopy(srcCHMFullPath, dstCHMFullPath);
   END;
  END;
  RETURN dstCHMFullPath;
 END {rf_GetCHMFullName};

 PUBLIC PROCEDURE ClearDir(DirName: STRING[]);
  VAR
   File,
   Folder       : FOLDER_ENTRY;
   FolderContent: ENUMERATOR;
 BEGIN
  Folder       :=FOLDER_ENTRY.Create(DirName);
  FolderContent:=CreateENUMERATOR(Folder.FolderContent);
  WHILE ~FolderContent.AtEnd() DO
   File:=FolderContent.Item();
   IF File.Attributes.Directory THEN
    ClearDir(File.Path);
    FIO.RmDir(File.Path);
   ELSE
    FIO.Erase(File.Path);
   END;
   FolderContent.MoveNext();
  END;
 END {ClearDir};

 PUBLIC PROCEDURE RemoveDir(DirName: STRING[]);
  VAR
   fso: OBJECT;
 BEGIN
  fso:=CreateOBJECT("Scripting.FileSystemObject");
  IF fso.FolderExists(DirName) THEN
   TRY
    fso.DeleteFolder(DirName);
   EXCEPT
   ON Exc: Exc_Object DO
    IF L_MSWStp.ScodeCode(Exc.ErrorCode) = 70 THEN
     L_General.Error(" 㤠 㤠  '" + DirName + "'||" +
                     ",    㤠 .");
    ELSE
     RAISE();
    END;
   END; (* TRY *)
  END;
 END {RemoveDir};

 PUBLIC PROCEDURE MakeDir(FullDirName: STRING[]): STRING[];
 BEGIN
  IF ~FIO.DirExists(FullDirName) THEN
   FIO.MkDir(FullDirName);
  END;
  RETURN FullDirName;
 END {MakeDir};

 PUBLIC PROCEDURE SplitFolders(    srcFolder    : STRING[];
                               VAR Folders      : STRING[][];
                               VAR FoldersAmount: NUMERIC[0]);
 BEGIN
  FoldersAmount:=0;
  LOOP
   IF L_Str.Char(srcFolder, Str.Length(srcFolder)-1) = "\" THEN
    srcFolder:=Str.Delete(srcFolder, Str.Length(srcFolder)-1, 1);
   END;
   L_General.Assert(FoldersAmount<=HIGH(Folders), "誮    (ᨬ - " + L_Str.IntStr(HIGH(Folders)+1) + ")");
   FIO.SplitPath(srcFolder, Folders[FoldersAmount], , srcFolder);
   IF !Folders[FoldersAmount] THEN
    Folders[FoldersAmount]:=srcFolder + "\";
    SETEMPTY(srcFolder);
   END;
   IF !srcFolder THEN
    EXIT;
   END;
   INC(FoldersAmount);
  END; (* LOOP *)
 END {SplitFolders};

 PUBLIC PROCEDURE MakeSubDirs(srcFolder: STRING[]);
  VAR
   newFolder    : STRING[];
   Folders      : STRING[][20];
   FoldersAmount: NUMERIC[0];
 BEGIN
  IF ~Str.Match(srcFolder, "?:\*") & ~Str.Match(srcFolder, "\\*") THEN
   L_General.Error("४  '" + srcFolder + "'");
  END;
  IF ~FIO.DirExists(srcFolder) THEN
   SplitFolders(srcFolder, Folders, FoldersAmount);
   SETEMPTY(newFolder);
   WHILE FoldersAmount >= 0 DO
    newFolder:=FIO.BuildPath(Folders[FoldersAmount], , newFolder);
    IF ~FIO.DirExists(newFolder) THEN
     FIO.MkDir(newFolder);
    END;
    DEC(FoldersAmount);
   END; (* WHILE FoldersAmount >= 0...*)
  END;
 END {MakeSubDirs};

 PUBLIC PROCEDURE CreateTempDir(DirName: STRING[]): STRING[];
 BEGIN
  RETURN MakeDir(FIO.BuildPath(DirName, , FIO.AsbSpecialFolders("ASBTemp")));
 END {CreateTempDir};

 PUBLIC PROCEDURE intGetUniqueFileNameInFolder(FileName,
                                               Folder     : STRING[];
                                               IndexLength: NUMERIC[0];
                                               forceRename: SCAL[2]): STRING[];
  VAR
   Idx     : SCAL[1E4];
   fName,
   fExt,
   fPath,
   tmpFName: STRING[];
 BEGIN
  FIO.SplitPath(FileName, fName, fExt, fPath);
  tmpFName:=fName;
  Idx     :=0001;
  LOOP
   IF forceRename THEN
    fName:=tmpFName + "_" + FLDTOSTR(Idx, "L", IndexLength);
   ELSE
    fName:=tmpFName;
   END;
   FileName:=FIO.BuildPath(fName, fExt, Folder);
   IF ~FIO.Exists(FileName) THEN
    EXIT;
   END;
   IF forceRename THEN
    INC(Idx);
   END;
   forceRename:=1;
  END; (* LOOP *)
  RETURN FileName;
 END {intGetUniqueFileNameInFolder};

 PUBLIC PROCEDURE GetUniqueFileNameInFolder(FileName, Folder: STRING[]): STRING[];
 BEGIN
  RETURN intGetUniqueFileNameInFolder(FileName, Folder, 4, 0);
 END {GetUniqueFileNameInFolder};

 PUBLIC PROCEDURE IsDirectory(FullFileName: STRING[]): SCAL[2];
  VAR
   FileObject: FOLDER_ENTRY;
 BEGIN
  FileObject:=FOLDER_ENTRY.Create(FullFileName);
  RETURN FileObject.Attributes.Directory;
 END {IsDirectory};

 PUBLIC PROCEDURE IsFullUNCPath(Path: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Slice(Path, 0, 2) = "\\";
 END {IsFullUNCPath};

END.
