(*
%Copyright   (c) Polak IT
%Author      12.08.2010 11:07:14 POLAK\San
%Purpose     প Guard
%Type        library
%Specific    Core
%Modified    13.03.2014 16:49:59 POLAK\GAV
%Environment
*)

 MODULE L_Guard [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_PCS, FIO, L_General, L_Str, L_Basis, L_DatTim, L_SES, L_StdFrm, L_AR;

 PUBLIC CONST
  MessagesCheckPeriod = 30;

 PRIVATE CONST
  GuardJobName      = "Guard";
  ses_GuardPassword = "GuardPassword";

  ec_rt_SchedulerCreateProcessFailed = 533;

 PRIVATE PROCEDURE GuardLoginName(): STRING[];
 BEGIN
  RETURN L_PCS.GuardLoginName();
 END {GuardLoginName};

 PUBLIC PROCEDURE AcsObjOfGuard(): LIKE(AcsObj.AcsObj);
 BEGIN
  IF L_AR.StandOnUser(GuardLoginName()) THEN
   RETURN AcsObj.AcsObj;
  ELSE
   RETURN EMPTYOF(AcsObj.AcsObj);
  END;
 END {AcsObjOfGuard};

 PUBLIC PROCEDURE IsJobConnection(c: ASBCONNECTION): SCAL[2];
 BEGIN
  RETURN ~!c.Job;
 END {IsJobConnection};

 PUBLIC PROCEDURE IsGuardConnection(c: ASBCONNECTION): SCAL[2];
 BEGIN
  RETURN IsJobConnection(c) & (c.Job.Name = GuardJobName);
 END {IsGuardConnection};

 PRIVATE PROCEDURE CreateJob(Name: STRING[]): ASBJOB;
  VAR
   Result   : ASBJOB;
   Encrypted: STRING[];
 BEGIN
  Result:=ASBJOB.Create(Name, GuardLoginName());
  IF L_SES.GetValue(L_SES.sit_Global, ses_GuardPassword, Encrypted) THEN
   Result.Password:=DECRYPTSTR(Encrypted, "654321", CALG_RC2, 128, , CSP_NAME_MS_STRONG);
  END;
  RETURN Result;
 END {CreateJob};

 PRIVATE PROCEDURE intSetGuard(forced: SCAL[2]);
  VAR
   asbJob: ASBJOB;
 BEGIN
  IF APP.Get().ProductiveCopy | forced THEN
   asbJob                 :=CreateJob(GuardJobName);
   asbJob.OnStart         :="";
   asbJob.RestartOnFailure:=1;
   asbJob.PatienceTimeout :=10*60;
   asbJob.Apply();
  END;
 END {intSetGuard};

 PUBLIC PROCEDURE SetGuard();
  VAR
   asbJob  : ASBJOB;
   ErrorMsg: STRING[];
 BEGIN
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   intSetGuard(0);
  EXCEPT
  ON Exc: Exc_Root DO
   IF (Exc.ErrorCode = ec_rt_SchedulerCreateProcessFailed      ) |
      (Exc.ErrorCode = L_General.ec_rt_IncorrectlyEncodedString) |
      (Exc.ErrorCode = L_General.ec_rt_CantDecodeString        ) THEN
    ErrorMsg:="室 믮 ன ஫ 짮⥫ " + GuardLoginName() +
              "|"+
              "('Ctrl+G - ࢨ 楤')";
   ELSE
    ErrorMsg:=Exc.MsgOriginalText;
   END;
   IF ~L_General.IsAdmin() THEN
    ErrorMsg:=", ᮮ  ᫥饥:||" + ErrorMsg;
   END;
   L_StdFrm.RedInform(ErrorMsg);
  END; (* TRY *)
 END {SetGuard};

 PUBLIC PROCEDURE SetupGuardPassword();
  VAR
   Password: STRING[];
 BEGIN
  L_General.SilentAdminOnly();
  LOOP
   SETEMPTY(Password);
   STDFORM(L_StdFrm.sf_Choice("|஫:        %####################%|", "%LSI"),
           "ன ஫ 짮⥫ " + GuardLoginName(),
           INFORM,
           ,
           Password);
   TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
    Password:=ENCRYPTSTR(Password, "654321", CALG_RC2, 128, , CSP_NAME_MS_STRONG);
    L_SES.PutValueAtCurTrn(L_SES.sit_Global, ses_GuardPassword, Password);
    ASBJOB.Get(GuardJobName).Eliminate();
    intSetGuard(1);
    L_StdFrm.Inform("ன 믮");
    EXIT;
   EXCEPT
   ON Exc: Exc_Root DO
    IF Exc.ErrorCode = ec_rt_SchedulerCreateProcessFailed THEN
     L_StdFrm.RedInform("஫ 짮⥫ " + GuardLoginName() + " 㪠 ୮," +
                        "||" +
                        " 짮⥫  ⠪    (Active Directory)");
    ELSE
     RAISE();
    END;
   END; (* TRY *)
  END;
 END {SetupGuardPassword};

 PUBLIC PROCEDURE IamAlive();
 BEGIN
  L_General.CallUs(IsGuardConnection(APP.Get().MyConnection));
  APP.Get().MyConnection.Job.PatienceTimeout:=10*60;
  APP.Get().MyConnection.Job.Apply();
 END {IamAlive};

 PUBLIC PROCEDURE SetNewJob(Name, OnStart: STRING[]; Timeout: NUMERIC[0]; Qualified: SCAL[2]);
  CONST
   sleepPeriod = 0.5;
  VAR
   asbJob    : ASBJOB;
   wastedTime: NUMERIC[1];
 BEGIN
  L_General.CallUs(Name # GuardJobName);
  asbJob:=CreateJob(Name);
  asbJob.OnStart:=OnStart;
  asbJob.RestartOnFailure:=0;
  asbJob.PatienceTimeout :=Timeout;
  asbJob.Apply();
  IF Qualified THEN
   wastedTime:=0;
   WHILE asbJob.Valid & (asbJob.TimeLastActivity = asbJob.TimeCreation) DO
    PROMPT("  (" + FLDTOSTR(wastedTime, "1.", 0) + "): " + Name);
    SLEEP(sleepPeriod);
    INC(wastedTime, sleepPeriod);
    asbJob.Refresh();
   END;
  END;
 END {SetNewJob};

 PUBLIC PROCEDURE ExecuteMyself();
 BEGIN
  L_General.CallUs(IsJobConnection(APP.Get().MyConnection));
  TRY [TRANSACTION(TRANSACTION_SLAVE), MSG_SUPPRESSION(MSG_SUPPRESSION_ALL), TERM_BY_USER(TERM_BY_USER_THROW)]
   GRANTALLRIGHTS();
   CALL(^(APP.Get().MyConnection.Job.OnStart));
  EXCEPT
  ON Exc: Exc_Root DO
  END; (* TRY *)
  ABORTASBPROCESSOR();
 END {ExecuteMyself};

END.
