(*
%Copyright   (c) Polak IT
%Author      01.09.2010 13:35:09 POLAK\San
%Purpose     GUI
%Type        library
%Specific    Core
%Modified    25.08.2014 12:09:52 POLAK\Bor
%Environment
*)

 MODULE L_GUI [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC CONST
  ta_Left   = "left";
  ta_Center = "center";
  ta_Right  = "right";

  br_st_ThinRidge = "thin ridge";
  br_st_Ridge     = "ridge";
  br_st_None      = "none";
  br_st_Solid     = "solid";
  br_st_Dashed    = "dashed";
  br_st_Thin      = "thin";
  br_st_OutSet    = "outSet";

  fs_StandartButton           = 26;

  fw_Normal = 0;
  fw_Bold   = 1;

  FontWeight_Pool   = "normal|bold";

  vb_Hidden  = 0;
  vb_Visible = 1;

  smallWidth  = 800;
  smallHeight = 600;

  normalWidth  = 1024;
  normalHeight = 768;

  fn_LucidaConsole = "Lucida Console";
  fn_Arial         = "Arial";
  fn_CourierNew    = "Courier New";
  fn_TimesNewRoman = "Times New Roman";
  fn_Georgia       = "Georgia";
  fn_Tahoma        = "Tahoma";

  fn_Default       = fn_Georgia;

 PUBLIC VAR
  DEBUG: SCAL[2];

 PRIVATE VAR
  dlgPagesCollection: OBJECT;

 IMPORT L_General, Str, L_Str, Math, L_AFM, FIO, L_FIO;

 PUBLIC PROCEDURE IsAcceptableIEVersion(): SCAL[2];
  VAR
   IEV_Major: NUMERIC[0];
 BEGIN
  IEVERSION(IEV_Major);
  RETURN IEV_Major >= 8.0;
 END {IsAcceptableIEVersion};

 PUBLIC PROCEDURE ConvText(S: STRING[]): STRING[];
 BEGIN
  RETURN L_Str.Replace(S, "|", L_Str.CRLF);
 END {ConvText};

 PRIVATE PROCEDURE Points2Himetric(Points: FLOAT): NUMERIC[0];
 BEGIN
  RETURN (Points / 72) * 2540;
 END {Points2Himetric};

 PRIVATE PROCEDURE Himetric2Points(Himetric: NUMERIC[0]): FLOAT;
 BEGIN
  RETURN Himetric / 2540 * 72;
 END {Himetric2Points};

 PUBLIC PROCEDURE Height2Points(Height: NUMERIC[0]): FLOAT;
  VAR
   hmY, dummyX: NUMERIC[0];
 BEGIN
  PIXELSTOHIMETRIC(dummyX, hmY, 0, Height);
  RETURN Himetric2Points(hmY);
 END {Height2Points};

 PUBLIC PROCEDURE pc_AddId(Id, Page: STRING[]);
  VAR
   Pages: STRING[];
 BEGIN
  IF dlgPagesCollection.Exists(Id) = -1 THEN
   Pages:=dlgPagesCollection.Item(Id);
   dlgPagesCollection.Remove(Id);
  ELSE
   SETEMPTY(Pages);
  END;
  Pages:=L_Str.AddStrDlm(Pages, "|") + Page;
  dlgPagesCollection.Add(Id, Pages);
 END {pc_AddId};

 PUBLIC PROCEDURE pc_GetPageById(Id: STRING[]): STRING[];
  VAR
   Pages,
   Result: STRING[];
 BEGIN
  Pages :=dlgPagesCollection.Item(Id);
  Result:=L_Str.PoolItem(Pages, 0);
  L_General.CallUs(~!Result & !L_Str.PoolItem(Pages, 1));
  RETURN Result;
 END {pc_GetPageById};

 PUBLIC PROCEDURE pc_Get(): OBJECT;
 BEGIN
  RETURN dlgPagesCollection;
 END {pc_Get};

 PUBLIC PROCEDURE pc_Set(trgPagesCollection: OBJECT);
 BEGIN
  dlgPagesCollection:=trgPagesCollection;
 END {pc_Set};

 PUBLIC PROCEDURE ShowPages(dlg           : DIALOG;
                            PagesList     : STRING[];
                            HideOtherPages: SCAL[2]);
  VAR
   c         : CONTROL;
   i, Idx    : NUMERIC[0];
   Id, Pages,
   SinglePage: STRING[];
 BEGIN
  Idx:=0;
  WHILE Idx < dlg.Controls.Count DO
   c:=dlg.Controls.Item(Idx);
   i:=0;
   LOOP
    SinglePage:=L_Str.PoolItem(dlgPagesCollection.Item(c.Id), i);
    IF !SinglePage THEN
     EXIT;
    END;
    IF    ~!L_Str.PositionInPool(PagesList, SinglePage) THEN
     c.Visible:=1;
    ELSIF HideOtherPages THEN
     c.Visible:=0;
    END;
    INC(i);
   END; (* LOOP *)
   INC(Idx);
  END; (* WHILE Idx < ... *)
 END {ShowPages};

 PUBLIC PROCEDURE internalSetupButtonRects(b: PUSH_BUTTON);
  CONST
   ButtonImageDecrease = 10;
 BEGIN
  IF ~!b.Picture THEN
   b.TextRect.Left:=(b.Rect.Right-b.Rect.Left) * 0.285 (* sic! *);

   b.PictureRect.Left  :=ButtonImageDecrease;
   b.PictureRect.Top   :=ButtonImageDecrease;
   b.PictureRect.Right :=b.TextRect.Left - ButtonImageDecrease;
   b.PictureRect.Bottom:=b.Rect.Bottom - b.Rect.Top - ButtonImageDecrease;
  ELSE
   b.TextRect.Left:=0;
  END;
  b.TextRect.Top   :=0;
  b.TextRect.Right :=b.Rect.Right  - b.Rect.Left;
  b.TextRect.Bottom:=b.Rect.Bottom - b.Rect.Top;
 END {internalSetupButtonRects};

 PRIVATE PROCEDURE intLoadPicture(FullName: STRING[]; Strong: SCAL[2]): OBJECT;
  CONST
   ec_rt_IncorrectFormat   = 182;
   ec_rt_CannotLoadPicture = 183;
  VAR
   Result: OBJECT;
 BEGIN
  SETEMPTY(Result);
  TRY [EXCEPTIONHANDLING(EXCEPTIONHANDLING_OFF)]
   Result:=LOAD_PICTURE(FullName);
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Strong THEN
    IF    Exc.ErrorCode = ec_rt_IncorrectFormat THEN
     STDMSG(" ଠ 䠩 ࠦ|'"+FullName+"'"+L_StdFrm.HR()+"  ", STOP_AND_CALL_ADMIN);
    ELSIF Exc.ErrorCode = ec_rt_CannotLoadPicture THEN
     STDMSG(" 㤠 㧨 ࠦ  䠩|'"+FullName+"'"+L_StdFrm.HR()+"஡ 믮   ࠧ|᫨ 訡 㤥  -   ", STOP_AND_CALL_ADMIN);
    ELSE
     RAISE();
    END;
   END;
  END;
  RETURN Result;
 END {intLoadPicture};

 PUBLIC PROCEDURE SoftLoadPicture(FullName: STRING[]): OBJECT;
 BEGIN
  RETURN intLoadPicture(FullName, 0);
 END {SoftLoadPicture};

 PUBLIC PROCEDURE LoadPicture(FullName: STRING[]): OBJECT;
 BEGIN
  RETURN intLoadPicture(FullName, 1);
 END {LoadPicture};

 PRIVATE PROCEDURE IsDEBUGMode(): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  Result:=0;
  IF VAR_IMPL.Get(DEBUG).Initialized & DEBUG THEN
   Result:=~APP.Get().ProductiveCopy | APP.Get().SupportMode;
  END;
  RETURN Result;
 END {IsDEBUGMode};

 PRIVATE PROCEDURE AddDEBUGToolTipText(c: CONTROL; Id: STRING[]);
 BEGIN
  IF IsDEBUGMode() THEN
   c.ToolTipText:=Id;
  END;
 END {AddDEBUGToolTipText};

 PRIVATE PROCEDURE AddDEBUGBorder(c: CONTROL);
 BEGIN
  IF IsDEBUGMode() THEN
   c.Border:=1;
  END;
 END {AddDEBUGBorder};

 PUBLIC PROCEDURE SetControlFgColor(c: CONTROL; trgColor: RGBCOLOR);
 BEGIN
  c.Style.Color.Red  :=trgColor.Red;
  c.Style.Color.Green:=trgColor.Green;
  c.Style.Color.Blue :=trgColor.Blue;
 END {SetControlFgColor};

 PUBLIC PROCEDURE SetControlBkColor(c: CONTROL; trgColor: RGBCOLOR);
 BEGIN
  c.Style.BkTransparent:=0;
  c.Style.BkColor.Red  :=trgColor.Red;
  c.Style.BkColor.Green:=trgColor.Green;
  c.Style.BkColor.Blue :=trgColor.Blue;
 END {SetControlBkColor};

 PUBLIC PROCEDURE add_Button(dlg         : DIALOG;
                             Id,
                             Text,
                             FullPictName: STRING[];
                             Left,
                             Top,
                             Width,
                             Height,
                             FontSize    : FLOAT;
                             FontFace    : STRING[];
                             OnClick     : PROC);
  VAR
   b: PUSH_BUTTON;
 BEGIN
  L_General.CallUs(~!Id);
  b:=PUSH_BUTTON.Create(Left, Top, Left+Width, Top+Height);
  b.Id:=Id;
  b.Text:=ConvText(Text);
  b.Style.Font.Size    :=FontSize;
  b.Style.Font.Face    :=FontFace;
  IF ~!FullPictName THEN
   b.Picture:=SoftLoadPicture(FullPictName);
  END;
  internalSetupButtonRects(b);
  b.Visible:=0;
  b.OnClick:=OnClick;
  AddDEBUGToolTipText(b, Id);
  dlg.Controls.Add(b);
 END {add_Button};

 PUBLIC PROCEDURE put_Button(dlg     : DIALOG;
                             Page,
                             Id,
                             Text    : STRING[];
                             Left,
                             Top,
                             Width,
                             Height  : FLOAT;
                             FontSize: FLOAT;
                             FontFace: STRING[];
                             OnClick : PROC);
 BEGIN
  add_Button(dlg, Id, Text, "", Left, Top, Width, Height, FontSize, FontFace, OnClick);
  pc_AddId(Id, Page);
 END {put_Button};

 PUBLIC PROCEDURE put_ColorButton(dlg     : DIALOG;
                                  Page,
                                  Id,
                                  Text    : STRING[];
                                  Left,
                                  Top,
                                  Width,
                                  Height,
                                  FontSize: FLOAT;
                                  trgColor: RGBCOLOR;
                                  OnClick : PROC);
 BEGIN
  put_Button(dlg, Page, Id, Text, Left, Top, Width, Height, FontSize, fn_Default, OnClick);
  dlg.Controls.Item(Id).OldStyleDrawing:=1;
  SetControlBkColor(dlg.Controls.Item(Id), trgColor);
 END {put_ColorButton};

 PUBLIC PROCEDURE put_ImgButton(dlg         : DIALOG;
                                Page,
                                Id,
                                Text,
                                FullPictName: STRING[];
                                Left,
                                Top,
                                Width,
                                Height      : FLOAT;
                                OnClick     : PROC);
 BEGIN
  add_Button(dlg, Id, Text, FullPictName, Left, Top, Width, Height, fs_StandartButton, fn_Default, OnClick);
  pc_AddId(Id, Page);
 END {put_ImgButton};

 PUBLIC PROCEDURE add_Picture(dlg     : DIALOG;
                              Id,
                              PictPath: STRING[];
                              Left,
                              Top,
                              Width,
                              Height  : NUMERIC[0]);
  VAR
   sp: STATIC_PICTURE;
 BEGIN
  L_General.CallUs(~!Id);
  sp:=STATIC_PICTURE.Create(Left, Top, Left+Width, Top+Height);
  sp.Id:=Id;
  IF ~!PictPath THEN
   sp.Picture:=SoftLoadPicture(PictPath);
  END;
  sp.Visible:=0;
  AddDEBUGToolTipText(sp, Id);
  dlg.Controls.Add(sp);
 END {add_Picture};

 PUBLIC PROCEDURE put_Picture(dlg     : DIALOG;
                              Page,
                              Id,
                              PictPath: STRING[];
                              Left,
                              Top,
                              Width,
                              Height  : NUMERIC[0]);
 BEGIN
  add_Picture(dlg, Id, PictPath, Left, Top, Width, Height);
  pc_AddId(Id, Page);
 END {put_Picture};

 PUBLIC PROCEDURE upd_Picture(dlg     : DIALOG;
                              Id,
                              PictPath: STRING[];
                              Left,
                              Top,
                              Width,
                              Height  : NUMERIC[0]);
  VAR
   sp: STATIC_PICTURE;
 BEGIN
  sp:=dlg.Controls.Item(Id);
  sp.Rect.Left  :=Left;
  sp.Rect.Top   :=Top;
  sp.Rect.Right :=Left + Width;
  sp.Rect.Bottom:=Top  + Height;
  IF ~!PictPath THEN
   sp.Picture:=SoftLoadPicture(PictPath);
  END;
 END {upd_Picture};

 PUBLIC PROCEDURE add_StaticText(dlg        : DIALOG;
                                 Id,
                                 Text       : STRING[];
                                 Left,
                                 Top,
                                 Width,
                                 Height     : NUMERIC[0];
                                 FontSize   : FLOAT;
                                 HA         : NUMERIC[0];
                                 BorderStyle: STRING[]);
  VAR
   st: STATIC_TEXT;
 BEGIN
  L_General.CallUs(~!Id);
  st:=STATIC_TEXT.Create(Left, Top, Left+Width, Top+Height);
  st.Id:=Id;
  st.Text:=ConvText(Text);
  IF ~!FontSize THEN
   st.Style.Font.Size:=FontSize;
   st.Style.Font.Face:=fn_Default;
  END;
  st.HorizontalAlignment:=HA;
  st.VerticalAlignment  :=CONTROL.VerticalAlignment_Center;
  IF BorderStyle # br_st_None THEN
   st.Border:=1;
  END;
  st.Visible:=0;
  AddDEBUGToolTipText(st, Id);
  AddDEBUGBorder(st);
  dlg.Controls.Add(st);
 END {add_StaticText};

 PUBLIC PROCEDURE put_StaticText(dlg        : DIALOG;
                                 Page,
                                 Id,
                                 Text       : STRING[];
                                 Left,
                                 Top,
                                 Width,
                                 Height     : NUMERIC[0];
                                 FontSize   : FLOAT;
                                 HA         : NUMERIC[0];
                                 BorderStyle: STRING[]);
 BEGIN
  add_StaticText(dlg, Id, Text, Left, Top, Width, Height, FontSize, HA, BorderStyle);
  pc_AddId(Id, Page);
 END {put_StaticText};

 PUBLIC PROCEDURE put_Border(dlg   : DIALOG;
                             Page,
                             Id    : STRING[];
                             Left,
                             Top,
                             Width,
                             Height: NUMERIC[0]);
 BEGIN
  put_StaticText(dlg, Page, Id, "",
                 Left, Top, Width, Height,
                 0, CONTROL.HorizontalAlignment_Center, br_st_ThinRidge);
 END {put_Border};

 PUBLIC PROCEDURE add_Edit(dlg        : DIALOG;
                           Id         : STRING[];
                           Left,
                           Top,
                           Width,
                           Height     : NUMERIC[0];
                           FontSize   : FLOAT;
                           HA         : NUMERIC[0];
                           BorderStyle: STRING[]);
  VAR
   e: EDIT;
 BEGIN
  L_General.CallUs(~!Id);
  e:=EDIT.Create(Left, Top, Left+Width, Top+Height);
  e.Id:=Id;
  IF ~!FontSize THEN
   e.Style.Font.Size:=FontSize;
   e.Style.Font.Face:=fn_Default;
  END;
  e.HorizontalAlignment:=HA;
  IF BorderStyle # br_st_None THEN
   e.Border:=1;
  END;
  e.Value  :="";
  e.Visible:=0;
  AddDEBUGToolTipText(e, Id);
  AddDEBUGBorder(e);
  dlg.Controls.Add(e);
 END {add_Edit};

 PUBLIC PROCEDURE put_Edit(dlg        : DIALOG;
                           Page,
                           Id         : STRING[];
                           Left,
                           Top,
                           Width,
                           Height     : NUMERIC[0];
                           FontSize   : FLOAT;
                           HA         : NUMERIC[0];
                           BorderStyle: STRING[]);
 BEGIN
  add_Edit(dlg, Id, Left, Top, Width, Height, FontSize, HA, BorderStyle);
  pc_AddId(Id, Page);
 END {put_Edit};

 PUBLIC PROCEDURE put_Rectangle_Select_Tool(dlg              : DIALOG;
                                            Page,
                                            Id               : STRING[];
                                            Left,  Top,
                                            Width, Height    : NUMERIC[0];
                                            tcId             : STRING[];
                                            FixAsR           : SCAL[2];
                                            OnUserChangesRect: PROC);
  VAR
   rst: RECTANGLE_SELECT_TOOL;
 BEGIN
  rst:=RECTANGLE_SELECT_TOOL.Create();
  rst.Id              :=Id;
  rst.TargetControl   :=dlg.Controls.Item(tcId);
  rst.Visible         :=0;
  rst.FixedAspectRatio:=FixAsR;
  dlg.Controls.Add(rst);
  rst.Rect.Left        :=Left + rst.TargetControl.Rect.Left;
  rst.Rect.Top         :=Top + rst.TargetControl.Rect.Top;
  rst.Rect.Right       :=rst.Rect.Left + Width;
  rst.Rect.Bottom      :=rst.Rect.Top + Height;
  rst.OnUserChangesRect:=OnUserChangesRect;
  pc_AddId(Id, Page);
 END {put_Rectangle_Select_Tool};

 PUBLIC PROCEDURE upd_Rectangle_Select_Tool(dlg   : DIALOG;
                                            Id    : STRING[];
                                            Left,
                                            Top,
                                            Width,
                                            Height: NUMERIC[0]);
  VAR
   rst: RECTANGLE_SELECT_TOOL;
 BEGIN
  rst       :=dlg.Controls.Item(Id);
  rst.Rect.Left  :=Left;
  rst.Rect.Top   :=Top;
  rst.Rect.Right :=Left + Width;
  rst.Rect.Bottom:=Top  + Height;
 END {upd_Rectangle_Select_Tool};

 PUBLIC PROCEDURE put_WebBrowser(dlg        : DIALOG;
                                 Page,
                                 Id,
                                 Url        : STRING[];
                                 Left,
                                 Top,
                                 Width,
                                 Height     : NUMERIC[0];
                                 BorderStyle: STRING[]);
  VAR
   wb: WEB_BROWSER_CONTROL;
 BEGIN
  L_General.CallUs(~!Id);
  wb:=WEB_BROWSER_CONTROL.Create(Left, Top, Left+Width, Top+Height);
  wb.Id:=Id;
  IF BorderStyle # br_st_None THEN
   wb.Border:=1;
  END;
  wb.Url:=Url;
  wb.DragScrolling:=1;
  wb.Visible:=0;
  AddDEBUGToolTipText(wb, Id);
  AddDEBUGBorder(wb);
  dlg.Controls.Add(wb);
  pc_AddId(Id, Page);
 END {put_WebBrowser};

 PUBLIC PROCEDURE put_ActiveX(dlg        : DIALOG;
                              Page,
                              Id,
                              Content    : STRING[];
                              Left,
                              Top,
                              Width,
                              Height     : NUMERIC[0];
                              BorderStyle: STRING[]);
  VAR
   c: CONTROL;
 BEGIN
  L_General.CallUs(~!Id);
  IF IsAcceptableIEVersion() THEN
   c:=ACTIVEX_CONTROL.Create(Left, Top, Left+Width, Top+Height);
   c.Content:=Content;
  ELSE
   c:=STATIC_TEXT.Create(Left, Top, Left+Width, Top+Height);
   SetControlFgColor(c, RGBCOLOR.Create(255, 255, 255));
   SetControlBkColor(c, RGBCOLOR.Create(128, 0,   0));
   c.MaxCharactersInRow:=15;
   c.Style.Font.Weight :=FONT.Weight_Bold;
   c.Text              :=ConvText(" IE!!!");
  END;
  c.Id:=Id;
  IF BorderStyle # br_st_None THEN
   c.Border:=1;
  END;
  c.Visible:=0;
  AddDEBUGToolTipText(c, Id);
  AddDEBUGBorder(c);
  dlg.Controls.Add(c);
  pc_AddId(Id, Page);
 END {put_ActiveX};

 PUBLIC PROCEDURE ShowPictInActiveX(dlg: DIALOG; Id: STRING[]; PictFullPath: STRING[]);
  VAR
   c        : CONTROL;
   WidthAttr: STRING[];
 BEGIN
  IF IsAcceptableIEVersion() THEN
   c:=dlg.Controls.Item(Id);
   IF ~!PictFullPath THEN
    WidthAttr:='width:' + L_Str.IntStr((c.Rect.Right-c.Rect.Left-2) * dlg.ScaleX) + ';' +
               'height:' + L_Str.IntStr((c.Rect.Bottom-c.Rect.Top-2) * dlg.ScaleY) + ';';
    c.Content:='MSHTML:<IMG src = "' + PictFullPath + '"' +
                          ' STYLE="position:absolute; TOP:1px; LEFT:1px; ' + WidthAttr + '"' +
                          '/>';
    c.Scroll :=0;
   ELSE
    c.Content:='about:blank';
   END;
   c.Redraw();
  END;
 END {ShowPictInActiveX};

 PUBLIC PROCEDURE SetTaskBarVisible(Visible: SCAL[2]);
 BEGIN
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   TASKBAR.Create().Visible:=Visible;
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode # 420 THEN
    RAISE();
   END;
  ELSE
   RAISE();
  END; (* TRY *)
 END {SetTaskBarVisible};

 PUBLIC PROCEDURE extCreateDialog(FullScreen: SCAL[2]): DIALOG;
  VAR
   dlg: DIALOG;
 BEGIN
  SetTaskBarVisible(~FullScreen);
  dlg:=DIALOG.Create();
  dlg.ReplaceMainWindow   :=1;
  dlg.StandardClientWidth :=normalWidth;
  dlg.StandardClientHeight:=normalHeight;

  IF FullScreen THEN
   dlg.WindowSizeStatus :=DIALOG.WindowSizeStatus_Maximized;
   dlg.WindowStyle      :=DIALOG.WindowStyle_Popup;
  ELSE
   dlg.WindowSizeStatus :=DIALOG.WindowSizeStatus_Restored;
   dlg.WindowStyle      :=DIALOG.WindowStyle_FixedAspectRatio;
   dlg.ClientRect.Right :=dlg.ClientRect.Left + normalWidth;
   dlg.ClientRect.Bottom:=dlg.ClientRect.Top  + normalHeight;
  END;
  dlg.Redraw();
  RETURN dlg;
 END {extCreateDialog};

 PUBLIC PROCEDURE CreateDialog(FullScreen: SCAL[2]): DIALOG;
 BEGIN
  dlgPagesCollection:=L_General.CreateDictionary();
  RETURN extCreateDialog(FullScreen);
 END {CreateDialog};

 PUBLIC PROCEDURE CreateNoSizebleDialog(WS      : SCAL[4];
                                        Width,
                                        Height  : NUMERIC[0];
                                        Title   : STRING[];
                                        OnCloseP: PROC): DIALOG;
  VAR
   dlg: DIALOG;
 BEGIN
  dlg                  :=extCreateDialog(0);
  dlg.ReplaceMainWindow:=0;
  dlg.CaptionText      :=Title;
  dlg.OnClose          :=OnCloseP;
  dlg.WindowStyle      :=WS;
  dlg.Redraw(); (*  ⮣  묨 樥 ScaleX, ScaleY *)
  dlg.StandardClientWidth :=Width;
  dlg.StandardClientHeight:=Height;
  dlg.ClientRect.Right    :=dlg.ClientRect.Left + Width;
  dlg.ClientRect.Bottom   :=dlg.ClientRect.Top  + Height;
  RETURN dlg;
 END {CreateNoSizebleDialog};

 PUBLIC PROCEDURE ConvertPixelsRect2HimetricRect(PixRect: IRECT): IRECT;
  VAR
   Result: IRECT;
 BEGIN
  Result:=IRECT.Create();
  PIXELSTOHIMETRIC(Result.Left,  Result.Top,    PixRect.Left,  PixRect.Top);
  PIXELSTOHIMETRIC(Result.Right, Result.Bottom, PixRect.Right, PixRect.Bottom);
  RETURN Result;
 END {ConvertPixelsRect2HimetricRect};

 PUBLIC PROCEDURE CutPieceOfImage(srcFileName      : STRING[];
                                  dstFileName      : STRING[];
                                  SrcPicture       : STATIC_PICTURE;
                                  CropRect         : RECTANGLE_SELECT_TOOL;
                                  WidthToHeightCoef: FLOAT);
  VAR
   sW_ScaleCrop,
   sH_ScaleCrop,
   imgScaleX,
   imgScaleY    : FLOAT;
   ScaleCrop,
   oImgFile     : OBJECT;
 BEGIN
  oImgFile:=CreateOBJECT("Wia.ImageFile");
  oImgFile.LoadFile(srcFileName);

  imgScaleX:=oImgFile.Width / (SrcPicture.Rect.Right  - SrcPicture.Rect.Left);
  imgScaleY:=oImgFile.Height / (SrcPicture.Rect.Bottom - SrcPicture.Rect.Top);

  sW_ScaleCrop:=(CropRect.Rect.Right - CropRect.Rect.Left) * imgScaleX;
  sH_ScaleCrop:=sW_ScaleCrop * WidthToHeightCoef;

  ScaleCrop:=CreateOBJECT("Wia.ImageProcess");
  ScaleCrop.Filters.Add(ScaleCrop.FilterInfos("Crop").FilterID);
  ScaleCrop.Filters.Item(1).Properties("Left").Value  :=(CropRect.Rect.Left  - SrcPicture.Rect.Left) * imgScaleX;
  ScaleCrop.Filters.Item(1).Properties("Top").Value   :=(CropRect.Rect.Top   - SrcPicture.Rect.Top)  * imgScaleY;
  ScaleCrop.Filters.Item(1).Properties("Right").Value :=Math.Max(0, oImgFile.Width - (ScaleCrop.Filters.Item(1).Properties("Left").Value + sW_ScaleCrop));
  ScaleCrop.Filters.Item(1).Properties("Bottom").Value:=Math.Max(0, oImgFile.Height - (ScaleCrop.Filters.Item(1).Properties("Top").Value  + sH_ScaleCrop));

  oImgFile:=ScaleCrop.Apply(oImgFile);
  FIO.Erase(dstFileName);
  oImgFile.SaveFile(dstFileName);
 END {CutPieceOfImage};

 PRIVATE PROCEDURE OnClose_dlg(dlg: DIALOG);
 BEGIN
  dlg.Visible:=0;
  dlg.Redraw();
 END {OnClose_dlg};

 PRIVATE PROCEDURE OnClick_OkBtn(Self: OBJECT);
 BEGIN
  OnClose_dlg(Self.Parent.Parent);
 END {OnClick_OkBtn};

 PUBLIC PROCEDURE Error(dlg: DIALOG; Text: STRING[]);
  CONST
   fs_dlgErr    = 19;
   sW_OkBtn     = 100;
   sH_OkBtn     = 30;
   sH_MainShift = 3;
  VAR
   dlgErr    : DIALOG;
   CntStr,
   sW_dlgErr,
   sH_dlgErr,
   sW_dlgPar,
   sH_dlgPar : NUMERIC[0];
   OKBtn     : CONTROL;
 BEGIN
  sW_dlgPar:=dlg.Rect.Right  - dlg.Rect.Left + 1;
  sH_dlgPar:=dlg.Rect.Bottom - dlg.Rect.Top  + 1;
  Text     :=ConvText(Text);
  CntStr   :=Math.Max(Str.Length(Text) DIV 40 + 1, 3);

  sH_dlgErr:= CntStr*fs_dlgErr+sH_OkBtn+sH_MainShift;
  sW_dlgErr:= Math.Div(sW_dlgPar, 2);

  dlgErr                     :=DIALOG.Create(dlg);
  dlgErr.OnClose             :=PROCREF(OnClose_dlg);
  dlgErr.WindowSizeStatus    :=DIALOG.WindowSizeStatus_Restored;
  dlgErr.WindowStyle         :=DIALOG.WindowStyle_FixedSize;
  dlgErr.StandardClientWidth :=sW_dlgErr;
  dlgErr.StandardClientHeight:=sH_dlgErr;
  dlgErr.ClientRect.Right    :=dlgErr.ClientRect.Left + dlgErr.StandardClientWidth - 1;
  dlgErr.ClientRect.Bottom   :=dlgErr.ClientRect.Top  + dlgErr.StandardClientHeight- 1;

  add_StaticText(dlgErr, "id_dlgErrText", Text,
                 0, 0, sW_dlgErr+1, sH_dlgErr+1-sH_OkBtn-sH_MainShift,
                 fs_dlgErr, CONTROL.HorizontalAlignment_Center, br_st_None);
  dlgErr.Controls.Item(0).Visible:=1;

  add_Button(dlgErr, "id_OkBtn", "OK", "",
             Math.Div(sW_dlgErr - sW_OkBtn, 2), sH_dlgErr - sH_OkBtn - 2*sH_MainShift,
             sW_OkBtn, sH_OkBtn,
             fs_dlgErr, fn_LucidaConsole, PROCREF(OnClick_OkBtn));
  OKBtn                :=dlgErr.Controls.Item(1);
  OKBtn.Style.Font.Face:=fn_Default;
  OKBtn.Visible        :=1;

  dlgErr.Visible:=1;
  dlgErr.Redraw();
  WHILE dlgErr.Visible DO
   dlgErr.Wait(0.5);
  END;
 END {Error};

 PUBLIC PROCEDURE GetDlgTextSize(Text: STRING[]; VAR W, H: FLOAT);
  CONST
   id_Test = "st_Test";
  VAR
   dlg       : DIALOG;
   WA, WB, WC: FLOAT;
 BEGIN
  L_General.CallUs(L_General.IsAcceptableProcessorVersion(14174030));
  dlg:=extCreateDialog(0);
  add_StaticText(dlg, id_Test, "",
                 0, 0, 100, 100,
                 EMPTYOF(FLOAT),
                 CONTROL.HorizontalAlignment_Left,
                 br_st_None);
  dlg.TextExtent(WA, WB, WC, H, Text, dlg.Controls.Item(id_Test).Style);
  W:=WA + WB + WC;
 END {GetDlgTextSize};

 PUBLIC PROCEDURE SetDlgBkColor(dlg: DIALOG; Color: RGBCOLOR);
  CONST
   id_Background = "st_Background";
   fS_dummy      = 20;
 BEGIN
  add_StaticText(dlg, id_Background, "",
                 0, 0,
                 dlg.ClientRect.Right - dlg.ClientRect.Left,
                 dlg.ClientRect.Bottom - dlg.ClientRect.Top,
                 fS_dummy,
                 CONTROL.HorizontalAlignment_Left,
                 br_st_None);
  SetControlBkColor(dlg.Controls.Item(id_Background), Color);
  dlg.Controls.Item(id_Background).Visible:=1;
 END {SetDlgBkColor};

 PUBLIC PROCEDURE SetToolTipText(c: CONTROL; ToolTipText: STRING[]);
  VAR
   trgId: STRING[];
 BEGIN
  IF IsDEBUGMode() THEN
   trgId:=Str.Item(c.ToolTipText, L_Str.CRLF, 0);
   L_General.CallUs(~!trgId);
   ToolTipText:=trgId + L_Str.CRLF + ToolTipText;
  END;
  c.ToolTipText:=ToolTipText;
 END {SetToolTipText};

END.
