(*
%Copyright   (c) Polak Computer System
%Author      05.03.1999 19:55:22 SAN
%Purpose     祭 ࠧ   
%Type        library
%Specific    SYS
%Modified    05.09.2014 11:39:22 POLAK\Gina
%Environment
*)

 MODULE L_Hld [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_Str;

 PRIVATE CONST
  HolidayNum         = 17;
  SearchTransferDays = 14;

 PRIVATE PERMANENT VAR
  HD,
  tillHD: DATE[32];
  fH,
  fT    : OBJECT;

 PRIVATE PROCEDURE hdId(D: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(D, "DN");
 END {hdId};

 PRIVATE PROCEDURE tdId(D: DATE): STRING[];
 BEGIN
  RETURN FLDTOSTR(D);
 END {tdId};

 PUBLIC PROCEDURE DayIsHoliday(D: DATE): SCAL[2];
  VAR
   i: NUMERIC[0];
 BEGIN
  IF fH.Exists(hdId(D)) THEN
   i:=STRTOFLD(fH.Item(hdId(D)), NUMERIC);
   IF (D >= HD[i]) & (D<=tillHD[i]) THEN
    RETURN 1;
   END;
  END;
  RETURN 0;
 END {DayIsHoliday};

 PRIVATE PROCEDURE IsNST(D: DATE): SCAL[2];
 BEGIN
  RETURN fT.Exists(tdId(D)) # 0;
 END {IsNST};

 PRIVATE PROCEDURE Day_wNST(D: DATE): DATE;
 BEGIN
  IF IsNST(D) THEN
   RETURN STRTOFLD(fT.Item(tdId(D)), DATE);
  ELSE
   RETURN D;
  END;
 END {Day_wNST};

 PRIVATE PROCEDURE IsOriginalRestDay(D: DATE): SCAL[2];
 BEGIN
  RETURN L_DatTim.WEEKDAY(D) > L_DatTim.wd_Fri;
 END {IsOriginalRestDay};

 PRIVATE PROCEDURE IsExcRuleTranferDay(D: DATE): SCAL[2];
  VAR
   Day, Month, Year: NUMERIC[0];
 BEGIN
  UnPkDATE(D, Day, Month, Year);
  RETURN ((Year >= 2013) & (Month = 0) & (Day >=01) & (Day <= 08)) |
         (D = 23\02\2014);
 END {IsExcRuleTranferDay};

 PRIVATE PROCEDURE IsNSTRestDay(D: DATE): SCAL[2];
 BEGIN
  RETURN (L_DatTim.WEEKDAY(Day_wNST(D)) > L_DatTim.wd_Fri) & ~IsExcRuleTranferDay(D);
 END {IsNSTRestDay};

 PRIVATE PROCEDURE intDayIsStandartTransferedHoliday(D: DATE): SCAL[2];
  VAR
   d      : DATE;
   count  : NUMERIC[0];
   DaysAgo: NUMERIC[0];
 BEGIN
  IF L_DatTim.MONTH(D) = 0 THEN
   DaysAgo:=SearchTransferDays;
  ELSE
   DaysAgo:=SearchTransferDays / 2;
  END;
  d    :=D - DaysAgo;
  count:=0;
  WHILE d < D DO
   IF DayIsHoliday(d) & IsNSTRestDay(d) THEN
    INC(count);
   END;
   INC(d);
  END;
  IF count > 0 THEN
   d    :=D;
   count:=0;
   WHILE d > (D - SearchTransferDays) DO
    IF    DayIsHoliday(d) THEN
     IF IsNSTRestDay(d) THEN
      DEC(count);
     END;
    ELSIF ~IsNSTRestDay(d) THEN
     INC(count);
    END;
    IF count = 0 THEN
     RETURN 1;
    END;
    DEC(d);
   END;
  END;
  RETURN 0;
 END {intDayIsStandartTransferedHoliday};

 PRIVATE PROCEDURE DayIsStandartTransferedHoliday(D: DATE): SCAL[2];
 BEGIN
  IF IsOriginalRestDay(D) | DayIsHoliday(D) | IsNST(D) THEN
   RETURN 0;
  ELSE
   RETURN intDayIsStandartTransferedHoliday(D);
  END;
 END {DayIsStandartTransferedHoliday};

 PUBLIC PROCEDURE WeekDay_TransfersConsidered(D: DATE): L_DatTim.tWeekday;
  VAR
   d        : DATE;
   probably,
   count    : NUMERIC[0];
 BEGIN
  IF    DayIsHoliday(D) THEN
   RETURN L_DatTim.wd_Hol;
  ELSIF IsNST(D) THEN
   RETURN L_DatTim.WEEKDAY(Day_wNST(D));
  ELSIF IsOriginalRestDay(D) THEN
   RETURN L_DatTim.WEEKDAY(D);
  ELSIF intDayIsStandartTransferedHoliday(D) THEN
   d:=D + 1;
   count:=0;
   probably:=0;
   WHILE d < (D + SearchTransferDays) DO
    IF DayIsHoliday(d) & IsNSTRestDay(d) THEN
     INC(probably);
    ELSIF DayIsStandartTransferedHoliday(d) THEN
     INC(count, 1 - probably);
    END;
    INC(d);
   END;
   d:=D;
   WHILE count >= 0 DO
    DEC(d);
    IF DayIsHoliday(d) & IsNSTRestDay(d) THEN
     DEC(count);
    END;
   END;
   RETURN L_DatTim.WEEKDAY(d);
  ELSE
   RETURN L_DatTim.WEEKDAY(D);
  END;
 END {WeekDay_TransfersConsidered};

 PUBLIC PROCEDURE DayIsBeforeHoliday(D: DATE): SCAL[2];
 BEGIN
  RETURN ~DayIsHoliday(D) & DayIsHoliday(Day_wNST(D)+1);
 END {DayIsBeforeHoliday};

 PRIVATE PROCEDURE AddTransfer(WD, RD: DATE);
 BEGIN
  IF ~IsExcRuleTranferDay(WD) THEN
   fT.Add(tdId(WD), FLDTOSTR(RD));
  END;
  fT.Add(tdId(RD), FLDTOSTR(WD));
 END {AddTransfer};

 PRIVATE PROCEDURE Add2StepsTransfer(HlD, WD, RD: DATE);
  (*७  WD  RD,   WD ।⥫쭮   믮 ७  ⠭ ࠢ  ࠧ HlD         *)
  (*! HlD   祭  ᪫祭 IsExcRuleTranferDay(), ⮡  RD  ७ᥭ    HlD,   "ࠧ"*)
  (*0101103 26.12.2013 19:39:54 POLAK\Bor*)
 BEGIN
  fT.Add(tdId(WD), FLDTOSTR(RD));
  fT.Add(tdId(RD), FLDTOSTR(HlD));
 END {Add2StepsTransfer};

 PRIVATE PROCEDURE Setup();
  VAR
   i: SCAL[HolidayNum];
 BEGIN
  SETEMPTY(HD, tillHD); (* ⬥ ࠧ*)
  HD[00]:=01\01\1904;
  HD[01]:=02\01\1904;
  HD[02]:=03\01\2005;
  HD[03]:=04\01\2005;
  HD[04]:=05\01\2005;
  HD[05]:=06\01\2013;
  HD[06]:=07\01\1904;
  HD[07]:=08\01\2013;
  HD[08]:=23\02\2002;
  HD[09]:=08\03\1917;
  HD[10]:=01\05\1917;
  HD[11]:=02\05\1917; tillHD[11]:=02\05\2004;
  HD[12]:=09\05\1945;
  HD[13]:=12\06\1992;
  HD[14]:=04\11\2005;
  HD[15]:=07\11\1917; tillHD[15]:=07\11\2004;
  HD[16]:=12\12\1993; tillHD[16]:=12\12\2004;
 
  L_General.CreatePermanentDictionary(fH);
  FOR_EACH i DO
   fH.Add(hdId(HD[i]), L_Str.IntStr(i));
  END;

  L_General.CreatePermanentDictionary(fT);
  AddTransfer(06\05\2000, 08\05\2000);
  AddTransfer(04\11\2000, 06\11\2000);
  AddTransfer(09\12\2000, 11\12\2000);
  AddTransfer(11\03\2001, 09\03\2001);
  AddTransfer(28\04\2001, 30\04\2001);
  AddTransfer(09\06\2001, 11\06\2001);
  AddTransfer(29\12\2001, 31\12\2001);
  AddTransfer(27\04\2002, 03\05\2002);
  AddTransfer(18\05\2002, 10\05\2002);
  AddTransfer(10\11\2002, 08\11\2002);
  AddTransfer(15\12\2002, 13\12\2002);
  AddTransfer(04\01\2003, 03\01\2003);
  AddTransfer(05\01\2003, 06\01\2003);
  AddTransfer(21\06\2003, 13\06\2003);
  AddTransfer(05\03\2005, 07\03\2005);
  AddTransfer(14\05\2005, 10\05\2005);
  AddTransfer(26\02\2006, 24\02\2006);
  AddTransfer(06\05\2006, 08\05\2006);
  AddTransfer(28\04\2007, 30\04\2007);
  AddTransfer(09\06\2007, 11\06\2007);
  AddTransfer(29\12\2007, 31\12\2007);
  AddTransfer(04\05\2008, 02\05\2008);
  AddTransfer(07\06\2008, 13\06\2008);
  AddTransfer(01\11\2008, 03\11\2008);
  AddTransfer(11\01\2009, 09\01\2009);
  AddTransfer(27\02\2010, 22\02\2010);
  AddTransfer(13\11\2010, 05\11\2010);
  AddTransfer(05\03\2011, 07\03\2011);
  AddTransfer(11\03\2012, 09\03\2012);
  AddTransfer(28\04\2012, 30\04\2012);
  AddTransfer(05\05\2012, 07\05\2012);
  AddTransfer(12\05\2012, 08\05\2012);
  AddTransfer(09\06\2012, 11\06\2012);
  AddTransfer(29\12\2012, 31\12\2012);
  AddTransfer(05\01\2013, 02\05\2013);
  AddTransfer(06\01\2013, 03\05\2013);
  AddTransfer(23\02\2013, 10\05\2013);
  AddTransfer(04\01\2014, 02\05\2014);
  AddTransfer(05\01\2014, 13\06\2014);
  Add2StepsTransfer(23\02\2014, 24\02\2014, 03\11\2014);
  AddTransfer(03\01\2015, 09\01\2015);
  AddTransfer(04\01\2015, 04\05\2015);
 END {Setup};

BEGIN
 IF ~VAR_IMPL.Get(HD).Initialized[0] THEN
  Setup();
 END;
END.
(*
७  2015  ᫥騥 室 :
  㡡 3 ﭢ  ⭨ 9 ﭢ;
  ᥭ 4 ﭢ  쭨 4 .
७  2014  ᫥騥 室 :
  㡡 4 ﭢ  ⭨ 2 ;
  ᥭ 5 ﭢ  ⭨ 13 ;
  쭨 24 䥢ࠫ  쭨 3 .
७  2013  ᫥騥 室 :
  㡡 5 ﭢ  ⢥ 2 ;
  ᥭ 6 ﭢ  ⭨ 3 ;
  쭨 25 䥢ࠫ  ⭨ 10 .
७  2012  ᫥騥 室 :
  ᥭ 11   ⭨ 9 ;
  㡡 28 ५  쭨 30 ५;
  㡡 5   쭨 7 ;
  㡡 12   ୨ 8 ;
  㡡 9   쭨 11 ;
  㡡 29   쭨 31 .
७  2011  室 
  㡡       5   쭨    7 .
७  2010  ᫥騥 室 :
  㡡 27 䥢ࠫ  쭨 22 䥢ࠫ;
  㡡 13   ⭨ 5 .
७  2009  室 
  ᥭ  11 ﭢ   ⭨      9 ﭢ.
७  2008  ᫥騥 室 :
  ᥭ   4       ⭨      2 ;
  㡡       7      ⭨     13 ;
  㡡       1    쭨  3 .
७  2007  ᫥騥 室 :
  㡡      28 ५   쭨 30 ५;
  㡡       9      쭨 11 ;
  㡡      29   쭨 31 .
७  2006  室 :
  ᥭ  26 䥢ࠫ  ⭨     24 䥢ࠫ
  㡡 6   쭨 8 
७  2005  室 
  㡡     14       ୨     10 .
७  2005  ᫥騥 室 :
  㡡      5     쭨  7 ;
७  2003  ᫥騥 室 :
  㡡      4 ﭢ   ⭨      3 ﭢ;
  ᥭ  5 ﭢ   쭨  6 ﭢ;
  㡡     21      ⭨     13 .
७  2002  ᫥騥 室 :
  㡡     27 ५   ⭨      3 ;
  㡡     18       ⭨     10 ;
  ᥭ 10    ⭨      8 ;
  ᥭ 15   ⭨     13 .
७  2001  ᫥騥 室 :
  ᥭ 11     ⭨      9 ;
  㡡     28 ५   쭨 30 ५;
  㡡      9      쭨 11 ;
  㡡     29   쭨 31 .
७  2000  ᫥騥   (室 ):
  㡡      6       쭨  8 ;
  㡡      4    쭨  6 ;
  㡡      9   쭨 11 .
*)