(*
%Copyright   (c) Polak IT
%Author      28.01.2010 19:23:39 POLAK\LDA
%Purpose
%Type        control,report,temporary,convertor,operation,library,to paste
%Specific    AP,AP_PC,AP_PT,SYS,TOOL,LOC
%Modified    17.09.2013 14:17:49 POLAK\LDA
%Environment
*)

 MODULE L_IsSData [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_WDO, L_MDoc, P_Math, L_MathR, P_Cntr, L_Acnt, L_WDO, L_General, E_Bills, Math, L_Str,
        P_MDoc, P_OwnData;

 PUBLIC TYPE
  tFieldModifiableArray = SCAL[2][10];

 PUBLIC CONST
  fmi_Math     = 0;
  fmi_Amount   = 1;
  fmi_opManner = 2;
  fmi_iPrice   = 3;
  fmi_oPrice   = 4;
  fmi_pPrice   = 5;
  fmi_Extra_0  = 6;
  fmi_Extra_1  = 7;
  fmi_Round    = 8;

  mt_Set             = 0;
  mt_CellOfSet       = 1;
  mt_Composite       = 2;
  mt_CellOfComposite = 3;
  mt_Single          = 4;

  i_Extra_0 = 0;
  i_Extra_1 = 1;

  m_Pool = "㬬|業|㬬 ";

  m_Sum       = 0;
  m_Extra     = 1;
  m_Sum_InRub = 2;

  idm_View    = 0;
  idm_Edit    = 1;
  idm_Rebuild = 2;

  c_Rouble = 1;
  c_Dollar = 2;

 PUBLIC PROCEDURE AcceptableMathForCalculate(_Type: LIKE(IsSData.ISDMType)): SCAL[2];
 BEGIN
  RETURN (_Type = mt_CellOfSet) | (_Type = mt_Composite) | (_Type = mt_Single) ;
 END {AcceptableMathForCalculate};

 PUBLIC PROCEDURE MathRequiredRound(_Type: LIKE(IsSData.ISDMType)): SCAL[2];
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Bills.id_MathRequiredRound);
  IF ~!Local THEN
   RETURN Local(_Type);
  ELSE
   RETURN (_Type = mt_Set) | (_Type = mt_Composite) | (_Type = mt_Single);
  END;
 END {MathRequiredRound};

 PUBLIC PROCEDURE SetFieldModifiable(FieldModifiable: SCAL[2][]);
 BEGIN
  L_WDO.SetFieldModifiable("IsSData.Math",     FieldModifiable[fmi_Math]);
  L_WDO.SetFieldModifiable("IsSData.Amount",   FieldModifiable[fmi_Amount]);
  L_WDO.SetFieldModifiable("IsSData.opManner", FieldModifiable[fmi_opManner]);
  L_WDO.SetFieldModifiable("IsSData.iPrice",   FieldModifiable[fmi_iPrice]);
  L_WDO.SetFieldModifiable("IsSData.oPrice",   FieldModifiable[fmi_oPrice]);
  L_WDO.SetFieldModifiable("IsSData.pPrice",   FieldModifiable[fmi_pPrice]);
  L_WDO.SetFieldModifiable("IsSData.Extra[0]", FieldModifiable[fmi_Extra_0]);
  L_WDO.SetFieldModifiable("IsSData.Extra[1]", FieldModifiable[fmi_Extra_1]);
  L_WDO.SetFieldModifiable("IsSData.Round",    FieldModifiable[fmi_Round]);
 END {SetFieldModifiable};

 PUBLIC PROCEDURE sfm_OnBeforeInsert();
  VAR
   FieldModifiable: tFieldModifiableArray;
 BEGIN
  SETZERO(FieldModifiable);
  FieldModifiable[fmi_Math]  :=1;
  FieldModifiable[fmi_Amount]:=1;
  SetFieldModifiable(FieldModifiable);
 END {sfm_OnBeforeInsert};

 PUBLIC PROCEDURE smf_OnBeforeEdit();
  VAR
   FieldModifiable: tFieldModifiableArray;
   CompositeManner: LIKE(IsSData.opManner);
 BEGIN
  SETEMPTY(CompositeManner);
  IF IsSData.ISDMType = mt_CellOfComposite THEN
   SAVE(IsSData);
   SELECT(IsSData, MDoc, , IsSData.MDocKind, IsSData.MDoc, IsSData.MDLNbr, 1);
   IF IsSData.ISDMType = mt_Composite THEN
    CompositeManner:=IsSData.opManner;
   END;
   RESTORE(IsSData);
  END;

  SETZERO(FieldModifiable);
  IF    AcceptableMathForCalculate(IsSData.ISDMType) THEN
   IF    IsSData.opManner = m_Sum THEN
    FieldModifiable[fmi_oPrice]:=1;
   ELSIF IsSData.opManner = m_Sum_InRub THEN
    FieldModifiable[fmi_pPrice]:=1;
   ELSIF IsSData.opManner = m_Extra THEN
    IF IsSData.ISDMType # mt_Composite THEN
     FieldModifiable[fmi_iPrice]:=1;
    END;
    FieldModifiable[fmi_Extra_0]:=1;
   END;
  ELSIF (IsSData.ISDMType = mt_CellOfComposite) & ((CompositeManner = m_Extra) | !CompositeManner) THEN
   FieldModifiable[fmi_iPrice]:=1;
  END;

  SEARCH(MDocList, MDoc, , IsSData.MDocKind, IsSData.MDoc, IsSData.MDLNbr);
  IF MathRequiredRound(IsSData.ISDMType) THEN
   FieldModifiable[fmi_Round]:=1;
  END;

  SetFieldModifiable(FieldModifiable);
 END {smf_OnBeforeEdit};

 PUBLIC PROCEDURE IsCompositeMath(_Math: LIKE(Math.Math); D: DATE): SCAL[2];
  VAR
   Dummy: STRING[];
 BEGIN
  SETEMPTY(Dummy);
  RETURN P_Math.IsCompositeMath(_Math, D) & !P_Math.GetPPrice(_Math, D, 10, 0, Dummy);
 END {IsCompositeMath};

 PUBLIC PROCEDURE E_GetISDMTypeByCurMDL(): LIKE(IsSData.ISDMType);
 BEGIN
  IF    IsCompositeMath(MDocList.Math, MDoc.Date) THEN
   RETURN mt_Composite;
  ELSIF MDocList.MMode = L_MathR.mm_Level2 THEN
   RETURN mt_CellOfSet;
  ELSIF L_MathR.E_IsComplexMath(MDoc.Date) THEN
   RETURN mt_Set;
  ELSE
   RETURN mt_Single;
  END;
 END {E_GetISDMTypeByCurMDL};

 PUBLIC PROCEDURE RoundMoney(s, r: FLOAT): NUMERIC[2];
 BEGIN
  IF r = 0 THEN
   r:=0.001;
  END;
  RETURN Math.Round(s*100, r*100)/100;
 END {RoundMoney};

 PRIVATE PROCEDURE GetRound(d: DATE): NUMERIC[2];
 BEGIN
  IF ~!IsSData.Round THEN
   RETURN IsSData.Round;
  ELSE
   RETURN 0;
  END;
 END {GetRound};

 PUBLIC PROCEDURE GetSum_wExtra(_Price, _Extra: FLOAT): FLOAT;
 BEGIN
  IF ~!_Extra THEN
   RETURN _Price * (1 + _Extra / 100);
  ELSE
   RETURN _Price;
  END;
 END {GetSum_wExtra};

 PUBLIC PROCEDURE GetFullpPrice(): LIKE(IsSData.Total);
 BEGIN
  RETURN GetSum_wExtra(IsSData.pPrice, IsSData.Extra[i_Extra_1]);
 END {GetFullpPrice};

 PUBLIC PROCEDURE GetTotal(): LIKE(IsSData.Total);
 BEGIN
  IF ~!IsSData.pPrice THEN
   RETURN GetFullpPrice() * IsSData.Amount;
  ELSE
   RETURN L_General.EMPTY_NUMBER();
  END;
 END {GetTotal};

 PUBLIC PROCEDURE GetInclNetto(_Math: LIKE(Math.Math); _Total: FLOAT): FLOAT;
 BEGIN
  IF P_Cntr.GetSpecialAttitudeToVAT(MDoc.Contract, MDoc.Date) # P_Cntr.satVAT_woFeatures THEN
   RETURN _Total;
  ELSE
   RETURN _Total - L_Acnt.GetInclMathVAT(_Math, MDoc.Date, _Total);
  END;
 END {GetInclNetto};

 PUBLIC PROCEDURE GetExclVAT(_Math: LIKE(Math.Math); _Total: FLOAT): FLOAT;
 BEGIN
  IF P_Cntr.GetSpecialAttitudeToVAT(MDoc.Contract, MDoc.Date) = P_Cntr.satVAT_woFeatures THEN
   RETURN L_Acnt.GetExclMathVAT(_Math, MDoc.Date, _Total);
  ELSE
   RETURN 0;
  END;
 END {GetExclVAT};

 PUBLIC PROCEDURE GetCurrencyAttr(VAR usd     : NUMERIC[4];
                                  VAR FinExtra: NUMERIC[3];
                                  VAR BillCU  : NUMERIC[6]);
  VAR
   trgValue: STRING[];
 BEGIN
  trgValue:=P_MDoc.GetMDocProStrong(MDoc.MDocKind, MDoc.MDoc, P_MDoc.P_BilCURate);
  BillCU  :=STRTOFLD(L_Str.AnyItem(trgValue, ";", 0), NUMERIC);
  usd     :=STRTOFLD(L_Str.AnyItem(trgValue, ";", 1), NUMERIC);
  FinExtra:=STRTOFLD(L_Str.AnyItem(trgValue, ";", 2), NUMERIC);
 END {GetCurrencyAttr};

 PUBLIC PROCEDURE GetBillCU(): FLOAT;
  VAR
   usd     : NUMERIC[4];
   FinExtra: NUMERIC[3];
   BillCU  : FLOAT;
 BEGIN
  IF IsSData.Crncy = c_Rouble THEN
   RETURN 1;
  ELSE
   CONFORM(IsSData, MDoc);
   GetCurrencyAttr(usd, FinExtra, BillCU);
   RETURN BillCU;
  END;
 END {GetBillCU};

 PUBLIC PROCEDURE GetPublicPrice(BillCU: FLOAT): LIKE(IsSData.pPrice);
 BEGIN
  IF ~!IsSData.oPrice THEN
   RETURN IsSData.oPrice * BillCU;
  ELSE
   RETURN L_General.EMPTY_NUMBER();
  END;
 END {GetPublicPrice};

 PUBLIC PROCEDURE ReCalcTotals();
  VAR
   id_Total,
   id_Income,
   SingleIncome: NUMERIC[2];
 BEGIN
  SETZERO(id_Total, id_Income);
  SAVE(IsSData);
  SELECT(IsSData, MDoc, , IsSData.MDocKind, IsSData.MDoc);
  WHILE OCCURE(IsSData) DO
   IF ~!IsSData.Total THEN
    INC(id_Total,  IsSData.Total);
   END;
   IF ~!IsSData.oPrice & ~!IsSData.iPrice THEN
    SingleIncome:=IsSData.Amount * (IsSData.oPrice - IsSData.iPrice);
    IF IsSData.Crncy # c_Rouble THEN
     SingleIncome:=SingleIncome * GetBillCU();
    END;
    INC(id_Income, SingleIncome);
   END;
   NEXT(IsSData);
  END; (* WHILE OCCURE(IsSData) ... *)
  RESTORE(IsSData);

  L_WDO.wz_setFieldValue("IsSData", 0, "v_IsSData_Total",  id_Total);
  L_WDO.wz_setFieldValue("IsSData", 0, "v_IsSData_Income", id_Income );
  L_WDO.wz_setFieldValue("IsSData", 0, "v_IsSDataTotal_Warning", L_Str.PoolItem(" |!", (id_Total # 0) & (id_Total # MDoc.Total)));
 END {ReCalcTotals};

 PUBLIC PROCEDURE CalcCompositePrice(_MDLNbr: LIKE(IsSData.ISDNbr));
  VAR
   CompositePrice: NUMERIC[5];
 BEGIN
  SETZERO(CompositePrice);

  SELECT(IsSData, MDoc, , MDoc.MDocKind, MDoc.MDoc, _MDLNbr);
  WHILE OCCURE(IsSData) DO
   IF (IsSData.ISDMType = mt_CellOfComposite) &
      (~!CompositePrice) THEN
    IF ~!IsSData.iPrice THEN
     INC(CompositePrice, IsSData.Amount * IsSData.iPrice);
    ELSE
     SETEMPTY(CompositePrice);
    END;
   END;
   NEXT(IsSData);
  END; (* WHILE OCCURE(IsSData) ... *)

  SELECT(IsSData, MDoc, , MDoc.MDocKind, MDoc.MDoc, _MDLNbr, 1);
  IF IsSData.ISDMType = mt_Composite THEN
   IsSData.iPrice:=CompositePrice;
   REPLACE(IsSData);
  END;
 END {CalcCompositePrice};

 PUBLIC PROCEDURE ReCalcSingleMath(wRecalcTotals: SCAL[2]);
  VAR
   BillCU: FLOAT;
 BEGIN
  BillCU:=GetBillCU();
  IF IsSData.opManner = m_Sum THEN
   IsSData.pPrice:=GetPublicPrice(BillCU);
   IF ~!IsSData.iPrice THEN
    IsSData.Extra[i_Extra_0]:=(IsSData.oPrice / IsSData.iPrice - 1) * 100;
   ELSE
    SETEMPTY(IsSData.Extra[i_Extra_0]);
   END;
  ELSIF IsSData.opManner = m_Sum_InRub THEN
   IsSData.oPrice    :=IsSData.pPrice / BillCU;
   IF ~!IsSData.iPrice & (IsSData.iPrice # 0) THEN
    IsSData.Extra[i_Extra_0]:=(IsSData.oPrice / IsSData.iPrice - 1) * 100;
   ELSE
    SETEMPTY(IsSData.Extra[i_Extra_0]);
   END;
  ELSIF IsSData.opManner = m_Extra THEN
   IF ~!IsSData.iPrice THEN
    IsSData.oPrice:=GetSum_wExtra(IsSData.iPrice, IsSData.Extra[i_Extra_0]);
    IsSData.pPrice:=GetPublicPrice(BillCU);
   ELSE
    SETEMPTY(IsSData.oPrice, IsSData.pPrice);
   END;
  END;

  IF ~!IsSData.pPrice THEN
   IsSData.Total :=GetTotal();
  END;

  REPLACE(IsSData);
  IF wRecalcTotals THEN
   ReCalcTotals();
  END;

  IF IsSData.ISDMType = mt_CellOfComposite THEN
   SAVE(IsSData);
   CalcCompositePrice(IsSData.MDLNbr);
   ReCalcSingleMath(wRecalcTotals);
   RESTORE(IsSData);
  END;
 END {ReCalcSingleMath};

 PUBLIC PROCEDURE StandOnIsSData();
  VAR
   _Type: LIKE(IsSData.ISDMType);
 BEGIN
(*
  IF IsCompositeMath(MDocList.Math, MDoc.Date) THEN
   _Type:=mt_Composite;
  ELSE
   _Type:=E_GetISDMTypeByCurMDL();
  END;
*)
(*!!!*)
  L_General.Assert(SEARCH(IsSData, MDoc, MDoc.MDocKind, MDoc.MDoc, MDocList.MDLNbr, 01),
                   " ⠢    㬥 ᮣᮢ.|" +
                   "室 믮 楤 ନ஢ ");
 END {StandOnIsSData};

 PUBLIC PROCEDURE ip_GetDefaultCurrency(): LIKE(Crncy.Crncy);
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Bills.ip_GetDefaultCurrency);
  IF ~!Local THEN
   RETURN Local();
  ELSE
   RETURN c_Rouble;
  END;
 END {ip_GetDefaultCurrency};

 PUBLIC PROCEDURE id_FillAppendix();
  VAR
   usd     : NUMERIC[4];
   BillCU  : FLOAT;
   FinExtra: NUMERIC[3];
   Appendix: STRING[];
 BEGIN
  SETEMPTY(Appendix);
  IF ip_GetDefaultCurrency() = c_Dollar THEN
   GetCurrencyAttr(usd, FinExtra, BillCU);
   Appendix:="   (USD) "       + FLDTOSTR(usd,      "4.") + "                " +
             "     " + FLDTOSTR(FinExtra, "2.") + "                 " +
             "  "           + FLDTOSTR(BillCU,   "6.");
  END;
  APP.Get().Forms("IsSData").Zones(0).Fields("v_Appendix").Value:=Appendix;
 END {id_FillAppendix};

 PUBLIC PROCEDURE SetMDocListSum(Brutto: NUMERIC[2]; trgRound: LIKE(IsSData.Round));
 BEGIN
  MDocList.Sum:=RoundMoney(GetInclNetto(MDocList.Math, Brutto), trgRound);
  MDocList.VAT:=Brutto - MDocList.Sum;
  REPLACE(MDocList);
  INC(MDoc.Total, MDocList.VAT+MDocList.Sum);
 END {SetMDocListSum};

 PUBLIC PROCEDURE SaveBillCU(trgMDocKind: LIKE(MDoc.MDocKind);
                             trgMDoc    : LIKE(MDoc.MDoc);
                             usd        : NUMERIC[4];
                             FinExtra   : NUMERIC[3];
                             force      : SCAL[2]);
  VAR
   doIns: SCAL[2];
 BEGIN
  IF force THEN
   P_MDoc.RemoveMDocPro(trgMDocKind, trgMDoc, P_MDoc.P_BilCURate);
   doIns:=1;
  ELSE
   doIns:=~SELECT(MDocPro, MDoc, trgMDocKind, trgMDoc, P_MDoc.P_BilCURate);
  END;

  IF doIns THEN
   P_MDoc.InsMDocPro(trgMDocKind, trgMDoc, P_MDoc.P_BilCURate, 1, FLDTOSTR(usd * (1 + FinExtra / 100), "6.", 0) + ";" +
                                                                  FLDTOSTR(usd, "4.", 0) + ";" +
                                                                  FLDTOSTR(FinExtra, "2.", 0));
  END;
 END {SaveBillCU};

 PUBLIC PROCEDURE SaveDefaultBillCU(trgMDocKind: LIKE(MDoc.MDocKind);
                                    trgMDoc    : LIKE(MDoc.MDoc));
 BEGIN
  SAVE(MDoc);
  SEARCH(MDoc, MDoc, , trgMDocKind, trgMDoc);
  SaveBillCU(MDoc.MDocKind, MDoc.MDoc, P_OwnData.USDRate(MDoc.Date), 3.0, 0);
  RESTORE(MDoc);
 END {SaveDefaultBillCU};

END.
