(*
%Copyright   (c) Polak IT
%Author      10.11.2006 18:30:42 POLAK\Bor
%Purpose
%Type        library
%Specific    SYS
%Modified    05.06.2014 19:14:32 ?
%Environment
*)

 MODULE L_MSWStp [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_StdFrm, L_Str, L_Math, FIO, Shell, Str, Reg, L_FIO, L_Guard, L_DatTim;

 PUBLIC VAR
  HasError         : SCAL[2];
  MakeReport       : SCAL[2];

  MSExcelVersion,
  MSOfficeFolder   : STRING[];
  IsExcelXPAndWin9x: SCAL[2];

 PUBLIC CONST
  aslDBModification_Off                = 0;
  aslDBModification_InitializationOnly = 1;
  aslDBModification_On                 = 2;

  AutoparkLNKName = "AUTOPARK.LNK";

 PRIVATE CONST
  DigitValue      = 0;
  StringValue     = 1;

  ODBCRegPath      = "HKLM\Software\ODBC\ODBC.INI";
  JetRegPath_30    = "HKLM\Software\Microsoft\Jet\3.0\Engines\Text";
  JetRegPath_35    = "HKLM\Software\Microsoft\Jet\3.5\Engines\Text";
  JetRegPath_40    = "HKLM\Software\Microsoft\Jet\4.0\Engines\Text";

  MSWindowsVersion_asStr = "ଠ                : ";
  MSOfficePath_asStr     = " MS Office          : ";
  ASBMSOfficePath_asStr  = " 䠩 ஥     : ";
  MSExcelVersion_asStr   = " MS Excel          : ";
  DstFontsFolder_asStr   = " ࠭ ⮢ : ";
  SrcFontsFolder_asStr   = "室  ⮢   : ";

  AutoparkXLSName = "AUTOPARK.XLS";

  jrp_CharacterSet = "CharacterSet";
  jrp_Format       = "Format";

  DialogWidth     = 74;
  LineWidth       = 70;
  LeftWidth       = 30;
  Shift           = "   ";

 PUBLIC PROCEDURE NeedElevation(): SCAL[2];
  VAR
   Platform: L_General.tOSPlatform;
   osMajor : NUMERIC[0];
 BEGIN
  OSVERSION(Platform, osMajor);
  RETURN (Platform = L_General.osp_WinNT) & (osMajor >= 6) & ~ISLOCALADMIN();
 END {NeedElevation};

 PRIVATE PROCEDURE IsNonInteractiveSession(): SCAL[2];
 BEGIN
  RETURN L_Guard.IsGuardConnection(APP.Get().MyConnection) |
         L_General.BatchExecution_IsRunning()              |
         L_General.KioskExecution_IsRunning();
 END {IsNonInteractiveSession};

 PUBLIC PROCEDURE GetElevatedASL(ModuleName: STRING[]; aslDBModificationMode: SCAL[3]): OBJECT;
  VAR
   v32,
   asl  : OBJECT;
   DEBUG: SCAL[2];
 BEGIN
  IF NeedElevation() THEN
   L_General.CallUs(~IsNonInteractiveSession());
   v32:=CREATE_ELEVATED_OBJECT("ASB.Application");
   v32.OpenProject(FIO.UNCPath(FIO.BuildPath(DATABASE.Get().Name, "red", FIO.AsbSpecialFolders("ClientRedFileDir"))), 0);
   DEBUG:=0;
   DEBUGBREAK();
   asl  :=v32.CreateASLEngine(ModuleName, aslDBModificationMode, DEBUG);
  ELSE
   SETEMPTY(asl);
  END;
  RETURN asl;
 END {GetElevatedASL};

 PRIVATE PROCEDURE ScodeIsError(Scode: NUMERIC[0]): SCAL[2];
 BEGIN
  RETURN (Scode < 0);
 END {ScodeIsError};
 
 PRIVATE PROCEDURE ScodeFacility(Scode: NUMERIC[0]): NUMERIC[0];
  VAR
   a,b,FacilityCode: NUMERIC[0];
 BEGIN
  IF Scode < 0 THEN
   Scode := 2147483648+Scode;
  END;
  a            := (Scode DIV 65536);
  b            := a MOD 8192;
  FacilityCode := (Scode DIV 65536) MOD 8192;
  RETURN FacilityCode;
 END {ScodeFacility};
 
 PUBLIC PROCEDURE ScodeCode(Scode: NUMERIC[0]): NUMERIC[0];
  VAR Code: NUMERIC[0];
 BEGIN
  IF Scode < 0 THEN
   Scode := 2147483648+Scode;
  END;
  Code := Scode MOD 65536;
  IF Code > 32767 THEN
   Code := Code-65536;
  END;
  RETURN Code;
 END {ScodeCode};

 PRIVATE PROCEDURE IsRAISEOrHALTExeption(VAR code: NUMERIC[0]): SCAL[2];
 BEGIN
  IF    code < -10000 THEN
   INC(code, 10000);
   RETURN 1;
  ELSIF code >  10000 THEN
   DEC(code, 10000);
   RETURN 1;
  ELSE
   RETURN 0;
  END;
 END {IsRAISEOrHALTExeption};

 PRIVATE PROCEDURE IsAbortProgByUser(ErrorCode: NUMERIC[0]): SCAL[2];
  VAR
   code: NUMERIC[0];
 BEGIN
  code:=ScodeCode(ErrorCode);
  IF (code >= 513) & (code <= 10000) THEN
   DEC(code, 512);
(*믮 ஢ ASL ࢠ 짮⥫   ⫠稪*)
   RETURN (code = 300) | (code = 301);
  ELSE
   RETURN 0;
  END;
 END {IsAbortProgByUser};

 PRIVATE PROCEDURE IsAbortElevationByUser(ErrorCode: NUMERIC[0]): SCAL[2];
  CONST
   FACILITY_WIN32        = 7;
   WIN32_ERROR_CANCELLED = 1223;
 BEGIN
  RETURN ScodeIsError(ErrorCode) &
         (ScodeFacility(ErrorCode) = FACILITY_WIN32) &
         (ScodeCode(ErrorCode) = WIN32_ERROR_CANCELLED);
 END {IsAbortElevationByUser};

 PUBLIC PROCEDURE AsbApplicationErrorHandler(Exc: Exc_Object);
  CONST
   FACILITY_ITF = 4;
  VAR
   code         : NUMERIC[0];
   RAISERequired: SCAL[2];
 BEGIN
  RAISERequired:=1;
  IF ScodeIsError(Exc.ErrorCode) THEN
   code := ScodeCode(Exc.ErrorCode);
   IF    (ScodeFacility(Exc.ErrorCode) = FACILITY_ITF) & (Exc.Source = "ASB.Application") THEN
    IF    IsRAISEOrHALTExeption(code) THEN
     STDMSG(Exc.MsgOriginalText, STOP, code);
    ELSIF IsAbortProgByUser(Exc.ErrorCode) THEN
     HALT(1);
    END;
   ELSIF IsAbortElevationByUser(Exc.ErrorCode) THEN
    L_General.Error("ॡ 襭 稩 짮⥫.|" +
                    "믮塞     믮|" +
                    " 騬 ﬨ.");
   END;
  END;
  IF RAISERequired THEN
   RAISE();
  END;
 END {AsbApplicationErrorHandler};

 PRIVATE PROCEDURE ExtractCAB(    CABPath   : STRING[];
                              VAR ASBTempDir: STRING[]);
 BEGIN
  ASBTempDir:=L_FIO.CreateTempDir(L_FIO.effp_FileName_woExt(CABPath));
  Shell.Execute(, "extrac32", "/Y " + CABPath + " *.*", ASBTempDir, 0, 1);
 END {ExtractCAB};

 PRIVATE PROCEDURE RegSvr(ComponentName, ComponentExt: STRING[]; Install: SCAL[2]);
  VAR
   Res: NUMERIC[0];
   Cmd: STRING[];
 BEGIN
  PROMPT(L_Str.PoolItem("⠫|⠭", Install));
  IF Install THEN
   SETEMPTY(Cmd);
  ELSE
   Cmd:="/u ";
  END;
  Res:=Shell.Execute(, "regsvr32", Cmd + "/s " + ComponentName + "." + ComponentExt,
                     Shell.SpecialFolders("System"), , 1);
  L_General.Assert(Res = 0,
                   " 㤠 " + L_Str.PoolItem("⠫஢|ॣ஢", Install) +
                   "  " + ComponentName + "." + ComponentExt + ", 訡 " + FLDTOSTR(Res, "0", 0));
 END {RegSvr};

 PUBLIC PROCEDURE _InstallDistributedVBControl(ComponentName,
                                               ComponentExt,
                                               CABPath          : STRING[];
                                               ReInstallRequired: SCAL[2]);
  VAR
   Res          : NUMERIC[0];
   InfPath,
   ASBTempDir,
   ComponentPath: STRING[];
 BEGIN
  L_General.Assert(ISLOCALADMIN(), " 믮  樨 ॡ|" +
                                   " 쭮  ");
  IF ReInstallRequired THEN
   RegSvr(ComponentName, ComponentExt, 0);
  END;

  PROMPT("ᯠ ");
  ExtractCAB(CABPath, ASBTempDir);
  InfPath      :=FIO.BuildPath(ComponentName, "inf", ASBTempDir);
  ComponentPath:=FIO.BuildPath(ComponentName, ComponentExt, ASBTempDir);
  L_General.Assert(FIO.Exists(InfPath) & FIO.Exists(ComponentPath),
                   " 㤠   "+ComponentName+"."+ComponentExt+
                   "  ਡ⨢ "+'"'+CABPath+'"');

  (*  믮  Install,    ஢: ஢ઠ *)
  (* ᨩ, ᨬ, ...  WinXP     "APInst",   *)
  (* ॣ 믮.                                            *)
  PROMPT("஢");
  Res:=Shell.Execute("Install", InfPath, , ASBTempDir, , 1);
  L_General.Assert(Res = 0,
                   " 㤠 ⠫஢  "+
                   ComponentName+"."+ComponentExt+", 訡 "+FLDTOSTR(Res, "0", 0));
  RegSvr(ComponentName, ComponentExt, 1);
  L_FIO.RemoveDir(ASBTempDir);
 END {_InstallDistributedVBControl};

 PRIVATE PROCEDURE InstallDistributedVBControl(ComponentName    : STRING[];
                                               ComponentExt     : STRING[];
                                               CABPath          : STRING[];
                                               ReInstallRequired: SCAL[2]);
  VAR
   asl: OBJECT;
 BEGIN
  IF ~ReInstallRequired THEN
   L_StdFrm.Question("⮡ 믮 , ।⥫쭮 室 믮|" +
                     "⮬ ⠭  " + ComponentName);
  END;
  L_General.Assert(FIO.Exists(CABPath),
                   "ਡ⨢  "+ComponentName+"."+ComponentExt+"  ");
  APP.Get().NewPrompt("⠭  " + ComponentName + "." + ComponentExt);
  IF NeedElevation() THEN
   TRY
    asl := GetElevatedASL("Sys_Env", aslDBModification_Off);
    asl.Proc("L_MSWStp._InstallDistributedVBControl", ComponentName, ComponentExt, CABPath, ReInstallRequired);
   EXCEPT
   ON Exc: Exc_Object DO
    AsbApplicationErrorHandler(Exc);
   END; (* TRY *)
  ELSE
   _InstallDistributedVBControl(ComponentName, ComponentExt, CABPath, ReInstallRequired);
  END;
 END {InstallDistributedVBControl};

 PUBLIC PROCEDURE InstallScriptingDictionary();
 BEGIN
  APP.Get().NewPrompt("⠭  Scripting.Dictionary");
  IF NeedElevation() THEN
   L_General.Error(" ॥  ॣ஢  Scripting.Dictionary.||" +
                   " 室 믮  'regsvr32 scrrun.dll'");
  ELSE
   RegSvr("scrrun", "dll", 1);
  END;
 END {InstallScriptingDictionary};

 PUBLIC PROCEDURE IsReInstallComponentRequired(ComponentName,
                                               ComponentExt,
                                               ComponentCabPath: STRING[]): SCAL[2];
  CONST
   nMSInetVrs = 6001098016;
   sMSInetVrs = "6.1.98.16";
  VAR
   Result                  : SCAL[2];
   Msg,
   ASBTempDir,
   srcComponentFullFileName,
   dstComponentFullFileName: STRING[];
   fv                      : FILE_VERSION;
   msinetversion           : NUMERIC[0];
 BEGIN
  Result                  :=0;
  dstComponentFullFileName:=FIO.BuildPath(ComponentName, ComponentExt, Shell.SpecialFolders("System"));
  IF (ComponentName = "MSINet") & (ComponentExt = "ocx") THEN
   IF FIO.Exists(dstComponentFullFileName) THEN
    fv           :=FILE_VERSION.Get(dstComponentFullFileName);
    msinetversion:=((fv.FileMajor * 1000 + fv.FileMinor) * 1000 + fv.FileBuild) * 1000 + fv.FileSubbuild;
    IF    msinetversion > nMSInetVrs THEN
     IF IsNonInteractiveSession() THEN
      RETURN 0;
     ELSE
      RETURN STDFORM("|⠭   " + ComponentName + " , 祬 ." +
                     "||⠭  " + L_Str.IntStr(fv.FileMajor) + "." +
                                               L_Str.IntStr(fv.FileMinor) + "." +
                                               L_Str.IntStr(fv.FileBuild) + "." +
                                               L_Str.IntStr(fv.FileSubbuild) +
                     ", ॡ  " + sMSInetVrs + "." +
                     "||, ஡ த    " +
                     "| ᮮ ࠧࠡ稪  १." +
                     "|| 砥, ᫨  ⠭ ᨥ  ஡," +
                     "|   롥 ⠭  " + sMSInetVrs +
                     "||" + L_StdFrm.DrawButton("த  ", 27, "X") + "       " +
                     L_StdFrm.DrawButton("⠭  " + sMSInetVrs, 29, "DB"), , INFORM) = 1;
     END;
    ELSIF msinetversion < nMSInetVrs THEN
     Result:=1;
    END;
   END;
  ELSE
   ExtractCAB(ComponentCabPath, ASBTempDir);
   srcComponentFullFileName:=FIO.BuildPath(ComponentName, ComponentExt, ASBTempDir);
   L_General.Assert(FIO.Exists(srcComponentFullFileName), "  䠩 " + srcComponentFullFileName);
 
   IF FIO.Exists(dstComponentFullFileName) THEN
    Result:=FOLDERENTRY.Create(srcComponentFullFileName).Size # FOLDERENTRY.Create(dstComponentFullFileName).Size;
   END;
   L_FIO.RemoveDir(ASBTempDir);
  END;
  IF Result THEN
   Msg:="|⠭   " + ComponentName + " ५." +
        "|| ४⭮ ࠡ " + L_Str.PoolItem("४|室", IsNonInteractiveSession()) + " 믮  ᨨ.";
   IF IsNonInteractiveSession() THEN
    L_General.Error(Msg);
   ELSE
    Result:=STDFORM(Msg +
                    "|| " + L_StdFrm.DrawButton(" ", 27, "DB") + "       " +
                    L_StdFrm.DrawButton("த  ", 27, "X") + " ", , INFORM) = 1;
   END;
  END;
  RETURN Result;
 END {IsReInstallComponentRequired};

 PUBLIC PROCEDURE extInstallDVBCAndCreateObject2(ObjectName,
                                                 LicKey,
                                                 ComponentName,
                                                 ComponentExt,
                                                 ComponentCabPath : STRING[];
                                                 ReInstallRequired: SCAL[2]): OBJECT;
  VAR
   O: OBJECT;
 BEGIN
  IF ReInstallRequired THEN
   InstallDistributedVBControl(ComponentName, ComponentExt, ComponentCabPath, 1);
  END;
  TRY [EXCEPTIONHANDLING(EXCEPTIONHANDLING_OFF)]
   O := CreateOBJECT2(ObjectName, LicKey);
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode = L_General.ec_rt_ClassNotRegistered THEN
    InstallDistributedVBControl(ComponentName, ComponentExt, ComponentCabPath, 0);
   ELSE
    RAISE();
   END;
   O := CreateOBJECT2(ObjectName, LicKey);
  END; (* TRY *)
  RETURN O;
 END {extInstallDVBCAndCreateObject2};

 PUBLIC PROCEDURE InstallDVBCAndCreateObject2(ObjectName,
                                              LicKey,
                                              ComponentName,
                                              ComponentExt,
                                              ComponentCabPath: STRING[]): OBJECT;
 BEGIN
  RETURN extInstallDVBCAndCreateObject2(ObjectName, LicKey, ComponentName, ComponentExt, ComponentCabPath, 0);
 END {InstallDVBCAndCreateObject2};

 PRIVATE PROCEDURE ShowDialog(InfoStr,
                              TextMessage,
                              Buttons,
                              Title       : STRING[]): NUMERIC[0];
  VAR
   R: NUMERIC[0];
 BEGIN
  R:=0;
  RETURN STDFORM("|"+ Str.Align(MSWindowsVersion_asStr + L_General.GetOSName(), DialogWidth, LEFT) + "|" +
                 InfoStr + "|" +
                 "%L" + L_Str.IntStr(LineWidth) + ",10<" + TextMessage + "|>I||||||||||" +
                 Buttons, Title, INFORM_NEUTRAL, , R);
 END {ShowDialog};

 PRIVATE PROCEDURE CondRWRegValue(    RegPartition,
                                      RegParam,
                                      RegValue     : STRING[];
                                      Type         : SCAL[2];
                                  VAR ErrMessage   : STRING[];
                                      CaseSensitive: SCAL[2]);
 BEGIN
  IF MakeReport THEN
   IF ~ReadRegValue(RegPartition, RegParam, RegValue, Type) THEN
    ErrMessage:=ErrMessage + SplitString(Shift + RegParam, RegValue) + "|";
    HasError  :=1;
   END;
  ELSE
   IF Type = StringValue THEN
    L_General.Reg_Str_Setup(Reg.BuildPath(RegPartition, RegParam), RegValue, CaseSensitive);
   ELSE
    L_General.Reg_DWord_Setup(Reg.BuildPath(RegPartition, RegParam), RegValue);
   END;
  END; (* IF *)
 END {CondRWRegValue};

 PRIVATE PROCEDURE ReadRegValue(RegPartition,
                                RegParam,
                                RegValue     : STRING[];
                                Type         : SCAL[2]): SCAL[2];
  VAR
   DValue: NUMERIC[0];
   SValue: STRING[];
   Res   : SCAL[2];
 BEGIN
  SETEMPTY(DValue, SValue);
  IF Type = StringValue THEN
   Res:=Reg.Read(Reg.BuildPath(RegPartition, RegParam), SValue);
  ELSE
   Res:=Reg.Read(Reg.BuildPath(RegPartition, RegParam), DValue);
  END; (* IF *)
  IF ~Res |
     ((Type = StringValue) & (SValue # RegValue)) |
     ((Type = DigitValue) & (L_Math.HexToDec(RegValue) # DValue)) THEN
   RETURN 0;
  ELSE
   RETURN 1;
  END;
 END {ReadRegValue};

 PRIVATE PROCEDURE GetButtonList(): STRING[];
 BEGIN
  IF HasError THEN
   RETURN "|%<  믮  >B                 %<    室    >X";
  ELSE
   RETURN "|%<    室    >X";
  END;
 END {GetButtonList};

 PRIVATE PROCEDURE ISFReturn(Err, FontFileExist: SCAL[2]; RegValue: STRING[]): STRING[];
 BEGIN
  RETURN L_Str.PoolItem("-|", FontFileExist) + " " + Str.Align(RegValue, 55, LEFT) +
         L_Str.PoolItem("⠭| ⠭", Err);
 END {ISFReturn};

 PRIVATE PROCEDURE InstallSingleFont(FontFileName,
                                     SrcFontsDir,
                                     FontsRegPath,
                                     RegParam,
                                     RegValue     : STRING[]): STRING[];
  VAR
   Err,
   FontFileExist: SCAL[2];
   FullFileName,
   DstFileName  : STRING[];
 BEGIN
  FullFileName :=FIO.BuildPath(FontFileName, , SrcFontsDir);
  FontFileExist:=FIO.Exists(FullFileName);
  Err          :=~ReadRegValue(FontsRegPath, RegParam, RegValue, StringValue) |
                 ~FIO.Exists(FIO.BuildPath(FontFileName, , Shell.SpecialFolders("Fonts")));
  IF Err & FontFileExist & ~MakeReport THEN
   DstFileName:=FIO.BuildPath(FontFileName, , Shell.SpecialFolders("Fonts"));
   LOOP
    TRY [MSGSUPPRESSIONMODE(MSGSUPPRESS_ALL)]
     L_FIO.FileCopy(FullFileName, DstFileName);
     EXIT;
    EXCEPT
    ON Exc: Exc_RunTime DO
     IF ~STDFORM("| " + DstFileName + " ᯮ 㣨 짮⥫  ணࠬ." +
                 "||ன  ணࠬ,   ᯮ짮  䠩   ." +
                 "||" + " " + L_StdFrm.DrawButton("⬥", 11, "X") + "       " + L_StdFrm.DrawButton("", 11, "DB") + " ",
                 ,
                 STOP,
                 -1) THEN
      RETURN ISFReturn(1, FontFileExist, RegValue);
     END;
    END; (* TRY *)
   END; (*LOOP*)
   Reg.Write(Reg.BuildPath(FontsRegPath, RegParam), RegValue, "REG_SZ");
   ADDFONTRES(DstFileName);
   Err:=0;
  END;

  IF FontFileExist THEN
   IF Err THEN
    HasError:=Err;
   END;
   RETURN ISFReturn(Err, FontFileExist, RegValue);
  ELSE
   L_General.Assert(~L_General.IsOurEngineeringCopy(), " ᭮ 䠩   䠩 ⮢ " + FontFileName);
   RETURN "";
  END;

 END {InstallSingleFont};

 PUBLIC PROCEDURE _InstallFonts(SrcFontsDir: STRING[]): STRING[];
  CONST
   FontsRegPath = "HKLM\Software\Microsoft\Windows NT\CurrentVersion\Fonts";

   bf_RegParam  = "IDAutomation.com Code39 (TrueType)|IntP36DlTt (TrueType)|EanGnivc (TrueType)|EanP36Tt (TrueType)|PostIndex (TrueType)";
   bf_RegValue  = "IDAUTOMATION.COM CODE 39.TTF|V300019_.TTF|EanGnivc.TTF|EanP000.ttf|PostIndex.ttf";
  VAR
   i                      : NUMERIC[0];
   FontFileName,
   InstallFont_AsStr,
   InstallSingleFont_AsStr: STRING[];
 BEGIN
  i                :=0;
  InstallFont_AsStr:="";
  LOOP
   FontFileName:=L_Str.AnyItem(bf_RegValue, "|", i);
   IF !FontFileName THEN
    EXIT;
   END;
   InstallSingleFont_AsStr:=InstallSingleFont(FontFileName,
                                              SrcFontsDir,
                                              FontsRegPath,
                                              L_Str.AnyItem(bf_RegParam, "|", i),
                                              L_Str.AnyItem(bf_RegValue, "|", i));

   IF ~!InstallSingleFont_AsStr THEN
    InstallFont_AsStr:=InstallFont_AsStr + InstallSingleFont_AsStr + "|";
   END;
   INC(i);
  END; (* LOOP *)
  RETURN InstallFont_AsStr;
 END {_InstallFonts};

 PRIVATE PROCEDURE InstallFonts(SrcFontsDir: STRING[]): STRING[];
  VAR
   asl: OBJECT;
   Res: STRING[];
 BEGIN
  IF ~MakeReport & NeedElevation() THEN
   APP.Get().NewPrompt("⠭ ⮢");
   TRY
    asl:=GetElevatedASL("Sys_Env", aslDBModification_Off);
    asl.Var("L_MSWStp.HasError").Value  :=HasError;
    asl.Var("L_MSWStp.MakeReport").Value:=MakeReport;
    Res:=asl.Func("L_MSWStp._InstallFonts", SrcFontsDir);
    HasError:=asl.Var("L_MSWStp.HasError").Value;
    RETURN Res;
   EXCEPT
   ON Exc: Exc_Object DO
    AsbApplicationErrorHandler(Exc);
    HALT();
   END; (* TRY *)
  ELSE
   RETURN _InstallFonts(SrcFontsDir);
  END;
 END {InstallFonts};

 PUBLIC PROCEDURE Fonts_SetUp();
  CONST
   FontsFileName = "FONTS.ZIP";
  VAR
   React,
   ReturnCode      : NUMERIC[0];
   Dummy,
   tmpFontsDir,
   srcFontsFileName: STRING[];
 BEGIN
  SETEMPTY(srcFontsFileName);
  IF L_General.IsOurEngineeringCopy() THEN
   srcFontsFileName:=FIO.FullPath(FIO.BuildPath(FontsFileName, , "TMP"));
   IF ~FIO.Exists(srcFontsFileName) |
      ~L_StdFrm.RedAlternative("ᯮ짮 ⮢ 䠩  ⠬||" + srcFontsFileName + "|| ⫠ 楤 ⠭ ⮢?") THEN
    SETEMPTY(srcFontsFileName);
   END;
  END;
  IF !srcFontsFileName THEN
   srcFontsFileName:=L_General.GetREDISTRFullPath(FontsFileName);
  END;
  L_General.Assert(FIO.Exists(srcFontsFileName), "  室 䠩  ⠬|" + srcFontsFileName);
  tmpFontsDir:=FIO.BuildPath("FONTS", , FIO.AsbSpecialFolders("ASBTemp"));
  IF FIO.DirExists(tmpFontsDir) THEN
   L_FIO.ClearDir(tmpFontsDir);
  END;
  IF L_General.Launch_PKUNZIP("-over=all -dir=full", srcFontsFileName, tmpFontsDir) = L_General.eprc_Normal THEN
   LOOP
    HasError  :=0;
    MakeReport:=1;
    TRY
     React:=ShowDialog(Str.Align(DstFontsFolder_asStr + Shell.SpecialFolders("Fonts"), DialogWidth, LEFT) + "|",
                       InstallFonts(tmpFontsDir),
                       GetButtonList(),
                       "   ⠭ ⮢ ");
     IF React = 1 THEN
      MakeReport:=0;
      Dummy:=InstallFonts(tmpFontsDir);
     ELSE
      EXIT;
     END;
    EXCEPT
    ON Exc: Exc_Object DO
     IF ~IsAbortElevationByUser(Exc.ErrorCode) THEN  (* Cancel  樨 *)
      RAISE();
     END;
    END; (* TRY *)
   END; (* LOOP *)
  END;
  L_FIO.RemoveDir(tmpFontsDir);
 END {Fonts_SetUp};

 PRIVATE PROCEDURE CutLongStr(TmpStr: STRING[]; MaxWidth: NUMERIC[0]): STRING[];
  CONST
   EndOfLine = "...";
 BEGIN
  IF Str.Length(TmpStr) > MaxWidth THEN
   RETURN Str.Slice(TmpStr, 0, MaxWidth - Str.Length(EndOfLine)) + EndOfLine;
  ELSE
   RETURN TmpStr;
  END;
 END {CutLongStr};

 PRIVATE PROCEDURE SplitString(LStr, RStr: STRING[]): STRING[];
 BEGIN
  LStr:=CutLongStr(LStr, LeftWidth);
  RStr:=CutLongStr(RStr, LineWidth - LeftWidth);
  RETURN Str.Align(LStr, LeftWidth,             LEFT) +
         Str.Align(RStr, LineWidth - LeftWidth, RIGHT);
 END {SplitString};

 PRIVATE PROCEDURE GetASBMSOfficePath(): STRING[];
 BEGIN
  RETURN FIO.BuildPath("MSOffice", "", FIO.AsbSpecialFolders("ASBProcessorRunFrom"));
 END {GetASBMSOfficePath};

 PRIVATE PROCEDURE GetMSOfficeFolder();
  CONST
   ExcelXP_Version = 10.0;
  VAR
   excel       : OBJECT;
   tmpVersion  : NUMERIC[1];
 BEGIN
  SETEMPTY(excel);
  TRY
   excel:=CreateOBJECT("Excel.Application");
  EXCEPT
  ELSE
   TRY
    IF ~!excel THEN
     excel.Quit();
     SETEMPTY(excel);
    END;
   EXCEPT
   END; (* TRY *)
   RAISE();
  END; (* TRY *)
  L_General.Assert(~!excel, "  Excel");
  MSOfficeFolder:=excel.Path;
  MSExcelVersion:=excel.Version;
  SETEMPTY(tmpVersion);
  TRY
   tmpVersion:=STRTOFLD(MSExcelVersion, NUMERIC);
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode # L_General.ec_rt_FailedSTRTOFLD THEN
    RAISE();
   END;
  END; (* TRY *)
  IsExcelXPAndWin9x:=~!tmpVersion                   &
                     (tmpVersion = ExcelXP_Version) &
                     (L_General.GetOSPlatform() = L_General.osp_Win9x);
  excel.Quit();
  SETEMPTY(excel);
 END {GetMSOfficeFolder};

 PRIVATE PROCEDURE GetAutoParkXLSFullName(): STRING[];
 BEGIN
  RETURN FIO.BuildPath(AutoparkXLSName, "", GetASBMSOfficePath());
 END {GetAutoParkXLSFullName};

 PRIVATE PROCEDURE AutoParkXLSEqual(MainXLSFile, LocalXLSFile: STRING[]): SCAL[2];
  VAR
   Main,
   Local: FOLDERENTRY;
 BEGIN
  TRY
   Main :=FOLDERENTRY.Create(MainXLSFile);
   Local:=FOLDERENTRY.Create(LocalXLSFile);
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode = 649 THEN
    L_General.Error(Exc.MsgOriginalText);
   ELSE
    RAISE();
   END;
  END; (* TRY *)
  RETURN (Main.DateCreated = Local.DateCreated) &
         (Main.DateLastModified = Local.DateLastModified) &
         (Main.Size = Local.Size);
 END {AutoParkXLSEqual};

 PRIVATE PROCEDURE CheckExcelComponents(RegPath, Msg: STRING[]; VAR Err: SCAL[2]): STRING[];
  VAR
   FullPathToLnk,
   FullPathToXls,
   _MSOfficeFolder: STRING[];
   oShell, oLink  : OBJECT;
 BEGIN
  _MSOfficeFolder:=FIO.BuildPath("XLSTART",       "", MSOfficeFolder);
  FullPathToLnk  :=FIO.BuildPath(AutoparkLNKName, "", _MSOfficeFolder);
  FullPathToXls  :=FIO.BuildPath(AutoparkXLSName, "", _MSOfficeFolder);
  IF IsExcelXPAndWin9x THEN
   Err     :=~FIO.Exists(FullPathToXls) | ~AutoParkXLSEqual(GetAutoParkXLSFullName(), FullPathToXls);
   Msg     :=SplitString(Shift + " 䠩 ஥",
                         L_Str.PoolItem("-|", ~Err)) + "|";
  ELSE
   Err     :=~FIO.Exists(FullPathToLnk);
   Msg     :=SplitString(Shift + " " + AutoparkLNKName,
                         L_Str.PoolItem("-|", ~Err)) + "|";
   IF ~Err THEN
    oShell:=CreateOBJECT("WScript.Shell");
    oLink :=oShell.CreateShortcut(FullPathToLnk);
    Err   :=~Str.Match(Str.Cap(oLink.WorkingDirectory), Str.Cap(GetASBMSOfficePath()));
    Msg   :=Msg + SplitString(Shift + "ন 몠 " + AutoparkLNKName,
                              L_Str.PoolItem("-|", ~Err)) + "|";
   END;
  END;
  Msg     :=Msg + SplitString(Shift + " ஥ " + AutoparkXLSName,
                              L_Str.PoolItem("-|", FIO.Exists(GetAutoParkXLSFullName()))) + "|";
  IF ~FIO.Exists(GetAutoParkXLSFullName()) THEN
   L_General.Error("  䠩 " + GetAutoParkXLSFullName() + ".| |" +
                   ",   ࠡ稪").
  END;
  IF ~MakeReport THEN
   IF IsExcelXPAndWin9x THEN
    FIO.Erase(FullPathToLnk);
    L_FIO.ExactFileCopy(GetAutoParkXLSFullName(), FullPathToXls);
    L_StdFrm.Inform(" AUTOPARK.LNK 㤠." +
                    "|| AUTOPARK.XLS ᪮஢  |" + _MSOfficeFolder);
   ELSE
    FIO.Erase(FIO.BuildPath(AutoparkXLSName, "", _MSOfficeFolder));
    Shell.CreateShortcut(FullPathToLnk, "AUTOPARK", GetAutoParkXLSFullName(), "", GetASBMSOfficePath());
   END;
  END;
  RETURN Msg;
 END {CheckExcelComponents};

 PRIVATE PROCEDURE CheckJet(RegPath, Msg: STRING[]; VAR Err: SCAL[2]): STRING[];
 BEGIN
  CondRWRegValue(JetRegPath_30, jrp_CharacterSet, "OEM",          StringValue, Msg, 1);
  CondRWRegValue(JetRegPath_35, jrp_CharacterSet, "OEM",          StringValue, Msg, 1);
  CondRWRegValue(JetRegPath_40, jrp_CharacterSet, "OEM",          StringValue, Msg, 1);
  CondRWRegValue(JetRegPath_40, jrp_Format,       "Delimited(;)", StringValue, Msg, 1);
  Err:=~!Msg;
  RETURN Msg;
 END {CheckJet};

 PRIVATE PROCEDURE CheckODBC(RegPath, Msg: STRING[]; VAR Err: SCAL[2]): STRING[];
  VAR
   Path: STRING[];
 BEGIN
  Path:=Reg.BuildPath(RegPath, "ODBC Data Sources");
  CondRWRegValue(Path, "Text Files",         "Microsoft Text Driver (*.txt; *.csv)",
                                                                StringValue, Msg, 1);
  Path:=Reg.BuildPath(RegPath, "Text Files");
  CondRWRegValue(Path, "Driver",             FIO.BuildPath("ODBCJT32", "DLL", Shell.SpecialFolders("System")),
                                                                StringValue, Msg, 0);
  CondRWRegValue(Path, "DefaultDir",         "C:\",             StringValue, Msg, 0);
  CondRWRegValue(Path, "DriverId",           "1b",              DigitValue,  Msg, 1);
  CondRWRegValue(Path, "FIL",                "text;",           StringValue, Msg, 1);
  CondRWRegValue(Path, "SafeTransactions",   "0",               DigitValue,  Msg, 1);
  CondRWRegValue(Path, "UID",                "",                StringValue, Msg, 1);
  Path:=Reg.BuildPath(Path, "Engines");
  Path:=Reg.BuildPath(Path, "Text");
  CondRWRegValue(Path, "Extensions",         "txt,csv,tab,asc", StringValue, Msg, 1);
  CondRWRegValue(Path, "ImplicitCommitSync", "",                StringValue, Msg, 1);
  CondRWRegValue(Path, "Threads",            "3",               DigitValue,  Msg, 1);
  CondRWRegValue(Path, "UserCommitSync",     "Yes",             StringValue, Msg, 1);
  Err:=~!Msg;
  RETURN Msg;
 END {CheckODBC};

 PRIVATE PROCEDURE _CheckSystemEnv(Proc: PROC; ODBCRegPath, Title: STRING[]): STRING[];
  VAR
   Msg: STRING[];
   Err: SCAL[2];
 BEGIN
  SETEMPTY(Msg);
  Err:=0;
  Msg:=Proc(ODBCRegPath, Msg, Err);
  IF Err THEN
   HasError:=Err;
  END;
  RETURN SplitString(Title, L_Str.PoolItem("|-", Err)) + "|" + Msg;
 END {_CheckSystemEnv};

 PUBLIC PROCEDURE _SetSystemEnv(ODBCRegPath: STRING[]): STRING[];
 BEGIN
  RETURN _CheckSystemEnv(PROCREF(CheckExcelComponents), "",          "⠭  Excel") +
         _CheckSystemEnv(PROCREF(CheckJet),             "",          "஢ઠ ஢") +
         _CheckSystemEnv(PROCREF(CheckODBC),            ODBCRegPath, "஢ઠ ॥");
 END {_SetSystemEnv};

 PRIVATE PROCEDURE SetSystemEnv(ODBCRegPath: STRING[]): STRING[];
  VAR
   asl: OBJECT;
   Res: STRING[];
 BEGIN
  IF ~MakeReport & NeedElevation() THEN
   TRY
    APP.Get().NewPrompt("ன 㦥  ࠡ  MS Excel");
    asl:=GetElevatedASL("Sys_Env", aslDBModification_Off);
    asl.Var("L_MSWStp.MSExcelVersion").Value   :=MSExcelVersion;
    asl.Var("L_MSWStp.MSOfficeFolder").Value   :=MSOfficeFolder;
    asl.Var("L_MSWStp.IsExcelXPAndWin9x").Value:=IsExcelXPAndWin9x;
    asl.Var("L_MSWStp.HasError").Value         :=HasError;
    asl.Var("L_MSWStp.MakeReport").Value       :=MakeReport;
    Res:=asl.Func("L_MSWStp._SetSystemEnv", ODBCRegPath);
    HasError:=asl.Var("L_MSWStp.HasError").Value;
    RETURN Res;
   EXCEPT
   ON Exc: Exc_Object DO
    AsbApplicationErrorHandler(Exc);
   END; (* TRY *)
  ELSE
   RETURN _SetSystemEnv(ODBCRegPath);
  END;
 END {SetSystemEnv};

 PUBLIC PROCEDURE MSExcel_SetUp();
  VAR
   React  : NUMERIC[0];
   Dummy,
   InfoStr: STRING[];
 BEGIN
  L_General.Assert(~L_General.IsLocalMode(), "ᯮ짮 ᢮ ⠡ MS Excel|" +
                                             " ਠ  ⠫樨  ");

  GetMSOfficeFolder();
  InfoStr:=Str.Align(MSOfficePath_asStr +
                     CutLongStr(MSOfficeFolder, DialogWidth - Str.Length(MSOfficePath_asStr)),
                     DialogWidth,
                     LEFT) + "|" +
           Str.Align(ASBMSOfficePath_asStr +
                     CutLongStr(GetASBMSOfficePath(), DialogWidth - Str.Length(ASBMSOfficePath_asStr)),
                     DialogWidth,
                     LEFT) + "|" +
           Str.Align(MSExcelVersion_asStr + MSExcelVersion,
                     DialogWidth,
                     LEFT) + "|";
  LOOP
   HasError  :=0;
   MakeReport:=1;
   TRY
    React:=ShowDialog(InfoStr, SetSystemEnv(ODBCRegPath), GetButtonList(),
                      "  ஢ન ⥬ 㦥 ");
    IF React = 1 THEN
     MakeReport:=0;
     Dummy     :=SetSystemEnv(ODBCRegPath);
     L_StdFrm.Inform("ன ⥬ 㦥 ᯥ譮 襭");
    ELSE
     EXIT;
    END;
   EXCEPT
   ON Exc: Exc_Object DO
    IF ~IsAbortElevationByUser(Exc.ErrorCode) THEN  (* Cancel  樨 *)
     RAISE();
    END;
   END; (* TRY *)
  END; (* LOOP *)
 END {MSExcel_SetUp};

 PRIVATE PROCEDURE IsUNC(Path: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Slice(Path, 0, 2) = "\\";
 END {IsUNC};

 PRIVATE PROCEDURE TrustLocKey(Field: STRING[]): STRING[];
  CONST
   Excel12TrustedLocationsKey = "HKCU\Software\Microsoft\Office\12.0\Excel\Security\Trusted Locations";
 BEGIN
  RETURN Reg.BuildPath(Excel12TrustedLocationsKey, Field);
 END {TrustLocKey};

 PRIVATE PROCEDURE SetTrustLocation(LocNbr: NUMERIC[0]; Path, Description: STRING[]; AllowSubFolders: SCAL[2]);
  VAR
   Key: STRING[];
 BEGIN
  Key:="Location" + FLDTOSTR(LocNbr, "0", 0);
  Reg.Write(TrustLocKey(Key + "\Path"),            Path,            "REG_SZ");
  Reg.Write(TrustLocKey(Key + "\Description"),     Description,     "REG_SZ");
  Reg.Write(TrustLocKey(Key + "\AllowSubFolders"), AllowSubFolders, "REG_DWORD");
 END {SetTrustLocation};

 PUBLIC PROCEDURE AddExcel12TrustedLocations();
  VAR
   ASBClientMSOfficePath,
   ClientProjectPath            : STRING[];
   ASBClientMSOfficeLocationNbr,
   ClientProjectLocationNbr,
   SubClientProjectLocationNbr,
   NeedAllowNetworkLocations,
   AllowNetworkLocations,
   AllowSubFolders              : NUMERIC[0];
   EmptyLocationNbrs            : NUMERIC[0][2];
   N                            : NUMERIC[0];
   LocationPath                 : STRING[];
 BEGIN
  (* ᫨  ࠧ *)
  IF Reg.Read("HKCU\Software\Microsoft\Office\12.0\Excel") THEN
   ASBClientMSOfficePath:=FIO.UNCPath(FIO.BuildPath("MSOffice", , FIO.AsbSpecialFolders("ASBProcessorRunFrom")));
   IF ~L_General.IsOurSite() THEN
    ClientProjectPath   :=FIO.UNCPath(FIO.AsbSpecialFolders("ClientProjectRoot"));
   ELSE
    IF L_General.IsPolakSpecific() THEN
     ClientProjectPath  :="\\polaknt03\Account\Polak\";
    ELSE
     ClientProjectPath  :="\\polaknt03\Prj_Currents\";
    END;
   END;
   NeedAllowNetworkLocations:=IsUNC(ASBClientMSOfficePath) | IsUNC(ASBClientMSOfficePath);
 
   AllowNetworkLocations := 0;
   Reg.Read(TrustLocKey("AllowNetworkLocations"), AllowNetworkLocations);
 
   (* ⢨ ய᪮ ஢  ࠭ *)
   SETEMPTY(ASBClientMSOfficeLocationNbr);
   SETEMPTY(ClientProjectLocationNbr);
   SETEMPTY(SubClientProjectLocationNbr);
   SETEMPTY(EmptyLocationNbrs);
   N := 0;
   WHILE N < 100 DO
    IF Reg.Read(TrustLocKey("Location" + FLDTOSTR(N, "0", 0) + "\Path"), LocationPath) THEN
     IF FIO.EqualPath(LocationPath, ASBClientMSOfficePath) THEN
      ASBClientMSOfficeLocationNbr :=N;
     END;
     IF FIO.EqualPath(LocationPath, ClientProjectPath) THEN
      ClientProjectLocationNbr     :=N;
      AllowSubFolders              :=0;
      Reg.Read(TrustLocKey("Location" + FLDTOSTR(N, "0", 0) + "\AllowSubFolders"), AllowSubFolders);
      IF AllowSubFolders THEN
       SubClientProjectLocationNbr :=N;
      END;
     END;
    ELSIF !EmptyLocationNbrs[0] THEN
     EmptyLocationNbrs[0]          := N;
    ELSIF !EmptyLocationNbrs[1] THEN
     EmptyLocationNbrs[1]          := N;
    END;
    INC(N);
   END;
 
   IF !ASBClientMSOfficeLocationNbr |
      !ClientProjectLocationNbr |
      !SubClientProjectLocationNbr |
      NeedAllowNetworkLocations & ~AllowNetworkLocations THEN
    N := 0;
    IF !ASBClientMSOfficeLocationNbr & ~!EmptyLocationNbrs[N] THEN
     ASBClientMSOfficeLocationNbr := EmptyLocationNbrs[N];
     INC(N);
     SetTrustLocation(ASBClientMSOfficeLocationNbr, ASBClientMSOfficePath, "ASB", 0);
    END;
    IF !ClientProjectLocationNbr & ~!EmptyLocationNbrs[N] THEN
     ClientProjectLocationNbr := EmptyLocationNbrs[N];
     INC(N);
     SetTrustLocation(ClientProjectLocationNbr, ClientProjectPath, "AutoPark", 1);
    END;
    (* LocationNN\AllowSubFolders *)
    IF !SubClientProjectLocationNbr & ~!ClientProjectLocationNbr THEN
     Reg.Write(TrustLocKey("Location" + FLDTOSTR(ClientProjectLocationNbr, "0", 0) + "\AllowSubFolders"),
               1,
               "REG_DWORD");
     SubClientProjectLocationNbr := ClientProjectLocationNbr;
    END;
    (* AllowNetworkLocations *)
    IF NeedAllowNetworkLocations & ~AllowNetworkLocations THEN
     Reg.Write(TrustLocKey("AllowNetworkLocations"), 1, "REG_DWORD");
    END;
   END;
  END;
 END {AddExcel12TrustedLocations};

END.
