(*
%Copyright   (c) Polak Computer System
%Author      28.05.2002 16:02:02 GAV
%Purpose     প OLE
%Type        library
%Specific    SYS
%Modified    26.08.2014 15:21:16 POLAK\GAV
%Environment
*)

 MODULE L_OLE [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT Str, L_Str, Reg, DBD, FIO, L_General, L_StdFrm, L_MSWStp, Shell,
        L_AFM, L_DatTim, L_Prop, L_PCS, L_PDBS, L_Excel;

 PUBLIC TYPE
  tKeyValue = SCAL[1E7];

 PRIVATE PERMANENT VAR
  UsedBasisList: STRING[];

 PUBLIC PROCEDURE PathToXLA(BookName: STRING[]): STRING[];
  VAR
   Path: STRING[];
 BEGIN
  L_General.Assert(FIO.Exists(BookName, "XLA", Path),
                   " " + BookName + ".XLA  ");
  RETURN Path;
 END {PathToXLA};

 PUBLIC PROCEDURE SetUsedBasisList(_UsedBasisList: STRING[]);
 BEGIN
  UsedBasisList:=_UsedBasisList;
 END {AddToUsedBasisList};

 PRIVATE PROCEDURE ApplyIndexTime(_I: STRING[]; VAR _Date: DATE; VAR _Time: TIME);
  VAR
   _D: DATE;
   _T: TIME;
 BEGIN
  GETTIMESTAMP(^_I, _D, _T);
  IF (_D < _Date) | ((_D = _Date) & (_T < _Time)) THEN
   _Date:=_D;
   _Time:=_T;
  END;
 END {ApplyIndexTime};

 PUBLIC PROCEDURE RFsBuildTime(): STRING[];
  VAR
   Idx  : NUMERIC[0];
   _Date: DATE;
   _Time: TIME;
 BEGIN
  _Date:=SYSDATE();
  _Time:=SYSTIME();

  Idx:=0;
  WHILE ~!L_Str.PoolItem(UsedBasisList, Idx) DO
   ApplyIndexTime(L_Str.PoolItem(UsedBasisList, Idx), _Date, _Time);
   INC(Idx);
  END;
  RETURN L_DatTim.DateStr(_Date) + " " + L_DatTim.TimeStr(_Time);
 END {RFsBuildTime};

 PUBLIC PROCEDURE RFsAmount(): NUMERIC[0];
  VAR
   MainIndex: STRING[];
 BEGIN
  MainIndex:=L_Str.PoolItem(UsedBasisList, 0);
  IF ~!MainIndex THEN
   RETURN AMOUNT(^MainIndex);
  END;
  RETURN 0;
 END {RFsAmount};

 PUBLIC PROCEDURE DeInit();
  VAR
   Idx: NUMERIC[0];
 BEGIN
  IF L_General.VariableIsInitializing(UsedBasisList) THEN
   Idx:=0;
   WHILE ~!L_Str.PoolItem(UsedBasisList, Idx) DO
    L_AFM.AF_PurgeTemp(L_Str.PoolItem(UsedBasisList, Idx));
    INC(Idx);
   END;
  END;
 END {DeInit};

 PUBLIC PROCEDURE SliceBySingle(S: STRING[]; VAR Sliced: STRING[][]);
  VAR
   i: NUMERIC[0];
 BEGIN
  SETEMPTY(Sliced);
  i:=0;
  LOOP
   Sliced[i]:=Str.Condense(Str.Item(S, ";", i));
   IF !Sliced[i] THEN
    EXIT;
   END;
   INC(i);
  END;
 END {SliceBySingle};

 PUBLIC PROCEDURE FillByKeyWord(Params : STRING[][];
                                keyWord: STRING[];
                            VAR Values : tKeyValue[]);
  VAR
   i: NUMERIC[0];
 BEGIN
  SETEMPTY(Values);
  i:=0;
  WHILE ~!Params[i] DO
   IF keyWord = Str.Item(Params[i], ": ", 0) THEN
    L_Str.StrToScalArr(Str.Item(Params[i], ": ", 1), Values);
   END;
   INC(i);
  END;
 END {FillByKeyWord};

 PUBLIC PROCEDURE FillStrArrByKeyWord(Params : STRING[][];
                                      keyWord: STRING[];
                                  VAR Values : STRING[][]);
  VAR
   i: NUMERIC[0];
 BEGIN
  SETEMPTY(Values);
  i:=0;
  WHILE ~!Params[i] DO
   IF keyWord = Str.Item(Params[i], ": ", 0) THEN
    L_Str.StrToArr(Str.Item(Params[i], ": ", 1), Values);
   END;
   INC(i);
  END;
 END {FillByKeyWord};

 PUBLIC PROCEDURE SetByKeyWord(Params : STRING[][];
                               keyWord: STRING[];
                           VAR Value  : STRING[]);
  VAR
   i: NUMERIC[0];
 BEGIN
  SETEMPTY(Value);
  i:=0;
  WHILE ~!Params[i] DO
   IF keyWord = Str.Item(Params[i], ": ", 0) THEN
    Value:=Str.Item(Params[i], ": ", 1);
   END;
   INC(i);
  END;
 END {SetByKeyWord};

 PUBLIC PROCEDURE HasCondition(V: STRING[][]): SCAL[2];
 BEGIN
  RETURN ~!V[0];
 END {HasCondition};

 PUBLIC PROCEDURE StrCheck(VAR (*!*)Value: STRING[];
                              Constraints: STRING[][]): SCAL[2];
  VAR
   i: NUMERIC[0];
 BEGIN
  IF HasCondition(Constraints) THEN
   i:=0;
   WHILE ~!Constraints[i] DO
    IF Value = Constraints[i] THEN
     RETURN 1;
    END;
    INC(i)
   END;
   RETURN 0;
  ELSE
   RETURN 1;
  END;
 END {StrCheck};

 PUBLIC PROCEDURE Check(VAR (*!*)Value: tKeyValue;
                           Constraints: tKeyValue[]): SCAL[2];
  VAR
   i: NUMERIC[0];
 BEGIN
  IF HasCondition(Constraints) THEN
   i:=0;
   WHILE ~!Constraints[i] DO
    IF Value = Constraints[i] THEN
     RETURN 1;
    END;
    INC(i)
   END;
   RETURN 0;
  ELSE
   RETURN 1;
  END;
 END {Check};

 PUBLIC PROCEDURE CheckRegistry();
  CONST
   StdRegistryKey = "HKLM\SOFTWARE\Polak IT\AutoPark\ClientRedFile";
   OurRegistryKey = "HKCU\SOFTWARE\Polak IT\AutoPark\ClientRedFileForVAutSrv";
   BadInstallMsg  = " ⮬   믮|" +
                    " 믮 ४⭮|" +
                    "⥢ c⠫ ⮏";
  VAR
   TellAdminMsg,
   Dummy1,
   Dummy2,
   Project,
   ClientRedFile: STRING[];
   ASBApplObject: OBJECT;
 BEGIN
  IF L_General.IsOurSite() THEN
   SETEMPTY(TellAdminMsg);
  ELSE
   TellAdminMsg:=L_StdFrm.HR() + "  .";
  END;
  L_General.Assert(Reg.Read(StdRegistryKey, ClientRedFile), BadInstallMsg + TellAdminMsg);
  TRY
   ASBApplObject:=CreateOBJECT("ASB.Application");
  EXCEPT
  ELSE
   L_General.Error(BadInstallMsg + TellAdminMsg);
  END;
  IF (ASBApplObject.Version > DBD.ProcessorVersion()) |
     (ASBApplObject.Version < DBD.ProcessorVersion()) & ~L_General.IsOurEngineeringCopy() THEN
   L_General.Error(BadInstallMsg +
                   L_StdFrm.HR() +
                   " ࢥ ⮬⨧樨: " + L_Str.IntStr(ASBApplObject.Version) +
                  "| V32.exe:               " + L_Str.IntStr(DBD.ProcessorVersion()) +
                   TellAdminMsg);
  END;
  SETEMPTY(ASBApplObject);
  FIO.SplitPath(ClientRedFile, Project, Dummy1, Dummy2);
  IF L_General.IsOurSite() | APP.Get().SupportMode |
     (L_General.GetAPSiteNumber() = 25) |
     (L_General.GetAPSiteNumber() = 90) THEN
   ClientRedFile:=FIO.MakePath(DBD.BaseName(),
                               "red",
                               FIO.AsbSpecialFolders("ClientRedFileDir"));
   Reg.Write(OurRegistryKey, ClientRedFile);
  ELSE
   L_General.Assert(Str.Cap(Project)=Str.Cap(DBD.BaseName()), BadInstallMsg);
  END;
 END {CheckRegistry};

 PUBLIC PROCEDURE OpenReport(Mask: STRING[]);
  VAR
   FilePool,
   FileName,
   Path     : STRING[];
   fso,
   SrcFold  : OBJECT;
   SrcFiles : ENUMERATOR;
 BEGIN
  L_General.Assert(~L_General.IsLocalMode(), "ᯮ짮 OLE  ਠ  ⠫樨  ");

  SETEMPTY(FilePool);
  IF FIO.Exists(Mask + "*", "xls", Path) | FIO.Exists(Mask + "*", "xlsm", Path) THEN
   fso     :=CreateOBJECT("Scripting.FileSystemObject");
   FIO.SplitPath(Path, FileName, , Path);
   SrcFold :=fso.GetFolder(Path);
   SrcFiles:=CreateENUMERATOR(SrcFold.Files);
   WHILE ~SrcFiles.AtEnd() DO
    FileName:=SrcFiles.Item().Name;
    IF Str.Match(FileName, Mask + "*.xls") | Str.Match(FileName, Mask + "*.xlsm") THEN
     FilePool:=FilePool+"|"+FileName;
    END;
    SrcFiles.MoveNext();
   END;
  END;
  L_General.Assert(~!FilePool,
                   " ⮢  " + Mask + ".|" +
                   "|,   뢠  , ⮬,"+
                   "|,   ࠡ稪");
  FileName:=L_Str.PoolItem(FilePool,
                           L_StdFrm.ShowListAsMenu(FilePool, "롮 䠩"));
  FileName:=FIO.MakePath(FileName, , Path);
  CheckRegistry();
  L_MSWStp.AddExcel12TrustedLocations();
  Shell.Execute("open", FileName);
 END {OpenReport};

 PUBLIC PROCEDURE ODFirmName(D: DATE): STRING[];
 BEGIN
  RETURN L_PCS.ODFirmName(D);
 END {ODFirmName};

(* =========================================== *)

 PUBLIC PROCEDURE _Assert(ErrCode: NUMERIC[2]; Cond: SCAL[2]; Text: STRING[]);
 BEGIN
  IF ~Cond THEN
   RAISE(ErrCode, Text);
  END;
 END {_Assert};

(* =========== *)

 PUBLIC PROCEDURE IsPropertyPart(Part: STRING[]): SCAL[2];
 BEGIN
  RETURN DBD.LinkExist(^(L_PDBS.FileOfPseudonim(Part)), Property);
 END {IsPropertyPart};

 PRIVATE PROCEDURE IsWithRows(Part: STRING[]): SCAL[2];
 BEGIN
  RETURN DBD.FieldExist(^Part, PropRow);
 END {IsWithRows};

 PUBLIC PROCEDURE GetValue(v_Value, RootKey: STRING[]; D: DATE): STRING[];
  VAR
   ft,
   Idx            : NUMERIC[0];
   Value,
   PrevPart,
   Part,
   NextPart,
   FieldName,
   ReturnValue    : STRING[];
   _Proc          : PROC;
   NeedReturnValue: SCAL[2];
 BEGIN
  PrevPart:=Str.Item(v_Value, ".", 0);
  Part    :=Str.Item(v_Value, ".", 1);
  NextPart:=Str.Item(v_Value, ".", 2);
  Idx:=2;

  IF !PrevPart | !RootKey THEN
   RETURN "";
  ELSIF PrevPart = "Staff" THEN
   _Assert(-6, SEARCH(Staff, TSN,    RootKey), " ࠡ⭨");
  ELSIF PrevPart = "Man" THEN
   _Assert(-6, SEARCH(Man,   Man,    RootKey), "⭮ ");
  ELSIF PrevPart = "Car" THEN
   _Assert(-6, SEARCH(Car,   BoxNbr, RootKey), "⭮ ");
  ELSE
   _Assert(-7, 0, " ঠ ᯨ᮪  䠩 " + PrevPart);
  END;

  WHILE ~!NextPart DO
    _Assert(-2, DBD.FileExist(^(L_PDBS.FileOfPseudonim(Part))),
            '୮  䠩 - "' + Part + '"');
    IF    IsPropertyPart(Part) THEN
     IF IsWithRows(Part) THEN
      _Assert(-3, Str.Item(v_Value, ".", Idx+1) = "Value",
              " " + Str.Small(DBD.FileDescription(^Part)) +
              ' ⨬ 뢮 ⮫쪮  "Value"');
      Value:="";
      CONFORM(^PrevPart, ^Part);
      WHILE OCCURE(^Part) DO
       IF FLDTOSTR(^(Part+".Property")) = NextPart THEN
        IF !Value THEN
         Value:=^(Part+".Value");
        ELSE
         Value:=Value+"; "+^(Part+".Value");
        END;
       END;
       NEXT(^Part);
      END; (* WHILE OCCURE(^Part) ... *)
      RETURN Value;
     ELSIF ~L_PDBS.HasActualPro(PrevPart, Part, NextPart, D) THEN
      RETURN "";
     END;
     INC(Idx);
     NextPart:=Str.Item(v_Value, ".", Idx);
    ELSIF L_PDBS.ConformToPseudoFile(PrevPart, Part, D, NeedReturnValue, ReturnValue) THEN
     IF NeedReturnValue THEN
      RETURN ReturnValue;
     END;
    ELSE
     HISTCONF(D, ^PrevPart, ^Part);
     IF ~OCCURE(^Part) THEN
      RETURN "-";
     END;
    END;
    INC(Idx);
    PrevPart:=(L_PDBS.FileOfPseudonim(Part));
    Part    :=NextPart;
    NextPart:=Str.Item(v_Value, ".", Idx);
  END;

  IF L_PDBS.GetPseudoFieldValue(PrevPart, Part, D, ReturnValue) THEN
   RETURN ReturnValue;
  ELSE
   FieldName:=PrevPart+"."+Part;
   _Assert(-4, DBD.FieldExist(^PrevPart, ^(Str.Item(Part, "[", 0))),
           ' 䠩 ' + PrevPart + '   "' + Part + '"');

   IF    TYPEOF(^FieldName) = L_General.ft_DATE THEN
    RETURN FLDTOSTR(^FieldName, L_Excel.DateFormat());
   ELSIF TYPEOF(^FieldName) = L_General.ft_TIME THEN
    RETURN FLDTOSTR(^FieldName, L_Excel.TimeFormat());
   ELSIF TYPEOF(^FieldName) = L_General.ft_TIMESTAMP THEN
    RETURN FLDTOSTR(^FieldName, L_Excel.DateFormat() + " H:M:S");
   ELSIF TYPEOF(^FieldName) = L_General.ft_SCAL THEN
    RETURN L_PCS.GetScalValueInterpretation(PrevPart, Part, "");
   ELSIF TYPEOF(^FieldName) = L_General.ft_STRING THEN
    RETURN ^FieldName;
   ELSIF (TYPEOF(^FieldName) = L_General.ft_NUMERIC) |
         (TYPEOF(^FieldName) = L_General.ft_FLOAT)  THEN
    RETURN Str.Condense(L_Str.Replace(FLDTOSTR(^FieldName), ".", L_Excel.DecimalSeparator()));
   ELSE
    RETURN Str.Condense(FLDTOSTR(^FieldName));
   END;
  END;
  RETURN "";
 END {GetValue};

(* =========================================== *)

 PUBLIC PROCEDURE FileDescr(fileName: STRING[]): STRING[];
  VAR
   descr: STRING[];
 BEGIN
  descr:=L_PDBS.PseudoFileDescription(fileName);
  IF !descr THEN
   descr:=DBD.FileDescription(^fileName);
  END;
  IF !descr THEN
   descr:=fileName;
  END;
  RETURN descr;
 END {FileDescr};

 PRIVATE PROCEDURE AddToPool(VAR NamePool: STRING[]; Name: STRING[]);
 BEGIN
  IF ~!Name THEN
   IF !NamePool THEN
    NamePool:=Name;
   ELSE
    NamePool:=NamePool + "|" + Name;
   END;
  END;
 END {AddToPool};

 PUBLIC PROCEDURE CooFiles(srcFile: STRING[]): STRING[];
  VAR
   kind,
   Idx,
   Count    : NUMERIC[0];
   NamePool,
   fileName : STRING[];
 BEGIN
  srcFile:=L_PDBS.FileOfPseudonim(srcFile);
  NamePool:="";
  Count:=DBD.LinkAmount(^srcFile);
  Idx:=0;
  WHILE Idx < Count DO
   kind    :=DBD.LinkKind(^srcFile, ^(DBD.LinkName(^srcFile, Idx)));
   fileName:=DBD.LinkCooFileName(^srcFile, ^(DBD.LinkName(^srcFile, Idx)));
   IF (kind = -3) | (kind > 0) | IsPropertyPart(fileName) THEN
    AddToPool(NamePool, fileName);
   END;
   INC(Idx);
  END;
  AddToPool(NamePool, L_PDBS.PseudoFiles(srcFile));
  RETURN NamePool;
 END {CooFiles};

 PUBLIC PROCEDURE FieldDescr(srcFile, fieldName: STRING[]): STRING[];
  VAR
   descr: STRING[];
 BEGIN
  srcFile:=L_PDBS.FileOfPseudonim(srcFile);
  fieldName:=Str.Item(fieldName, "[", 0);
  descr:=L_PDBS.PseudoFieldDescription(srcFile, fieldName);
  IF !descr THEN
   descr:=DBD.FieldDescription(^srcFile, ^fieldName);
  END;
  IF !descr THEN
   descr:=fieldName;
  END;
  RETURN descr;
 END {FieldDescr};

 PUBLIC PROCEDURE Fields(srcFile: STRING[]): STRING[];
  VAR
   i,
   Idx,
   Count   : NUMERIC[0];
   Name,
   NamePool: STRING[];
 BEGIN
  srcFile:=L_PDBS.FileOfPseudonim(srcFile);
  IF IsWithRows(srcFile) THEN
   RETURN "Value";
  ELSE
   NamePool:="";
   Count:=DBD.FieldAmount(^srcFile);
   Idx:=0;
   WHILE Idx < Count DO
    Name:=DBD.FieldName(^srcFile, Idx);
    IF RANGEOF(^(srcFile + "." + Name)) > 1 THEN
     i:=0;
     WHILE i < RANGEOF(^(srcFile + "." + Name)) DO
      AddToPool(NamePool, Name + "[" + L_Str.IntStr(i) + "]");
      INC(i);
     END;
    ELSE
     AddToPool(NamePool, Name);
    END;
    INC(Idx);
   END;
   AddToPool(NamePool, L_PDBS.PseudoFields(srcFile));
   RETURN NamePool;
  END;
 END {Fields};

 PUBLIC PROCEDURE PropertyDescr(prop: LIKE(Property.Property)): STRING[];
 BEGIN
  IF SEARCH(Property, Property, prop) THEN
   RETURN Property.Property + " " + Property.Name;
  ELSE
   RETURN "";
  END;
 END {PropertyDescr};

 PUBLIC PROCEDURE Properties(srcFile: STRING[]): STRING[];
  VAR
   NamePool: STRING[];
 BEGIN
  srcFile:=L_PDBS.FileOfPseudonim(srcFile);
  SETEMPTY(NamePool);
  SELECT(Property, PropObj, , L_Prop.GetObjNum(srcFile));
  WHILE OCCURE(Property) DO
   IF Property.IsUsed THEN
    AddToPool(NamePool, Property.Property);
   END;
   NEXT(Property);
  END; (* WHILE OCCURE(Property) ... *)
  RETURN NamePool;
 END {Properties};

END.
