(*
%Copyright   (c) Polak IT
%Author      07.02.2008 15:09:07 POLAK\kns
%Purpose     ਥ   ⥬ Parsec
%Type        library
%Specific    SYS
%Modified    17.02.2014 18:17:19 POLAK\San
%Environment
*)

 MODULE L_Parsec [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT P_OwnData, Str, L_General, L_ACS, L_StdFrm;

 PRIVATE CONST
  AntyPassIn                = 36;  (*No entry - antipassback*)
  AntyPassOut               = 37;
  TranCodeInOpening         = 64;
  TranCodeOutOpening        = 65;
  TranCodeIn                = 72;
  TranCodeOutOffTimeProfile = 66;
  TranCodeOut               = 73;

 PRIVATE PROCEDURE SQLConnect(User, Pass: STRING[]): OBJECT;
  VAR
   Result : OBJECT;
   Server : STRING[];
   SrvType: P_OwnData.t_acsp_SystemType;
 BEGIN
  SETEMPTY(Server);
  P_OwnData.GetACSSystemServer(SYSDATE(), SrvType, Server);
  IF SrvType = P_OwnData.c_acsp_SrvType_Parsec THEN
   Result:=CreateOBJECT("ADODB.Connection");
   Result.ConnectionString:="Provider= SQLOLEDB.1;" +
                            "Server=" + Server + ";"+
                            "database=ParsecDB;" +
                            "uid=" + User + ";" +
                            "pwd=" + Pass + ";"; 
   Result.Open();
  ELSE
   L_General.CallUs(0);
  END;
  RETURN Result;
 END {SQLConnect};

 PRIVATE PROCEDURE SQLSelect(Connection: OBJECT; Select: STRING[]): OBJECT;
  VAR
   Result : OBJECT;
 BEGIN
  Result:=CreateOBJECT("ADODB.RecordSet");
  Result.Open(Select, Connection, 2(*adOpenDynamic*), 3(*adLockOptimistic*), 1(*adCmdText*));
  Result.CacheSize:=10000000;
  RETURN Result;
 END {SQLSelect};

 PRIVATE PROCEDURE GetParsecEvent(Event: NUMERIC[0]): LIKE(ACSEvent.Event);
 BEGIN
  IF    Event = AntyPassIn THEN
   RETURN 0;
  ELSIF Event = AntyPassOut THEN
   RETURN 1;
  ELSIF Event = TranCodeIn THEN
   RETURN 2;
  ELSIF Event = TranCodeOut THEN
   RETURN 3;
  ELSIF Event = TranCodeOutOffTimeProfile THEN
   RETURN 5;
  ELSIF Event = TranCodeInOpening THEN
   RETURN 6;
  ELSIF Event = TranCodeOutOpening THEN
   RETURN 7;
  ELSE
   RETURN EMPTYOF(ACSEvent.Event);
  END;
 END {GetParsecEvent};

 PUBLIC PROCEDURE Fill_ACSEvent(From, UpTo : DATE; _Man: LIKE(Man.Man); _CardCode: NUMERIC[0]);
  VAR
   SQLConnection,
   SQLRecordSet  : OBJECT;
   ConStr        : STRING[];
   curTranCode   : NUMERIC[0];
   curEvent      : LIKE(ACSEvent.Event);
 BEGIN
  P_OwnData.GetACSSystemOutposts(SYSDATE());

  SQLConnection:=SQLConnect("sa", "parsec");
  TRY
   ConStr:="select P.TabNum, TL.Param2 as CardNumber, TL.TranCode, TL.TranDateTime " +
           ", D.ID_DOOR " +
           "from TransLog as TL " +
           "left join Personel as P " +
           "on P.ID_USER = TL.TranUserID " +
           "left join Doors as D " +
           "on D.ID_DOOR=TL.TranCtrlID " +
           "where (not(P.TabNum is NULL))";

   IF (~!From) & (~!UpTo) THEN
    ConStr:=ConStr + " and (TL.TranDateTime between '"+FLDTOSTR(From, "y.N.D" ) + "' and '" +
                                               FLDTOSTR(UpTo+1, "y.N.D" ) + "')";
   ELSIF (~!From) THEN
    ConStr:=ConStr + " and (TL.TranDateTime >= '"+FLDTOSTR(From, "y.N.D" ) + "')";
   ELSIF (~!UpTo) THEN
    ConStr:=ConStr + " and (TL.TranDateTime <= '"+FLDTOSTR(UpTo, "y.N.D" ) + "')";
   END;

   IF (~!_CardCode) THEN
    ConStr:=ConStr + " and (P.CardNumber = "+ FLDTOSTR(_CardCode) +")";
    ConStr:=ConStr + " and (P.TabNum <> '"+ _Man +"')";
   ELSIF (~!_Man) THEN
    ConStr:=ConStr + " and (P.TabNum = '"+ _Man +"')";
   END;

   ConStr:=ConStr + " order by P.TabNum, TL.TranDateTime";

   PROMPT(" 襭 믮    Parsec");
   SQLRecordSet:=SQLSelect(SQLConnection, ConStr);
   TRY
    WHILE ~SQLRecordSet.EOF() DO
     curTranCode:=SQLRecordSet.Fields("TranCode").Value;
     curEvent:=GetParsecEvent(curTranCode);
     IF !curEvent THEN
(*
      L_ACS.AddErr(Str.Condense(SQLRecordSet.Fields("TabNum").Value), EMPTYOF(ACSEvent.Man),
                   SQLRecordSet.Fields("CardNumber").Value, 0,
                   "⨥   " + SQLRecordSet.Fields("TranCode").Value + "  ঠ",
                   SQLRecordSet.Fields("TranDateTime").Value);
 
*)
     ELSE
      L_ACS.AddACSEvent(SQLRecordSet.Fields("TranDateTime").Value,
                        SQLRecordSet.Fields("CardNumber").Value,
                        SQLRecordSet.Fields("ID_DOOR").Value,
                        Str.Condense(SQLRecordSet.Fields("TabNum").Value),
                        GetParsecEvent(SQLRecordSet.Fields("TranCode").Value));
     END; (* IF !curEvent *)
     SQLRecordSet.MoveNext();
    END (*~SQLRecordSet.EOF() *);
   EXCEPT
   ELSE
    SQLRecordSet.Close();
    RAISE();
   END;
  SQLRecordSet.Close();
  EXCEPT
  ELSE
   SQLConnection.Close();
   RAISE();
  END;
  SQLConnection.Close();
  P_OwnData.Purge_Outposts();
 END {Fill_ACSEvent};

 (*誨*)
 PUBLIC PROCEDURE CM_PutStaff(CardCode: NUMERIC[0]); (*뤠/ *)
 BEGIN
  L_StdFrm.Choice("뤠 ய᪠ 㤥 ந ⮫쪮  ⮏." +
                  "||室 ⠪ ந 뤠 ய᪠ ।⢠ Parsec.");
 END {CM_PutStaff};

 PUBLIC PROCEDURE CM_Renovate(CardCode: NUMERIC[0]);
 BEGIN
  L_General.Error(" ଠ樨 (⮏ -> )  ন   Parsec.");
 END {CM_Renovate};

 PUBLIC PROCEDURE CM_Withdraw(CardCode: NUMERIC[0]); (*⨥ *)
 BEGIN
  L_StdFrm.Choice("⨥ ய᪠ 㤥 ந ⮫쪮  ⮏." +
                  "||室 ⠪ ந ⨥ ய᪠ ।⢠ Parsec.");
 END {CM_Withdraw};

 PUBLIC PROCEDURE CM_ToStopList(CardCode: NUMERIC[0]);
 BEGIN
  L_StdFrm.Choice("饭  ⮯- ய᪠ 㤥 ந ⮫쪮  ⮏." +
                  "||室 ⠪ ந 饭 ய᪠  ⮯- ।⢠ Parsec.");
 END {CM_ToStopList};

 PUBLIC PROCEDURE CM_ThrowFromStopList(CardCode: NUMERIC[0]);
 BEGIN
  L_StdFrm.Choice("  ⮯- ய᪠ 㤥 ந ⮫쪮  ⮏." +
                  "||室 ⠪ ந 㤠 ய᪠  ⮯- ।⢠ Parsec.");
 END {CM_ThrowFromStopList};

END.
