(*
%Copyright   (c) Polak IT
%Author      02.02.2005 15:06:45 POLAK\GAV
%Purpose
%Type        library
%Specific    SYS
%Modified    04.08.2014 09:47:59 POLAK\LDA
%Environment
*)

 MODULE L_Print [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC CONST
  pt_Amount   = 9;

 PUBLIC TYPE
  tOrientation   = SCAL[2];
  tPaperTrayType = SCAL[3];
  tPaperTrayIdx  = SCAL[1E3];
  tPrinterType   = SCAL[pt_Amount];
  tBandType      = SCAL[2];

 IMPORT L_General, L_XDoc, FIO, L_StdFrm, Str, L_Str, L_SES, L_FIO, L_ACL;

 PRIVATE CONST
  pt_Pool_Genitive = "PDF ਭ|"                                    +
                     "Image ਭ|"                                  +
                     "ਭ   |"                     +
                     "祪 ਭ|"                               +
                     "ਭ   ய᪮|"                   +
                     "ਭ   㬥⮢  |"         +
                     "ਭ   㬥⮢   |" +
                     "ਭ   ⥫ 㬥⮢|"   +
                     "ਭ  㬮砭";
  pt_Pool_SESValId = "Printer.PDF|"         +
                     "Printer.Image|"       +
                     "Printer.Stick|"       +
                     "Printer.Cheque|"      +
                     "Printer.Pass|"        +
                     "Printer.Doc2Blank|"   +
                     "Printer.BlankAndDoc|" +
                     "Printer.AddRep|"      +
                     " |";

 PUBLIC CONST
  pt_PDF         = 0; (* tPrinterType *)
  pt_Image       = 1;
  pt_Stick       = 2;
  pt_Cheque      = 3;
  pt_Pass        = 4;
  pt_Doc2Blank   = 5;
  pt_BlankAndDoc = 6;
  pt_AddRep      = 7;
  pt_Default     = 8;

  ptt_Pool       = " ⮪| ⮪|⮪ 筮 ";
  ptt_Main      = 0; (* tPaperTrayType *)
  ptt_Specified = 1;
  ptt_Manual    = 2;

  bt_Pool   = "ୠ|";
  bt_Single = 0; (* tBandType *)
  bt_Double = 1;

  pf_Cheque_Pool = "[72, 297]|[76, 297]|[80, 297]|[80, 279]|[80, 3251]";
  pf_Stick_6040  = "[60, 40]";
  pf_Stick_4525  = "[45, 25]";
  pf_Stick_Pool  = pf_Stick_6040 + "|" + pf_Stick_4525;
  pf_A3          = "A3";
  pf_A4          = "A4";
  pf_A5          = "A5";
  pf_A5onA4      = "A5onA4";

  o_Portrait  = 0;
  o_Landscape = 1;

  dm_Off        = 0;
  dm_Vertical   = 1;
  dm_Horizontal = 2;

  cp_OEM_Russian        = 866;
  cp_ISO_8859_1_Latin_I = 28591;
  cp_ANSI_Latin_I       = 1252;

 PRIVATE VAR
  pf_Left,
  pf_Top : FLOAT;
  PageNbr: NUMERIC[0];

 PUBLIC PROCEDURE IsNormalPrinterType(PrinterType: tPrinterType): SCAL[2];
 BEGIN
  RETURN (PrinterType = pt_Doc2Blank) | (PrinterType = pt_BlankAndDoc) | (PrinterType = pt_AddRep);
 END {IsNormalPrinterType};

 PUBLIC PROCEDURE IsVirtualPrinterType(PrinterType: tPrinterType): SCAL[2];
 BEGIN
  RETURN (PrinterType = pt_PDF) | (PrinterType = pt_Image);
 END {IsVirtualPrinterType};

 PUBLIC PROCEDURE PrinterTypeSESValId(PrinterType: tPrinterType): L_SES.tValId;
 BEGIN
  RETURN L_Str.PoolItem(pt_Pool_SESValId, PrinterType);
 END {PrinterTypeSESValId};

 PUBLIC PROCEDURE PrinterTypeSESSbjIdType(PrinterType: tPrinterType): L_SES.tSbjIdType;
  VAR
   Result: L_SES.tSbjIdType;
 BEGIN
  IF IsVirtualPrinterType(PrinterType) THEN
   Result:=L_SES.sit_User;
  ELSE
   Result:=L_SES.sit_Station;
  END;
  RETURN Result;
 END {PrinterTypeSESSbjIdType};

 PUBLIC PROCEDURE PrinterTypeAsStr(PrinterType: tPrinterType): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(pt_Pool_Genitive, PrinterType);
 END {PrinterTypeAsStr};

 PRIVATE PROCEDURE NeedConfigPrinterMsg(PrinterType: tPrinterType): STRING[];
 BEGIN
  RETURN "室 믮 ன '" + PrinterTypeAsStr(PrinterType) + "'";
 END {NeedConfigPrinterMsg};

 PRIVATE PROCEDURE GetPA_Check(IsError: SCAL[2]; PrinterType: tPrinterType);
 BEGIN
  L_General.Assert(~IsError, NeedConfigPrinterMsg(PrinterType));
 END {GetPA_Check};

 PRIVATE PROCEDURE GetSESValue(trgSbjIdType: L_SES.tSbjIdType; trgValId: L_SES.tValId): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  IF L_SES.GetValue(trgSbjIdType, trgValId, Result) THEN
   RETURN Result;
  ELSE
   RETURN "";
  END;
 END {GetSESValue};

 PRIVATE PROCEDURE TSScour_SinglePrinter(Printer: PRINTER): STRING[];
  CONST
   RedirectSign = "(७ࠢ";
  VAR
   Result: STRING[];
 BEGIN
  Result:=Printer.Name;
  IF Printer.TS THEN
   IF Str.Match(Result, "*" + RedirectSign + "*") THEN
    Result:=Str.Delete(Result, Str.Pos(Result, RedirectSign, , BACKWARD)+Str.Length(RedirectSign), 999999);
   ELSE
    Result:=Str.Delete(Result, Str.Pos(Result, ")", , BACKWARD)+1, 999999);
   END;
  END;
  RETURN Result;
 END {TSScour_SinglePrinter};

 PRIVATE PROCEDURE TS_Adjust(PrinterType: tPrinterType; PrinterName: STRING[]; Strong: SCAL[2]): STRING[];
  VAR
   Printers: PRINTERS;
   p       : NUMERIC[0];
   Result  : STRING[];
 BEGIN
  SETEMPTY(Result);
  IF ~!PrinterName THEN
   Printers:=PRINTERS.Create();
   p       :=0;
   WHILE (p < Printers.Count) & !Result DO
    IF PrinterName = TSScour_SinglePrinter(Printers.Item(p)) THEN
     Result:=Printers.Item(p).Name;
    END;
    INC(p);
   END;
   L_General.Assert(~Strong | ~!Result, " ன 㪠 騩 ਭ.|" + NeedConfigPrinterMsg(PrinterType));
  END;
  RETURN Result;
 END {TS_Adjust};

 PUBLIC PROCEDURE TS_ScourSingle(PrinterName: STRING[]): STRING[];
  VAR
   Printers: PRINTERS;
   Result  : STRING[];
 BEGIN
  Printers:=PRINTERS.Create();
  IF ~!Printers.Item(PrinterName) THEN
   Result:=TSScour_SinglePrinter(Printers.Item(PrinterName));
  ELSE
   Result:=PrinterName;
  END;
  RETURN Result;
 END {TS_ScourSingle};

 PRIVATE PROCEDURE intGetPA_Default(VAR PrinterName, PrinterDescription: STRING[]);
 BEGIN
  PrinterName:=TS_ScourSingle(GET_DEFAULT_PRINTER());
  SETEMPTY(PrinterDescription);
 END {intGetPA_Default};

 PRIVATE PROCEDURE GetPA_Common(    PrinterType       : tPrinterType;
                                    Strong            : SCAL[2];
                                VAR PrinterName,
                                    PrinterDescription: STRING[]);
  VAR
   trgValue: STRING[];
 BEGIN
  IF PrinterType = pt_Default THEN
   intGetPA_Default(PrinterName, PrinterDescription);
  ELSE
   trgValue          :=GetSESValue(PrinterTypeSESSbjIdType(PrinterType), PrinterTypeSESValId(PrinterType));
   PrinterName       :=L_Str.AnyItem(trgValue, ";", 0);
   PrinterDescription:=Str.Delete(trgValue, 0, Str.Length(PrinterName)+1);
  END;
  IF Strong & !PrinterName THEN
   IF IsNormalPrinterType(PrinterType) THEN
    intGetPA_Default(PrinterName, PrinterDescription);
   ELSE
    GetPA_Check(1, PrinterType);
   END;
  END;
  PrinterName:=TS_Adjust(PrinterType, PrinterName, Strong);
 END {GetPA_Common};

 PUBLIC PROCEDURE GetPA_Normal(    PrinterType  : tPrinterType;
                               VAR PrinterName  : STRING[];
                               VAR PaperTrayType: tPaperTrayType;
                               VAR PaperTrayIdx : tPaperTrayIdx);
  VAR
   PrinterDsc: STRING[];
 BEGIN
  L_General.CallUs(IsNormalPrinterType(PrinterType));
  GetPA_Common(PrinterType, 1, PrinterName, PrinterDsc);
  IF ~!PrinterName & ~!PrinterDsc THEN
   PaperTrayType:=L_Str.AnyItem(PrinterDsc, ";", 0);
   PaperTrayIdx :=L_Str.AnyItem(PrinterDsc, ";", 1);
   GetPA_Check(!PaperTrayType | ((PaperTrayType = ptt_Specified) & !PaperTrayIdx), PrinterType);
  ELSE
   PaperTrayType:=ptt_Main;
   SETEMPTY(PaperTrayIdx);
  END;
 END {GetPA_Normal};

 PUBLIC PROCEDURE Normal_PaperTrayType(PrinterType: tPrinterType): tPaperTrayType;
  VAR
   dmyPrinterName : STRING[];
   Result         : tPaperTrayType;
   dmyPaperTrayIdx: tPaperTrayIdx;
 BEGIN
  GetPA_Normal(PrinterType, dmyPrinterName, Result, dmyPaperTrayIdx);
  RETURN Result;
 END {Normal_PaperTrayType};

 PUBLIC PROCEDURE GetPA_Virtual(    PrinterType: tPrinterType;
                                VAR PrinterName,
                                    FilePath,
                                    FileName   : STRING[]);
  VAR
   PrinterDsc: STRING[];
 BEGIN
  L_General.CallUs(IsVirtualPrinterType(PrinterType));
  GetPA_Common(PrinterType, 1, PrinterName, PrinterDsc);
  FilePath:=L_Str.AnyItem(PrinterDsc, ";", 0);
  FileName:=L_Str.AnyItem(PrinterDsc, ";", 1);
  IF (PrinterType = pt_Image) & !FileName THEN
   FileName:="*.JPG";
  END;
  GetPA_Check(!FilePath | !FileName, PrinterType);
 END {GetPA_Virtual};

 PUBLIC PROCEDURE GetPA_Stick(VAR PrinterName,
                                  PaperForm  : STRING[]);
 BEGIN
  GetPA_Common(pt_Stick, 1, PrinterName, PaperForm);
  GetPA_Check(!PaperForm, pt_Stick);
 END {GetPA_Stick};

 PUBLIC PROCEDURE Stick_NamePaperForm(): STRING[];
  VAR
   PrinterName,
   dmyPaperForm: STRING[];
 BEGIN
  GetPA_Stick(PrinterName, dmyPaperForm);
  RETURN PRINTERS.Create().Item(PrinterName).OSDefaultDocumentProperties.PaperForm;
 END {Stick_NamePaperForm};

 PUBLIC PROCEDURE GetPA_Cheque(VAR PrinterName,
                                   PaperForm  : STRING[];
                               VAR BandType   : tBandType);
  VAR
   PrinterDsc: STRING[];
 BEGIN
  GetPA_Common(pt_Cheque, 1, PrinterName, PrinterDsc);
  BandType :=L_Str.AnyItem(PrinterDsc, ";", 0);
  PaperForm:=L_Str.AnyItem(PrinterDsc, ";", 1);
  GetPA_Check(!BandType | !PaperForm, pt_Cheque);
 END {GetPA_Cheque};

 PUBLIC PROCEDURE Cheque_BandType(): tBandType;
  VAR
   dmyPrinterName: STRING[];
   Result        : tBandType;
   dmyPaperForm  : STRING[];
 BEGIN
  GetPA_Cheque(dmyPrinterName, dmyPaperForm, Result);
  RETURN Result;
 END {Cheque_BandType};

 PUBLIC PROCEDURE Cheque_PaperForm(): STRING[];
  VAR
   dmyPrinterName: STRING[];
   dmyBandType   : tBandType;
   Result        : STRING[];
 BEGIN
  GetPA_Cheque(dmyPrinterName, Result, dmyBandType);
  RETURN Result;
 END {Cheque_PaperForm};

 PUBLIC PROCEDURE GetPA_Pass(VAR PrinterName: STRING[]);
  VAR
   dmyPrinterDsc: STRING[];
 BEGIN
  GetPA_Common(pt_Pass, 1, PrinterName, dmyPrinterDsc);
 END {GetPA_Pass};

 PUBLIC PROCEDURE GetPA_Default(VAR PrinterName: STRING[]);
  VAR
   dmyPrinterDsc: STRING[];
 BEGIN
  GetPA_Common(pt_Default, 1, PrinterName, dmyPrinterDsc);
 END {GetPA_Default};

 PUBLIC PROCEDURE GetPrinterName(PrinterType: tPrinterType): STRING[];
  VAR
   dmyPrinterDsc,
   Result      : STRING[];
 BEGIN
  GetPA_Common(PrinterType, 0, Result, dmyPrinterDsc);
  RETURN Result;
 END {GetPrinterName};

 PRIVATE PROCEDURE PageWidth(pf: STRING[]): FLOAT;
 BEGIN
  IF    pf = pf_A3             THEN RETURN 297;
  ELSIF pf = pf_A4             THEN RETURN 210;
  ELSIF pf = pf_A5             THEN RETURN 148.5;
  ELSIF pf = pf_A5onA4         THEN RETURN 210;
  ELSIF Str.Match(pf, "[*,*]") THEN RETURN STRTOFLD(Str.Item(pf, "[ ,", 0), NUMERIC);
  ELSE
   L_General.Error(" ଠ 㬠");
  END;
 END {PageWidth};

 PRIVATE PROCEDURE PageHeight(pf: STRING[]): FLOAT;
 BEGIN
  IF    pf = pf_A3             THEN RETURN 420;
  ELSIF pf = pf_A4             THEN RETURN 297;
  ELSIF pf = pf_A5             THEN RETURN 210;
  ELSIF pf = pf_A5onA4         THEN RETURN 148.5;
  ELSIF Str.Match(pf, "[*,*]") THEN RETURN STRTOFLD(Str.Item(pf, "[ ,", 1), NUMERIC);
  ELSE
   L_General.Error(" ଠ 㬠");
  END;
 END {PageHeight};

 PUBLIC PROCEDURE SetPageNbr(_PageNbr: NUMERIC[0]);
 BEGIN
  PageNbr:=_PageNbr;
 END {SetPageNbr};

 PUBLIC PROCEDURE IncPageNbr();
 BEGIN
  INC(PageNbr);
 END {IncPageNbr};

 PUBLIC PROCEDURE GetPageNbr(): NUMERIC[0];
 BEGIN
  RETURN PageNbr;
 END {GetPageNbr};

 PUBLIC PROCEDURE GetBlankMargins(VAR ShtLft, ShtTop: FLOAT);
  VAR
   ShtTopAsStr, ShtLftAsStr: STRING[];
 BEGIN
  IF ~L_SES.GetValue(L_SES.sit_Station, "Memo.ShtTop", ShtTopAsStr) | !Str.Condense(ShtTopAsStr) THEN
   ShtTop:=0;
  ELSE
   ShtTop:=STRTOFLD(ShtTopAsStr, NUMERIC);
  END;
  IF ~L_SES.GetValue(L_SES.sit_Station, "Memo.ShtLft", ShtLftAsStr) | !Str.Condense(ShtLftAsStr) THEN
   ShtLft:=0;
  ELSE
   ShtLft:=STRTOFLD(ShtLftAsStr, NUMERIC);
  END;
 END {GetBlankMargins};

 PRIVATE PROCEDURE SetPrintableRegionBegining(pj         : PRINTJOB;
                                              trgShtLeft,
                                              trgShtTop  : FLOAT);
 BEGIN
  IF pj.DocumentProperties.Orientation = o_Landscape THEN
   IF ~pj.RotatedLandscape THEN  (* ଠ쭠 졮 ਥ 90 ࠤ ࠢ*)
    pf_Left:=trgShtLeft - pj.DocumentProperties.BottomMargin;
    pf_Top :=trgShtTop  - pj.DocumentProperties.LeftMargin;
   ELSE
    pf_Left:=trgShtLeft - pj.DocumentProperties.TopMargin;
    pf_Top :=trgShtTop  - pj.DocumentProperties.RightMargin;
   END;
  ELSE
   pf_Left:=trgShtLeft - pj.DocumentProperties.LeftMargin;
   pf_Top :=trgShtTop  - pj.DocumentProperties.TopMargin;
  END;
 END {SetPrintableRegionBegining};

 PRIVATE PROCEDURE SetPrinterParams(pj         : PRINTJOB;
                                    PrinterType: tPrinterType);
  VAR
   PrinterName  : STRING[];
   PaperTrayType: tPaperTrayType;
   PaperTrayIdx : tPaperTrayIdx;
 BEGIN
  GetPA_Normal(PrinterType, PrinterName, PaperTrayType, PaperTrayIdx);
  IF ~!PrinterName THEN
   IF    PaperTrayType = ptt_Main THEN
    pj.DocumentProperties.PaperTray:=pj.Printer.DefaultDocumentProperties.PaperTray;
   ELSIF PaperTrayType = ptt_Specified THEN
    TRY
     pj.DocumentProperties.PaperTray:=PaperTrayIdx;
    EXCEPT
    ON Exc: Exc_Root DO
     IF Exc.ErrorCode = 472  THEN
      L_General.Error(Exc.MsgOriginalText + "| |" +
                      "室   ⪠  ன ਭ");
     ELSE
      RAISE();
     END;
    END; (* TRY *)
   ELSIF PaperTrayType = ptt_Manual THEN
    IF ~!pj.Printer.ASBPrintSettings.ManualPaperTray THEN
     pj.DocumentProperties.PaperTray:=pj.Printer.ASBPrintSettings.ManualPaperTray;
    END;
   ELSE
    L_General.CallUs(0);
   END; (* IF PaperTrayType = ... *)
  END; (* IF ~!PrinterName...*)
 END {SetPrinterParams};

 PUBLIC PROCEDURE CreatePrintJob(PrinterType: tPrinterType;
                                 Name,
                                 PaperForm  : STRING[];
                                 Orientation: tOrientation): PRINTJOB;
  VAR
   pj           : PRINTJOB;
   ShtLft,
   ShtTop       : FLOAT;
   PrinterName,
   dmyPrinterDsc: STRING[];
 BEGIN
  GetPA_Common(PrinterType, 1, PrinterName, dmyPrinterDsc);
  TRY
   pj:=PRINTJOB.Create(PrinterName);
  EXCEPT
  ON Exc:Exc_RunTime DO
   IF Exc.ErrorCode = L_General.ec_rt_UnableToOpenThePrinter THEN
    L_General.Error(" 㤠 ᮧ   . , ਭ|" +
                    "'" + PrinterName + "'|" +
                    "ࠢ.   .");
   ELSE
    RAISE();
   END;
  END; (* TRY *)

  IF PaperForm = pf_A5onA4 THEN
   PaperForm:=pf_A4;
  END;
  pj.Open(Name);
  pj.DocumentProperties.PaperForm   :=PaperForm;
  pj.DocumentProperties.Orientation :=Orientation;
  pj.DocumentProperties.LeftMargin  :=0;
  pj.DocumentProperties.TopMargin   :=0;
  pj.DocumentProperties.RightMargin :=0;
  pj.DocumentProperties.BottomMargin:=0;

  IF    PrinterType = pt_Doc2Blank THEN
   GetBlankMargins(ShtLft, ShtTop);
   SetPrintableRegionBegining(pj, ShtLft, ShtTop);
  ELSIF PrinterType = pt_Cheque THEN
   pf_Left:=0;
   pf_Top :=2;
  ELSE
   SetPrintableRegionBegining(pj, 0, 0);
  END;
  IF IsNormalPrinterType(PrinterType) THEN
   SetPrinterParams(pj, PrinterType);
  END;
  RETURN pj;
 END {CreatePrintJob};

 PUBLIC PROCEDURE ts_Create(size: FLOAT; face: STRING[]): TEXTSTYLE;
  VAR
   P: TEXTSTYLE;
 BEGIN
  P               :=TEXTSTYLE.Create();
  P.CharExtra     :=0;
  P.Font.Size     :=size;
  P.Font.Charset  :=FONT.Charset_Russian;
  P.Font.Face     :=face;
  P.Font.Weight   :=FONT.Weight_Normal;
  P.Font.Italic   :=0;
  P.Font.Underline:=0;
  RETURN P;
 END {ts_Create};

 PUBLIC PROCEDURE ts_Resize(Pattern: TEXTSTYLE; size: FLOAT): TEXTSTYLE;
  VAR
   P: TEXTSTYLE;
 BEGIN
  P          :=TEXTSTYLE.Create();
  P.Copy(Pattern);
  P.Font.Size:=size;
  RETURN P;
 END {ts_Resize};

 PUBLIC PROCEDURE extTypeS(pj: PRINTJOB;
                           X,
                           Y : NUMERIC[1];
                           S : STRING[];
                           P : TEXTSTYLE;
                           CP: NUMERIC[0]);
 BEGIN
  INC(X, pf_Left);
  INC(Y, pf_Top);
  pj.TextOut(X, Y, S, P, PRINTJOB.TextAlign_Left + PRINTJOB.TextAlign_BaseLine, CP);
 END {extTypeS};

 PUBLIC PROCEDURE TypeS(pj: PRINTJOB;
                        X,
                        Y : NUMERIC[1];
                        S : STRING[];
                        P : TEXTSTYLE);
 BEGIN
  extTypeS(pj, X, Y, S, P, cp_OEM_Russian);
 END {TypeS};

 PUBLIC PROCEDURE PrintCancelledXDoc(_XDocId: L_XDoc.tXDocId);
  VAR
   pj: PRINTJOB;
 BEGIN
  pj:=CreatePrintJob(pt_Default, " 㫨஢ 㬥", pf_A4, o_Portrait);
  pj.StartPage();
  TypeS(pj, 20, 20,
        "㬥 " + L_XDoc.extXDocStr(_XDocId, L_XDoc.xdast_wDate) + " 㫨஢",
        ts_Create(12, "Lucida Console"));
  pj.EndPage();
  pj.Close();
 END {PrintCancelledXDoc};

 PUBLIC PROCEDURE pj_Create_ChequeBand(Name: STRING[]): PRINTJOB;
 BEGIN
  RETURN CreatePrintJob(pt_Cheque, Name, Cheque_PaperForm(), o_Portrait);
 END {pj_Create_ChequeBand};

 PUBLIC PROCEDURE ChequeBand_BeforePrintLine(pj: PRINTJOB; VAR y: FLOAT; depth: FLOAT);
  VAR
   h: FLOAT;
 BEGIN
  h:=STRTOFLD(Str.Item(Cheque_PaperForm(), "[ ,]", 1), FLOAT);
  IF y + depth > h THEN
   pj.EndPage();
   pj.StartPage();
   y:=0;
  END;
 END {ChequeBand_BeforePrintLine};

 PRIVATE PROCEDURE IsRealPrinter(PrinterName: STRING[]): SCAL[2];
  VAR
   PrinterType: tPrinterType;
 BEGIN
  FOR_EACH PrinterType DO
   IF IsVirtualPrinterType(PrinterType) THEN
    IF PrinterName = GetPrinterName(PrinterType) THEN
     RETURN 0;
    END;
   END;
  END; (* FOR_EACH PrinterType ... *)
  RETURN 1;
 END {IsRealPrinter};

 PUBLIC PROCEDURE SetDuplexMode(pj: PRINTJOB; DuplexMode: SCAL[3]);
 BEGIN
  IF IsRealPrinter(pj.Printer.Name) THEN
   IF pj.Printer.ASBPrintSettings.UseDriverForDuplex THEN
    pj.DocumentProperties.Duplex:=DuplexMode;
   ELSE
    L_General.Assert(DuplexMode = dm_Off,
                     " 㬥⮢   ᪠|" +
                     "⮫쪮  ਭ  㫥 ஭ ");
   END;
  END;
 END {SetDuplexMode};

 PUBLIC PROCEDURE PrintPicture(pj                 : PRINTJOB;
                               X, Y, Width, Height: FLOAT;
                               FullFileName       : STRING[]);
  VAR
   oPicture: OBJECT;
 BEGIN
  IF FIO.Exists(FullFileName) THEN
   IF L_General.IsAcceptableProcessorVersion(014071019) THEN
    pj.LoadAndDrawPicture(X + pf_Left, Y + pf_Top, Width, Height, FullFileName);
   ELSE
    oPicture:=LOADPICTURE(FullFileName);
    pj.DrawPicture(X + pf_Left, Y + pf_Top, Width, Height, oPicture);
   END;
  ELSE
   L_StdFrm.RedInform("  䠩|" + FullFileName);
  END;
 END {PrintPicture};

 PUBLIC PROCEDURE GetDefaultBlankBackgroundSize(    pj       : PRINTJOB;
                                                    pf       : STRING[];
                                                VAR picWidth : FLOAT;
                                                VAR picHeight: FLOAT);
 BEGIN
  IF pj.DocumentProperties.Orientation = o_Landscape THEN
   picWidth :=PageHeight(pf);
   picHeight:=PageWidth(pf);
  ELSE
   picWidth :=PageWidth(pf);
   picHeight:=PageHeight(pf);
  END;
 END {GetDefaultBlankBackgroundSize};

 PUBLIC PROCEDURE extPrintBlankBackground(pj      : PRINTJOB;
                                          pf,
                                          FileName: STRING[]);
  VAR
   picWidth,
   picHeight: FLOAT;
 BEGIN
  GetDefaultBlankBackgroundSize(pj, pf, picWidth, picHeight);
  PrintPicture(pj, 0, 0, picWidth, picHeight, L_FIO.rf_GetFullFileName(FileName));
 END {extPrintBlankBackground};

 PUBLIC PROCEDURE PrintBlankBackground(pj      : PRINTJOB;
                                       FileName: STRING[]);
 BEGIN
  extPrintBlankBackground(pj, pf_A4, FileName);
 END {PrintBlankBackground};

 PUBLIC PROCEDURE PrintStamp(pj: PRINTJOB; X, Y, Width, Height: FLOAT; StampFileName: STRING[]);
  CONST
  VAR
   FullFileName: STRING[];
   Pt08        : TEXTSTYLE;
 BEGIN
  FullFileName:=L_FIO.rf_GetFullFileName(StampFileName);
  IF FIO.Exists(FullFileName) THEN
   PrintPicture(pj, X, Y, Width, Height, FullFileName);
  ELSE
   Pt08:=ts_Create(08, "Tahoma");
   TypeS(pj, X, Y, "  ⠬ ࣠樨", Pt08);
  END;
 END {PrintStamp};

 PUBLIC PROCEDURE GetPrinterList(): STRING[];
  VAR
   Printers: PRINTERS;
   Result  : STRING[];
   i       : NUMERIC[0];
 BEGIN
  SETEMPTY(Result);
  Printers:=PRINTERS.Create();
  i:=0;
  WHILE i < Printers.Count DO
   Result:=Result + Printers.Item(i).Name + "|";
   INC(i);
  END; (* WHILE i > 0...*)
  RETURN Result;
 END {GetPrinterList};

 PUBLIC PROCEDURE GetTrayList(PrinterName: STRING[]): STRING[];
  VAR
   Printers: PRINTERS;
   Printer : PRINTER;
   Result  : STRING[];
   i       : NUMERIC[0];
 BEGIN
  SETEMPTY(Result);
  Printers:=PRINTERS.Create();
  Printer :=Printers.Item(PrinterName);
  IF ~!Printer THEN
   i:=0;
   WHILE i < Printer.PaperTrays.Count DO
    IF Printer.PaperTrays.Item(i).TrayNo < 1000 THEN
     Result:=Result + Str.Slice(L_Str.IntStr(Printer.PaperTrays.Item(i).TrayNo) + " " + Printer.PaperTrays.Item(i).Name, 0, 22) + "|";
    END;
    INC(i);
   END; (* WHILE i > 0...*)
  END;
  RETURN Result;
 END {GetTrayList};

 PUBLIC PROCEDURE GetTrayNoByIdx(PrinterName: STRING[]; Idx: SCAL[1E3]): NUMERIC[0];
  VAR
   Printers: PRINTERS;
   Printer : PRINTER;
 BEGIN
  Printers:=PRINTERS.Create();
  Printer :=Printers.Item(PrinterName);
  IF ~!Printer THEN
   RETURN Printer.PaperTrays.Item(Idx).TrayNo;
  ELSE
   RETURN L_General.EMPTY_NUMBER();
  END;
 END {GetTrayNoByIdx};

 PUBLIC PROCEDURE GetTrayIdxByNo(PrinterName: STRING[]; No: NUMERIC[0]): SCAL[1E3];
  VAR
   Printers: PRINTERS;
   Printer : PRINTER;
   i       : NUMERIC[0];
 BEGIN
  Printers:=PRINTERS.Create();
  Printer :=Printers.Item(PrinterName);
  IF ~!Printer THEN
   i:=0;
   WHILE i < Printer.PaperTrays.Count DO
    IF No = Printer.PaperTrays.Item(i).TrayNo THEN
     RETURN i;
    END;
    INC(i);
   END; (* WHILE i > 0...*)
  END;
  RETURN L_General.EMPTY_NUMBER();
 END {GetTrayIdxByNo};

END.
