(*
%Copyright   (c) Polak IT
%Author      15.07.2011 13:02:20 POLAK\San
%Purpose     ⥪  প ⥬ 뫮   ࠢ ⠡ (Relations to Records)
%Type        library
%Specific    SYS
%Modified    28.02.2014 18:39:38 POLAK\San
%Environment
*)

 MODULE L_r2r [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_Str, Str, L_DBD, L_qs, L_kn;

 PUBLIC PROCEDURE IsStampStr(trgStr: STRING[]): SCAL[2];
 BEGIN
  RETURN Str.Match(trgStr, "r:?????,??????????");
 END {IsStampStr};

 PUBLIC PROCEDURE Decode(Stamp: STRING[]): STRING[];
  VAR
   fiOwn : LIKE(r2r.fiOwn);
   riOwn : LIKE(r2r.riOwn);
   OwnFN : STRING[200];
   Result: STRING[];
 BEGIN
  IF ~!Stamp THEN
   L_General.CallUs(IsStampStr(Stamp));
   fiOwn:=Str.Slice(Stamp, 2, 5);
   riOwn:=Str.Slice(Stamp, 8, 10);
   OwnFN:=DATABASE.Get().TableById(fiOwn).Name;
   SAVE((OwnFN));
   IF SEARCH((OwnFN), RECORDID, riOwn) THEN
    Result:=L_DBD.GetPrimaryIndexValue(OwnFN);
   ELSE
    L_General.CallUs(0);
   END;
   RESTORE((OwnFN));
  ELSE
   SETEMPTY(Result);
  END;
  RETURN Result;
 END {Decode};

 (*!!!*)PUBLIC PROCEDURE DecodeReferenceStamp(Stamp: STRING[]): STRING[];
 BEGIN
  RETURN Decode(Stamp);
 END {DecodeReferenceStamp};

 PRIVATE PROCEDURE BuildReferenceStamp(fi, ri: NUMERIC[0]): STRING[];
 BEGIN
  RETURN "r:" + L_Str.FixedIntStr(fi, 5) + "," + L_Str.FixedIntStr(ri, 10);
 END {BuildReferenceStamp};

 PUBLIC PROCEDURE PutReference(usrFileName, ownFileName: STRING[]): STRING[];
  VAR
   usrTable,
   ownTable : TABLE;
   Result   : STRING[];
 BEGIN
  usrTable:=TABLE.Get(usrFileName);
  r2r.fiUsr:=usrTable.Id;
  r2r.riUsr:=^(usrTable.Fields("RECORDID").ObtainName());
  L_General.CallUs(~!r2r.riUsr);
  ownTable:=TABLE.Get(ownFileName);
  r2r.fiOwn:=ownTable.Id;
  r2r.riOwn:=^(ownTable.Fields("RECORDID").ObtainName());
  SETEMPTY(r2r.RECORDID);
  Result:=BuildReferenceStamp(r2r.fiOwn, r2r.riOwn);
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   INSERT(r2r);
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode # L_General.ec_rt_UniquenessViolation THEN
    RAISE();
   END;
  END;
  RETURN Result;
 END {PutReference};

 PUBLIC PROCEDURE Put(usrFileName, ownFileName, ownKeyValue: STRING[]): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  IF ~!ownKeyValue THEN
   SAVE((ownFileName));
   SEARCH((ownFileName), (L_DBD.GetPrimaryIndex(ownFileName)), , ownKeyValue);
   Result:=PutReference(usrFileName, ownFileName);
   RESTORE((ownFileName));
  ELSE
   Result:="";
  END;
  RETURN Result;
 END {Put};

 PUBLIC PROCEDURE MakeCopy(srcUsrFileName: STRING[]; srcUsrRI, dstUsrRI: LIKE(r2r.riUsr));
  VAR
   srcUsrFI: LIKE(r2r.fiUsr);
 BEGIN
  srcUsrFI:=TABLE.Get(srcUsrFileName).Id;
  SELECT(r2r, r2r, , srcUsrFI, srcUsrRI);
  WHILE OCCURE(r2r) DO
   SAVE(r2r);
   SETEMPTY(r2r.RECORDID);
   r2r.riUsr:=dstUsrRI;
   INSERT(r2r);
   RESTORE(r2r);
   NEXT(r2r);
  END; (* WHILE OCCURE(r2r) ... *)
 END {MakeCopy};

 PRIVATE PROCEDURE intOwner_OnBeforeDelete(FileName: STRING[]);
  VAR
   ownTable: TABLE;
   ri      : FIELD;
   usrTable: TABLE;
   usrList : STRING[];
 BEGIN
  ownTable:=TABLE.Get(FileName);
  ri:=ownTable.Fields("RECORDID");
  IF ~!ri & SELECT(r2r, byOwn, ownTable.Id, ^(ri.ObtainName())) THEN
    usrList:="";
    WHILE OCCURE(r2r) DO
     usrTable:=DATABASE.Get().TableById(r2r.fiUsr);
     INC(usrList, usrTable.Description + "|");
     NARROWSEL(r2r, fiUsr);
     NEXTINEXTENDEDSEL(r2r, fiUsr);
    END;
    L_General.Error("   㤠, ⮬    뫠  㣨 䠩:" +
                    "||" + usrList);
  END; (* IF ~!ri & SELECT(r2r... *)
  L_qs.OnBeforeDelete(FileName);
  L_kn.OnBeforeDelete(FileName);
 END {intOwner_OnBeforeDelete};

 PUBLIC PROCEDURE User_OnBeforeModify(FileName: STRING[]);
  VAR
   usrTable: TABLE;
   ri      : FIELD;
 BEGIN
  usrTable:=TABLE.Get(FileName);
  ri:=usrTable.Fields("RECORDID");
  IF ~!ri & SELECT(r2r, r2r, usrTable.Id, ^(ri.ObtainName())) THEN
   REMSEL(r2r);
  END;
 END {User_OnBeforeModify};

 PUBLIC PROCEDURE OnBeforeInteractiveDelete(curFN: STRING[]);
  VAR
   curTable: TABLE;
   curLink : LINK;
   trgFN   : STRING[];
 BEGIN
  intOwner_OnBeforeDelete(curFN);
  User_OnBeforeModify(curFN);

  curTable:=TABLE.Get(curFN);
  FOR_EACH curLink IN curTable.Links DO
   IF (curLink.Kind = LINK.Kind_Lend) | (curLink.Kind = LINK.Kind_ReverseHistory) THEN
    trgFN:=curLink.CooIndex.Table.Name;
    IF curLink.Kind = LINK.Kind_Lend THEN
     CONFORM((curFN), (trgFN));
    ELSE
     HISTCONF(SYSDATE(), (curFN), (trgFN));
     FIRST((trgFN));
    END;
    WHILE OCCURE((trgFN)) DO
     OnBeforeInteractiveDelete(trgFN);
     NEXT((trgFN));
    END; (* WHILE OCCURE((trgFN)) *)
   END;
  END; (* FOR_EACH curLink ... *)
 END {OnBeforeInteractiveDelete};

 PUBLIC PROCEDURE Owner_OnBeforeDelete(curFN: STRING[]);
 BEGIN
  OnBeforeInteractiveDelete(curFN);
 END {Owner_OnBeforeDelete};

END.
