(*
%Copyright   (c) Polak IT
%Author      19.05.2004 11:30:06 POLAK\ATS
%Purpose     প ᯥ樠쭮 㤮
%Type        library
%Specific    SYS
%Modified    02.08.2013 18:54:46 POLAK\LDA
%Environment
*)

 MODULE L_SpcEq [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tEquipmentType = SCAL[7];
  tProximityCardReaderType = SCAL[4];

 PUBLIC CONST
  et_OPOSBCScanner          = 0;
  et_StreamVideo            = 1;
  et_IPCamera               = 2;
  et_CameraImageOrientation = 4;
  et_ProximityCardReader    = 5;
  et_pcr_COMPort            = 6;

  pcr_NotAlowed  = 0;
  pcr_Perco      = 1;
  pcr_Parsec     = 2;
  pcr_SevenSeals = 3;

 IMPORT L_SES, L_General, L_StdFrm, L_Str, L_WIA, L_EC, E_Utils, L_Print, L_BarCode, L_AR;

 PRIVATE CONST
  svi_OPOSBCSCanner          = "OPOSBCScanner";
  svi_StreamVideoDevice      = "StreamVideoDevice";
  svi_IPCamera               = "IPCamera";
  svi_CameraImageOrientation = "CameraImageOrientation";
  svi_ProximityCardReader    = "ProximityCardReader";
  svi_pcr_COMPort            = "ProximityCardReaderCOMPort";

  t_svd_NotAlowed = "  ࠭";
  t_svd_IPCamera  = "IP ";

  pcr_List = " ⠭|Perco|Parsec|Seven Seals";

 PRIVATE PROCEDURE svi_EquipmentName(eq: tEquipmentType): STRING[];
 BEGIN
  IF    eq = et_OPOSBCScanner THEN
   RETURN svi_OPOSBCSCanner;
  ELSIF eq = et_StreamVideo THEN
   RETURN svi_StreamVideoDevice;
  ELSIF eq = et_IPCamera THEN
   RETURN svi_IPCamera;
  ELSIF eq = et_CameraImageOrientation THEN
   RETURN svi_CameraImageOrientation;
  ELSIF eq = et_ProximityCardReader THEN
   RETURN svi_ProximityCardReader;
  ELSIF eq = et_pcr_COMPort THEN
   RETURN svi_pcr_COMPort;
  ELSE
   L_General.Error("L_SpcEq.svi_DeviceName:  ⨯ 㤮 " + eq);
  END;
 END {svi_EquipmentName};

 PUBLIC PROCEDURE GetSpcEqValue(eq: tEquipmentType): STRING[];
  VAR
   OPOSBCScanner         : SCAL[2];
   CameraImageOrientation: SCAL[3];
   ProximityCardReader   : tProximityCardReaderType;
   pcr_COMPort           : STRING[];
   svd_Id                : STRING[];
 BEGIN
  IF    eq = et_OPOSBCScanner THEN
   IF L_SES.GetValue(L_SES.sit_Station, svi_EquipmentName(eq), OPOSBCScanner) THEN
    RETURN OPOSBCScanner;
   ELSE
    RETURN "0";
   END;
  ELSIF (eq = et_StreamVideo) | (eq = et_IPCamera) THEN
   IF L_SES.GetValue(L_SES.sit_Station, svi_EquipmentName(et_StreamVideo), svd_Id) THEN
    RETURN svd_Id;
   ELSE
    RETURN t_svd_NotAlowed;
   END;
  ELSIF eq = et_CameraImageOrientation THEN
   IF L_SES.GetValue(L_SES.sit_Station, svi_EquipmentName(eq), CameraImageOrientation) THEN
    RETURN CameraImageOrientation;
   ELSE
    RETURN "0";
   END;
  ELSIF eq = et_ProximityCardReader THEN
   IF L_SES.GetValue(L_SES.sit_Station, svi_EquipmentName(eq), ProximityCardReader) THEN
    RETURN ProximityCardReader;
   ELSE
    RETURN "0";
   END;
   RETURN ;
  ELSIF eq = et_pcr_COMPort THEN
   IF L_SES.GetValue(L_SES.sit_Station, svi_EquipmentName(eq), pcr_COMPort) THEN
    RETURN pcr_COMPort;
   ELSE
    RETURN "";
   END;
  END;
 END {GetSpcEqValue};

 PUBLIC PROCEDURE IsInstalled(eq: tEquipmentType): SCAL[2];
  VAR
   DeviceDescr: STRING[];
 BEGIN
  DeviceDescr:=GetSpcEqValue(eq);
  IF    eq = et_OPOSBCScanner THEN
   RETURN STRTOFLD(DeviceDescr, SCAL);
  ELSIF eq = et_StreamVideo THEN
   RETURN DeviceDescr # t_svd_NotAlowed;
  ELSIF eq = et_IPCamera THEN
   RETURN DeviceDescr = svi_IPCamera;
  ELSIF eq = et_ProximityCardReader THEN
   RETURN STRTOFLD(DeviceDescr, SCAL) >= pcr_Perco;
  END;
  RETURN 0;
 END {IsInstalled};

 PRIVATE PROCEDURE E_IPCameraExist(D: DATE): SCAL[2];
 BEGIN
  RETURN L_EC.ExecE_Func_Scal2(D, PROCREF(E_Utils.IPCameraExist), 0);
 END {E_IPCameraExist};

 PRIVATE PROCEDURE GetDevicesDescription(DeviceType  : L_WIA.tDeviceType;
                                         VAR svd_Ids : STRING[];
                                         VAR svd_List: STRING[]);

  VAR
   _Ids, _List: STRING[];
 BEGIN
  SETEMPTY(_List, _Ids);
  L_WIA.GetDevicesDescription(DeviceType, _Ids, _List);
  svd_Ids :=L_Str.AddStrDlm(svd_Ids,  "|") + _Ids;
  svd_List:=L_Str.AddStrDlm(svd_List, "|") + _List;
 END {GetDevicesDescription};

 PRIVATE PROCEDURE asAdmin(): SCAL[2];
  VAR
   Login : STRING[];
   aoUser: LIKE(AcsObj.AcsObj);
 BEGIN
  IF L_General.IsAdmin() THEN
   RETURN 1;
  ELSIF UNDERREMOTECONTROL(Login) THEN
   L_General.CallUs(L_AR.StandOnUser(Login));
   aoUser:=AcsObj.AcsObj;
   L_General.CallUs(L_AR.StandOnGroup(L_General.AdminGroupName()));
   RETURN SEARCH(AOMem, AcsObj, AcsObj.AcsObj, aoUser);
  ELSE
   RETURN 0;
  END;
 END {asAdmin};

 PUBLIC PROCEDURE Setup();
  CONST
   COMPortPool = "1|2";
  VAR
   bcs_Device     : SCAL[2];
   bcs_Alowed     : STRING[];

   svd_Nbr        : NUMERIC[0];
   svd_Device,
   svd_Ids,
   svd_List,
   svd_Alowed     : STRING[];
   svd_Orientation: SCAL[3];

   pcr_Device     : tProximityCardReaderType;
   pcr_COMPort    : NUMERIC[0];

   i              : NUMERIC[0];
   Msg            : STRING[];
   NormalExit     : SCAL[2];
 BEGIN
  bcs_Device:=GetSpcEqValue(et_OPOSBCScanner);
  IF asAdmin() THEN
   bcs_Alowed:="";
  ELSE
   bcs_Alowed:="N";
  END;

  SETEMPTY(svd_List, svd_Ids, svd_Orientation);
  svd_Device     :=GetSpcEqValue(et_StreamVideo);
  svd_Orientation:=GetSpcEqValue(et_CameraImageOrientation);

  IF L_WIA.AcceptableWindowsVersion() THEN
   IF L_WIA.IsWIAInstalled(0) THEN
    svd_Ids   :=t_svd_NotAlowed;
    svd_List  :=t_svd_NotAlowed;
    GetDevicesDescription(L_WIA.dt_Camera,      svd_Ids, svd_List);
    GetDevicesDescription(L_WIA.dt_StreamVideo, svd_Ids, svd_List);
    IF E_IPCameraExist(SYSDATE()) THEN
     svd_Ids  :=svd_Ids  + "|" + svi_IPCamera;
     svd_List :=svd_List + "|" + t_svd_IPCamera;
    END;

    i:=0;
    LOOP
     IF !L_Str.PoolItem(svd_Ids, i) THEN
      EXIT;
     END;
     INC(i);
    END;

    IF i = 1 THEN
     svd_Ids   :=" 祭   ";
     svd_List  :=" 祭   ";
     svd_Alowed:="N";
    ELSE
     svd_Alowed:="";
    END;
   ELSE
    svd_Ids   :=t_svd_NotAlowed + "|";
    svd_List  :=" OLE-⮬⨧樨 WIA  ⠭|";
    svd_Alowed:="N";
   END;
  ELSE
   svd_Ids   :=t_svd_NotAlowed + "|";
   svd_List  :="㯭  Windows XP SP1  |";
   svd_Alowed:="N";
  END;

  svd_Nbr:=L_Str.PositionInPool(svd_Ids, svd_Device);
  IF !svd_Nbr THEN
   svd_Nbr:=0;
  END;

  pcr_Device :=GetSpcEqValue(et_ProximityCardReader);
  pcr_COMPort:=L_Str.PositionInPool(COMPortPool, GetSpcEqValue(et_pcr_COMPort));

  LOOP
   STDFORM(L_StdFrm.sf_Choice( "OPOS ᪠ -                                                 " +
                              "|%####################################################################%" +
                              "|" +
                              "|                                       ࠦ " +
                              "|%#########################################% %#####################%   " +
                              "|" +
                              "|뢠⥫ ᪮⠪    COM-                       " +
                              "|%#################%             %#%                                   ",
                              "%< ⠭|⠭>" +                         bcs_Alowed + "I" +
                              "%L<" + svd_List + "|>" +                               svd_Alowed + "I" +
                              "%L<|  90 ࠤᮢ|ࠢ  90 ࠤᮢ|>" + svd_Alowed + "I" +
                              "%L<" + pcr_List + ">I" +
                              "%L<" + COMPortPool + "|>YI"),
           " ᠭ ᯥ樠쭮 㤮 ",
           ,
           ,
           bcs_Device, svd_Nbr, svd_Orientation, pcr_Device, pcr_COMPort);

   NormalExit:=1;
   IF bcs_Device THEN
    TRY
     L_BarCode.InitScanner(1);
    EXCEPT
    ON Exc: Exc_Raise DO
     IF APP.Get().SupportMode | ~APP.Get().ProductiveCopy THEN
      L_StdFrm.Inform("  ᪠஬  " + L_Str.PoolItem("୮ | AP_Support", APP.Get().SupportMode) + "|" +
                      " ⮫쪮  ⫠筮 ०");
      NormalExit:=1;
     ELSE
      NormalExit:=0;
     END;
    ELSE
     RAISE();
    END; (* TRY *)
   END;

   L_SES.PutValueAtCurTrn(L_SES.sit_Station, svi_OPOSBCSCanner,          bcs_Device);

   L_SES.PutValueAtCurTrn(L_SES.sit_Station, svi_StreamVideoDevice,      L_Str.PoolItem(svd_Ids, svd_Nbr));
   L_SES.PutValueAtCurTrn(L_SES.sit_Station, svi_CameraImageOrientation, svd_Orientation);

   L_SES.PutValueAtCurTrn(L_SES.sit_Station, svi_ProximityCardReader,    pcr_Device);
   IF pcr_Device > pcr_NotAlowed THEN
    IF    ~!pcr_COMPort THEN
     L_SES.PutValueAtCurTrn(L_SES.sit_Station, svi_pcr_COMPort, L_Str.PoolItem(COMPortPool, pcr_COMPort));
    ELSIF (pcr_Device = pcr_Perco) | (pcr_Device = pcr_SevenSeals) THEN
     L_StdFrm.RedInform(" 㪠 COM- 祭 뢠⥫ ᪮⠪ ");
     NormalExit:=0;
    END;
   ELSE
    L_SES.PutValueAtCurTrn(L_SES.sit_Station, svi_pcr_COMPort, "");
   END;

   IF NormalExit THEN
    EXIT;
   END;
  END; (* LOOP *)
 END {Setup};

END.
