(*
%Copyright   (c) Polak Computer System
%Author      22.10.2000 16:59:29 SAN
%Purpose     ⥪ ࠡ  STDFORM
%Type        library
%Specific    SYS
%Modified    21.04.2014 15:34:17 POLAK\San
%Environment
*)

 MODULE L_StdFrm [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT Str, Math, L_General, L_Str, L_WDO, L_Msg, L_AR;

 PUBLIC VAR
  Warnings: STRING[];

 PRIVATE PERMANENT VAR
  Messages: OBJECT;

 PRIVATE PROCEDURE intHR(Title: STRING[]): STRING[];
 BEGIN
  IF ~!Title THEN
   Title:=" TITLE=" + Title;
  END;
  RETURN CHR(29) + "HR" + Title + CHR(29);
 END {intHR};

 PUBLIC PROCEDURE HR(): STRING[];
 BEGIN
  RETURN intHR("");
 END {HR};

 PUBLIC PROCEDURE THR(Title: STRING[]): STRING[];
 BEGIN
  RETURN intHR(Title);
 END {THR};

 PUBLIC PROCEDURE wGUI(): SCAL[2];
 BEGIN
  RETURN L_General.IsAcceptableProcessorVersion(14208380) & (APP.Get().PaletteKit.TextPalette.Count > 0);
 END {wGUI};

 PUBLIC PROCEDURE DrawButton(Text: STRING[]; Length: NUMERIC[0]; Type: STRING[]): STRING[];
 BEGIN
  RETURN "%<"+Str.Align(Text, Math.Max(Length, Str.Length(Text)), CENTRE)+">"+Type;
 END {DrawButton};

 PUBLIC PROCEDURE DrawEmpty(Length: NUMERIC[0]): STRING[];
 BEGIN
  RETURN L_Str.Repeat(" ", Length);
 END {DrawEmpty};

 PUBLIC PROCEDURE DrawInLabeledBox(Label, S: STRING[]; L: NUMERIC[0]): STRING[];
 BEGIN
  RETURN "" + Label + L_Str.Repeat("", L - Str.Length(Label) - 1) + "" + "|"+
         "" + S                                                     + "" + "|"+
         "" + L_Str.Repeat("", L)                                  + ""      ;
 END {DrawInLabeledBox};

 PUBLIC PROCEDURE _DrawInBox(S: STRING[]; L: NUMERIC[0]): STRING[];
 BEGIN
  RETURN DrawInLabeledBox("", S, L);
 END {_DrawInBox};

 PUBLIC PROCEDURE DrawLineInBox(S: STRING[]): STRING[];
 BEGIN
  RETURN _DrawInBox(S, Str.Length(S));
 END {DrawLineInBox};

 PUBLIC PROCEDURE GetMaxItemLength(Pool: STRING[]): NUMERIC[0];
  VAR
   i, j,
   l,
   maxl  : NUMERIC[0];
 BEGIN
  l:=Str.Length(Pool); i:=0; maxl:=0;
  REPEAT
   j:=Str.Pos(Pool, "|", i);
   maxl:=Math.Max(maxl, j-i);
   i:=j+1;
  UNTIL i >= l;
  RETURN maxl;
 END {GetMaxItemLength};

 PUBLIC PROCEDURE AlignPoolItems(Pool, Format: STRING[]): STRING[];
  VAR
   MaxLng,
   Idx   : NUMERIC[0];
   Result: STRING[];
 BEGIN
  SETEMPTY(Result);
  MaxLng:=GetMaxItemLength(Pool);
  Idx:=0;
  WHILE ~!L_Str.PoolItem(Pool, Idx) DO
   INC(Result, Str.Align(Str.Condense(L_Str.PoolItem(Pool, Idx)), MaxLng, ^(Format)) + "|");
   INC(Idx);
  END; (* WHILE ~!L_Str.PoolItem ... *)
  RETURN Result;
 END {AlignPoolItems};

 PUBLIC PROCEDURE Choice(Msg: STRING[]);
 BEGIN
  STDMSG(Msg, CHOICE);
 END {Choice};

 PUBLIC PROCEDURE Add2Warnings(Msg: STRING[]);
 BEGIN
  Warnings:=L_Str.AddStrDlm(Warnings, L_Str.CRLF) + Msg;
 END {Add2Warnings};

 PUBLIC PROCEDURE Warning(Msg: STRING[]);
 BEGIN
  IF L_General.BatchExecution_IsRunning() THEN
   Add2Warnings(Msg);
  ELSE
   Choice(Msg);
  END;
 END {Warning};

 PUBLIC PROCEDURE Confirm(Msg: STRING[]);
 BEGIN
  IF ~L_General.BatchExecution_IsRunning() THEN
   Choice(Msg);
  END;
 END {Confirm};

(*old*)
 PUBLIC PROCEDURE OrdinaryCHOICE(Msg: STRING[]);
 BEGIN
  Choice(Msg);
 END {OrdinaryCHOICE};

 PUBLIC PROCEDURE ConditionalCHOICE(Msg: STRING[]; Condition: SCAL[2]);
 BEGIN
  IF Condition THEN
   Choice(Msg);
  ELSE
   L_General.Error(Msg);
  END;
 END {ConditionalCHOICE};

 PUBLIC PROCEDURE BlueStop(Msg: STRING[]);
 BEGIN
  STDFORM(Str.StdFormEscape(Msg) + "|" + DrawButton("OK", 10, "X"), , INFORM, -1);
 END {BlueStop};

 PUBLIC PROCEDURE BlueHalt(Msg: STRING[]);
 BEGIN
  STDFORM(Str.StdFormEscape(Msg) + "|" + DrawButton("OK", 10, "Z"), , INFORM);
 END {BlueHalt};

(*old*) PUBLIC PROCEDURE ShowSTOP_AsINFORM(Msg: STRING[]);
 BEGIN
  BlueHalt(Msg);
 END {ShowSTOP_AsINFORM};

 PUBLIC PROCEDURE GetRandomNumber(): NUMERIC[0];
  VAR
   Dummy: TIME;
   Sec  : NUMERIC[3];
 BEGIN
  Dummy:=SYSTIME(Sec);
  RETURN Sec * 1000 MOD 10000;
 END {GetRandomNumber};

 PRIVATE PROCEDURE intAlternative(Msg: STRING[]; asStop: SCAL[2]): SCAL[2];
 BEGIN
  Msg:=Str.StdFormEscape(Msg) + "|" + " " + DrawButton("", 5, "X") + "       " + DrawButton("", 4, "DB") + " ";
  IF asStop THEN
   RETURN STDFORM(Msg, , CHOICE);
  ELSE
   RETURN STDFORM(Msg, , QUESTION);
  END;
 END {intAlternative};

 PUBLIC PROCEDURE Alternative(Msg: STRING[]): SCAL[2];
 BEGIN
  RETURN intAlternative(Msg, 0);
 END {Alternative};

 PUBLIC PROCEDURE RedAlternative(Msg: STRING[]): SCAL[2];
 BEGIN
  RETURN intAlternative(Msg, 1);
 END {RedAlternative};

 PUBLIC PROCEDURE Question(Msg: STRING[]);
 BEGIN
  IF ~L_General.BatchExecution_IsRunning() THEN
   STDMSG(Msg, QUESTION);
  END;
 END {Question};

 (*old*)
 PUBLIC PROCEDURE JustQuestion(Msg: STRING[]);
 BEGIN
  Question(Msg);
 END {JustQuestion};

 PUBLIC PROCEDURE TimedINFORM_AsSTOP(Msg: STRING[]; Timeout: NUMERIC[0]);
 BEGIN
  IF Timeout > 0 THEN
   STDMSG(Msg, INFORM_RED, , Timeout);
  END;
 END {TimedINFORM_AsSTOP};

 PUBLIC PROCEDURE TimedINFORM(Msg: STRING[]; Timeout: NUMERIC[0]);
 BEGIN
  IF Timeout > 0 THEN
   STDMSG(Msg, INFORM, , Timeout);
  END;
 END {TimedINFORM};

 PUBLIC PROCEDURE Inform(Msg: STRING[]);
 BEGIN
  IF ~L_General.BatchExecution_IsRunning() THEN
   STDMSG(Msg, INFORM);
  END;
 END {Inform};

 PUBLIC PROCEDURE RedInform(Msg: STRING[]);
 BEGIN
  IF ~L_General.BatchExecution_IsRunning() THEN
   STDMSG(Msg, INFORM_RED);
  END;
 END {RedInform};

 (*old*) PUBLIC PROCEDURE ShowINFORM_AsSTOP(Msg: STRING[]);
 BEGIN
  RedInform(Msg);
 END {ShowINFORM_AsSTOP};

 PUBLIC PROCEDURE intPartForm(PoolIns, Blank: STRING[]): STRING[];
  VAR
   Insert   : STRING[];
   WidthPos,
   Position,
   Count    : NUMERIC[0];
 BEGIN
  Count   :=1;
  Position:=0;
  LOOP
   Insert  :=Str.Item(PoolIns, "%", Count-1);
   Position:=Str.Pos(Blank, "%", Position);
   IF Position = Str.Length(Blank) THEN
    L_General.CallUs(!Insert);
    EXIT;
   ELSE
    L_General.CallUs(~!Insert);
    WidthPos:=Str.Pos(Blank, "%", Position+1) - Position;
    Blank   :=Str.Delete(Blank, Position, WidthPos);
    IF L_Str.Right(Insert, 1) = "M" THEN
     INC(WidthPos, 2);
    END;
    IF L_Str.Right(Insert, 1) = "T" THEN
     INC(WidthPos, 2);
    END;
    IF L_Str.Right(Insert, 1) = "E" THEN
     DEC(WidthPos, 4);
    END;
    IF L_Str.Right(Insert, 1) = "O" THEN
     DEC(WidthPos, 4);
    END;
    Insert  :=Str.Insert(Insert, L_Str.IntStr(WidthPos-1), (Insert[0]="L") | (Insert[0]="C") | (Insert[0]="R"));
    Blank   :=Str.Insert(Blank, Insert, Position+1);
    INC(Count);
    INC(Position);
   END;
  END;
  Position:=0;
  LOOP
   Position:=Str.Pos(Blank, "$", Position);
   IF Position = Str.Length(Blank) THEN
    RETURN Blank;
   END;
   Blank:=Str.Insert(Str.Delete(Blank, Position, 1), "%", Position);
  END; (* LOOP *)
 END {intPartForm};

 PUBLIC PROCEDURE PartForm(part: STRING[]): STRING[];
 BEGIN
  RETURN intPartForm(Str.Item(part, "@", 0), Str.Item(part, "@", 1));
 END {PartForm};

 PUBLIC PROCEDURE sf_Choice_wAddBtns(Blank, PoolIns, AddBtnsBlank, AddBtnsPoolIns: STRING[]): STRING[];
  VAR
   btnsDlm,
   lastDlm: STRING[];
 BEGIN
  IF wGUI() THEN lastDlm:="|";  SETEMPTY(btnsDlm);
  ELSE           lastDlm:="||"; btnsDlm:=" ";
  END;
  RETURN intPartForm(PoolIns + "%C< 믮 >DB%C< ⮯! >Z" + AddBtnsPoolIns,
                     Blank   + lastDlm + "%###########%" + btnsDlm + "%#########%" + btnsDlm + AddBtnsBlank);
 END {sf_Choice_wAddBtns};

 PUBLIC PROCEDURE sf_Choice(Blank, PoolIns: STRING[]): STRING[];
 BEGIN
  RETURN sf_Choice_wAddBtns(Blank, PoolIns, EMPTYOF(STRING[]), EMPTYOF(STRING[]));
 END {sf_Choice};

 PUBLIC PROCEDURE sf_Confirm(Blank, PoolIns: STRING[]): STRING[];
  CONST
   CncBtnType_Pool = "Z|X";
 BEGIN
  RETURN intPartForm(PoolIns + "%< 믮 >DB %< ⮯! >" + L_Str.PoolItem(CncBtnType_Pool, L_General.BatchExecution_IsRunning()),
                     Blank   + "||%###########% %##########%");
 END {sf_Confirm};

 PUBLIC PROCEDURE extCrucialChoice(curFileN, Msg4Show, Msg4Message, GroupN: STRING[]; wChoice: SCAL[2]);
  VAR
   Text,
   Recievers: STRING[];
 BEGIN
  IF wChoice THEN
   Choice(Msg4Show);
  END;
  IF L_AR.StandOnGroup(GroupN) THEN
   Recievers:=" 㯯 " + GroupN + ' "' + AcsObj.FullName + '"';
  ELSE
   Recievers:=" ⮏";
  END;
  Text:="";
  LOOP
   STDFORM("|" + Str.StdFormEscape(Msg4Show) + "|" +
           intHR(", ᭨ ᢮ 롮") +
           "|%72,3KI|||" +
           "%C<(C饭  訬 ᭥ﬨ  " + Recievers + ")>M" +
           "|" + DrawButton("믮", 11, "DB") + DrawButton("⮯!", 9, "Z"),
           "  祭  롮! ", STOP, , Text);
   IF Str.Length(Text) >= 5 THEN
    EXIT;
   ELSE
    Inform(",  ஡");
   END;
  END; (* LOOP *)
  L_Msg.PutCurRecordCrucialChoice(curFileN, Msg4Message + L_Str.CRLF + '᭥ 롮: '+ Text, GroupN);
 END {extCrucialChoice};

 PUBLIC PROCEDURE CrucialChoice_Full(Msg4Show, Msg4Message, GroupN: STRING[]);
 BEGIN
  IF ~!RTI.Get().CurZone & ~!RTI.Get().CurZone.MasterTable THEN
   extCrucialChoice(RTI.Get().CurZone.MasterTable.Name, Msg4Show, Msg4Message, GroupN, 1);
  ELSE
   extCrucialChoice("",                                 Msg4Show, Msg4Message, GroupN, 1);
  END;
 END {CrucialChoice_Full};

 PUBLIC PROCEDURE CrucialChoice(Msg, GroupN: STRING[]);
 BEGIN
  CrucialChoice_Full(Msg, '  롮: ' + Str.Flatten(Msg), GroupN);
 END {CrucialChoice};

 PUBLIC PROCEDURE CrucialChoice_FileExplicit(curFileN, Msg, GroupN: STRING[]);
 BEGIN
  extCrucialChoice(curFileN, Msg, '  롮: ' + Str.Flatten(Msg), GroupN, 1);
 END {CrucialChoice_FileExplicit};

 PUBLIC PROCEDURE SFMenuStyle(): STRING[];
 BEGIN
  IF ~L_General.IsAcceptableProcessorVersion(14172350) | ~wGUI() THEN
   RETURN "MENU";
  ELSE
   RETURN "INFORM_NEUTRAL";
  END;
 END {SFMenuStyle};

 PUBLIC PROCEDURE ShowListAsMenu_wButtons(List,
                                          Title,
                                          Caption     : STRING[];
                                          InitialValue: NUMERIC[0];
                                          Buttons     : STRING[][];
                                          VAR
                                           ListResult,
                                           FormResult : NUMERIC[0]);
  VAR
   CurS   : STRING[];
   maxL,
   maxX,
   maxY,
   sizY,
   Count  : NUMERIC[0];
   oPrompt: PROMPT_LINE;
   Msg    : STRING[];
   bt     : STRING[];
   i      : NUMERIC[0];
 BEGIN
  oPrompt:=PROMPT_LINE.Create("⮢ ''");
  Count:=0;
  maxL :=Str.Length(Title);
  LOOP
   CurS:=L_Str.PoolItem(List, Count);
   PROMPT(CurS);
   IF !CurS THEN
    EXIT;
   END;
   maxL:=Math.Max(maxL, Str.Length(CurS));
   INC(Count);
  END;
  IF ~!Title THEN
   Title:=" " + Str.Condense(Title) + " ";
  END;
  IF Count = 0 THEN
   STDFORM("᮪ |" + DrawButton("", 10, "X"), Title, INFORM, -1);
  ELSE
   GE_APPWIN_SIZE(maxX, maxY);
(*
   INC(Count, wGUI());
*)
   sizY:=Math.Min(Math.Max(Count, 2), maxY*0.8);
   ListResult:=InitialValue;
   IF ~!Caption THEN
    Caption:=Caption + "|";
   ELSIF wGUI() & (sizY < Count) THEN
    Caption:=Caption + "|";
   END;
   SETEMPTY(oPrompt);
   Msg:=Str.StdFormEscape(Caption) +
        "%" + L_Str.IntStr(Math.Min(maxL, maxX*0.9)) + "," +
         L_Str.IntStr(sizY) + "AI" + L_Str.Repeat("|", sizY-1);
   i:=0;
   WHILE (i <= HIGH(Buttons)) & ~!Buttons[i] DO
    IF    i = 0 THEN
     INC(Msg, "|");
     IF ~wGUI() THEN
      INC(Msg, "|");
     ELSIF wGUI() & (sizY < Count) THEN
      INC(Msg, "|"); (*  ப⪨ *)
     END;
     bt:="D";
    ELSIF i = 1 THEN
     bt:="X";
    ELSE
     bt:="B";
    END;
    INC(Msg, DrawButton(Buttons[i], Str.Length(Buttons[i])+2, bt) + "  ");
    INC(i);
   END;
   FormResult:=STDFORM(Msg, Title, ^(SFMenuStyle()), -1, List, ListResult);
  END;
 END {ShowListAsMenu_wButtons};

 PUBLIC PROCEDURE ShowListAsMenu_Full(List,
                                      Title,
                                      Caption     : STRING[];
                                      InitialValue: NUMERIC[0]): NUMERIC[0];
  VAR
   FormResult,
   ListResult : NUMERIC[0];
 BEGIN
  IF wGUI() THEN
   ShowListAsMenu_wButtons(List, Title, Caption, InitialValue, ["믮", "⮯!"], ListResult, FormResult);
  ELSE
   ShowListAsMenu_wButtons(List, Title, Caption, InitialValue, "", ListResult, FormResult);
  END;
  RETURN ListResult;
 END {ShowListAsMenu_Full};

 PUBLIC PROCEDURE ShowListAsMenu_wInit(List,
                                       Title       : STRING[];
                                       InitialValue: NUMERIC[0]): NUMERIC[0];
 BEGIN
  RETURN ShowListAsMenu_Full(List, Title, "", InitialValue);
 END {ShowListAsMenu_wInit};

 PUBLIC PROCEDURE ShowListAsMenu_wCaption(List,
                                   Title,
                                   Caption: STRING[]): NUMERIC[0];
 BEGIN
  RETURN ShowListAsMenu_Full(List, Title, Caption, 0);
 END {ShowListAsMenu_wCaption};

 PUBLIC PROCEDURE ShowListAsMenu(List, Title: STRING[]): NUMERIC[0];
 BEGIN
  RETURN ShowListAsMenu_wInit(List, Title, 0);
 END {ShowListAsMenu};

 PUBLIC PROCEDURE PrepareLongStringForASLPART(S: STRING[]; VAR ErrorDescriptionArr:STRING[][]; LStr: NUMERIC[0]);
 BEGIN
  SETEMPTY(ErrorDescriptionArr);
  Str.Format(ErrorDescriptionArr,
             L_Str.ExclIllegalChars(S, "|" + L_Str.CRLF, " "),
             LStr,
             LEFT);
 END {PrepareLongStringForASLPART};

 PUBLIC PROCEDURE CustomQuestion(Msg: STRING[]);
  VAR
   React                           : NUMERIC[0];
   DoNotShowMessageInCurrentSession: SCAL[2];
 BEGIN
  IF ~VAR_IMPL.Get(Messages).Initialized THEN
   Messages:=L_General.CreateDictionary();
  END;
  IF Messages.Exists(Msg) THEN
   DoNotShowMessageInCurrentSession:=1;
  ELSE
   DoNotShowMessageInCurrentSession:=0;
  END;

  IF ~DoNotShowMessageInCurrentSession THEN
   React:=STDFORM(PartForm("%L< 뢠 ᮮ饭  ⥪饩 ᨨ>E" +
                           "%<   ⮯!   >Z" +
                           "%< 믮 >D" +
                           "@" +
                           "|" +
                           Msg +
                           "||" +
                           "%############################################%" +
                           "||" +
                           "%############% %###########%"),
                           "",
                           INFORM,
                           ,
                           DoNotShowMessageInCurrentSession);

   IF DoNotShowMessageInCurrentSession THEN
    Messages.Add(Msg, "1");
   END;
  END;
 END {CustomQuestion};

 PUBLIC PROCEDURE WrongDB(Msg: STRING[]);
 BEGIN
  INC(Msg, THR("䨪 ஡"));
  IF L_WDO.IamInModification() THEN
   INC(Msg,  " 訡    ");
   INC(Msg, "|஡  ,  - ");
  ELSE
   INC(Msg,  "冷  ");
  END;
  INC(Msg, THR("   ࠩ 砥"));
  IF L_General.IsAdmin() THEN
   INC(Msg,  "᫨  ,       浪," +
            "|  㬥  ," +
            "|,   ࠧࠡ稪");
  ELSE
   INC(Msg,  "᫨ ࠧ ᠬ⥫쭮 㤭, ,   ");
  END;
  STDMSG(Msg, STOP_AND_CALL_ADMIN);
 END {WrongDB};

END.
