(*
%Copyright   (c) Polak IT
%Author      23.11.2004 19:40:33 POLAK\LDA
%Purpose     ⥪  ࠡ  ⠩ࠬ (KBWAIT)
%Type        library
%Specific    SYS
%Modified    25.11.2011 18:49:53 POLAK\MLB
%Environment
*)

 MODULE L_Timer [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PRIVATE CONST
  t_Amount = 10;

 PRIVATE PERMANENT VAR
  t_Ids,
  t_ProcToRun      : STRING[][t_Amount];
  t_TimeToRun,
  t_TimeToNextCalls: NUMERIC[0][t_Amount];
  last_HRTIMER     : FLOAT;

 IMPORT L_General, Math, L_Str;

 PRIVATE PROCEDURE DecreaseTimers(TimeToDecrease: NUMERIC[0]);
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=0;
  WHILE (i <= HIGH(t_Ids)) & ~!t_Ids[i] DO
   DEC(t_TimeToRun[i], Math.Min(TimeToDecrease, t_TimeToRun[i]));
   INC(i);
  END; (* WHILE i <= HIGH(t_Ids)...*)
 END {DecreaseTimers};

 PRIVATE PROCEDURE GetMinTimeToRun(): NUMERIC[0];
  VAR
   i, minTime: NUMERIC[0];
 BEGIN
  SETEMPTY(minTime);
  i:=0;
  WHILE (i <= HIGH(t_Ids)) & ~!t_Ids[i] DO
   IF t_TimeToRun[i] # 0 THEN
    IF !minTime THEN
     minTime:=t_TimeToRun[i];
    ELSE
     minTime:=Math.Min(minTime, t_TimeToRun[i]);
    END;
   END;
   INC(i);
  END; (* WHILE i <= HIGH(t_Ids)...*)
  IF ~!minTime THEN
   RETURN minTime;
  ELSE
   RETURN 0;
  END;
 END {GetMinTimeToRun};

 PRIVATE PROCEDURE KbWait();
  VAR
   minTime: NUMERIC[0];
 BEGIN
  last_HRTIMER:=HIGH_RESOLUTION_TIMER();
  minTime     :=GetMinTimeToRun();
  KBWAIT(minTime, minTime);
 END {KbWait};

 PRIVATE PROCEDURE GetTimerIdx(TimerId: STRING[]): NUMERIC[0];
  VAR
   i, Idx: NUMERIC[0];
 BEGIN
  i:=0;
  WHILE (i <= HIGH(t_Ids)) & ~!t_Ids[i] DO
   IF t_Ids[i] = TimerId THEN
    RETURN i;
   END;
   INC(i);
  END; (* WHILE i <= HIGH(t_Ids)...*)
  L_General.CallUs(i < t_Amount);
  t_Ids[i]:=TimerId;
  RETURN i;
 END {GetTimerIdx};

 PUBLIC PROCEDURE SetTimer(TimerId        : STRING[];
                           TimeToFirstCall: NUMERIC[0];
                           TimeToNextCall : NUMERIC[0];
                           ProcToRun      : STRING[]);
  VAR
   Idx: NUMERIC[0];
 BEGIN
  IF L_General.VariableIsInitialized(last_HRTIMER) THEN
   DecreaseTimers(HIGH_RESOLUTION_TIMER() - last_HRTIMER);
  END;
  Idx                   :=GetTimerIdx(TimerId);
  t_TimeToRun[Idx]      :=TimeToFirstCall;
  t_TimeToNextCalls[Idx]:=TimeToNextCall;
  t_ProcToRun[Idx]      :=ProcToRun;
  KbWait();
 END {SetTimer};

 PUBLIC PROCEDURE ResetTimer(TimerId: STRING[]);
  VAR
   Idx: NUMERIC[0];
 BEGIN
  TRY [TERMBYUSERMODE(TERMBYUSERMODE_IGNORE)]
   Idx:=GetTimerIdx(TimerId);
   SETEMPTY(t_Ids[Idx], t_TimeToRun[Idx], t_TimeToNextCalls[Idx], t_ProcToRun[Idx]);
  EXCEPT
  END;
 END {ResetTimer};

 PUBLIC PROCEDURE UpdateTimers();
  VAR
   List,
   Command: STRING[];
   i      : NUMERIC[0];
 BEGIN
  DecreaseTimers(GetMinTimeToRun());
  SETEMPTY(List);
  i:=0;
  WHILE (i <= HIGH(t_Ids)) & ~!t_Ids[i] DO
   L_General.CallUs(t_TimeToRun[i] >= 0);
   IF t_TimeToRun[i] = 0 THEN
    IF    L_Str.IsStrInStr(t_ProcToRun[i], ",") THEN
     Command:='RunProg, "' + t_ProcToRun[i] + '"'
    ELSIF L_Str.IsStrInStr(t_ProcToRun[i], ".") THEN
     Command:='CALL, "' + t_ProcToRun[i] + '"';
    ELSIF !t_ProcToRun[i] THEN
     SETEMPTY(Command);
    ELSE
     L_General.CallUs(0);
    END;
    IF ~!Command THEN
     IF ~!List THEN
      INC(List, ",");
     END;
     INC(List, Command);
    END;
    t_TimeToRun[i]:=t_TimeToNextCalls[i];
   END;
   INC(i);
  END; (* WHILE i <= HIGH(t_Ids)...*)
  KbWait();
  LIKEMENU(^List);
 END {UpdateTimers};

 PUBLIC PROCEDURE InitTimers();
 BEGIN
  SETEMPTY(t_Ids);
 END {InitTimers};

END.
