(*
%Copyright   (c) Polak Computer System
%Author
%Purpose      室 
%Type        library
%Specific    SYS
%Modified    26.02.2013 13:52:40 ROBOT by POLAK\LDA
%Environment
*)

 MODULE L_Tx2000 [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_OwnData, L_APtm, L_AFM, L_Staff, L_Prop,
        L_PRoll, L_Math, L_Tax, P_Staff, Str, Memo, L_Env;

 PRIVATE CONST
  _mc_TaxF   = 099;
  _mc_TaxC   = 100;
  _mc_Pns    = 101;

  _ta_PdTax  = 07;
  _ta_OvrTxC = 08;
  _ta_NoTxd  = 23;
  _ta_OvrTxF = 48;
  _ta_NYInc  = 30;
  _ta_DurHld = 35;
  _ta_RetOvr = 80;

  _tr_PrInc  = 00;
  _tr_IncWA  = 01;
  _tr_Persn  = 02;
  _tr_Child  = 03;
  _tr_DecWA  = 04;
  _tr_PdTxF  = 10;
  _tr_PdTxC  = 05;
  _tr_NoTxd  = 06;
  _tr_RetOr  = 07;
  _tr_PdPns  = 08;
  _tr_TxFre  = 09;

 PRIVATE VAR
  InitIsMade    : SCAL[2];

  ManIsWorkingInYear : SCAL[2][12];
  FutTimes           : NUMERIC[3][12];

  AvgInYearMin,
  ActualMinimum : NUMERIC[2];
  MinimumsArr   : NUMERIC[2][12];
  ActualTaxBase : NUMERIC[2][7];

  Man_TotInc    : NUMERIC[2][12];

  Man_Dec,
  Man_Pns,
  Man_OwnPInc,
  Man_ToTax,
  Man_PaidTaxF,
  Man_PaidTaxC  : NUMERIC[2];

  _CurDate      : DATE;

  _TaxAmo: NUMERIC[0];
  _TaxBas,
  _TaxSum: NUMERIC[2][7];
  _TaxPrc: NUMERIC[0][7];

  _MESAmo: NUMERIC[0];
  _MESBas,
  _MESNbr: NUMERIC[2][7];
  _MESArr: SCAL[3][12];
  EstimDone: SCAL[2];

  PassWord: STRING[];

 PRIVATE PROCEDURE InterpretValueAsSum(_InqType: LIKE(TaxInq.InqType)): SCAL[2];
 BEGIN
  RETURN (_InqType = L_Tax.inqt_Other) |
         (_InqType = L_Tax.inqt_Fict)  |
         (_InqType = L_Tax.inqt_InAPFY);
 END {InterpretValueAsSum};

 PRIVATE PROCEDURE PrepareMinimus(P: DATE);
  VAR
   _Period      : LIKE(Month.Period);
   MonthMinimum,
   _Minimum     : NUMERIC[2];
   MonthsDone   : SCAL[13];
 BEGIN
  SETZERO(_Minimum, MonthsDone, MinimumsArr);
  _Period:=L_DatTim.YearBeg(Memo.tmp_cd());
  REPEAT
   L_APtm.MonthAccess(_Period);
   MonthMinimum:=L_OwnData.ODMinimum(_Period);
   MinimumsArr[L_DatTim.MONTH(_Period)]:=MonthMinimum;
   INC(_Minimum, MonthMinimum);
   INC(MonthsDone);
   _Period:=L_DatTim.IncMonth(_Period, +1);
  UNTIL _Period > P;
  TaxScal(P);
  ActualTaxBase:=_TaxBas;
  AvgInYearMin:=(_Minimum + MonthMinimum*(12 - MonthsDone)) / 12;
  WHILE MonthsDone <= 11 DO
   MinimumsArr[MonthsDone]:=MonthMinimum;
   INC(MonthsDone);
  END;
 END {PrepareMinimus};

 PUBLIC PROCEDURE FillManIsWorking(P: DATE);
  VAR
   p: DATE;
 BEGIN
  SAVE(Staff);
  SETZERO(ManIsWorkingInYear);
  SELECT(Staff, Man, , Man.Man);
  WHILE OCCURE(Staff) DO
   p:=L_DatTim.YearBeg(P);
   WHILE p <= P DO
    IF L_Staff.IsWorking(p) THEN
     ManIsWorkingInYear[L_DatTim.MONTH(p)]:=1;
    END;
    p:=L_DatTim.IncMonth(p, +1);
   END;
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
  RESTORE(Staff);
 END {FillManIsWorking};

 PUBLIC PROCEDURE _NTax_InitValues(P: DATE; fictOnly: SCAL[2]);
  VAR
   _TaxAdv: LIKE(ManTA.TaxAdv);
   _StartD: LIKE(ManTA.StartD);
   _Number: NUMERIC[0];
 BEGIN
  L_Tax.Tax_CheckYear(P);
  IF ~InitIsMade THEN
   L_AFM.MakeIndex("_MnTA", "_Man    : LIKE(Man.Man)       ;"+
                               "_MdfType: SCAL[32]            ;"+
                               "_TaxAdv : LIKE(TaxAdv.TaxAdv) ;"+
                               "_Source : SCAL[2]             ;"+
                               "_TaxInq : LIKE(ManTA.TaxInq)  ;"+
                               "_StartD : DATE                ;"+
                               "Number  : NUMERIC[0]           ");
   PrepareMinimus(P);
   InitIsMade:=1;
  END;

  L_General.CallUs(~SELECT(_MnTA, _MnTA, Man.Man));

  SETEMPTY(Man_OwnPInc, Man_TotInc, Man_ToTax, Man_PaidTaxF, Man_PaidTaxC);
  SELECT(TaxInq, MyFirm, , Staff.MyFirm, Staff.Man);
  WHILE OCCURE(TaxInq) DO
   IF fictOnly = (TaxInq.InqType = L_Tax.inqt_Fict) THEN
    SETEMPTY(_TaxAdv, _StartD);
    SELECT(ManTA, TaxInq, , TaxInq.TaxInq);
    WHILE OCCURE(ManTA) DO
     IF (ManTA.TaxAdv # _TaxAdv) | (ManTA.StartD # _StartD) THEN
      _TaxAdv:=ManTA.TaxAdv;
      _StartD:=ManTA.StartD;
      _Number:=1;
     ELSE
      INC(_Number);
     END;
     IF (ManTA.StartD <= Memo.tmp_me()) & (ManTA.StopD >= L_DatTim.YearBeg(Memo.tmp_cd())) THEN
      CONFORM(ManTA, TaxAdv);
      _MnTA._Man    :=Staff.Man;
      _MnTA._MdfType:=TaxAdv.Reason;
      _MnTA._TaxAdv :=ManTA.TaxAdv;
      _MnTA._Source :=TaxInq.InqType = L_Tax.inqt_Other;
      _MnTA._TaxInq :=ManTA.TaxInq;
      _MnTA._StartD :=ManTA.StartD;
      _MnTA.Number  :=_Number;
      INSERT(_MnTA);
     END;
     NEXT(ManTA);
    END; (* WHILE OCCURE(ManTA) ... *)
   END;
   NEXT(TaxInq);
  END; (* WHILE OCCURE(TaxInq) ... *)
  FillManIsWorking(P);
 END {_NTax_InitValues};

 PUBLIC PROCEDURE NTax_InitValues(P: DATE);
 BEGIN
  _NTax_InitValues(P, 0);
 END {NTax_InitValues};

 PUBLIC PROCEDURE NTax_FreeValues();
 BEGIN
  IF InitIsMade THEN
   L_AFM.AF_PurgeTemp("_MnTA");
   InitIsMade:=0;
  END;
 END {NTax_FreeValues};

 PUBLIC PROCEDURE AddManTA(P     : DATE;
                           _TaxAdv: LIKE(TaxAdv.TaxAdv);
                           s      : LIKE(ManTA.Value));
 BEGIN
  IF s # 0 THEN
   L_General.Assert(SEARCH(TaxAdv, TaxAdv, _TaxAdv),
                    "L_Tx2000.AddManTA - 䨪 |" +
                    "  ࠢ筨.  "+FLDTOSTR(_TaxAdv));
   IF ~SEARCH(TaxInq, MyFirm, Staff.MyFirm, Staff.Man,
                           L_Tax.inqt_Auto,
                           Memo.tmp_mb()) THEN
    DEFAULT(TaxInq);
    TaxInq.InqType:=L_Tax.inqt_Auto;
    TaxInq.Man    :=Staff.Man;
    TaxInq.MyFirm :=Staff.MyFirm;
    TaxInq.SrcNum :=PassWord;
    TaxInq.Date   :=Memo.tmp_mb();
    INSERT(TaxInq);
   END;
   ADD(ManTA, TaxInq, , TaxInq.TaxInq, _TaxAdv, P,
                         ManTA.Value, s);
   ManTA.StopD:=L_DatTim.MonthEnd(P);
   REPLACE(ManTA);
  END; (* IF s # 0 ... *)
 END {AddManTA};

 PRIVATE PROCEDURE PlacePureIncSpecifyTax(P: DATE);
  VAR
   _Value    : LIKE(StaffPro.Value);
   SpecifyTax: SCAL[100];
 BEGIN
  L_Tax.Tax_CheckYear(P);
  SAVE(Staff);
  SELECT(Staff, Man, , Man.Man);
  WHILE OCCURE(Staff) DO
   _Value:=P_Staff.GetStaffProSoft(Staff.TSN,
                                   L_DatTim.MaxDATE(P, Staff.BegDate),
                                   P_Staff.P_SpecifyTax);
   IF ~!_Value THEN
    SpecifyTax:=L_Prop.GetProValueAsScal(P_Staff.GetStaffProPool(P_Staff.P_SpecifyTax), _Value);
    SAVE(PCalc);
    SAVE(TaxInq);
    SAVE(ManTA);
    L_General.Assert(SEARCH(PCalc, TSN, Staff.TSN, P),
                     " ப 楢   "+ L_DatTim.PeriodStr(P));
    IF    SpecifyTax = 0(*StaffNoTxd*) THEN
     AddManTA(P, _ta_NoTxd, PCalc.PurToTax);
    ELSIF SpecifyTax = 1(*DuringHolid*) THEN
     AddManTA(P, _ta_DurHld, PCalc.PurToTax);
    ELSE
     L_General.Error("⭮ 祭 ᢮⢠ " + FLDTOSTR(P_Staff.P_SpecifyTax) +
                     "| " + Money.TSN);
    END;
    RESTORE(PCalc);
    RESTORE(TaxInq);
    RESTORE(ManTA);
   END;
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
  RESTORE(Staff);
 END {PlacePureIncSpecifyTax};

 PUBLIC PROCEDURE FormTaxAdvByMoney(VAR NextYearInc: NUMERIC[2]);
 BEGIN
  IF Money.Sum # 0 THEN
   IF (Money.Period >= Memo.tmp_mb()) &
      (L_DatTim.YearBeg(Money.Period) = L_DatTim.YearBeg(Memo.tmp_cd())) &
      (MCList.TimeType > 0) &
      ((TaxAdv.Reason = _tr_PrInc) |
       (TaxAdv.Reason = _tr_IncWA)) THEN
    INC(FutTimes[L_DatTim.MONTH(Money.Period)-L_DatTim.MONTH(Memo.tmp_mb())],
        Money.Tim);
   END;
   IF (Money.MCode # _mc_TaxF) &
      (Money.MCode # _mc_TaxC) &
      (Money.MCode # _mc_Pns) THEN
    CONFORM(Money, MCList, TaxAdv);
    L_General.Assert(TaxAdv.Type = 0,
                     " 䨪樨  - 㬬 /.  / - " +
                     MCList.MCode);
    IF Money.Source # L_PRoll.src_FtSys THEN
     IF TaxAdv.Reason = _tr_PrInc THEN
      L_General.Assert(MCList.MCTyp = 0,
                       "   㤥ঠ 室.  "+MCList.MCode);
      INC(PCalc.PurToTax, Money.Sum);      (* storno ? *)
      IF L_DatTim.YearBeg(Money.Period) > L_DatTim.YearBeg(Memo.tmp_cd()) THEN
       INC(NextYearInc, Money.Sum);        (* storno ? *)
      END;
     ELSIF (TaxAdv.Reason = _tr_IncWA) | (TaxAdv.Reason = _tr_DecWA) THEN
      AddManTA(Memo.tmp_mb(), TaxAdv.TaxAdv, Money.Sum);
     ELSIF (TaxAdv.Reason = _tr_NoTxd) THEN
      L_General.Assert(MCList.MCTyp = 0,
                       "   㤥ঠ 室.  "+MCList.MCode);
      AddManTA(Memo.tmp_mb(), TaxAdv.TaxAdv, Money.Sum);
     ELSIF (TaxAdv.Reason = _tr_PdPns) THEN
      L_General.Assert(MCList.MCTyp = 1,
                       "   ᫥ ᨮ ᮬ.  " +
                       MCList.MCode);
      AddManTA(Memo.tmp_mb(), TaxAdv.TaxAdv, Money.Sum);
     END;
    END;
   END; (* IF Money.MCode # ... *)
  END; (* IF Money.Sum # 0 ... *)
 END {FormTaxAdvByMoney};

 PUBLIC PROCEDURE EstimateDone(): SCAL[2];
 BEGIN
  RETURN EstimDone;
 END {EstimateDone};

 PUBLIC PROCEDURE NeedEstimate(_TaxAdv: LIKE(TaxAdv.TaxAdv)): SCAL[2];
 BEGIN
  RETURN (_TaxAdv = 01) | (_TaxAdv = 02) | (_TaxAdv = 52);
 END {NeedEstimate};

 PRIVATE PROCEDURE Estimate(M: SCAL[12]; VAR _Value: LIKE(ManTA.Value));
  VAR
   i: NUMERIC[0];
 BEGIN
  L_General.Assert(~!Man_TotInc[M], "EstimateMinimums: ⮩ Man_TotInc");
  i:=0;
  IF Man_TotInc[M] > _MESBas[_MESAmo-1] THEN
   WHILE (i < _MESAmo) & (Man_TotInc[M] < _MESBas[i]) DO
    INC(i);
   END;
  ELSE
   i:=_MESAmo-1;
  END;
  L_General.Assert(i < _MESAmo, "ୠ 誠  ਬ 㬮");
  IF ~EstimDone THEN
   EstimDone:=_MESNbr[i]>1;
  END;
  _MESArr[M]:=_MESNbr[i];
  _Value:=_Value * _MESNbr[i];
 END {Estimate};

 PUBLIC PROCEDURE GetEstimateArr(VAR Arr1, Arr2: NUMERIC[2][12]);
  VAR
   i: NUMERIC[0];
 BEGIN
  SETZERO(Arr2);
  i:=0;
  WHILE i < 12 DO
   IF ~!_MESArr[i] & (_MESArr[i]=2) THEN
    Arr2[i]:=Arr1[i];
    Arr1[i]:=0;
   END;
   INC(i);
  END;
 END {GetEstimateArr};

 PUBLIC PROCEDURE GetTAValue(P: DATE; WithEstimate: SCAL[2]): LIKE(ManTA.Value);
  VAR
   _Value: LIKE(ManTA.Value);
 BEGIN
  IF ~!ManTA.Value THEN
   L_General.Assert(!TaxAdv.DefValue |
                    (TaxInq.InqType = L_Tax.inqt_Auto) |
                    InterpretValueAsSum(TaxInq.InqType),
                    "᪮  " + TaxInq.Man+
                    " 䨪  " + ManTA.TaxAdv +
                    "|   㪠 稭 䨪樨");
   _Value:=ManTA.Value;
  ELSE
   L_General.Assert(~!TaxAdv.DefValue,
                    "᪮  " + TaxInq.Man +
                    " 䨪  " + ManTA.TaxAdv +
                    "|   㪠 稭 䨪樨");
   _Value:=TaxAdv.DefValue;
  END;
  IF WithEstimate & NeedEstimate(ManTA.TaxAdv) THEN
   Estimate(L_DatTim.MONTH(P), _Value);
  END;
  RETURN _Value;
 END {GetTAValue};

 PRIVATE PROCEDURE SingleTime(P: DATE): NUMERIC[2];
  VAR
   Sum: NUMERIC[2];
 BEGIN
  IF ~InterpretValueAsSum(TaxInq.InqType) THEN
   IF    TaxAdv.Type = 0 THEN (* 㬬   *)
    Sum:=GetTAValue(P, 0);
   ELSIF TaxAdv.Type = 1 THEN (*  *)
    IF TaxInq.InqType # L_Tax.inqt_Auto THEN
     Sum:=GetTAValue(P, 1)*MinimumsArr[L_DatTim.MONTH(P)];
    ELSE
     Sum:=GetTAValue(P, 1)*MinimumsArr[L_DatTim.MONTH(TaxInq.Date)];
    END;
   ELSIF TaxAdv.Type = 2 THEN (* 誠   *)
    Sum:=ActualTaxBase[GetTAValue(P, 0)];
   ELSE
    L_General.Error("࠭ TaxAdv.Type = " + TaxAdv.Type);
   END; (* IF TaxAdv.Type ... *)
   IF (TaxAdv.TakenOft > 0(**)) & ~ManIsWorkingInYear[L_DatTim.MONTH(P)] THEN
    RETURN 0;
   ELSE
    RETURN Sum;
   END; (* IF PerDays ... *)
  ELSE
   RETURN GetTAValue(P, 0);
  END; (* IF ~InterpretValueAsSum ... *)
 END {SingleTime};

 PRIVATE PROCEDURE Limit(Sum: NUMERIC[2]): NUMERIC[2];
  VAR
   LimSum: NUMERIC[2];
 BEGIN
  IF    TaxAdv.LimType = 0 (*  *) THEN
   LimSum:=Sum;
  ELSIF TaxAdv.LimType = 1 (*  *) THEN
   LimSum:=TaxAdv.LimValue;
  ELSIF TaxAdv.LimType = 2 (*  *) THEN
   LimSum:=TaxAdv.LimValue * AvgInYearMin;
  ELSIF TaxAdv.LimType = 3 (* 誫 *) THEN
   LimSum:=ActualTaxBase[TaxAdv.LimValue];
  ELSE
   L_General.Error("࠭ TaxAdv.LimType = " + TaxAdv.LimType);
  END; (* IF TaxAdv.LimType ... *)
  IF Sum > LimSum THEN
   RETURN LimSum;
  ELSIF TaxAdv.Reason = _tr_PrInc THEN
   RETURN 0;
  ELSE
   RETURN Sum;
  END; (* IF Sum > ... *)
 END {Limit};

 PRIVATE PROCEDURE GetGate(P, F: DATE; VAR P1, P2: DATE);
  VAR
   MinD: DATE;
 BEGIN
  IF (TaxAdv.TakenOft = 0(**)) |
     InterpretValueAsSum(TaxInq.InqType) THEN
   MinD:=L_DatTim.YearBeg(L_Env.aptm_Start);
   IF ManTA.StartD < L_DatTim.YearBeg(Memo.tmp_cd()) THEN
    P1:=L_DatTim.YearBeg(Memo.tmp_cd());
   ELSE
    P1:=L_DatTim.Per(ManTA.StartD);
   END;
   IF P >= P1 THEN
    P2:=P1;
   ELSE
    P2:=P1-1;
   END;
  ELSE
   MinD:=L_Env.aptm_Start;
   IF TaxAdv.TakenOft = 2(*/  ⮬ 믫 *) THEN
    P:=F;
   END;
   IF ManTA.StartD < L_DatTim.YearBeg(Memo.tmp_cd()) THEN
    P1:=L_DatTim.YearBeg(Memo.tmp_cd());
   ELSE
    P1:=L_DatTim.Per(ManTA.StartD);
   END;
   IF !ManTA.StopD | (L_DatTim.Per(ManTA.StopD) > P) THEN
    P2:=P;
   ELSE
    P2:=L_DatTim.Per(ManTA.StopD);
   END;
  END; (* IF TaxAdv.TakenOft = 0 ... *)
  P1:=L_DatTim.MaxDATE(P1, MinD);
 END {GetGate};

 PRIVATE PROCEDURE IncArrayCell(VAR Array: NUMERIC[2][];
                                Value: NUMERIC[2];
                                i    : NUMERIC[0]);
 BEGIN
  INC(Array[L_Math.MIN(i, HIGH(Array))], Value);
 END {IncArrayCell};

 PRIVATE PROCEDURE PutInArray(VAR Array: NUMERIC[2][]; P: DATE; S: NUMERIC[2]);
 BEGIN
  IF HIGH(Array) = 0 THEN
   INC(Array[       0], S);
  ELSE
   INC(Array[L_DatTim.MONTH(P)], S);
  END;
 END {PutInArray};

 PUBLIC PROCEDURE _GetOneTaxAdv(    P,
                                    F      : DATE;
                                    _Reason: LIKE(TaxAdv.Reason);
                                    _TaxAdv: LIKE(TaxAdv.TaxAdv);
                                VAR S      : NUMERIC[2][];
                                VAR L      : NUMERIC[2];
                                    _Source: SCAL[3]): SCAL[2];
  VAR
   Sum,
   Itog   : NUMERIC[2];
   P1,
   P2     : DATE;
   Dummy  : SCAL[2];
 BEGIN
  IF _Source < 2 THEN
   Dummy:=SELECT(_MnTA, _MnTA, Man.Man, _Reason, _TaxAdv, _Source);
  ELSE
   Dummy:=SELECT(_MnTA, _MnTA, Man.Man, _Reason, _TaxAdv);
  END;
  IF Dummy THEN
   SETZERO(Itog, L);
   SEARCH(TaxAdv, TaxAdv, , _MnTA._TaxAdv);
   REPEAT
    L_Tax.Tax_AccessManTA_by_MnTA();
    SEARCH(TaxInq, TaxInq, , ManTA.TaxInq);
    GetGate(P, F, P1, P2);
    WHILE P1 <= P2 DO
     Sum:=SingleTime(P1);
     PutInArray(S, P1, Sum);
     IF TaxAdv.LimOft = 0(**) THEN
      INC(Itog, Sum);
     ELSE
      INC(L, Limit(Sum));
     END;
     P1:=L_DatTim.IncMonth(P1, +1);
    END; (* WHILE P1 <= P2 ... *)
   UNTIL ~NEXT(_MnTA);
   IF TaxAdv.LimOft = 0(**) THEN
    L:=Limit(Itog);
   END;
   RETURN 1;
  ELSE
   RETURN 0;
  END; (* IF SELECT(ManTA ... *)
 END {_GetOneTaxAdv};

 PUBLIC PROCEDURE GetOneTaxAdv(    P,
                                   F      : DATE;
                                   _Reason: LIKE(TaxAdv.Reason);
                                   _TaxAdv: LIKE(TaxAdv.TaxAdv);
                               VAR S      : NUMERIC[2][];
                               VAR L      : NUMERIC[2]): SCAL[2];
 BEGIN
  RETURN _GetOneTaxAdv(P, F, _Reason, _TaxAdv, S, L, 2);
 END {GetOneTaxAdv};

 PRIVATE PROCEDURE GetStartToTax(): DATE;
 BEGIN
  IF L_Env.aptm_Start > L_DatTim.YearBeg(Memo.tmp_cd()) THEN
   RETURN L_Env.aptm_Start;
  ELSE
   RETURN L_DatTim.YearBeg(Memo.tmp_cd());
  END;
 END {GetStartToTax};

 PUBLIC PROCEDURE GetPureIncome(P: DATE; VAR Inc: NUMERIC[2][]; PIncSel: SCAL[3]);
  VAR
   _Value: LIKE(StaffPro.Value);
 BEGIN
  L_Tax.Tax_CheckYear(P);
  L_General.CallUs((PIncSel = 0(* .*)) |
                   (PIncSel = 1(*..*)) |
                   (PIncSel = 2(*..*)));
  SAVE(Staff);
  SAVE(PCalc);
  SELECT(Staff, Man, , Man.Man);
  WHILE OCCURE(Staff) DO
   IF ( PIncSel = 0) |
      ((PIncSel = 1) & (Staff.Contract = 0)) |
      ((PIncSel = 2) & (Staff.Contract > 0)) THEN
    SELECT(PCalc, TSN, ,  Staff.TSN);
    SRCINSEL(PCalc, GE, , GetStartToTax());
    WHILE OCCURE(PCalc) & (PCalc.Period <=  P) DO
     _Value:=P_Staff.GetStaffProSoft(Staff.TSN,
                                     L_DatTim.MaxDATE(PCalc.Period, Staff.BegDate),
                                     P_Staff.P_SpecifyTax);
     IF !_Value THEN
      PutInArray(Inc, PCalc.Period, PCalc.PurToTax);
     END;
     NEXT(PCalc);
    END; (* WHILE OCCURE(PCalc) ... *)
   END;
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
  RESTORE(PCalc);
  RESTORE(Staff);
 END {GetPureIncome};

 PUBLIC PROCEDURE _GetTaxAndPens(P            : DATE;
                                 VAR _TaxArrF,
                                     _TaxArrC,
                                     _PnsArr,
                                     _PnsArrA : NUMERIC[2][]);
 BEGIN
  L_Tax.Tax_CheckYear(P);
  SAVE(Staff);
  SELECT(Staff, Man, , Man.Man);
  WHILE OCCURE(Staff) DO
   SELECT(Money,  TSN, , Staff.TSN);
   SRCINSEL(Money, GE, , GetStartToTax());
   WHILE OCCURE(Money) & (Money.Closed <= P) DO
    CHANGESEL(Money, TSN, , Money.TSN, Money.Closed, Money.WFPeriod,
                            Money.ODate, Money.Period);
    SRCINSEL(Money, GE, , _mc_TaxF);
    WHILE OCCURE(Money) & (Money.MCode = _mc_TaxF) DO
     PutInArray(_TaxArrF, Money.Closed, Money.Sum);
     NEXT(Money);
    END;
    SRCINSEL(Money, GE, , _mc_TaxC);
    WHILE OCCURE(Money) & (Money.MCode = _mc_TaxC) DO
     PutInArray(_TaxArrC, Money.Closed, Money.Sum);
     NEXT(Money);
    END;
    SRCINSEL(Money, GE, , _mc_Pns);
    WHILE OCCURE(Money) & (Money.MCode = _mc_Pns) DO
     IF (Money.Closed >= 01\01\1999) |
        ((Staff.Contract # L_Staff.cnt_ExternalCombine) &
         (Staff.Contract # L_Staff.cnt_ContractorsAgreement)) THEN
      PutInArray(_PnsArr, Money.Closed, Money.Sum);
     END;
     PutInArray(_PnsArrA, Money.Closed, Money.Sum);
     NEXT(Money);
    END;
    LAST(Money);
    CHANGESEL(Money, TSN, , Money.TSN);
    NEXT(Money);
   END; (* WHILE OCCURE(Money) ... *)
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
  RESTORE(Staff);
 END {_GetTaxAndPens};

 PUBLIC PROCEDURE GetTaxAndPens(P: DATE; VAR _TaxArr, _PnsArr, _PnsArrA: NUMERIC[2][]);
  VAR
   _TaxArrF,
   _TaxArrC : NUMERIC[2][12];
   i        : NUMERIC[0];
 BEGIN
  L_General.Assert(HIGH(_TaxArr) <= HIGH(_TaxArrF),
                   "L_Tax.GetTaxAndPens: 訡");
  SETZERO(_TaxArrF, _TaxArrC);
  _GetTaxAndPens(P, _TaxArrF, _TaxArrC, _PnsArr, _PnsArrA);
  i:=0;
  WHILE i <= HIGH(_TaxArrF) DO
   IncArrayCell(_TaxArr, _TaxArrF[i]+_TaxArrC[i], i);
   INC(i);
  END;
 END {GetTaxAndPens};

 PUBLIC PROCEDURE GetArrSum(S: NUMERIC[2][]): NUMERIC[2];
  VAR
   i: NUMERIC[0];
   T: NUMERIC[2];
 BEGIN
  SETZERO(T, i);
  WHILE i <= HIGH(S) DO
   INC(T, S[i]);
   INC(i);
  END;
  RETURN T;
 END {GetArrSum};

 PUBLIC PROCEDURE _Wo_Lim(    _Reason: LIKE(TaxAdv.Reason);
                              Tmp    : NUMERIC[2][];
                              Lim    : NUMERIC[2];
                          VAR Result : NUMERIC[2][]);
  VAR
   i: NUMERIC[0];
 BEGIN
  L_General.Assert(HIGH(Result) <= HIGH(Tmp), "L_Tax._Wo_Lim: 訡");
  i:=0;
  WHILE i <= HIGH(Tmp) DO
   IncArrayCell(Result, Tmp[i], i);
   INC(i);
  END;
 END {_Wo_Lim};

 PUBLIC PROCEDURE _W_Lim(    _Reason: LIKE(TaxAdv.Reason);
                             Tmp    : NUMERIC[2][];
                             Lim    : NUMERIC[2];
                         VAR Result : NUMERIC[2][]);
  VAR
   i: NUMERIC[0];
   T: NUMERIC[2];
 BEGIN
  L_General.Assert(HIGH(Result) <= HIGH(Tmp), "L_Tax._W_Lim: 訡");
  IF Lim = 0 THEN
   _Wo_Lim(_Reason, Tmp, Lim, Result);
  ELSIF _Reason = _tr_IncWA THEN
   T:=0; i:=0;
   WHILE i <= HIGH(Tmp) DO
    INC(T, Tmp[i]);
    IF Lim < T THEN
     IncArrayCell(Result, L_Math.MIN(T-Lim, Tmp[i]), i);
    END;
    INC(i);
   END; (* WHILE i ... *)
  ELSIF Lim >= GetArrSum(Tmp) THEN
   _Wo_Lim(_Reason, Tmp, Lim, Result);
  ELSE
   i:=0;
   WHILE i <= HIGH(Tmp) DO
    T:=L_Math.MIN(Lim, Tmp[i]);
    IncArrayCell(Result, T, i);
    DEC(Lim,  T);
    INC(i);
   END; (* WHILE i ... *)
  END; (* IF Lim = 0 ... *)
 END {_W_Lim};

 PRIVATE PROCEDURE _GetOneReason(    Proc   : PROC;
                                     P,
                                     F      : DATE;
                                     _Reason: LIKE(TaxAdv.Reason);
                                 VAR S      : NUMERIC[2][];
                                     _Source: SCAL[3]);
  VAR
   Tmp: NUMERIC[2][12];
   Lim: NUMERIC[2];
 BEGIN
  SELECT(_MnTA, _MnTA, , Man.Man, _Reason);
  WHILE OCCURE(_MnTA) DO
   CHANGESEL(_MnTA, _MnTA, , _MnTA._Man, _MnTA._MdfType, _MnTA._TaxAdv);
   SAVE(_MnTA);
   SETZERO(Tmp, Lim);
   IF _GetOneTaxAdv(P, F, _Reason, _MnTA._TaxAdv, Tmp, Lim, _Source) THEN
    Proc(_Reason, Tmp, Lim, S);
   END; (* IF Get ... *)
   RESTORE(_MnTA);
   LAST(_MnTA);
   CHANGESEL(_MnTA, _MnTA, , _MnTA._Man, _MnTA._MdfType);
   NEXT(_MnTA);
  END; (* WHILE OCCURE(_MnTA) ... *)
 END {_GetOneReason};

 PRIVATE PROCEDURE GetOneReason(    P,
                                    F      : DATE;
                                    _Reason: LIKE(TaxAdv.Reason);
                                VAR S      : NUMERIC[2][]);
 BEGIN
  _GetOneReason(PROCREF(_W_Lim), P, F, _Reason, S, 2);
 END {GetOneReason};

 PRIVATE PROCEDURE BadTaxScale(Cond: SCAL[2]);
 BEGIN
  L_General.Assert(Cond, "ࠢ쭠 誠 室 ");
 END {BadTaxScale};

 PRIVATE PROCEDURE TaxScales(_Date: DATE; ScaleType: SCAL[8]): STRING[250];
  VAR
   S: STRING[250][8];
 BEGIN
  IF    _Date >= 01\01\2000 THEN
   S[0]:="0 12;50000 20;150000 30";
   S[1]:="50000 0;15000 1;0 2";
   S[2]:="0 0";
  ELSIF _Date >= 01\01\1999 THEN
   S[0]:="0 9;30000 12;60000 17;90000 22;150000 32;300000 42";
   S[1]:="30000 0;10000 1;0 2";
   S[2]:="0 3";
  ELSIF _Date >= 01\01\1998 THEN
   S[0]:="0 12;20000 15;40000 20;60000 25;80000 30;100000 35";
   S[1]:="20000 0;5000 1;0 2";
   S[2]:="0 0";
  ELSIF _Date >= 01\01\1996 THEN
   S[0]:="0 12;12000000 20;24000000 25;36000000 30;48000000 35";
   S[1]:="0 1";
   S[2]:="0 0";
  ELSIF _Date >= 01\01\1995 THEN
   S[0]:="0 12;10000000 20;50000000 30";
   S[1]:="0 1";
   S[2]:="0 0";
  ELSIF _Date >= 01\12\1994 THEN
   S[0]:="0 12;10000000 20;50000000 30";
   S[1]:="0 0";
   S[2]:="0 0";
  ELSIF _Date >= 01\01\1994 THEN
   S[0]:="0 12;5000000 20;10000000 30";
   S[1]:="0 0";
   S[2]:="0 0";
  ELSIF _Date >= 01\12\1993 THEN
   S[0]:="0 12;3000000 20; 6000000 30";
   S[1]:="0 0";
   S[2]:="0 0";
  ELSIF _Date >= 01\01\1993 THEN
   S[0]:="0 12;1000000 20; 2000000 30";
   S[1]:="0 0";
   S[2]:="0 0";
  END;
  RETURN S[ScaleType];
 END {TaxScales};

 PRIVATE PROCEDURE TaxScal(_Date: DATE);
  VAR
   _s,
   _S   : STRING[];
   _Bas,
   _Sum : NUMERIC[2];
   _Prc,
   _Amn : NUMERIC[0];
 BEGIN
  SETZERO(_Sum, _Bas, _Prc, _TaxAmo);
  SETEMPTY(_TaxBas, _TaxPrc);
  _S:=TaxScales(_Date, 0);
  _s:=Str.Item(_S, ";", _TaxAmo);
  WHILE ~!_s DO
   _TaxBas[_TaxAmo]:=STRTOFLD(Str.Item(_s, " ", 0), NUMERIC);
   _TaxPrc[_TaxAmo]:=STRTOFLD(Str.Item(_s, " ", 1), NUMERIC);
   BadTaxScale(_TaxBas[_TaxAmo]>=_Bas);
   BadTaxScale(_TaxPrc[_TaxAmo]> _Prc);
   _TaxSum[_TaxAmo]:=_Sum+(_TaxBas[_TaxAmo]-_Bas)*_Prc/100;
   _Sum:=_TaxSum[_TaxAmo];
   _Prc:=_TaxPrc[_TaxAmo];
   _Bas:=_TaxBas[_TaxAmo];
   INC(_TaxAmo);
   _s:=Str.Item(_S, ";", _TaxAmo);
  END;
  BadTaxScale(_TaxAmo>0);

  _S:=TaxScales(_Date, 1);
  _MESAmo:=0;
  LOOP
   _s:=Str.Item(_S, ";", _MESAmo);
   IF !_s THEN
    EXIT;
   END;
   _MESBas[_MESAmo]:=STRTOFLD(Str.Item(_s, " ", 0), NUMERIC);
   _MESNbr[_MESAmo]:=STRTOFLD(Str.Item(_s, " ", 1), NUMERIC);
   INC(_MESAmo);
  END;
 END {TaxScal};

 PRIVATE PROCEDURE GetTaxScale(TT: NUMERIC[2]): NUMERIC[0];
  VAR
   i: NUMERIC[0];
 BEGIN
  i:=1;
  WHILE (i < _TaxAmo) & (_TaxBas[i] < TT) DO
   INC(i);
  END;
  RETURN i - 1;
 END {GetTaxScale};

 PUBLIC PROCEDURE SetManTotInc(Inc: NUMERIC[2][]);
  VAR
   i: NUMERIC[0];
   T: NUMERIC[2];
 BEGIN
  T:=0;
  i:=0;
  WHILE i < 12 DO
   Man_TotInc[i]:=Inc[i] + T;
   INC(T, Inc[i]);
   INC(i);
  END;
 END {SetManTotInc};

 PUBLIC PROCEDURE NeedCalcTaxF(D: DATE): SCAL[2];
 BEGIN
  RETURN (D >= 01\01\1999) & (D < 01\01\2000);
 END {NeedCalcTaxF};

 PUBLIC PROCEDURE __GetOverTax(    P,
                                   F       : DATE;
                               VAR OvrTaxF,
                                   OvrTaxC : NUMERIC[2];
                                   fictOnly: SCAL[2]);
  VAR
   Scale  : SCAL[30];
   _Inc   : NUMERIC[2][12];
   TaxF,
   TaxC,
   Inc,
   Dec,
   Pns,
   PnsA,
   ToTax  : NUMERIC[2];
 BEGIN
  L_Tax.Tax_CheckYear(P);
  SETZERO(_Inc, Inc, Dec, TaxF, TaxC, Pns);
  IF ~fictOnly THEN
   GetPureIncome(P, _Inc, 0);
  END;
  Man_OwnPInc:=GetArrSum(_Inc);

  GetOneReason(P, F, _tr_PrInc, _Inc); (* pure inc   *)
  GetOneReason(P, F, _tr_IncWA, _Inc); (* inc w/a    *)

  SetManTotInc(_Inc);

  Inc:=GetArrSum(_Inc);

  GetOneReason(P, F, _tr_Persn, Dec ); (* personal   *)
  GetOneReason(P, F, _tr_Child, Dec ); (* children   *)
  GetOneReason(P, F, _tr_DecWA, Dec ); (* expand w/a *)
  GetOneReason(P, F, _tr_PdTxF, TaxF); (* paid F tax *)
  GetOneReason(P, F, _tr_PdTxC, TaxC); (* paid C tax *)
  GetOneReason(P, F, _tr_PdPns, Pns ); (* paid pns   *)
  PnsA:=Pns;
  IF ~fictOnly THEN
   _GetTaxAndPens(P, TaxF, TaxC, Pns, PnsA);
  END;

  Man_PaidTaxF:=TaxF;
  Man_PaidTaxC:=TaxC;
  Man_Dec     :=Dec + Pns;
  Man_Pns     :=Pns;

  ToTax:=Inc - Dec - Pns;
  IF ToTax < 0 THEN
   ToTax:=0;
  END;
  Man_ToTax   :=ToTax;

  Scale:=GetTaxScale(ToTax);

  OvrTaxC:=L_Math.ROUND(_TaxSum[Scale] - TaxC +
                        _TaxPrc[Scale] / 100 * (ToTax - _TaxBas[Scale]), 1);
  IF NeedCalcTaxF(P) THEN
   OvrTaxF:=L_Math.ROUND(ToTax * 0.03 - TaxF, 1);
  ELSE
   OvrTaxF:=0;
   OvrTaxC:=OvrTaxC - L_Math.ROUND(TaxF, 1);
  END;
 END {__GetOverTax};

 PUBLIC PROCEDURE _GetOverTax(    P,
                                  F       : DATE;
                              VAR OvrTaxF,
                                  OvrTaxC : NUMERIC[2]);
 BEGIN
  __GetOverTax(P, F, OvrTaxF, OvrTaxC, 0);
 END {_GetOverTax};

 PUBLIC PROCEDURE GetOverTax(P, F: DATE): NUMERIC[2];
  VAR
   OvrTaxF,
   OvrTaxC : NUMERIC[2];
 BEGIN
  SETZERO(OvrTaxF, OvrTaxC);
  _GetOverTax(P, F, OvrTaxF, OvrTaxC);
  RETURN OvrTaxF+OvrTaxC;
 END {GetOverTax};

 PRIVATE PROCEDURE _PlaceOverTax(P      : DATE;
                                 NYOT   : SCAL[2];
                                 Tax    : NUMERIC[2];
                                 _TaxAdv: LIKE(TaxAdv.TaxAdv));
 BEGIN
  IF Tax < 0 THEN
   IF  NYOT THEN
    AddManTA(L_DatTim.IncMonth(P, +1), _TaxAdv, -Tax);
   ELSE
    AddManTA(L_DatTim.IncMonth(P, +1),
             _TaxAdv,
             L_General.DenominateNM(P, -Tax));
   END;
  END;
 END {_PlaceOverTax};

 PRIVATE PROCEDURE PlaceOverTax(P: DATE; NYOT: SCAL[2]; VAR TaxF, TaxC: NUMERIC[2]);
  VAR
   Tax,
   OTC,
   OTF : NUMERIC[2];
 BEGIN
  Tax:=TaxF + TaxC;
  IF Tax < 0 THEN
   SELECT(TaxInq, MyFirm, , Staff.Man, Staff.Man, L_Tax.inqt_Own);
   SRCINSEL(TaxInq, GE, , P);
   WHILE OCCURE(TaxInq) &
         ~SELECT(ManTA, TaxInq, TaxInq.TaxInq, _ta_RetOvr) DO
    NEXT(TaxInq);
   END; (* WHILE OCCURE(TaxInq) ... *)

   IF ~OCCURE(ManTA) | NYOT THEN    
    OTF:=L_Math.MIN(0, L_Math.MAX(TaxF, Tax));
    OTC:=L_Math.MIN(0, L_Math.MAX(TaxC, Tax));
    IF L_DatTim.MONTH(P) = 11 THEN
     _PlaceOverTax(P, NYOT, OTF, _ta_OvrTxF);
     _PlaceOverTax(P, NYOT, OTC, _ta_OvrTxC);
    END;
    DEC(TaxF, OTF);
    DEC(TaxC, OTC);
   END; (* IF ~OCCURE(ManTA) ... *)
  END;
 END {PlaceOverTax};

 PRIVATE PROCEDURE MoveEM_wf(P, F: DATE);
  VAR
   _Value: LIKE(ManTA.Value);
 BEGIN
  SELECT(TaxInq, MyFirm, , Staff.MyFirm, Staff.Man);
  WHILE OCCURE(TaxInq) DO
   IF (TaxInq.InqType # L_Tax.inqt_Auto) &
      (TaxInq.InqType # L_Tax.inqt_Fict) THEN
    SELECT(ManTA, TaxInq, , TaxInq.TaxInq);
    WHILE OCCURE(ManTA) DO
     IF L_DatTim.IsActiveInIntvl_Inc(F, L_DatTim.MonthEnd(F),
                                     ManTA.StartD, ManTA.StopD) THEN
      CONFORM(ManTA, TaxAdv);
      IF TaxAdv.TakenOft = 2(*/  ⮬ 믫 *) THEN
       SAVE(TaxInq);
       SAVE(ManTA);
       _Value:=GetTAValue(P, 0);
       AddManTA(F, TaxAdv.TaxAdv, -_Value);
       AddManTA(P, TaxAdv.TaxAdv,  _Value);
       RESTORE(ManTA);
       RESTORE(TaxInq);
      END; (* IF IF TaxAdv.TakenOft ... *)
     END; (* IF IsActiveInIntvl_Inc ... *)
     NEXT(ManTA);
    END; (* WHILE OCCURE(ManTA) ... *)
   END; (* IF TaxInq.InqType ... *)
   NEXT(TaxInq);
  END; (* WHILE OCCURE(TaxInq) ... *)
 END {MoveEM_wf};

 PRIVATE PROCEDURE ApproachFuture(P: DATE);
  VAR
   i: NUMERIC[0];
 BEGIN
  IF ~L_Staff.IsLeave(P) THEN
   i:=1;
   WHILE (i <= HIGH(FutTimes)) & (FutTimes[i] > 0) DO
    MoveEM_wf(P, L_DatTim.IncMonth(P, i));
    INC(i);
   END; (* WHILE i < ... *)
  END;
 END {ApproachFuture};

 PRIVATE PROCEDURE CompensateNYInc();
  VAR
   S: NUMERIC[2];
 BEGIN
  IF Memo.tmp_mb() = 01\12\2000 THEN
   S:=0;
   SELECT(TaxInq, MyFirm, , Staff.MyFirm, Staff.Man, L_Tax.inqt_Auto);
   SRCINSEL(TaxInq, GE, , L_DatTim.YearBeg(Memo.tmp_cd()));
   WHILE OCCURE(TaxInq) DO
    CONFORM(TaxInq, ManTA);
    WHILE OCCURE(ManTA) DO
     IF (ManTA.TaxAdv = 30) & (ManTA.StartD = 01\01\2001) THEN
      INC(S, ManTA.Value);
     END;
     NEXT(ManTA);
    END; (* WHILE OCCURE(ManTA) ... *)
    NEXT(TaxInq);
   END; (* WHILE OCCURE(TaxInq) ... *)
   IF S > 0 THEN
    AddManTA(01\01\2001, 30, -S);
    AddManTA(01\12\2000, 30,  S);
   END;
  END;
 END {CompensateNYInc};

 PUBLIC PROCEDURE Calc_Tax(P: DATE; VAR ToPayThisYearF, ToPayThisYearC: NUMERIC[2]);
 BEGIN
  SETZERO(ToPayThisYearF, ToPayThisYearC);
  IF L_Staff.IsMainForTax(L_DatTim.MonthEnd(P)) THEN
   L_General.Assert(Man.Man = Staff.Man, "???????????");
   PlacePureIncSpecifyTax(P);
   ApproachFuture(P);
   CompensateNYInc();
   NTax_InitValues(P);
   TaxScal(Memo.tmp_mb());
   _GetOverTax(P, P, ToPayThisYearF, ToPayThisYearC);
   PlaceOverTax(P, 0, ToPayThisYearF, ToPayThisYearC);
  END; (* IF IsMainForTax( ... *)
 END {Calc_Tax};

BEGIN
 SETZERO(InitIsMade, EstimDone, FutTimes);
 SETEMPTY(_CurDate, _MESArr);
 PassWord:=" ⥬";
END.
