(*
%Copyright   (c) Polak IT
%Author      11.01.2005 10:11:06 POLAK\LDA
%Purpose     ⥪ WIA (Windows Image Acquisition)
%Type        library
%Specific    AP
%Modified    13.07.2012 16:22:25 ROBOT by POLAK\LDA
%Environment
*)

 MODULE L_WIA [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General;

 PUBLIC TYPE
  tDeviceType = SCAL[4];

 PUBLIC CONST
  cmd_TakePicture = "{AF933CAC-ACAD-11D2-A093-00C04F72DC3C}";
  cmd_Synchronize = "{9B26B7B2-ACAD-11D2-A093-00C04F72DC3C}";

  if_JPEG   = "{B96B3CAE-0728-11D3-9D7B-0000F81EF32E}";
  if_Folder = "{81A566E7-8620-4FBA-BC8E-B27C17AD9EFD}";

  dt_Unspecified = 0;
  dt_Scanner     = 1;
  dt_Camera      = 2;
  dt_StreamVideo = 3;

  dt_Pool = "⭮ ன⢮|||⮪ ";

 PUBLIC PROCEDURE AcceptableWindowsVersion(): SCAL[2];
  VAR
   platform    : L_General.tOSPlatform;
   major,
   minor,
   IEV_Major,
   IEV_Minor,
   IEV_Build,
   IEV_SubBuild: NUMERIC[0];
 BEGIN
  OSVERSION(platform, major, minor);
  IF    (platform < L_General.osp_WinNT) |
        (major < 5) |
        (major = 5) & (minor < 1) THEN
   RETURN 0;
  END;
  IEVERSION(IEV_Major, IEV_Minor, IEV_Build, IEV_SubBuild);
  RETURN (IEV_Major > 6) |
         (IEV_Major = 6) & (IEV_Minor > 0) |
         (IEV_Major = 6) & (IEV_Minor = 0) & (IEV_Build > 2800) |
         (IEV_Major = 6) & (IEV_Minor = 0) & (IEV_Build = 2800) & (IEV_SubBuild >= 1106);
 END {AcceptableWindowsVersion};

 PUBLIC PROCEDURE WindowsVersionSupportedPhotography(): SCAL[2];
  VAR
   platform: L_General.tOSPlatform;
   major,
   minor,
   prType  : NUMERIC[0];
 BEGIN
  OSVERSION(platform, major, minor, , , prType);
  RETURN (platform = L_General.osp_WinNT) &
         (major    < 6        );
 END {WindowsVersionSupportedPhotography};

 PUBLIC PROCEDURE IsWIAInstalled(Strong: SCAL[2]): SCAL[2];
  VAR
   DeviceManager: OBJECT;
 BEGIN
  TRY [EXCEPTIONHANDLING(EXCEPTIONHANDLING_OFF)]
   DeviceManager:=CreateOBJECT("Wia.DeviceManager");
   RETURN 1;
  EXCEPT
  ON Exc: Exc_RunTime DO
   IF Exc.ErrorCode = 140 THEN
    L_General.Assert(~Strong,
                     Exc.MsgOriginalText +"| |" + "  ");
    RETURN 0;
   ELSE
    RAISE();
   END;
  ELSE
   RAISE();
  END; (* TRY *)
 END {IsWIAInstalled};

 PUBLIC PROCEDURE GetDevicesDescription(DeviceType  : tDeviceType;
                                        VAR svd_Ids : STRING[];
                                        VAR svd_List: STRING[]);
  VAR
   i            : NUMERIC[0];
   DeviceManager: OBJECT;
 BEGIN
  SETEMPTY(svd_Ids, svd_List);
  IF IsWIAInstalled(1) THEN
   i:=1;
   DeviceManager:=CreateOBJECT("Wia.DeviceManager");
   WHILE i <= DeviceManager.DeviceInfos.Count DO
    IF !DeviceType |
       (DeviceManager.DeviceInfos.Item(i).Type() = DeviceType) THEN
     svd_Ids :=svd_Ids  + DeviceManager.DeviceInfos(i).DeviceID() + "|";
     svd_List:=svd_List + DeviceManager.DeviceInfos(i).Properties("Name").Value + "|";
    END;
    INC(i);
   END;
  END; (* IF IsWIAInstalled...*)
 END {GetDevicesDescription};

 PUBLIC PROCEDURE ConnectDevice(Id: STRING[]): OBJECT;
  VAR
   Dev,
   DeviceManager: OBJECT;
   i: NUMERIC[0];
 BEGIN
  SETEMPTY(Dev);
  DeviceManager:=CreateOBJECT("Wia.DeviceManager");
  i:=1;
  WHILE i <= DeviceManager.DeviceInfos.Count DO
   IF DeviceManager.DeviceInfos.Item(i).DeviceID = Id THEN
    Dev:=DeviceManager.DeviceInfos.Item(i).Connect();
    RETURN Dev;
   END;
   INC(i);
  END;
  RETURN Dev;
 END {ConnectDevice};

END.
