(*
%Copyright   (c) Polak IT
%Author      05.10.2004 03:18:34 POLAK\Mog
%Purpose     প XML
%Type        library
%Specific    SYS
%Modified    26.12.2013 09:41:15 POLAK\GAV
%Environment
*)

 MODULE L_XML [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 CONST
  wDebug = 0;

 PUBLIC TYPE
  tDocument   = OBJECT;
  tNode       = OBJECT;
  tDictionary = OBJECT;

 IMPORT FIO, L_General, L_Str, Str;

 PUBLIC VAR
  NeedConvertStringVal: SCAL[2];

 PRIVATE VAR
  NeedConvertRestricted: SCAL[2];

 PUBLIC PROCEDURE DoConvertRestrictedChars();
 BEGIN
  NeedConvertRestricted:=1;
 END {DoConvertRestrictedChars};

 PRIVATE PROCEDURE ConvertRestricted(VAR S: STRING[]; Encode: SCAL[2]);
  VAR
   chr: SCAL[256];
 BEGIN
  IF Encode THEN
   FOR_EACH chr IN [001, 002, 003, 004, 005, 006, 007, 008, 011, 012, 014, 016, 017, 018,
                    019, 022, 023, 024, 025, 026, 027, 028, 029, 030, 031, 127, 249, 251] DO
    IF Str.Pos(S, CHR(chr)) < Str.Length(S) THEN
     L_General.Assert(NeedConvertRestricted, ' ⨬  XML ᨬ "' + CHR(chr) + '"');
     S:=L_Str.Replace(S, CHR(chr), "&CHR["+chr+"]");
    END;
   END;
  ELSIF NeedConvertRestricted THEN
   FOR_EACH chr IN [001, 002, 003, 004, 005, 006, 007, 008, 011, 012, 014, 016, 017, 018,
                   019, 022, 023, 024, 025, 026, 027, 028, 029, 030, 031, 127, 249, 251] DO
    S:=L_Str.Replace(S, "&CHR["+chr+"]", CHR(chr));
   END;
  END; (* IF Encode *)
 END {ConvertRestricted};

 PRIVATE PROCEDURE ConvertString(S: STRING[]; Encode: SCAL[2]): STRING[];
  CONST
   forbiddenChar = "&|<|>|'|" + '"|' + L_Str.CR;
   Substitutes   = "&amp;|&lt;|&gt;|&apos;|&quot;|&CR";
  VAR
   i: NUMERIC[0];
   Char,
   SrcChars,
   DstChars: STRING[];
 BEGIN
  IF Encode THEN
   ConvertRestricted(S, Encode);
  END;
  IF NeedConvertStringVal THEN
   IF ~!S THEN
    IF Encode THEN
     SrcChars:=forbiddenChar;
     DstChars:=Substitutes;
    ELSE
     SrcChars:=Substitutes;
     DstChars:=forbiddenChar;
    END;
    i:=0;
    LOOP
     Char:=L_Str.PoolItem(SrcChars, i);
     IF !Char THEN
      EXIT;
     END;
     S:=L_Str.Replace(S, Char, L_Str.PoolItem(DstChars, i));
     INC(i);
    END; (* LOOP *)
   END;
  END;
  IF ~Encode THEN
   ConvertRestricted(S, Encode);
  END;
  RETURN S;
 END {ConvertString};

 PRIVATE PROCEDURE IsElementNode(Node: OBJECT): SCAL[2];
 BEGIN
  RETURN Node.nodeTypeString = "element";
 END {IsElementNode};

 PUBLIC PROCEDURE ICN(Prefix, Name: STRING[]): STRING[];
 BEGIN
  IF ~!Prefix THEN
   RETURN Prefix + "." + Name;
  ELSE
   RETURN Name;
  END;
 END {ICN};

 PUBLIC PROCEDURE AssembleIdexedIN(Name: STRING[]; Idx: NUMERIC[0]): STRING[];
 BEGIN
  RETURN Name + "[" + L_Str.IntStr(Idx) + "]";
 END {AssembleIdexedIN};

 PRIVATE PROCEDURE GetItemAmountName(ItemName: STRING[]): STRING[];
 BEGIN
  RETURN ICN(ItemName, "Amount");
 END {GetAmountKey};

 PRIVATE PROCEDURE AddToDct(Dct: OBJECT; Key, Value: STRING[]);
 BEGIN
  Dct.Add(Key, Value);
  IF wDebug THEN
   FIO.WrLn(r);
   FIO.WrStr(r, " ⫠. " + Key + " = " + Dct.Item(Key));
  END;
 END {AddToDct};

 PRIVATE PROCEDURE SupportItemAmount(Dct: OBJECT;
                                     curItemName: STRING[]): STRING[];
  VAR
   Result,
   ian     : STRING[];
   _Amount : NUMERIC[0];
 BEGIN
  ian:=GetItemAmountName(curItemName);
  IF Dct.Exists(ian) THEN
   _Amount:=STRTOFLD(Dct.Item(ian), NUMERIC);
   Dct.Remove(ian);
  ELSE
   _Amount:=0;
  END;
  Result:=AssembleIdexedIN(curItemName, _Amount);
  AddToDct(Dct, ian, L_Str.IntStr(_Amount+1));
  RETURN Result;
 END {SupportItemAmount};

 PRIVATE PROCEDURE Parse(Dct     : OBJECT;
                         NodeList: OBJECT;
                         Prefix  : STRING[]);
  VAR
   Node,
   attributes,
   curItem    : OBJECT;
   curItemName: STRING[];
   n, a       : NUMERIC[0];
 BEGIN
  n:=0;
  WHILE n < NodeList.length DO
   Node:=NodeList.Item(n);
   IF (Node.childNodes.length > 0) | IsElementNode(Node) THEN
    curItemName:=ICN(Prefix, Node.baseName);
   ELSE
    curItemName:=Prefix;
   END;
   IF (Node.parentNode.childNodes.length > 1) | IsElementNode(Node) THEN
    curItemName:=SupportItemAmount(Dct, curItemName);
   END;

   IF Node.childNodes.length > 0 THEN
    Parse(Dct, Node.childNodes, curItemName);
   ELSE
    AddToDct(Dct, curItemName, Node.nodeTypedValue);
   END; (* IF IF Node.childNodes.length ... *)

   attributes:=Node.attributes;
   IF ~!attributes THEN
    a:=0;
    WHILE a < attributes.length DO
     curItem:=attributes.Item(a);
     AddToDct(Dct, ICN(curItemName, curItem.baseName), curItem.nodeTypedValue);
     INC(a);
    END;
   END; (* IF ~!Node.attributes ... *)
   INC(n);
  END; (* WHILE n ... *)
 END {Parse};

 PRIVATE PROCEDURE PutInDictionary(Doc: tDocument): tDictionary;
  VAR
   Result: tDictionary;
 BEGIN
  Result:=L_General.CreateDictionary();
  Parse(Result, Doc.documentElement.childNodes, Doc.documentElement.baseName);
  RETURN Result;
 END {PutInDictionary};

 PUBLIC PROCEDURE CreateXMLOBJECT(): OBJECT;
  VAR
   XMLVers: STRING[][3];
   xml    : OBJECT;
   i      : NUMERIC[0];
 BEGIN
  XMLVers[0]:="MSXML2.DOMDocument.4.0";
  XMLVers[1]:="MSXML2.DOMDocument";
  XMLVers[2]:="MSXML.DOMDocument";

  SETEMPTY(xml);
  i:=0;
  WHILE (i <= HIGH(XMLVers)) & !xml DO
   TRY [EXCEPTIONHANDLING(EXCEPTIONHANDLING_OFF)]
    xml:=CreateOBJECT(XMLVers[i]);
   EXCEPT
   ON Exc: Exc_Root DO
   END; (* TRY *)
   INC(i);
  END; (* WHILE (i <= HIGH(XMLVers)) & !xml...*)
  RETURN xml;
 END {CreateXMLOBJECT};

 PUBLIC PROCEDURE Load(fullFileName: STRING[]): tDictionary;
  VAR
   result: tDictionary;
   xml: OBJECT;
 BEGIN
  NeedConvertStringVal:=1;
  NeedConvertRestricted:=0;
  xml:=CreateXMLOBJECT();
  L_General.Assert(xml.Load(fullFileName) # 0, " 㤠 㧨 䠩" + "|" + fullFileName);
  IF wDebug THEN
   FIO.Open(r, "xml", "txt", Create);
   result:=PutInDictionary(xml);
   FIO.Close(r);
   RETURN result;
  ELSE
   RETURN PutInDictionary(xml);
  END;
 END {Load};

 PUBLIC PROCEDURE LoadXML(XML: STRING[]): tDictionary;
  VAR
   xml: OBJECT;
 BEGIN
  NeedConvertStringVal:=1;
  NeedConvertRestricted:=0;
  xml:=CreateXMLOBJECT();
  L_General.Assert(xml.LoadXML(XML) # 0, " 㤠 ஢ ப XML");
  RETURN PutInDictionary(xml);
 END {LoadXML};

 PUBLIC PROCEDURE dct_getStr(dct: OBJECT; Key: STRING[]): STRING[];
 BEGIN
  IF dct.Exists(Key) THEN
   RETURN ConvertString(dct.Item(Key), 0);
  ELSE
   L_General.Error("L_XML.dct_getStr.Error: Key=" + Key);
  END;
 END {dct_getStr};

 PUBLIC PROCEDURE dct_getNum(dct: OBJECT; Key: STRING[]): FLOAT;
 BEGIN
  RETURN STRTOFLD(dct_getStr(dct, Key), FLOAT);
 END {dct_getNum};

 PUBLIC PROCEDURE dct_getDat(dct: OBJECT; Key: STRING[]): DATE;
 BEGIN
  RETURN STRTOFLD(dct_getStr(dct, Key), DATE);
 END {dct_getDat};

 PUBLIC PROCEDURE dct_getTim(dct: OBJECT; Key: STRING[]): TIME;
 BEGIN
  RETURN STRTOFLD(dct_getStr(dct, Key), TIME);
 END {dct_getTim};

 PUBLIC PROCEDURE DebuggingPrint(Dct: tDictionary; outputFileId: STRING[]);
  VAR
   Keys: STRING[][8192];
   i   : NUMERIC[0];
 BEGIN
  Keys:=Dct.Keys();
  i:=0;
  WHILE i < Dct.Count DO
   FIO.WrStr(^outputFileId, Keys[i] + "=" + dct_getStr(Dct, Keys[i]));
   FIO.WrLn(^outputFileId);
   INC(i);
  END;
 END {DebuggingPrint};

 PUBLIC PROCEDURE bd_Create_Ext(RootNode, Declaration, Target, ExtDeclaration: STRING[]): tNode;
  VAR
   xml        : tDocument;
   emptyObject: tNode;
 BEGIN
  NeedConvertStringVal:=1;
  NeedConvertRestricted:=0;
  xml:=CreateXMLOBJECT();
  SETEMPTY(emptyObject);
  xml.insertBefore(xml.createProcessingInstruction("xml",
                                                   Declaration),
                                                   emptyObject);
  IF ~!Target THEN
   xml.insertBefore(xml.createProcessingInstruction(Target,
                                                    ExtDeclaration),
                                                    emptyObject);
  END;
  RETURN xml.appendChild(xml.createElement(RootNode));
 END {bd_Create_Ext};

 PUBLIC PROCEDURE bd_Create_wExtDeclaration(RootNode, Target, ExtDeclaration: STRING[]): tNode;
 BEGIN
  RETURN bd_Create_Ext(RootNode, "version='1.0' encoding='windows-1251'", Target, ExtDeclaration);
 END {bd_Create_wExtDeclaration};

 PUBLIC PROCEDURE bd_Create(RootNode: STRING[]): tNode;
 BEGIN
  RETURN bd_Create_wExtDeclaration(RootNode, "", "");
 END {bd_Create};

 PUBLIC PROCEDURE bd_SetAttribute(Node: tNode;
                           AttrName, AttrValue: STRING[]);
 BEGIN
  Node.setAttribute(AttrName, AttrValue);
 END {bd_SetAttribute};

 PUBLIC PROCEDURE bd_GetDocument(Node: tNode): tDocument;
 BEGIN
  RETURN Node.ownerDocument;
 END {bd_GetDocument};

 PUBLIC PROCEDURE bd_PutNode(Parent: tNode; Name, Value: STRING[]): tNode;
  VAR
   blin   : OBJECT;
   tmpNode: tNode;
 BEGIN
(*
  tmpNode:=bd_GetDocument(Parent).createElement(Name);
*)
  blin:=bd_GetDocument(Parent);
  tmpNode:=blin.createElement(Name);
  tmpNode.nodeTypedValue:=ConvertString(Value, 1);
(*
  tmpNode.nodeTypedValue:=Value;
*)
  RETURN Parent.appendChild(tmpNode);
 END {bd_PutNode};

 PUBLIC PROCEDURE bd_PutLeafNode(Parent: tNode; Name, Value: STRING[]);
  VAR
   Dummy: tNode;
 BEGIN
  Dummy:=bd_PutNode(Parent, Name, Value);
 END {bd_PutLeafNode};

 PUBLIC PROCEDURE GetAsStr(Doc: tDocument): STRING[];
 BEGIN
  RETURN Doc.xml;
 END {GetAsStr};

 PUBLIC PROCEDURE bd_Save(Doc: tDocument; FileName: STRING[]);
 BEGIN
  Doc.save(FileName);
 END {bd_Save};

END.
