(*
%Copyright   (c) Polak IT
%Author      06.12.2007 12:35:08 POLAK\GAV
%Purpose     稪
%Type        control,report
%Specific    SYS
%Modified    11.06.2014 19:10:08 POLAK\GAV
%Environment
*)

 MODULE Meter [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Events, L_General, L_Meter, L_XDoc, L_OwnData, L_DatTim,
        L_Str, L_Basis, Memo, FIO, L_StdFrm, L_APEMl, L_EMail, Str,
        E_Utils, Math, L_bp;

 PRIVATE CONST
  MtRoP_RepName = "MtRoP";
  Mtr01_RepName = "Mtr01";

 PRIVATE PERMANENT VAR
  v_Meter: LIKE(Meter.Meter);

 PRIVATE VAR
  Mtr01_LostCnt: NUMERIC[0];

 PRIVATE PROCEDURE SetMtrTypeFormat(Zone: NUMERIC[0]; Name: STRING[]);
 BEGIN
  RTI.Get().CurWindow.Zones(Zone).Fields(Name).Format:=L_Meter.mt_Pool;
 END {SetMtrTypeFormat};

 PRIVATE PROCEDURE Meter_1_OnZoneOpen();
 BEGIN
  SetMtrTypeFormat(1, "Meter.MtrType");
 END {Meter_1_OnZoneOpen};

 PRIVATE PROCEDURE Meter_MtrType_OnSuccessfulChange();
  VAR
   Format: STRING[];
 BEGIN
  IF Meter.MtrType = L_Meter.mt_Electric THEN
   Format:=L_Meter.mm_Electric_Pool;
  ELSE
   Format:="";
  END;
  RTI.Get().CurZone.Fields("Meter.MtrModel").Format:=Format;
 END {Meter_MtrType_OnSuccessfulChange};

 PRIVATE PROCEDURE Meter_OnReposition();
 BEGIN
  IF ACCESS(Meter) THEN
   SetMtrTypeFormat(GE_CURZONE(), "Meter.MtrType");
   Meter_MtrType_OnSuccessfulChange();
  END;
 END {Meter_OnReposition};

 PRIVATE PROCEDURE MeterLog_Name_OnSuccessfulChange();
 BEGIN
  IF !MeterLog.DateE THEN
   Meter.asName:=MeterLog.Name;
   REPLACE(Meter);
  END;
 END {MeterLog_Name_OnSuccessfulChange};

 PRIVATE PROCEDURE MeterLog_Contract_OnSuccessfulChange();
 BEGIN
  IF ~!MeterLog.Contract THEN
   CONFORM(MeterLog, Contract);
   L_General.Assert(Contract.Leaf, "ᯮ짮 蠡 ஢ ⨬");
  END;
 END {MeterLog_Contract_OnSuccessfulChange};

 PRIVATE PROCEDURE MtrRgKnd_1_OnZoneOpen();
 BEGIN
  SetMtrTypeFormat(1, "MtrRgKnd.MtrType");
 END {MtrRgKnd_1_OnZoneOpen};

 PRIVATE PROCEDURE MtrRdout_1_OnZoneOpen();
 BEGIN
  SetMtrTypeFormat(1, "Meter.MtrType");
 END {MtrRdout_1_OnZoneOpen};

 PRIVATE PROCEDURE MtrRdoE_1_OnZoneOpen();
 BEGIN
  SetMtrTypeFormat(0, "Meter.MtrType");
 END {MtrRdoE_1_OnZoneOpen};

(*
$P{ MtRoP_Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
MtRoP_RepName, L, ""
@
####################################################################### ########## #####
=#####==================================================================================
|                         ⮪   稪                          |
}
*)

(*
$P{ MtRoP_MeterTop:
Meter.Meter, L, ""
Meter.MtrType, L, L_Meter.mt_Pool
Meter.asName, L, ""
?, L, "D.N.y"
?, L, "H:M"
?, L, ""
?, L, "H:M"
?, L, ""
@
|======================================================================================|
|####### #### ############################################ ########## #####:##-#####:##|
|--------------------------------------------------------------------------------------|
|                                   ਮ                           -|
|--------------------------------------------------------------------------------------|
}
*)

(*
$P{ MtRoP_Line:
MtrRgKnd.Sign, L, ""
MtrRgKnd.Name, L, ""
?, L, ""
?, R, ""
@
|##### ######################################## ################################# #####|
}
*)

(*
$P{ MtRoP_BadLine:
Meter.Meter, L, ""
Meter.MtrType, L, L_Meter.mt_Pool
Meter.asName, L, ""
?, L, "D.N.y"
?, L, "H:M"
?, L, ""
?, L, ""
@
|======================================================================================|
|####### #### ############################################ ########## #####:##         |
|######################################################################################|
}
*)

(*
$P{ MtRoP_OKLine:
@
|--------------------------------------------------------------------------------------|
|                                                                                      |
|                              ⠭ ᯥ譮                             |
|                                                                                      |
}
*)

(*
$P{ MtRoP_Bot:
@
=(c) Polak==============================================================================
}
*)

 PRIVATE PROCEDURE getVal(_Meter: LIKE(MtrRg.Meter); _MtrRgKnd: LIKE(MtrRg.MtrRgKnd); _DTS: TIMESTAMP): LIKE(MtrRg.Value);
 BEGIN
  IF ~SELECT(MtrRg, Meter, _Meter, _MtrRgKnd, _DTS) THEN
   SEARCH(MtrRgKnd, MtrRgKnd, , _MtrRgKnd);
   L_General.Error(L_DatTim.TimestampStr(_DTS) + "  祭 " + MtrRgKnd.Sign);
  ELSE
   RETURN MtrRg.Value;
  END;
 END {getVal};

 PRIVATE PROCEDURE ReadFromDevice(VAR Trouble: STRING[]);
  CONST
   MaxAllowedTimeDelta = 5/60/24;
  VAR
   _MtrRdout : LIKE(MtrRdout.MtrRdout);
   RdCnt     : NUMERIC[0];
   DTStr     : STRING[];
   D         : DATE;
   _Energy,
   whmSum    : NUMERIC[4];
   whmSum5   : NUMERIC[5];
   IsHld,
   PortIsOpen: SCAL[2];
 BEGIN
  SETEMPTY(Trouble);
  HISTCONF(SYSDATE(), Meter, MeterLog);
  IF ~OCCURE(MeterLog) THEN
   Trouble:="  ᪠ ଠ  稪|";
  END;

  TRY
   _MtrRdout:=L_Meter.StartMtrRdout(Meter.Meter, PortIsOpen);
   L_XDoc.XDocJrn_Start(L_XDoc.xdt_MtrRdout, L_XDoc.EmptyXDocKind, _MtrRdout, L_XDoc.action_AutoBuild);
   IF ~MeterLog.TurnedOn THEN
    Trouble:=Trouble + "ᯥ譮 祭   몫祭 稪|";
   END;
   IF Math.Abs(MtrRdout.TimDelta) > MaxAllowedTimeDelta THEN
    Trouble:=Trouble + "⪫ ६ 稪  5  (" +
             Str.Condense(FLDTOSTR(MtrRdout.TimDelta*24*60, "2.")) + " .)|";
   ELSE
    IF Math.Abs(MtrRdout.TimDelta) > MaxAllowedTimeDelta * 0.7 THEN
     Trouble:=Trouble + "⪫ ६ 稪 ਡ  ᪮|";
    END;
    L_Meter.FillMtrRdout();
   END;
   L_Meter.FinishMtrRdout();
   L_XDoc.XDocJrn_Stop(L_XDoc.xdt_MtrRdout, L_XDoc.EmptyXDocKind, _MtrRdout);
  EXCEPT
  ELSE
   IF PortIsOpen THEN
    L_Meter.AbortMtrRdout();
   END;
   RAISE();
  END; (* TRY *)
  PROMPT(" ⮪. 稪 " + Meter.Meter);
  MAKEPART(R, MtRoP_MeterTop, XDocJrn.Date,
                              XDocJrn.BegTime,
                              L_Str.FixedIntStr(XDocJrn.BegSec, 2),
                              XDocJrn.EndTime,
                              L_Str.FixedIntStr(XDocJrn.EndSec, 2));

  SELECT(MtrRg, RgKnd, , MtrRdout.MtrRdout);
  WHILE OCCURE(MtrRg) DO
   CONFORM(MtrRg, MtrRgKnd);
   RdCnt:=0;
   DTStr:=L_DatTim.TimestampStr(MtrRg.DTS);
   NARROWSEL(MtrRg, MtrRgKnd);
   WHILE OCCURE(MtrRg) DO
    IF OCCURE(MeterLog) THEN
     IF    MtrRg.MtrRgKnd = L_Meter.mrk_WattageHalfhourMax THEN
      IF (MtrRg.Value < MeterLog.MinWHM) | (MtrRg.Value > MeterLog.MaxWHM) THEN
       Trouble:=Trouble + L_DatTim.TimestampStr(MtrRg.DTS) +
                " ଠ쭮 祭  (" + FLDTOSTR(MtrRg.Value, "4.", 0) + ")|";
      END;
     ELSIF (MtrRg.MtrRgKnd = L_Meter.mrk_Energy1) & (L_DatTim.GetTimeOfTIMESTAMP(MtrRg.DTS) = 00:00) THEN
      SAVE(MeterLog);
      D:=L_DatTim.GetDateOfTIMESTAMP(MtrRg.DTS) - 1;
      HISTCONF(D, Meter, MeterLog);
      IF OCCURE(MeterLog) & MeterLog.TurnedOn THEN
       IsHld:=~L_DatTim.IsWorkDay(D);
       SAVE(MtrRg);
       TRY
        _Energy:=getVal(Meter.Meter, L_Meter.mrk_Energy1, PackTIMESTAMP(D + 1, 00:00)) -
                 getVal(Meter.Meter, L_Meter.mrk_Energy1, PackTIMESTAMP(D    , 00:00)) +
                 getVal(Meter.Meter, L_Meter.mrk_Energy2, PackTIMESTAMP(D + 1, 00:00)) -
                 getVal(Meter.Meter, L_Meter.mrk_Energy2, PackTIMESTAMP(D    , 00:00)) +
                 getVal(Meter.Meter, L_Meter.mrk_Energy3, PackTIMESTAMP(D + 1, 00:00)) -
                 getVal(Meter.Meter, L_Meter.mrk_Energy3, PackTIMESTAMP(D    , 00:00)) +
                 getVal(Meter.Meter, L_Meter.mrk_Energy4, PackTIMESTAMP(D + 1, 00:00)) -
                 getVal(Meter.Meter, L_Meter.mrk_Energy4, PackTIMESTAMP(D    , 00:00));
        IF (_Energy < MeterLog.MinEnrgy[IsHld]) | (_Energy > MeterLog.MaxEnrgy[IsHld]) THEN
         Trouble:=Trouble + L_DatTim.DateStr(D) +
                  " ଠ쭮 祭 ࣨ (" + FLDTOSTR(_Energy, "4.", 0) + ")|";
        END;
 
        whmSum:=0;
        SETSEL(MtrRg, Meter, Meter.Meter, L_Meter.mrk_WattageHalfhourMax);
        SRCINSEL(MtrRg, GE, , D);
        WHILE OCCURE(MtrRg) & (MtrRg.DTS < PackTIMESTAMP(D + 1)) DO
         INC(whmSum, MtrRg.Value);
         NEXT(MtrRg);
        END; (* WHILE OCCURE(MtrRg) ... *)
        IF Math.Abs(_Energy - whmSum / 2) > (MeterLog.WHMInacr + 0.00005) THEN
         Trouble:=Trouble + L_DatTim.DateStr(D) +
                  " ࣨ  ⥫ (" + FLDTOSTR(_Energy, "4.", 0) +
                  ")  ᮢ  ࣨ   (";
         whmSum5:=whmSum / 2;
         whmSum :=whmSum / 2;
         IF whmSum # whmSum5 THEN
          Trouble:=Trouble + FLDTOSTR(whmSum5, "5.", 0) + ")|";
         ELSE
          Trouble:=Trouble + FLDTOSTR(whmSum, "4.", 0) + ")|";
         END;
        END;
       EXCEPT
       ON Exc: Exc_Raise DO
        Trouble:=Trouble + Exc.Description + "|";
       END; (* TRY *)
       RESTORE(MtrRg);
      END;
      RESTORE(MeterLog);
     END;
    END;
    INC(RdCnt);
    NEXT(MtrRg);
   END; (* WHILE OCCURE(MtrRg) ... *)
   IF RdCnt > 1 THEN
    LAST(MtrRg);
    DTStr:=DTStr + "-" + L_DatTim.TimestampStr(MtrRg.DTS);
   END;
   MAKEPART(R, MtRoP_Line, DTStr, RdCnt);
   NEXTINEXTENDEDSEL(MtrRg, MtrRgKnd);
  END; (* WHILE OCCURE(MtrRg) ... *)
 END {ReadFromDevice};

 PRIVATE PROCEDURE TypeTrouble(Rep, Trouble: STRING[]);
  VAR
   Sec, i: NUMERIC[0];
 BEGIN
  i:=0;
  WHILE ~!L_Str.PoolItem(Trouble, i) DO
   MAKEPART(^Rep, MtRoP_BadLine, SYSDATE(), SYSTIME(Sec), L_Str.FixedIntStr(Sec, 2), L_Str.PoolItem(Trouble, i));
   INC(i);
  END;
 END {TypeTrouble};

 PUBLIC PROCEDURE ReadFromCurDevice();
  VAR
   Trouble: STRING[];
 BEGIN
  L_General.Assert(ACCESS(Meter), " ⠭ 稪");
  OPENREP(R, L_General.NameConnection(MtRoP_RepName + "_" + Meter.Meter), Create);
  MAKEPART(R, MtRoP_Top);
  ReadFromDevice(Trouble);
  TypeTrouble("R", Trouble);
  MAKEPART(R, MtRoP_Bot);
  CLOSEREP(R);
  L_General.SurveyLastReport();
 END {ReadFromCurDevice};

 PRIVATE PROCEDURE E_ReportContactTo(): L_EMail.tContact;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Utils.Meter_ReportContactTo);
  IF ~!Local THEN
   RETURN Local();
  ELSE
   RETURN L_APEMl.CurrentSenderContact();
  END;
 END {E_ReportContactTo};

 PRIVATE PROCEDURE E_ReportContactFrom(): L_EMail.tContact;
  VAR
   Local: PROC;
 BEGIN
  Local:=PROCREF(E_Utils.Meter_ReportContactFrom);
  IF ~!Local THEN
   RETURN Local();
  ELSE
   RETURN L_APEMl.CurrentSenderContact();
  END;
 END {E_ReportContactFrom};

 PUBLIC PROCEDURE ReadFromAllDevices();
  VAR
   i,
   Sec,
   BadCnt       : NUMERIC[0];
   IsSheduledRun: SCAL[2];
   TinyRepStr,
   EMlMsg,
   Trouble      : STRING[];

 BEGIN
  IsSheduledRun:=~!APP.Get().SesEnv.Item("RunProgAtStart");
  TinyRepStr   :="";
  BadCnt:=0;
  OPENREP(R, L_General.NameConnection(MtRoP_RepName + "_All"), Create);
  MAKEPART(R, MtRoP_Top);
  OPENREP(BR, L_General.NameConnection(MtRoP_RepName + "_Bad"), Create);
  MAKEPART(BR, MtRoP_Top);
  SELECT(Meter, Meter);
  WHILE OCCURE(Meter) DO
   TRY
    ReadFromDevice(Trouble);
   EXCEPT
   ON Exc: Exc_Root DO
    HISTCONF(SYSDATE(), Meter, MeterLog);
    IF ~OCCURE(MeterLog) | MeterLog.TurnedOn THEN
     MAKEPART(R,  MtRoP_BadLine, SYSDATE(), SYSTIME(Sec), L_Str.FixedIntStr(Sec, 2), Exc.Description);
     MAKEPART(BR, MtRoP_BadLine, SYSDATE(), SYSTIME(Sec), L_Str.FixedIntStr(Sec, 2), Exc.Description);
     TinyRepStr:=TinyRepStr + L_Str.CRLF +
                 Meter.Meter + " " + Str.Align(Meter.asName, 64, LEFT) + " " + Exc.Description;
     INC(BadCnt);
    END;
   END; (* TRY *)
   TypeTrouble("R", Trouble);
   TypeTrouble("BR", Trouble);
   i:=0;
   WHILE ~!L_Str.PoolItem(Trouble, i) DO
    TinyRepStr:=TinyRepStr + L_Str.CRLF +
                Meter.Meter + " " + Str.Align(Meter.asName, 64, LEFT) + " " + L_Str.PoolItem(Trouble, i);
    INC(BadCnt);
    INC(i);
   END;
   NEXT(Meter);
  END; (* WHILE OCCURE(Meter) ... *)
  MAKEPART(R, MtRoP_Bot);
  CLOSEREP(R);
  IF BadCnt = 0 THEN
   MAKEPART(BR, MtRoP_OKLine);
  END;
  MAKEPART(BR, MtRoP_Bot);
  CLOSEREP(BR);
  IF IsSheduledRun & (BadCnt > 0) THEN
   EMlMsg:=" ⨨  稪 " + Str.EnumObject(BadCnt, "ந諠|ந諮|ந諮") + " " +
           L_Str.IntStr(BadCnt) + " " + Str.EnumObject(BadCnt, "訡|訡|訡") + ":" + L_Str.CRLF +
           TinyRepStr;
   TRY
    L_APEMl.EasySend(E_ReportContactTo(), E_ReportContactFrom(),
                     "⮪   稪", "", EMlMsg);
   EXCEPT
   ON Exc:Exc_Root DO
    L_StdFrm.RedInform(" 㤠 ࠢ ⮪   稪." +
                       "||" + Exc.MsgOriginalText);
   END; (* TRY *)
  END;
  L_General.SurveyLastReport();
 END {ReadFromAllDevices};

 PRIVATE PROCEDURE Mtr01_MakeTop(F: STRING[]);
 BEGIN
  L_Basis.wb_WrStr(F, "稪");
  L_Basis.wb_WrStr(F, "稪");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "ࠣ");
  L_Basis.wb_WrStr(F, "ࠣ");
  L_Basis.wb_WrStr(F, "饭");
  L_Basis.wb_WrStr(F, "稪");
  L_Basis.wb_WrStr(F, "稪");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "");
  L_Basis.wb_WrStr(F, "६");
  L_Basis.wb_WrStr(F, "⠂६");
  L_Basis.wb_WrStr(F, "祭");
  L_Basis.wb_TermLine(F);
 END {Mtr01_MakeTop};

 PRIVATE PROCEDURE Mtr01_MakeLine_AnyData(F        : STRING[];
                                          _MtrRgKnd: LIKE(MtrRgKnd.MtrRgKnd);
                                          _Sign    : LIKE(MtrRgKnd.Sign);
                                          _Date    : DATE;
                                          _Time    : TIME;
                                          _Value   : LIKE(MtrRg.Value));
 BEGIN
  L_Basis.wb_WrScl(F, Meter.Meter);
  L_Basis.wb_WrStr(F, MeterLog.Name);
  IF ~!MeterLog.Contract THEN
   CONFORM(MeterLog, Contract);
   CONFORM(Contract, Firm);
   L_Basis.wb_WrScl(F, Contract.Contract);
   L_Basis.wb_WrScl(F, Firm.Firm);
   L_Basis.wb_WrStr(F, Firm.Name);
  ELSE
   L_Basis.wb_WrStr(F, "");
   L_Basis.wb_WrStr(F, "");
   L_Basis.wb_WrStr(F, "");
  END;
  L_Basis.wb_WrStr(F, MeterLog.Location);
  L_Basis.wb_WrStr(F, L_Str.PoolItem(L_Meter.mt_Pool, Meter.MtrType));
  L_Basis.wb_WrStr(F, Meter.Serial);
  L_Basis.wb_WrScl(F, _MtrRgKnd);
  L_Basis.wb_WrStr(F, _Sign);
  L_Basis.wb_WrDat(F, _Date);
  L_Basis.wb_WrTim(F, _Time);
  L_Basis.wb_WrOrg(F, L_DatTim.DateTimeStr(_Date, _Time)+";");
  L_Basis.wb_WrNum(F, _Value, 4);
  L_Basis.wb_TermLine(F);
 END {Mtr01_MakeLine_AnyData};

 PRIVATE PROCEDURE Mtr01_MakeLine(F: STRING[]);
 BEGIN
  Mtr01_MakeLine_AnyData(F, MtrRgKnd.MtrRgKnd, MtrRgKnd.Sign, L_DatTim.GetDateOfTIMESTAMP(MtrRg.DTS),
                         L_DatTim.GetTimeOfTIMESTAMP(MtrRg.DTS), MtrRg.Value);
 END {Mtr01_MakeLine};

(*
$P{ LWHMR_Top:
?, L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
Memo.ud_fr(), L, "D.N.y"
Memo.ud_ut(), L, "D.N.y"
@
=#####==================================================########## #####=
|   ᮪ ய饭  稪  ##########  ##########   |
|-----------------------------------------------------------------------|
}
*)

(*
$P{ LWHMR_Line1:
Meter.Meter, L, ""
?, L, "D.N.y H:M"
MtrRgKnd.Sign, C, ""
?, L, ""
@
|#######|################                   | ##### #################|
}
*)

(*
$P{ LWHMR_Line2:
Meter.Meter, L, ""
?, L, "D.N.y H:M"
?, L, "D.N.y H:M"
MtrRgKnd.Sign, C, ""
@
|#######|################ - ################|  #####        |
}
*)

(*
$P{ LWHMR_Bot:
@
=(c) Polak===============================================================
}
*)

 PRIVATE PROCEDURE Mtr01_intMakeMeter(From, UpTo: DATE);
  VAR
   nxt,
   frDTS,
   utDTS : TIMESTAMP;
   val   : FLOAT;
 BEGIN
  frDTS:=PackTIMESTAMP(From, 00:00);
  utDTS:=PackTIMESTAMP(UpTo + 1, 00:00);
  SETSEL(MtrRdout, Meter, Meter.Meter);
  SRCINSEL(MtrRdout, GE, , frDTS);
  WHILE OCCURE(MtrRdout) & (MtrRdout.PassDTS < utDTS) DO
   PROMPT(MtrRdout.Meter + "  " + L_DatTim.TimestampStr(MtrRdout.PassDTS));
   IF ~!MtrRdout.TimDelta THEN
    val:=MtrRdout.TimDelta*24*60*60;
   ELSE
    SETEMPTY(val);
   END;
   SELECT(XDocJrn, XDocJrn, , L_XDoc.xdt_MtrRdout, L_XDoc.EmptyXDocKind, MtrRdout.MtrRdout);
   L_General.CallUs(XDocJrn.Action = L_XDoc.action_AutoBuild);
   SAVE(MeterLog);
   HISTCONF(XDocJrn.Date, Meter, MeterLog);
   Mtr01_MakeLine_AnyData("F", "", "", XDocJrn.Date, XDocJrn.BegTime, val);
   RESTORE(MeterLog);
   NEXT(MtrRdout);
  END; (* WHILE OCCURE(MtrRdout) ... *)

  SELECT(MtrRg, Meter, , Meter.Meter);
  WHILE OCCURE(MtrRg) DO
   CONFORM(MtrRg, MtrRgKnd);
   NARROWSEL(MtrRg, MtrRgKnd);
   nxt:=PackTIMESTAMP(From, 00:00);
   SRCINSEL(MtrRg, GE, , From);
   IF L_Meter.IsEnergy(MtrRgKnd.MtrRgKnd) & (~OCCURE(MtrRg) | (MtrRg.DTS # frDTS)) THEN
    MAKEPART(Mtr01_LostWHMRep, LWHMR_Line1, frDTS, " 砫 ਮ");
    INC(Mtr01_LostCnt);
   END;
   LOOP
    IF ~OCCURE(MtrRg) | (MtrRg.DTS > utDTS) THEN
     IF L_Meter.IsEnergy(MtrRgKnd.MtrRgKnd) THEN
      MAKEPART(Mtr01_LostWHMRep, LWHMR_Line1, utDTS, "  ਮ");
      INC(Mtr01_LostCnt);
     END;
     EXIT;
    ELSIF (MtrRg.MtrRgKnd = L_Meter.mrk_WattageHalfhourMax) & (MtrRg.DTS = utDTS) THEN
     EXIT;
    END;
    PROMPT(MtrRg.Meter + " " + MtrRg.MtrRgKnd + " " + L_DatTim.TimestampStr(MtrRg.DTS));
    IF (MtrRg.Value # 0) |
       L_Meter.IsEnergy(MtrRgKnd.MtrRgKnd) & ((MtrRg.DTS = frDTS) | (MtrRg.DTS = utDTS)) THEN
     Mtr01_MakeLine("F");
    END;
    IF (MtrRgKnd.MtrRgKnd = L_Meter.mrk_WattageHalfhourMax) & (MtrRg.DTS # nxt) THEN
     MAKEPART(Mtr01_LostWHMRep, LWHMR_Line2, nxt, MtrRg.DTS);
     INC(Mtr01_LostCnt);
    END;
    nxt:=MtrRg.DTS + 0.5/24;
    IF MtrRg.DTS = utDTS THEN
     EXIT;
    END;
    NEXT(MtrRg);
   END; (* LOOP*)

   IF (MtrRgKnd.MtrRgKnd = L_Meter.mrk_WattageHalfhourMax) & (nxt # utDTS) THEN
    MAKEPART(Mtr01_LostWHMRep, LWHMR_Line2, nxt, utDTS - 0.5/24);
    INC(Mtr01_LostCnt);
   END;
   NEXTINEXTENDEDSEL(MtrRg, MtrRgKnd);
  END; (* WHILE OCCURE(MtrRg) ... *)
 END {Mtr01_intMakeMeter};

 PRIVATE PROCEDURE Mtr01_MakeMeter(From, UpTo: DATE);
  VAR
   curContract: LIKE(Contract.Contract);
 BEGIN
  SETSEL(MeterLog, Meter, Meter.Meter);
  IF ~SRCINSEL(MeterLog, LE, From) THEN
   FIRST(MeterLog);
  END;
  WHILE OCCURE(MeterLog) & (MeterLog.Date <= UpTo) DO
   IF MeterLog.TurnedOn THEN
    From:=Math.Max(From, MeterLog.Date);
    curContract:=MeterLog.Contract;
    WHILE OCCURE(MeterLog) & (MeterLog.Contract = curContract) & (MeterLog.Date <= UpTo) & MeterLog.TurnedOn DO
     NEXT(MeterLog);
    END;
    IF OCCURE(MeterLog) THEN
     PREV(MeterLog);
    ELSE
     LAST(MeterLog);
    END;
    Mtr01_intMakeMeter(From, Math.Min(MeterLog.DateE, UpTo));
   END;
   NEXT(MeterLog);
  END;
 END {Mtr01_MakeMeter};

 PRIVATE PROCEDURE Mtr01_Init(RepName: STRING[]);
  CONST
   LWHMR_RepName = "LWHMR";
 BEGIN
  FIO.Open(F, L_General.NameConnection(RepName), "REP", Create);
  OPENREP(Mtr01_LostWHMRep, L_General.NameConnection(LWHMR_RepName), Create);
  MAKEPART(Mtr01_LostWHMRep, LWHMR_Top, LWHMR_RepName);
  Mtr01_LostCnt:=0;
  Mtr01_MakeTop("F");
 END {Mtr01_Init};

 PRIVATE PROCEDURE Mtr01_End();
 BEGIN
  FIO.Close(F);
  MAKEPART(Mtr01_LostWHMRep, LWHMR_Bot);
  CLOSEREP(Mtr01_LostWHMRep);
  IF Mtr01_LostCnt THEN
   L_StdFrm.RedInform("    稪  ਮ.  㤥 ." +
                      '||᮪ ய饭  ࠭   "LWHMR"');
  END;
 END {Mtr01_End};

 PUBLIC PROCEDURE Mtr01();
  CONST
   bpId = "Mtr01";
  VAR
   From,
   UpTo : DATE;
 BEGIN
  RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
  L_bp.Begin(bpId, " 稪  ਮ", L_bp.t_Basis, L_General.tc_inIntvl);
  Mtr01_Init(bpId);
  SELECT(Meter, Meter);
  WHILE OCCURE(Meter) DO
   Mtr01_MakeMeter(Memo.ud_fr(), Memo.ud_ut());
   NEXT(Meter);
  END; (* WHILE OCCURE(Meter) ... *)
  Mtr01_End();
  L_bp.End();
 END {Mtr01};

 PUBLIC PROCEDURE Mtr01_RT_Make();
 BEGIN
  RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
  ACCESS(Meter);
  Mtr01_Init(Mtr01_RepName);
  Mtr01_MakeMeter(Memo.ud_fr(), Memo.ud_ut());
  Mtr01_End();
  L_Basis.PlayMakedRunTimeRep(Mtr01_RepName, L_General.timeCond_InIntvl, " 稪 " + Meter.Meter + " " + Meter.asName, 0);
 END {Mtr01_RT_Make};

 PRIVATE PROCEDURE MeterLog_OnReposition();
  VAR
   oField: FORMFIELD;
 BEGIN
  IF ACCESS(MeterLog) THEN
   CONFORM(MeterLog, Meter);
   IF    Meter.MtrModel = L_Meter.mm_Electric_Usual THEN
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_Port");
    oField.Modifiable:=1;
    oField.Color:=FORMFIELD.Color_Native;
    oField.Value:=STRTOFLD(L_Str.AnyItem(MeterLog.InitPrm, ";", 0), NUMERIC);
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_Addr");
    oField.Modifiable:=1;
    oField.Color:=FORMFIELD.Color_Native;
    oField.Value:=STRTOFLD(L_Str.AnyItem(MeterLog.InitPrm, ";", 1), NUMERIC);
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_BaudRate");
    oField.Modifiable:=0;
    oField.Color:=FORMFIELD.Color_Field4;
    SETEMPTY(oField.Value);
   ELSIF Meter.MtrModel = L_Meter.mm_Electric_6823 THEN
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_Port");
    oField.Modifiable:=1;
    oField.Color:=FORMFIELD.Color_Native;
    oField.Value:=STRTOFLD(L_Str.AnyItem(MeterLog.InitPrm, ";", 0), NUMERIC);
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_Addr");
    oField.Modifiable:=0;
    oField.Color:=FORMFIELD.Color_Field4;
    SETEMPTY(oField.Value);
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_BaudRate");
    oField.Modifiable:=0;
    oField.Color:=FORMFIELD.Color_Field4;
    SETEMPTY(oField.Value);
   ELSIF Meter.MtrModel = L_Meter.mm_Electric_301 THEN
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_Port");
    oField.Modifiable:=1;
    oField.Color:=FORMFIELD.Color_Native;
    oField.Value:=STRTOFLD(L_Str.AnyItem(MeterLog.InitPrm, ";", 0), NUMERIC);
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_Addr");
    oField.Modifiable:=0;
    oField.Color:=FORMFIELD.Color_Field4;
    SETEMPTY(oField.Value);
    oField:=RTI.Get().CurWindow.ActiveZone.Fields("ip_BaudRate");
    oField.Modifiable:=1;
    oField.Color:=FORMFIELD.Color_Native;
    oField.Value:=STRTOFLD(L_Str.AnyItem(MeterLog.InitPrm, ";", 2), NUMERIC);
   ELSE
    L_General.Error("  稪");
   END;
  END;
 END {MeterLog_OnReposition};

 PRIVATE PROCEDURE MeterLog_OnBeforeEdit();
 BEGIN
 END {MeterLog_OnBeforeEdit};

 PRIVATE PROCEDURE MeterLog_OnBeforeInsert();
  VAR
   _Contract: LIKE(MeterLog.Contract);
   _TurnedOn: LIKE(MeterLog.TurnedOn);
   _MinWHM  : LIKE(MeterLog.MinWHM);
   _MaxWHM  : LIKE(MeterLog.MaxWHM);
   _MinEnrgy: LIKE(MeterLog.MinEnrgy);
   _MaxEnrgy: LIKE(MeterLog.MaxEnrgy);
   _WHMInacr: LIKE(MeterLog.WHMInacr);
   _Name    : LIKE(MeterLog.Name);
   _InitPrm : LIKE(MeterLog.InitPrm);
   _Location: LIKE(MeterLog.Location);
   _BaudRate: LIKE(MeterLog.BaudRate);
 BEGIN
  CHANGESEL(MeterLog, Meter, , MeterLog.Meter);
  IF PREV(MeterLog) THEN
   _Contract:=MeterLog.Contract;
   _TurnedOn:=MeterLog.TurnedOn;
   _MinWHM  :=MeterLog.MinWHM;
   _MaxWHM  :=MeterLog.MaxWHM;
   _MinEnrgy:=MeterLog.MinEnrgy;
   _MaxEnrgy:=MeterLog.MaxEnrgy;
   _WHMInacr:=MeterLog.WHMInacr;
   _Name    :=MeterLog.Name;
   _InitPrm :=MeterLog.InitPrm;
   _Location:=MeterLog.Location;
   _BaudRate:=MeterLog.BaudRate;
   NEXT(MeterLog);
   MeterLog.Contract:=_Contract;
   MeterLog.TurnedOn:=_TurnedOn;
   MeterLog.MinWHM  :=_MinWHM;
   MeterLog.MaxWHM  :=_MaxWHM;
   MeterLog.MinEnrgy:=_MinEnrgy;
   MeterLog.MaxEnrgy:=_MaxEnrgy;
   MeterLog.WHMInacr:=_WHMInacr;
   MeterLog.Name    :=_Name;
   MeterLog.InitPrm :=_InitPrm;
   MeterLog.Location:=_Location;
   MeterLog.BaudRate:=_BaudRate;
   REPLACE(MeterLog);
   FIXALTERING();
  END;
 END {MeterLog_OnBeforeInsert};

 PRIVATE PROCEDURE MeterLog_OnAfterEdInsDel(wCheckContract: SCAL[2]);
  VAR
   _DateE: DATE;
 BEGIN
  IF wCheckContract & ~!MeterLog.Contract THEN
   CONFORM(MeterLog, Contract);
   L_General.Assert(L_DatTim.IsActive(MeterLog.Date, Contract.Begin, Contract.Till),
                    " " + Contract.Contract + "   " + L_DatTim.DateStr(MeterLog.Date));
  END;

  SAVE(MeterLog);
  ACCESS(Meter);
  IF SELECT(MeterLog, Meter, Meter.Meter) THEN
   LAST(MeterLog);
   SETEMPTY(_DateE);
   REPEAT
    MeterLog.DateE:=_DateE;
    REPLACE(MeterLog);
    _DateE:=MeterLog.Date-1;
   UNTIL ~PREV(MeterLog);
  END;
  RESTORE(MeterLog);
 END {MeterLog_OnAfterEdInsDel};

 PRIVATE PROCEDURE MeterLog_OnAfterEdIns();
 BEGIN
  MeterLog.InitPrm:=L_Str.IntStr(RTI.Get().CurWindow.ActiveZone.Fields("ip_Port").Value) + ";" +
                    L_Str.IntStr(RTI.Get().CurWindow.ActiveZone.Fields("ip_Addr").Value) + ";" +
                    L_Str.IntStr(RTI.Get().CurWindow.ActiveZone.Fields("ip_BaudRate").Value);
  REPLACE(MeterLog);
  MeterLog_OnAfterEdInsDel(1);
 END {MeterLog_OnAfterEdIns};

 PRIVATE PROCEDURE MeterLog_OnAfterInsert();
 BEGIN
  MeterLog_OnAfterEdIns();
 END {MeterLog_OnAfterInsert};

 PRIVATE PROCEDURE MeterLog_OnAfterEdit();
 BEGIN
  MeterLog_OnAfterEdIns();
 END {MeterLog_OnAfterEdit};

 PRIVATE PROCEDURE MeterLog_OnAfterDelete();
 BEGIN
  MeterLog_OnAfterEdInsDel(0);
 END {MeterLog_OnAfterDelete};

 PRIVATE PROCEDURE Meter_Serial_OnSuccessfulChange();
 BEGIN
  L_General.Assert(~SELECT(MtrRdout, Meter, Meter.Meter),
                   "⨬  ᪮   稨  ");
 END {Meter_Serial_OnSuccessfulChange};

 PRIVATE PROCEDURE Meter_OnAfterEdIns();
 BEGIN
  L_General.Assert(~!Meter.Serial, " 㪠 ᪮ ");
  L_General.Assert(SELECT(MeterLog, Meter, Meter.Meter), " ᪮ ଠ樨  稪");
 END {Meter_OnAfterEdIns};

 PRIVATE PROCEDURE Meter_OnAfterInsert();
 BEGIN
  Meter_OnAfterEdIns();
 END {Meter_OnAfterInsert};

 PRIVATE PROCEDURE Meter_OnAfterEdit();
 BEGIN
  Meter_OnAfterEdIns();
 END {Meter_OnAfterEdit};

 PRIVATE PROCEDURE MtrRg_OnAfterInsert();
 BEGIN
  CONFORM(MtrRg, MtrRdout);
  MtrRg.Meter:=MtrRdout.Meter;
  REPLACE(MtrRg);
 END {MtrRg_OnAfterInsert};

 PRIVATE PROCEDURE MtrRdout_OnAfterEdit();
 BEGIN
  L_XDoc.XDocJrn_Mark(L_XDoc.xdt_MtrRdout, L_XDoc.EmptyXDocKind, MtrRdout.MtrRdout, L_XDoc.action_Edit);
 END {MtrRdout_OnAfterEdit};

 PRIVATE PROCEDURE MtrRdout_OnBeforeInsert();
 BEGIN
  L_General.Error("⠢ 饭");
 END {MtrRdout_OnBeforeInsert};

 PRIVATE PROCEDURE MtrRdout_OnBeforeDelete();
 BEGIN
  L_General.Error(" 饭");
 END {MtrRdout_OnBeforeDelete};

 PRIVATE PROCEDURE MtrRdout_Meter_OnSuccessfulChange();
 BEGIN
  L_General.Error(" 稪 饭");
 END {MtrRdout_Meter_OnSuccessfulChange};

 PUBLIC PROCEDURE ViewMtrRdout_SetCursor();
 BEGIN
  RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
  IF SELECT(MtrRdout, Meter, v_Meter) THEN
   LAST(MtrRdout);
   SE_WZD_CURSOR(GE_CURWIN(), GE_CURZONE(), MtrRdout);
  ELSE
   L_StdFrm.Inform(" ᥠᮢ   稪");
   SE_WZD_CURSOR(GE_CURWIN(), GE_CURZONE());
  END;
 END {ViewMtrRdout_SetCursor};

 PUBLIC PROCEDURE ViewMtrRdout_SetCursor_All();
 BEGIN
  SE_WZD_CURSOR(GE_CURWIN(), GE_CURZONE());
 END {ViewMtrRdout_SetCursor_All};

 PUBLIC PROCEDURE ViewMtrRdout();
 BEGIN
  L_General.Assert(ACCESS(Meter), " ⠭ 稪");
  v_Meter:=Meter.Meter;
  LIKEMENU(View, "MtrRdout 1", CALL, "Meter.ViewMtrRdout_SetCursor()");
 END {ViewMtrRdout};

 PUBLIC PROCEDURE FillMeterSerial();
 BEGIN
  L_General.Assert(ACCESS(Meter), " ⠭ 稪");
  L_Meter.FillMeterSerial();
 END {FillMeterSerial};

BEGIN
 L_Events.ExecuteHandler();
END.