(*
%Copyright   (c) Polak Computer System
%Author      SAN
%Purpose     ஫ 筮    
%Type        control
%Specific    SYS
%Modified    16.09.2014 19:01:01 POLAK\San
%Environment
*)

 MODULE Money [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_PRoll, L_Staff, L_Events, Memo, L_AcsPnt, L_APtm, Math,
        L_StdFrm, L_SFee, L_iga, L_WDO;

 PRIVATE PROCEDURE Check();
 BEGIN
  L_General.Assert(Money.Source=L_PRoll.src_Operator, " ⮬᪨ ᫥ 㬬  ந");
  L_PRoll.Salary_CheckNoTouch(Money.Closed);
  L_PRoll.Salary_CheckClcPer();
 END {Check};

 PRIVATE PROCEDURE Reset();
 BEGIN
  Check();
  CONFORM(Money, Staff);
  L_PRoll.ShtDone(0);
 END {Reset};

 PRIVATE PROCEDURE Money_OnBeforeEdit();
 BEGIN
  Check();
  L_General.AdminOnly();
  L_PRoll.Remove_Money(0);
 END {Money_OnBeforeEdit};

 PRIVATE PROCEDURE Money_OnBeforeInsert();
 BEGIN
  L_PRoll.Salary_CheckNoTouch(Memo.gl_mb());
  L_General.PrgsOnly();
  CONFORM(Money, Staff);
  L_Staff.PosToStaffLog(Money.TSN, Memo.gl_cd(), 1);
  Money.Closed  :=Memo.gl_mb();
  Money.WFPeriod:=Memo.gl_mb();
  Money.Period  :=Memo.gl_mb();
  Money.ODate   :=StaffLog.Date;
  REPLACE(Money);
 END {Money_OnBeforeInsert};

 PRIVATE PROCEDURE Money_OnBeforeDelete();
 BEGIN
  Reset();
  L_PRoll.Remove_Money(0);
  L_General.AdminOnly();
 END {Money_OnBeforeDelete};

 PRIVATE PROCEDURE Money_AfterEditInsert();
 BEGIN
  CONFORM(Money, Staff);
  HISTCONF(Money.ODate, Staff, StaffLog);
  L_Staff.CheckManyStaffLog(Money.ODate);
  L_PRoll.CheckMoneyLegality(Money.Period, Money.Closed, Money.MCode, Money.Sum, Money.Tim);
  Reset();
  ADD(MoneyDst, MoneyDst, , Money.RECORDID, EMPTYOF(MoneyDst.XDocId), EMPTYOF(MoneyDst.XObject), EMPTYOF(MoneyDst.AcCut), 0,
       MoneyDst.Tim, Money.Tim,
       MoneyDst.Sum, Money.Sum);
 END {Money_AfterEditInsert};

 PRIVATE PROCEDURE Money_OnAfterEdit();
 BEGIN
  Money_AfterEditInsert();
 END {Money_OnAfterEdit};

 PRIVATE PROCEDURE Money_OnAfterInsert();
 BEGIN
  Money_AfterEditInsert();
 END {Money_OnAfterInsert};

 PRIVATE PROCEDURE Money0_1_OnZoneOpen();
 BEGIN
  RTI.Get().CurZone.Fields(GE_CURFILE()+".Source").Format:=L_PRoll.src_Pool;
 END {Money0_1_OnZoneOpen};

 PRIVATE PROCEDURE Money_OnInitializeWindow();
 BEGIN
  L_WDO.iZone        (0).ViewOnly:=1;
  L_WDO.iEvents      (1, [L_WDO.e_OnAnyModification]);
  L_WDO.iZone        (1).Narrowing:="(Money.TSN=Staff.TSN)&(Money.Closed=Memo.MonthB)";
  L_WDO.iField       (1, "Money.Period").ReverseSorting:=1;
  L_WDO.iField       (1, "Money.WFPeriod").ReverseSorting:=1;
  L_WDO.iField       (1, "Money.ODate").ReverseSorting:=1;
  L_WDO.iUnmodifiable(1, ["Money.Source", "Money.WFPeriod"]);
  L_WDO.iField       (1, "Money.Source").Format:=L_PRoll.src_Pool;
  L_WDO.iZone        (2).ViewOnly:=1;
  L_WDO.iZone        (2).Narrowing:="(PCalc.TSN=Staff.TSN)&(PCalc.Period=Memo.MonthB)";
  L_WDO.iField       (2, "PCalc.Done[1]").Format:="()|ᥣ";
 END {Money_OnInitializeWindow};

 PRIVATE PROCEDURE eMoney_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="Money";
  L_WDO.iEvents      (1, [L_WDO.e_OnAnyModification]);
  L_WDO.iZone        (1).Narrowing:="eMoney.ori=Money.RECORDID";
  L_WDO.iProgs       (1, "Money");
  L_WDO.iField       (1, "eMoney.Type").Format:=L_PRoll.et_Pool;
  L_WDO.iField       (1, "eMoney.SubType").Format:=L_PRoll.est_Pool;
 END {eMoney_OnInitializeWindow};

 PRIVATE PROCEDURE eMoney_OnBeforeAnyModification();
 BEGIN
  SEARCH(Money, RECORDID, , eMoney.ori);
  IF RTI.Get().CurProcess.Kind = FORM_PROCESS.Kind_Insert THEN
   eMoney.TSN:=Money.TSN;
   REPLACE(eMoney);
  END;
  L_AcsPnt.PassAccessPoint(0000051);
  IF RTI.Get().CurProcess.Kind # FORM_PROCESS.Kind_Delete THEN
   IF ~L_APtm.MonthIsClosed(Money.Closed, L_APtm.system_Salary) THEN
    L_StdFrm.WrongDB(" ⥫쭮 ଠ樨  ⮫쪮  ⮬ ( ௫) ");
   END;
  END;
 END {eMoney_OnBeforeAnyModification};

 PRIVATE PROCEDURE RepealSFee(prvClcPer: DATE);
  VAR
   PerRepeal: DATE;
 BEGIN
  SEARCH(Money, RECORDID, , eMoney.ori);
  IF ~!prvClcPer THEN
   PerRepeal:=prvClcPer;
  ELSE
   PerRepeal:=Money.Closed;
  END;

  IF L_DatTim.YEAR(eMoney.ClcPer) = L_DatTim.YEAR(PerRepeal) THEN
   PerRepeal:=Math.Min(eMoney.ClcPer, PerRepeal);
  END;

  IF L_iga.HasAction(L_iga.a_CalculateSFee, PerRepeal) THEN
   L_StdFrm.Question("㤥 믮     "+L_DatTim.PeriodStr(PerRepeal));
   L_SFee.RepealSFee(PerRepeal, "७  ");
  END;
 END {RepealSFee};

 PRIVATE PROCEDURE eMoney_OnBeforeInsert();
 BEGIN
  eMoney_OnBeforeAnyModification();
  L_General.CallUs(eMoney.ori=Money.RECORDID);
 END {eMoney_OnBeforeInsert};

 PRIVATE PROCEDURE eMoney_OnBeforeEdit();
 BEGIN
  eMoney_OnBeforeAnyModification();
 END {eMoney_OnBeforeEdit};

 PRIVATE PROCEDURE eMoney_OnBeforeDelete();
 BEGIN
  eMoney_OnBeforeAnyModification();
  RepealSFee(EMPTYOF(DATE));
 END {eMoney_OnBeforeDelete};

 PRIVATE PROCEDURE eMoney_OnAfterInsert();
 BEGIN
  RepealSFee(EMPTYOF(DATE));
 END {eMoney_OnAfterInsert};

 PRIVATE PROCEDURE eMoney_OnAfterEdit();
 BEGIN
  RepealSFee(GE_PVALUE(eMoney.ClcPer));
 END {eMoney_OnAfterEdit};

 PRIVATE PROCEDURE PCalc_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="Money";
  L_WDO.iZone        (0).ViewOnly:=1;
  L_WDO.iZone        (0).ViewOnly:=1;
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iField       (1, "PCalc.Done[3]").Format:=" |";
  L_WDO.iField       (1, "PCalc.Done[0]").Format:=" |";
  L_WDO.iField       (1, "PCalc.Done[1]").Format:=" |";
 END {PCalc_OnInitializeWindow};

BEGIN
 L_Events.ExecuteHandler();
END.
