(*
%Copyright   (c) Polak Computer System
%Author      MOG
%Purpose          ᭮ 
%Type        control
%Specific    SYS
%Modified    03.09.2014 12:48:15 POLAK\Gina
*)

 MODULE Month [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT Memo, L_General, L_APtm, L_DatTim, L_Events, L_Hld, L_AR, L_StdFrm,
        L_XDoc, L_AcsPnt, P_OwnData, L_iga, L_WDO, L_PCS;

 PRIVATE CONST
  DuratWorkDay = 8;

 PRIVATE PROCEDURE CheckNoTouchChangingOrder(trgSystem: L_APtm.tSystems);
 BEGIN
  SAVE(Month);
  IF Month.NoTouch[trgSystem] THEN
   L_General.Assert(~PREV(Month) | Month.NoTouch[trgSystem],
                    "  뢠 ᫥⥫쭮");
  ELSE
   L_APtm.CheckIfBeforeSystemStart(Month.Period, trgSystem);
   L_General.Assert(~NEXT(Month) | ~Month.NoTouch[trgSystem],
                    "  뢠 ᫥⥫쭮");
  END;
  RESTORE(Month);
 END {CheckNoTouchChangingOrder};

 PRIVATE PROCEDURE SetByPayments(_System: L_APtm.tSystems);
 BEGIN
  Month.NoTouch [_System]:=Month.NoTouch [L_APtm.system_Payments];
  Month.NoTouchD[_System]:=Month.NoTouchD[L_APtm.system_Payments];
  Month.NoTouchT[_System]:=Month.NoTouchT[L_APtm.system_Payments];
  REPLACE(Month);
  CheckNoTouchChangingOrder(_System);
 END {SetByPayments};

 PRIVATE PROCEDURE FixNoTouchChanging(_System: L_APtm.tSystems);
  VAR
   _Action: L_iga.tAction;
 BEGIN
  CheckNoTouchChangingOrder(_System);
  IF _System = L_APtm.system_Salary THEN
   _Action:= L_iga.a_CloseMonth_Salary;
  ELSIF _System = L_APtm.system_Payments THEN
   _Action:= L_iga.a_CloseMonth_Payment;
   SetByPayments(L_APtm.system_Math);
   SetByPayments(L_APtm.system_PDoc);
   SetByPayments(L_APtm.system_KeyAss);
  END;
  IF Month.NoTouch[_System] THEN
   L_iga.Put(_Action, Month.Period, "");
  ELSE
   L_iga.Repeal(_Action, Month.Period, "");
  END;

  SAVE(Month);
  Memo.gl_SetCurDate(Memo.gl_cd());
  RESTORE(Month);
 END {FixNoTouchChanging};

 PRIVATE PROCEDURE Month_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="Calendar";
  L_WDO.iSubwindow   (1, "Calendar");
  L_WDO.iProgs       (1, "Month");
  L_WDO.iField       (1, "Month.NoTouch[4]").Format:=" |";
  L_WDO.iField       (1, "Month.NoTouch[0]").Format:=" |";
 END {Month_OnInitializeWindow};

 PRIVATE PROCEDURE Month_OnBeforeDelete();
 BEGIN
  L_General.Error(" 饭");
 END {Month_OnBeforeDelete};

 PRIVATE PROCEDURE Month_OnBeforeInsert();
 BEGIN
  CHANGESEL(Month, Period);
  PREV(Month);
  L_StdFrm.BlueStop("'筮'   ᥩ 饭.|  ⮬᪨  'ᮬ'."+
                    "|᫨ ⢨⥫쭮 筮    "+L_DatTim.PeriodStr(Month.Period)+","+
                    "|,   ࠡ稪");
 END {Month_OnBeforeInsert};

 PRIVATE PROCEDURE GetBal(): NUMERIC[3];
 BEGIN
  IF L_Hld.DayIsBeforeHoliday(Calendar.Date) THEN
   RETURN DuratWorkDay - 1;
  END;
  RETURN DuratWorkDay;
 END {GetBal};

 PUBLIC PROCEDURE CalcBalInfo();
 BEGIN
  SETZERO(Month.WorkDays, Month.WD6, Month.HBal);
  SELECT(Calendar, Period);
  SRCINSEL(Calendar, EQ, , Month.Period, 01);
  WHILE OCCURE(Calendar) & (Calendar.Period = Month.Period) DO
   PROMPT(L_DatTim.DateStr(Calendar.Date));
   IF Calendar.WeekDay < L_DatTim.wd_Sat THEN
    INC(Month.WorkDays);
    INC(Month.HBal, GetBal());
   END;
   IF Calendar.WeekDay < L_DatTim.wd_Sun THEN
    INC(Month.WD6);
   END;
   NEXT(Calendar);
  END;
  REPLACE(Month);
 END {CalcBalInfo};

 PRIVATE PROCEDURE Month_OnAfterEdit();
  VAR
   Idx       : NUMERIC[0];
   xdc       : STRING[];
   Fields    : FORM_FIELDS;
   oldBan,
   Ban       : SCAL[2];
   pB, pE    : DATE;
   old_arr_pB,
   old_arr_pE: DATE[L_XDoc.txdclsAmount];
   curAcsPnt : LIKE(AcsPnt.AcsPnt);
 BEGIN
  IF RTI.Get().CurWindow.Name = "BanModif" THEN
   P_OwnData.xdmb_Get(old_arr_pB, old_arr_pE);
   SETEMPTY(curAcsPnt);
   Fields:=RTI.Get().CurWindow.Zones(0).Fields;
   Idx   :=0;
   LOOP
    xdc:=L_Str.PoolItem(L_XDoc.xdcls_Pool, Idx);
    IF !xdc THEN
     EXIT;
    END;
    oldBan:=~!old_arr_pB[Idx] & ~!old_arr_pE[Idx];
    Ban   :=Fields(L_WDO.Fld_wIdx("Ban", Idx)).Value;
    pB    :=Fields(L_WDO.Fld_wIdx("pB",  Idx)).Value;
    pE    :=Fields(L_WDO.Fld_wIdx("pE",  Idx)).Value;
    IF ~!pE THEN
     pE:=L_DatTim.MonthEnd(pE);
    END;
    IF (Ban # oldBan) | (pB # old_arr_pB[Idx]) | (pE # old_arr_pE[Idx]) THEN
     IF L_XDoc.xdmb_GetAccessPoint(Idx) # curAcsPnt THEN
      curAcsPnt:=L_XDoc.xdmb_GetAccessPoint(Idx);
      L_XDoc.xdmb_PassAccessPoint(curAcsPnt, "");
     END;
     IF Ban THEN
      IF (pB > Month.Period) THEN
       L_General.Error(" 㬥⮢ '" + xdc + "':  砫 ਮ  ⥪饣");
      ELSIF (pB > pE) THEN
       L_General.Error(" 㬥⮢ '" + xdc + "':  砫 ਮ   ");
      ELSE
       P_OwnData.xdmb_Put(Idx, pB, pE);
      END;
     ELSE
      P_OwnData.xdmb_Put(Idx, EMPTYOF(DATE), EMPTYOF(DATE));
     END; (* IF Ban ... *)
    END; (* IF (Ban # oldBan) ... *)
    INC(Idx);
   END; (* LOOP *)
  ELSE
   CalcBalInfo();
  END;
 END {Month_OnAfterEdit};

 PRIVATE PROCEDURE Month_OnAfterInsert();
 BEGIN
  CalcBalInfo();
 END {Month_OnAfterInsert};

 PRIVATE PROCEDURE Month_NoTouch_OnSuccessfulChange();
  VAR
   _System: L_APtm.tSystems;
 BEGIN
  _System:=RTI.Get().CurZone.ActiveField.ArrayIndex;
  L_General.Assert(Month.NoTouch[_System] = 0, "⨥  - . ᮮ⢥騩 㭪  '⮣'");
  IF _System = L_APtm.system_Salary THEN
   IF ~L_General.IsAdmin() THEN
    L_General.Assert((SYSDATE()-2 < Month.NoTouchD[_System]) & (L_DatTim.DAY(SYSDATE()) < 15),
                     "    ⮫쪮 ");
   END;
   L_AcsPnt.PassAccessPoint(0000026);
  ELSE
   L_AcsPnt.PassAccessPoint(0000028);
  END;
  SETEMPTY(Month.NoTouchD[_System], Month.NoTouchT[_System]);
  REPLACE(Month);
  FixNoTouchChanging(_System);
 END {Month_NoTouch_OnSuccessfulChange};

 PRIVATE PROCEDURE Calendar_OnBeforeInsert();
 BEGIN
  L_General.Error("⠢ 饭");
 END {Calendar_OnBeforeInsert};

 PRIVATE PROCEDURE Calendar_OnBeforeDelete();
 BEGIN
  L_General.Error(" 饭");
 END {Calendar_OnBeforeDelete};

 PRIVATE PROCEDURE Calendar_WDWay_OnSuccessfulChange();
 BEGIN
  IF ~!Calendar.WDWay THEN
   TOOLTIP("㤥    '  室'");
   Calendar.WDInc:=Calendar.WDWay;
  END;
  REPLACE(Calendar);
 END {Calendar_WDWay_OnSuccessfulChange};

 PUBLIC PROCEDURE CloseMonth(_System: L_APtm.tSystems);
 BEGIN
  L_AR.CheckDatabaseTransparency();
  L_General.Assert(Month.NoTouch[_System] = 0,
                   L_DatTim.PeriodStr(Month.Period) + " - 㦥 ");
  Month.NoTouch [_System]:=1;
  Month.NoTouchD[_System]:=SYSDATE();
  Month.NoTouchT[_System]:=SYSTIME();
  REPLACE(Month);
  FixNoTouchChanging(_System);
 END {CloseMonth};

 PUBLIC PROCEDURE FillCalendar();
  VAR
   EndDate: DATE;
 BEGIN
  L_General.Assert(ACCESS(Month), " । ⥪騩 ");
  CONFORM(Month, Calendar);
  IF OCCURE(Calendar) THEN
   STDMSG(" 㦥 . ?", CHOICE);
   REMSEL(Calendar);
  END;

  SEARCH(Calendar, Date, , Month.Period-1);
  IF ~OCCURE(Calendar) THEN
   SELECT(Calendar, Date);
   L_General.Assert(~OCCURE(Calendar),
                    "    ।騩 ");
  END;

  DEFAULT(Calendar);
  Calendar.Date  :=Month.Period;
  Calendar.Period:=Month.Period;
  EndDate        :=L_DatTim.MonthEnd(Calendar.Date);

  WHILE Calendar.Date <= EndDate DO
   PROMPT(L_DatTim.DateStr(Calendar.Date));
   Calendar.Day    :=L_DatTim.DAY(Calendar.Date);
   Calendar.WeekDay:=L_Hld.WeekDay_TransfersConsidered(Calendar.Date);
   Calendar.WDWay  :=Calendar.WeekDay;
   Calendar.WDInc  :=Calendar.WeekDay;
   INSERT(Calendar);
   INC(Calendar.Date);
  END; (* WHILE *)
  CONFORM(Month, Calendar);
 END {FillCalendar};

 PRIVATE PROCEDURE BanModif_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnOpenZone]);
  L_WDO.iEvents      (0, [L_WDO.e_OnAfterEdit]);
  L_WDO.iScsChange   (0, ["Ban_0", "Ban_1", "Ban_2", "Ban_3"]);
  RTI.Get().CurWindow.OnOpenWindow:=1;
 END {BanModif_OnInitializeWindow};

 PRIVATE PROCEDURE BanModif_OnOpenWindow();
  VAR
   Fields   : FORM_FIELDS;
   Idx      : SCAL[L_XDoc.txdclsAmount];
   xdc      : STRING[];
   trgAcsPnt: LIKE(AcsPnt.AcsPnt);
   trgMayPass: SCAL[2];
 BEGIN
  Fields:=RTI.Get().CurWindow.Zones(0).Fields;
  Idx:=0;
  LOOP
   xdc:=L_Str.PoolItem(L_XDoc.xdcls_Pool, Idx);
   IF !xdc THEN
    EXIT;
   END;
   trgAcsPnt :=L_XDoc.xdmb_GetAccessPoint(Idx);
   trgMayPass:=L_AcsPnt.MayPassAccessPoint(trgAcsPnt);
   Fields(L_WDO.Fld_wIdx("xdc", Idx)).Modifiable:=0;
   Fields(L_WDO.Fld_wIdx("Ban", Idx)).Modifiable:=trgMayPass;
   Fields(L_WDO.Fld_wIdx("pB",  Idx)).Modifiable:=trgMayPass;
   Fields(L_WDO.Fld_wIdx("pE",  Idx)).Modifiable:=trgMayPass;
   INC(Idx);
  END;
 END {BanModif_OnOpenWindow};

 PRIVATE PROCEDURE BanModif_SetFieldsVisible(Fields: FORM_FIELDS; fldIdx: NUMERIC[0]; trgVisible: SCAL[2]);
  VAR
   curBan: SCAL[2];
 BEGIN
  Fields(L_WDO.Fld_wIdx("pB", fldIdx)).Visible:=trgVisible;
  Fields(L_WDO.Fld_wIdx("pE", fldIdx)).Visible:=trgVisible;
  curBan:=Fields(L_WDO.Fld_wIdx("Ban", fldIdx)).Value;
  IF curBan THEN
   Fields(L_WDO.Fld_wIdx("Ban", fldIdx)).Color:=FORM_FIELD.Color_MarkedRecord;
  ELSE
   Fields(L_WDO.Fld_wIdx("Ban", fldIdx)).Color:=FORM_FIELD.Color_Text;
  END;
 END {BanModif_SetFieldsVisible};

 PRIVATE PROCEDURE BanModif_0_OnZoneOpen();
  VAR
   Fields   : FORM_FIELDS;
   Idx      : SCAL[L_XDoc.txdclsAmount];
   xdc      : STRING[];
   arrBan   : SCAL[2][L_XDoc.txdclsAmount];
   arr_pB,
   arr_pE   : DATE[L_XDoc.txdclsAmount];
   curBan   : SCAL[2];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  P_OwnData.xdmb_Get(arr_pB, arr_pE);
  Idx   :=0;
  LOOP
   xdc:=L_Str.PoolItem(L_XDoc.xdcls_Pool, Idx);
   IF !xdc THEN
    EXIT;
   END;
   Fields(L_WDO.Fld_wIdx("xdc", Idx)).Value     :=xdc;
   Fields(L_WDO.Fld_wIdx("Ban", Idx)).Format    :="|";
   curBan                                       :=~!arr_pB[Idx] & ~!arr_pE[Idx];
   Fields(L_WDO.Fld_wIdx("Ban", Idx)).Value:=curBan;
   Fields(L_WDO.Fld_wIdx("pB",  Idx)).Value:=arr_pB[Idx];
   Fields(L_WDO.Fld_wIdx("pE",  Idx)).Value:=arr_pE[Idx];
   BanModif_SetFieldsVisible(Fields, Idx, curBan);
   INC(Idx);
  END;
 END {BanModif_0_OnZoneOpen};

 PRIVATE PROCEDURE Ban_X_OnSuccessfulChange();
  VAR
   Fields: FORM_FIELDS;
   Idx   : NUMERIC[0];
   curBan: SCAL[2];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Idx   :=L_Prop.GetCurVideoFieldArrIdx();
  curBan:=Fields(L_WDO.Fld_wIdx("Ban", Idx)).Value;
  BanModif_SetFieldsVisible(Fields, Idx, curBan);
  IF curBan THEN
   ACCESS(Month);
   Fields(L_WDO.Fld_wIdx("pB", Idx)).Value:=Month.Period;
   Fields(L_WDO.Fld_wIdx("pE", Idx)).Value:=Month.Period;
  ELSE
   SETEMPTY(Fields(L_WDO.Fld_wIdx("pB", Idx)).Value, Fields(L_WDO.Fld_wIdx("pE", Idx)).Value);
  END;
 END {Ban_X_OnSuccessfulChange};

 PRIVATE PROCEDURE Month_Ban_0_OnSuccessfulChange();
 BEGIN
  Ban_X_OnSuccessfulChange();
 END {Month_Ban_0_OnSuccessfulChange};

 PRIVATE PROCEDURE Month_Ban_1_OnSuccessfulChange();
 BEGIN
  Ban_X_OnSuccessfulChange();
 END {Month_Ban_1_OnSuccessfulChange};

 PRIVATE PROCEDURE Month_Ban_2_OnSuccessfulChange();
 BEGIN
  Ban_X_OnSuccessfulChange();
 END {Month_Ban_2_OnSuccessfulChange};

 PRIVATE PROCEDURE Month_Ban_3_OnSuccessfulChange();
 BEGIN
  Ban_X_OnSuccessfulChange();
 END {Month_Ban_3_OnSuccessfulChange};

 PUBLIC PROCEDURE SetBanModif();
  VAR
   Idx       : NUMERIC[0];
   MayPassAll,
   MayPassOne: SCAL[2];
   trgAcsPnt : LIKE(AcsPnt.AcsPnt);
   xdc       : STRING[];
 BEGIN
  SETZERO(Idx, MayPassOne);
  MayPassAll:=1;
  LOOP
   xdc:=L_Str.PoolItem(L_XDoc.xdcls_Pool, Idx);
   IF !xdc THEN
    EXIT;
   END;
   trgAcsPnt :=L_XDoc.xdmb_GetAccessPoint(Idx);
   MayPassOne:=MayPassOne | L_AcsPnt.MayPassAccessPoint(trgAcsPnt);
   MayPassAll:=MayPassAll & MayPassOne;
   INC(Idx);
  END;
  IF MayPassOne THEN
   IF ~MayPassAll THEN
    TOOLTIP("     ⠭  䨪樨  ᮢ 㬥⮢");
   END;
   LIKEMENU(Edit, "BanModif 0");
  ELSE
   TOOLTIP("   稩  ⠭  䨪樨 ࢨ 㬥⮢.|"+
           "㯥 ⮫쪮 ᬮ");
   LIKEMENU(ShowWin, "BanModif 0");
  END;
 END {SetBanModif};

BEGIN
 L_Events.ExecuteHandler();
END.
