(*
%Copyright   (c) Polak IT
%Author      21.07.2004 18:54:35 POLAK\ATS
%Purpose     ⥪  ࠡ  ᢮⢠ DsptMode
%Type        library
%Specific    SYS
%Modified    12.09.2012 19:12:57 POLAK\Gina
%Environment
*)

 MODULE P_DsptM [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_Prop, Str, L_Str, L_Dspt, Math, L_r2r, L_WDO;

 PUBLIC TYPE
  tDeadLine = NUMERIC[0][L_Dspt.rp_Amount];
  tTimeReq  = SCAL[2]   [L_Dspt.rp_Amount];

 PUBLIC CONST
  P_Courier     = 8500;
  P_CSFirm      = 8501;
  P_Weight      = 8502;
  P_SendingRate = 8503;
  P_Deadlines   = 8504;

  dw_InLetter    = 0;
  dw_OutLetter   = 1;
  dl_rp_Amount   = 8; (*RefPoint 8  㦥  ᢮⢥*)

 PUBLIC PROCEDURE GetSubWindowName(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    trgProperty = P_CSFirm THEN
   RETURN "P_8501";
  ELSIF trgProperty = P_Weight THEN
   RETURN "P_8502";
  ELSIF trgProperty = P_SendingRate THEN
   RETURN "P_8503";
  ELSIF trgProperty = P_Deadlines THEN
   RETURN "P_8504";
  ELSE
   RETURN "";
  END;
 END {GetSubWindowName};

 PUBLIC PROCEDURE IsSimpleUserOf(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF trgProperty = P_Courier THEN
   RETURN "Staff";
  ELSE
   RETURN "";
  END;
 END {IsSimpleUserOf};

 PRIVATE PROCEDURE StrToNumeric(_Str: STRING[]): FLOAT;
 BEGIN
  IF ~!_Str THEN
   RETURN STRTOFLD(_Str, NUMERIC);
  ELSE
   RETURN 0;
  END;
 END {StrToNumeric};

 PRIVATE PROCEDURE Decode_Weight(    Value    : STRING[];
                                 VAR InWeight : L_Dspt.tWeight;
                                 VAR OutWeight: L_Dspt.tWeight);
 BEGIN
  InWeight :=StrToNumeric(L_Prop.getRecordElement(Value, 0));
  OutWeight:=StrToNumeric(L_Prop.getRecordElement(Value, 1));
 END {Decode_Weight};

 PRIVATE PROCEDURE formSet_Weight();
 BEGIN
  Decode_Weight(DsptMPro.Value, RTI.Get().CurZone.Fields("InWeight").Value, RTI.Get().CurZone.Fields("OutWeight").Value);
 END {formSet_Weight};

 PRIVATE PROCEDURE Decode_SendingRate(    Value    : STRING[];
                                      VAR MinWeight,
                                          AddWeight,
                                          MinPrice,
                                          AddPrice ,
                                          ProcWrth : L_Dspt.tWeight;
                                      VAR TaxWeight,
                                          TaxPrice : L_Dspt.tWeight[8]);
  VAR
   i     : NUMERIC[0];
   Record: STRING[];
 BEGIN
  SETEMPTY(MinWeight, AddWeight, MinPrice, AddPrice, ProcWrth, TaxWeight, TaxPrice);
  Record   :=L_Prop.getRecordElement(Value, 0);
  MinWeight:=StrToNumeric(Str.Item(Record, ",", 0));
  AddWeight:=StrToNumeric(Str.Item(Record, ",", 1));
  MinPrice :=StrToNumeric(Str.Item(Record, ",", 2));
  AddPrice :=StrToNumeric(Str.Item(Record, ",", 3));
  ProcWrth :=StrToNumeric(Str.Item(Record, ",", 4));
  i:=0;
  WHILE i <= HIGH(TaxWeight) DO
   Record      :=L_Prop.getRecordElement(Value, i+1);
   TaxWeight[i]:=StrToNumeric(Str.Item(Record, ",", 0));
   TaxPrice [i]:=StrToNumeric(Str.Item(Record, ",", 1));
   INC(i);
  END;
 END {Decode_SendingRate};

 PRIVATE PROCEDURE formSet_SendingRate();
  VAR
   Fields   : FORM_FIELDS;
   TaxWeight,
   TaxPrice : L_Dspt.tWeight[8];
   i        : NUMERIC[0];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Decode_SendingRate(DsptMPro.Value,
                     Fields("MinWeight").Value, Fields("AddWeight").Value,
                     Fields("MinPrice").Value,  Fields("AddPrice").Value,
                     Fields("ProcWrth").Value,
                     TaxWeight, TaxPrice);
  i:=0;
  WHILE i<=HIGH(TaxWeight) DO
   Fields(L_WDO.Fld_wIdx("TaxWeight", i)).Value:=TaxWeight[i];
   Fields(L_WDO.Fld_wIdx("TaxPrice",  i)).Value:=TaxPrice [i];
   INC(i);
  END;
 END {formSet_SendingRate};

 PRIVATE PROCEDURE Decode_CSFirm(    Value       : STRING[];
                                 VAR IncomingFirm: LIKE(Firm.Firm);
                                 VAR OutgoingFirm: LIKE(Firm.Firm));
 BEGIN
  IF ~!Value THEN
   IncomingFirm:=L_r2r.DecodeReferenceStamp(L_Prop.getRecordElement(Value, 0));
   OutgoingFirm:=L_r2r.DecodeReferenceStamp(L_Prop.getRecordElement(Value, 1));
  ELSE
   SETEMPTY(IncomingFirm, OutgoingFirm);
  END;
 END {Decode_CSFirm};

 PRIVATE PROCEDURE formSet_CSFirm();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  L_WDO.CreateFormLink("Firm", "Incoming");
  L_WDO.CreateFormLink("Firm", "Outgoing");
  Decode_CSFirm(DsptMPro.Value, Fields("IncomingFirm").Value, Fields("OutgoingFirm").Value);
 END {formSet_CSFirm};

 PRIVATE PROCEDURE Decode_Deadlines(    Value   : STRING[];
                                    VAR Deadline: tDeadLine;
                                    VAR TimeReq : tTimeReq);
  VAR
   SingleValue: STRING[];
   i          : NUMERIC[0];
 BEGIN
  SETEMPTY(Deadline);
  SETZERO(i, TimeReq);
  WHILE i < dl_rp_Amount DO
   SingleValue:=L_Prop.getArrayItem(Value, i);
   IF ~!SingleValue THEN
    Deadline[i]:=StrToNumeric(L_Prop.getRecordElement(SingleValue, 0));
    TimeReq[i] :=L_Prop.getRecordElement(SingleValue, 1);
   END;
   INC(i);
  END;
 END {Decode_Deadlines};

 PRIVATE PROCEDURE formSet_Deadlines();
  VAR
   i       : NUMERIC[0];
   Deadline: tDeadLine;
   TimeReq : tTimeReq;
   Fields  : FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Decode_Deadlines(DsptMPro.Value, Deadline, TimeReq);
  i:=0;
  WHILE i < dl_rp_Amount DO
   Fields(L_WDO.Fld_wIdx("Name",     i)).Value:=L_Str.PoolItem(L_Dspt.dt_RefPointPool, i);
   Fields(L_WDO.Fld_wIdx("Deadline", i)).Value:=Deadline[i];
   Fields(L_WDO.Fld_wIdx("TimeReq",  i)).Value:=TimeReq[i];
   Fields(L_WDO.Fld_wIdx("TimeReq", i)).Format:="-|";
   INC(i);
  END;
 END {formSet_Deadlines};

 PUBLIC PROCEDURE SetDataInForm(trgProperty: LIKE(Property.Property));
 BEGIN
  IF    trgProperty = P_CSFirm THEN
   formSet_CSFirm();
  ELSIF trgProperty = P_Weight THEN
   formSet_Weight();
  ELSIF trgProperty = P_SendingRate THEN
   formSet_SendingRate();
  ELSIF trgProperty = P_Deadlines THEN
   formSet_Deadlines();
  END;
 END {SetDataInForm};

 PRIVATE PROCEDURE Encode_CSFirm(VAR Value       : STRING[];
                                     IncomingFirm: LIKE(Firm.Firm);
                                     OutgoingFirm: LIKE(Firm.Firm));
 BEGIN
  L_Prop.putRES(Value, L_r2r.Put("DsptMPro", "Firm", IncomingFirm));
  L_Prop.putRES(Value, L_r2r.Put("DsptMPro", "Firm", OutgoingFirm));
 END {Encode_CSFirm};

 PRIVATE PROCEDURE formGet_CSFirm();
  VAR
   Fields: FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Encode_CSFirm(DsptMPro.Value, Fields("IncomingFirm").Value, Fields("OutgoingFirm").Value);
 END {formGet_CSFirm};

 PRIVATE PROCEDURE Encode_Weight(VAR Value    : STRING[];
                                     InWeight : L_Dspt.tWeight;
                                     OutWeight: L_Dspt.tWeight);

 BEGIN
  IF !InWeight  THEN InWeight :=0; END;
  IF !OutWeight THEN OutWeight:=0; END;
  L_Prop.putRecordElement(Value, InWeight);
  L_Prop.putRecordElement(Value, OutWeight);
 END {Encode_Weight};

 PRIVATE PROCEDURE formGet_Weight();
 BEGIN
  Encode_Weight(DsptMPro.Value, RTI.Get().CurZone.Fields("InWeight").Value, RTI.Get().CurZone.Fields("OutWeight").Value);
 END {formGet_Weight};

 PRIVATE PROCEDURE Encode_SendingRate(VAR Value    : STRING[];
                                          MinWeight,
                                          AddWeight,
                                          MinPrice,
                                          AddPrice ,
                                          ProcWrth : L_Dspt.tWeight;
                                          TaxWeight,
                                          TaxPrice : L_Dspt.tWeight[8]);
  VAR
   i     : NUMERIC[0];
   Record: STRING[];
 BEGIN
  Record:=L_Str.NumericStr(MinWeight, 2) + "," +
          L_Str.NumericStr(AddWeight, 2) + "," +
          L_Str.NumericStr(MinPrice,  2) + "," +
          L_Str.NumericStr(AddPrice,  2) + "," +
          L_Str.NumericStr(ProcWrth,  2);
  L_Prop.putRecordElement(Value, Record);
  i:=0;
  WHILE i <= HIGH(TaxWeight) DO
   IF ~!TaxWeight[i] THEN
    Record:=L_Str.NumericStr(TaxWeight[i], 2) + "," +
            L_Str.NumericStr(TaxPrice[i],  2);
    L_Prop.putRecordElement(Value, Record);
   END;
   INC(i);
  END; (* WHILE (i<=... *)
 END {Encode_SendingRate};

 PRIVATE PROCEDURE formGet_SendingRate();
  VAR
   Fields   : FORM_FIELDS;
   TaxWeight,
   TaxPrice : L_Dspt.tWeight[8];
   i        : NUMERIC[0];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  i:=0;
  WHILE i <= HIGH(TaxWeight) DO
   TaxWeight[i]:=Fields(L_WDO.Fld_wIdx("TaxWeight", i)).Value;
   TaxPrice [i]:=Fields(L_WDO.Fld_wIdx("TaxPrice",  i)).Value;
   INC(i);
  END;
  Encode_SendingRate(DsptMPro.Value,
                     Fields("MinWeight").Value, Fields("AddWeight").Value,
                     Fields("MinPrice").Value,  Fields("AddPrice").Value,
                     Fields("ProcWrth").Value,
                     TaxWeight, TaxPrice);
 END {formGet_SendingRate};

 PRIVATE PROCEDURE Encode_Deadlines(VAR Value   : STRING[];
                                        Deadline: tDeadLine;
                                        TimeReq : tTimeReq);
  VAR
   i     : NUMERIC[0];
   Record: STRING[];
 BEGIN
  i:=0;
  WHILE i < dl_rp_Amount DO
   SETEMPTY(Record);
   IF ~!Deadline[i] THEN
    L_Prop.putRecordElement(Record, Deadline[i]);
    L_Prop.putRecordElement(Record, TimeReq[i]);
   END;
   L_Prop.putArrayItem(Value, Record);
   INC(i);
  END; (* WHILE (i < ... *)
 END {Encode_Deadlines};

 PRIVATE PROCEDURE formGet_Deadlines();
  VAR
   Fields  : FORM_FIELDS;
   i       : NUMERIC[0];
   Deadline: tDeadLine;
   TimeReq : tTimeReq;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  SETEMPTY(Deadline, TimeReq);
  i:=0;
  WHILE i < dl_rp_Amount DO
   Deadline[i]:=Fields(L_WDO.Fld_wIdx("Deadline", i)).Value;
   TimeReq [i]:=Fields(L_WDO.Fld_wIdx("TimeReq",  i)).Value;
   INC(i);
  END;
  Encode_Deadlines(DsptMPro.Value, Deadline, TimeReq);
 END {formGet_Deadlines};

 PUBLIC PROCEDURE GetDataFromForm(trgProperty: LIKE(Property.Property));
 BEGIN
  SETEMPTY(DsptMPro.Value);
  IF    trgProperty = P_CSFirm THEN
   formGet_CSFirm();
  ELSIF trgProperty = P_Weight THEN
   formGet_Weight();
  ELSIF trgProperty = P_SendingRate THEN
   formGet_SendingRate();
  ELSIF trgProperty = P_Deadlines THEN
   formGet_Deadlines();
  ELSE
   L_General.CallUs(0);
  END;
  REPLACE(DsptMPro);
 END {GetDataFromForm};

 PUBLIC PROCEDURE HasDsptMPro(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN SELECT(DsptMPro, DsptMode, trgDsptMode, trgProperty) & SRCINSEL(DsptMPro, LE, D);
 END {HasDsptMPro};

 PUBLIC PROCEDURE HasActualDsptMPro(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasDsptMPro(trgDsptMode, D, trgProperty) & (!DsptMPro.DocExpD | (D < DsptMPro.DocExpD));
 END {HasActualDsptMPro};

 PRIVATE PROCEDURE GetDsptMPro(trgDsptMode: LIKE(DsptMode.DsptMode);
                               D          : DATE;
                               trgProperty: LIKE(Property.Property);
                               Strong     : SCAL[2]): STRING[];
  VAR
   _Value: LIKE(DsptMPro.Value);
 BEGIN
  IF HasActualDsptMPro(trgDsptMode, D, trgProperty) THEN
   IF ~!IsSimpleUserOf(trgProperty) THEN
    _Value:=L_r2r.DecodeReferenceStamp(DsptMPro.Value);
   ELSE
    _Value:=DsptMPro.Value;
   END;
  ELSE
   IF Strong THEN
    SEARCH(DsptMode, DsptMode, , trgDsptMode);
    L_Prop.ShowUndefinedProValueError(TABLE.Get("DsptMode").Id, DsptMode.RECORDID, trgProperty, D);
   ELSE
    SETEMPTY(_Value);
   END;
  END;
  RETURN _Value;
 END {GetDsptMPro};

 PUBLIC PROCEDURE GetDsptMProStrong(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetDsptMPro(trgDsptMode, D, trgProperty, 1);
 END {GetManProStrong};

 PUBLIC PROCEDURE GetDsptMProSoft(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetDsptMPro(trgDsptMode, D, trgProperty, 0);
 END {GetDsptMProSoft};

 PUBLIC PROCEDURE GetCSFirm(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; dir: LIKE(Dspt.Type)): LIKE(Firm.Firm);
  VAR
   IncomingFirm,
   OutgoingFirm: LIKE(Firm.Firm);
 BEGIN
  Decode_CSFirm(GetDsptMProSoft(trgDsptMode, D, P_CSFirm), IncomingFirm, OutgoingFirm);
  IF     dir = L_Dspt.t_Outgoing THEN
   RETURN OutgoingFirm;
  ELSIF dir = L_Dspt.t_Incoming THEN
   RETURN IncomingFirm;
  END;
 END {GetCSFirm};

 PUBLIC PROCEDURE GetWeight(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; ocur: SCAL[2]): L_Dspt.tWeight;
  VAR
   InWeight,
   OutWeight: L_Dspt.tWeight;
 BEGIN
  Decode_Weight(GetDsptMProSoft(trgDsptMode, D, P_Weight), InWeight, OutWeight);
  IF    ocur = dw_InLetter THEN
   RETURN InWeight;
  ELSIF ocur = dw_OutLetter THEN
   RETURN OutWeight;
  END;
 END {GetWeight};

 PUBLIC PROCEDURE GetSendingRate(trgDsptMode: LIKE(DsptMode.DsptMode); D: DATE; Weight: LIKE(EnvlLttr.Weight); Worth: LIKE(Dspt.Worth)): LIKE(Dspt.Freight);
  VAR
   i        : NUMERIC[0];
   Val      : LIKE(DsptMPro.Value);
   Price    : NUMERIC[2];
   MinWeight,
   AddWeight,
   MinPrice,
   AddPrice ,
   ProcWrth : L_Dspt.tWeight;
   TaxWeight,
   TaxPrice : L_Dspt.tWeight[8];
 BEGIN
  Val:=GetDsptMProSoft(trgDsptMode, D, P_SendingRate);
  SETZERO(Price, ProcWrth);
  IF ~!Val THEN
   Decode_SendingRate(Val, MinWeight, AddWeight, MinPrice, AddPrice, ProcWrth, TaxWeight, TaxPrice);
   i:=1;
   WHILE (TaxWeight[i] # 0) & (Price = 0) DO
    IF Weight <= TaxWeight[i] THEN
     Price:=TaxPrice[i];
    END;
    INC(i);
   END;
   IF Price = 0 THEN
    Price:=MinPrice + (Math.Div(Weight - MinWeight, AddWeight) +
                       (Math.Mod(Weight - MinWeight, AddWeight) > 0)) * AddPrice * (Weight > MinWeight);
   END;
  END;
  IF ~!Worth THEN
   Price:=Price+Worth * ProcWrth/100;
  END;
  RETURN Price;
 END {GetSendingRate};

 PUBLIC PROCEDURE GetDeadline(    trgDsptMode: LIKE(DsptMode.DsptMode);
                                  D          : DATE;
                              VAR DeadLines  : tDeadLine;
                              VAR TimeReq    : tTimeReq);
 BEGIN
  Decode_Deadlines(GetDsptMProSoft(trgDsptMode, D, P_Deadlines), DeadLines, TimeReq);
 END {GetDeadline};

 PUBLIC PROCEDURE GetVal2Show(): STRING[];
  VAR
   Result      : STRING[];
   IncomingFirm,
   OutgoingFirm: LIKE(Firm.Firm);
 BEGIN
  SETEMPTY(Result);
  IF DsptMPro.Property = P_Courier THEN
   IF ~!DsptMPro.Value THEN
    L_General.CallUs(SEARCH(Staff, TSN, L_r2r.DecodeReferenceStamp(DsptMPro.Value)));
    CONFORM(Staff, Man);
    Result:=Staff.TSN + " " + Man.Name;
   END;
  ELSIF  DsptMPro.Property = P_CSFirm THEN
   Decode_CSFirm(DsptMPro.Value, IncomingFirm, OutgoingFirm);
   IF ~!IncomingFirm THEN
    L_General.CallUs(SEARCH(Firm, Firm, IncomingFirm));
    Result:=" " + Firm.Firm + " " + Firm.Name + "; ";
   END;
   IF ~!OutgoingFirm THEN
    L_General.CallUs(SEARCH(Firm, Firm, OutgoingFirm));
    Result:=Result + " " + Firm.Firm + " " + Firm.Name;
   END;
  ELSE
   Result:=DsptMPro.Value;
  END;
  RETURN Result;
 END {GetVal2Show};

END.
