(*
%Copyright   (c) Polak IT
%Author      09.03.2013 10:01:57 POLAK\MLB
%Purpose
%Type        library
%Specific    AP
%Modified    20.03.2013 18:20:23 POLAK\San
%Environment
*)

 MODULE P_MarkG [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC CONST
  P_NormExpTR_TO = 2201;

  MaxAmntNormExp = 35;

 PUBLIC TYPE
  tRunLim  = NUMERIC[0];
  tMathExp = NUMERIC[2];
  tTimeExp = NUMERIC[2];

 IMPORT L_Prop, L_General, L_DatTim, L_WDO, L_Str, L_Way, L_StdFrm, L_Car;

 PUBLIC PROCEDURE GetSubWindowName(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    trgProperty = P_NormExpTR_TO THEN
   RETURN "P_2201";
  ELSE
   RETURN EMPTYOF(STRING[]);
  END;
 END {GetSubWindowName};

 PRIVATE PROCEDURE AssertCompatibility(Cond: SCAL[2]);
 BEGIN
  L_General.Assert(Cond, "⢮  ⨬   㯯 ப " + MarkG.MarkGT);
 END {AssertCompatibility};

 PUBLIC PROCEDURE CanHaveProp(trgProperty: LIKE(Property.Property));
 BEGIN
  CONFORM(MarkGPro, MarkG);
  IF trgProperty = P_NormExpTR_TO THEN
   AssertCompatibility(MarkG.MarkGT=L_Way.mgt_CostAccounting);
  END;
 END {CanHaveProp};

 PUBLIC PROCEDURE HasPro(trgMarkG: LIKE(MarkG.MarkG); D: DATE; trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN SELECT(MarkGPro, MarkG, trgMarkG, trgProperty) & SRCINSEL(MarkGPro, LE, D);
 END {HasPro};

 PUBLIC PROCEDURE HasActualMarkGPro(trgMarkG: LIKE(MarkG.MarkG); D: DATE; trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPro(trgMarkG, D, trgProperty) & (!MarkGPro.DocExpD | (D < MarkGPro.DocExpD));
 END {HasActualMarkGPro};

 PRIVATE PROCEDURE GetProValue(trgMarkG         : LIKE(MarkG.MarkG);
                               D                : DATE;
                               trgProperty      : LIKE(Property.Property);
                               PresenceCondition: L_Prop.tPresenceCondition): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  IF HasActualMarkGPro(trgMarkG, D, trgProperty) THEN
   Result:=MarkGPro.Value;
  ELSE
   SETEMPTY(Result);
   IF PresenceCondition = L_Prop.pc_Strong THEN
    SEARCH(MarkG, MarkG, , trgMarkG);
    L_Prop.ShowUndefinedProValueError(TABLE.Get("MarkG").Id, MarkG.RECORDID, trgProperty, D);
   END;
  END;
  RETURN Result;
 END {GetProValue};

 PUBLIC PROCEDURE GetProValueSoft(trgMarkG   : LIKE(MarkG.MarkG);
                                  D          : DATE;
                                  trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetProValue(trgMarkG, D, trgProperty, L_Prop.pc_Soft);
 END {GetProValueSoft};

 PUBLIC PROCEDURE GetProValueStrong(trgMarkG   : LIKE(MarkG.MarkG);
                                    D          : DATE;
                                    trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetProValue(trgMarkG, D, trgProperty, L_Prop.pc_Strong);
 END {GetProValueStrong};

 PRIVATE PROCEDURE Decode_NormExpTR_TO(    Value  : STRING[];
                                       VAR RunLim : tRunLim[];
                                       VAR MathExp: tMathExp[];
                                       VAR TimeExp: tTimeExp[]);
  VAR
   Item: STRING[];
   i   : NUMERIC[0];
 BEGIN
  SETEMPTY(RunLim, MathExp, TimeExp);
  i:=0;
  LOOP
   Item:=L_Prop.getAIS(Value, i);
   IF !Item THEN
    EXIT;
   END;
   RunLim [i]:=STRTOFLD(L_Prop.getRES(Item, 0), NUMERIC);
   MathExp[i]:=STRTOFLD(L_Prop.getRES(Item, 1), NUMERIC);
   TimeExp[i]:=STRTOFLD(L_Prop.getRES(Item, 2), NUMERIC);
   INC(i);
  END; (* LOOP *)
 END {Decode_NormExpTR_TO};

 PRIVATE PROCEDURE formSet_NormExpTR_TO();
  VAR
   Fields : FORM_FIELDS;
   Idx    : SCAL[MaxAmntNormExp];
   RunLim : tRunLim [MaxAmntNormExp];
   MathExp: tMathExp[MaxAmntNormExp];
   TimeExp: tTimeExp[MaxAmntNormExp];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Decode_NormExpTR_TO(MarkGPro.Value, RunLim, MathExp, TimeExp);
  FOR_EACH Idx DO
   Fields(L_WDO.Fld_wIdx("RunLim",  Idx)).Value:=RunLim [Idx];
   Fields(L_WDO.Fld_wIdx("MathExp", Idx)).Value:=MathExp[Idx];
   Fields(L_WDO.Fld_wIdx("TimeExp", Idx)).Value:=TimeExp[Idx];
  END;
 END {formSet_NormExpTR_TO};

 PUBLIC PROCEDURE SetDataInForm(trgProperty: LIKE(Property.Property));
 BEGIN
  IF    trgProperty = P_NormExpTR_TO THEN
   formSet_NormExpTR_TO();
  ELSE
   L_General.CallUs(0);
  END;
 END {SetDataInForm};

 PRIVATE PROCEDURE Encode_NormExpTR_TO(VAR Value  : STRING[];
                                           RunLim : tRunLim[];
                                           MathExp: tMathExp[];
                                           TimeExp: tTimeExp[]);
  VAR
   Item: STRING[];
   Idx : NUMERIC[0];
 BEGIN
  SETEMPTY(Value);
  Idx:=0;
  WHILE Idx < MaxAmntNormExp DO
   IF ~!RunLim[Idx] | ~!MathExp[Idx] | ~!TimeExp[Idx] THEN
    SETEMPTY(Item);
    L_Prop.putREV(Item, RunLim [Idx]);
    L_Prop.putREV(Item, MathExp[Idx]);
    L_Prop.putREV(Item, TimeExp[Idx]);
    L_Prop.putAIS(Value, Item);
   END;
   INC(Idx);
  END; (* WHILE Idx < ... *)
 END {Encode_NormExpTR_TO};

 PRIVATE PROCEDURE NondecreasingControl(Array: NUMERIC[3][]; Idx: NUMERIC[0]; MayBeEqual: SCAL[2]; IndexName: STRING[]);
  VAR
   j: NUMERIC[0];
 BEGIN
  IF (Idx > 0) & ~!Array[Idx] THEN
   j:=Idx-1;
   WHILE j >=0 DO
    IF ~!Array[j] THEN
     IF Array[Idx] < Array[j] THEN
      L_General.Error(IndexName + "  ப " + L_Str.IntStr(Idx+1) +
                      " , 祬  ப " + L_Str.IntStr(j+1));
     ELSIF ~MayBeEqual & (Array[Idx] = Array[j]) THEN
      L_General.Error(IndexName + "  ப " + L_Str.IntStr(Idx+1) +
                      " ⠪    ப " + L_Str.IntStr(j+1));
     END;
    END;
    DEC(j);
   END; (* WHILE j >= ... *)
  END; (* IF Idx > 0 ... *)
 END {NondecreasingControl};

 PRIVATE PROCEDURE formGet_NormExpTR_TO();
  VAR
   Fields : FORM_FIELDS;
   Idx    : SCAL[MaxAmntNormExp];
   RunLim : tRunLim [MaxAmntNormExp];
   MathExp: tMathExp[MaxAmntNormExp];
   TimeExp: tTimeExp[MaxAmntNormExp];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  SETEMPTY(RunLim, MathExp, TimeExp);
  FOR_EACH Idx DO
   RunLim [Idx]:=Fields(L_WDO.Fld_wIdx("RunLim",  Idx)).Value;
   MathExp[Idx]:=Fields(L_WDO.Fld_wIdx("MathExp", Idx)).Value;
   TimeExp[Idx]:=Fields(L_WDO.Fld_wIdx("TimeExp", Idx)).Value;
   L_General.Assert((!RunLim[Idx] = !MathExp[Idx]) & (!MathExp[Idx] = !TimeExp[Idx]),
                    " ⥫  ப     ६." + L_Str.CRLF +
                    "ப " + L_Str.IntStr(Idx + 1) + "   ");
   NondecreasingControl(RunLim,  Idx, 0, "। ஡");
   NondecreasingControl(MathExp, Idx, 1, "");
   NondecreasingControl(TimeExp, Idx, 1, "ମ");
  END;
  Encode_NormExpTR_TO(MarkGPro.Value, RunLim, MathExp, TimeExp);
 END {formGet_NormExpTR_TO};

 PUBLIC PROCEDURE GetDataFromForm(trgProperty: LIKE(Property.Property));
 BEGIN
  SETEMPTY(MarkGPro.Value);
  IF    trgProperty = P_NormExpTR_TO THEN
   formGet_NormExpTR_TO();
  ELSE
   L_General.CallUs(0);
  END;
  REPLACE(MarkGPro);
 END {GetDataFromForm};

 PRIVATE PROCEDURE MarkG_Get_NormExpTR_TO(trgMarkG: LIKE(MarkG.MarkG);
                                          D       : DATE;
                                          TotRun  : NUMERIC[3];
                                          VAR curRunLim : tRunLim;
                                          VAR curMathExp: tMathExp;
                                          VAR curTimeExp: tTimeExp);
  VAR
   Idx    : NUMERIC[0];
   RunLim : tRunLim [MaxAmntNormExp];
   MathExp: tMathExp[MaxAmntNormExp];
   TimeExp: tTimeExp[MaxAmntNormExp];
 BEGIN
  Decode_NormExpTR_TO(GetProValueStrong(trgMarkG, D, P_NormExpTR_TO), RunLim, MathExp, TimeExp);
  Idx:=0;
  WHILE (Idx < MaxAmntNormExp) & ~!RunLim[Idx] & (RunLim[Idx]*1000 < TotRun) DO
   INC(Idx);
  END; (* WHILE Idx < ... *)
  IF (Idx < MaxAmntNormExp) & ~!RunLim[Idx] & (RunLim[Idx]*1000 >= TotRun) THEN
   curRunLim :=RunLim [Idx]*1000;
   curMathExp:=MathExp[Idx];
   curTimeExp:=TimeExp[Idx];
  ELSE
   L_General.CallUs(SEARCH(MarkG, MarkG, trgMarkG));
   L_General.Error("㯯 ப " + trgMarkG + ", ᢮⢮ " + L_Str.IntStr(P_NormExpTR_TO) + ":"
                 + "||   ஡ " + L_Str.NumericStr(TotRun, 0) + " "
                 + "||( ﭨ  " + L_DatTim.DateStr(D) + ")");
  END;
 END {MarkG_Get_NormExpTR_TO};

 PUBLIC PROCEDURE Car_Get_NormExpTR_TO(trgBoxNbr: LIKE(Car.BoxNbr);
                                       D        : DATE;
                                       TotRun   : NUMERIC[3];
                                       VAR curRunLim : tRunLim;
                                       VAR curMathExp: tMathExp;
                                       VAR curTimeExp: tTimeExp);
 BEGIN
  L_Car.StandOnCarLog(trgBoxNbr, D);
  MarkG_Get_NormExpTR_TO(L_Way.GetMarkG(CarLog.Mark, L_Way.mgt_CostAccounting), D, TotRun,
                         curRunLim, curMathExp, curTimeExp);
 END {Car_Get_NormExpTR_TO};

END.
