(*
%Copyright   (c) Polak IT
%Author      20.03.2009 15:04:16 POLAK\GAV
%Purpose
%Type        library
%Specific    SYS
%Modified    24.09.2014 16:14:43 LDA
%Environment
*)

 MODULE P_ODoc [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_Prop, L_DatTim, L_r2r;

 PUBLIC CONST
  P_PaymentRevisePeriod = 2500;
  P_ExpirationDate      = 2501;
  P_Content             = 2502;
  P_Staff               = 2503;
  P_Note                = 2504;

 PUBLIC PROCEDURE GetSubWindowName(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    trgProperty = P_PaymentRevisePeriod THEN RETURN "P_2500";
  ELSIF trgProperty = P_ExpirationDate      THEN RETURN "P_2501";
  ELSIF trgProperty = P_Staff               THEN RETURN L_Prop.GetStandartWindowName("ODocPro");
  ELSE                                           RETURN "";
  END;
 END {GetSubWindowName};

 PUBLIC PROCEDURE IsSimpleUserOf(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    trgProperty = P_Staff THEN
   RETURN "Staff";
  ELSE
   RETURN "";
  END;
 END {IsSimpleUserOf};

 PUBLIC PROCEDURE PropertyModificationAllowed(trgProperty: LIKE(Property.Property)): L_Prop.tModifAllowedCondition;
 BEGIN
  IF (trgProperty = P_Content) | (trgProperty = P_Staff) | (trgProperty = P_Note) THEN
   RETURN L_Prop.mac_AutoOnly;
  ELSE
   RETURN L_Prop.mac_Inside;
  END;
 END {PropertyModificationAllowed};

 PUBLIC PROCEDURE HasPro(trgODocKind: LIKE(ODoc.ODocKind);
                         trgODoc    : LIKE(ODoc.ODoc);
                         trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN SELECT(ODocPro, ODoc, trgODocKind, trgODoc, trgProperty) & ~!ODocPro.Value;
 END {HasPro};

 PUBLIC PROCEDURE HasActualPro(trgODocKind: LIKE(ODoc.ODocKind);
                               trgODoc    : LIKE(ODoc.ODoc);
                               trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPro(trgODocKind, trgODoc, trgProperty);
 END {HasActualPro};

 PRIVATE PROCEDURE Decode_Pro(trgValue   : LIKE(ODocPro.Value);
                              trgProperty: LIKE(Property.Property)): LIKE(ODocPro.Value);
 BEGIN
  L_General.CallUs(~!IsSimpleUserOf(trgProperty));
  RETURN L_r2r.Decode(trgValue);
 END {Decode_Pro};

 PRIVATE PROCEDURE GetPro(trgODocKind: LIKE(ODoc.ODocKind);
                          trgODoc    : LIKE(ODoc.ODoc);
                          trgProperty: LIKE(Property.Property);
                          Strong     : SCAL[2]): LIKE(ODocPro.Value);
  VAR
   Result: LIKE(ODocPro.Value);
 BEGIN
  SETEMPTY(Result);
  IF    HasActualPro(trgODocKind, trgODoc, trgProperty) THEN
   IF ~!IsSimpleUserOf(trgProperty) THEN
    Result:=Decode_Pro(ODocPro.Value, trgProperty);
   ELSE
    Result:=ODocPro.Value;
   END;
  ELSIF Strong THEN
   SEARCH(ODoc, ODoc, , trgODocKind, trgODoc);
   L_Prop.ShowUndefinedProValueError(TABLE.Get("ODoc").Id, ODoc.RECORDID, trgProperty, L_General.EMPTY_DATE());
  END;
  RETURN Result;
 END {GetPro};

 PUBLIC PROCEDURE GetProStrong(trgODocKind: LIKE(ODoc.ODocKind);
                               trgODoc    : LIKE(ODoc.ODoc);
                               trgProperty: LIKE(Property.Property)): LIKE(ODocPro.Value);
 BEGIN
  RETURN GetPro(trgODocKind, trgODoc, trgProperty, 1);
 END {GetProStrong};

 PUBLIC PROCEDURE GetProSoft(trgODocKind: LIKE(ODoc.ODocKind);
                             trgODoc    : LIKE(ODoc.ODoc);
                             trgProperty: LIKE(Property.Property)): LIKE(ODocPro.Value);
 BEGIN
  RETURN GetPro(trgODocKind, trgODoc, trgProperty, 0);
 END {GetProSoft};

 PRIVATE PROCEDURE Decode_ExpirationDate(    trgValue      : STRING[];
                                         VAR ExpirationDate: DATE);
 BEGIN
  SETEMPTY(ExpirationDate);
  IF ~!trgValue THEN
   ExpirationDate:=L_Prop.GetDateFromStr(trgValue, 0);
  END;
 END {Decode_ExpirationDate};

 PRIVATE PROCEDURE formSet_ExpirationDate();
 BEGIN
  Decode_ExpirationDate(ODocPro.Value, RTI.Get().CurZone.Fields("v_ExpirationDate").Value);
 END {formSet_ExpirationDate};

 PUBLIC PROCEDURE GetExpirationDate(trgODocKind: LIKE(ODoc.ODocKind);
                                    trgODoc    : LIKE(ODoc.ODoc)): DATE;
  VAR
   Result: DATE;
 BEGIN
  Decode_ExpirationDate(GetProStrong(trgODocKind, trgODoc, P_ExpirationDate), Result);
  RETURN Result;
 END {GetExpirationDate};

 PRIVATE PROCEDURE Decode_PaymentRevisePeriod(trgValue: STRING[]; VAR trgBegin, trgTill: DATE);
 BEGIN
  trgBegin:=L_Prop.GetDateFromStr(L_Prop.getRES(trgValue, 0), 1);
  trgTill :=L_Prop.GetDateFromStr(L_Prop.getRES(trgValue, 1), 1);
 END {Decode_PaymentRevisePeriod};

 PRIVATE PROCEDURE prp_NotEmptyTill(trgODocKind: LIKE(ODoc.ODocKind); trgODoc: LIKE(ODoc.ODoc)): SCAL[2];
 BEGIN
  SEARCH(ODoc, ODoc, , trgODocKind, trgODoc);
  RETURN (ODoc.Sense = L_ODoc.fms_PaymentReviseAct_InPer);
 END {prp_NotEmptyTill};

 PRIVATE PROCEDURE formSet_PaymentRevisePeriod();
  VAR
   tillField: FORM_FIELD;
   FlagEdit: SCAL[2];
 BEGIN
  Decode_PaymentRevisePeriod(ODocPro.Value, RTI.Get().CurZone.Fields("v_prp_Begin").Value, RTI.Get().CurZone.Fields("v_prp_Till" ).Value);
  FlagEdit:=prp_NotEmptyTill(ODoc.ODocKind, ODoc.ODoc);
  tillField:=RTI.Get().CurZone.Fields("v_prp_Till");
  IF ~FlagEdit THEN
   IF ~!tillField.Value THEN
    SETEMPTY(tillField.Value);
    FIXALTERING();
   END;
  END;
  tillField.Modifiable    :=FlagEdit;
  tillField.MustBeNonempty:=FlagEdit;
 END {formSet_PaymentRevisePeriod};

 PUBLIC PROCEDURE SetDataInForm(trgProperty: LIKE(Property.Property));
 BEGIN
  IF    trgProperty = P_PaymentRevisePeriod THEN
   formSet_PaymentRevisePeriod();
  ELSIF trgProperty = P_ExpirationDate THEN
   formSet_ExpirationDate();
  END;
 END {SetDataInForm};

 PRIVATE PROCEDURE formGet_PaymentRevisePeriod();
  VAR
   trgBegin,
   trgTill : DATE;
 BEGIN
  trgBegin:=RTI.Get().CurZone.Fields("v_prp_Begin").Value;
  trgTill :=RTI.Get().CurZone.Fields("v_prp_Till" ).Value;
  L_General.Assert(trgBegin < trgTill, " 砫 ਮ ᢥન ⮢  |" +
                                       "  砭 ਮ ᢥન");
  L_General.Assert(!trgTill | (trgTill <= ODoc.Date), " 砭 ਮ ᢥન ⮢  |" +
                                                       "  㬥");
  ODocPro.Value:="";
  L_Prop.putRED(ODocPro.Value, trgBegin);
  L_Prop.putRED(ODocPro.Value, trgTill);
 END {formGet_PaymentRevisePeriod};

 PRIVATE PROCEDURE formGet_P_ExpirationDate();
  VAR
   trgDate: DATE;
 BEGIN
  trgDate:=RTI.Get().CurZone.Fields("v_ExpirationDate").Value;
  IF ~!trgDate THEN
   ODocPro.Value:=L_DatTim.DateStr(trgDate);
  END;
 END {formGet_P_ExpirationDate};

 PUBLIC PROCEDURE GetDataFromForm(trgProperty: LIKE(Property.Property));
 BEGIN
  SETEMPTY(ODocPro.Value);
  IF    trgProperty = P_PaymentRevisePeriod THEN
   formGet_PaymentRevisePeriod();
  ELSIF trgProperty = P_ExpirationDate THEN
   formGet_P_ExpirationDate();
  ELSE
   L_General.CallUs(0);
  END;
  REPLACE(ODocPro);
 END {GetDataFromForm};

 PUBLIC PROCEDURE InsertProperty(trgODocKind: LIKE(ODocKind.ODocKind);
                                 trgODoc    : LIKE(ODoc.ODoc);
                                 trgProperty: LIKE(Property.Property);
                                 trgRow     : LIKE(ODocPro.PropRow);
                                 trgValue   : STRING[]);
 BEGIN
  DEFAULT(ODocPro);
  ODocPro.ODocKind:=trgODocKind;
  ODocPro.ODoc    :=trgODoc;
  ODocPro.PropRow :=trgRow;
  ODocPro.Property:=trgProperty;
  ODocPro.Value   :=trgValue;
  INSERT(ODocPro);
 END {InsertProperty};

 PUBLIC PROCEDURE Encode_ViolationActContent(trgEDOKind: LIKE(EDOKind.EDOKind)): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  SETEMPTY(Result);
  L_Prop.putRES(Result, L_r2r.Put("ODocPro", "EDOKind", trgEDOKind));
  RETURN Result;
 END {Encode_ViolationActContent};

 PUBLIC PROCEDURE Get_ViolationActContent(    trgODocKind: LIKE(ODoc.ODocKind);
                                              trgODoc    : LIKE(ODoc.ODoc);
                                          VAR trgEDOKind : LIKE(EDOKind.EDOKind));
 BEGIN
  SETEMPTY(trgEDOKind);
  IF HasPro(trgODocKind, trgODoc, P_Content) THEN
   trgEDOKind:=L_r2r.Decode(L_Prop.getRES(ODocPro.Value, 0));
  END;
 END {Get_ViolationActContent};

 PUBLIC PROCEDURE GetPaymentRevisePeriod(trgODocKind : LIKE(ODoc.ODocKind);
                                         trgODoc     : LIKE(ODoc.ODoc);
                                         VAR trgBegin,
                                             trgTill : DATE);
  VAR
   curValue: LIKE(ODocPro.Value);
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   curValue:=GetProStrong(trgODocKind, trgODoc, P_PaymentRevisePeriod);
   Decode_PaymentRevisePeriod(curValue, trgBegin, trgTill);
   IF prp_NotEmptyTill(ODoc.ODocKind, ODoc.ODoc) & !trgTill THEN
    L_General.Error(" 祣 㬥   ᮮ饭 '"+L_Str.PoolItem(L_ODoc.fms_Pool, ODoc.Sense)+"'|"+
                    "  㪠  砭 ਮ ᢥન ⮢"+L_StdFrm.HR()+
                    "室 ࠢ쭮  ᢮⢮ "+L_Str.IntStr(P_PaymentRevisePeriod)+"  祬 㬥|"+L_ODoc.GetXDocName(ODoc.XDocId));
   END;
  EXCEPT
  END; (* TRY *)
 END {GetPaymentRevisePeriod};

 PUBLIC PROCEDURE P_2500_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="ObjPro";
  L_WDO.iEvents      (0, [L_WDO.e_OnAnyModification, L_WDO.e_OnReposition]);
  L_WDO.iUnmodifiable(0, ["ObjectDescr", "ObjectTitle"]);
  L_WDO.iMayBeEmpty  (0, ["v_prp_Till"]);
  L_WDO.iProgs       (0, "ObjPro");
 END {P_2500_OnInitializeWindow};

END.
