(*
%Copyright   (c) Polak Computer System
%Author      08.08.2001 19:56:08 BOR
%Purpose     ⥪  ࠡ  ᢮⢠ PDoc
%Type        library
%Specific    SYS
%Modified    25.04.2014 15:43:56 POLAK\GAV
%Environment
*)

 MODULE P_PDoc [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC CONST
  P_CBStamp       = 6100;
  P_CBDlStamp     = 6101;
  P_MyPlace       = 6102;
  P_Comment       = 6103;

 IMPORT L_General, L_Prop, L_WDO, L_r2r, P_Firm;

 PRIVATE CONST
  wLinkWindowName = "PDocPrL";

 PUBLIC PROCEDURE PropertyModificationAllowed(trgProperty: LIKE(Property.Property)): L_Prop.tModifAllowedCondition;
 BEGIN
  IF (trgProperty = P_CBStamp) | (trgProperty = P_CBDlStamp) | (trgProperty = P_Comment) THEN
   RETURN L_Prop.mac_AutoOnly;
  ELSE
   RETURN L_Prop.mac_Inside;
  END;
 END {PropertyModificationAllowed};

 PUBLIC PROCEDURE GetPDocProPool(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF L_Prop.IsLocalProperty(trgProperty) THEN
   RETURN L_Prop.E_GetPool(trgProperty);
  ELSE
   RETURN "";
  END;
 END {GetPDocProPool};

 PUBLIC PROCEDURE IsSimpleUserOf(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF trgProperty = P_MyPlace THEN
   RETURN "MyPlace";
  ELSE
   RETURN "";
  END;
 END {IsSimpleUserOf};

 PUBLIC PROCEDURE GetSubWindowName(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF ~!IsSimpleUserOf(trgProperty) THEN
   RETURN wLinkWindowName;
  ELSE
   RETURN "";
  END;
 END {GetSubWindowName};

 PUBLIC PROCEDURE Value_OnSuccessfulChange(trgProperty: LIKE(Property.Property));
  VAR
   trgMyPlace: LIKE(MyPlace.MyPlace);
 BEGIN
  IF trgProperty = P_MyPlace THEN
   trgMyPlace:=RTI.Get().CurZone.Fields("Value").Value;
   IF ~!trgMyPlace & (trgMyPlace = P_Firm.mp_Fict) THEN
    L_General.Error("'⮯' 0000    㪠  ᢮⢥");
   END;
  END;
 END {Value_OnSuccessfulChange};

 PRIVATE PROCEDURE Decode_PDocPro(trgValue   : LIKE(PDocPro.Value);
                                  trgProperty: LIKE(PDocPro.Property)): LIKE(PDocPro.Value);
 BEGIN
  L_General.CallUs(~!IsSimpleUserOf(trgProperty));
  RETURN L_r2r.Decode(trgValue);
 END {Decode_PDocPro};

 PRIVATE PROCEDURE formSet_PDocPro(trgProperty: LIKE(PDocPro.Property));
  VAR
   W: FORM_WINDOW;
   Z: FORM_ZONE;
 BEGIN
  W:=RTI.Get().CurWindow;
  Z:=RTI.Get().CurZone;
  IF W.OriginalName = wLinkWindowName THEN
   WHILE Z.Links.Count DO
    Z.Links.Remove(Z.Links(0));
   END;
   L_WDO.CreateFormLink_Simple(IsSimpleUserOf(Property.Property), "Value");
   Z.Fields("Value").Value:=Decode_PDocPro((^(Z.MasterTable.Fields("Value").ObtainName())), trgProperty);
  END;
 END {formSet_PDocPro};

 PUBLIC PROCEDURE SetDataInForm(trgProperty : LIKE(Property.Property));
 BEGIN
  IF ~!IsSimpleUserOf(trgProperty) THEN
   formSet_PDocPro(trgProperty);
  END;
 END {SetDataInForm};

 PUBLIC PROCEDURE Encode_PDocPro(trgValue   : LIKE(PDocPro.Value);
                                 trgProperty: LIKE(PDocPro.Property)): LIKE(PDocPro.Value);
 BEGIN
  L_General.CallUs(~!IsSimpleUserOf(trgProperty));
  RETURN L_r2r.Put("PDocPro", IsSimpleUserOf(trgProperty), trgValue);
 END {Encode_PDocPro};

 PUBLIC PROCEDURE GetDataFromForm(trgProperty : LIKE(Property.Property));
 BEGIN
  SETEMPTY(PDocPro.Value);
  IF ~!IsSimpleUserOf(trgProperty) THEN
   PDocPro.Value:=Encode_PDocPro(RTI.Get().CurZone.Fields("Value").Value, trgProperty);
  END;
  REPLACE(PDocPro);
 END {GetDataFromForm};

 PUBLIC PROCEDURE HasPDocPro(trgPDocKind: LIKE(PDoc.PDocKind); trgPDoc: LIKE(PDoc.PDoc); trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN SELECT(PDocPro, PDoc, trgPDocKind, trgPDoc, trgProperty) & ~!PDocPro.Value;
 END {HasPDocPro};

 PUBLIC PROCEDURE HasActualPDocPro(trgPDocKind: LIKE(PDoc.PDocKind); trgPDoc: LIKE(PDoc.PDoc); trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPDocPro(trgPDocKind, trgPDoc, trgProperty);
 END {HasActualPDocPro};

 PRIVATE PROCEDURE GetPDocPro(trgPDocKind: LIKE(PDoc.PDocKind);
                              trgPDoc    : LIKE(PDoc.PDoc);
                              trgProperty: LIKE(Property.Property);
                              trgPC      : L_Prop.tPresenceCondition): LIKE(PDocPro.Value);
 BEGIN
  IF HasPDocPro(trgPDocKind, trgPDoc, trgProperty) THEN
   IF ~!IsSimpleUserOf(trgProperty) THEN
    RETURN Decode_PDocPro(PDocPro.Value, trgProperty);
   ELSE
    RETURN PDocPro.Value;
   END;
  ELSE
   IF trgPC = L_Prop.pc_Strong THEN
    SEARCH(PDoc, PDoc, , trgPDocKind, trgPDoc);
    L_Prop.ShowUndefinedProValueError(TABLE.Get("PDoc").Id, PDoc.RECORDID, trgProperty, EMPTYOF(DATE));
   END;
   RETURN "";
  END;
 END {GetPDocPro};

 PUBLIC PROCEDURE GetPDocProStrong(trgPDocKind: LIKE(PDoc.PDocKind); trgPDoc: LIKE(PDoc.PDoc); trgProperty: LIKE(Property.Property)): LIKE(PDocPro.Value);
 BEGIN
  RETURN GetPDocPro(trgPDocKind, trgPDoc, trgProperty, L_Prop.pc_Strong);
 END {GetPDocProStrong};

 PUBLIC PROCEDURE GetPDocProSoft(trgPDocKind: LIKE(PDoc.PDocKind); trgPDoc: LIKE(PDoc.PDoc); trgProperty: LIKE(Property.Property)): LIKE(PDocPro.Value);
 BEGIN
  RETURN GetPDocPro(trgPDocKind, trgPDoc, trgProperty, L_Prop.pc_Soft);
 END {GetPDocProSoft};

 PUBLIC PROCEDURE SetPDocPro(trgPDocKind: LIKE(PDoc.PDocKind); trgPDoc: LIKE(PDoc.PDoc); trgProperty: LIKE(Property.Property); trgValue: STRING[]);
  VAR
   _PropRow: LIKE(PDocPro.PropRow);
 BEGIN
  IF SELECT(PDocPro, PDoc, trgPDocKind, trgPDoc) & LAST(PDocPro) THEN
   _PropRow:=PDocPro.PropRow+1;
  ELSE
   _PropRow:=1;
  END;
  DEFAULT(PDocPro);
  PDocPro.PDocKind:=trgPDocKind;
  PDocPro.PDoc    :=trgPDoc;
  PDocPro.PropRow :=_PropRow;
  PDocPro.Property:=trgProperty;
  PDocPro.Value   :=trgValue;
  PDocPro.Val2Show:=GetVal2Show();
  INSERT(PDocPro);
 END {SetPDocPro};

 PUBLIC PROCEDURE RemovePDocPro(trgPDocKind: LIKE(PDoc.PDocKind); trgPDoc: LIKE(PDoc.PDoc); trgProperty: LIKE(PDocPro.Property));
 BEGIN
  WHILE SELECT(PDocPro, PDoc, trgPDocKind, trgPDoc, trgProperty) DO
   REMOVE(PDocPro);
  END;
 END {RemovePDocPro};

 PUBLIC PROCEDURE GetVal2Show(): STRING[];
  VAR
   Result    : STRING[];
   trgMyPlace: LIKE(MyPlace.MyPlace);
 BEGIN
  IF PDocPro.Property = P_MyPlace THEN
   trgMyPlace:=L_r2r.Decode(PDocPro.Value);
   Result:=trgMyPlace;
   IF ~!trgMyPlace THEN
    SEARCH(MyPlace, MyPlace, , trgMyPlace);
    Result:=Result+ " " + MyPlace.Name;
   END;
  ELSE
   Result:=PDocPro.Value;
  END;
  RETURN Result;
 END {GetVal2Show};

END.
