(*
%Copyright   (c) Polak IT
%Author      28.10.2008 11:07:37 POLAK\GAV
%Purpose
%Type        control,report,temporary,convertor,operation,library,to paste
%Specific    AP,AP_PC,AP_PT,SYS,TOOL,LOC
%Modified    01.07.2013 13:43:08 POLAK\LDA
%Environment
*)

 MODULE P_Rpr [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tContractCondition = SCAL[2];

 PUBLIC CONST
  P_AdditionalInfo     = 4501;
  P_Location           = 4502;
  P_RepeatedRprS       = 4503;
  P_RepeatedRprJ       = 4504;
  P_FailureOutput      = 4505;
  P_RprTFailReason     = 4506;
  P_ShortRprSDescr     = 4507;
  P_ContractConditions = 4508;
  P_EngineParams       = 4509;


  fo_TypePool = "||室| 襤訩  ";

  cc_Usual    = 0;
  cc_Specific = 1;

  CntCylinder = 6;

 PRIVATE CONST
  pool_Location       = 0000984;
  pool_RprTFailReason = 0000986;
  pool_ShortRprSDescr = 0000987;

 IMPORT L_General, L_DatTim, L_Prop, L_Rpr, L_Str, Str, L_Pool, L_Events, L_WDO;

 PUBLIC PROCEDURE GetSubWindowName(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    trgProperty = P_EngineParams THEN RETURN "P_4509";
  ELSE                                    RETURN "";
  END;
 END {GetSubWindowName};

 PUBLIC PROCEDURE Decode_EngineParams(    trgValue   : LIKE(RprPro.Value);
                                      VAR Pressure   : NUMERIC[1];
                                      VAR Temperature: NUMERIC[0];
                                      VAR Compression: NUMERIC[0][CntCylinder]);
  VAR
   Idx: SCAL[CntCylinder];
 BEGIN
  Pressure   :=L_Prop.GetNumericValueFromStr(L_Prop.getRecordElement(trgValue, 0));
  Temperature:=L_Prop.GetNumericValueFromStr(L_Prop.getRecordElement(trgValue, 1));
  FOR_EACH Idx DO
   Compression[Idx]:=L_Prop.GetNumericValueFromStr(L_Prop.getRecordElement(trgValue, 2+Idx));
  END;
 END {Decode_EngineParams};

 PRIVATE PROCEDURE formSet_EngineParams();
  VAR
   Fields     : FORM_FIELDS;
   Idx        : SCAL[CntCylinder];
   Compression: NUMERIC[0][CntCylinder];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  Decode_EngineParams(RprPro.Value, Fields("Pressure").Value, Fields("Temperature").Value, Compression);
  FOR_EACH Idx DO
   Fields(L_Prop.BuildVideoFieldName("Compression", Idx)).Value:=Compression[Idx];
  END;
 END {formSet_EngineParams};

 PUBLIC PROCEDURE SetDataInForm(trgProperty: LIKE(Property.Property));
 BEGIN
  IF    trgProperty = P_EngineParams THEN
   formSet_EngineParams();
  END;
 END {SetDataInForm};

 PRIVATE PROCEDURE formGet_EngineParams();
  VAR
   Fields     : FORM_FIELDS;
   Idx        : SCAL[CntCylinder];
   Compression: NUMERIC[0];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  L_General.Assert((Fields("Pressure").Value>=1.1) & (Fields("Pressure").Value<=4.5), " ᫠    । 1.1-4.5");
  L_General.Assert((Fields("Temperature").Value>=0) & (Fields("Temperature").Value<=110), "    । 0-110");
  L_Prop.putRecordElement(RprPro.Value, Fields("Pressure"   ).Value);
  L_Prop.putRecordElement(RprPro.Value, Fields("Temperature").Value);
  FOR_EACH Idx DO
   Compression:=Fields(L_Prop.BuildVideoFieldName("Compression", Idx)).Value;
   L_General.Assert((Compression>=18) & (Compression<=30), "  樫 " + L_Str.IntStr(Idx+1)+ "    । 18-30");
   L_Prop.putRecordElement(RprPro.Value, Compression);
  END;
 END {formGet_EngineParams};

 PUBLIC PROCEDURE GetDataFromForm(trgProperty: LIKE(Property.Property));
 BEGIN
  SETEMPTY(RprPro.Value);
  IF    trgProperty = P_EngineParams THEN
   formGet_EngineParams();
  ELSE
   L_General.CallUs(0);
  END;
  REPLACE(RprPro);
 END {GetDataFromForm};

 PUBLIC PROCEDURE OnBeforeInsert();
  VAR
   IsUnique: SCAL[2];
 BEGIN
  SAVE(RprPro);
  SELECT(RprPro, RprS, , RprPro.RprS, RprPro.RprT, RprPro.RprJ, RprPro.Property);
  IsUnique:=~NEXT(RprPro);
  RESTORE(RprPro);
  L_General.Assert(IsUnique, "  ᢮⢮ " + RprPro.Property);

  CONFORM(RprPro, Property);
  IF     (RprPro.Property = P_RepeatedRprJ)  |
         (RprPro.Property = P_EngineParams)  THEN
   L_General.Assert(~!RprPro.RprJ,
                    "C⢮ " + Property.Property + " '" + Property.Name + "'|" +
                    "   ⮫쪮  ࠡ  ");
  ELSIF (RprPro.Property = P_Location)       |
        (RprPro.Property = P_RepeatedRprS)   |
        (RprPro.Property = P_FailureOutput)  |
        (RprPro.Property = P_ShortRprSDescr) |
        (RprPro.Property = P_ContractConditions) THEN
   L_General.Assert(!RprPro.RprT & !RprPro.RprJ,
                    "C⢮ " + Property.Property + " '" + Property.Name + "'|" +
                    "   ⮫쪮  ");
  END;

  CONFORM(RprPro, RprS);
  IF RprPro.Property = P_FailureOutput THEN
   L_General.Assert(L_Rpr.IsRprS_w_Car(), "⢮ " + RprPro.Property + " ଫ ⮫쪮    ᮡ⢥ ");
  END;
 END {OnBeforeInsert};

 PUBLIC PROCEDURE Value_OnSuccessfulChange(trgProperty: LIKE(Property.Property));
 BEGIN
  IF    trgProperty = P_Location       THEN L_Pool.Put(pool_Location,       RprPro.Value);
  ELSIF trgProperty = P_RprTFailReason THEN L_Pool.Put(pool_RprTFailReason, RprPro.Value);
  ELSIF trgProperty = P_ShortRprSDescr THEN L_Pool.Put(pool_ShortRprSDescr, RprPro.Value);
  END;
 END {Value_OnSuccessfulChange};

 PUBLIC PROCEDURE GetPool(_Property: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    _Property = P_FailureOutput THEN
   RETURN fo_TypePool;
  ELSIF _Property = P_Location THEN
   RETURN L_Pool.Get(pool_Location);
  ELSIF _Property = P_RprTFailReason THEN
   RETURN L_Pool.Get(pool_RprTFailReason);
  ELSIF _Property = P_ShortRprSDescr THEN
   RETURN L_Pool.Get(pool_ShortRprSDescr);
  ELSIF _Property = P_ContractConditions THEN
   RETURN "|樠";
  ELSE
   RETURN "";
  END;
 END {GetPool};

 PUBLIC PROCEDURE OnReposition(_Property: LIKE(Property.Property));
 BEGIN
  RTI.Get().CurZone.Fields("RprT_TaskKind").Format:=L_Rpr.rpr_TaskKind_Pool;
 END {OnReposition};

 PUBLIC PROCEDURE HasPro(_RprS: LIKE(RprS.RprS);
                         _RprT: LIKE(RprT.RprT);
                         _RprJ: LIKE(RprJ.RprJ);
                         _Property: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN SELECT(RprPro, RprS, _RprS, _RprT, _RprJ, _Property);
 END {HasPro};

 PUBLIC PROCEDURE HasRprSPro(_RprS: LIKE(RprS.RprS); _Property: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPro(_RprS, "", "", _Property);
 END {HasRprSPro};

 PUBLIC PROCEDURE HasRprTPro(_RprS: LIKE(RprS.RprS);
                             _RprT: LIKE(RprT.RprT);
                             _Property: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPro(_RprS, _RprT, "", _Property);
 END {HasRprTPro};

 PUBLIC PROCEDURE HasActualPro(_RprS: LIKE(RprS.RprS);
                               _RprT: LIKE(RprT.RprT);
                               _RprJ: LIKE(RprJ.RprJ);
                               _Property: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasPro(_RprS, _RprT, _RprJ, _Property);
 END {HasActualPro};

 PUBLIC PROCEDURE GetProStrong(_RprS: LIKE(RprS.RprS);
                               _RprT: LIKE(RprT.RprT);
                               _RprJ: LIKE(RprJ.RprJ);
                               _Property: LIKE(Property.Property)): LIKE(CIPro.Value);
 BEGIN
  AccessPro(_RprS, _RprT, _RprJ, _Property);
  RETURN RprPro.Value;
 END {GetProStrong};

 PUBLIC PROCEDURE GetProSoft(_RprS: LIKE(RprS.RprS);
                             _RprT: LIKE(RprT.RprT);
                             _RprJ: LIKE(RprJ.RprJ);
                             _Property: LIKE(Property.Property)): LIKE(CIPro.Value);
  VAR
   _Value: LIKE(RprPro.Value);
 BEGIN
  IF HasActualPro(_RprS, _RprT, _RprJ, _Property) THEN
   _Value:=RprPro.Value;
  ELSE
   SETEMPTY(_Value);
  END;
  RETURN _Value;
 END {GetProSoft};

 PUBLIC PROCEDURE AccessPro(_RprS: LIKE(RprS.RprS);
                            _RprT: LIKE(RprT.RprT);
                            _RprJ: LIKE(RprJ.RprJ);
                            _Property: LIKE(Property.Property));
 BEGIN
  IF ~HasPro(_RprS, _RprT, _RprJ, _Property) THEN
   SEARCH(RprS, RprS, , _RprS);
   L_Prop.ShowUndefinedProValueError(TABLE.Get("RprS").Id, RprS.RECORDID, _Property, L_General.EMPTY_DATE());
  END;
 END {AccessPro};

 PUBLIC PROCEDURE GetShortRprSDescr(trgRprS: LIKE(RprS.RprS)): STRING[];
 BEGIN
  IF HasRprSPro(trgRprS, P_ShortRprSDescr) THEN
   RETURN Str.Condense(Str.Replace(RprPro.Value, L_Str.CRLF, " "));
  ELSE
   RETURN "";
  END;
 END {GetShortRprSDescr};

 PUBLIC PROCEDURE GetContractCondition(trgRprS: LIKE(RprS.RprS)): tContractCondition;
 BEGIN
  IF HasRprSPro(trgRprS, P_ContractConditions) THEN
   RETURN L_Str.PositionInPool(GetPool(P_ContractConditions), RprPro.Value);
  ELSE
   RETURN cc_Usual;
  END;
 END {GetContractCondition};

END.
