(*
%Copyright   (c) Polak Computer System
%Author      27.12.2000 18:32:08 BOR
%Purpose       ᢮⢠ ⥣ਨ
%Type        library
%Specific    SYS
%Modified    01.08.2012 20:28:49 POLAK\Gina
%Environment
*)

 MODULE P_Union [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tAveIndexation  = NUMERIC[3];

 PUBLIC CONST
  P_YPScale       = 4100;
  P_AveIndexation = 4900;
  P_Bonus         = 4901;
  P_Avans         = 4902;

 IMPORT L_General, L_DatTim, L_Prop, L_Str, L_WDO, L_StdFrm;

 PRIVATE CONST
  amnt_Bonus         = 4;
  amnt_AveIndexation = 3;

  AvansTypePool      = " %| ";

 PUBLIC TYPE
  tAveIndexationArr  = tAveIndexation[amnt_AveIndexation];

 PUBLIC PROCEDURE GetSubWindowName(trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  IF    trgProperty = P_AveIndexation THEN
   RETURN "P_4900";
  ELSIF trgProperty = P_Bonus THEN
   RETURN "P_4901";
  ELSIF trgProperty = P_Avans THEN
   RETURN "P_4902";
  ELSE
   RETURN "";
  END;
 END {GetSubWindowName};

 PRIVATE PROCEDURE CheckExpValidity(trgProperty: LIKE(Property.Property); trgDate, trgDocExpD: DATE);
 BEGIN
   L_General.Assert((trgDate=L_DatTim.MonthBeg(trgDate)) & (!trgDocExpD | (trgDocExpD=L_DatTim.MonthBeg(trgDocExpD))),
                    "⢮  ⢮ 楫 ᫮ 楢");
 END {CheckExpValidity};

 PUBLIC PROCEDURE Check(trgProperty: LIKE(Property.Property));
 BEGIN
  CheckExpValidity(trgProperty, UnionPro.Date, UnionPro.DocExpD);
  IF trgProperty = P_YPScale THEN
   L_Prop.CheckScalValue(UnionPro.Value, "", 99);
  END;
 END {Check};

 PUBLIC PROCEDURE HasUnionPro(trgUnion   : LIKE(Union.Union);
                              D          : DATE;
                              trgProperty: LIKE(Property.Property);
                              dsc        : L_Prop.tOnDateSearchCondition): SCAL[2];
 BEGIN
  IF dsc = L_Prop.dsc_Exactly THEN
   L_General.CallUs(D = L_DatTim.Per(D));
  END;
  SELECT(UnionPro, Union, , trgUnion, trgProperty);
  IF     dsc = L_Prop.dsc_Normal THEN
   SRCINSEL(UnionPro, LE, , D);
  ELSIF (dsc = L_Prop.dsc_Exactly) | (dsc = L_Prop.dsc_Exactly_OnAnyDate) THEN
   SRCINSEL(UnionPro, EQ, , D);
  ELSE
   L_General.CallUs(0);
  END;
  RETURN OCCURE(UnionPro);
 END {HasUnionPro};

 PUBLIC PROCEDURE HasActualPro(trgUnion   : LIKE(Union.Union);
                               D          : DATE;
                               trgProperty: LIKE(Property.Property);
                               dsc        : L_Prop.tOnDateSearchCondition): SCAL[2];
 BEGIN
  RETURN HasUnionPro(trgUnion, D, trgProperty, dsc) & (!UnionPro.DocExpD | (D < UnionPro.DocExpD));
 END {HasActualPro};

 PUBLIC PROCEDURE HasActualPro_Normal(trgUnion   : LIKE(Union.Union);
                                      D          : DATE;
                                      trgProperty: LIKE(Property.Property)): SCAL[2];
 BEGIN
  RETURN HasActualPro(trgUnion, D, trgProperty, L_Prop.dsc_Normal);
 END {HasActualPro_Normal};

 PRIVATE PROCEDURE GetProValue(trgUnion         : LIKE(Union.Union);
                               D                : DATE;
                               trgProperty      : LIKE(Property.Property);
                               dsc              : L_Prop.tOnDateSearchCondition;
                               PresenceCondition: L_Prop.tPresenceCondition): STRING[];
 BEGIN
  IF HasActualPro(trgUnion, D, trgProperty, dsc) THEN
   RETURN UnionPro.Value;
  ELSE
   IF PresenceCondition = L_Prop.pc_Strong THEN
    SEARCH(Union, Union, , trgUnion);
    L_Prop.ShowUndefinedProValueError(TABLE.Get("Union").Id, Union.RECORDID, trgProperty, D);
   ELSE
    RETURN "";
   END; (* IF *)
  END; (* IF *)
 END {GetProValue};

 PUBLIC PROCEDURE GetProValueSoft(trgUnion   : LIKE(Union.Union);
                                  D          : DATE;
                                  trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetProValue(trgUnion, D, trgProperty, L_Prop.dsc_Normal, L_Prop.pc_Soft);
 END {GetProValueSoft};

 PUBLIC PROCEDURE GetProValueStrong(trgUnion   : LIKE(Union.Union);
                                    D          : DATE;
                                    trgProperty: LIKE(Property.Property)): STRING[];
 BEGIN
  RETURN GetProValue(trgUnion, D, trgProperty, L_Prop.dsc_Normal, L_Prop.pc_Strong);
 END {GetProValueSoft};

 PRIVATE PROCEDURE sfv_Decode(    Value: STRING[];
                              VAR Arr  : NUMERIC[3][]);
  VAR
   i   : NUMERIC[0];
   Item: STRING[];
 BEGIN
  SETZERO(Arr);
  i:=0;
  WHILE i <= HIGH(Arr) DO
   Item:=L_Prop.getRecordElement(Value, i);
   IF ~!Item THEN
    Arr[i]:=STRTOFLD(Item, FLOAT);
   END;
   INC(i);
  END; (* WHILE i <= HIGH(Arr) ... *)
 END {sfv_Decode};

 PUBLIC PROCEDURE GetAveIndexation(trgUnion: LIKE(Union.Union);
                                   D       : DATE;
                                   trgAveNo: LIKE(AveType.AveNo)): tAveIndexation;
  VAR
   Value: tAveIndexationArr;
 BEGIN
  IF HasActualPro(trgUnion, D, P_AveIndexation, L_Prop.dsc_Exactly) THEN
   sfv_Decode(UnionPro.Value, Value);
   RETURN Value[trgAveNo];
  ELSE
   RETURN 1;
  END;
 END {GetAveIndexation};

 PUBLIC PROCEDURE GetBonus(trgUnion: LIKE(Union.Union);
                           D       : DATE;
                           BonNo   : SCAL[amnt_Bonus];
                           dsc     : L_Prop.tOnDateSearchCondition): NUMERIC[2];
  VAR
   Value: NUMERIC[2][amnt_Bonus];
 BEGIN
  IF HasActualPro(trgUnion, D, P_Bonus, dsc) THEN
   sfv_Decode(UnionPro.Value, Value);
  ELSE
   SETEMPTY(Value);
  END;
  RETURN Value[BonNo];
 END {GetBonus};

 PRIVATE PROCEDURE Decode_Avans(     Value    : STRING[];
                                VAR AvansBySum: SCAL[2];
                                VAR Avans     : NUMERIC[2]);
 BEGIN
  IF ~!Value THEN
   AvansBySum:=L_Str.PositionInPool(AvansTypePool, L_Prop.getRecordElement(Value, 0));
   Avans     :=STRTOFLD(L_Prop.getRecordElement(Value, 1), FLOAT);
  ELSE
   SETZERO(AvansBySum, Avans);
  END;
 END {Decode_Avans};

 PUBLIC PROCEDURE GetAvans(    trgUnion         : LIKE(Union.Union);
                               D                : DATE;
                               PresenceCondition: L_Prop.tPresenceCondition;
                           VAR AvansbySum       : SCAL[2];
                           VAR Avans            : NUMERIC[2]);
 BEGIN
  IF HasActualPro(trgUnion, D, P_Avans, L_Prop.dsc_Normal) THEN
   Decode_Avans(UnionPro.Value, AvansbySum, Avans);
  ELSE
   IF PresenceCondition = L_Prop.pc_Strong THEN
    L_General.Error(" ଠ樨  ⥣ਨ " + trgUnion + "|ਮ " + L_DatTim.PeriodStr(D));
   ELSE
    SETEMPTY(AvansbySum, Avans);
   END;
  END;
 END {GetAvans};

 PRIVATE PROCEDURE formSet_AveIndexation();
  VAR
   AveIndexation: tAveIndexationArr;
   i            : SCAL[amnt_AveIndexation];
   Fields       : FORM_FIELDS;
 BEGIN
  sfv_Decode(UnionPro.Value, AveIndexation);
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH i DO
   Fields(L_WDO.Fld_wIdx("AveIndexation", i)).Value:=AveIndexation[i];
  END;
 END {formSet_AveIndexation};

 PRIVATE PROCEDURE formSet_Bonus();
  VAR
   Bonus : NUMERIC[2][amnt_Bonus];
   Fields: FORM_FIELDS;
   i     : SCAL[amnt_Bonus];
 BEGIN
  sfv_Decode(UnionPro.Value, Bonus);
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH i DO
   Fields(L_WDO.Fld_wIdx("Bonus", i)).Value:=Bonus[i];
  END;
 END {formSet_Bonus};

 PRIVATE PROCEDURE formSet_Avans();
  VAR
   Fields    : FORM_FIELDS;
   AvansBySum: SCAL[2];
   Avans     : NUMERIC[2];
 BEGIN
  Decode_Avans(UnionPro.Value, AvansBySum, Avans);
  Fields:=RTI.Get().CurZone.Fields;
  Fields("AvansBySum").Format:=AvansTypePool;
  Fields("AvansBySum").Value :=AvansBySum;
  Fields("Avans"     ).Value :=Avans;
 END {formSet_Avans};

 PUBLIC PROCEDURE SetDataInForm(trgProperty: LIKE(Property.Property));
 BEGIN
  IF    trgProperty = P_AveIndexation THEN
   formSet_AveIndexation();
  ELSIF trgProperty = P_Bonus THEN
   formSet_Bonus();
  ELSIF trgProperty = P_Avans THEN
   formSet_Avans();
  END;
 END {SetDataInForm};

 PRIVATE PROCEDURE sfv_EnCode(Arr   : NUMERIC[3][];
                              Format: STRING[]);
  VAR
   i   : NUMERIC[0];
   Item: STRING[];
 BEGIN
  IF ~L_Prop.IsEmptyNumArray(Arr) THEN
   i:=0;
   WHILE i <= HIGH(Arr) DO
    Item:=FLDTOSTR(Arr[i], Format);
    L_Prop.putRecordElement(UnionPro.Value, Item);
    INC(i);
   END; (* WHILE i <= ... *)
  END;
 END {sfv_EnCode};

 PUBLIC PROCEDURE CheckIndexation(AveIndexation: tAveIndexationArr);
  VAR
   i: SCAL[amnt_AveIndexation];
 BEGIN
  FOR_EACH i DO
   L_General.Assert(AveIndexation[i] >= 1.000, "樥 樨   1.000  ");
   IF AveIndexation[i] > 1.5 THEN
    L_StdFrm.Confirm(" 㢥७,  㪠 ࠢ 樥 樨 "+L_Str.PoolItem(" ᪠| 쭨筮| ।ᤥ쭮", i)+"|"+
                      L_Str.NumericStr(AveIndexation[i], 3)+" (㢥祭  "+L_Str.IntStr(AveIndexation[i]*100-100)+"%)");
   END;
  END;
 END {CheckIndexation};

 PRIVATE PROCEDURE formGet_AveIndexation();
  VAR
   AveIndexation: tAveIndexationArr;
   i            : SCAL[amnt_AveIndexation];
   Fields       : FORM_FIELDS;
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH i DO
   AveIndexation[i]:=Fields(L_WDO.Fld_wIdx("AveIndexation", i)).Value;
  END;
  CheckIndexation(AveIndexation);
  sfv_EnCode(AveIndexation, "3.");
 END {formGet_AveIndexation};

 PRIVATE PROCEDURE formGet_Bonus();
  VAR
   Bonus : NUMERIC[2][amnt_Bonus];
   Fields: FORM_FIELDS;
   i     : SCAL[amnt_Bonus];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  FOR_EACH i DO
   Bonus[i]:=Fields(L_WDO.Fld_wIdx("Bonus", i)).Value;
  END;
  sfv_EnCode(Bonus, "2.");
 END {formGet_Bonus};

 PRIVATE PROCEDURE sfv_AvansEnCode(AvansbySum: SCAL[2]; Avans: NUMERIC[2]);
  VAR
   Item: STRING[];
 BEGIN
  IF ~!AvansbySum & ~!Avans THEN
   Item:=L_Str.PoolItem(AvansTypePool, AvansbySum);
   L_Prop.putRecordElement(UnionPro.Value, Item);
   Item:=FLDTOSTR(Avans, "2.");
   L_Prop.putRecordElement(UnionPro.Value, Item);
  END;
 END {sfv_AvansEnCode};

 PRIVATE PROCEDURE formGet_Avans();
  VAR
   Fields    : FORM_FIELDS;
   AvansBySum: SCAL[2];
   Avans     : NUMERIC[2];
 BEGIN
  Fields:=RTI.Get().CurZone.Fields;
  AvansBySum:=Fields("AvansBySum").Value;
  Avans     :=Fields("Avans").Value;
  L_General.Assert(~!AvansBySum = ~!Avans,
                   "祭 ⮣ ᢮⢠     ६");
  sfv_AvansEnCode(AvansBySum, Avans);
 END {formGet_Avans};

 PUBLIC PROCEDURE GetDataFromForm(trgProperty: LIKE(Property.Property));
 BEGIN
  SETEMPTY(UnionPro.Value);
  IF    trgProperty = P_AveIndexation THEN
   formGet_AveIndexation();
  ELSIF trgProperty = P_Bonus THEN
   formGet_Bonus();
  ELSIF trgProperty = P_Avans THEN
   formGet_Avans();
  ELSE
   L_General.CallUs(0);
  END;
  REPLACE(UnionPro);
 END {GetDataFromForm};

END.
