(*
%Copyright   (c) Polak Computer System
%Author      23.01.2001 12:16:34 GINA
%Purpose       室  ( ᥬ ;   )
%Type        report
%Specific    SYS
%Modified    12.05.2014 13:26:57 POLAK\Gina
%Environment
*)

 MODULE T3_2005 [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_AFM, L_Str, L_Staff, L_Tax, L_Packet,
        L_Man, L_PRoll, P_Man, P_Firm, Math, Str, Memo, L_XML,
        L_Russia, L_StdFrm, P_MyFirm, P_MyPlc, P_MySite;

 TYPE
  tSumsByMonth = NUMERIC[2][12];
  tMode        = SCAL[4];

 CONST
  ScaleAmnt = MAXOF(L_Tax.tTax2001_ScalesType) + 1;
  KBKPool   = "18210102021011000110|18210102030011000110|18210102040011000110|18210102010011000110";

  mode_ToPaper         = 0; (*tMode*)
  mode_ByCurrentPacket = 1;
  mode_ForInq          = 2;
  mode_All             = 3;

 VAR
  CurP, CurPE: DATE;
  _FictOnly  : SCAL[2];
  NDFLAmount : NUMERIC[0];
  curMode    : tMode;

 PRIVATE PROCEDURE GetOKATO(): STRING[];
 BEGIN
  RETURN P_MyPlc.GetP_OKATO(P_Firm.GetMyFirmOfFirm(Firm.Firm), EMPTYOF(MyPlace.MyPlace), CurPE);
 END {GetOKATO};

 PRIVATE PROCEDURE GetKPP(): STRING[];
 BEGIN
  RETURN P_MySite.GetP_KPP(P_Firm.GetMyFirmOfFirm(Firm.Firm), EMPTYOF(MyPlace.MyPlace), CurPE);
 END {GetKPP};

 PRIVATE PROCEDURE ToPaper(): SCAL[2];
 BEGIN
  RETURN curMode = mode_ToPaper;
 END {ToPaper};

 PRIVATE PROCEDURE ByCurrentPacket(): SCAL[2];
 BEGIN
  RETURN curMode = mode_ByCurrentPacket;
 END {ByCurrentPacket};

 PRIVATE PROCEDURE MakeForInq(): SCAL[2];
 BEGIN
  RETURN curMode = mode_ForInq;
 END {MakeForInq};

 PRIVATE PROCEDURE AllMode(): SCAL[2];
 BEGIN
  RETURN curMode = mode_All;
 END {AllMode};

 PRIVATE PROCEDURE Create_T3_MTA();
 BEGIN
  CreateFILE(T3_MTA, "", "室", "");

  CreateFIELD(T3_MTA, Scale,   L_Tax.tTax2001_ScalesType, "", "", "",  "", "");
  CreateFIELD(T3_MTA, Man,     LIKE(Man.Man),       "", "", "",  "", "");
  CreateFIELD(T3_MTA, MdfType, LIKE(TaxAdv.Reason), "", "", "",  "稭", "");
  CreateFIELD(T3_MTA, Month,   SCAL[13],            "", "", "",  "", "");
  CreateFIELD(T3_MTA, IncCode, LIKE(TaxAdv.VCode),  "", "", "",  " 室", "");
  CreateFIELD(T3_MTA, IncVal,  NUMERIC[2],          "", "0", "", "㬬 室", "");
  CreateFIELD(T3_MTA, DedCode, LIKE(TaxAdv.LCode),  "", "", "",  " ", "");
  CreateFIELD(T3_MTA, DedVal,  NUMERIC[2],          "", "0", "", "㬬 ", "");

  CreateINDEX(T3_MTA, ScaleM,  UNIQUE, "", "", Scale, Man, MdfType, Month, IncCode, DedCode);
  CreateINDEX(T3_MTA, ScaleMI, PLURAL, "", "", Scale, Man, MdfType, IncCode, Month);
  CreateINDEX(T3_MTA, ScaleMD, PLURAL, "", "", Scale, Man, DedCode);
  CreateINDEX(T3_MTA, MdfType, PLURAL, "", "", MdfType);

  SetVersionOfFILE(T3_MTA, 0);
  L_AFM.AF_CreateTemp("T3_MTA", "T3_MTA");

  CreateFILE(T3_Total, "", "⮣", "");

  CreateFIELD(T3_Total, Man,              LIKE(Man.Man),                "", "", "",  "", "");
  CreateFIELD(T3_Total, Scale,            L_Tax.tTax2001_ScalesType[ScaleAmnt], "", "", "",  "", "");
  CreateFIELD(T3_Total, IncVal,           NUMERIC[2][ScaleAmnt],                "", "0", "", "騩 室", "");
  CreateFIELD(T3_Total, ClcTax,           NUMERIC[2][ScaleAmnt],                "", "0", "", "᫥ ", "");
  CreateFIELD(T3_Total, PaidTax,          NUMERIC[2][ScaleAmnt],                "", "0", "", "ঠ ", "");
  CreateFIELD(T3_Total, RetnTax,          NUMERIC[2][ScaleAmnt],                "", "0", "", "㬬      室  ", "");
  CreateFIELD(T3_Total, OverTax,          NUMERIC[2][ScaleAmnt],                "", "0", "", "㬬, ⥭  㯫     室   ", "");
  CreateFIELD(T3_Total, DedTax,           NUMERIC[2][ScaleAmnt],                "", "0", "", "㬬, 㤥ঠ  㯫     室  ", "");
  CreateFIELD(T3_Total, DbtTax,           NUMERIC[2][ScaleAmnt],                "", "0", "", "    ⥫騪", "");
  CreateFIELD(T3_Total, CrdTax,           NUMERIC[2][ScaleAmnt],                "", "0", "", "㬬 , 譥 㤥ঠ  ⮬", "");
  CreateFIELD(T3_Total, EnfTax,           NUMERIC[2][ScaleAmnt],                "", "0", "", "㬬 , ।  ᪠   ࣠", "");

  CreateINDEX(T3_Total, Man,     UNIQUE, "", "", Man);

  SetVersionOfFILE(T3_Total, 0);
  L_AFM.AF_CreateTemp("T3_Total", "T3_Total");

 END {Create_T3_MTA};

 PRIVATE PROCEDURE SubTitle(): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  IF    ToPaper()         THEN Result:=" 㬠 ⥫";
  ELSIF MakeForInq()      THEN Result:=" , 騬 '।  ᪠'";
  ELSIF ByCurrentPacket() THEN Result:=" , 室騬  ⥪騩 ";
  ELSIF AllMode()         THEN Result:=" ᥬ ";
  ELSE
   L_General.CallUs(0);
  END;
  IF P_Firm.E_wMultipleMyFirm(SYSDATE()) THEN
   P_Firm.PositionOnMyFirm(L_Packet.PckMyFirm);
   Result:=Result + " (: " + MyFirm.MyFirm + " " + MyFirm.vzSign + ")";
  END;
  RETURN Result;
 END {SubTitle};

 PRIVATE PROCEDURE FictOnly(): SCAL[2];
 BEGIN
  RETURN _FictOnly;
 END {FictOnly};

 PRIVATE PROCEDURE CheckPer(withVariants: SCAL[2]);
  VAR
   S: STRING[];
 BEGIN
  S:="  ਮ  "+L_DatTim.DateStr(L_DatTim.YearBeg(CurP))+"  "+L_DatTim.DateStr(CurPE) +
     "||"+SubTitle();
  IF withVariants THEN
   S:="|"+S+"||"+"%<    믫⠬ 襩 ࣠樨   >DB"+
            "|" +"%<  믫⠬   >B" +
            "|" +"%<            ⬥               >Z" ;
   _FictOnly:=(STDFORM(S, , INFORM) = 2);
  ELSE
   L_StdFrm.JustQuestion(S);
   _FictOnly:=0;
  END;
 END {CheckPer};

 PRIVATE PROCEDURE DocNum(_Scale: L_Tax.tTax2001_ScalesType): STRING[];
 BEGIN
  RETURN FLDTOSTR(Man.Man, "", 7) + _Scale;
 END {DocNum};

 PRIVATE PROCEDURE GetFirmGNI(): STRING[];
 BEGIN
  RETURN P_Firm.FP_TaxDepCode(Firm.Firm, SYSDATE());
 END {GetFirmGNI};

(*
$P{ Part1:
CurP, L, "y"
?, L, "", "U"
GetFirmGNI(), L, "", "U"
P_Firm.FP_INN(Firm.Firm, CurPE)+"/"+GetKPP(), L, "", "U B"
P_Firm.FP_AcntPhone(Firm.Firm, SYSDATE()), L, "", "U B"
P_Firm.FP_FirmName(Firm.Firm, CurPE), L, "", "U B"
@
                                                                    ⢥ত
                                                                    ਪ  ᨨ
                                                                     25.11.2005  -3-04/616@

                                                                                     ଠ 2-
           ࠢ  室 䨧᪮   ####  N ########    ####

1.   
1.1. / ( ࣠樨    㠫쭮 ।ਭ⥫) ######################
1.2. 䮭 ##################################
1.3.  ࣠樨 / , , ⢮ 㠫쭮 ।ਭ⥫
#################################################################################################
}
*)

 PRIVATE PROCEDURE GetCitizenCountry(): LIKE(Country.Country);
  VAR
   CitizenCountry: LIKE(Country.Country);
 BEGIN
  SAVE(ManAdr);
  IF ~L_Man.AccessManAdr(Man.Man, CurPE, L_Russia.at_CtMA) THEN
   L_Man.AccessRgManAdr(Man.Man, CurPE);
  END;
  CitizenCountry:=ManAdr.Country;
  RESTORE(ManAdr);
  RETURN CitizenCountry;
 END {GetCitizenCountry};

 PRIVATE PROCEDURE GetManDocCodeMNS(): STRING[30];
  VAR
   _DocType: LIKE(Man.DocType);
 BEGIN
  _DocType:=Man.DocType;
  IF (CurP >= 01\01\2006) &
     ((_DocType=03(*ࠢ ᢮*)) | (_DocType=10(* *)) |
      (_DocType=11(* *)) | (_DocType=09(*..*)) |
      (_DocType=13(*ᯮ 䫮*)) | (_DocType=14(*ᯮ 猪*)) |
      (_DocType=15(*   *)) | (_DocType=16(*.ᯮ *))
     ) THEN
   _DocType:=17(* -*);
  END;
  RETURN L_Man.GetManDocCode(_DocType);
 END {GetManDocCodeMNS};

(*
$P{ Part2:
Man.INN, L, "", "U B"
P_Man.GetManName(Man.Man, SYSDATE()), L, "", "U B"
?, L, "2|1", "U B"
Man.Sex, L, "2|1", "U B"
Man.Birth, L, "D.N.y", "U B"
GetCitizenCountry(), L, "", "U B"
GetManDocCodeMNS(), L, "", "U B"
Man.PspSer + " " + Man.PspNbr, L, "", "U B"
ManAdr.Zip, L, "", "U B"
Str.Cap(ManAdr.Region), L, "", "U B"
?, L, "", "U B"
?, L, "", "U B"
?, L, "", "U B"
?, L, "", "U B"
ManAdr.House, L, "", "U B"
?, L, "", "U B"
ManAdr.Flat, L, "", "U B"
?, L, ""
?, L, ""
?, L, ""
@
2.  䨧᪮  - ⥫ 室
2.1.  ############
2.2. , , ⢮ #####################################################################
2.3.  (1-१,2-१) # 2.4  (1-, 2 - ) #
2.5.  ஦ ############# 2.6. ࠦ⢮ ( ࠭) ###
2.7.  㬥, 㤮⮢饣 筮 ##    2.8. ,  㬥 ##################
2.9.   ⥫⢠  ᨩ᪮ 樨: ⮢  ######  ॣ ##
 ####################################### த #############################################
. 㭪 ##########################################  #####################################
 ############  ##############  #################################################
2.10.  ࠭ ஦:  ࠭ ###  ############################################
#################################################################################################
}
*)

(*
$P{ Part3_1:
?, R, "", "U B"
@
3. 室,   ⠢ ## %
       㬬 室     㬬           㬬 室     㬬 
      室                                      室                
}
*)

(*
$P{ Part3_2:
?, L, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
?, L, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
@
 ##   ####   ##############  ####  ############    ##   ####   ##############  ####  ############
}
*)

(*
$P{ Part4_1:
@
4. ⠭  ⢥   ( )
4.1. 㬬  ⮢, ࠢ  祭    ⥫騪
           㬬                  㬬                  㬬                  㬬
}
*)

(*
$P{ Part4_2:
?, L, "", "U B"
?, R, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
?, L, "", "U B"
?, R, "", "U B"
@
#### ##############       #### ##############       #### ##############       #### ##############
}
*)

(*
$P{ Part4_3:
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
@
4.2.  㢥, ⢥ত饣 ࠢ  ⢥   ##################
4.3.  뤠 㢥 ##########    4.4.   ࣠, 뤠襣 㢥 ####
4.5.  㬬 ।⠢ ⠭  ⮢              #####################
4.6.  㬬 ।⠢ ⢥  ⮢            #####################
}
*)

(*
$P{ Part5:
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
@
5.  㬬 室    室  ⮣  ਮ
5.1.  㬬 室       ##################    5.2.  㬬 室 ################
5.3. 㬬  ᫥ ##################    5.4. 㬬  㤥ঠ ################
5.5. 㬬      室                  ##################
5.6. 㬬, ⥭  㯫     室     ##################
5.7. 㬬, 㤥ঠ  㯫     室    ##################
5.8.     ⥫騪                             ##################
5.9. 㬬 , 譥 㤥ঠ  ⮬                        ##################
5.10.㬬 , ।  ᪠   ࣠                   ##################
}
*)

(*
$P{ Part6:
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
?, R, "", "U B"
L_Str.ShortName(P_Firm.FP_AcntName(Firm.Firm, SYSDATE())), L, "", "U B"
SYSDATE(), L, "D.N.y", "U B"
SYSTIME(), L, "H:M"
@
6.   ᫥   
     ⭮        㬬                 ⭮        㬬
            䨪樨        ᫥               䨪樨        ᫥
                                                                             
########### #################### ##############   ########### #################### ##############
########### #################### ##############   ########### #################### ##############

                                                                               
  __________________ #####################################   ⠢ ##########
 (c) Polak                                                                                  #####

  ..
}
*)

 PRIVATE PROCEDURE PrintMoney(N: NUMERIC[2]): STRING[];
 BEGIN
  RETURN FLDTOSTR(N, "2.", 0);
 END {PrintMoney};

 PRIVATE PROCEDURE PrintMoneyE(N: NUMERIC[2]): STRING[];
 BEGIN
  IF N = 0 THEN
   RETURN "";
  ELSE
   RETURN PrintMoney(N);
  END;
 END {PrintMoneyE};

 PRIVATE PROCEDURE PrintIntgMoney(N: NUMERIC[2]): STRING[];
 BEGIN
  RETURN FLDTOSTR(N, "0", 0);
 END {PrintIntgMoney};

 PRIVATE PROCEDURE Prompt(Stage: NUMERIC[0]);
 BEGIN
  PROMPT(Man.Man + " : ଠ N3 -  " + L_Str.IntStr(Stage));
 END {Prompt};

 PRIVATE PROCEDURE GetPrcByScale(_Scale: L_Tax.tTax2001_ScalesType): STRING[];
 BEGIN
  RETURN L_Str.PoolItem("13|30|35|9", _Scale)
 END {GetPrcByScale};

 PRIVATE PROCEDURE TypePart1(_Scale: L_Tax.tTax2001_ScalesType);
 BEGIN
  Prompt(1);
  MAKEPART(R, Part1, DocNum(_Scale));
 END {TypePart1};

 PRIVATE PROCEDURE PutError(_Man: LIKE(Man.Man); Msg: STRING[]);
 BEGIN
  IF ToPaper() THEN
   L_General.Error(Msg);
  ELSE
   MAKEPART(Err, Error_Line, _Man, Msg);
  END;
 END {PutError};

 PRIVATE PROCEDURE StandOnManAdrForTax(_Man: LIKE(Man.Man));
  VAR
   S, tmp: STRING[];
 BEGIN
  TRY
   L_Tax.StandOnManAdrForTax(_Man, CurPE);
   tmp:=L_Man.GetAdrForTax(S);
  EXCEPT
  ON Exc: Exc_Root DO
   IF ToPaper() THEN
    tmp:=Exc.MsgOriginalText;
   ELSE
    tmp:=Exc.Description;
   END;
  END;
  IF ~!tmp THEN
   PutError(_Man, tmp);
  END;
 END {StandOnManAdrForTax};

 PRIVATE PROCEDURE PrepareManAdressForTax(VAR _Distr,
                                      _Town,
                                      _Village,
                                      _Street,
                                      _Block: STRING[]);
 BEGIN
  SETEMPTY(_Distr, _Town, _Village, _Street, _Block);
  IF ~!ManAdr.Town THEN
   IF ~( ((ManAdr.Region = 77)&Str.Match(Str.Cap(ManAdr.Town), "")) |
         ((ManAdr.Region = 78)&Str.Match(Str.Cap(ManAdr.Town), "")) ) THEN
    _Town:=Str.Cap(ManAdr.Town+" "+L_Russia.TownT_InMNSFormat(ManAdr.TownT));
   END;
  END;
  IF ~!ManAdr.Distr THEN
   _Distr:=Str.Cap(ManAdr.Distr + " -");
  END;
  IF ~!ManAdr.Village THEN
   _Village:=Str.Cap(ManAdr.Village+" "+L_Russia.VillageT_InMNSFormat(ManAdr.VillageT));
  END;
  IF ~!ManAdr.Street THEN
   _Street:=Str.Cap(ManAdr.Street+" "+L_Russia.StreetT_InMNSFormat(ManAdr.StreetT));
  END;
  IF ~!ManAdr.Block THEN
   _Block:=ManAdr.Block;
  END;
  IF ~!ManAdr.Build THEN
   IF ~!ManAdr.Block THEN
    _Block:=_Block+" ";
   END;
   _Block:=_Block+"."+ManAdr.Build;
  END;
 END {PrepareManAdressForTax};

 PRIVATE PROCEDURE GetForeignAddr(IsResident                    : SCAL[2];
                          VAR ForeignCountry            : LIKE(Country.Country);
                          VAR ForeignAddr1, ForeignAddr2: STRING[]);
 BEGIN
  IF ~IsResident | (L_Man.Man_Citizen(Man.Man, CurPE) > L_Man.ct_Russia) THEN
   L_Man.StrongAccessManAdr(Man.Man, CurPE, L_Russia.at_RgMA);
   ForeignCountry:=ManAdr.Country;
   Str.Format(ForeignAddr1,
              Str.Cap(L_Man.GetAdr_asShortStr(Man.Man, CurPE, L_Russia.at_RgMA)),
              44,
              LEFT,
              ForeignAddr2)
  ELSE
   SETEMPTY(ForeignCountry, ForeignAddr1, ForeignAddr2);
  END;
 END {GetForeignAddr};

 PRIVATE PROCEDURE TypePart2();
  VAR
   ForeignAddr1,
   ForeignAddr2  : STRING[];
   ForeignCountry: LIKE(Country.Country);
   _Distr,
   _Town,
   _Village,
   _Street,
   _Block        : STRING[];
   IsResident    : SCAL[2];
 BEGIN
  Prompt(2);
  IsResident:=L_Tax.Tax_IsResident(CurP);
  GetForeignAddr(IsResident, ForeignCountry, ForeignAddr1, ForeignAddr2);
  StandOnManAdrForTax(Man.Man);
  PrepareManAdressForTax(_Distr, _Town, _Village, _Street, _Block);
  MAKEPART(R, Part2, IsResident, _Distr, _Town, _Village, _Street, _Block, ForeignCountry, ForeignAddr1, ForeignAddr2);
 END {TypePart2};

 PRIVATE PROCEDURE TypePart3(_Scale: L_Tax.tTax2001_ScalesType);
  CONST
   Column   = 2;
   MinLines = 9;
   MaxLines = 50 * Column;
  VAR
   _Month  : STRING[][MaxLines];
   _IncCode: LIKE(TaxAdv.VCode)[MaxLines];
   _DedCode: LIKE(TaxAdv.LCode)[MaxLines];
   _Income,
   _Deduct : STRING[][MaxLines];
   i,
   LinesAm : NUMERIC[0];
 BEGIN
  Prompt(3);
  SETEMPTY(_Month, _IncCode, _DedCode, _Income, _Deduct);
  i:=0;
  SELECT(T3_MTA, ScaleM, , _Scale, Man.Man, L_Tax.Tax2001_mt_Inc);
  WHILE OCCURE(T3_MTA) DO
   L_General.Assert(T3_MTA.IncVal >= 0,
                    " " + FLDTOSTR(T3_MTA.Month, "L", 2) + ": ⥫ 室.|" +
                    " 室 " + T3_MTA.IncCode + ", 㬬 " + PrintMoney(T3_MTA.IncVal));
   IF T3_MTA.IncVal # 0 THEN
    _Month  [i]:=L_Str.IntStr(T3_MTA.Month);
    _IncCode[i]:=T3_MTA.IncCode;
    _Income [i]:=PrintMoney(T3_MTA.IncVal);
    _DedCode[i]:=MNSVCode(T3_MTA.DedCode);
    _Deduct [i]:=PrintMoneyE(T3_MTA.DedVal);
    INC(i);
   END;
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)

  IF Math.Mod(i, 2) # 0 THEN
   INC(i);
  END;
  LinesAm:=Math.Max(i / 2, MinLines);

  MAKEPART(R, Part3_1, GetPrcByScale(_Scale));
  i:=0;
  WHILE i < LinesAm DO
   MAKEPART(R, Part3_2, _Month[i],           _IncCode[i],           _Income[i],           _DedCode[i],           _Deduct[i],
                        _Month[i + LinesAm], _IncCode[i + LinesAm], _Income[i + LinesAm], _DedCode[i + LinesAm], _Deduct[i + LinesAm]);
   INC(i);
  END; (* WHILE i < Lines ...*)
 END {TypePart3};


 PRIVATE PROCEDURE GetAnyMdfTypeDedSum(_Scale : L_Tax.tTax2001_ScalesType;
                               MdfType: L_Tax.tTax2001_MdfType): NUMERIC[2];
  VAR
   Deduct: NUMERIC[2];
 BEGIN
  SETZERO(Deduct);
  SELECT(T3_MTA, ScaleM, , _Scale, Man.Man, MdfType);
  WHILE OCCURE(T3_MTA) DO
   INC(Deduct, T3_MTA.DedVal);
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)
  RETURN Deduct;
 END {GetAnyMdfTypeDedSum};

 PRIVATE PROCEDURE GetTaxDeductSum(_Scale : L_Tax.tTax2001_ScalesType): NUMERIC[2];
 BEGIN
  RETURN GetAnyMdfTypeDedSum(_Scale, L_Tax.Tax2001_mt_Inc);
 END {GetTaxDeductSum};

 PRIVATE PROCEDURE GetStdDeductSum(_Scale : L_Tax.tTax2001_ScalesType): NUMERIC[2];
 BEGIN
  RETURN GetAnyMdfTypeDedSum(_Scale, L_Tax.Tax2001_mt_StD);
 END {GetStdDeductSum};

 PRIVATE PROCEDURE GetExpDeductSum(_Scale : L_Tax.tTax2001_ScalesType): NUMERIC[2];
 BEGIN
  RETURN GetAnyMdfTypeDedSum(_Scale, L_Tax.Tax2001_mt_Exp);
 END {GetExpDeductSum};

 PRIVATE PROCEDURE StandOnExpTaxInq(): SCAL[2];
 BEGIN
  L_Tax.Tax_StandOnSingleSpecifiedTaxAdv(CurP, L_Tax.Tax2001__oc_IsMNS, "0311");
  IF L_Tax.Tax_StandOnManTAByTaxAdv(L_DatTim.YearBeg(CurP), CurPE, TaxAdv.TaxAdv) THEN
   CONFORM(ManTA, TaxInq);
   RETURN 1;
  ELSE
   RETURN 0;
  END;
 END {StandOnExpTaxInq};

 PRIVATE PROCEDURE MNSVCode(S: STRING[]): STRING[];
 BEGIN
  IF Str.Match(S, "0???") THEN
   S:=Str.Delete(S, 0, 1);
  END;
  RETURN S;
 END {MNSVCode};

 PRIVATE PROCEDURE GetExpTaxInqAttr(_Scale: L_Tax.tTax2001_ScalesType;
                            VAR eti_Num, eti_Date, eti_GNI: STRING[]);
 BEGIN
  SETEMPTY(eti_Num, eti_Date, eti_GNI);
  IF L_Tax.IsMainScale(_Scale) & StandOnExpTaxInq() THEN
   eti_Num :=TaxInq.SrcNum;
   eti_Date:=L_DatTim.DateStr(TaxInq.Date);
   eti_GNI :=P_Man.GetManProSoft(Man.Man, CurPE, P_Man.P_TaxDep)
  END;
 END {GetExpTaxInqAttr};

 PRIVATE PROCEDURE CalcGivedStdAndExpDedVal(_Scale: L_Tax.tTax2001_ScalesType;
                                    VAR GivedStdDedVal: NUMERIC[2];
                                    VAR GivedExpDedVal: NUMERIC[2]);
  VAR
   TotalStdDedVal,
   TotalExpDedVal,
   _IncVal       : NUMERIC[2];
 BEGIN
  _IncVal:=T3_Total.IncVal[_Scale];
  DEC(_IncVal, GetTaxDeductSum(_Scale));

  TotalStdDedVal:=GetStdDeductSum(_Scale);
  TotalExpDedVal:=GetExpDeductSum(_Scale);

  GivedStdDedVal:=Math.Min(_IncVal, TotalStdDedVal);
  DEC(_IncVal, TotalStdDedVal);
  GivedExpDedVal:=Math.Max(Math.Min(_IncVal, TotalExpDedVal), 0);
 END {CalcGivedStdAndExpDedVal};

 PRIVATE PROCEDURE TypePart4(_Scale: L_Tax.tTax2001_ScalesType);
  CONST
   ArrSize = 12;
   Lines   = 3;
  VAR
   se_DedCode   : LIKE(TaxAdv.VCode)[ArrSize];
   se_DedVal    : NUMERIC[2][ArrSize];
   i            : NUMERIC[0];
   SingleDedVal,
   GivedStdDedVal,
   GivedExpDedVal: NUMERIC[2];
   eti_Num,
   eti_Date,
   eti_GNI: STRING[];
 BEGIN
  Prompt(4);
  SETEMPTY(se_DedCode);
  SETZERO(se_DedVal);
  i:=0;
  SELECT(T3_MTA, ScaleMD, , _Scale, Man.Man);
  WHILE OCCURE(T3_MTA) DO
   CHANGESEL(T3_MTA, ScaleMD, , T3_MTA.Scale, T3_MTA.Man, T3_MTA.DedCode);
   WHILE OCCURE(T3_MTA) DO
    IF (T3_MTA.MdfType = L_Tax.Tax2001_mt_StD) | (T3_MTA.MdfType = L_Tax.Tax2001_mt_Exp) THEN
     INC(se_DedVal[i], T3_MTA.DedVal);
    END;
    NEXT(T3_MTA);
   END; (* WHILE OCCURE(T3_MTA) ... *)
   LAST(T3_MTA);
   IF se_DedVal[i] # 0 THEN
    se_DedCode[i]:=MNSVCode(T3_MTA.DedCode);
    INC(i);
   END;
   CHANGESEL(T3_MTA, ScaleMD, , T3_MTA.Scale, T3_MTA.Man);
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)

  MAKEPART(R, Part4_1);
  i:=0;
  WHILE i < Lines DO
   MAKEPART(R, Part4_2, se_DedCode[i],           PrintMoneyE(se_DedVal[i]),
                        se_DedCode[i + Lines],   PrintMoneyE(se_DedVal[i + Lines]),
                        se_DedCode[i + Lines*2], PrintMoneyE(se_DedVal[i + Lines*2]),
                        se_DedCode[i + Lines*3], PrintMoneyE(se_DedVal[i + Lines*3]));
   INC(i);
  END; (* WHILE i < Lines ...*)

  GetExpTaxInqAttr(_Scale, eti_Num, eti_Date, eti_GNI);

  CalcGivedStdAndExpDedVal(_Scale, GivedStdDedVal, GivedExpDedVal);

  MAKEPART(R, Part4_3, eti_Num, eti_Date, eti_GNI, PrintMoneyE(GivedStdDedVal), PrintMoneyE(GivedExpDedVal));
 END {TypePart4};

 PRIVATE PROCEDURE TypePart5(_Scale: L_Tax.tTax2001_ScalesType);
  VAR
   TaxableInc: NUMERIC[2];
   GivedStdDedVal,
   GivedExpDedVal: NUMERIC[2];
 BEGIN
  Prompt(5);
  CalcGivedStdAndExpDedVal(_Scale, GivedStdDedVal, GivedExpDedVal);
  TaxableInc:=T3_Total.IncVal[_Scale] - (GetTaxDeductSum(_Scale) + GivedStdDedVal + GivedExpDedVal);

  MAKEPART(R, Part5, PrintMoney(T3_Total.IncVal[_Scale]),    PrintMoney(TaxableInc),
                     PrintIntgMoney(T3_Total.ClcTax[_Scale]), PrintIntgMoney(T3_Total.PaidTax[_Scale]),
                     PrintIntgMoney(T3_Total.RetnTax[_Scale]),
                     PrintIntgMoney(T3_Total.OverTax[_Scale]), PrintIntgMoney(T3_Total.DedTax[_Scale]),
                     PrintIntgMoney(T3_Total.DbtTax[_Scale]),  PrintIntgMoney(T3_Total.CrdTax[_Scale]),
                     PrintIntgMoney(T3_Total.EnfTax[_Scale]));
 END {TypePart5};

 PRIVATE PROCEDURE TypePart6(_Scale : L_Tax.tTax2001_ScalesType);
 BEGIN
  MAKEPART(R, Part6, GetOKATO(), L_Str.PoolItem(KBKPool, _Scale), T3_Total.PaidTax[_Scale]);
 END {TypePart6};

 PRIVATE PROCEDURE SetDeductLimit(IncomeByMonth: tSumsByMonth; _Deduct: NUMERIC[2]): tSumsByMonth;
  VAR
   DeductLimit: tSumsByMonth;
   i          : NUMERIC[0];
 BEGIN
  SETZERO(DeductLimit);
  i:=0;
  WHILE (_Deduct > 0) & (i <= HIGH(IncomeByMonth)) DO
   IF IncomeByMonth[i] > 0 THEN
    IF IncomeByMonth[i] >= _Deduct THEN
     INC(DeductLimit[i], _Deduct);
    ELSE
     INC(DeductLimit[i], IncomeByMonth[i]);
    END;
    DEC(_Deduct, IncomeByMonth[i]);
   END;
   INC(i);
  END; (* WHILE i < ... *)
  RETURN DeductLimit;
 END {SetDeductLimit};

 PRIVATE PROCEDURE GatherSingleTAReason(_Scale  : L_Tax.tTax2001_ScalesType;
                                _MdfType: LIKE(TaxAdv.Reason));
  VAR
   IncomeByMonth,
   DeductByMonth: tSumsByMonth;
   _Deduct     : NUMERIC[2];
   _IncCode    : LIKE(TaxAdv.VCode);
   _DedCode    : LIKE(TaxAdv.LCode);
   i           : NUMERIC[0];
   Msg         : STRING[];
 BEGIN
  SELECT(_MnTA, _MnTA, , Man.Man, _MdfType);
  WHILE OCCURE(_MnTA) DO
   SAVE(_MnTA);
   L_Tax.Tax_AccessManTA_by_MnTA();
   CONFORM(ManTA, TaxAdv);
   SETZERO(IncomeByMonth, _Deduct);
   IF L_Tax.Tax2001__GetOneTaxAdv(CurP, CurP, _MdfType, TaxAdv.TaxAdv, IncomeByMonth, _Deduct, 0) THEN
    SETZERO(DeductByMonth);
    IF _MdfType = L_Tax.Tax2001_mt_Inc THEN
     IF TaxAdv.TaxAdv = 30(*Tax2001__oc_NYInc*) THEN
      _IncCode:=L_Tax.Tax2001_PureIncomVCode;
     ELSE
      _IncCode:=TaxAdv.VCode;
     END;
     _DedCode:=L_Tax.Tax2001_taGetLCode();
     IF ~!_DedCode THEN
      DeductByMonth:=SetDeductLimit(IncomeByMonth, _Deduct);
     END;
    ELSE       (**)
     _DedCode     :=TaxAdv.VCode;
     DeductByMonth:=IncomeByMonth;
     SETEMPTY(_IncCode);
     SETZERO(IncomeByMonth);
    END;

    i:=0;
    WHILE i <= HIGH(IncomeByMonth) DO
     IF (IncomeByMonth[i] # 0) | (DeductByMonth[i] # 0) THEN
      ADD(T3_MTA, ScaleM, , _Scale, Man.Man, _MdfType, i+1, _IncCode, _DedCode,
          T3_MTA.IncVal, IncomeByMonth[i],
          T3_MTA.DedVal, DeductByMonth[i]);
     END;
     INC(i);
    END; (* WHILE i <= HIGH(IncomeByMonth) *)
   END; (* IF L_Tax.Tax2001__GetOneTaxAdv ... *)

   RESTORE(_MnTA);
   CHANGESEL(_MnTA, _MnTA, , _MnTA._Man, _MnTA._MdfType, _MnTA._TaxAdv);
   LAST(_MnTA);
   CHANGESEL(_MnTA, _MnTA, , _MnTA._Man, _MnTA._MdfType);
   NEXT(_MnTA);
  END; (* WHILE OCCURE(_MnTA) ... *)
 END {GatherSingleTAReason};

 PRIVATE PROCEDURE CalcPureInc(_Scale: L_Tax.tTax2001_ScalesType);
  VAR
   IncomeByMonth: tSumsByMonth;
   i            : NUMERIC[0];
 BEGIN
  IF ~FictOnly() THEN
   SETZERO(IncomeByMonth);
   L_Tax.Tax2001_GetPureIncome(CurP, IncomeByMonth);
   i:=0;
   WHILE i <= HIGH(IncomeByMonth) DO
    IF IncomeByMonth[i] # 0 THEN
     ADD(T3_MTA, ScaleM, , _Scale, Man.Man, L_Tax.Tax2001_mt_Inc, i+1, L_Tax.Tax2001_PureIncomVCode, "",
         T3_MTA.IncVal, IncomeByMonth[i]);
    END;
    INC(i);
   END; (* WHILE i <= HIGH(IncomeByMonth) *)
  END;
 END {CalcPureInc};

 PRIVATE PROCEDURE GetSenderAttr(VAR ManName, PostName, WrkPhone: STRING[]);
 BEGIN
  ManName :=P_MyFirm.Tax3SenderName    (L_Packet.PckMyFirm, SYSDATE());
  PostName:=P_MyFirm.Tax3SenderPostName(L_Packet.PckMyFirm, SYSDATE());
  WrkPhone:=P_MyFirm.Tax3SenderPhone   (L_Packet.PckMyFirm, SYSDATE());
 END {GetSenderAttr};

 PRIVATE PROCEDURE PutManNameInXML(Node: L_XML.tNode; ManName: STRING[]);
  VAR
   Surname,
   FirstName,
   Patronymic: STRING[];
 BEGIN
  L_Str.SplitManName(ManName, Surname, FirstName, Patronymic);
  L_XML.bd_PutLeafNode(Node, "",  Surname);
  L_XML.bd_PutLeafNode(Node, "",      FirstName);
  L_XML.bd_PutLeafNode(Node, "⢮", Patronymic);
 END {PutManNameInXML};

 PRIVATE PROCEDURE MakeCommonXMLPart(VAR rootNode: L_XML.tNode; VAR GUID: STRING[]);
 BEGIN
  GUID    :=Str.Cap(CreateUUID(1));
  rootNode:=L_XML.bd_Create("2");
  L_XML.bd_SetAttribute(rootNode, "xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
  L_XML.bd_SetAttribute(rootNode, "ᔮ", "4.00001");
  L_XML.bd_SetAttribute(rootNode, "ண", " ⮏");
  L_XML.bd_SetAttribute(rootNode, "",   L_Packet.Get2NDFLFileName(GUID));

  L_XML.NeedConvertStringVal:=0;
 END {MakeCommonXMLPart};

 PRIVATE PROCEDURE MakeXML2NDFL(rootNode: L_XML.tNode;
                        _Scale  : L_Tax.tTax2001_ScalesType);
  VAR
   subNode0,
   subNode1,
   subNode2,
   subNode3,
   subNode4,
   subNode5      : L_XML.tNode;
   ForeignAddr1,
   ForeignAddr2  : STRING[];
   ForeignCountry: LIKE(Country.Country);
   _Distr,
   _Town,
   _Village,
   _Street,
   _Block        : STRING[];
   IsResident    : SCAL[2];
   _DedVal,
   TaxableInc,
   GivedStdDedVal,
   GivedExpDedVal: NUMERIC[2];
   eti_Num,
   eti_Date,
   eti_GNI: STRING[];
   FirstPass,
   AlreadyInsAttr: SCAL[2];
 BEGIN
  subNode0:=L_XML.bd_PutNode(rootNode, "ࠢ",    "");
   subNode1:=L_XML.bd_PutNode(subNode0, "鑢",   "");
    L_XML.bd_PutLeafNode(subNode1, "",    L_Str.IntStr(L_DatTim.YEAR(CurPE)));
    L_XML.bd_PutLeafNode(subNode1, "",    DocNum(_Scale));

    L_XML.bd_PutLeafNode(subNode1, "⠑",   L_DatTim.DateStr(L_Packet.PrepareDate));
    L_XML.bd_PutLeafNode(subNode1, "",    GetFirmGNI());
   subNode1:=L_XML.bd_PutNode(subNode0, "℮",   "");
    subNode2:=L_XML.bd_PutNode(subNode1, "",    "");
     L_XML.bd_PutLeafNode(subNode2, "",    P_Firm.FP_INN(Firm.Firm, CurPE));
     L_XML.bd_PutLeafNode(subNode2, "",      GetKPP());
     L_XML.bd_PutLeafNode(subNode2, "",  P_Firm.FP_FirmName(Firm.Firm, CurPE));
     L_XML.bd_PutLeafNode(subNode2, "",    GetOKATO());
     L_XML.bd_PutLeafNode(subNode2, "䮭",  P_Firm.FP_Phone(Firm.Firm, CurPE));
   subNode1:=L_XML.bd_PutNode(subNode0, "焮",   "");
    IF ~!Man.INN THEN
     L_XML.bd_PutLeafNode(subNode1, "",  Man.INN);
    END;
    subNode2:=L_XML.bd_PutNode(subNode1, "",    "");
     PutManNameInXML(subNode2, P_Man.GetManName(Man.Man, SYSDATE()));
  IsResident:=L_Tax.Tax_IsResident(CurP);
    L_XML.bd_PutLeafNode(subNode1, "",      L_Str.PoolItem("2|1", IsResident));
    L_XML.bd_PutLeafNode(subNode1, "⠐",    L_DatTim.DateStr(Man.Birth));
    L_XML.bd_PutLeafNode(subNode1, "ࠦ⢮", GetCitizenCountry());
    subNode2:=L_XML.bd_PutNode(subNode1, "",    "");
     L_XML.bd_PutLeafNode(subNode2, "",     GetManDocCodeMNS());
     L_XML.bd_PutLeafNode(subNode2, "",     Man.PspSer + " " + Man.PspNbr);
    subNode2:=L_XML.bd_PutNode(subNode1, "",    "");

  StandOnManAdrForTax(Man.Man);
  PrepareManAdressForTax(_Distr, _Town, _Village, _Street, _Block);
     L_XML.bd_PutLeafNode(subNode2, "", ManAdr.Zip);
     L_XML.bd_PutLeafNode(subNode2, "", Str.Cap(ManAdr.Region));
     L_XML.bd_PutLeafNode(subNode2, "", _Distr);
     L_XML.bd_PutLeafNode(subNode2, "த", _Town);
     L_XML.bd_PutLeafNode(subNode2, "㭪", _Village);
     L_XML.bd_PutLeafNode(subNode2, "", _Street);
     L_XML.bd_PutLeafNode(subNode2, "",  ManAdr.House);
     L_XML.bd_PutLeafNode(subNode2, "", _Block);
     L_XML.bd_PutLeafNode(subNode2, "", ManAdr.Flat);
(*
    06.03.2007 16:41:33 POLAK\San - 0056525
    GetForeignAddr(IsResident, ForeignCountry, ForeignAddr1, ForeignAddr2);
    IF ~!ForeignCountry | ~!ForeignAddr1 | ~!ForeignAddr2 THEN
     subNode2:=L_XML.bd_PutNode(subNode1, "",    "");
      L_XML.bd_PutLeafNode(subNode2, "",  ForeignCountry);
      L_XML.bd_PutLeafNode(subNode2, "", ForeignAddr1 + ForeignAddr2);
    END;
*)
   subNode1:=L_XML.bd_PutNode(subNode0, "⠢",   "");
    L_XML.bd_PutLeafNode(subNode1, "⠢", GetPrcByScale(_Scale));

  SELECT(T3_MTA, ScaleM, , _Scale, Man.Man, L_Tax.Tax2001_mt_Inc);
  WHILE OCCURE(T3_MTA) DO
   IF T3_MTA.IncVal # 0 THEN
    subNode2:=L_XML.bd_PutNode(subNode1, "",   "");
    L_XML.bd_PutLeafNode(subNode2, "",  L_Str.FixedIntStr(T3_MTA.Month, 2));
    L_XML.bd_PutLeafNode(subNode2, "室",  T3_MTA.IncCode);
    L_XML.bd_PutLeafNode(subNode2, "㬄室",  PrintMoney(T3_MTA.IncVal));
    IF T3_MTA.IncVal < 0 THEN
     MAKEPART(Err, Error_Line, Man.Man, " " + FLDTOSTR(T3_MTA.Month, "L", 2) + ": ⥫ 室.  室 " + T3_MTA.IncCode + ", 㬬 " + PrintMoney(T3_MTA.IncVal));
    END;
    IF T3_MTA.DedVal # 0 THEN
     L_XML.bd_PutLeafNode(subNode2, "",  MNSVCode(T3_MTA.DedCode));
     L_XML.bd_PutLeafNode(subNode2, "㬂",  PrintMoneyE(T3_MTA.DedVal));
    END;
   END;
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)

  AlreadyInsAttr:=0;
  FirstPass     :=1;
  SELECT(T3_MTA, ScaleMD, , _Scale, Man.Man);
  WHILE OCCURE(T3_MTA) DO
   CHANGESEL(T3_MTA, ScaleMD, , T3_MTA.Scale, T3_MTA.Man, T3_MTA.DedCode);
   SETZERO(_DedVal);
   WHILE OCCURE(T3_MTA) DO
    IF (T3_MTA.MdfType = L_Tax.Tax2001_mt_StD) THEN
     INC(_DedVal, T3_MTA.DedVal);
    END;
    NEXT(T3_MTA);
   END; (* WHILE OCCURE(T3_MTA) ... *)
   LAST(T3_MTA);
   IF _DedVal # 0 THEN
    IF FirstPass THEN
     subNode2:=L_XML.bd_PutNode(subNode1,   "⠭",   "");
     FirstPass:=0;
    END;
    subNode3:=L_XML.bd_PutNode(subNode2, "ࠢ",   "");
     L_XML.bd_PutLeafNode(subNode3, "",  MNSVCode(T3_MTA.DedCode));
     L_XML.bd_PutLeafNode(subNode3, "㬂",  PrintMoney(_DedVal));
    AlreadyInsAttr:=1;
   END;
   CHANGESEL(T3_MTA, ScaleMD, , T3_MTA.Scale, T3_MTA.Man);
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)

  CalcGivedStdAndExpDedVal(_Scale, GivedStdDedVal, GivedExpDedVal);

  IF AlreadyInsAttr THEN
   L_XML.bd_PutLeafNode(subNode2, "㬑⍠",  PrintMoney(GivedStdDedVal));
  END;

  AlreadyInsAttr:=0;
  FirstPass     :=1;
  SELECT(T3_MTA, ScaleMD, , _Scale, Man.Man);
  WHILE OCCURE(T3_MTA) DO
   CHANGESEL(T3_MTA, ScaleMD, , T3_MTA.Scale, T3_MTA.Man, T3_MTA.DedCode);
   SETZERO(_DedVal);
   WHILE OCCURE(T3_MTA) DO
    IF (T3_MTA.MdfType = L_Tax.Tax2001_mt_Exp) THEN
     INC(_DedVal, T3_MTA.DedVal);
    END;
    NEXT(T3_MTA);
   END; (* WHILE OCCURE(T3_MTA) ... *)
   LAST(T3_MTA);
   IF _DedVal # 0 THEN
    IF FirstPass THEN
     subNode2:=L_XML.bd_PutNode(subNode1,   "",   "");
     FirstPass:=0;
    END;
    subNode3:=L_XML.bd_PutNode(subNode2, "ࠢ",   "");
     L_XML.bd_PutLeafNode(subNode3, "",  MNSVCode(T3_MTA.DedCode));
     L_XML.bd_PutLeafNode(subNode3, "㬂",  PrintMoney(_DedVal));
    AlreadyInsAttr:=1;
   END;
   CHANGESEL(T3_MTA, ScaleMD, , T3_MTA.Scale, T3_MTA.Man);
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)

  IF AlreadyInsAttr THEN
   GetExpTaxInqAttr(_Scale, eti_Num, eti_Date, eti_GNI);
   IF ~!eti_Date & !eti_Num THEN
    MAKEPART(Err, Error_Line, Man.Man, "    㢥");
   END;
   L_XML.bd_PutLeafNode(subNode2, "",  eti_Num);
   L_XML.bd_PutLeafNode(subNode2, "⠓",   eti_Date);
   L_XML.bd_PutLeafNode(subNode2, "",   eti_GNI);
   L_XML.bd_PutLeafNode(subNode2, "㬈",  PrintMoney(GivedExpDedVal));
  END;
  TaxableInc:=T3_Total.IncVal[_Scale] - (GetTaxDeductSum(_Scale) + GivedStdDedVal + GivedExpDedVal);
   subNode2:=L_XML.bd_PutNode(subNode1,   "",   "");
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintMoney(T3_Total.IncVal[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintMoney(TaxableInc));
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintIntgMoney(T3_Total.ClcTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintIntgMoney(T3_Total.PaidTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintIntgMoney(T3_Total.RetnTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintIntgMoney(T3_Total.OverTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "㬬",  PrintIntgMoney(T3_Total.DedTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "",  PrintIntgMoney(T3_Total.DbtTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "",  PrintIntgMoney(T3_Total.CrdTax[_Scale]));
    L_XML.bd_PutLeafNode(subNode2, "᪈",  PrintIntgMoney(T3_Total.EnfTax[_Scale]));
 END {MakeXML2NDFL};

 PRIVATE PROCEDURE MakeInfXMLPart(rootNode: L_XML.tNode);
  VAR
   _Scale: L_Tax.tTax2001_ScalesType;
 BEGIN
  FOR_EACH _Scale DO
   IF T3_Total.IncVal[_Scale] # 0 THEN
    MakeXML2NDFL(rootNode, _Scale);
    MAKEPART(R, Register_Line, DocNum(_Scale), GetPrcByScale(_Scale), T3_Total.IncVal[_Scale], T3_Total.ClcTax[_Scale], T3_Total.PaidTax[_Scale]);
   END;
  END;
 END {MakeInfXMLPart};

 PRIVATE PROCEDURE CloseXMLFile(rootNode: L_XML.tNode);
 BEGIN
  L_XML.bd_Save(L_XML.bd_GetDocument(rootNode), L_Packet.Packet2NDFLTrnFileName());
 END {CloseXMLFile};

 PRIVATE PROCEDURE PrintSingleScale2NDFL(_Scale: L_Tax.tTax2001_ScalesType);
 BEGIN
  TypePart1(_Scale);
  TypePart2();
  TypePart3(_Scale);
  TypePart4(_Scale);
  TypePart5(_Scale);
  TypePart6(_Scale);
 END {PrintSingleScale2NDFL};

 PRIVATE PROCEDURE Print2NDFL();
  CONST
   RepDepth  = 90;
  VAR
   rootNode    : L_XML.tNode;
   GUID        : STRING[];
   _Scale      : L_Tax.tTax2001_ScalesType;
   NeedFormFeed: SCAL[2];
 BEGIN
  L_General.Assert(SEARCH(T3_Total, Man, Man.Man), "   ନ஢ ᢥ  室");
  OPENREP(R, L_General.NameConnection("Tax03"), Create);
  SetPrintAttr(R, MinLinesPerPage, RepDepth,
                  MaxPages,        1,
                  FontEnlargement, 1.2,
                  FontProperty,    "/U /B",
                  Duplex,          Off);
  NeedFormFeed:=0;
  FOR_EACH _Scale DO
   IF T3_Total.IncVal[_Scale] # 0 THEN
    IF NeedFormFeed THEN
     FORMFEED(R);
    END;
    PrintSingleScale2NDFL(_Scale);
    NeedFormFeed:=1;
   END;
  END;
  CLOSEREP(R);
 END {Print2NDFL};

 PRIVATE PROCEDURE GetIncomeSum(_Scale: L_Tax.tTax2001_ScalesType): NUMERIC[2];
  VAR
   Income: NUMERIC[2];
 BEGIN
  SETZERO(Income);
  SELECT(T3_MTA, ScaleM, , _Scale, Man.Man, L_Tax.Tax2001_mt_Inc);
  WHILE OCCURE(T3_MTA) DO
   INC(Income,     T3_MTA.IncVal);
   NEXT(T3_MTA);
  END; (* WHILE OCCURE(T3_MTA) ... *)
  RETURN Income;
 END {GetIncomeSum};

 PRIVATE PROCEDURE InsertT3_Total(_Scale: L_Tax.tTax2001_ScalesType; ClcTax, PaidTax, RetnTax, OverTax, DedTax, DbtTax, CrdTax, EnfTax: NUMERIC[2]);
  VAR
   _IncVal : NUMERIC[2];
   wReplace: SCAL[2];
 BEGIN
  wReplace:=SEARCH(T3_Total, Man, Man.Man);
  _IncVal:=GetIncomeSum(_Scale);
  IF _IncVal # 0 THEN
   IF ~wReplace THEN
    DEFAULT(T3_Total);
    T3_Total.Man           :=Man.Man;
   END;
   T3_Total.Scale  [_Scale]:=_Scale;
   T3_Total.IncVal [_Scale]:=_IncVal;
   T3_Total.ClcTax [_Scale]:=ClcTax;
   T3_Total.PaidTax[_Scale]:=PaidTax;
   T3_Total.RetnTax[_Scale]:=RetnTax;
   T3_Total.OverTax[_Scale]:=OverTax;
   T3_Total.DedTax [_Scale]:=DedTax;
   T3_Total.DbtTax [_Scale]:=DbtTax;
   T3_Total.CrdTax [_Scale]:=CrdTax;
   T3_Total.EnfTax [_Scale]:=EnfTax;
   IF ~wReplace THEN
    INSERT(T3_Total);
   ELSE
    REPLACE(T3_Total);
   END;
   INC(NDFLAmount);
  END;
 END {InsertT3_Total};

 PRIVATE PROCEDURE thni_curT3_MTA(): SCAL[2];
  VAR
   trgValue,
   decValue : NUMERIC[2];
   Result   : SCAL[2];
 BEGIN
  trgValue:=-T3_MTA.IncVal;
  CHANGESEL(T3_MTA, ScaleMI, , T3_MTA.Scale, T3_MTA.Man, T3_MTA.MdfType, T3_MTA.IncCode);
  SAVE(T3_MTA);
  LOOP
   IF trgValue = 0 THEN
    Result:=1;
    EXIT;
   ELSIF ~PREV(T3_MTA) THEN
    Result:=0;
    EXIT;
   END;
   IF (T3_MTA.IncVal > 0) & (T3_MTA.DedVal = 0) THEN
    decValue:=Math.Min(T3_MTA.IncVal, trgValue);
    DEC(trgValue, decValue);
   END;
  END; (* LOOP *)
  RESTORE(T3_MTA);
  IF Result THEN
   trgValue:=-T3_MTA.IncVal;
   T3_MTA.IncVal:=0;
   REPLACE(T3_MTA);
   LOOP
    IF trgValue = 0 THEN
     EXIT;
    ELSIF ~PREV(T3_MTA) THEN
     L_General.ConsultToUs(0, "thni_curT3_MTA.Error 123");
    END;
    IF (T3_MTA.IncVal > 0) & (T3_MTA.DedVal = 0) THEN
     decValue:=Math.Min(T3_MTA.IncVal, trgValue);
     DEC(trgValue, decValue);
     DEC(T3_MTA.IncVal, decValue);
     REPLACE(T3_MTA);
    END;
   END; (* LOOP *)
  END;
  RETURN Result;
 END {thni_curT3_MTA};

 PRIVATE PROCEDURE TryHideNegativeIncome(_Scale: L_Tax.tTax2001_ScalesType);
  VAR
   Enough: SCAL[2];
 BEGIN
  REPEAT
   Enough:=1;
   SELECT(T3_MTA, ScaleM, , _Scale, Man.Man, L_Tax.Tax2001_mt_Inc);
   LOOP
    IF ~OCCURE(T3_MTA) THEN
     EXIT;
    END;
    IF T3_MTA.IncVal < 0 THEN
     IF (T3_MTA.DedVal = 0) & thni_curT3_MTA() THEN
      Enough:=0;
      EXIT;
     ELSE
      RETURN;
     END;
    END;
    NEXT(T3_MTA);
   END; (* LOOP ... *)
  UNTIL Enough;
 END {TryHideNegativeIncome};

 PRIVATE PROCEDURE Make2NDFL(NeedInit: SCAL[2]);
  VAR
   DedTax,
   OverTax,
   DbtTax,
   CrdTax,
   RetnTax,
   PaidTax,
   EnfTax,
   ClcTax,
   ThrdScaleClcTax,
   FrthScaleClcTax : NUMERIC[2];
   MainScale       : L_Tax.tTax2001_ScalesType;
 BEGIN
  Prompt(0);
  IF NeedInit THEN
   L_Tax.Tax2001__InitValues(CurP, 0);
  ELSE
   L_Tax.Tax_FillManIsWorking(CurP);
  END;

  MainScale:=L_Tax.Tax_GetScaleForMainScale(CurP);

  L_Tax.SetTax2001_ScaleToGather(MainScale);

  IF (MainScale = L_Tax.Tax2001_MainScale) | (MainScale = L_Tax.Tax2001_ScndScale) THEN
   CalcPureInc(MainScale);
   L_Tax.SetTax2001_ScaleToGather(L_Tax.Tax2001_MainScale);  (* ᮡ 室  Tax2001_MainScale *)
   GatherSingleTAReason(MainScale, L_Tax.Tax2001_mt_Inc);
   TryHideNegativeIncome(MainScale);
  END; (* IF MainScale... *)

  ThrdScaleClcTax:=L_Tax._Tax2001_GetClcTax (CurP, CurP, FictOnly(), L_Tax.Tax2001_ThrdScale);
  FrthScaleClcTax:=L_Tax._Tax2001_GetClcTax (CurP, CurP, FictOnly(), L_Tax.Tax2001_FrthScale);
  ClcTax         :=L_Tax._Tax2001_GetClcTax (CurP, CurP, FictOnly(), L_Tax.Tax2001_EvryScale);
  PaidTax        :=L_Tax._Tax2001_GetPaidTax(CurP, CurP, FictOnly(), L_Tax.Tax2001_EvryScale);

  DEC(ClcTax,  ThrdScaleClcTax + FrthScaleClcTax);
  DEC(PaidTax, ThrdScaleClcTax + FrthScaleClcTax);

  SETZERO(EnfTax);
  L_Tax.Tax2001_GetOneType(CurP, CurP, L_Tax.Tax2001_mt_EFT, EnfTax);

  SETZERO(DbtTax, CrdTax);
  IF ClcTax <= PaidTax THEN (*   㦭 ।  ⮣,   ९ OverTax*)
   CrdTax:=PaidTax-ClcTax;
  END;

  IF L_Tax.Tax_IsResident(CurP) THEN
   GatherSingleTAReason(MainScale, L_Tax.Tax2001_mt_StD);      (* ⠭   *)
  END;
  GatherSingleTAReason(MainScale, L_Tax.Tax2001_mt_Exp);       (* ⢥   *)

  SETZERO(OverTax); (* 모뢠 ९ *)
  L_Tax.Tax2001_GetOneOwnCode(CurP, CurP, L_Tax.Tax2001__oc_PYTax, OverTax);
  L_Tax.Tax2001_GetOneOwnCode(CurP, CurP, L_Tax.Tax2001__oc_OYRec, OverTax);
  IF OverTax#0 THEN
   DEC(PaidTax, OverTax);
  END;

  SETZERO(RetnTax);
  IF PaidTax<0 THEN
   RetnTax:=-PaidTax;
   PaidTax:=0;
   IF RetnTax > OverTax THEN
    PutError(Man.Man, "'⥭' ॢ蠥 '९'");
   END;
  END;

  IF OverTax < 0 THEN
   DedTax :=-OverTax;
   OverTax:=0;
  ELSE
   DedTax :=0;
  END;

  IF PaidTax >= EnfTax THEN
   DEC(PaidTax, EnfTax);
  END;

  IF ClcTax > (PaidTax+OverTax) THEN (* ⥫騪 㦭 ।  ⮬ EnfTax,   ⮬ ९*)
   DbtTax:=ClcTax-PaidTax+OverTax;
  END;

  L_Tax.SetTax2001_ScaleToGather(L_Tax.Tax2001_ThrdScale);
  GatherSingleTAReason(L_Tax.Tax2001_ThrdScale, L_Tax.Tax2001_mt_Inc);

  L_Tax.SetTax2001_ScaleToGather(L_Tax.Tax2001_FrthScale);
  GatherSingleTAReason(L_Tax.Tax2001_FrthScale, L_Tax.Tax2001_mt_Inc);

  InsertT3_Total(MainScale,               ClcTax,          PaidTax,         RetnTax, OverTax, DedTax, DbtTax, CrdTax, EnfTax);
  InsertT3_Total(L_Tax.Tax2001_ThrdScale, ThrdScaleClcTax, ThrdScaleClcTax, 0,       0,       0,      0,      0,      0);
  InsertT3_Total(L_Tax.Tax2001_FrthScale, FrthScaleClcTax, FrthScaleClcTax, 0,       0,       0,      0,      0,      0);
 END {Make2NDFL};

(*
$P{ Register_Top:
CurP, L, "y"
GetFirmGNI(), L, ""
?, L, ""
P_Firm.FP_FirmName(Firm.Firm, CurPE), L, ""
P_Firm.FP_INN(Firm.Firm, CurPE)+"/"+GetKPP(), L, ""
?, L, ""
?, L, ""
?, L, "0"
@
                                                 
                              ᢥ  室  䨧᪨   #### 
                                               ########

: ################################################################################################
 : #####################################################################################
  : ###############################################################################
ࠢ⥫:
: ###########################################################################################
: #################################################################################################
⢮ ।⠢ 㬥⮢ (ࠢ  室): ##########

।⠢ 㬥
======================================================================================================
|   N    |, , ⢮                     |⠢|      㬬| 㬬| 㬬|
|ࠢ |                                           ||          室|     |     |
|        |                                           |      |                |᫥| 㤥ঠ|
|--------|-------------------------------------------|------|----------------|-----------|-----------|
}
*)

(*
$P{ Register_Line:
?, L, ""
P_Man.GetManName(Man.Man, SYSDATE()), L, ""
?, R, ""
?, R, "2."
?, R, "0"
?, R, "0"
@
|########|###########################################|######|################|###########|###########|
}
*)

(*
$P{ Register_Bot:
?, R, "2."
?, R, "0"
?, R, "0"
@
|-----------------------------------------------------------|----------------|-----------|-----------|
|⮣                                                      |################|###########|###########|
======================================================================================================


                                  ________________________

                          ..

 ।⠢   ࣠  "_____"_____________________ ________ .

 ਭ   ࣠       "_____"_____________________ ________ .

 ࠡ⭨  ࣠   ___________________________________________
}
*)

(*
$P{ Error_Top:
P_Firm.FP_FirmName(Firm.Firm, CurPE), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
@
########################################################################## ########## #####
===========================================================================================
|                 祭 , 祭  ᢥ  室  訡                 |
|            !  ᢥ  室  ४஢ !             |
|-----------------------------------------------------------------------------------------|
|᪮ |稭                                                                  |
|---------------|-------------------------------------------------------------------------|
}
*)

(*
$P{ Error_Line:
?, L, ""
?, L, ""
@
|###############|#########################################################################|
}
*)

(*
$P{ Error_Bot:
@
=(c) Polak=================================================================================
}
*)

 PRIVATE PROCEDURE GetPacketNum(): SCAL[50];
 BEGIN
  IF    ByCurrentPacket() THEN
   RETURN Packet.ExtNbr;
  ELSIF ToPaper() THEN
   RETURN 00;
  ELSIF FictOnly() THEN
   RETURN 09;
  ELSIF MakeForInq() THEN
   RETURN 11+L_DatTim.MONTH(CurP);
  ELSE
   RETURN 01;
  END;
 END {GetPacketNum};

 PRIVATE PROCEDURE MakePacket();
  VAR
   i             : NUMERIC[0];
   rootNode      : L_XML.tNode;
   GUID,
   ManName,
   PostName,
   WrkPhone      : STRING[];
   TotIncVal,
   TotClcTax,
   TotPaidTax    : NUMERIC[2];
 BEGIN
  L_Packet.SetDTStamp();
  L_Packet.PrepareDate:=SYSDATE();

  MakeCommonXMLPart(rootNode, GUID);
  L_Packet.PutPacketAttr(Packet.Packet, L_Packet.patt_GUID, GUID);

  OPENREP(R, L_Packet.Packet2NDFLRepFileName("R"), Create);
  GetSenderAttr(ManName, PostName, WrkPhone);
  MAKEPART(R, Register_Top, L_Packet.Get2NDFLXMLFileName(GUID), PostName, ManName, NDFLAmount);

  SETZERO(TotIncVal, TotClcTax, TotPaidTax);
  SELECT(T3_Total, Man);
  WHILE OCCURE(T3_Total) DO
   SEARCH(Man, Man, , T3_Total.Man);
   PROMPT("㧪   " + Man.Man);

   MakeInfXMLPart(rootNode);
   IF ~ByCurrentPacket() THEN
    L_General.CLEAR("PckBody");
    PckBody.Packet:=Packet.Packet;
    PckBody.SeqNbr:="";
    PckBody.Man   :=Man.Man;
    INSERT(PckBody);
   END;

   INC(TotIncVal,  L_Tax.Tax2001_GetArrSum(T3_Total.IncVal));
   INC(TotClcTax,  L_Tax.Tax2001_GetArrSum(T3_Total.ClcTax));
   INC(TotPaidTax, L_Tax.Tax2001_GetArrSum(T3_Total.PaidTax));
   NEXT(T3_Total);
  END; (* WHILE OCCURE(T3_Total) ... *)
  CloseXMLFile(rootNode);

  MAKEPART(R, Register_Bot, TotIncVal, TotClcTax, TotPaidTax);
  CLOSEREP(R);

  Packet.PrpDate:=L_Packet.PrepareDate;
  Packet.DTStamp:=L_Packet.DTStamp;
  Packet.Sums[0]:=TotIncVal;
  Packet.Sums[1]:=TotPaidTax;
  REPLACE(Packet);
 END; {MakePacket}

 PRIVATE PROCEDURE PutManInIndex();
  VAR
   ContractAmount: NUMERIC[0];
 BEGIN
  ContractAmount:=0;
  SELECT(Staff, MyFirm, , L_Packet.PckMyFirm, Man.Man);
  WHILE OCCURE(Staff) DO
   IF L_Staff.IsMainForTax(CurPE) THEN
    L_General.ConsultToUs(ContractAmount=0, "2  ࠪ");
    L_PRoll.CheckSalDone(CurP);
    Make2NDFL(1);
    INC(ContractAmount);
   END;
   NEXT(Staff);
  END; (* WHILE OCCURE(Staff) ... *)
 END {PutManInIndex};

 PRIVATE PROCEDURE Make2NDFLByAll();
 BEGIN
  SELECT(Man, Man);
  WHILE OCCURE(Man) DO
   PutManInIndex();
   NEXT(Man);
  END; (* WHILE OCCURE(Man) ... *)
  MakePacket();
 END {Make2NDFLByAll};

 PRIVATE PROCEDURE Make2NDFLByCurrentPacket();
 BEGIN
  SELECT(PckBody, Packet, , Packet.Packet);
  WHILE OCCURE(PckBody) DO
   CONFORM(PckBody, Man);
   PutManInIndex();
   NEXT(PckBody);
  END; (* WHILE OCCURE(PckBody) ... *)
  MakePacket();
 END {Make2NDFLByCurrentPacket};

 PRIVATE PROCEDURE CheckForInq(_InqType: LIKE(TaxInq.InqType); _TaxAdv: LIKE(TaxAdv.TaxAdv)): SCAL[2];
 BEGIN
  SETSEL(TaxInq, MyFirm, L_Packet.PckMyFirm, Man.Man, _InqType);
  SRCINSEL(TaxInq, GE, , L_DatTim.YearBeg(CurP));
  WHILE OCCURE(TaxInq) & (TaxInq.Date < CurPE) DO
   CONFORM(TaxInq, ManTA);
   WHILE OCCURE(ManTA) DO
    IF ManTA.TaxAdv = _TaxAdv THEN
     RETURN 1;
    END;
    NEXT(ManTA);
   END; (* WHILE OCCURE(ManTA) ... *)
   NEXT(TaxInq);
  END; (* WHILE OCCURE(TaxInq) *)
  RETURN 0;
 END {CheckForInq};

 PRIVATE PROCEDURE Make2NDFLByAllHasInq();
  VAR
   _TaxAdv_oc_EFTax: LIKE(TaxAdv.TaxAdv);
 BEGIN
  L_Tax.Tax2001_InitCodes(L_DatTim.YearBeg(CurP));
  L_Tax.Tax_StandOnSingleSpecifiedTaxAdv(L_DatTim.YearBeg(CurP), L_Tax.Tax2001__oc_EFTax, "");
  _TaxAdv_oc_EFTax:=TaxAdv.TaxAdv;

  SELECT(Man, Man);
  WHILE OCCURE(Man) DO
   PROMPT("⮢  " + Man.Man);
   IF CheckForInq(L_Tax.inqt_Own, _TaxAdv_oc_EFTax) | CheckForInq(L_Tax.inqt_Auto, _TaxAdv_oc_EFTax) THEN
    PutManInIndex();
   END;
   NEXT(Man);
  END; (* WHILE OCCURE(Man) ... *)
  MakePacket();
 END {Make2NDFLByAllHasInq};

 PRIVATE PROCEDURE Prepare2NDFLList(Proc: PROC);
 BEGIN
  IF ~ByCurrentPacket() THEN
   L_Packet.InsIncTaxPacket(L_Str.IntStr(GetPacketNum()), EMPTYOF(MyPlace.MyPlace), L_DatTim.YearBeg(CurP), CurPE);
  END;

  OPENREP(Err, L_Packet.Packet2NDFLRepFileName("E"), Create);
  MAKEPART(Err, Error_Top);
  Proc();
  MAKEPART(Err, Error_Bot);
  CLOSEREP(Err);
 END {Prepare2NDFLList};

 PUBLIC PROCEDURE T3_2005(trgMode: tMode);
 BEGIN
  curMode:=trgMode;
  IF curMode > mode_All THEN
   L_StdFrm.BlueStop("    ⮣ ਮ");
  END;
  Create_T3_MTA();
  IF    ToPaper() THEN
   L_General.Assert(ACCESS(Staff), " ⠭ ⥪騩 ࠡ⭨");
   L_Packet.DefinePckMyFirm(L_Packet.bm_byStaff);
  ELSIF ByCurrentPacket() THEN
   L_General.Assert(ACCESS(Packet), " ⠭ ");
   L_Packet.DefinePckMyFirm(L_Packet.bm_byPacket);
  ELSE
   L_Packet.DefinePckMyFirm(L_Packet.bm_byAll);
  END;
 
  IF ByCurrentPacket() THEN
   L_DatTim.CheckWMIntvl(Packet.PerB, Packet.PerE);
   L_General.Assert(SELECT(PckBody, Packet, Packet.Packet), " ");
   CurP :=L_DatTim.MonthBeg(Packet.PerE);
   CurPE:=Packet.PerE;
  ELSE
   CurP :=Memo.ud_mb();
   CurPE:=Memo.ud_me();
  END;
 
  CheckPer(L_Tax.E_NeedOtherFirmToT3(L_DatTim.YearBeg(CurP)));
  IF ~FictOnly() THEN
   P_Firm.AccessFirm(P_Firm.fomf(L_Packet.PckMyFirm));
  ELSE
   P_Firm.AccessFirm(P_OwnData.GetOwnDProStrong(CurPE, P_OwnData.P_TradeUnionFirm));
  END;
 
  NDFLAmount:=0;
  IF    ToPaper() THEN
   L_PRoll.CheckSalDone(CurP);
   L_Staff.CheckCurrentAsMainForTax(CurPE);
   CONFORM(Staff, Man);
   Make2NDFL(1);
   Print2NDFL();
  ELSIF AllMode() THEN
   Prepare2NDFLList(PROCREF(Make2NDFLByAll));
  ELSIF ByCurrentPacket() THEN
   Prepare2NDFLList(PROCREF(Make2NDFLByCurrentPacket));
  ELSIF MakeForInq() THEN
   Prepare2NDFLList(PROCREF(Make2NDFLByAllHasInq));
  END;
 
  L_Tax.Tax_FreeValues(CurP);
  L_AFM.AF_PurgeTemp("T3_MTA");
  L_AFM.AF_PurgeTemp("T3_Total");
 
  IF ToPaper() THEN
   L_General.SurveyLastReport();
  ELSE
   L_Packet.ShowPacketFiles(L_Packet.fp_Prn);
  END;
 END {T3_2005};

END.
