(*
%Copyright   (c) Polak IT
%Author      21.08.2008 12:33:28 POLAK\San
%Purpose      ᮡ⨩ 㬥⮮
%Type        control
%Specific    SYS
%Modified    19.02.2013 19:27:14 POLAK\Bor
%Environment
*)

 MODULE XDocFlow [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_XDF, L_XDoc, L_StdFrm, L_Str, P_Man, L_DatTim, P_Firm, L_Events, L_WDO,
        L_Staff, Str, L_OwnData, Memo, L_AFM, L_Man, L_Prop, L_Pool, L_Msg, L_lsStore, L_APtm,
        L_SysLog, L_AR, L_APAR;

 PRIVATE PERMANENT VAR
  xdf_Action: LIKE(XDocFlow.Action);

 PRIVATE PROCEDURE GetXDocMyFirm(File: STRING[]): P_Firm.tMyFirm;
 BEGIN
  TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
   CONFORM(^File, MyFirm);
   RETURN MyFirm.MyFirm;
  EXCEPT
  ELSE
   RETURN EMPTYOF(P_Firm.tMyFirm);
  END; (* TRY *)
 END {GetXDocMyFirm};

 PUBLIC PROCEDURE ShowAction();
 BEGIN
  L_XDoc.LaunchViewingServiceFile("XDocFlow");
 END {ShowAction};

 PRIVATE PROCEDURE GetSexSuffix(): STRING[];
 BEGIN
  RETURN L_Str.PoolItem(" | ", Man.Sex);
 END {GetSexSuffix};

 PRIVATE PROCEDURE AccessByRI(fi, ri: NUMERIC[0]);
 BEGIN
  SEARCH((DATABASE.Get().TableById(fi).Name), , RECORDID, ri);
 END {AccessByRI};

 PRIVATE PROCEDURE GetDescriptionOfTo(): STRING[];
 BEGIN
  IF ~!XDocFlow.To THEN
   SEARCH(AcsObj, AcsObj, , XDocFlow.To);
   IF AcsObj.Type = L_AR.aot_User THEN
    SEARCH(Staff, TSN, , L_APAR.User2TSN(SYSDATE(), XDocFlow.To));
    CONFORM(Staff, Man);
    RETURN "ࠡ⭨ " + Staff.TSN + " " + Man.Name;
   ELSE
    RETURN "㯯 " + AcsObj.FullName;
   END;
  END;
 END {GetDescriptionOfTo};

 PRIVATE PROCEDURE ToIs2AcsObj(trgAcsObj: LIKE(AcsObj.AcsObj)): SCAL[2];
 BEGIN
  RETURN ~!XDocFlow.To & ((XDocFlow.To = trgAcsObj) | SEARCH(AOMem, AcsObj, XDocFlow.To, trgAcsObj));
 END {ToIs2AcsObj};

 PRIVATE PROCEDURE GetActInfo(): STRING[];
  VAR
   Msg,
   Act,
   Sex: STRING[];
 BEGIN
  SEARCH(Staff, TSN, , XDocFlow.Author);
  CONFORM(Staff, Man);
  Msg:=Man.Name + " (" + Staff.TSN + ")";

  Act:=GetAction();
  Sex:=GetSexSuffix();
  Msg:=Msg + "||" + Str.Small(Act) +
             "||" + L_XDF.XDocFullDescription(XDocFlow.XDocId);
  SEARCH(AcsObj, AcsObj, , XDocFlow.Executor);
  IF ~!XDocFlow.OnBehalf THEN
   SEARCH(Staff, TSN, , XDocFlow.OnBehalf);
   CONFORM(Staff, Man);
   INC(Msg, "||   ࠡ⭨||" + Man.Name + " (" + XDocFlow.Author + ")");
  END;
  IF XDocFlow.Action = L_XDF.a_Permission THEN
   IF ~!XDocFlow.To THEN
    INC(Msg, ".||।" + Sex + "㬥  ᯮ||" + GetDescriptionOfTo());
   END;
   IF ~!XDocFlow.RslTxt THEN
    Msg:=Msg + "|| १樥 " + CHR(34) +  XDocFlow.RslTxt + CHR(34);
   END;
   IF ~!XDocFlow.TrgDate THEN
    INC(Msg, "  " + L_DatTim.DateStr(XDocFlow.TrgDate));
   END;
  ELSIF XDocFlow.Action = L_XDF.a_HandElCopy THEN
   INC(Msg, "||" + GetDescriptionOfTo());
  END;
  RETURN Msg
 END {GetActInfo};

 PRIVATE PROCEDURE NeedNotify(Action: L_XDF.tAction): SCAL[2];
 BEGIN
  RETURN (Action = L_XDF.a_Permission  ) |
         (Action = L_XDF.a_Agreement   ) |
         (Action = L_XDF.a_HandElCopy  );
 END {NeedNotify};

 TYPE
  tTextType = SCAL[2];

 CONST
  tt_NotifyMessage   = 0;
  tt_AboutHasMessage = 1;

 PRIVATE PROCEDURE curXDF_Text(TextType: tTextType): STRING[];
  VAR
   Description,
   Result     : STRING[];
 BEGIN
  L_General.CallUs((TextType = tt_NotifyMessage) | (TextType = tt_AboutHasMessage));
  L_General.CallUs(NeedNotify(XDocFlow.Action));
  Description:=L_XDF.XDocFullDescription(XDocFlow.XDocId);
  IF    XDocFlow.Action = L_XDF.a_Permission   THEN
   Result:=L_Str.PoolItem("|㦥 ", TextType) + " ।  ᯮ 㬥" + L_Str.CRLF + Description;
   IF ~!XDocFlow.RslTxt THEN
    INC(Result, L_Str.CRLF + " १樥 '" +  XDocFlow.RslTxt + "'");
   END;
   IF ~!XDocFlow.TrgDate THEN
    INC(Result, "  " + L_DatTim.DateStr(XDocFlow.TrgDate));
   END;
  ELSIF XDocFlow.Action = L_XDF.a_Agreement    THEN
   SAVE(Staff);
   SAVE(Man);
   SEARCH(Staff, TSN, , XDocFlow.Author);
   CONFORM(Staff, Man);
   Result:=L_Str.PoolItem("|㦥 ", TextType) + " । 㬥" +
           L_Str.CRLF + Description + "," +
           L_Str.CRLF + "ᮣᮢ ࠡ⭨" +
           L_Str.CRLF + Man.Name + " (" + Staff.TSN + ")";
   RESTORE(Man);
   RESTORE(Staff);
  ELSIF XDocFlow.Action = L_XDF.a_HandElCopy   THEN
   Result:=L_Str.PoolItem("   㬥⮬|㦥 뫠 । ஭  㬥", TextType) +
           L_Str.CRLF + Description;
  ELSE
   L_General.CallUs(0);
  END;
  RETURN Result + L_Str.CRLF + "(⢨ 믮 " + FLDTOSTR(XDocFlow.SignDTS, "D.N.y H:M:S", 0) + ")";
 END {curXDF_Text};

 PRIVATE PROCEDURE GetMyFirm(): P_Firm.tMyFirm;
 BEGIN
  L_General.CallUs(L_XDoc.StandOnXDoc(XDocFlow.XDocId));
  RETURN GetXDocMyFirm(L_XDoc.Ref_FileName(L_XDoc.ExtractXDocType(XDocFlow.XDocId)));
 END {GetMyFirm};

 PRIVATE PROCEDURE GetAction(): STRING[];
  VAR
   Sex: STRING[];
 BEGIN
  Sex:=GetSexSuffix();
  IF    XDocFlow.Action = L_XDF.a_Permission     THEN RETURN "訫"   + Sex + " ᯮ";
  ELSIF XDocFlow.Action = L_XDF.a_Agreement      THEN RETURN "ᮢ" + Sex;
  ELSIF XDocFlow.Action = L_XDF.a_HandOriginal   THEN RETURN "稫"    + Sex + "ਣ 㬥";
  ELSIF XDocFlow.Action = L_XDF.a_HandCopy       THEN RETURN "稫"    + Sex + " 㬥";
  ELSIF XDocFlow.Action = L_XDF.a_Acquaint       THEN RETURN "" + Sex + "c 㬥⮬";
  ELSIF XDocFlow.Action = L_XDF.a_Implementation THEN RETURN "ᯮ"   + Sex;
  ELSIF XDocFlow.Action = L_XDF.a_HandElCopy     THEN RETURN "।"    + Sex + ".";
  ELSIF XDocFlow.Action = L_XDF.a_AcquaintElCopy THEN RETURN "" + Sex + " .";
  ELSIF XDocFlow.Action = L_XDF.a_SetAuthor      THEN RETURN "⠭" + Sex + " ⢥ ";
  ELSE
   L_General.CallUs(0);
  END;
 END {GetAction};

 PRIVATE PROCEDURE XDFE_0_Setup_Action();
 BEGIN
  IF ~!XDocFlow.OnBehalf THEN
   L_WDO.cField("TrueAuthor").Value:=XDocFlow.OnBehalf;
  ELSE
   L_WDO.cField("TrueAuthor").Value:=XDocFlow.Author;
  END;
  SEARCH(Staff, TSN, , L_WDO.cField("TrueAuthor").Value);
  CONFORM(Staff, Man);
  RTI.Get().CurZone.Fields("ActionAsStr").Value:=GetAction();
 END {XDFE_0_Setup_Action};

 PRIVATE PROCEDURE XDFE_0_Setup_To();
  VAR
   asAcsObjType: SCAL[MAXOF(L_AR.tAcsObjType) + 1 + 1];
 BEGIN
  SETEMPTY(L_WDO.cField("ToTSN").Value, L_WDO.cField("ToGroup").Value);
  IF ~!XDocFlow.To THEN
   SEARCH(AcsObj, AcsObj, , XDocFlow.To);
   asAcsObjType:=AcsObj.Type;
   IF asAcsObjType = L_AR.aot_User THEN
    L_WDO.cField("ToTSN").Value:=L_APAR.User2TSN(SYSDATE(), AcsObj.AcsObj);
   ELSE
    L_WDO.cField("ToGroup").Value:=XDocFlow.To;
   END;
  ELSE
   asAcsObjType:=MAXOF(L_AR.tAcsObjType) + 1;
  END;
  L_WDO.cField("ToTSN"  ).Modifiable:=(asAcsObjType = L_AR.aot_User ) | (asAcsObjType = MAXOF(asAcsObjType));
  L_WDO.cField("ToGroup").Modifiable:=(asAcsObjType = L_AR.aot_Group) | (asAcsObjType = MAXOF(asAcsObjType));
 END {XDFE_0_Setup_To};

 PRIVATE PROCEDURE XDFE_0_SetupFields();
 BEGIN
  L_WDO.cField("XDocFlow.OnBehalf").Visible:=L_XDF.IsUserSelfAction(XDocFlow.Action);
  L_WDO.cField("OnBehalfName"     ).Visible:=L_WDO.cField("XDocFlow.OnBehalf").Visible;
  L_WDO.cField("ToTSN"            ).Visible:=(XDocFlow.Action = L_XDF.a_Permission) | (XDocFlow.Action = L_XDF.a_Agreement) | (XDocFlow.Action = L_XDF.a_HandElCopy);
  L_WDO.cField("ToTSNName"        ).Visible:=L_WDO.cField("ToTSN").Visible;
  L_WDO.cField("ToTSNTitle"       ).Visible:=L_WDO.cField("ToTSN").Visible;
  L_WDO.cField("ToGroup"          ).Visible:=L_WDO.cField("ToTSN").Visible;
  L_WDO.cField("ToGroupName"      ).Visible:=L_WDO.cField("ToGroup").Visible;
  L_WDO.cField("ToGroupTitle"     ).Visible:=L_WDO.cField("ToGroup").Visible;
  L_WDO.cField("XDocFlow.RslTxt"  ).Visible:=XDocFlow.Action = L_XDF.a_Permission;
  L_WDO.cField("RslTxtTitle"      ).Visible:=L_WDO.cField("XDocFlow.RslTxt").Visible;
  L_WDO.cField("XDocFlow.TrgDate" ).Visible:=(XDocFlow.Action = L_XDF.a_Permission) | (XDocFlow.Action = L_XDF.a_HandElCopy);
  L_WDO.cField("TrgDateTitle"     ).Visible:=L_WDO.cField("XDocFlow.TrgDate").Visible;
  IF ~!XDocFlow.To THEN
   PROMPT(GetDescriptionOfTo());
  END;
  XDFE_0_Setup_Action();
  XDFE_0_Setup_To();
 END {XDFE_0_SetupFields};

 PRIVATE PROCEDURE DoAction(_Action: LIKE(XDocFlow.Action));
  VAR
   Dummy: L_XDoc.tXDocId;
 BEGIN
  Dummy:=L_XDoc.GetCurrentXDocId();
  xdf_Action:=_Action;
  LIKEMENU(Ins,  "XDFE 0 " + RTI.Get().CurWindow.Name + " " + L_Str.IntStr(RTI.Get().CurZone.Number));
 END {DoAction};

 PRIVATE CONST
  fuckedNoMaster = "AcCut";

 PRIVATE PROCEDURE XDFE_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, [L_WDO.e_OnAnyModification, L_WDO.e_OnReposition]);
  L_WDO.iField       (0, "XDoc_Description").LineBreakEnable:=1;
  L_WDO.iField       (0, "XDocFlow.OnBehalf").First:=1;
  L_WDO.iField       (0, "XDocFlow.AuthKind").Format:=L_XDF.ak_Pool;
  L_WDO.iField       (0, "XDocFlow.RslTxt").Template:=L_Pool.Get(L_XDF.pool_XDocFlowRslTxt);
  L_WDO.iScsChange   (0, ["XDocFlow.OnBehalf", "ToGroup", "ToTSN"]);
  L_WDO.iField       (0, "ToGroup").OwnerWindowOriginalName:="aoGroup";

  L_WDO.iModifiable  (0, ["XDocFlow.SrcDate", "XDocFlow.OnBehalf",
                          "XDocFlow.RslTxt", "XDocFlow.TrgDate",
                          "ToTSN", "ToGroup"]);
  L_WDO.iMayBeEmpty  (0, ["XDocFlow.OnBehalf", "ToGroup", "ToTSN"]);
  L_WDO.CreateFormLink_Simple("Staff",  "TrueAuthor");
  L_WDO.CreateFormLink_Simple("Staff",  "ToTSN");
  L_WDO.CreateFormLink_Simple("AcsObj", "ToGroup");
 END {XDFE_OnInitializeWindow};

 PRIVATE PROCEDURE XDFE_0_OnReposition();
 BEGIN
  L_WDO.cField("XDoc_Description").Value:=L_XDF.XDocFullDescription(XDocFlow.XDocId);
  XDFE_0_SetupFields();
 END {XDFE_0_OnReposition};

 PRIVATE PROCEDURE XDF_OnConformToWindow();
 BEGIN
  IF SELECT(XDocFlow, XDocFlow, L_XDoc.GetCurrentXDocId()) THEN
   LAST(XDocFlow);
  END;
  IF SELECT(XDFCase, XDocLM, L_XDoc.GetCurrentXDocId()) THEN
   LAST(XDocFlow);
  END;
SELECT((fuckedNoMaster), (fuckedNoMaster));
SE_WWD_CURSOR(RTI.Get().TrgWindow.Name, (fuckedNoMaster));
  SE_WZD_CURSOR(RTI.Get().TrgWindow.Name, 1, XDocFlow);
  SE_WZD_CURSOR(RTI.Get().TrgWindow.Name, 2, XDFCase);
  RTI.Get().TrgWindow.Zones(0).Fields("XDoc_Description").Value:=L_XDF.XDocFullDescription(L_XDoc.GetCurrentXDocId());
 END {XDF_OnConformToWindow};

 PRIVATE PROCEDURE XDFE_OnConformToWindow();
 BEGIN
  IF SELECT(XDocFlow, XDocFlow, L_XDoc.GetCurrentXDocId()) THEN
   LAST(XDocFlow);
  END;
  SE_WWD_CURSOR(RTI.Get().TrgWindow.Name, XDocFlow);
  SE_WZD_CURSOR(RTI.Get().TrgWindow.Name, 0, XDocFlow);
  RTI.Get().TrgWindow.Zones(0).Fields("XDoc_Description").Value:=L_XDF.XDocFullDescription(L_XDoc.GetCurrentXDocId());
 END {XDFE_OnConformToWindow};

 PRIVATE PROCEDURE XDF_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (0, "");
  L_WDO.iField       (0, "XDoc_Description").LineBreakEnable:=1;
  L_WDO.iEvents      (1, "");
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iField       (1, "XDocFlow.Action").Format  :=L_XDF.a_Pool_2;
  L_WDO.iSubwindow   (1, "XDFE");
  L_WDO.iProgs       (1, "XDocFlow");
  L_WDO.iZone        (2).ViewOnly:=1;
(*
  L_WDO.iZone        (2).Narrowing:="XDFCase.XDocId=XDocFlow.XDocId";
*)
  L_WDO.iField       (2, "XDFCase.ToDo").Format:=L_XDF.ctd_Pool;

  L_WDO.iWindow().SetMaster(TABLE.Get(fuckedNoMaster));
  L_WDO.iZone        (0).SetMaster(TABLE.Get(fuckedNoMaster));
  L_WDO.iZone        (1).ParentZone:=L_WDO.iZone(0);
  L_WDO.iZone        (2).ParentZone:=L_WDO.iZone(0);
 END {XDF_OnInitializeWindow};

 PRIVATE PROCEDURE aoXDF_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().MNUFileName:="XDFCase";
  L_WDO.iZone        (1).ViewOnly:=1;
  L_WDO.iZone        (1).Narrowing:="(AcsObj.Type=" + L_Str.IntStr(L_AR.aot_Group) + ")&(AcsObj.inXDF=1)";
  L_WDO.iZone        (2).ViewOnly:=1;
  L_WDO.iZone        (2).Narrowing:="(AOMem.AcsObj=AcsObj.AcsObj)";
 END {aoXDF_OnInitializeWindow};

 PRIVATE PROCEDURE XDocFlow_OnBeforeInsert();
 BEGIN
  XDocFlow.Executor:=L_General.curUSER;
  XDocFlow.Action  :=xdf_Action;
  XDocFlow.SrcDate :=SYSDATE();
  IF (XDocFlow.Action = L_XDF.a_HandElCopy) THEN
   XDocFlow.TrgDate:=SYSDATE()+1;
  END;
  IF L_XDF.IsUserSelfAction(XDocFlow.Action) THEN
   XDocFlow.AuthKind:=L_XDF.ak_Internal;
   SEARCH(Man, Man, , P_Man.curUSER2Man());
   XDocFlow.Author:=L_Staff.Man2TSN(SYSDATE(), Man.Man, EMPTYOF(P_Firm.tMyFirm), 0, 1);
  ELSE
   XDocFlow.AuthKind:=L_XDF.E_XDF_AuthenticationKind();
   IF    XDocFlow.AuthKind = L_XDF.ak_PassCard THEN
    CALL(Staff.AuthoriseByProximityCard("믮 ⢨  㬥⮬"));
    XDocFlow.Author:=L_Staff.Man2TSN(SYSDATE(), Man.Man, EMPTYOF(P_Firm.tMyFirm), 0, 1);
   ELSIF XDocFlow.AuthKind = L_XDF.ak_BarCode THEN
    L_XDoc.PositionOnNonEmptyXDoc(L_XDoc.GetRequiredTypeXDocIdByBC("믮 ⢨  㬥⮬", L_XDoc.xdt_Staff, 0));
    XDocFlow.Author:=Staff.TSN;
    CONFORM(Staff, Man);
   ELSE
    L_General.CallUs();
   END;
  END;
  L_General.Assert(~!XDocFlow.Author, " । ⠡  " + Man.Man);
  XDocFlow.SignDTS:=L_DatTim.GetSysTIMESTAMP();
  REPLACE(XDocFlow);
  FIXALTERING();
  XDFE_0_SetupFields();
 END {XDocFlow_OnBeforeInsert};

 PRIVATE PROCEDURE XDocFlow_OnBeforeEdit();
 BEGIN
  L_General.Error("䨪 饭.  ⮫쪮 㤠   ᫥ ⢨");
 END {XDocFlow_OnBeforeEdit};

 PRIVATE PROCEDURE XDocFlow_OnBeforeDelete();
  VAR
   curFile  : STRING[];
 BEGIN
  SAVE(XDocFlow);
  CHANGESEL(XDocFlow, XDocFlow, , XDocFlow.XDocId);
  IF NEXT(XDocFlow) THEN
   L_General.Error("᪠ 㤠 ⮫쪮 ᫥  ६ ");
  END;
  RESTORE(XDocFlow);
  IF XDocFlow.Executor # L_General.curUSER THEN
   L_General.Error("   ⢨, 믮 㣨 짮⥫");
  ELSE
   L_StdFrm.Choice(GetActInfo() + L_StdFrm.HR() + "⬥ ⢨ (㤠   )?");
   IF NeedNotify(XDocFlow.Action) & ~!XDocFlow.To THEN
    curFile  :=L_XDoc.Ref_FileName(L_XDoc.ExtractXDocType(XDocFlow.XDocId));
    ACCESS((curFile));
    SETSEL(Message, RecordId, TABLE.Get(curFile).Id, ^(TABLE.Get(curFile).Fields("RECORDID").ObtainName()));
    SRCINSEL(Message, GE, , XDocFlow.SignDTS);
    LOOP
     IF    ~OCCURE(Message) THEN
      EXIT;
     ELSIF L_Str.IsStrInStr(L_lsStore.Get(Message.Text), curXDF_Text(tt_NotifyMessage)) THEN
      IF    Message.Status = L_Msg.ms_Unread THEN
       REMOVE(Message);
      ELSIF Message.Status = L_Msg.ms_Read   THEN
       L_General.Error(" ⬥ ᫥ ⢨,|" +
                       "..  ⮬ ⢨ ᮧ ᮮ饭   㦥 ⠭");
      ELSE
       L_General.CallUs(0);
      END;
     ELSE
      NEXT(Message);
     END;
    END; (* LOOP *)
   END; (* IF NeedNotify ... *)
  END; (* IF XDocFlow.Executor = L_General.curUSER ... *)
 END {XDocFlow_OnBeforeDelete};

 PRIVATE PROCEDURE NotifySingleUser();
  VAR
   curXDFNotifyMsg,
   Msg,
   curFile          : STRING[];
   curXDocId        : L_XDoc.tXDocId;
   AcqSignDTS,
   curSignDTS       : LIKE(XDocFlow.SignDTS);
   curRI            : LIKE(Message.RecordId);
   curAcsObj        : LIKE(AcsObj.AcsObj);
   curAction        : LIKE(XDocFlow.Action);
 BEGIN
  curAcsObj :=AcsObj.AcsObj;
  curSignDTS:=XDocFlow.SignDTS;
  curAction :=XDocFlow.Action;
  curXDocId :=XDocFlow.XDocId;
  curFile   :=L_XDoc.Ref_FileName(L_XDoc.ExtractXDocType(curXDocId));
  L_General.CallUs(L_XDoc.StandOnXDoc(curXDocId));
  curRI     :=^(TABLE.Get(curFile).Fields("RECORDID").ObtainName());
  SETSEL(Message, RecordId, TABLE.Get(curFile).Id, curRI);
  curXDFNotifyMsg:=curXDF_Text(tt_NotifyMessage);
  TRY
   RTI.Get().TryBlock.AccessDataMode:=adm_FINISHED;
   SETSEL(XDocFlow, XDocFlow, curXDocId);
   SRCINSEL(XDocFlow, LT, , curSignDTS);
   LOOP
    IF    ~OCCURE(XDocFlow) THEN
     EXIT;
    ELSIF XDocFlow.Action = curAction THEN
     IF (XDocFlow.Action = L_XDF.a_HandElCopy) &
        ToIs2AcsObj(curAcsObj) &
        L_XDF.HasAcquaintElCopyByCurHandElCopy(XDocFlow.XDocId, curAcsObj, AcqSignDTS) THEN
      L_StdFrm.Choice("짮⥫ " + AcsObj.FullName + "|" +
                      "㦥  " + L_DatTim.TimestampStr(AcqSignDTS) + "|" +
                      "c ஭  㬥|" +
                      L_XDF.XDocFullDescription(XDocFlow.XDocId) + "." +
                      "|| । ᭮?");
      EXIT;
     END; (* IF (XDocFlow.Action = L_XDF.a_HandElCopy) ... *)
     SRCINSEL(Message, GE, , XDocFlow.SignDTS);
     LOOP
      IF    ~OCCURE(Message) THEN
       EXIT;
      ELSIF (Message.Type = L_Msg.mt_Org) & (Message.Rsp = curAcsObj) &
            L_Str.IsStrInStr(L_lsStore.Get(Message.Text), curXDFNotifyMsg) THEN
       Msg:="짮⥫ " + AcsObj.FullName + "|" +
            curXDF_Text(tt_AboutHasMessage)            + "|" +
            "  㢥  ⮬ " + L_DatTim.TimestampStr(Message.DTS) + " ᮮ饭 " + Message.Message +".|" +
            L_Str.PoolItem(" |,      ஭ .", XDocFlow.Action = L_XDF.a_HandElCopy);
       IF    Message.Status = L_Msg.ms_Unread THEN
        Msg:=Msg + "||" + "饭   ⠭";
       ELSIF Message.Status = L_Msg.ms_Read   THEN
        Msg:=Msg + "||" + "饭 㦥 ⠭";
       ELSE
        L_General.CallUs(0);
       END;
       L_General.Error(Msg);
      END;
      NEXT(Message);
     END; (* LOOP *)
    END; (* IF ~OCCURE(XDocFlow) ... *)
    PREV(XDocFlow);
   END; (* LOOP *)
   L_General.RaiseLockReleaseException();
  EXCEPT
  ON Exc: Exc_Root DO
   L_General.CheckIsLockReleaseException(Exc);
  END;
  L_Msg.MakeAnyRecordMsg(L_Msg.mt_Org, XDocFlow.Executor, curAcsObj, L_Msg.mi_Normal, curXDFNotifyMsg,
                         SYSTIMESTAMP(), TABLE.Get(curFile).Id, curRI);
  L_Msg.NotifyCurMessage(1);
 END {NotifySingleUser};

 PRIVATE PROCEDURE XDocFlow_OnAfterInsert();
  VAR
   SuccessCnt: NUMERIC[0];
 BEGIN
  IF (xdf_Action = L_XDF.a_Permission) | (xdf_Action = L_XDF.a_HandElCopy) THEN
   IF !XDocFlow.To THEN
    L_General.Error(" 㪠  '।'");
   END;
  END;
  IF NeedNotify(XDocFlow.Action) & ~!XDocFlow.To THEN
   SEARCH(AcsObj, AcsObj, , XDocFlow.To);
   IF AcsObj.Type = L_AR.aot_User THEN
    NotifySingleUser();
   ELSE
    SuccessCnt:=0;
    SELECT(AOMem, AcsObj, , XDocFlow.To);
    WHILE OCCURE(AOMem) DO
     SEARCH(AcsObj, AcsObj, , AOMem.AcsObj2);
     IF AcsObj.Till > SYSDATE() THEN
      TRY
       NotifySingleUser();
       INC(SuccessCnt);
      EXCEPT
      ON Exc: Exc_Raise DO
       L_StdFrm.RedInform(Exc.MsgOriginalText);
      END;
     END;
     NEXT(AOMem);
    END; (* WHILE OCCURE(AOMem) ... *)
    L_General.Assert(SuccessCnt > 0, " ⠢ 㯯  ,   ᫥ 㢥");
   END; (* IF AcsObj.Type = L_AR.aot_User *)
  END; (* IF NeedNotify(XDocFlow.Action) ... *)
  L_XDF.PutFlowInCase();
  L_StdFrm.TimedINFORM(Str.StdFormEscape(GetActInfo()), 10);
 END {XDocFlow_OnAfterInsert};

 PRIVATE PROCEDURE XDocFlow_OnBehalf_OnSuccessfulChange();
 BEGIN
  IF ~!XDocFlow.OnBehalf THEN
   SEARCH(Staff, TSN, , XDocFlow.OnBehalf);
   L_General.Assert(L_Staff.rInState(SYSDATE()), "⭨  ⮨    " + L_DatTim.DateStr(SYSDATE()));
  END;
  XDFE_0_Setup_Action();
 END {XDocFlow_OnBehalf_OnSuccessfulChange};

 PRIVATE PROCEDURE XDocFlow_ToGroup_OnSuccessfulChange();
 BEGIN
  IF ~!L_WDO.cField("ToGroup").Value THEN
   SEARCH(AcsObj, AcsObj, , L_WDO.cField("ToGroup").Value);
   L_General.Assert(AcsObj.Type = L_AR.aot_Group, "  㪠뢠 짮⥫. 쪮 㯯");
   XDocFlow.To:=AcsObj.AcsObj;
  ELSE
   SETEMPTY(XDocFlow.To);
  END;
  REPLACE(XDocFlow);
  XDFE_0_Setup_To();
 END {XDocFlow_ToGroup_OnSuccessfulChange};

 PRIVATE PROCEDURE XDocFlow_ToTSN_OnSuccessfulChange();
 BEGIN
  IF ~!L_WDO.cField("ToTSN").Value THEN
   XDocFlow.To:=L_APAR.TSN2User(SYSDATE(), L_WDO.cField("ToTSN").Value);
  ELSE
   SETEMPTY(XDocFlow.To);
  END;
  REPLACE(XDocFlow);
  XDFE_0_Setup_To();
 END {XDocFlow_ToTSN_OnSuccessfulChange};

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
Staff.TSN + " " + Man.Name, C, ""
Memo.ud_fr(), L, "D.N.y"
Memo.ud_ut(), L, "D.N.y"
@
############################################################################################## ########## #####
=#####=========================================================================================================
|#############################################################################################################|
|                         ⨥  㬥⮮  ਮ c ##########  ##########                     |
|-------------------------------------------------------------------------------------------------------------|
|ᠭ                                                          |६           |⢨               ||
|------------------------------------------------------------------|----------------|-----------------------|-|
}
*)

(*
$P{ Line:
L_XDoc.extXDocStr(XDocFlow.XDocId, L_XDoc.xdast_wStatus), L, ""
XDocFlow.SignDTS, R, "D.N.y"
L_Str.PoolItem(L_XDF.a_Pool_2, XDocFlow.Action), L, ""
L_Str.PoolItem(" |!", ~!XDocFlow.OnBehalf), L, ""
@
|##################################################################|################|#######################|#|
}
*)

(*
$P{ TopDlv:
@
|-------------------------------------------------------------------------------------------------------------|
|                                       믮 ⢨ " "                                        |
|-------------------------------------------------------------------------------------------------------------|
|ᠭ                                                      |६           |⢨               |. |
|--------------------------------------------------------------|----------------|-----------------------|-----|
}
*)

(*
$P{ LineDlv:
L_XDoc.extXDocStr(XDocFlow.XDocId, L_XDoc.xdast_wStatus), L, ""
XDocFlow.SignDTS, R, "D.N.y H:M"
L_Str.PoolItem(L_XDF.a_Pool_2, XDocFlow.Action), L, ""
XDocFlow.Author, L, ""
@
|##############################################################|################|#######################|#####|
}
*)

(*
$P{ TopCurUser:
@
|-------------------------------------------------------------------------------------------------------------|
|                                        믮 ⢨ ᮢ⭮                                        |
|-------------------------------------------------------------------------------------------------------------|
|ᠭ                                                |६           |⢨               |. |   |
|--------------------------------------------------------|----------------|-----------------------|-----|-----|
}
*)

(*
$P{ LineCurUser:
L_XDoc.extXDocStr(XDocFlow.XDocId, L_XDoc.xdast_wStatus), L, ""
XDocFlow.SignDTS, R, "D.N.y H:M"
L_Str.PoolItem(L_XDF.a_Pool_2, XDocFlow.Action), L, ""
XDocFlow.Author, L, ""
?, L, ""
@
|########################################################|################|#######################|#####|#####|
}
*)

(*
$P{ Bot:
@
=(c) Polak=====================================================================================================
}
*)

 PRIVATE PROCEDURE CreateTmpXDF();
 BEGIN
  CreateFILE(XdfTmp, "", "६ 䠩  XDocFlow", "");
  CreateFIELD(XdfTmp, XDocId,  LIKE(XDocFlow.XDocId), "", "", "", "Id 㬥", "");
  CreateFIELD(XdfTmp, SignDTS, LIKE(XDocFlow.SignDTS), "", "", "", "६ ⠬", "");
  CreateINDEXandLINK(XdfTmp, XDocFlow, UNIQUE, XDocFlow, XDocFlow, USER, "", "", XDocId, SignDTS);
  SetVersionOfFILE(XdfTmp, 0);
  L_AFM.AF_CreateTemp("XdfTmp", "XdfTmp");
 END {CreateTmpXDF};

 PRIVATE PROCEDURE CreateTmpXDoc();
 BEGIN
  CreateFILE(XDocTmp, "", "㬥 ", "");
  CreateFIELD(XDocTmp, XDocId,  LIKE(XDocFlow.XDocId), "", "", "", "Id 㬥", "");
  CreateINDEX(XDocTmp, XDocId, UNIQUE, "", "", XDocId);
  SetVersionOfFILE(XDocTmp, 0);
  L_AFM.AF_CreateTemp("XDocTmp", "XDocTmp");
 END {CreateTmpXDoc};

 PRIVATE PROCEDURE Make(MakePart: SCAL[3]; MakeEmpty: SCAL[2]);
  VAR
   _From, _UpTo: TIMESTAMP;
 BEGIN
  L_General.Assert(ACCESS(Staff), " ࠭ ࠡ⭨");
  _From:=L_DatTim.GetTIMESTAMPOfDate(Memo.ud_fr());
  _UpTo:=L_DatTim.GetTIMESTAMPOfDate(Memo.ud_ut() + 1);
  IF    MakePart = 0 THEN
   SELECT(XDocFlow, Author, , Staff.TSN);
  ELSIF MakePart = 1 THEN
   SELECT(XDocFlow, OnBehalf, , Staff.TSN);
  ELSE
   CONFORM(Staff, Man);
   SELECT(XDocFlow, Executor, , L_APAR.TSN2User(SYSDATE(), Staff.TSN));
  END;
  IF (OCCURE(XDocFlow) & SRCINSEL(XDocFlow, GE, _From) & (XDocFlow.SignDTS < _UpTo)) |
     MakeEmpty THEN
   IF MakePart = 1 THEN
    MAKEPART(R, TopDlv);
   ELSIF MakePart = 0 THEN
    MAKEPART(R, Top, "PrDcF");
   ELSE
    MAKEPART(R, TopCurUser);
   END;
  END;
  WHILE OCCURE(XDocFlow) & (XDocFlow.SignDTS <_UpTo) DO
   PROMPT(XDocFlow.XDocId);
   XdfTmp.XDocId :=XDocFlow.XDocId;
   XdfTmp.SignDTS:=XDocFlow.SignDTS;
   IF (MakePart = 2) THEN
    IF (XDocFlow.Author # Staff.TSN) & (XDocFlow.OnBehalf # Staff.TSN) THEN
     INSERT(XdfTmp);
    END;
   ELSE
    INSERT(XdfTmp);
   END;
   NEXT(XDocFlow);
  END; (* WHILE OCCURE(XDocFlow) ... *)
  SELECT(XdfTmp, XDocFlow);
  WHILE OCCURE(XdfTmp) DO
   CONFORM(XdfTmp, XDocFlow);
   PROMPT(XDocFlow.XDocId);
   IF MakePart = 1 THEN
    MAKEPART(R, LineDlv);
   ELSIF MakePart = 0 THEN
    MAKEPART(R, Line);
   ELSE
    IF ~!XDocFlow.OnBehalf THEN
     SEARCH(Staff, TSN, , XDocFlow.OnBehalf);
     MAKEPART(R, LineCurUser, Staff.TSN);
    ELSE
     MAKEPART(R, LineCurUser, "");
    END;
   END;
   IF ~SEARCH(XDocTmp, XDocId, XDocFlow.XDocId) THEN
    L_XDoc.sl_AddItem(XDocFlow.XDocId, L_XDF.XDocFullDescription(XDocFlow.XDocId));
    XDocTmp.XDocId:=XDocFlow.XDocId;
    INSERT(XDocTmp);
   END;
   NEXT(XdfTmp);
  END; (* WHILE OCCURE(XdfTmp) ... *)
  REMSEL(XdfTmp);
 END {Make};

 PUBLIC PROCEDURE PrDcF();
  VAR
   _TSN : LIKE(Staff.TSN);
 BEGIN
  L_General.Assert(ACCESS(Staff), " ࠭ ࠡ⭨");
  CONFORM(Staff, Man);
  OPENREP(R, L_General.NameConnection("PrDcF"), Create);
  CreateTmpXDF();
  CreateTmpXDoc();
  L_XDoc.sl_Init("㬥");
  Make(0, 1);
  Make(1, 0);
  Make(2, 0);
  L_AFM.AF_PurgeTemp("XDocTmp");
  L_AFM.AF_PurgeTemp("XdfTmp");
  MAKEPART(R, Bot);
  CLOSEREP(R);
  L_XDoc.sl_Done();
 END {PrDcF};

 PUBLIC PROCEDURE Mark_Permission();
 BEGIN
  DoAction(L_XDF.a_Permission);
 END {Mark_Permission};

 PUBLIC PROCEDURE Mark_Agreement();
 BEGIN
  DoAction(L_XDF.a_Agreement);
 END {Mark_Agreement};

 PUBLIC PROCEDURE Mark_HandOriginal();
 BEGIN
  DoAction(L_XDF.a_HandOriginal);
 END {Mark_HandOriginal};

 PUBLIC PROCEDURE Mark_HandCopy();
 BEGIN
  DoAction(L_XDF.a_HandCopy);
 END {Mark_HandCopy};

 PUBLIC PROCEDURE Mark_Acquaint();
 BEGIN
  DoAction(L_XDF.a_Acquaint);
 END {Mark_Acquaint};

 PUBLIC PROCEDURE Mark_Implementation();
 BEGIN
  DoAction(L_XDF.a_Implementation);
 END {Mark_Implementation};

 PUBLIC PROCEDURE Mark_HandElCopy();
 BEGIN
  DoAction(L_XDF.a_HandElCopy);
 END {Mark_HandElCopy};

 PUBLIC PROCEDURE Mark_AcquaintElCopy();
  VAR
   curXDocId : L_XDoc.tXDocId;
   trgSignDTS: LIKE(XDocFlow.SignDTS);
 BEGIN
  curXDocId:=L_XDoc.GetCurrentXDocId();
  SELECT(XDocFlow, XDocFlow, , curXDocId);
  LAST(XDocFlow);
  LOOP
   IF    ~OCCURE(XDocFlow) THEN
    L_General.Error("  । ஭ ");
   ELSIF (XDocFlow.Action = L_XDF.a_HandElCopy) & ToIs2AcsObj(L_General.curUSER) THEN
    L_General.Assert(~L_XDF.HasAcquaintElCopyByCurHandElCopy(curXDocId, L_General.curUSER, trgSignDTS),
                     " 㦥   ஭  㬥 " + L_DatTim.TimestampStr(trgSignDTS));
    EXIT;
   END; (* LOOP *)
   PREV(XDocFlow);
  END;
  DoAction(L_XDF.a_AcquaintElCopy);
 END {Mark_AcquaintElCopy};

 PUBLIC PROCEDURE Mark_SetAuthor();
 BEGIN
  SELECT(XDocFlow, XDocFlow, , L_XDoc.GetCurrentXDocId());
  LAST(XDocFlow);
  LOOP
   IF    ~OCCURE(XDocFlow) THEN
    EXIT;
   ELSIF XDocFlow.Action = L_XDF.a_SetAuthor THEN
    SEARCH(Staff, TSN, , XDocFlow.Author);
    L_StdFrm.Choice(" 㬥 㦥 ⠭: " + Staff.TSN + " " + Staff.Name);
    EXIT;
   END; (* LOOP *)
   PREV(XDocFlow);
  END;
  DoAction(L_XDF.a_SetAuthor);
 END {Mark_SetAuthor};

BEGIN
 L_Events.ExecuteHandler();
END.
