(*
%Copyright   (c) Polak IT
%Author      27.02.2003 18:19:06 POLAK\BOR
%Purpose      KLADR
%Type        operation
%Specific    SYS
%Modified    11.07.2014 11:48:42 POLAK\LDA
%Environment
*)

 MODULE KlAdr [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_General, L_DatTim, L_AFM, L_ExtDB, L_Russia, L_Str, L_OwnData, L_StdFrm, L_FIO,
        Str, DBF, Math, FIO, L_HTTP, L_Events, L_WDO, L_bp;

 PRIVATE CONST
  DEBUG = 0;

  dbf_Kladr  = 0;
  dbf_Street = 1;
  dbf_House  = 2;
  dbf_Flat   = 3;

  dbf_Files = [ "KLADR.DBF", "STREET.DBF", "DOMA.DBF", "FLAT.DBF" ];

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
?, L, "D.N.y H:M"
@
######################################################################################################## ########## #####
=#####====================================================================================================================================
|                                          ⮪   (  ################)                                           |
}
*)

(*
$P{ SubTop:
?, L, ""
@
|----------------------------------------------------------------------------------------------------------------------------------------|
|########################################################################################################################################|
|----------------------------------------------------------------------------------------------------------------------------------------|
|     ||-||.|. ||                                |ਬ砭                                                     |
||  |   |   |   |    |    |                                        |                                                               |
|------|--|---|---|---|----|----|----------------------------------------|---------------------------------------------------------------|
}
*)

(*
$P{ Line:
?, R, ""
?, L, ""
?, L, ""
?, L, ""
?, L, ""
?, L, ""
?, L, ""
?, L, ""
?, L, ""
@
|######|##|###|###|###|####|####|########################################|###############################################################|
}
*)

(*
$P{ Bot:
?, R, ""
?, C, ""
@
|----------------------------------------------------------------------------------------------------------------------------------------|
|                                                                                                         ᥣ 訡 ᥩ  ######|
|----------------------------------------------------------------------------------------------------------------------------------------|
|########################################################################################################################################|
|                                                       믮 ᯥ譮                                                      |
=(c) Polak================================================================================================================================
}
*)

 PRIVATE VAR
  NumError: NUMERIC[0];
  NeedSubTop: SCAL[2];
  SubTopName: STRING[];
  glb_Num,
  glb_TotNum: NUMERIC[0];

 PRIVATE PROCEDURE SetSubTop(Name: STRING[]);
 BEGIN
  NeedSubTop:=1;
  SubTopName:=Name;
 END {SetSubTop};

 PRIVATE PROCEDURE MakeSubTop();
 BEGIN
  IF NeedSubTop THEN
   MAKEPART(R, SubTop, SubTopName);
   NeedSubTop:=0;
  END;
 END {MakeSubTop};

 PRIVATE PROCEDURE MakeLine(i: NUMERIC[0]; Msg: STRING[]);
 BEGIN
  MakeSubTop();
  MAKEPART(R, Line, i, KlAdr.Region, KlAdr.District, KlAdr.City, KlAdr.Village, KlAdr.Street,
                    KlAdr.House, KlAdr.Socr+" "+KlAdr.Name, Msg);
 END {MakeLine};

 PRIVATE PROCEDURE CheckSocrName(i        : NUMERIC[0];
                                 _Level   : LIKE(KlAdr.Level);
                                 _Region  : LIKE(KlAdr.Region);
                                 _District: LIKE(KlAdr.District);
                                 _City    : LIKE(KlAdr.City);
                                 _Village : LIKE(KlAdr.Village);
                                 _Street  : LIKE(KlAdr.Street);
                                 _House   : LIKE(KlAdr.House);
                                 _Name    : STRING[];
                                 _Socr    : LIKE(KlAdr.Socr));
  CONST
   lv_Pool = "---|ॣ|ࠩ|த|.㭪|㫨||";
 BEGIN
  IF SELECT(socrb, level, _Level, _Socr) THEN
   L_General.InformUs("    ⨯ " + L_Str.PoolItem(lv_Pool, _Level) + " '" + _Socr +"'");
  ELSE
   MakeSubTop();
   MAKEPART(R, Line, i, _Region, _District, _City, _Village, _Street, _House, _Socr + " " + _Name,
                     " ⨯ " + L_Str.PoolItem(lv_Pool, _Level) + " '" + _Socr +"'");
   INC(NumError);
  END;
 END {CheckSocrName};

 PRIVATE PROCEDURE GetDFBFileKlAdrVersion(Strong: SCAL[2]): TIMESTAMP;
  VAR
   trgVerDTS: TIMESTAMP;
   cur_dbf  : STRING[];
 BEGIN
  trgVerDTS:=MINOF(trgVerDTS);

  FOR_EACH cur_dbf IN dbf_Files DO
   IF ~FIO.Exists(cur_dbf) THEN
    L_General.Assert(~Strong, "  䠩 " + cur_dbf);
   ELSE
    trgVerDTS:=Math.Max(L_FIO.GetFileLastModifiedDTS(cur_dbf), trgVerDTS);
   END;
  END; (* FOR_EACH cur_dbf IN ... *)

  RETURN trgVerDTS;
 END {GetDFBFileKlAdrVersion};

 PRIVATE PROCEDURE ImpKLADR(DBFName: STRING[]; trgVerDTS: TIMESTAMP);
  VAR
   i          : NUMERIC[0];
   Code       : STRING[];
   curType    : LIKE(KlAdr.Type);
   curLevel   : LIKE(KlAdr.Level);
   curRegion  : LIKE(KlAdr.Region);
   curDistrict: LIKE(KlAdr.District);
   curCity    : LIKE(KlAdr.City);
   curVillage : LIKE(KlAdr.Village);
   curStreet  : LIKE(KlAdr.Street);
   curHouse   : LIKE(KlAdr.House);
   curName    : LIKE(KlAdr.Name);
   curSocr    : LIKE(KlAdr.Socr);
   curAddCode : LIKE(KlAdr.AddCode);
   curStatus  : LIKE(KlAdr.Status);
   TwnPool,
   StrPool,
   VilPool    : STRING[];
   IsNewRec   : SCAL[2];
 BEGIN
  APP.Get().NewPrompt("㧪  䠩: " + DBFName);
  StrPool:=L_Russia.GetStreetTPool_InMNSFormat();
  VilPool:=L_Russia.GetVillageTPool_InMNSFormat();
  TwnPool:=L_Russia.GetTownTPool_InMNSFormat();
  SetSubTop(DBFName);

  DBF.Open(D, DBFName);
  i:=0;

  LOOP
   TRY [TRANSACTION(TRANSACTION_SLAVE2)]
    REPEAT
     IF ~DBF.Seek(D, i) | ((DEBUG # 0) & (i >= DEBUG)) THEN
      EXIT;
     END;
     IF DBF.Deleted(D) THEN
      PROMPT("<DELETED>");
     ELSE
      PROMPT(FLDTOSTR(i, ",0") + " (" +  FLDTOSTR(Math.Trunc(glb_Num/glb_TotNum*100, 0.01), "2.") + "%)");
      Code:=DBF.GetField(D, "CODE");
  
      curRegion   :=Str.Slice(Code, 0, 2);
      curDistrict :=Str.Slice(Code, 2, 3);
      curCity     :=Str.Slice(Code, 5, 3);
      curVillage  :=Str.Slice(Code, 8, 3);
      curName     :=DBF.GetField(D, "NAME");
      curSocr     :=DBF.GetField(D, "SOCR");
  
      SETEMPTY(curType);
  
      IF DBFName = dbf_Files[dbf_Kladr] THEN
       curStatus  :=DBF.GetField(D, "STATUS");
       IF Str.Match(Code, "??000000000??") THEN
        curLevel  :=L_Russia.lv_Region;
        curType   :=0;
       ELSIF Str.Match(Code, "?????000000??") THEN
        curLevel  :=L_Russia.lv_District;
        curType   :=0;
       ELSIF Str.Match(Code, "????????000??") THEN
        curLevel  :=L_Russia.lv_City;
        curType   :=L_Str.PositionInPool(TwnPool, Str.Cap(curSocr));
       ELSE
        curLevel  :=L_Russia.lv_Village;
        curType   :=L_Str.PositionInPool(VilPool, Str.Cap(curSocr));
       END;
       curAddCode :=Str.Slice(Code, 11, 2);
       curStreet  :=000000;
       curHouse   :=0000;
      ELSE
       curStatus  :=0;
       curStreet  :=Str.Slice(Code, 11, 4);
       IF    DBFName = dbf_Files[dbf_Street] THEN
        curLevel  :=L_Russia.lv_Street;
        curType   :=L_Str.PositionInPool(StrPool, Str.Cap(curSocr));
        curHouse  :=0000;
        curAddCode:=Str.Slice(Code, 15, 2);;
       ELSE
        curHouse  :=Str.Slice(Code, 15, 4);
        curAddCode:=00;
        IF    DBFName = dbf_Files[dbf_House] THEN
         curLevel :=L_Russia.lv_House;
         curType  :=L_Str.PositionInPool("", Str.Cap(curSocr));;
        ELSIF DBFName = dbf_Files[dbf_Flat] THEN
         curLevel :=L_Russia.lv_Flat;
         curType  :=0;
        ELSE
         L_General.CallUs(0);
        END;
       END;
      END;
  
      IF ~!curType THEN
       IF SEARCH(KlAdr, KlAdr, curLevel, curRegion, curDistrict, curCity, curVillage, curStreet, curHouse, curAddCode) THEN
        IF KlAdr.VerDTS # trgVerDTS THEN
         IsNewRec:=0;
        ELSE
         SETEMPTY(IsNewRec); (* ஢  ।饬 ᪥ *)
        END;
       ELSE
        IsNewRec:=1;
        KlAdr.Level   :=curLevel;
        KlAdr.Region  :=curRegion;
        KlAdr.District:=curDistrict;
        KlAdr.City    :=curCity;
        KlAdr.Village :=curVillage;
        KlAdr.Street  :=curStreet;
        KlAdr.House   :=curHouse;
        KlAdr.AddCode :=curAddCode;
       END;
       IF ~!IsNewRec THEN
        KlAdr.Name  :=curName;
        KlAdr.Socr  :=curSocr;
        KlAdr.Type  :=curType;
        KlAdr.Status:=curStatus;
        KlAdr.Index :=DBF.GetField(D, "INDEX");
        KlAdr.Gnimb :=DBF.GetField(D, "GNINMB");
        KlAdr.VerDTS:=trgVerDTS;
   
        IF IsNewRec THEN
         INSERT(KlAdr);
        ELSE
         REPLACE(KlAdr);
        END;
       END; (* IF ~!IsNewRec ... *)
      ELSE
       CheckSocrName(i, curLevel, curRegion, curDistrict, curCity, curVillage, curStreet, curHouse, curName, curSocr);
      END;
     END; (* IF DBF.Deleted(D) ... *)
     INC(i);
     INC(glb_Num);
    UNTIL (Math.Mod(i, 1000) = 0); (*蠥 ࠭  1000 ᥩ*)
 
   EXCEPT
   END; (* TRY *)
  END; (* LOOP ... *)

  DBF.Close(D);
 END {ImpKLADR};

 PRIVATE PROCEDURE ClearOldVer(curVerDTS: TIMESTAMP);
 BEGIN
  APP.Get().NewPrompt(" ன ᨨ");
  SELECT(KlAdr, VerDTS);
  WHILE OCCURE(KlAdr) DO
   NARROWSEL(KlAdr, VerDTS);
   IF KlAdr.VerDTS # curVerDTS THEN
    PROMPT(L_DatTim.TimestampStr(KlAdr.VerDTS));
    REMSEL(KlAdr);
   END;
   NEXTINEXTENDEDSEL(KlAdr, VerDTS);
  END; (* WHILE OCCURE(KlAdr) *)
 END {ClearOldVer};

 PRIVATE PROCEDURE SetKlAdrWindowTitle();
  VAR
   trgTitle : STRING[];
   curVerDTS: TIMESTAMP;
 BEGIN
  trgTitle:="䨪 ᮢ ()";
  IF SELECT(KlAdr, VerDTS) THEN
   NARROWSEL(KlAdr, VerDTS);
   curVerDTS:=KlAdr.VerDTS;
   IF ~!curVerDTS THEN
    trgTitle:=trgTitle + ".  㠫쭮 " + L_DatTim.DateStr(L_DatTim.GetDateOfTIMESTAMP(curVerDTS));
   END;
   IF NEXTINEXTENDEDSEL(KlAdr, VerDTS) THEN
    trgTitle:=trgTitle + " (楤    믮  !)";
   END;
  END;
  RTI.Get().CurWindow.Title:=trgTitle;
 END {SetKlAdrWindowTitle};

 PRIVATE PROCEDURE DownloadKlAdr(FullURL, dstArchFullFileName: STRING[]);
  VAR
   tmpArchFullFileName: STRING[];
 BEGIN
  tmpArchFullFileName:=L_FIO.tf_GetFullPath(L_FIO.effp_FileName_woExt(dstArchFullFileName), L_FIO.effp_FileExt(dstArchFullFileName));
  L_General.Assert(L_HTTP.httpGetFile(FullURL, tmpArchFullFileName), " 㤠 㧨 " + FullURL);
  L_FIO.ExactFileCopy(tmpArchFullFileName, dstArchFullFileName);
 END {DownloadKlAdr};

 PRIVATE PROCEDURE PrepareForImportKlAdr();
  CONST
   z7FileName = "BASE.7z";
(*
   URL        = "http://www.gnivc.ru/html/gnivcsoft/KLADR/";
*)
   URL        = "http://www.nalog.ru/files/kladr/";
  VAR
   cur_dbf,
   bt_UseFile,
   bt_UseInternet,
   b_UseFile,
   b_UseInternet,
   FullURL,
   dstArchFullFileName: STRING[];
   ArchDTS,
   dbfDTS,
   curVerDTS,
   httpVerDTS         : TIMESTAMP;
   ProceedDBF,
   ProceedArch,
   DownloadArchAllowed: SCAL[2];
   React, httpSize    : NUMERIC[1];
 BEGIN
  FullURL            :=URL + z7FileName;
  httpVerDTS         :=L_HTTP.GetFileLastModifiedDTS(FullURL);

  dstArchFullFileName:=FIO.MakePath(z7FileName, , FIO.MakePath("TXT", , FIO.AsbSpecialFolders("ClientProjectRoot")));

  IF ~L_General.BatchExecution_IsRunning() THEN
   dbfDTS           :=GetDFBFileKlAdrVersion(0);
   IF FIO.Exists(dstArchFullFileName) THEN
    ArchDTS    :=L_FIO.GetFileLastModifiedDTS(dstArchFullFileName);
    ProceedDBF :=dbfDTS >= ArchDTS;
    ProceedArch:=~ProceedDBF;
   ELSE
    SETEMPTY(ArchDTS);
    ProceedDBF :=dbfDTS # MINOF(TIMESTAMP);
    ProceedArch:=0;
   END;
   IF    ProceedDBF THEN
    bt_UseFile:="  DBF (" + L_DatTim.DateStr(L_DatTim.GetDateOfTIMESTAMP(dbfDTS)) + ")";
    b_UseFile :="B";
   ELSIF ProceedArch THEN
    bt_UseFile:="  娢 (" + L_DatTim.DateStr(L_DatTim.GetDateOfTIMESTAMP(ArchDTS)) + ")";
    b_UseFile :="B";
   ELSE
    bt_UseFile:="    ";
    b_UseFile :="BN";
   END;
 
   IF ~!httpVerDTS THEN
    httpSize           :=L_HTTP.GetFileSize(FullURL);
    DownloadArchAllowed:=1;
    bt_UseInternet     :="㧪  ୥ (" + L_DatTim.DateStr(L_DatTim.GetDateOfTIMESTAMP(httpVerDTS)) +
                       ", " + FLDTOSTR(httpSize/1024/1024, "1.") + ")";
   ELSE
    DownloadArchAllowed:=0;
    bt_UseInternet     :="㧪  ୥: 䠩 㯥";
   END;
 
   IF ~DownloadArchAllowed THEN
    IF ~ProceedDBF & ~ProceedArch THEN
     L_General.Error("  " +
                     "|" +
                     "| 祭  ୥  㯥 " +
                     "| 娢  DBF-䠩                    " +
                     "| DBF-䠩                              ");
    END;
    b_UseInternet:="NB";
    b_UseFile    :="DB";
   ELSE
    IF ProceedDBF | ProceedArch THEN
     b_UseInternet:="B";
     b_UseFile    :="DB";
    ELSE
     b_UseInternet:="DB";
     b_UseFile    :="NB";
    END;
   END;
 
   React:=STDFORM("楤   த⥫쭮 ६,  砥" +
                  "|室 믮  ࢠ." +
                  "|| ᫥饬 ᪥ ࠭ ஢" +
                  "|  ⥭, ୠ 㧪 䠩" +
                  "| ୥ 믮  㤥" +
                  "||" +
                  "%< ⮯! >X" +
                  "%<" + bt_UseFile + ">"+b_UseFile+"  "+
                  "%<" + bt_UseInternet + ">"+b_UseInternet+"  ",
                  "   ",
                  INFORM_NEUTRAL,
                  1);
 
   IF  React = 2 THEN
    DownloadKlAdr(FullURL, dstArchFullFileName);
    ProceedArch:=1;
   END;
  ELSE
   L_General.Assert(~!httpVerDTS, "㧪   ୥ :| 祭  ୥  㯥 ");
   IF ~FIO.Exists(dstArchFullFileName) | (L_FIO.GetFileLastModifiedDTS(dstArchFullFileName) < httpVerDTS) THEN
    DownloadKlAdr(FullURL, dstArchFullFileName);
   END;
   ProceedArch:=1;
  END; (* IF ~L_General.BatchExecution_IsRunning()...*)

  IF ProceedArch THEN
   L_General.Assert(L_General.Extract_7z("-aoa", dstArchFullFileName, L_FIO.effp_Folder(dstArchFullFileName)) = L_General.eprc_Normal,
                    "ந諠 訡  ᯠ 娢 " + dstArchFullFileName);
  END;
 END {PrepareForImportKlAdr};

 PRIVATE PROCEDURE InsertEmptySingleRecord();
 BEGIN
  TRY [TRANSACTION(TRANSACTION_SLAVE)]
   KlAdr.Level   :=L_Russia.lv_Region;
   KlAdr.Region  :=00;
   KlAdr.District:=000;
   KlAdr.City    :=000;
   KlAdr.Village :=000;
   KlAdr.Street  :=0000;
   KlAdr.House   :=0000;
   KlAdr.AddCode :=00;
   SETEMPTY(KlAdr.Name, KlAdr.Socr, KlAdr.Type, KlAdr.Status, KlAdr.Index, KlAdr.Gnimb, KlAdr.VerDTS);
   INSERT(KlAdr);
  EXCEPT
  END; (* TRY *)
 END {InsertEmptySingleRecord};

 PUBLIC PROCEDURE KlAdrImport();
  CONST
   bpId    = "KAImp";
  VAR
   cur_dbf  : STRING[];
   curVerDTS,
   trgVerDTS: TIMESTAMP;
 BEGIN
  L_bp.extBegin(bpId, " ", L_bp.t_Prc_wRp, L_General.tc_Entire, L_bp.pt_Low);
  PrepareForImportKlAdr();

  trgVerDTS:=GetDFBFileKlAdrVersion(1);
  IF (SYSTIMESTAMP() - trgVerDTS) > 180 THEN
   L_StdFrm.Choice("      " + L_DatTim.TimestampStr(trgVerDTS) +
                   "|஢ 稥  ᨨ  ᠩ www.gnivc.ru" +
                   "|| ࠢ 믮 ?");
  END;

  IF SELECT(KlAdr, VerDTS) THEN
   NARROWSEL(KlAdr, VerDTS);
   curVerDTS:=KlAdr.VerDTS;
   IF ~NEXTINEXTENDEDSEL(KlAdr, VerDTS) THEN
    IF    curVerDTS = trgVerDTS THEN
     L_StdFrm.BlueStop(" ᨨ  " + L_DatTim.TimestampStr(trgVerDTS) + " 㦥 ஢");
    ELSIF ~!curVerDTS & (curVerDTS > trgVerDTS) THEN
     L_StdFrm.BlueStop(" 믮   㠫쭮 ᨨ   " + L_DatTim.TimestampStr(curVerDTS));
    END;
   END;
  ELSE
   InsertEmptySingleRecord();
  END;

  NeedSubTop:=0;

  IF (DEBUG > 0) & L_General.Debug(1) THEN
   (*⫠*)
  END;

  L_ExtDB.MakeDBFMirror("socrb", "socrbase", "level,scname");

  SETZERO(NumError);
  OPENREP(R, L_General.NameConnection(bpId), Create);
  MAKEPART(R, Top, bpId, trgVerDTS);

  glb_Num:=0;
  glb_TotNum:=0;
  FOR_EACH cur_dbf IN dbf_Files DO
   DBF.Open(D, cur_dbf);
   INC(glb_TotNum, DBF.Amount(D));
   DBF.Close(D);
  END; (* FOR_EACH cur_dbf IN ... *)

  TRY [TERM_BY_USER(TERM_BY_USER_THROW)]
   FOR_EACH cur_dbf IN dbf_Files DO
    ImpKLADR(cur_dbf, trgVerDTS);
   END; (* FOR_EACH cur_dbf IN ... *)
  EXCEPT
  ON Exc: Exc_Root DO
   RAISE();
  FINALLY
   IF ~L_General.BatchExecution_IsRunning() THEN
    SetKlAdrWindowTitle();
   END;
  END; (* TRY *)

  L_ExtDB.PurgeDBFMirror("socrb");

  ClearOldVer(trgVerDTS);

  IF NumError = 0 THEN
   MAKEPART(R, Bot, NumError, );
  ELSE
   MAKEPART(R, Bot, NumError, "訡 맢 ४    ⠢⥫ﬨ.     ࠡ  ࠢ筨");
  END;
  CLOSEREP(R);
  L_bp.End();
 END {KlAdrImport};

 PUBLIC PROCEDURE ChangeLevel();
  VAR
   curRegion  : LIKE(KlAdr.Region);
   curDistrict: LIKE(KlAdr.District);
   curCity    : LIKE(KlAdr.City);
   curVillage : LIKE(KlAdr.Village);
   curStreet  : LIKE(KlAdr.Street);
   curHouse   : LIKE(KlAdr.House);
   NewLevel,
   OldLevel,
   SelLevel: LIKE(KlAdr.Level);
   PoolStr: STRING[];
 BEGIN
  L_General.Assert(ACCESS(KlAdr), " ।  ");
  OldLevel:=KlAdr.Level;
  curRegion  :=KlAdr.Region;
  curDistrict:=KlAdr.District;
  curCity    :=KlAdr.City;
  curVillage :=KlAdr.Village;
  curStreet  :=KlAdr.Street;
  curHouse   :=KlAdr.House;

  NewLevel:=OldLevel-1;
  PoolStr:=Str.Slice(L_Russia.lv_Pool, Str.ItemPos(L_Russia.lv_Pool, "|", L_Russia.lv_Region), Str.Length(L_Russia.lv_Pool));
  STDFORM(L_StdFrm.sf_Choice("|%####################%| | | | | | |", "%,7<"+PoolStr+">O"),
          "३  ஢ ",
          ,
          -1,
          NewLevel);
  INC(NewLevel);

  IF NewLevel < OldLevel THEN
   SelLevel:=NewLevel+1;
  ELSE
   SelLevel:=NewLevel;
  END;
  IF    SelLevel = L_Russia.lv_Region THEN
   SELECT(KlAdr, KlAdr, , NewLevel);
  ELSIF SelLevel = L_Russia.lv_District THEN
   SELECT(KlAdr, KlAdr, , NewLevel, curRegion);
  ELSIF SelLevel = L_Russia.lv_City THEN
   SELECT(KlAdr, KlAdr, , NewLevel, curRegion, curDistrict);
  ELSIF SelLevel = L_Russia.lv_Village THEN
   SELECT(KlAdr, KlAdr, , NewLevel, curRegion, curDistrict, curCity);
  ELSIF SelLevel = L_Russia.lv_Street THEN
   SELECT(KlAdr, KlAdr, , NewLevel, curRegion, curDistrict, curCity, curVillage);
  ELSIF SelLevel = L_Russia.lv_House THEN
   SELECT(KlAdr, KlAdr, , NewLevel, curRegion, curDistrict, curCity, curVillage, curStreet);
  ELSIF SelLevel = L_Russia.lv_Flat THEN
   SELECT(KlAdr, KlAdr, , NewLevel, curRegion, curDistrict, curCity, curVillage, curStreet, curHouse);
  ELSE
   L_General.CallUs(0)
  END;
  IF ~OCCURE(KlAdr) THEN
   L_StdFrm.BlueStop(" 室 ᥩ ஢ "+L_Str.PoolItem(L_Russia.lv_Pool, NewLevel));
  ELSE
   SE_WZD_CURSOR(GE_CURWIN(), GE_CURZONE(), KlAdr);
  END;
 END {ChangeLevel};

 PRIVATE PROCEDURE KlAdr_OnOpenWindow();
 BEGIN
  SetKlAdrWindowTitle();
 END {KlAdr_OnOpenWindow};

 PRIVATE PROCEDURE KlAdr_OnInitializeWindow();
 BEGIN
  L_WDO.iWindow().OnOpenWindow:=1;
  L_WDO.iZone(1).ViewOnly:=1;
 END {KlAdr_OnInitializeWindow};

 PRIVATE PROCEDURE Region_OnAnyModification();
 BEGIN
  L_General.Error("ࠢ筨  ⮬᪨, ।஢ ");
 END {Region_OnAnyModification};

 PRIVATE PROCEDURE Region_OnBeforeEdit();
 BEGIN
  Region_OnAnyModification();
 END {Region_OnBeforeEdit};

 PRIVATE PROCEDURE Region_OnBeforeInsert();
 BEGIN
  Region_OnAnyModification();
 END {Region_OnBeforeInsert};

 PRIVATE PROCEDURE Region_OnBeforeDelete();
 BEGIN
  Region_OnAnyModification();
 END {Region_OnBeforeDelete};

 PRIVATE PROCEDURE Region_OnInitializeWindow();
 BEGIN
  L_WDO.iEvents      (1, [L_WDO.e_OnAnyModification]);
  L_WDO.iMayBeEmpty  (1, ["Region.Capital", "Region.OKATO"]);
  L_WDO.iProgs       (1, "KLADR");
 END {Region_OnInitializeWindow};

BEGIN
 L_Events.ExecuteHandler();
END.
