(*
%Copyright   (c) Polak IT
%Author      30.01.2003 10:25:00 POLAK\BOR
%Purpose      ࠡ  䠩  ஢ 
%Type        library
%Specific    SYS
%Modified    05.09.2014 14:08:43 POLAK\Gina
%Environment
*)

 MODULE L_DBD [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 PUBLIC TYPE
  tFileRecordsValidityType = SCAL[3];

 PUBLIC CONST
  frvt_None    = 0;
  frvt_Normal  = 1;
  frvt_wIncED  = 2;

 IMPORT DBD, L_Str, Str, L_General, L_AFM, L_StdFrm, L_DatTim, L_PCS, L_WDO, Math, L_Env, L_XDoc, L_AR;

 PUBLIC PROCEDURE GetTable(fi: L_General.tFileId): TABLE;
 BEGIN
  RETURN DATABASE.Get().TableById(fi);
 END {GetTable};

 PUBLIC PROCEDURE StandOnRecord(iri: L_General.tInternalRecordId);
 BEGIN
  SEARCH((GetTable(L_General.iri_fi(iri)).Name), RECORDID, , L_General.iri_ri(iri));
 END {StandOnRecord};

 PUBLIC PROCEDURE GetFieldName(File, Field: STRING[]): STRING[];
 BEGIN
  RETURN File + "." + Field;
 END {GetFieldName};

 PUBLIC PROCEDURE GetFullFieldName(File, Field: STRING[]; Idx: NUMERIC[0]): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  Result:=GetFieldName(File, Field);
  IF RANGEOF(^Result) > 1 THEN
   L_General.CallUs(Idx < RANGEOF(^Result));
   Result:=Result + "[" + L_Str.IntStr(Idx) + "]";
  END;
  RETURN Result;
 END {GetFullFieldName};

 PUBLIC PROCEDURE ffn_GetFile(ffName: STRING[]): STRING[];
 BEGIN
  RETURN Str.Item(ffName, ".", 0);
 END {ffn_GetFile};

 PUBLIC PROCEDURE ffn_GetField(ffName: STRING[]): STRING[];
 BEGIN
  RETURN Str.Item(ffName, ".[", 1);
 END {ffn_GetField};

 PUBLIC PROCEDURE ffn_GetIndex(ffName: STRING[]): NUMERIC[0];
  VAR Result: STRING[];
 BEGIN
  Result:=Str.Item(ffName, "[", 1);
  IF !Result THEN
   RETURN 0;
  ELSE
   RETURN L_Str.StrToScal(Str.Item(Result, "]", 0));
  END;
 END {ffn_GetIndex};

 PUBLIC PROCEDURE TableFieldToStr(curField: FIELD): STRING[];
  VAR
   FName: STRING[];
 BEGIN
  FName:=curField.ObtainName();
  IF ~!^FName THEN
   IF    curField.Type.Id = TYPESPEC.Id_Numeric THEN
    RETURN Str.Condense(FLDTOSTR(^FName));
   ELSIF curField.Type.Id = TYPESPEC.Id_Float THEN
    RETURN L_Str.NumToStr_woNZ(^FName);
   ELSE
    RETURN FLDTOSTR(^FName);
   END;
  ELSE
   RETURN "";
  END;
 END {TableFieldToStr};

(*!*) PUBLIC PROCEDURE FieldToStr(Field     : STRING[];
                                  FieldType : L_General.ft_DataType): STRING[];
 BEGIN
  IF ~!^Field THEN
   IF    FieldType = L_General.ft_NUMERIC THEN
    RETURN Str.Condense(FLDTOSTR(^Field));
   ELSIF FieldType = L_General.ft_FLOAT THEN
    RETURN L_Str.NumToStr_woNZ(^Field);
   ELSE
    RETURN FLDTOSTR(^Field);
   END;
  ELSE
   RETURN "";
  END;
 END {FieldToStr};

 PUBLIC PROCEDURE Get2FilesLinkName(ownFile, cooFile: STRING[]): STRING[];
  VAR
   l: LINK;
 BEGIN
  FOR_EACH l IN TABLE.Get(ownFile).Links DO
   IF l.CooIndex.Table.Name = cooFile THEN
    RETURN l.Name;
   END;
  END; (* FOR_EACH *)
  RETURN "";
 END {Get2FilesLinkName};

 PUBLIC PROCEDURE Get2FilesLinkIndexName(File, cooFile: STRING[]): STRING[];
 BEGIN
  RETURN TABLE.Get(File).Links(Get2FilesLinkName(File, cooFile)).Index.Name;
 END {Get2FilesLinkIndexName};

 PUBLIC PROCEDURE GetPrimaryIndex(File: STRING[]): STRING[];
  VAR
   i,
   a     : NUMERIC[0];
   Lend,
   Result: STRING[];
 BEGIN
  Result:=DBD.FatherIndex((File));
  IF !Result THEN
   i:=0;
   a:=DBD.IndexAmount((File));
   LOOP
    IF i = a THEN
     Lend:=DBD.LendFile((File));
     IF ~!Lend THEN
      Result:=Get2FilesLinkIndexName(File, Lend);
     ELSE
      Result:=DBD.IndexName((File), 0);
     END;
     EXIT;
    END;
    Result:=DBD.IndexName((File), i);
    IF DBD.IndexUnique((File), (Result)) THEN
     EXIT;
    END;
    INC(i);
   END; (* LOOP *)
  END;
  RETURN Result;
 END {GetPrimaryIndex};

 PUBLIC PROCEDURE GetPartialIndexValue(File, Index: STRING[]; a: NUMERIC[0]): STRING[];
  VAR
   Fields: INDEXFIELDS;
   i     : NUMERIC[0];
   Result: STRING[];
 BEGIN
  Fields:=TABLE.Get(File).Indexes(Index).Fields;
  i:=0; Result:="";
  WHILE i < a DO
   IF ~!Result THEN
    Result:=Result + "|";
   END;
   Result:=Result + TableFieldToStr(Fields(i));
   INC(i);
  END;
  RETURN Result;
 END {GetPartialIndexValue};

 PUBLIC PROCEDURE GetIndexValue(File, Index: STRING[]): STRING[];
 BEGIN
  RETURN GetPartialIndexValue(File, Index, TABLE.Get(File).Indexes(Index).Fields.Count);
 END {GetIndexValue};

 PUBLIC PROCEDURE GetPrimaryIndexAndValue(File: STRING[];
                                          VAR
                                           Index,
                                           Value: STRING[]);
  VAR
   i         : NUMERIC[0];
   IndexField: STRING[];
 BEGIN
  Index:=GetPrimaryIndex(File);
  Value:=GetIndexValue(File, Index);
 END {GetPrimaryIndexAndValue};

 PUBLIC PROCEDURE GetPrimaryIndexValue(File: STRING[]): STRING[];
 BEGIN
  RETURN GetIndexValue(File, GetPrimaryIndex(File));
 END {GetPrimaryIndexValue};

 PRIVATE PROCEDURE ShortTypeNameOf(Field: FIELD): STRING[];
 BEGIN
  RETURN Str.Item(Field.Type.Name, "[", 0);
 END {ShortTypeNameOf};

 PUBLIC PROCEDURE AssignIndexFields(File, Index, IndexValue: STRING[]; a: NUMERIC[0]);
  VAR
   Fields: INDEXFIELDS;
   i     : NUMERIC[0];
 BEGIN
  Fields:=TABLE.Get(File).Indexes(Index).Fields;
  i:=0;
  WHILE i < a DO
   ^(Fields(i).ObtainName()):=STRTOFLD(L_Str.AnyItem(IndexValue, "|", i), ^(ShortTypeNameOf(Fields(i))));
   INC(i);
  END;
 END {AssignIndexFields};

 PUBLIC PROCEDURE IsIdenticalIndex(File, i1, i2: STRING[]): SCAL[2];
  VAR
   curIndexes: TABLEINDEXES ;
   curField: FIELD;
   j, a  : NUMERIC[0];
 BEGIN
  curIndexes:=TABLE.Get(File).Indexes;
  a:=curIndexes(i1).Fields.Count;
  IF a = curIndexes(i2).Fields.Count THEN
   j:=0;
   WHILE (j < a) DO
    IF curIndexes(i1).Fields(j).Name # curIndexes(i2).Fields(j).Name THEN
     RETURN 0;
    END;
    INC(j);
   END;
   RETURN 1;
  ELSE
   RETURN 0;
  END;
 END {IsIdenticalIndex};

 PUBLIC PROCEDURE AssignLinkFields(File, cooFile: STRING[]);
  VAR
   Index,
   Link,
   cooIndex  : STRING[];
 BEGIN
  Link     :=Get2FilesLinkName(File, cooFile);
  Index    :=DBD.LinkIndexName(^File, ^Link);
  cooIndex :=DBD.LinkCooIndexName(^File, ^Link);
  AssignIndexFields(File, Index, GetIndexValue(cooFile, cooIndex), Math.Min(DBD.IndexFieldAmount(^File, ^Index), DBD.IndexFieldAmount(^cooFile, ^cooIndex)));
 END {AssignLinkFields};

 PRIVATE PROCEDURE intGetPartialIndexFieldsName(trgIndex: INDEX; a: NUMERIC[0]; FullName: SCAL[2]): STRING[];
  VAR
   Fields: INDEXFIELDS;
   i     : NUMERIC[0];
   Result: STRING[];
 BEGIN
  Fields:=trgIndex.Fields;
  SETEMPTY(Result);
  i:=0;
  WHILE i < a DO
   IF i > 0 THEN
    Result:=Result + ", ";
   END;
   IF FullName THEN Result:=Result + Fields(i).ObtainName();
   ELSE             Result:=Result + Fields(i).Name;
   END;
   INC(i);
  END;
  RETURN Result;
 END {intGetPartialIndexFieldsName};

 PUBLIC PROCEDURE GetPartialIndexFieldsName(File, Index: STRING[]; a: NUMERIC[0]): STRING[];
 BEGIN
  RETURN intGetPartialIndexFieldsName(TABLE.Get(File).Indexes(Index), a, 1);
 END {intGetPartialIndexFieldsName};

 PUBLIC PROCEDURE GetIndexFieldsName(trgIndex: INDEX): STRING[];
 BEGIN
  RETURN intGetPartialIndexFieldsName(trgIndex, trgIndex.Fields.Count, 0);
 END {GetIndexFieldsName};

 PUBLIC PROCEDURE SelectCooFile(File, LinkName: STRING[]);
  VAR
   curLink : LINK;
   curIndex,
   cooIndex: INDEX;
   cooFile,
   Fields  : STRING[];
 BEGIN
  curLink :=TABLE.Get(File).Links(LinkName);
  curIndex:=curLink.Index;
  cooIndex:=curLink.CooIndex;

  Fields  :=GetPartialIndexFieldsName(File, curIndex.Name, Math.Min(curIndex.Fields.Count, cooIndex.Fields.Count));
  IF (curLink.Kind = L_General.lk_USER) | (curLink.Kind = L_General.lk_BORROW) | (curLink.Kind = L_General.lk_HISTORY) THEN
   SEARCH((cooIndex.Table.Name), (cooIndex.Name), , ^Fields);
  ELSE
   SELECT((cooIndex.Table.Name), (cooIndex.Name), , ^Fields);
  END;
 END {SelectCooFile};

 PUBLIC PROCEDURE SelectUsers(ownerFile, Link: STRING[]);
 BEGIN
  SelectCooFile(ownerFile, Link);
 END {SelectUsers};

 PUBLIC PROCEDURE IsEmptyLink(File, Link: STRING[]): SCAL[2];
  VAR
   Index,
   cooFile,
   cooIndex  : STRING[];
   i, a      : NUMERIC[0];
   Field     : STRING[];
 BEGIN
  Index    :=DBD.LinkIndexName(^File, ^Link);
  cooFile  :=DBD.LinkCooFileName(^File, ^Link);
  cooIndex :=DBD.LinkCooIndexName(^File, ^Link);
  i:=0;
  a:=Math.Min(DBD.IndexFieldAmount(^File, ^Index), DBD.IndexFieldAmount(^cooFile, ^cooIndex));
  WHILE i < a DO
   Field:=GetFieldName(File, DBD.IndexFieldName(^File, ^Index, i));
   IF !^Field THEN
    RETURN 1;
   END;
   INC(i);
  END;
  RETURN 0;
 END {IsEmptyLink};

 PUBLIC PROCEDURE SelectByIndexValue(File, Index, Value: STRING[]): SCAL[2];
  VAR
   i           : NUMERIC[0];
   fValue,
   FieldsValue,
   curFieldName: STRING[];
 BEGIN
  i:=0;
  FieldsValue:="";
  WHILE i < DBD.IndexFieldAmount(^File, ^Index) DO
   IF ~!FieldsValue THEN
    FieldsValue:=FieldsValue + ",";
   END;
   fValue:=L_Str.AnyItem(Value, "|", i);
   curFieldName:=GetFieldName(File, DBD.IndexFieldName(^File, ^Index, i));
   IF    !fValue THEN
    fValue:="EMPTYOF(" + curFieldName + ")";
   ELSIF TYPEOF(^curFieldName) = L_General.ft_STRING THEN
    fValue:='"' + fValue + '"';
   END;
   FieldsValue:=FieldsValue + fValue;
   INC(i);
  END;
  RETURN SELECT(^File, ^Index, ^FieldsValue);
 END {SelectByIndexValue};

 PUBLIC PROCEDURE GetActiveUsers(File: STRING[]; VAR Users: STRING[][]);
  VAR
   curLink: LINK;
   usersCnt: NUMERIC[0];
   userFile: STRING[];
 BEGIN
  SETEMPTY(Users);
  usersCnt:=0;
  FOR_EACH curLink IN TABLE.Get(File).Links DO
   IF curLink.Kind = LINK.Kind_Owner THEN
    userFile:=curLink.CooIndex.Table.Name;
    TRY [RESTORE_CURSORS(RC_ON)]
     SelectUsers(File, curLink.Name);
     IF OCCURE(^userFile) THEN
      Users[usersCnt]:=userFile;
      INC(usersCnt);
     END;
    EXCEPT
    END; (* TRY *)
   END;
  END;
 END {GetActiveUsers};

 PRIVATE PROCEDURE _CheckUsers(File: STRING[]; Silent: SCAL[2]; Strong: SCAL[2]): SCAL[2];
  VAR
   Users   : STRING[][64];
   usersCnt: NUMERIC[0];
   Msg     : STRING[];
 BEGIN
  Msg:="";
  GetActiveUsers(File, Users);
  usersCnt:=0;
  WHILE ~!Users[usersCnt] DO
   Msg:=Msg + "|" +
        Str.Align(L_Str.CutLongStr(TABLE.Get(Users[usersCnt]).Description, 57), 57, LEFT) + " " +
        Str.Align("(" + Users[usersCnt] + ")", 10, LEFT);
   INC(usersCnt);
  END;
  IF (usersCnt > 0) & ~Silent THEN
   Msg:=" ⥪   뫪  䠩"+Str.Item(":|:", "|", usersCnt > 1) + "| " + Msg;
   IF Strong THEN
    L_General.Error(Msg);
   ELSE
    L_StdFrm.Inform(Msg);
   END;
  END;
  RETURN usersCnt = 0;
 END {_CheckUsers};

 PUBLIC PROCEDURE TestUsers(File: STRING[]; Silent: SCAL[2]): SCAL[2];
 BEGIN
  RETURN _CheckUsers(File, Silent, 0);
 END {TestUsers};

 PUBLIC PROCEDURE CheckUsers(File: STRING[]);
  VAR
   Ok: SCAL[2];
 BEGIN
  Ok:=_CheckUsers(File, 0, 1);
 END {CheckUsers};

 PUBLIC PROCEDURE CanHasUser(File: STRING[]): SCAL[2];
  VAR
   curLink: LINK;
 BEGIN
  FOR_EACH curLink IN TABLE.Get(File).Links DO
   IF curLink.Kind = LINK.Kind_Owner THEN
    RETURN 1;
   END;
  END; (* FOR_EACH curLink ... *)
  RETURN 0;
 END {CanHasUser};

 PUBLIC PROCEDURE _ChooseFileName(Title: STRING[]; Cond: PROC; ShowAllFiles: SCAL[2]): STRING[];
  VAR
   curTable: TABLE;
   i,
   Pos     : NUMERIC[0];
   PShow,
   FileName: STRING[];
 BEGIN
  APP.Get().NewPrompt("");
  FileName:=GE_CURFILE();
  IF ShowAllFiles | !FileName THEN
   L_AFM.MakeIndex("_Show", "_Show: STRING[70];");
   FOR_EACH curTable IN DATABASE.Get().Tables DO
    PROMPT(curTable.Name);
    IF curTable.Regular &
       (!Cond | Cond(curTable.Name)) THEN
     _Show._Show:=Str.Align(curTable.Name, 8, LEFT) + " " + curTable.Description;
     INSERT(_Show);
    END;
   END; (* FOR_EACH curTable  *)
   PShow  :="";
   Pos    :=0;
   i      :=0;
   SELECT(_Show, _Show);
   WHILE OCCURE(_Show) DO
    PROMPT(_Show._Show);
    IF ~!FileName & (Str.Item(_Show._Show, " ", 0) = FileName) THEN
     Pos:=i;
    END;
    PShow:=PShow + _Show._Show + "|";
    NEXT(_Show);
    INC(i);
   END; (* WHILE OCCURE(_Show) ... *)
   L_AFM.AF_PurgeTemp("_Show");
   Pos     :=L_StdFrm.ShowListAsMenu_wInit(PShow, Title, Pos);
   FileName:=Str.Item(L_Str.PoolItem(PShow, Pos), " ", 0);
  END;
  RETURN FileName;
 END {_ChooseFileName};

 PUBLIC PROCEDURE ChooseFileName(Title: STRING[]; ShowAllFiles: SCAL[2]): STRING[];
 BEGIN
  RETURN _ChooseFileName(Title, EMPTYOF(PROC), ShowAllFiles);
 END {ChooseFileName};

 PRIVATE PROCEDURE CollateFieldName(trgFields: TABLEFIELDS ; FieldNames: STRING[][]): STRING[];
  VAR
   FieldName: STRING[];
 BEGIN
  FOR_EACH FieldName IN FieldNames DO
   IF ~!trgFields(FieldName) THEN
    RETURN FieldName;
   END;
  END;
  RETURN "";
 END {CollateFieldName};

 PUBLIC PROCEDURE CollateValidityFieldNames(FileName: STRING[]; VAR BDField, EDField: STRING[]);
 BEGIN
  BDField:=CollateFieldName(TABLE.Get(FileName).Fields, ["Begin", "BegDate", "Date", "StartD", "FromD", "From"]);
  EDField:=CollateFieldName(TABLE.Get(FileName).Fields, ["Till", "EndDate", "DateE", "StopD", "UpDate", "UpTo"]);
  IF !BDField | !EDField THEN
   SETEMPTY(BDField, EDField);
  END;
 END {CollateValidityFieldNames};

 PRIVATE PROCEDURE intFileHasValidityFields(FileName, BDField, EDField: STRING[]): tFileRecordsValidityType;
 BEGIN
  IF ~!BDField & ~!EDField THEN
   IF (EDField = "Till") | (EDField = "UpDate") | (FileName = "Car") THEN
    RETURN frvt_Normal;
   ELSE
    RETURN frvt_wIncED;
   END;
  ELSE
   RETURN frvt_None;
  END;
 END {intFileHasValidityFields};

 PRIVATE PROCEDURE xFileHasValidityFields(FileName: STRING[]; VAR BDField, EDField: STRING[]): tFileRecordsValidityType;
 BEGIN
  CollateValidityFieldNames(FileName, BDField, EDField);
  RETURN intFileHasValidityFields(FileName, BDField, EDField);
 END {xFileHasValidityFields};

 PUBLIC PROCEDURE FileHasValidityFields(FileName: STRING[]): tFileRecordsValidityType;
  VAR
   dmyBDField,
   dmyEDField: STRING[];
 BEGIN
  RETURN xFileHasValidityFields(FileName, dmyBDField, dmyEDField);
 END {FileHasValidityFields};

 PRIVATE PROCEDURE NeedCheckValidity(curTable, cooTable: TABLE): SCAL[2];
 BEGIN
  RETURN (AMOUNT((cooTable.Name)) / AMOUNT((curTable.Name))) < 100;
 END {NeedCheckValidity};

 PRIVATE PROCEDURE xIsActive(D:DATE; trgFRVT: tFileRecordsValidityType; trgBD, trgED: DATE): SCAL[2];
 BEGIN
  IF    trgFRVT = frvt_Normal THEN
   RETURN L_DatTim.IsActive(D, trgBD, trgED);
  ELSIF trgFRVT = frvt_wIncED THEN
   RETURN L_DatTim.IsActive_Inc(D, trgBD, trgED);
  ELSE
   RETURN 1;
  END;
 END {xIsActive};

 PUBLIC PROCEDURE xRecordIsActive(D: DATE; FileName, BDField, EDField: STRING[]): SCAL[2];
  VAR
   frvt: tFileRecordsValidityType;
 BEGIN
  frvt:=intFileHasValidityFields(FileName, BDField, EDField);
  IF frvt # frvt_None THEN
   RETURN xIsActive(D, frvt, ^(GetFieldName(FileName, BDField)), ^(GetFieldName(FileName, EDField)));
  ELSE
   RETURN 1;
  END;
 END {xRecordIsActive};

 PUBLIC PROCEDURE RecordIsActive(D: DATE; FileName: STRING[]): SCAL[2];
  VAR
   BDField, EDField: STRING[];
 BEGIN
  CollateValidityFieldNames(FileName, BDField, EDField);
  RETURN xRecordIsActive(D, FileName, BDField, EDField);
 END {RecordIsActive};

 PUBLIC PROCEDURE xRecordIsActiveInIntvl(IntvlB, IntvlE: DATE; FileName, BDField, EDField: STRING[]): SCAL[2];
  VAR
   frvt: tFileRecordsValidityType;
 BEGIN
  frvt:=intFileHasValidityFields(FileName, BDField, EDField);
  IF    frvt = frvt_Normal THEN
   RETURN L_DatTim.IsActiveInIntvl(IntvlB, IntvlE, ^(GetFieldName(FileName, BDField)), ^(GetFieldName(FileName, EDField)));
  ELSIF frvt = frvt_wIncED THEN
   RETURN L_DatTim.IsActiveInIntvl_Inc(IntvlB, IntvlE, ^(GetFieldName(FileName, BDField)), ^(GetFieldName(FileName, EDField)));
  ELSE
   RETURN 1;
  END;
 END {xRecordIsActiveInIntvl};

 PRIVATE PROCEDURE GetMatchED(trgFRVT: tFileRecordsValidityType; trgED: DATE): DATE;
 BEGIN
  IF trgFRVT = frvt_Normal THEN
   RETURN L_DatTim.till2ED(trgED);
  ELSE
   RETURN trgED;
  END;
 END {GetMatchED};

 PRIVATE PROCEDURE cfs_CreateIndex(srcFileName, srcIndexName, srcLinkName, dstFileName: STRING[]);
  VAR
   srcIndexFieldList: STRING[];
   i                : NUMERIC[0];
 BEGIN
  SETEMPTY(srcIndexFieldList);
  i:=0;
  WHILE i < DBD.IndexFieldAmount(^srcFileName, ^srcIndexName) DO
   srcIndexFieldList:=L_Str.AddStrDlm(srcIndexFieldList, ",") + DBD.IndexFieldName(^srcFileName, ^srcIndexName, i);
   INC(i);
  END;
  IF ~!srcLinkName THEN
   CreateINDEXandLINK(^dstFileName,
                      ^srcIndexName,
                      ^(L_Str.PoolItem("PLURAL|UNIQUE|AUTOINCR", DBD.IndexUnique(^srcFileName, ^srcIndexName))),
                      ^(DBD.LinkCooFileName(^srcFileName, ^srcLinkName)),
                      ^(DBD.LinkCooIndexName(^srcFileName, ^srcLinkName)),
                      ^(L_Str.PoolItem("⮩ |USER|BORROW|HISTORY", DBD.LinkKind(^srcFileName, ^srcLinkName))),
                      DBD.IndexDescription(^srcFileName, ^srcIndexName),
                      DBD.IndexComment(^srcFileName, ^srcIndexName),
                      ^(srcIndexFieldList));
  ELSE
   CreateINDEX(^dstFileName,
               ^srcIndexName,
               ^(L_Str.PoolItem("PLURAL|UNIQUE|AUTOINCR", DBD.IndexUnique(^srcFileName, ^srcIndexName))),
               DBD.IndexDescription(^srcFileName, ^srcIndexName),
               DBD.IndexComment(^srcFileName, ^srcIndexName),
               ^(srcIndexFieldList));
  END;
 END {cfs_CreateIndex};

 PUBLIC PROCEDURE CloneFileStructure(srcFileName, dstFileName: STRING[]; wLinks: SCAL[2]);
  VAR
   i, LinkIdx   : NUMERIC[0];
   LinkList,
   srcLinkName,
   srcFieldName,
   srcIndexName,
   LinkIndexList: STRING[];
 BEGIN
  CreateFILE(^dstFileName, "", DBD.FileDescription(^srcFileName), "");
  i:=0;
  WHILE i < DBD.FieldAmount(^srcFileName) DO
   srcFieldName:=DBD.FieldName(^srcFileName, i);
   CreateFIELD(^dstFileName,
               ^srcFieldName,
               ^(TYPENAMEOF(^(DBD.FieldFullName(^srcFileName, i)))),
               DBD.FieldMin(^srcFileName, ^srcFieldName),
               DBD.FieldDefault(^srcFileName, ^srcFieldName),
               DBD.FieldMax(^srcFileName, ^srcFieldName),
               DBD.FieldDescription(^srcFileName, ^srcFieldName),
               DBD.FieldComment(^srcFileName, ^srcFieldName));
   INC(i);
  END; (* WHILE i < DBD.FileAmount(^srcFileName)...*)

  SETEMPTY(LinkList, LinkIndexList);
  IF wLinks THEN
   i:=0;
   WHILE i < DBD.LinkAmount(^srcFileName) DO
    srcLinkName:=DBD.LinkName(^srcFileName, i);
    IF (DBD.LinkKind(^srcFileName, ^srcLinkName) = LINK.Kind_User)   |
       (DBD.LinkKind(^srcFileName, ^srcLinkName) = LINK.Kind_Borrow) |
       (DBD.LinkKind(^srcFileName, ^srcLinkName) = LINK.Kind_History) THEN
     LinkList     :=LinkList      + srcLinkName + "|";
     LinkIndexList:=LinkIndexList + DBD.LinkIndexName(^srcFileName, ^srcLinkName) + "|";
    END;
    INC(i);
   END; (* WHILE i < DBD.LinkAmount(^srcFileName)...*)
  END;

  i:=0;
  WHILE i < DBD.IndexAmount(^srcFileName) DO
   srcIndexName:=DBD.IndexName(^srcFileName, i);
   LinkIdx:=L_Str.PositionInPool(LinkIndexList, srcIndexName);
   IF ~!LinkIdx THEN
    cfs_CreateIndex(srcFileName, srcIndexName, L_Str.PoolItem(LinkList, LinkIdx), dstFileName);
   ELSE
    cfs_CreateIndex(srcFileName, srcIndexName, "", dstFileName);
   END;
   INC(i);
  END; (* WHILE i < DBD.IndexAmount(^srcFileName)...*)
  SET_VERSION_OF_TABLE(^dstFileName, DBD.GetVersionOfFILE(^srcFileName));
 END {CloneFileStructure};

 PRIVATE CONST
  MaxParentsAmount = 10;

 PRIVATE PROCEDURE GetAllParentNames(TableName: STRING[]): STRING[][MaxParentsAmount];
  VAR
   Result   : STRING[][MaxParentsAmount];
   S,
   IndexName: STRING[];
   Cnt,
   curIdx   : NUMERIC[0];
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   SETEMPTY(Result);
   SETZERO(Cnt, curIdx);
   IndexName:=GetPrimaryIndex(TableName);
   WHILE ~!(^TableName.Parent) DO
    SEARCH(^TableName, ^IndexName, , ^TableName.Parent);
    Result[Cnt]:=^TableName.Name;
    INC(Cnt);
   END;
   WHILE curIdx < Cnt/2 DO
    S:=Result[curIdx];
    Result[curIdx]:=Result[Cnt-curIdx-1];
    Result[Cnt-curIdx-1]:=S;
    INC(curIdx);
   END;
  EXCEPT
  END; (* TRY *)
  RETURN Result;
 END {GetAllParentNames};

 PUBLIC PROCEDURE BuildHierarchyString(TableName: STRING[]; Width, Height: NUMERIC[0]): STRING[];
  VAR
   Arr    : STRING[][MaxParentsAmount];
   S      : STRING[];
   srcIdx,
   dstIdx,
   Pos    : NUMERIC[0];
   Offset : SCAL[2];
 BEGIN
  SETEMPTY(S);
  Arr:=GetAllParentNames(TableName);
  IF !Arr[0] THEN
   RETURN S;
  END;

  LOOP
   SETZERO(srcIdx, dstIdx);
   S:="";
   WHILE (srcIdx <=HIGH(Arr)) & ~!Arr[srcIdx] DO
    IF Str.Length(S + Arr[srcIdx]) <= Width - dstIdx - 5 THEN
     IF !S THEN
      S:=Arr[srcIdx];
     ELSE
      S:=S + "  " + Arr[srcIdx];
     END;
    ELSE
     IF !S THEN
      Arr[dstIdx]:=Arr[srcIdx];
     ELSE
      Arr[dstIdx]:=S;
      S:=Arr[srcIdx];
     END;
     INC(dstIdx);
    END;
    INC(srcIdx);
   END;
   IF ~!S THEN
    Arr[dstIdx]:=S;
    INC(dstIdx);
   END;
 
   srcIdx:=dstIdx;
   WHILE (srcIdx <= HIGH(Arr)) & ~!Arr[srcIdx] DO
    SETEMPTY(Arr[srcIdx]);
    INC(srcIdx);
   END;
 
   srcIdx:=0;
   IF dstIdx > Height THEN
    Pos:=Str.Pos(Arr[0], "");
    Offset:=(Pos = Str.Length(Arr[0]));
    WHILE (srcIdx <= HIGH(Arr)) & ~!Arr[srcIdx+Offset] DO
     Arr[srcIdx]:=Arr[srcIdx+Offset];
     INC(srcIdx);
    END;
    SETEMPTY(Arr[srcIdx]);
    IF ~Offset THEN
     Arr[0]:=Str.Slice(Arr[0], Pos+2, Str.Length(Arr[0])-Pos-2);
    END;
    IF L_Str.Char(Arr[srcIdx-1], 0) # ">" THEN
     DEC(srcIdx);
     WHILE srcIdx >=0 DO
      Arr[srcIdx]:=">" + Arr[srcIdx];
      DEC(srcIdx);
     END;
    END;
   ELSE
    S:="";
    WHILE (srcIdx <= HIGH(Arr)) & ~!Arr[srcIdx] DO
     IF L_Str.Char(Arr[srcIdx], 0) = ">" THEN
      IF srcIdx THEN
       Arr[srcIdx]:=L_Str.Repeat(" ", srcIdx) + " " + Str.Slice(Arr[srcIdx], 1, Str.Length(Arr[srcIdx])-1);
      ELSE
       Arr[srcIdx]:="> " + Str.Slice(Arr[srcIdx], 1, Str.Length(Arr[srcIdx])-1);
      END;
     ELSE
      IF srcIdx THEN
       Arr[srcIdx]:=L_Str.Repeat(" ", srcIdx) + " " + Arr[srcIdx];
      END;
     END;
     IF Str.Length(Arr[srcIdx]) > Width THEN
      S:=S + Str.Slice(Arr[srcIdx], 0, Width-3) + "~> ";
     ELSE
      S:=S + Arr[srcIdx] + L_Str.Repeat(" ", Width-Str.Length(Arr[srcIdx]));
     END;
     INC(srcIdx);
    END;
    EXIT;
   END; (* IF dstIdx > Height ... *)
  END; (* LOOP *)

  RETURN S;
 END {BuildHierarchyString};

 PUBLIC PROCEDURE RecordToStr(curTable: TABLE): STRING[];
  VAR
   Result: STRING[];
   Fields: INDEXFIELDS;
 BEGIN
  Fields:=curTable.Indexes.Item(GetPrimaryIndex(curTable.Name)).Fields;
  IF Fields.Count = 1 THEN
   Result:=^(Fields.Item(0).ObtainName());
   IF ~!curTable.Fields("Name") THEN
    Result:=Result+" "+^(curTable.Fields("Name").ObtainName());
   END;
  ELSE
   Result:=Str.Replace(GetPrimaryIndexValue(curTable.Name), "|", ", ");
  END;
  RETURN Result;
 END {RecordToStr};

 PRIVATE PROCEDURE RecordToFullStr(curTable: TABLE): STRING[];
  VAR
   prmIndex : INDEX;
   curField : FIELD;
   idxFields: INDEXFIELDS;
   Result   : STRING[];
 BEGIN
  Result:="";
  prmIndex:=curTable.Indexes.Item(GetPrimaryIndex(curTable.Name));
  FOR_EACH curField IN prmIndex.Fields DO
   IF ~!Result THEN
    INC(Result, L_Str.CRLF);
   END;
   INC(Result, TableFieldToStr(curField)+" ("+Str.Small(curField.Description)+")");
  END;
  RETURN Result;
 END {RecordToFullStr};

 PRIVATE PROCEDURE GetAnyProTableValidity(curTable: TABLE;
                                          curFRVT: tFileRecordsValidityType;
                                          VAR curBD, curED: DATE);
  VAR
   lndTable: TABLE;
   lndFRVT : tFileRecordsValidityType;
   lndBD,
   lndED   : DATE;
 BEGIN
  IF L_PCS.IsAnyProTable(curTable) & (curFRVT # frvt_None) THEN
   L_General.CallUs(~!curTable.LendLink);
   lndTable:=curTable.LendLink.CooIndex.Table;
   TRY [RESTORE_CURSORS(RC_ON)]
    SelectCooFile(curTable.Name, curTable.LendLink.Name);
    GetSrcFileValidity(lndTable, lndFRVT, lndBD, lndED);
 
    IF ~!lndFRVT & (lndFRVT # frvt_None) THEN
     L_General.CallUs(curFRVT = frvt_wIncED); (*᢮⢠ - ⮫쪮 ""*)
     curED:=Math.Min(GetMatchED(lndFRVT, lndED), curED);
    END;
   EXCEPT
   END; (* TRY *)
  END;
 END {GetAnyProTableValidity};

 PRIVATE PROCEDURE GetCurFileValidity(curTable: TABLE;
                                      VAR valFRVT: tFileRecordsValidityType;
                                      VAR valBD, valED: DATE);
  VAR
   curFRVT: tFileRecordsValidityType;
   BDField,
   EDField: STRING[];
 BEGIN
  curFRVT:=xFileHasValidityFields(curTable.Name, BDField, EDField);
  IF curFRVT # frvt_None THEN
   L_General.CallUs(valFRVT # frvt_None); (* 㬥    borrow  ਮ ਬ*)
   valBD:=^(curTable.Fields(BDField).ObtainName());
   valED:=^(curTable.Fields(EDField).ObtainName());
   valFRVT:=curFRVT;
  END;
 END {GetCurFileValidity};

 PUBLIC PROCEDURE GetSrcFileValidity(    srcTable: TABLE;
                                     VAR srcFRVT: tFileRecordsValidityType;
                                     VAR srcBD, srcED: DATE);
  VAR
   DateField: STRING[];
   curTable : TABLE;
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   SETEMPTY(srcFRVT);
   curTable:=srcTable;
   LOOP
    IF    L_PCS.IsSpecialValidityTable(curTable) THEN
     EXIT;
    ELSIF L_PCS.GetSpecialRecordValidity(curTable, srcFRVT, srcBD, srcED) THEN
     EXIT;
    ELSE
     IF FileHasValidityFields(curTable.Name) # frvt_None THEN
      GetCurFileValidity(curTable, srcFRVT, srcBD, srcED);
     ELSIF !curTable.LendLink THEN
      DateField:=L_PCS.AccountDateFieldName(curTable);
      IF ~!DateField THEN
       srcBD:=^(curTable.Fields(DateField).ObtainName());
       srcED:=EMPTYOF(srcED);
       srcFRVT:=frvt_None;
      END;
     END;
    END; (* IF L_PCS.IsSpecialValidityTable(curTable) ... *)
    IF ~!srcFRVT THEN
     EXIT;
    ELSIF ~!curTable.LendLink THEN
     SelectCooFile(curTable.Name, curTable.LendLink.Name);
     curTable:=curTable.LendLink.CooIndex.Table;
    ELSE
     EXIT;
    END;
   END; (* LOOP ... *)
  EXCEPT
  END;
 END {GetSrcFileValidity};

 PRIVATE PROCEDURE FormattedStr(Header, Msg: STRING[]; Len: NUMERIC[0]): STRING[];
  CONST
   MsgArrHigh = 10;
  VAR
   Result: STRING[];
   MsgArr: STRING[][MsgArrHigh];
   MsgLen: NUMERIC[0];
   i     : NUMERIC[0];
 BEGIN
  MsgLen:=Len-Str.Length(Header);
  Str.Format(MsgArr, Msg, MsgLen, LEFT);

  Result:=Str.Align(Header+MsgArr[0], Len, LEFT);
  i:=1;
  WHILE ~!MsgArr[i] DO
   Result:=Result+"|"+Str.Align(Str.Align(MsgArr[i], MsgLen, LEFT), Len, RIGHT);
   INC(i);
  END;
  RETURN Result;
 END {FormattedStr};

 PRIVATE PROCEDURE GetValidityStr(trgFRVT: tFileRecordsValidityType;
                                  BD, ED: DATE): STRING[];
  VAR
   Result: STRING[];
 BEGIN
  Result:=" "+L_DatTim.DateStr(BD);
  IF    trgFRVT = frvt_Normal THEN
   Result:=Result+"  ";
  ELSIF trgFRVT = frvt_wIncED THEN
   Result:=Result+"  ";
  ELSE
   L_General.CallUs(0);
  END;
  IF ~!ED THEN
   Result:=Result+L_DatTim.DateStr(ED);
  ELSE
   Result:=Result+"__________";
  END;
  RETURN Result;
 END {GetValidityStr};

 PUBLIC PROCEDURE extPosToCurRecordOfTable(trgTable: TABLE);
  VAR
   curTable,
   lndTable: TABLE;
 BEGIN
  curTable:=trgTable;
  REPEAT
   TRY [EXCEPTION_HANDLING(EXCEPTION_HANDLING_OFF)]
    L_WDO.PosToCurRecord(curTable.Name, "", 1);
   EXCEPT
   ON Exc: Exc_RunTime DO
   ON Exc: Exc_Raise DO
   END;
   (* 㤠 ᯮ樮஢ - ⪠ ᯮ樮஢  lend*)
   IF ~!curTable.LendLink THEN
    lndTable:=curTable.LendLink.CooIndex.Table;
    CONFORM((curTable.Name), (lndTable.Name));
    curTable:=lndTable;
   ELSE
    SETEMPTY(curTable);
   END;
  UNTIL !curTable;
  L_General.Error("  ३   䠩 "+'"'+trgTable.Description+'"');
 END {extPosToCurRecordOfTable};

 PRIVATE PROCEDURE PosToCurRecordOfTable(trgTable: TABLE);
 BEGIN
  CALL(CallStub.c_extPosToCurRecordOfTable(trgTable));
  HALT(1);
 END {PosToCurRecordOfTable};

 PRIVATE PROCEDURE CheckOneRecordValidity(curTable, srcTable, cooTable: TABLE;
                                          curFRVT: tFileRecordsValidityType;
                                          curBD, curED: DATE;
                                          cooIsPattern: SCAL[2];
                                          curTableFullStr: STRING[]);
  CONST
   MsgWidth = 73;
  VAR
   Msg,
   MustStr,
   Recomendation,
   Buttons: STRING[];
   patTable,
   incTable: TABLE;
   patBD,
   patED,
   incBD,
   incED,
   srcBD,
   srcED,
   cooBD,
   cooED  : DATE;
   incFRVT,
   patFRVT,
   srcFRVT,
   cooFRVT: tFileRecordsValidityType;
   cn,
   in     : NUMERIC[0];
   CanGoTo: SCAL[2];
 BEGIN
  IF ~L_PCS.IsSpecialValidityRecord(cooTable) THEN
   APP.Get().NewPrompt(cooTable.Description);
   PROMPT(RecordToStr(cooTable));
   GetSrcFileValidity(cooTable, cooFRVT, cooBD, cooED);
   IF ~!cooFRVT THEN
    Msg:="";
    Recomendation:="";

    IF curFRVT = frvt_None (*  *) THEN
     CanGoTo:=0;
     IF cooIsPattern THEN
      IF curBD < cooBD THEN
       Msg:=FormattedStr("",
                         "⭠      ("+L_DatTim.DateStr(curBD)+")    "+L_Str.CRLF+
                         " 砫    ("+L_DatTim.DateStr(cooBD)+")",
                         MsgWidth);
      END;
      IF GetMatchED(cooFRVT, cooED) < curBD THEN
       IF    cooFRVT = frvt_Normal THEN MustStr:="  ";
       ELSIF cooFRVT = frvt_wIncED THEN MustStr:="   ";
       ELSE
        L_General.CallUs(0);
       END;
       Msg:=FormattedStr("",
                        "⭠        ("+L_DatTim.DateStr(curBD)+") "+MustStr+L_Str.CRLF+
                        " 砭   ("+L_DatTim.DateStr(cooED)+")",
                        MsgWidth);
      END; (* IF curBD < cooBD ... *)
     END; (* IF cooIsPattern ... *)
    ELSIF cooFRVT = frvt_None THEN
     CanGoTo:=0;
     L_General.CallUs(~cooIsPattern);
     IF curBD > cooBD THEN
      Msg:=FormattedStr("",
                        " 砫      ("+L_DatTim.DateStr(curBD)+")    "+L_Str.CRLF+
                        "⭮    ("+L_DatTim.DateStr(cooBD)+")",
                        MsgWidth);
     END;
     IF GetMatchED(curFRVT, curED) < cooBD THEN
      IF    curFRVT = frvt_Normal THEN MustStr:="  ";
      ELSIF curFRVT = frvt_wIncED THEN MustStr:="   ";
      ELSE
       L_General.CallUs(0);
      END;
      Msg:=FormattedStr("",
                        " 砭     ("+L_DatTim.DateStr(curED)+") "+MustStr+L_Str.CRLF+
                        "⭮      ("+L_DatTim.DateStr(cooBD)+")",
                        MsgWidth);
     END; (* IF curBD < cooBD ... *)
    ELSE
     CanGoTo:=1;
     IF cooIsPattern THEN
      patFRVT :=cooFRVT;   incFRVT :=curFRVT;
      patBD   :=cooBD;     incBD   :=curBD;
      patED   :=cooED;     incED   :=curED;
      patTable:=cooTable;  incTable:=curTable;
     ELSE
      patFRVT :=curFRVT;   incFRVT :=cooFRVT;
      patBD   :=curBD;     incBD   :=cooBD;
      patED   :=curED;     incED   :=cooED;
      patTable:=curTable;  incTable:=cooTable;
     END;
     incBD:=Math.Max(incBD, L_Env.aptm_Start);
     IF incBD > GetMatchED(patFRVT, patED) THEN
      Recomendation:=Recomendation+
                     FormattedStr("",
                                  " 砫 ਮ "+L_Str.PoolItem("|", cooIsPattern)+
                                  "      砭 ਮ ਬ "+L_Str.PoolItem("|", cooIsPattern)+" ;",
                                  MsgWidth)+"|";
     ELSE
      IF incBD < patBD THEN
       Recomendation:=Recomendation+
                      FormattedStr("",
                                   " 砫 ਮ "+L_Str.PoolItem("|", cooIsPattern)+
                                   "       砫 "+L_Str.PoolItem("|", cooIsPattern)+" ;",
                                   MsgWidth)+"|";
      END;
      IF L_PCS.IsAnyProTable(incTable) THEN
       GetAnyProTableValidity(incTable, incFRVT, incBD, incED);
      END;
      IF GetMatchED(incFRVT, incED) > GetMatchED(patFRVT, patED) THEN
       IF incFRVT # patFRVT THEN
        IF    (incFRVT = frvt_wIncED) & (patFRVT = frvt_Normal) THEN
         MustStr:="  ";
        ELSIF (incFRVT = frvt_Normal) & (patFRVT = frvt_wIncED) THEN
         L_General.CallUs(0); (*㬥  ᢮⢠  㯠 ࠢ筨*)
        ELSE
         L_General.CallUs(0);
        END;
       ELSE
        MustStr:="   ";
       END;
       Recomendation:=Recomendation+
                      FormattedStr("",
                                   " 砭 ਮ "+L_Str.PoolItem("|", cooIsPattern)+
                                   "  "+MustStr+"  砭 "+L_Str.PoolItem("|", cooIsPattern)+" ;",
                                   MsgWidth)+"|";
      END; (* IF GetMatchED(incFRVT, incED) > ... *)
     END;
     IF ~!Recomendation THEN
      Msg:="ᮣᮢ ਮ ਬ ᥩ:|"+
           "⥪   "+GetValidityStr(curFRVT, curBD, curED)+"|"+
           "易 "+GetValidityStr(cooFRVT, cooBD, cooED);
     END;
    END;
    IF ~!Msg THEN
     Msg:=Msg + L_StdFrm.HR() +
          FormattedStr(":   ", curTable.Description, MsgWidth) + "|" +
          FormattedStr("           ", curTableFullStr, MsgWidth);
 
     Msg:=Msg + "||" +
          FormattedStr("易: ", cooTable.Description, MsgWidth) + "|" +
          FormattedStr("           ", RecordToFullStr(cooTable), MsgWidth);

     IF ~!Recomendation THEN
      Msg:=Msg+L_StdFrm.THR(" ᫥  冷")+Recomendation;
     END;
     Msg:=Msg+L_StdFrm.THR(" ")+
              FormattedStr("",
                           "室  易    ᫥ 䨪"
                           +"  (।,  ) ਢ  䫨  䨪樥 ⥪饩 ",
                           MsgWidth);

     Buttons:=L_StdFrm.DrawButton("⮯!", 9, "X");
     IF CanGoTo THEN
      Buttons:=Buttons+" "+L_StdFrm.DrawButton("믮", 11, "DB");
     END;
     Buttons:=Buttons+" "+L_StdFrm.DrawButton("{}३  易 ", 28, "B");
     IF STDFORM(Msg+"||"+Buttons, " ஢ઠ ਮ ਬ ", STOP, 1) = (1+CanGoTo) THEN
      PosToCurRecordOfTable(cooTable);
     END;
    END;
   END; (* IF ~!cooFRVT ... *)
  END; (* IF ~L_PCS.IsSpecialValidityRecord(cooTable) ... *)
 END {CheckOneRecordValidity};

 PRIVATE PROCEDURE intCheckR2ROwnersValidity(curTable,
                                             srcTable : TABLE;
                                             curFRVT  : tFileRecordsValidityType;
                                             curBD,
                                             curED    : DATE);
  VAR
   riField : FIELD;
   ownTable: TABLE;
   curTFS  : STRING[];
 BEGIN
  riField:=curTable.Fields("RECORDID");
  IF ~!riField &
     SELECT(r2r, r2r, curTable.Id, ^(riField.ObtainName())) THEN
   curTFS:=RecordToFullStr(curTable);
   FOR_EACH r2r DO
    ownTable:=GetTable(r2r.fiOwn);
    IF FileHasValidityFields(ownTable.Name) # frvt_None THEN
     TRY [RESTORE_CURSORS(RC_ON)]
      SEARCH((ownTable.Name), RECORDID, , r2r.riOwn);
      CheckOneRecordValidity(curTable, srcTable, ownTable, curFRVT, curBD, curED, 1, curTFS);
     EXCEPT
     END; (* TRY *)
    END; (* IF FileHasValidityFields ... *)
   END; (* FOR_EACH r2r *)
  END; (* IF ~!riField ... *)
 END {intCheckR2ROwnersValidity};

 PRIVATE PROCEDURE intCheckR2RUsersValidity(curTable,
                                            srcTable: TABLE;
                                            curFRVT : tFileRecordsValidityType;
                                            curBD,
                                            curED   : DATE);
  VAR
   riField : FIELD;
   usrTable: TABLE;
   curTFS  : STRING[];
   usrFRVT : tFileRecordsValidityType;
   dmyBD,
   dmyED : DATE;
 BEGIN
  riField:=curTable.Fields("RECORDID");
  IF ~!riField &
     SELECT(r2r, byOwn, curTable.Id, ^(riField.ObtainName())) THEN
   curTFS:=RecordToFullStr(curTable);
   FOR_EACH r2r DO
    usrTable:=GetTable(r2r.fiUsr);
    IF NeedCheckValidity(curTable, usrTable) THEN
     TRY [RESTORE_CURSORS(RC_ON)]
      SEARCH((usrTable.Name), RECORDID, , r2r.riUsr);
      GetSrcFileValidity(usrTable, usrFRVT, dmyBD, dmyED);
      IF ~!usrFRVT & (usrFRVT # frvt_None) THEN (*  ஢塞 ᯮ짮 ⥪饩   㬥*)
       CheckOneRecordValidity(curTable, srcTable, usrTable, curFRVT, curBD, curED, 0, curTFS);
      END;
     EXCEPT
     END; (* TRY *)
    END; (* IF NeedCheckValidity( ... *)
   END; (* FOR_EACH r2r *)
  END; (* IF ~!riField ... *)
 END {intCheckR2RUsersValidity};

 PRIVATE PROCEDURE intCheckOwnersValidity(curTable,
                                          srcTable : TABLE;
                                          curFRVT  : tFileRecordsValidityType;
                                          curBD,
                                          curED    : DATE);
  VAR
   curLink : LINK;
   lndTable,
   brwTable,
   ownTable: TABLE;
   curTFS  : STRING[];
 BEGIN
  IF ~L_PCS.IsSpecialValidityRecord(curTable) THEN
   GetCurFileValidity(curTable, curFRVT, curBD, curED);
   IF (curFRVT = frvt_Normal) & (curBD = curED) THEN
    RETURN; (* ஢塞    砫*)
   END;
   APP.Get().NewPrompt(curTable.Description);
   PROMPT(RecordToStr(curTable));
   curTFS:=RecordToFullStr(curTable);
   FOR_EACH curLink IN curTable.Links DO
    IF ~L_PCS.IsSpecialValidityLink(curLink) THEN
     IF    (curLink.Kind = LINK.Kind_Borrow) | (curLink.Kind = LINK.Kind_History) THEN
      lndTable:=curLink.CooIndex.Table;
      IF (curTable = srcTable (* 室 䠩. 稭    ⠪ ஢  ⢥ borrow*)) THEN
       IF ~!curFRVT &
          ~L_PCS.IsSpecialValidityTable(lndTable) THEN
        SelectCooFile(curTable.Name, curLink.Name);
        CheckOneRecordValidity(curTable, srcTable, lndTable, curFRVT, curBD, curED, 1, curTFS);
       END;
      END;
     ELSIF (curLink.Kind = LINK.Kind_Lend) | (curLink.Kind = LINK.Kind_ReverseHistory) THEN
      brwTable:=curLink.CooIndex.Table;
      IF ~L_PCS.IsAccumBorrow(brwTable) THEN
       SelectCooFile(curTable.Name, curLink.Name);
       FOR_EACH ^(brwTable.Name) DO
        IF FileHasValidityFields(brwTable.Name) # frvt_None THEN
         CheckOneRecordValidity(curTable, srcTable, brwTable, curFRVT, curBD, curED, 0, curTFS);
        END;
        intCheckOwnersValidity(brwTable, srcTable, curFRVT, curBD, curED);
       END; (* FOR_EACH ^(brwTable.Name) *)
      END; (* IF ~IsAccumBorrow(brwTable) ... *)
     ELSIF (curLink.Kind = LINK.Kind_User) THEN
      ownTable:=curLink.CooIndex.Table;
      IF ~L_PCS.IsSpecialValidityTable(ownTable) THEN
       IF FileHasValidityFields(ownTable.Name) # frvt_None THEN
        TRY [RESTORE_CURSORS(RC_ON)]
         SelectCooFile(curTable.Name, curLink.Name);
         IF OCCURE((ownTable.Name)) THEN
          CheckOneRecordValidity(curTable, srcTable, ownTable, curFRVT, curBD, curED, 1, curTFS);
         END;
        EXCEPT
        END; (* TRY *)
       END;
      END; (* IF ~IsSpecialValidityTable() ... *)
     END; (* IF curLink.Kind = ... *)
    END; (* IF ~L_PCS.IsSpecialValidityLink(curLink) ... *)
   END; (* FOR_EACH curLink ... *)
   intCheckR2ROwnersValidity(curTable, srcTable, curFRVT, curBD, curED);
  END; (* IF ~L_PCS.IsSpecialValidityRecord(curTable) ... *)
 END {intCheckOwnersValidity};

 PUBLIC PROCEDURE CheckOwnersValidity(curFileName: STRING[]);
  VAR
   curTable: TABLE;
   curFRVT : tFileRecordsValidityType;
   curBD,
   curED   : DATE;
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   L_AR.GrantRightsUnconditionally();
   RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
   APP.Get().NewPrompt("஢ઠ ਮ ਬ ࠢ筨");
   curTable:=TABLE.Get(curFileName);
   IF ~L_PCS.IsSpecialValidityTable(curTable) & ~L_PCS.IsSpecialValidityRecord(curTable) THEN
    GetSrcFileValidity(curTable, curFRVT, curBD, curED);
    IF ~!curFRVT THEN
     IF (curFRVT # frvt_None) THEN
      L_General.Assert(curBD <= curED, " 砭 ਮ ਬ ⥪饩 |   ࠭  砫");
     END;
     intCheckOwnersValidity(curTable, curTable, curFRVT, curBD, curED);
    END;
   END;
  EXCEPT
  END; (* TRY *)
 END {CheckOwnersValidity};

 PRIVATE PROCEDURE intCheckUsersValidity(curTable,
                                         srcTable: TABLE;
                                         curFRVT : tFileRecordsValidityType;
                                         curBD,
                                         curED   : DATE);
  VAR
   usrTable: TABLE;
   curLink : LINK;
   usrFRVT : tFileRecordsValidityType;
   dummyBD,
   dummyED : DATE;
   curTFS  : STRING[];
 BEGIN
  APP.Get().NewPrompt(curTable.Description);
  PROMPT(RecordToStr(curTable));
  curTFS:=RecordToFullStr(curTable);
  FOR_EACH curLink IN curTable.Links DO
   IF ~L_PCS.IsSpecialValidityLink(curLink) THEN
    IF (curLink.Kind = LINK.Kind_Owner) THEN
     usrTable:=curLink.CooIndex.Table;
     IF ~L_PCS.IsSpecialValidityTable(usrTable) &
        NeedCheckValidity(curTable, usrTable) THEN
      TRY [RESTORE_CURSORS(RC_ON)]
       SelectUsers(curTable.Name, curLink.Name);
       IF OCCURE((usrTable.Name)) THEN
        GetSrcFileValidity(usrTable, usrFRVT, dummyBD, dummyED);
        IF ~!usrFRVT & (usrFRVT # frvt_None) THEN (*  ஢塞 ᯮ짮 ⥪饩   㬥*)
         FOR_EACH ^(usrTable.Name) DO
          CheckOneRecordValidity(curTable, srcTable, usrTable, curFRVT, curBD, curED, 0, curTFS);
         END; (* FOR_EACH ^(usrTable.Name) *)
        END;
       END; (* IF OCCURE((usrTable.Name)) ... *)
      EXCEPT
      END; (* TRY *)
     END; (* IF ~L_PCS.IsSpecialValidityTable(usrTable) ... *)
    END; (* IF curLink.Kind = LINK.Kind_Owner ... *)
   END; (* IF ~L_PCS.IsSpecialValidityLink(curLink) ... *)
  END; (* FOR_EACH curLink ... *)
  intCheckR2RUsersValidity(curTable, srcTable, curFRVT, curBD, curED);
 END {intCheckUsersValidity};

 PUBLIC PROCEDURE CheckUsersValidity(curFileName: STRING[]);
  VAR
   curTable: TABLE;
   curFRVT : tFileRecordsValidityType;
   curBD,
   curED   : DATE;
 BEGIN
  TRY [RESTORE_CURSORS(RC_ON)]
   L_AR.GrantRightsUnconditionally();
   RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
   APP.Get().NewPrompt("஢ઠ ਮ ਬ 뫮  ");
   curTable:=TABLE.Get(curFileName);
   GetSrcFileValidity(curTable, curFRVT, curBD, curED);
   IF ~!curFRVT THEN
    IF (curFRVT = frvt_Normal) & (curBD = curED) THEN
     RETURN; (* ஢塞    砫*)
    END;
    intCheckUsersValidity(curTable, curTable, curFRVT, curBD, curED);
   END;
  EXCEPT
  END; (* TRY *)
 END {CheckUsersValidity};

 PUBLIC PROCEDURE CheckValidity(curFileName: STRING[]);
 BEGIN
  CheckOwnersValidity(curFileName);
  CheckUsersValidity(curFileName);
 END {CheckValidity};

 PUBLIC PROCEDURE CheckCurRecordsValidity(curFileName, cooFileName: STRING[]; cooIsPattern: SCAL[2]);
  VAR
   curTable,
   cooTable: TABLE;
   curFRVT : tFileRecordsValidityType;
   curBD,
   curED   : DATE;
 BEGIN
  curTable:=TABLE.Get(curFileName);
  cooTable:=TABLE.Get(cooFileName);
  TRY [RESTORE_CURSORS(RC_ON)]
   RTI.Get().TryBlock.AccessDataMode:=adm_SOFT;
   GetSrcFileValidity(curTable, curFRVT, curBD, curED);
   CheckOneRecordValidity(curTable, curTable, cooTable, curFRVT, curBD, curED, cooIsPattern, RecordToFullStr(curTable));
  EXCEPT
  END;
 END {CheckCurRecordsValidity};

 PUBLIC PROCEDURE FileHasXDocKind(File: STRING[]): SCAL[2];
  VAR
   xdt: L_XDoc.tXDocType;
 BEGIN
  xdt:=L_XDoc.GetFileXDT(File);
  RETURN ~!xdt & L_XDoc.XDocTypeHasXDocKind(xdt);
 END {FileHasXDocKind};

 PUBLIC PROCEDURE CopyFromSrcRecord_wExclude(srcFileName, PrimaryIndexValue, exclFields: STRING[]);
  CONST
   copyFileName = "fcopy";
   xPrf = "!!!";
  VAR
   curf        : FIELD;
   Index,
   trg_Text    : STRING[];
   Fields      : INDEXFIELDS;
   curIndex    : INDEX;
   NameIsUnique: SCAL[2];
   trg_lsStore : LIKE(lsStore.lsStore);
 BEGIN
  Index:=GetPrimaryIndex(srcFileName);
  NameIsUnique:=0;
  FOR_EACH curIndex IN TABLE.Get(srcFileName).Indexes DO
   IF (curIndex.Unique = INDEX.Unique_Unique) & (curIndex.Fields(curIndex.Fields.Count-1).Name = "Name") THEN
    NameIsUnique:=1;
   END;
  END;
  Fields:=TABLE.Get(srcFileName).Indexes(Index).Fields;
  exclFields:=exclFields + "|RECORDID|" + Fields(Fields.Count - 1).Name;
  SAVE((srcFileName));
  IF    Fields.Count = 1 THEN
   SELECT((srcFileName), (Index), , PrimaryIndexValue);
  ELSIF Fields.Count = 2 THEN
   SELECT((srcFileName), (Index), , L_Str.PoolItem(PrimaryIndexValue, 0),
                                    L_Str.PoolItem(PrimaryIndexValue, 1));
  ELSE
   L_General.CallUs(0);
  END;
  L_AFM.AF_MakeDataBaseStructureCopy(srcFileName, copyFileName, "", L_AFM.tdm_CurSelect);
  RESTORE((srcFileName));
  FOR_EACH curf IN TABLE.Get(srcFileName).Fields DO
   IF !L_Str.PositionInPool(exclFields, curf.Name) THEN
    IF    NameIsUnique & (curf.Name = "Name") THEN
     ^(curf.ObtainName()):=xPrf + ^(copyFileName + "." + curf.Name);
    ELSE
     ^(curf.ObtainName()):=^(copyFileName + "." + curf.Name);
    END;

    IF L_PCS.IsLinkTo_lsStore(srcFileName, curf.Name) THEN
     trg_lsStore:=^(curf.ObtainName());
     IF ~!trg_lsStore THEN
      trg_Text:=L_lsStore.Get(trg_lsStore);
      ^(curf.ObtainName()):=L_lsStore.Insert(trg_Text);
     END;
    END;
   END;
  END;
  L_AFM.AF_PurgeTemp(copyFileName);
 END {CopyFromSrcRecord_wExclude};

 PUBLIC PROCEDURE CopyFromSrcRecord(srcFileName, PrimaryIndexValue: STRING[]);
  VAR
   BDField,
   EDField : STRING[];
 BEGIN
  CollateValidityFieldNames(srcFileName, BDField, EDField);
  CopyFromSrcRecord_wExclude(srcFileName, PrimaryIndexValue, L_Str.AddStrDlm(BDField, "|") + EDField);
 END {CopyFromSrcRecord};

 PRIVATE PROCEDURE lr_Begin();
 BEGIN
  L_AFM.MakeMultipleIndex("LabRc", "file: STRING[16]; ri: NUMERIC[0]; pi: STRING[150]",
                          "ri: file, ri, pi; pi: file, pi");
 END {lr_Begin};

 PUBLIC PROCEDURE lr_Remember(File: STRING[]);
  VAR
   riField: FIELD;
 BEGIN
  IF !DATABASE.Get().Tables("LabRc") THEN
   lr_Begin();
  END;
  riField:=TABLE.Get(File).Fields("RECORDID");
  IF ~!riField & !L_PCS.GetPrototypeEntity(File) THEN
   ADD(LabRc, ri, , File, ^(riField.ObtainName()), "");
  ELSE
   ADD(LabRc, ri, , File, EMPTYOF(NUMERIC[0]), GetPrimaryIndexValue(File));
  END;
 END {lr_Remember};

 PUBLIC PROCEDURE lr_Label(File: STRING[]): SCAL[2];
  VAR
   riField: FIELD;
 BEGIN
  IF !DATABASE.Get().Tables("LabRc") THEN
   lr_Begin();
  END;
  riField:=TABLE.Get(File).Fields("RECORDID");
  IF ~!riField & !L_PCS.GetPrototypeEntity(File) THEN
   RETURN SELECT(LabRc, ri, File, ^(riField.ObtainName()));
  ELSE
   RETURN SELECT(LabRc, pi, File, GetPrimaryIndexValue(File));
  END;
 END {lr_Label};

 PUBLIC PROCEDURE lr_Forget(File: STRING[]);
 BEGIN
  IF lr_Label(File) THEN
   REMOVE(LabRc);
  END;
 END {lr_Forget};

 PRIVATE PROCEDURE lr_PosToCur(File: STRING[]);
 BEGIN
  IF ~!LabRc.ri THEN
   L_General.CallUs(SEARCH(^File, RECORDID, LabRc.ri));
  ELSE
   L_General.CallUs(SelectByIndexValue(File, GetPrimaryIndex(File), LabRc.pi));
  END;
 END {lr_PosToCur};

 PUBLIC PROCEDURE lr_PosToFirst(File: STRING[]): SCAL[2];
  VAR
   oc: SCAL[2];
 BEGIN
  oc:=SELECT(LabRc, ri, File);
  IF oc THEN
   lr_PosToCur(File);
  END;
  RETURN oc;
 END {lr_PosToFirst};

 PUBLIC PROCEDURE lr_PosToNext(File: STRING[]): SCAL[2];
  VAR
   oc: SCAL[2];
 BEGIN
  oc:=NEXT(LabRc);
  IF oc THEN
   lr_PosToCur(File);
  END;
  RETURN oc;
 END {lr_PosToNext};

END.
