(*
%Copyright   (c) Polak IT
%Author      29.07.2010 19:38:26 LDA
%Purpose     ⨢ ﭨ
%Type        report
%Specific    LOC
%Modified    10.05.2012 19:30:49 POLAK\LDA
%Environment
*)

 MODULE Rpr26 [DEFAULTACCESSLEVELFORPROCEDURE(PRIVATE)];

 IMPORT L_Rpr, L_StdFrm, L_Str, L_OwnData, L_AFM, L_Str, Str, P_Rpr, L_General, L_DatTim, L_CUC, L_XDoc,
        L_MathR, L_Car, Memo;

 PRIVATE TYPE
  tMode = SCAL[3];

 PRIVATE CONST
  ReportColumnWidth = 165;
  k_Repair     = 0;
  k_LongRepair = 1;
  k_Ready      = 2;

  k_Pool = "|⥫ ६|⮢";

  m_CurState   = 0;
  m_Last2Month = 1;
  m_Period     = 2;

 PRIVATE VAR
  Amount: NUMERIC[0][3];

(*
$P{ Top:
L_OwnData.ODFirmName(SYSDATE()), L, ""
SYSDATE(), L, "D.N.y"
SYSTIME(), L, "H:M"
?, L, ""
?, C, ""
@
#############################################################################################                                                          ########## #####
=#####=================================================================================================================================================================
|#####################################################################################################################################################################|
}
*)

(*
$P{ SubTop:
?, C, ""
?, R, ""
@
|---------------------------------------------------------------------------------------------------------------------------------------------------------------------|
|############################################################################################################################################################## ######|
|---------------------------------------------------------------------------------------------------------------------------------------------------------------------|
}
*)

(*
$P{ Line:
?, L, ""
@
|#####################################################################################################################################################################|
}
*)

(*
$P{ Bot:
@
=(c) Polak=============================================================================================================================================================
}
*)

 PRIVATE PROCEDURE DateTimeStr(Date: DATE; Time: TIME): STRING[];
 BEGIN
  RETURN FLDTOSTR(Date, "D.N.Y") + " " + L_DatTim.TimeStr(Time);
 END {DateTimeStr};

 PRIVATE PROCEDURE GetRprTime(): NUMERIC[3];
 BEGIN
  RETURN (L_DatTim.GetSysTIMESTAMP() - L_DatTim.GetTIMESTAMP(RprS.OrdDate, RprS.OrdTime)) * 24;
 END {GetRprTime};

 PRIVATE PROCEDURE GetSingleLine(wFirmName: SCAL[2]): STRING[];
  VAR
   RprSTaskLen     : NUMERIC[3];
   plnRprSEndDTS   : FLOAT;
   plnRprSEndDate  : DATE;
   plnRprSEndTime  : TIME;
   Line,
   RprObjIdDescr,
   RprSOrdDTAsStr,
   RprSTaskLenAsStr: STRING[];
 BEGIN
  SETEMPTY(Line);
  SEARCH(RprS, RprS, , _RprSL.RprS);


  IF _RprSL.Kind = k_Ready THEN
   RprSTaskLenAsStr:=DateTimeStr(RprS.EDate, RprS.ETime);
  ELSE
   RprSTaskLen:=L_Rpr.GetRprSTaskLen();
   IF ~!RprSTaskLen THEN
    plnRprSEndDTS   :=L_DatTim.GetVARIANTDATE(RprS.OrdDate, RprS.OrdTime, 00) + RprSTaskLen/24;
    plnRprSEndDate  :=L_DatTim.GetDateOfVARIANTDATE(plnRprSEndDTS);
    plnRprSEndTime  :=L_DatTim.GetTimeOfVARIANTDATE(plnRprSEndDTS);
    RprSTaskLenAsStr:=Str.Align(FLDTOSTR(RprSTaskLen, "2."), 6, RIGHT) + " " + DateTimeStr(plnRprSEndDate, plnRprSEndTime);
   ELSE
    RprSTaskLenAsStr:="      " + " " + "              ";
   END;
  END;

  IF    RprS.Location = L_Rpr.rprl_MaintenanceZone THEN
   IF L_Rpr.IsRprS_w_OutCar() THEN
    CONFORM(RprS, CUCopy);
    L_CUC.AccessOrigin();
    IF wFirmName THEN
     CONFORM(OutCar, Firm);
     RprObjIdDescr:=Firm.Name;
    ELSE
     L_Rpr.AccessMark(RprS.OrdDate);
     RprObjIdDescr:=Mark.Name;
    END;
   ELSE
    SETEMPTY(RprObjIdDescr);
   END;
  ELSIF RprS.Location = L_Rpr.rprl_Workshop THEN
   CONFORM(RprS, CUCopy);
   L_CUC.AccessOrigin();

   IF L_Rpr.IsRprS_w_CarBlock() THEN
    RprObjIdDescr:=CUCopy.Serial + " " + Math.Name;
   ELSE
    CONFORM(OutCBlc, Math);
    RprObjIdDescr:=CUCopy.Serial + " " + Math.Name;
    IF wFirmName THEN
     CONFORM(OutCBlc, Firm);
     RprObjIdDescr:=RprObjIdDescr + " (" + Firm.Name + ")";
    END;
   END;
  ELSE
   L_Rpr.AccessMark(RprS.OrdDate);
   RprObjIdDescr:=Mark.Name;
  END;

  RprSOrdDTAsStr:=DateTimeStr(RprS.OrdDate, RprS.OrdTime);

  Line:=L_Str.PoolItem(L_Rpr.rot_Pool_Short, RprS.RprObjT) + Str.Align(RprS.RprObjId, Str.MaxLength(RprS.RprObjId), LEFT) + " " +
        Str.Align(RprObjIdDescr, 52, LEFT) + " " +
        RprSOrdDTAsStr + " " +
        P_Rpr.GetShortRprSDescr(RprS.RprS);
  Line:=Str.Align(Line, ReportColumnWidth - (1 + Str.Length(RprSTaskLenAsStr)), LEFT) + " " + RprSTaskLenAsStr;
  RETURN Line;
 END {GetSingleLine};

 PRIVATE PROCEDURE GetLonRepairDuration(): NUMERIC[0];
 BEGIN
  IF L_Rpr.IsRprS_w_Car() | L_Rpr.IsRprS_w_OutCar() THEN
   RETURN 24;
  ELSE
   RETURN 64;
  END;
 END {GetLonRepairDuration};

 PRIVATE PROCEDURE MakeRep(RepName,
                           Title    : STRING[];
                           trgFirm  : LIKE(Firm.Firm));
 BEGIN
  OPENREP(R, L_General.NameConnection(RepName), Create);
  MAKEPART(R, Top, RepName,
                   Title,
                   Amount[k_Repair],
                   Amount[k_LongRepair],
                   Amount[k_Ready]);

  L_XDoc.sl_Init("᮪ ");
  IF ~!trgFirm THEN
   SELECT(_RprSL, Firm, , trgFirm);
  ELSE
   SELECT(_RprSL, Kind);
  END;
  WHILE OCCURE(_RprSL) DO
   NARROWSEL(_RprSL, Kind);
   MAKEPART(R, SubTop, L_Str.PoolItem(k_Pool, _RprSL.Kind) + " (" + L_Str.IntStr(Amount[_RprSL.Kind]) + ")",
                       L_Str.PoolItem(".ப|.ப|", _RprSL.Kind));
   IF ~!trgFirm THEN
    LAST(_RprSL);
   END;
   WHILE OCCURE(_RprSL) DO
    PROMPT(" " + L_Str.IntStr(_RprSL.RprS));
    SEARCH(RprS, RprS, , _RprSL.RprS);
    L_XDoc.sl_AddItem(L_XDoc.XDocId(L_XDoc.xdt_RprS, L_XDoc.EmptyXDocKind, RprS.RprS),
                      Str.Align(RprS.RprObjId, Str.MaxLength(RprS.RprObjId), LEFT) + " " +
                      " " + RprS.RprS + "  " +
                      L_DatTim.DateStr(RprS.OrdDate) + " (" +
                      L_DatTim.DateStr(RprS.EDate) + ") " +
                      L_Str.PoolItem("-|", RprS.Closed));
    MAKEPART(R, Line, GetSingleLine(!trgFirm));
    IF ~!trgFirm THEN
     PREV(_RprSL);
    ELSE
     NEXT(_RprSL);
    END;
   END; (* WHILE OCCURE(_RprSL) ... *)
   NEXTINEXTENDEDSEL(_RprSL, Kind);
  END; (* WHILE OCCURE(_RprSL) ... *)
  MAKEPART(R, Bot);
  CLOSEREP(R);
  L_XDoc.sl_Done();
 END {MakeRep};

 PRIVATE PROCEDURE IsAcceptableRprS(trgLocation: LIKE(RprS.Location)): SCAL[2];
  VAR
   Result: SCAL[2];
 BEGIN
  Result:=0;
  IF RprS.Status # L_Rpr.ms_Cancel THEN
   IF ~!trgLocation THEN
    Result:=RprS.Location = trgLocation;
   ELSE
    Result:=L_Rpr.IsRprS_w_OutCar() | L_Rpr.IsRprS_w_OutCBlc();
   END;
  END;
  RETURN Result;
 END {IsAcceptableRprS};

 PRIVATE PROCEDURE InsToTmp();
 BEGIN
  CONFORM(RprS, CUCopy);
  L_CUC.AccessOrigin();
  DEFAULT(_RprSL);
  IF    ~!RprS.EDate THEN
   _RprSL.Kind:=k_Ready;
  ELSIF GetRprTime() > GetLonRepairDuration() THEN
   _RprSL.Kind:=k_LongRepair;
  ELSE
   _RprSL.Kind:=k_Repair;
  END;
  _RprSL.RprS:=RprS.RprS;
  IF    L_Rpr.IsRprS_w_OutCar() THEN
   _RprSL.Firm:=OutCar.Firm;
  ELSIF L_Rpr.IsRprS_w_OutCBlc() THEN
   _RprSL.Firm:=OutCBlc.Firm;
  ELSIF L_Rpr.IsRprS_w_Car() THEN
   _RprSL.Firm:=L_Car.GetFirmOfCar(RprS.OrdDate);
  ELSIF L_Rpr.IsRprS_w_CarBlock() THEN
   CONFORM(CUCopy, Math);
   _RprSL.Firm:=L_MathR.GetFirmOfMath();
  END;
  INSERT(_RprSL);
 END {InsToTmp};

 PRIVATE PROCEDURE Rpr26_Gather(trgLocation: LIKE(RprS.Location);
                                Mode       : tMode);
 BEGIN
  L_AFM.MakeMultipleIndex("_RprSL", "Kind  : SCAL[3];" +
                                    "BegDTS: TIMESTAMP;" +
                                    "RprS  : LIKE(RprS.RprS);" +
                                    "Firm  : LIKE(Firm.Firm)",
                          "Kind:Kind,BegDTS,Firm;Firm:Firm,Kind,BegDTS");
  SETZERO(Amount);
  IF    Mode = m_CurState THEN
   SELECT(RprS, cRprObjId, , 0);
  ELSIF Mode = m_Last2Month THEN
   SELECT(RprS, OrdDate);
   SRCINSEL(RprS, GE, , SYSDATE() - 60);
  ELSIF Mode = m_Period THEN
   SELECT(RprS, OrdDate);
   SRCINSEL(RprS, GE, , Memo.ud_fr());
  END;
  WHILE OCCURE(RprS) & ((Mode # m_Period) | (RprS.OrdDate <= Memo.ud_ut())) DO
   PROMPT("ઠ " + RprS.RprS);
   IF IsAcceptableRprS(trgLocation) THEN
    InsToTmp();
    IF ~!trgLocation THEN
     INC(Amount[_RprSL.Kind]);
    END;
   END;
   NEXT(RprS);
  END; (* WHILE OCCURE(RprS) ... *)

  IF !trgLocation & (Mode = m_CurState) THEN
   SELECT(RprS, EDate, , SYSDATE());
   WHILE OCCURE(RprS) DO
    PROMPT("ઠ " + RprS.RprS);
    IF IsAcceptableRprS(trgLocation) THEN
     InsToTmp();
    END;
    NEXT(RprS);
   END; (* WHILE OCCURE(RprS) ... *)
  END;
 END {Rpr26_Gather};

 PUBLIC PROCEDURE Rpr26();
  VAR
   LocationList    : STRING[];
   i, _Location    : L_Rpr.tRprSLocation;
   RprSLocationView: SCAL[2][L_Rpr.rprl_Amount];
 BEGIN
  SE_ACCESSDATAMODE(AccessDataMode_SOFT);

  L_Rpr.GetDefaultViewRprSScopeMode(RprSLocationView);
  SETEMPTY(_Location);
  i:=0;
  SETEMPTY(LocationList);
  WHILE i < L_Rpr.rprl_Amount DO
   IF L_Rpr.E_AllowedRprSLocation(i) THEN
    IF !_Location & RprSLocationView[i] THEN
     _Location:=i;
    END;
    LocationList:=LocationList + L_Rpr.GetRprSLocation_AsStr(i) + "|";
   END;
   INC(i);
  END; (* WHILE i < rprl_Amount...*)

  IF ~!_Location THEN
   _Location:=L_Str.PositionInPool(LocationList, L_Str.PoolItem(L_Rpr.GetRprSLocation_Pool(0), _Location));
  ELSE
   _Location:=0;
  END;

  STDFORM(L_StdFrm.PartForm("%L<" + LocationList + ">I"+
                            "%< ⬥ >X"+
                            "%< 믮 >DB"+
                            "@"+
                            "|⮯ %##############################%"+
                            "|"+
                            "|%###########%     %##########%"),
                            " ⨢ ﭨ ",
                            ,
                            1,
                            _Location);

  _Location:=L_Str.PositionInPool(L_Rpr.GetRprSLocation_Pool(0), L_Str.PoolItem(LocationList, _Location));

  Rpr26_Gather(_Location, m_CurState);
  MakeRep("R26_" + _Location,
          "⨢ ﭨ (" + L_Rpr.GetRprSLocation_AsStr(_Location) + ")  " + L_DatTim.DateTimeStr(SYSDATE(), SYSTIME()),
          EMPTYOF(Firm.Firm));
 END {Rpr26};

 PUBLIC PROCEDURE ROFrm();
  CONST
   Mode_Pool = " ᥣ| ᫥ 2 | ਮ";

  VAR
   trgFirm     : LIKE(Firm.Firm);
   Choice      : NUMERIC[0];
   Mode        : SCAL[3];
   Title,
   FirmList,
   FirmNameList: STRING[];
 BEGIN
  SE_ACCESSDATAMODE(AccessDataMode_SOFT);
  Mode:=m_CurState;
  STDFORM(L_StdFrm.PartForm("%L<" + Mode_Pool + ">I"+
                            "%< ⬥ >X"+
                            "%< 믮 >DB"+
                            "@"+
                            "|%##############################%"+
                            "|"+
                            "|%###########%     %##########%"),
                            "  ࠣ⮢ ",
                            ,
                            1,
                            Mode);
  Rpr26_Gather(EMPTYOF(RprS.Location), Mode);
  SETEMPTY(FirmList, FirmNameList);
  SELECT(_RprSL, Firm);
  WHILE OCCURE(_RprSL) DO
   NARROWSEL(_RprSL, Firm);
   SEARCH(Firm, Firm, , _RprSL.Firm);
   FirmList    :=FirmList + _RprSL.Firm + "|";
   FirmNameList:=FirmNameList + _RprSL.Firm + " " + Firm.Name + "|";
   NEXTINEXTENDEDSEL(_RprSL, Firm);
  END; (* WHILE OCCURE(_RprSL) ... *)

  Choice :=L_StdFrm.ShowListAsMenu(FirmNameList, "롮 ࠣ");
  trgFirm:=L_Str.PoolItem(FirmList, Choice);
  SEARCH(Firm, Firm, , trgFirm);

  SETZERO(Amount);
  SELECT(_RprSL, Firm, , trgFirm);
  WHILE OCCURE(_RprSL) DO
   INC(Amount[_RprSL.Kind]);
   NEXT(_RprSL);
  END; (* WHILE OCCURE(_RprSL) ... *)

  IF Mode = m_Period THEN
   Title:=" ਮ  " + L_DatTim.DateStr(Memo.ud_fr()) + "  " + L_DatTim.DateStr(Memo.ud_ut());
  ELSE
   Title:=L_Str.PoolItem(Mode_Pool, Mode);
  END;

  MakeRep("ROFrm",
          " ࠣ " + trgFirm + " " + Firm.Name + " (" + Title  + ")",
          trgFirm);
 END {ROFrm};

END.
